/****************************************************************************
** Meta object code from reading C++ file 'fileinfohelper.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../dde-file-manager/src/dfm-base/utils/fileinfohelper.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileinfohelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmbase__FileInfoHelper_t {
    QByteArrayData data[30];
    char stringdata0[481];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__FileInfoHelper_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__FileInfoHelper_t qt_meta_stringdata_dfmbase__FileInfoHelper = {
    {
QT_MOC_LITERAL(0, 0, 23), // "dfmbase::FileInfoHelper"
QT_MOC_LITERAL(1, 24, 17), // "mediaDataFinished"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 10), // "sourceFile"
QT_MOC_LITERAL(4, 54, 50), // "QMap<dfmio::DFileInfo::Attrib..."
QT_MOC_LITERAL(5, 105, 10), // "properties"
QT_MOC_LITERAL(6, 116, 17), // "fileCountFinished"
QT_MOC_LITERAL(7, 134, 3), // "url"
QT_MOC_LITERAL(8, 138, 9), // "fileCount"
QT_MOC_LITERAL(9, 148, 20), // "fileMimeTypeFinished"
QT_MOC_LITERAL(10, 169, 9), // "QMimeType"
QT_MOC_LITERAL(11, 179, 4), // "type"
QT_MOC_LITERAL(12, 184, 39), // "QSharedPointer<FileInfoHelper..."
QT_MOC_LITERAL(13, 224, 4), // "data"
QT_MOC_LITERAL(14, 229, 12), // "fileMimeType"
QT_MOC_LITERAL(15, 242, 24), // "QMimeDatabase::MatchMode"
QT_MOC_LITERAL(16, 267, 4), // "mode"
QT_MOC_LITERAL(17, 272, 4), // "inod"
QT_MOC_LITERAL(18, 277, 6), // "isGvfs"
QT_MOC_LITERAL(19, 284, 15), // "fileInfoRefresh"
QT_MOC_LITERAL(20, 300, 32), // "QSharedPointer<dfmio::DFileInfo>"
QT_MOC_LITERAL(21, 333, 9), // "dfileInfo"
QT_MOC_LITERAL(22, 343, 19), // "fileRefreshFinished"
QT_MOC_LITERAL(23, 363, 7), // "infoPtr"
QT_MOC_LITERAL(24, 371, 9), // "isLinkOrg"
QT_MOC_LITERAL(25, 381, 18), // "fileRefreshRequest"
QT_MOC_LITERAL(26, 400, 24), // "QSharedPointer<FileInfo>"
QT_MOC_LITERAL(27, 425, 25), // "smbSeverMayModifyPassword"
QT_MOC_LITERAL(28, 451, 11), // "aboutToQuit"
QT_MOC_LITERAL(29, 463, 17) // "handleFileRefresh"

    },
    "dfmbase::FileInfoHelper\0mediaDataFinished\0"
    "\0sourceFile\0"
    "QMap<dfmio::DFileInfo::AttributeExtendID,QVariant>\0"
    "properties\0fileCountFinished\0url\0"
    "fileCount\0fileMimeTypeFinished\0QMimeType\0"
    "type\0QSharedPointer<FileInfoHelperUeserData>\0"
    "data\0fileMimeType\0QMimeDatabase::MatchMode\0"
    "mode\0inod\0isGvfs\0fileInfoRefresh\0"
    "QSharedPointer<dfmio::DFileInfo>\0"
    "dfileInfo\0fileRefreshFinished\0infoPtr\0"
    "isLinkOrg\0fileRefreshRequest\0"
    "QSharedPointer<FileInfo>\0"
    "smbSeverMayModifyPassword\0aboutToQuit\0"
    "handleFileRefresh"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__FileInfoHelper[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   69,    2, 0x06 /* Public */,
       6,    2,   74,    2, 0x06 /* Public */,
       9,    2,   79,    2, 0x06 /* Public */,
       8,    2,   84,    2, 0x06 /* Public */,
      14,    5,   89,    2, 0x06 /* Public */,
      19,    2,  100,    2, 0x06 /* Public */,
      22,    3,  105,    2, 0x06 /* Public */,
      25,    1,  112,    2, 0x06 /* Public */,
      27,    1,  115,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      28,    0,  118,    2, 0x08 /* Private */,
      29,    1,  119,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 4,    3,    5,
    QMetaType::Void, QMetaType::QUrl, QMetaType::Int,    7,    8,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 10,    7,   11,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 12,    7,   13,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 15, QMetaType::QString, QMetaType::Bool, 0x80000000 | 12,    7,   16,   17,   18,   13,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 20,    7,   21,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QString, QMetaType::Bool,    7,   23,   24,
    QMetaType::Void, 0x80000000 | 26,   21,
    QMetaType::Void, QMetaType::QUrl,    7,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 26,   21,

       0        // eod
};

void dfmbase::FileInfoHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileInfoHelper *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->mediaDataFinished((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QMap<dfmio::DFileInfo::AttributeExtendID,QVariant>(*)>(_a[2]))); break;
        case 1: _t->fileCountFinished((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2]))); break;
        case 2: _t->fileMimeTypeFinished((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QMimeType(*)>(_a[2]))); break;
        case 3: _t->fileCount((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QSharedPointer<FileInfoHelperUeserData>(*)>(_a[2]))); break;
        case 4: _t->fileMimeType((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QMimeDatabase::MatchMode(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const bool(*)>(_a[4])),(*reinterpret_cast< const QSharedPointer<FileInfoHelperUeserData>(*)>(_a[5]))); break;
        case 5: _t->fileInfoRefresh((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QSharedPointer<dfmio::DFileInfo>(*)>(_a[2]))); break;
        case 6: _t->fileRefreshFinished((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const bool(*)>(_a[3]))); break;
        case 7: _t->fileRefreshRequest((*reinterpret_cast< QSharedPointer<FileInfo>(*)>(_a[1]))); break;
        case 8: _t->smbSeverMayModifyPassword((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 9: _t->aboutToQuit(); break;
        case 10: _t->handleFileRefresh((*reinterpret_cast< QSharedPointer<FileInfo>(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileInfoHelper::*)(const QUrl & , QMap<dfmio::DFileInfo::AttributeExtendID,QVariant> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileInfoHelper::mediaDataFinished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileInfoHelper::*)(const QUrl & , const int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileInfoHelper::fileCountFinished)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileInfoHelper::*)(const QUrl & , const QMimeType & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileInfoHelper::fileMimeTypeFinished)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FileInfoHelper::*)(const QUrl & , const QSharedPointer<FileInfoHelperUeserData> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileInfoHelper::fileCount)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FileInfoHelper::*)(const QUrl & , const QMimeDatabase::MatchMode , const QString & , const bool , const QSharedPointer<FileInfoHelperUeserData> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileInfoHelper::fileMimeType)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (FileInfoHelper::*)(const QUrl & , QSharedPointer<dfmio::DFileInfo> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileInfoHelper::fileInfoRefresh)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (FileInfoHelper::*)(const QUrl , const QString & , const bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileInfoHelper::fileRefreshFinished)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (FileInfoHelper::*)(QSharedPointer<FileInfo> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileInfoHelper::fileRefreshRequest)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (FileInfoHelper::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileInfoHelper::smbSeverMayModifyPassword)) {
                *result = 8;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::FileInfoHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__FileInfoHelper.data,
    qt_meta_data_dfmbase__FileInfoHelper,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::FileInfoHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::FileInfoHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__FileInfoHelper.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmbase::FileInfoHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void dfmbase::FileInfoHelper::mediaDataFinished(const QUrl & _t1, QMap<dfmio::DFileInfo::AttributeExtendID,QVariant> _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmbase::FileInfoHelper::fileCountFinished(const QUrl & _t1, const int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmbase::FileInfoHelper::fileMimeTypeFinished(const QUrl & _t1, const QMimeType & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmbase::FileInfoHelper::fileCount(const QUrl & _t1, const QSharedPointer<FileInfoHelperUeserData> _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmbase::FileInfoHelper::fileMimeType(const QUrl & _t1, const QMimeDatabase::MatchMode _t2, const QString & _t3, const bool _t4, const QSharedPointer<FileInfoHelperUeserData> _t5)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmbase::FileInfoHelper::fileInfoRefresh(const QUrl & _t1, QSharedPointer<dfmio::DFileInfo> _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void dfmbase::FileInfoHelper::fileRefreshFinished(const QUrl _t1, const QString & _t2, const bool _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void dfmbase::FileInfoHelper::fileRefreshRequest(QSharedPointer<FileInfo> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void dfmbase::FileInfoHelper::smbSeverMayModifyPassword(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
