/****************************************************************************
** Meta object code from reading C++ file 'abstractjob.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-fileoperations/fileoperations/fileoperationutils/abstractjob.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstractjob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_fileoperations__AbstractJob_t {
    QByteArrayData data[17];
    char stringdata0[270];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_fileoperations__AbstractJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_fileoperations__AbstractJob_t qt_meta_stringdata_dfmplugin_fileoperations__AbstractJob = {
    {
QT_MOC_LITERAL(0, 0, 37), // "dfmplugin_fileoperations::Abs..."
QT_MOC_LITERAL(1, 38, 11), // "operateWork"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 34), // "AbstractJobHandler::SupportAc..."
QT_MOC_LITERAL(4, 86, 7), // "actions"
QT_MOC_LITERAL(5, 94, 11), // "errorNotify"
QT_MOC_LITERAL(6, 106, 14), // "JobInfoPointer"
QT_MOC_LITERAL(7, 121, 7), // "jobInfo"
QT_MOC_LITERAL(8, 129, 21), // "requestShowTipsDialog"
QT_MOC_LITERAL(9, 151, 43), // "dfmbase::AbstractJobHandler::..."
QT_MOC_LITERAL(10, 195, 4), // "type"
QT_MOC_LITERAL(11, 200, 11), // "QList<QUrl>"
QT_MOC_LITERAL(12, 212, 4), // "list"
QT_MOC_LITERAL(13, 217, 13), // "operateAation"
QT_MOC_LITERAL(14, 231, 11), // "handleError"
QT_MOC_LITERAL(15, 243, 23), // "handleRetryErrorSuccess"
QT_MOC_LITERAL(16, 267, 2) // "Id"

    },
    "dfmplugin_fileoperations::AbstractJob\0"
    "operateWork\0\0AbstractJobHandler::SupportActions\0"
    "actions\0errorNotify\0JobInfoPointer\0"
    "jobInfo\0requestShowTipsDialog\0"
    "dfmbase::AbstractJobHandler::ShowDialogType\0"
    "type\0QList<QUrl>\0list\0operateAation\0"
    "handleError\0handleRetryErrorSuccess\0"
    "Id"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_fileoperations__AbstractJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       5,    1,   47,    2, 0x06 /* Public */,
       8,    2,   50,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    1,   55,    2, 0x09 /* Protected */,
      14,    1,   58,    2, 0x09 /* Protected */,
      15,    1,   61,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 11,   10,   12,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::ULongLong,   16,

       0        // eod
};

void dfmplugin_fileoperations::AbstractJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AbstractJob *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->operateWork((*reinterpret_cast< AbstractJobHandler::SupportActions(*)>(_a[1]))); break;
        case 1: _t->errorNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 2: _t->requestShowTipsDialog((*reinterpret_cast< dfmbase::AbstractJobHandler::ShowDialogType(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2]))); break;
        case 3: _t->operateAation((*reinterpret_cast< AbstractJobHandler::SupportActions(*)>(_a[1]))); break;
        case 4: _t->handleError((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 5: _t->handleRetryErrorSuccess((*reinterpret_cast< const quint64(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::ShowDialogType >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AbstractJob::*)(AbstractJobHandler::SupportActions );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJob::operateWork)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AbstractJob::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJob::errorNotify)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (AbstractJob::*)(dfmbase::AbstractJobHandler::ShowDialogType , const QList<QUrl> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJob::requestShowTipsDialog)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_fileoperations::AbstractJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_fileoperations__AbstractJob.data,
    qt_meta_data_dfmplugin_fileoperations__AbstractJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_fileoperations::AbstractJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_fileoperations::AbstractJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_fileoperations__AbstractJob.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_fileoperations::AbstractJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_fileoperations::AbstractJob::operateWork(AbstractJobHandler::SupportActions _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_fileoperations::AbstractJob::errorNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_fileoperations::AbstractJob::requestShowTipsDialog(dfmbase::AbstractJobHandler::ShowDialogType _t1, const QList<QUrl> _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
