/****************************************************************************
** Meta object code from reading C++ file 'vaultmanagerdbus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/daemon/daemonplugin-vault/dbus/vaultmanagerdbus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vaultmanagerdbus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_VaultManagerDBus_t {
    QByteArrayData data[26];
    char stringdata0[439];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_VaultManagerDBus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_VaultManagerDBus_t qt_meta_stringdata_VaultManagerDBus = {
    {
QT_MOC_LITERAL(0, 0, 16), // "VaultManagerDBus"
QT_MOC_LITERAL(1, 17, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 33, 42), // "org.deepin.Filemanager.Daemon..."
QT_MOC_LITERAL(3, 76, 18), // "LockEventTriggered"
QT_MOC_LITERAL(4, 95, 0), // ""
QT_MOC_LITERAL(5, 96, 4), // "user"
QT_MOC_LITERAL(6, 101, 17), // "ChangedVaultState"
QT_MOC_LITERAL(7, 119, 3), // "map"
QT_MOC_LITERAL(8, 123, 14), // "SysUserChanged"
QT_MOC_LITERAL(9, 138, 7), // "curUser"
QT_MOC_LITERAL(10, 146, 14), // "SetRefreshTime"
QT_MOC_LITERAL(11, 161, 4), // "time"
QT_MOC_LITERAL(12, 166, 14), // "GetLastestTime"
QT_MOC_LITERAL(13, 181, 11), // "GetSelfTime"
QT_MOC_LITERAL(14, 193, 20), // "IsLockEventTriggered"
QT_MOC_LITERAL(15, 214, 16), // "TriggerLockEvent"
QT_MOC_LITERAL(16, 231, 14), // "ClearLockEvent"
QT_MOC_LITERAL(17, 246, 13), // "ComputerSleep"
QT_MOC_LITERAL(18, 260, 6), // "bSleep"
QT_MOC_LITERAL(19, 267, 26), // "GetLeftoverErrorInputTimes"
QT_MOC_LITERAL(20, 294, 6), // "userID"
QT_MOC_LITERAL(21, 301, 31), // "LeftoverErrorInputTimesMinusOne"
QT_MOC_LITERAL(22, 333, 30), // "RestoreLeftoverErrorInputTimes"
QT_MOC_LITERAL(23, 364, 32), // "StartTimerOfRestorePasswordInput"
QT_MOC_LITERAL(24, 397, 18), // "GetNeedWaitMinutes"
QT_MOC_LITERAL(25, 416, 22) // "RestoreNeedWaitMinutes"

    },
    "VaultManagerDBus\0D-Bus Interface\0"
    "org.deepin.Filemanager.Daemon.VaultManager\0"
    "LockEventTriggered\0\0user\0ChangedVaultState\0"
    "map\0SysUserChanged\0curUser\0SetRefreshTime\0"
    "time\0GetLastestTime\0GetSelfTime\0"
    "IsLockEventTriggered\0TriggerLockEvent\0"
    "ClearLockEvent\0ComputerSleep\0bSleep\0"
    "GetLeftoverErrorInputTimes\0userID\0"
    "LeftoverErrorInputTimesMinusOne\0"
    "RestoreLeftoverErrorInputTimes\0"
    "StartTimerOfRestorePasswordInput\0"
    "GetNeedWaitMinutes\0RestoreNeedWaitMinutes"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_VaultManagerDBus[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      16,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   96,    4, 0x06 /* Public */,
       6,    1,   99,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    1,  102,    4, 0x0a /* Public */,
      10,    1,  105,    4, 0x0a /* Public */,
      12,    0,  108,    4, 0x0a /* Public */,
      13,    0,  109,    4, 0x0a /* Public */,
      14,    0,  110,    4, 0x0a /* Public */,
      15,    0,  111,    4, 0x0a /* Public */,
      16,    0,  112,    4, 0x0a /* Public */,
      17,    1,  113,    4, 0x0a /* Public */,
      19,    1,  116,    4, 0x0a /* Public */,
      21,    1,  119,    4, 0x0a /* Public */,
      22,    1,  122,    4, 0x0a /* Public */,
      23,    1,  125,    4, 0x0a /* Public */,
      24,    1,  128,    4, 0x0a /* Public */,
      25,    1,  131,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QVariantMap,    7,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::ULongLong,   11,
    QMetaType::ULongLong,
    QMetaType::ULongLong,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Int, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Int, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Int,   20,

       0        // eod
};

void VaultManagerDBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<VaultManagerDBus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->LockEventTriggered((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->ChangedVaultState((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 2: _t->SysUserChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->SetRefreshTime((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 4: { quint64 _r = _t->GetLastestTime();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 5: { quint64 _r = _t->GetSelfTime();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->IsLockEventTriggered();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->TriggerLockEvent(); break;
        case 8: _t->ClearLockEvent(); break;
        case 9: _t->ComputerSleep((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: { int _r = _t->GetLeftoverErrorInputTimes((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->LeftoverErrorInputTimesMinusOne((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->RestoreLeftoverErrorInputTimes((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->StartTimerOfRestorePasswordInput((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: { int _r = _t->GetNeedWaitMinutes((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->RestoreNeedWaitMinutes((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (VaultManagerDBus::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VaultManagerDBus::LockEventTriggered)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (VaultManagerDBus::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VaultManagerDBus::ChangedVaultState)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject VaultManagerDBus::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_VaultManagerDBus.data,
    qt_meta_data_VaultManagerDBus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *VaultManagerDBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VaultManagerDBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_VaultManagerDBus.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int VaultManagerDBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void VaultManagerDBus::LockEventTriggered(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void VaultManagerDBus::ChangedVaultState(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
