/****************************************************************************
** Meta object code from reading C++ file 'docsheet.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../../dde-file-manager/src/apps/dde-file-manager-preview/pluginpreviews/pdf-preview/docsheet.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'docsheet.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_plugin_filepreview__DocSheet_t {
    QByteArrayData data[14];
    char stringdata0[166];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_plugin_filepreview__DocSheet_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_plugin_filepreview__DocSheet_t qt_meta_stringdata_plugin_filepreview__DocSheet = {
    {
QT_MOC_LITERAL(0, 0, 28), // "plugin_filepreview::DocSheet"
QT_MOC_LITERAL(1, 29, 15), // "sigPageModified"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 5), // "index"
QT_MOC_LITERAL(4, 52, 13), // "sigFileOpened"
QT_MOC_LITERAL(5, 66, 9), // "DocSheet*"
QT_MOC_LITERAL(6, 76, 5), // "sheet"
QT_MOC_LITERAL(7, 82, 15), // "Document::Error"
QT_MOC_LITERAL(8, 98, 5), // "error"
QT_MOC_LITERAL(9, 104, 8), // "onOpened"
QT_MOC_LITERAL(10, 113, 20), // "onBrowserPageChanged"
QT_MOC_LITERAL(11, 134, 4), // "page"
QT_MOC_LITERAL(12, 139, 17), // "onExtractPassword"
QT_MOC_LITERAL(13, 157, 8) // "password"

    },
    "plugin_filepreview::DocSheet\0"
    "sigPageModified\0\0index\0sigFileOpened\0"
    "DocSheet*\0sheet\0Document::Error\0error\0"
    "onOpened\0onBrowserPageChanged\0page\0"
    "onExtractPassword\0password"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_plugin_filepreview__DocSheet[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x06 /* Public */,
       4,    2,   42,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    1,   47,    2, 0x08 /* Private */,
      10,    1,   50,    2, 0x08 /* Private */,
      12,    1,   53,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, 0x80000000 | 5, 0x80000000 | 7,    6,    8,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::QString,   13,

       0        // eod
};

void plugin_filepreview::DocSheet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DocSheet *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sigPageModified((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->sigFileOpened((*reinterpret_cast< DocSheet*(*)>(_a[1])),(*reinterpret_cast< Document::Error(*)>(_a[2]))); break;
        case 2: _t->onOpened((*reinterpret_cast< Document::Error(*)>(_a[1]))); break;
        case 3: _t->onBrowserPageChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->onExtractPassword((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< DocSheet* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DocSheet::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DocSheet::sigPageModified)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DocSheet::*)(DocSheet * , Document::Error );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DocSheet::sigFileOpened)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject plugin_filepreview::DocSheet::staticMetaObject = { {
    QMetaObject::SuperData::link<Dtk::Widget::DSplitter::staticMetaObject>(),
    qt_meta_stringdata_plugin_filepreview__DocSheet.data,
    qt_meta_data_plugin_filepreview__DocSheet,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *plugin_filepreview::DocSheet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *plugin_filepreview::DocSheet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_plugin_filepreview__DocSheet.stringdata0))
        return static_cast<void*>(this);
    return Dtk::Widget::DSplitter::qt_metacast(_clname);
}

int plugin_filepreview::DocSheet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Dtk::Widget::DSplitter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void plugin_filepreview::DocSheet::sigPageModified(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void plugin_filepreview::DocSheet::sigFileOpened(DocSheet * _t1, Document::Error _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
