/****************************************************************************
** Meta object code from reading C++ file 'dbusdock1.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-core/screen/dbus-private/dbusdock1.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbusdock1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DBusDock_t {
    QByteArrayData data[47];
    char stringdata0[698];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DBusDock_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DBusDock_t qt_meta_stringdata_DBusDock = {
    {
QT_MOC_LITERAL(0, 0, 8), // "DBusDock"
QT_MOC_LITERAL(1, 9, 21), // "DockAppSettingsSynced"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 10), // "EntryAdded"
QT_MOC_LITERAL(4, 43, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(5, 59, 3), // "in0"
QT_MOC_LITERAL(6, 63, 3), // "in1"
QT_MOC_LITERAL(7, 67, 12), // "EntryRemoved"
QT_MOC_LITERAL(8, 80, 20), // "PluginSettingsSynced"
QT_MOC_LITERAL(9, 101, 16), // "ServiceRestarted"
QT_MOC_LITERAL(10, 118, 25), // "FrontendWindowRectChanged"
QT_MOC_LITERAL(11, 144, 15), // "HideModeChanged"
QT_MOC_LITERAL(12, 160, 19), // "__propertyChanged__"
QT_MOC_LITERAL(13, 180, 12), // "QDBusMessage"
QT_MOC_LITERAL(14, 193, 3), // "msg"
QT_MOC_LITERAL(15, 197, 14), // "ActivateWindow"
QT_MOC_LITERAL(16, 212, 19), // "QDBusPendingReply<>"
QT_MOC_LITERAL(17, 232, 19), // "CancelPreviewWindow"
QT_MOC_LITERAL(18, 252, 11), // "CloseWindow"
QT_MOC_LITERAL(19, 264, 25), // "GetDockedAppsDesktopFiles"
QT_MOC_LITERAL(20, 290, 30), // "QDBusPendingReply<QStringList>"
QT_MOC_LITERAL(21, 321, 11), // "GetEntryIDs"
QT_MOC_LITERAL(22, 333, 17), // "GetPluginSettings"
QT_MOC_LITERAL(23, 351, 26), // "QDBusPendingReply<QString>"
QT_MOC_LITERAL(24, 378, 8), // "IsDocked"
QT_MOC_LITERAL(25, 387, 23), // "QDBusPendingReply<bool>"
QT_MOC_LITERAL(26, 411, 8), // "IsOnDock"
QT_MOC_LITERAL(27, 420, 15), // "MakeWindowAbove"
QT_MOC_LITERAL(28, 436, 14), // "MaximizeWindow"
QT_MOC_LITERAL(29, 451, 19), // "MergePluginSettings"
QT_MOC_LITERAL(30, 471, 14), // "MinimizeWindow"
QT_MOC_LITERAL(31, 486, 9), // "MoveEntry"
QT_MOC_LITERAL(32, 496, 10), // "MoveWindow"
QT_MOC_LITERAL(33, 507, 13), // "PreviewWindow"
QT_MOC_LITERAL(34, 521, 25), // "QueryWindowIdentifyMethod"
QT_MOC_LITERAL(35, 547, 20), // "RemovePluginSettings"
QT_MOC_LITERAL(36, 568, 11), // "RequestDock"
QT_MOC_LITERAL(37, 580, 13), // "RequestUndock"
QT_MOC_LITERAL(38, 594, 21), // "SetFrontendWindowRect"
QT_MOC_LITERAL(39, 616, 3), // "in2"
QT_MOC_LITERAL(40, 620, 3), // "in3"
QT_MOC_LITERAL(41, 624, 17), // "SetPluginSettings"
QT_MOC_LITERAL(42, 642, 18), // "FrontendWindowRect"
QT_MOC_LITERAL(43, 661, 8), // "DockRect"
QT_MOC_LITERAL(44, 670, 8), // "HideMode"
QT_MOC_LITERAL(45, 679, 9), // "HideState"
QT_MOC_LITERAL(46, 689, 8) // "Position"

    },
    "DBusDock\0DockAppSettingsSynced\0\0"
    "EntryAdded\0QDBusObjectPath\0in0\0in1\0"
    "EntryRemoved\0PluginSettingsSynced\0"
    "ServiceRestarted\0FrontendWindowRectChanged\0"
    "HideModeChanged\0__propertyChanged__\0"
    "QDBusMessage\0msg\0ActivateWindow\0"
    "QDBusPendingReply<>\0CancelPreviewWindow\0"
    "CloseWindow\0GetDockedAppsDesktopFiles\0"
    "QDBusPendingReply<QStringList>\0"
    "GetEntryIDs\0GetPluginSettings\0"
    "QDBusPendingReply<QString>\0IsDocked\0"
    "QDBusPendingReply<bool>\0IsOnDock\0"
    "MakeWindowAbove\0MaximizeWindow\0"
    "MergePluginSettings\0MinimizeWindow\0"
    "MoveEntry\0MoveWindow\0PreviewWindow\0"
    "QueryWindowIdentifyMethod\0"
    "RemovePluginSettings\0RequestDock\0"
    "RequestUndock\0SetFrontendWindowRect\0"
    "in2\0in3\0SetPluginSettings\0FrontendWindowRect\0"
    "DockRect\0HideMode\0HideState\0Position"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DBusDock[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      29,   14, // methods
       4,  242, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  159,    2, 0x06 /* Public */,
       3,    2,  160,    2, 0x06 /* Public */,
       7,    1,  165,    2, 0x06 /* Public */,
       8,    0,  168,    2, 0x06 /* Public */,
       9,    0,  169,    2, 0x06 /* Public */,
      10,    0,  170,    2, 0x06 /* Public */,
      11,    0,  171,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    1,  172,    2, 0x08 /* Private */,
      15,    1,  175,    2, 0x0a /* Public */,
      17,    0,  178,    2, 0x0a /* Public */,
      18,    1,  179,    2, 0x0a /* Public */,
      19,    0,  182,    2, 0x0a /* Public */,
      21,    0,  183,    2, 0x0a /* Public */,
      22,    0,  184,    2, 0x0a /* Public */,
      24,    1,  185,    2, 0x0a /* Public */,
      26,    1,  188,    2, 0x0a /* Public */,
      27,    1,  191,    2, 0x0a /* Public */,
      28,    1,  194,    2, 0x0a /* Public */,
      29,    1,  197,    2, 0x0a /* Public */,
      30,    1,  200,    2, 0x0a /* Public */,
      31,    2,  203,    2, 0x0a /* Public */,
      32,    1,  208,    2, 0x0a /* Public */,
      33,    1,  211,    2, 0x0a /* Public */,
      34,    1,  214,    2, 0x0a /* Public */,
      35,    2,  217,    2, 0x0a /* Public */,
      36,    2,  222,    2, 0x0a /* Public */,
      37,    1,  227,    2, 0x0a /* Public */,
      38,    4,  230,    2, 0x0a /* Public */,
      41,    1,  239,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int,    5,    6,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 13,   14,
    0x80000000 | 16, QMetaType::UInt,    5,
    0x80000000 | 16,
    0x80000000 | 16, QMetaType::UInt,    5,
    0x80000000 | 20,
    0x80000000 | 20,
    0x80000000 | 23,
    0x80000000 | 25, QMetaType::QString,    5,
    0x80000000 | 25, QMetaType::QString,    5,
    0x80000000 | 16, QMetaType::UInt,    5,
    0x80000000 | 16, QMetaType::UInt,    5,
    0x80000000 | 16, QMetaType::QString,    5,
    0x80000000 | 16, QMetaType::UInt,    5,
    0x80000000 | 16, QMetaType::Int, QMetaType::Int,    5,    6,
    0x80000000 | 16, QMetaType::UInt,    5,
    0x80000000 | 16, QMetaType::UInt,    5,
    0x80000000 | 23, QMetaType::UInt,    5,
    0x80000000 | 16, QMetaType::QString, QMetaType::QStringList,    5,    6,
    0x80000000 | 25, QMetaType::QString, QMetaType::Int,    5,    6,
    0x80000000 | 25, QMetaType::QString,    5,
    0x80000000 | 16, QMetaType::Int, QMetaType::Int, QMetaType::UInt, QMetaType::UInt,    5,    6,   39,   40,
    0x80000000 | 16, QMetaType::QString,    5,

 // properties: name, type, flags
      42, 0x80000000 | 43, 0x00495009,
      44, QMetaType::Int, 0x00495103,
      45, QMetaType::Int, 0x00095001,
      46, QMetaType::Int, 0x00095103,

 // properties: notify_signal_id
       5,
       6,
       0,
       0,

       0        // eod
};

void DBusDock::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DBusDock *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->DockAppSettingsSynced(); break;
        case 1: _t->EntryAdded((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 2: _t->EntryRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->PluginSettingsSynced(); break;
        case 4: _t->ServiceRestarted(); break;
        case 5: _t->FrontendWindowRectChanged(); break;
        case 6: _t->HideModeChanged(); break;
        case 7: _t->__propertyChanged__((*reinterpret_cast< const QDBusMessage(*)>(_a[1]))); break;
        case 8: { QDBusPendingReply<> _r = _t->ActivateWindow((*reinterpret_cast< uint(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->CancelPreviewWindow();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<> _r = _t->CloseWindow((*reinterpret_cast< uint(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<QStringList> _r = _t->GetDockedAppsDesktopFiles();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<QStringList> _r = _t->GetEntryIDs();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusPendingReply<QString> _r = _t->GetPluginSettings();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<bool> _r = _t->IsDocked((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<bool> _r = _t->IsOnDock((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 16: { QDBusPendingReply<> _r = _t->MakeWindowAbove((*reinterpret_cast< uint(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 17: { QDBusPendingReply<> _r = _t->MaximizeWindow((*reinterpret_cast< uint(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 18: { QDBusPendingReply<> _r = _t->MergePluginSettings((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 19: { QDBusPendingReply<> _r = _t->MinimizeWindow((*reinterpret_cast< uint(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 20: { QDBusPendingReply<> _r = _t->MoveEntry((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 21: { QDBusPendingReply<> _r = _t->MoveWindow((*reinterpret_cast< uint(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 22: { QDBusPendingReply<> _r = _t->PreviewWindow((*reinterpret_cast< uint(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 23: { QDBusPendingReply<QString> _r = _t->QueryWindowIdentifyMethod((*reinterpret_cast< uint(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 24: { QDBusPendingReply<> _r = _t->RemovePluginSettings((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 25: { QDBusPendingReply<bool> _r = _t->RequestDock((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 26: { QDBusPendingReply<bool> _r = _t->RequestUndock((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 27: { QDBusPendingReply<> _r = _t->SetFrontendWindowRect((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3])),(*reinterpret_cast< uint(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 28: { QDBusPendingReply<> _r = _t->SetPluginSettings((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusMessage >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DBusDock::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusDock::DockAppSettingsSynced)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DBusDock::*)(const QDBusObjectPath & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusDock::EntryAdded)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DBusDock::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusDock::EntryRemoved)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DBusDock::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusDock::PluginSettingsSynced)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DBusDock::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusDock::ServiceRestarted)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DBusDock::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusDock::FrontendWindowRectChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DBusDock::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusDock::HideModeChanged)) {
                *result = 6;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< DockRect >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DBusDock *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< DockRect*>(_v) = _t->frontendWindowRect(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->hideMode(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->hideState(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->position(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DBusDock *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setHideMode(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setPosition(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject DBusDock::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_DBusDock.data,
    qt_meta_data_DBusDock,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DBusDock::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DBusDock::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DBusDock.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int DBusDock::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void DBusDock::DockAppSettingsSynced()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DBusDock::EntryAdded(const QDBusObjectPath & _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DBusDock::EntryRemoved(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void DBusDock::PluginSettingsSynced()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void DBusDock::ServiceRestarted()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DBusDock::FrontendWindowRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void DBusDock::HideModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
