/****************************************************************************
** Meta object code from reading C++ file 'sidebareventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-sidebar/events/sidebareventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sidebareventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_sidebar__SideBarEventReceiver_t {
    QByteArrayData data[19];
    char stringdata0[281];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_sidebar__SideBarEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_sidebar__SideBarEventReceiver_t qt_meta_stringdata_dfmplugin_sidebar__SideBarEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 39), // "dfmplugin_sidebar::SideBarEve..."
QT_MOC_LITERAL(1, 40, 26), // "handleSetContextMenuEnable"
QT_MOC_LITERAL(2, 67, 0), // ""
QT_MOC_LITERAL(3, 68, 6), // "enable"
QT_MOC_LITERAL(4, 75, 19), // "handleGetGroupItems"
QT_MOC_LITERAL(5, 95, 11), // "QList<QUrl>"
QT_MOC_LITERAL(6, 107, 5), // "winId"
QT_MOC_LITERAL(7, 113, 5), // "group"
QT_MOC_LITERAL(8, 119, 13), // "handleItemAdd"
QT_MOC_LITERAL(9, 133, 3), // "url"
QT_MOC_LITERAL(10, 137, 10), // "properties"
QT_MOC_LITERAL(11, 148, 16), // "handleItemRemove"
QT_MOC_LITERAL(12, 165, 16), // "handleItemUpdate"
QT_MOC_LITERAL(13, 182, 16), // "handleItemInsert"
QT_MOC_LITERAL(14, 199, 5), // "index"
QT_MOC_LITERAL(15, 205, 16), // "handleItemHidden"
QT_MOC_LITERAL(16, 222, 7), // "visible"
QT_MOC_LITERAL(17, 230, 21), // "handleItemTriggerEdit"
QT_MOC_LITERAL(18, 252, 28) // "handleSidebarUpdateSelection"

    },
    "dfmplugin_sidebar::SideBarEventReceiver\0"
    "handleSetContextMenuEnable\0\0enable\0"
    "handleGetGroupItems\0QList<QUrl>\0winId\0"
    "group\0handleItemAdd\0url\0properties\0"
    "handleItemRemove\0handleItemUpdate\0"
    "handleItemInsert\0index\0handleItemHidden\0"
    "visible\0handleItemTriggerEdit\0"
    "handleSidebarUpdateSelection"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_sidebar__SideBarEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x0a /* Public */,
       4,    2,   62,    2, 0x0a /* Public */,
       8,    2,   67,    2, 0x0a /* Public */,
      11,    1,   72,    2, 0x0a /* Public */,
      12,    2,   75,    2, 0x0a /* Public */,
      13,    3,   80,    2, 0x0a /* Public */,
      15,    2,   87,    2, 0x0a /* Public */,
      17,    2,   92,    2, 0x0a /* Public */,
      18,    1,   97,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    0x80000000 | 5, QMetaType::ULongLong, QMetaType::QString,    6,    7,
    QMetaType::Bool, QMetaType::QUrl, QMetaType::QVariantMap,    9,   10,
    QMetaType::Bool, QMetaType::QUrl,    9,
    QMetaType::Bool, QMetaType::QUrl, QMetaType::QVariantMap,    9,   10,
    QMetaType::Bool, QMetaType::Int, QMetaType::QUrl, QMetaType::QVariantMap,   14,    9,   10,
    QMetaType::Void, QMetaType::QUrl, QMetaType::Bool,    9,   16,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,    6,    9,
    QMetaType::Void, QMetaType::ULongLong,    6,

       0        // eod
};

void dfmplugin_sidebar::SideBarEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SideBarEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleSetContextMenuEnable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: { QList<QUrl> _r = _t->handleGetGroupItems((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QList<QUrl>*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->handleItemAdd((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->handleItemRemove((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->handleItemUpdate((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->handleItemInsert((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QVariantMap(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->handleItemHidden((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 7: _t->handleItemTriggerEdit((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 8: _t->handleSidebarUpdateSelection((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_sidebar::SideBarEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_sidebar__SideBarEventReceiver.data,
    qt_meta_data_dfmplugin_sidebar__SideBarEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_sidebar::SideBarEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_sidebar::SideBarEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_sidebar__SideBarEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_sidebar::SideBarEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
