/****************************************************************************
** Meta object code from reading C++ file 'workspaceeventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-workspace/events/workspaceeventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'workspaceeventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_workspace__WorkspaceEventReceiver_t {
    QByteArrayData data[77];
    char stringdata0[1492];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_workspace__WorkspaceEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_workspace__WorkspaceEventReceiver_t qt_meta_stringdata_dfmplugin_workspace__WorkspaceEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 43), // "dfmplugin_workspace::Workspac..."
QT_MOC_LITERAL(1, 44, 32), // "handleTileBarSwitchModeTriggered"
QT_MOC_LITERAL(2, 77, 0), // ""
QT_MOC_LITERAL(3, 78, 8), // "windowId"
QT_MOC_LITERAL(4, 87, 4), // "mode"
QT_MOC_LITERAL(5, 92, 25), // "handleShowCustomTopWidget"
QT_MOC_LITERAL(6, 118, 6), // "scheme"
QT_MOC_LITERAL(7, 125, 7), // "visible"
QT_MOC_LITERAL(8, 133, 17), // "handleSelectFiles"
QT_MOC_LITERAL(9, 151, 11), // "QList<QUrl>"
QT_MOC_LITERAL(10, 163, 5), // "files"
QT_MOC_LITERAL(11, 169, 15), // "handleSelectAll"
QT_MOC_LITERAL(12, 185, 19), // "handleReverseSelect"
QT_MOC_LITERAL(13, 205, 13), // "handleSetSort"
QT_MOC_LITERAL(14, 219, 26), // "dfmbase::Global::ItemRoles"
QT_MOC_LITERAL(15, 246, 4), // "role"
QT_MOC_LITERAL(16, 251, 24), // "handleNotSupportTreeView"
QT_MOC_LITERAL(17, 276, 22), // "handleSetSelectionMode"
QT_MOC_LITERAL(18, 299, 32), // "QAbstractItemView::SelectionMode"
QT_MOC_LITERAL(19, 332, 30), // "handleSetEnabledSelectionModes"
QT_MOC_LITERAL(20, 363, 39), // "QList<QAbstractItemView::Sele..."
QT_MOC_LITERAL(21, 403, 5), // "modes"
QT_MOC_LITERAL(22, 409, 24), // "handleSetViewDragEnabled"
QT_MOC_LITERAL(23, 434, 7), // "enabled"
QT_MOC_LITERAL(24, 442, 25), // "handleSetViewDragDropMode"
QT_MOC_LITERAL(25, 468, 31), // "QAbstractItemView::DragDropMode"
QT_MOC_LITERAL(26, 500, 27), // "handleClosePersistentEditor"
QT_MOC_LITERAL(27, 528, 19), // "handleGetViewFilter"
QT_MOC_LITERAL(28, 548, 19), // "handleGetNameFilter"
QT_MOC_LITERAL(29, 568, 19), // "handleSetViewFilter"
QT_MOC_LITERAL(30, 588, 13), // "QDir::Filters"
QT_MOC_LITERAL(31, 602, 7), // "filters"
QT_MOC_LITERAL(32, 610, 19), // "handleSetNameFilter"
QT_MOC_LITERAL(33, 630, 17), // "handleSetReadOnly"
QT_MOC_LITERAL(34, 648, 8), // "readOnly"
QT_MOC_LITERAL(35, 657, 21), // "handlePasteFileResult"
QT_MOC_LITERAL(36, 679, 7), // "srcUrls"
QT_MOC_LITERAL(37, 687, 8), // "destUrls"
QT_MOC_LITERAL(38, 696, 2), // "ok"
QT_MOC_LITERAL(39, 699, 6), // "errMsg"
QT_MOC_LITERAL(40, 706, 27), // "handleMoveToTrashFileResult"
QT_MOC_LITERAL(41, 734, 22), // "handleRenameFileResult"
QT_MOC_LITERAL(42, 757, 15), // "QMap<QUrl,QUrl>"
QT_MOC_LITERAL(43, 773, 11), // "renamedUrls"
QT_MOC_LITERAL(44, 785, 16), // "handleFileUpdate"
QT_MOC_LITERAL(45, 802, 3), // "url"
QT_MOC_LITERAL(46, 806, 21), // "handleCurrentSortRole"
QT_MOC_LITERAL(47, 828, 17), // "handleColumnRoles"
QT_MOC_LITERAL(48, 846, 33), // "QList<dfmbase::Global::ItemRo..."
QT_MOC_LITERAL(49, 880, 23), // "handleGetVisualGeometry"
QT_MOC_LITERAL(50, 904, 8), // "windowID"
QT_MOC_LITERAL(51, 913, 21), // "handleGetViewItemRect"
QT_MOC_LITERAL(52, 935, 24), // "handleSetDefaultViewMode"
QT_MOC_LITERAL(53, 960, 25), // "dfmbase::Global::ViewMode"
QT_MOC_LITERAL(54, 986, 24), // "handleGetDefaultViewMode"
QT_MOC_LITERAL(55, 1011, 24), // "handleGetCurrentViewMode"
QT_MOC_LITERAL(56, 1036, 22), // "handleRegisterFileView"
QT_MOC_LITERAL(57, 1059, 23), // "handleRegisterMenuScene"
QT_MOC_LITERAL(58, 1083, 5), // "scene"
QT_MOC_LITERAL(59, 1089, 19), // "handleFindMenuScene"
QT_MOC_LITERAL(60, 1109, 29), // "handleRegisterCustomTopWidget"
QT_MOC_LITERAL(61, 1139, 7), // "dataMap"
QT_MOC_LITERAL(62, 1147, 31), // "handleGetCustomTopWidgetVisible"
QT_MOC_LITERAL(63, 1179, 31), // "handleCheckSchemeViewIsFileView"
QT_MOC_LITERAL(64, 1211, 21), // "handleGetSelectedUrls"
QT_MOC_LITERAL(65, 1233, 25), // "handleSetCustomFilterData"
QT_MOC_LITERAL(66, 1259, 4), // "data"
QT_MOC_LITERAL(67, 1264, 29), // "handleSetCustomFilterCallback"
QT_MOC_LITERAL(68, 1294, 8), // "callback"
QT_MOC_LITERAL(69, 1303, 28), // "handleRegisterRoutePrehandle"
QT_MOC_LITERAL(70, 1332, 23), // "FileViewRoutePrehaldler"
QT_MOC_LITERAL(71, 1356, 10), // "prehandler"
QT_MOC_LITERAL(72, 1367, 23), // "handleRegisterDataCache"
QT_MOC_LITERAL(73, 1391, 34), // "handleSetAlwaysOpenInCurrentW..."
QT_MOC_LITERAL(74, 1426, 28), // "handleAboutToChangeViewWidth"
QT_MOC_LITERAL(75, 1455, 10), // "deltaWidth"
QT_MOC_LITERAL(76, 1466, 25) // "handleCheckMountedDevPath"

    },
    "dfmplugin_workspace::WorkspaceEventReceiver\0"
    "handleTileBarSwitchModeTriggered\0\0"
    "windowId\0mode\0handleShowCustomTopWidget\0"
    "scheme\0visible\0handleSelectFiles\0"
    "QList<QUrl>\0files\0handleSelectAll\0"
    "handleReverseSelect\0handleSetSort\0"
    "dfmbase::Global::ItemRoles\0role\0"
    "handleNotSupportTreeView\0"
    "handleSetSelectionMode\0"
    "QAbstractItemView::SelectionMode\0"
    "handleSetEnabledSelectionModes\0"
    "QList<QAbstractItemView::SelectionMode>\0"
    "modes\0handleSetViewDragEnabled\0enabled\0"
    "handleSetViewDragDropMode\0"
    "QAbstractItemView::DragDropMode\0"
    "handleClosePersistentEditor\0"
    "handleGetViewFilter\0handleGetNameFilter\0"
    "handleSetViewFilter\0QDir::Filters\0"
    "filters\0handleSetNameFilter\0"
    "handleSetReadOnly\0readOnly\0"
    "handlePasteFileResult\0srcUrls\0destUrls\0"
    "ok\0errMsg\0handleMoveToTrashFileResult\0"
    "handleRenameFileResult\0QMap<QUrl,QUrl>\0"
    "renamedUrls\0handleFileUpdate\0url\0"
    "handleCurrentSortRole\0handleColumnRoles\0"
    "QList<dfmbase::Global::ItemRoles>\0"
    "handleGetVisualGeometry\0windowID\0"
    "handleGetViewItemRect\0handleSetDefaultViewMode\0"
    "dfmbase::Global::ViewMode\0"
    "handleGetDefaultViewMode\0"
    "handleGetCurrentViewMode\0"
    "handleRegisterFileView\0handleRegisterMenuScene\0"
    "scene\0handleFindMenuScene\0"
    "handleRegisterCustomTopWidget\0dataMap\0"
    "handleGetCustomTopWidgetVisible\0"
    "handleCheckSchemeViewIsFileView\0"
    "handleGetSelectedUrls\0handleSetCustomFilterData\0"
    "data\0handleSetCustomFilterCallback\0"
    "callback\0handleRegisterRoutePrehandle\0"
    "FileViewRoutePrehaldler\0prehandler\0"
    "handleRegisterDataCache\0"
    "handleSetAlwaysOpenInCurrentWindow\0"
    "handleAboutToChangeViewWidth\0deltaWidth\0"
    "handleCheckMountedDevPath"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_workspace__WorkspaceEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      42,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,  224,    2, 0x0a /* Public */,
       5,    3,  229,    2, 0x0a /* Public */,
       8,    2,  236,    2, 0x0a /* Public */,
      11,    1,  241,    2, 0x0a /* Public */,
      12,    1,  244,    2, 0x0a /* Public */,
      13,    2,  247,    2, 0x0a /* Public */,
      16,    1,  252,    2, 0x0a /* Public */,
      17,    2,  255,    2, 0x0a /* Public */,
      19,    2,  260,    2, 0x0a /* Public */,
      22,    2,  265,    2, 0x0a /* Public */,
      24,    2,  270,    2, 0x0a /* Public */,
      26,    1,  275,    2, 0x0a /* Public */,
      27,    1,  278,    2, 0x0a /* Public */,
      28,    1,  281,    2, 0x0a /* Public */,
      29,    2,  284,    2, 0x0a /* Public */,
      32,    2,  289,    2, 0x0a /* Public */,
      33,    2,  294,    2, 0x0a /* Public */,
      35,    4,  299,    2, 0x0a /* Public */,
      40,    3,  308,    2, 0x0a /* Public */,
      41,    4,  315,    2, 0x0a /* Public */,
      44,    1,  324,    2, 0x0a /* Public */,
      46,    1,  327,    2, 0x0a /* Public */,
      47,    1,  330,    2, 0x0a /* Public */,
      49,    1,  333,    2, 0x0a /* Public */,
      51,    3,  336,    2, 0x0a /* Public */,
      52,    2,  343,    2, 0x0a /* Public */,
      54,    1,  348,    2, 0x0a /* Public */,
      55,    1,  351,    2, 0x0a /* Public */,
      56,    1,  354,    2, 0x0a /* Public */,
      57,    2,  357,    2, 0x0a /* Public */,
      59,    1,  362,    2, 0x0a /* Public */,
      60,    1,  365,    2, 0x0a /* Public */,
      62,    2,  368,    2, 0x0a /* Public */,
      63,    1,  373,    2, 0x0a /* Public */,
      64,    1,  376,    2, 0x0a /* Public */,
      65,    3,  379,    2, 0x0a /* Public */,
      67,    3,  386,    2, 0x0a /* Public */,
      69,    2,  393,    2, 0x0a /* Public */,
      72,    1,  398,    2, 0x0a /* Public */,
      73,    1,  401,    2, 0x0a /* Public */,
      74,    2,  404,    2, 0x0a /* Public */,
      76,    1,  409,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::ULongLong, QMetaType::Int,    3,    4,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QString, QMetaType::Bool,    3,    6,    7,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 9,    3,   10,
    QMetaType::Void, QMetaType::ULongLong,    3,
    QMetaType::Void, QMetaType::ULongLong,    3,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 14,    3,   15,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 18,    3,    4,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 20,    3,   21,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::Bool,    3,   23,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 25,    3,    4,
    QMetaType::Void, QMetaType::ULongLong,    3,
    QMetaType::Int, QMetaType::ULongLong,    3,
    QMetaType::QStringList, QMetaType::ULongLong,    3,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 30,    3,   31,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QStringList,    3,   31,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::Bool,    3,   34,
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 9, QMetaType::Bool, QMetaType::QString,   36,   37,   38,   39,
    QMetaType::Void, 0x80000000 | 9, QMetaType::Bool, QMetaType::QString,   36,   38,   39,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 42, QMetaType::Bool, QMetaType::QString,    3,   43,   38,   39,
    QMetaType::Void, QMetaType::QUrl,   45,
    0x80000000 | 14, QMetaType::ULongLong,    3,
    0x80000000 | 48, QMetaType::ULongLong,    3,
    QMetaType::QRectF, QMetaType::ULongLong,   50,
    QMetaType::QRectF, QMetaType::ULongLong, QMetaType::QUrl, 0x80000000 | 14,   50,   45,   15,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 53,    6,    4,
    0x80000000 | 53, QMetaType::QString,    6,
    0x80000000 | 53, QMetaType::ULongLong,   50,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    6,   58,
    QMetaType::QString, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QVariantMap,   61,
    QMetaType::Bool, QMetaType::ULongLong, QMetaType::QString,   50,    6,
    QMetaType::Bool, QMetaType::QString,    6,
    0x80000000 | 9, QMetaType::ULongLong,   50,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl, QMetaType::QVariant,   50,   45,   66,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl, QMetaType::QVariant,   50,   45,   68,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 70,    6,   71,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::ULongLong,   50,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::Int,   50,   75,
    QMetaType::Bool, QMetaType::QUrl,   45,

       0        // eod
};

void dfmplugin_workspace::WorkspaceEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<WorkspaceEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleTileBarSwitchModeTriggered((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 1: _t->handleShowCustomTopWidget((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 2: _t->handleSelectFiles((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2]))); break;
        case 3: _t->handleSelectAll((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 4: _t->handleReverseSelect((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 5: _t->handleSetSort((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< dfmbase::Global::ItemRoles(*)>(_a[2]))); break;
        case 6: _t->handleNotSupportTreeView((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->handleSetSelectionMode((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QAbstractItemView::SelectionMode(*)>(_a[2]))); break;
        case 8: _t->handleSetEnabledSelectionModes((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QAbstractItemView::SelectionMode>(*)>(_a[2]))); break;
        case 9: _t->handleSetViewDragEnabled((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        case 10: _t->handleSetViewDragDropMode((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QAbstractItemView::DragDropMode(*)>(_a[2]))); break;
        case 11: _t->handleClosePersistentEditor((*reinterpret_cast< const quint64(*)>(_a[1]))); break;
        case 12: { int _r = _t->handleGetViewFilter((*reinterpret_cast< const quint64(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 13: { QStringList _r = _t->handleGetNameFilter((*reinterpret_cast< const quint64(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->handleSetViewFilter((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QDir::Filters(*)>(_a[2]))); break;
        case 15: _t->handleSetNameFilter((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2]))); break;
        case 16: _t->handleSetReadOnly((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        case 17: _t->handlePasteFileResult((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 18: _t->handleMoveToTrashFileResult((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 19: _t->handleRenameFileResult((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QMap<QUrl,QUrl>(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 20: _t->handleFileUpdate((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 21: { dfmbase::Global::ItemRoles _r = _t->handleCurrentSortRole((*reinterpret_cast< quint64(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< dfmbase::Global::ItemRoles*>(_a[0]) = std::move(_r); }  break;
        case 22: { QList<dfmbase::Global::ItemRoles> _r = _t->handleColumnRoles((*reinterpret_cast< quint64(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QList<dfmbase::Global::ItemRoles>*>(_a[0]) = std::move(_r); }  break;
        case 23: { QRectF _r = _t->handleGetVisualGeometry((*reinterpret_cast< const quint64(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QRectF*>(_a[0]) = std::move(_r); }  break;
        case 24: { QRectF _r = _t->handleGetViewItemRect((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const dfmbase::Global::ItemRoles(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QRectF*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->handleSetDefaultViewMode((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const dfmbase::Global::ViewMode(*)>(_a[2]))); break;
        case 26: { dfmbase::Global::ViewMode _r = _t->handleGetDefaultViewMode((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< dfmbase::Global::ViewMode*>(_a[0]) = std::move(_r); }  break;
        case 27: { dfmbase::Global::ViewMode _r = _t->handleGetCurrentViewMode((*reinterpret_cast< const quint64(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< dfmbase::Global::ViewMode*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->handleRegisterFileView((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 29: _t->handleRegisterMenuScene((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 30: { QString _r = _t->handleFindMenuScene((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 31: _t->handleRegisterCustomTopWidget((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 32: { bool _r = _t->handleGetCustomTopWidgetVisible((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 33: { bool _r = _t->handleCheckSchemeViewIsFileView((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 34: { QList<QUrl> _r = _t->handleGetSelectedUrls((*reinterpret_cast< const quint64(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QList<QUrl>*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->handleSetCustomFilterData((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3]))); break;
        case 36: _t->handleSetCustomFilterCallback((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3]))); break;
        case 37: { bool _r = _t->handleRegisterRoutePrehandle((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const FileViewRoutePrehaldler(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 38: _t->handleRegisterDataCache((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 39: _t->handleSetAlwaysOpenInCurrentWindow((*reinterpret_cast< const quint64(*)>(_a[1]))); break;
        case 40: _t->handleAboutToChangeViewWidth((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 41: { bool _r = _t->handleCheckMountedDevPath((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::Global::ItemRoles >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDir::Filters >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::Global::ItemRoles >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::Global::ViewMode >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_workspace::WorkspaceEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_workspace__WorkspaceEventReceiver.data,
    qt_meta_data_dfmplugin_workspace__WorkspaceEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_workspace::WorkspaceEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_workspace::WorkspaceEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_workspace__WorkspaceEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_workspace::WorkspaceEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
