/****************************************************************************
** Meta object code from reading C++ file 'dbusdisplay1.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-core/screen/dbus-private/dbusdisplay1.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbusdisplay1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DBusDisplay_t {
    QByteArrayData data[56];
    char stringdata0[814];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DBusDisplay_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DBusDisplay_t qt_meta_stringdata_DBusDisplay = {
    {
QT_MOC_LITERAL(0, 0, 11), // "DBusDisplay"
QT_MOC_LITERAL(1, 12, 15), // "MonitorsChanged"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 14), // "PrimaryChanged"
QT_MOC_LITERAL(4, 44, 18), // "DisplayModeChanged"
QT_MOC_LITERAL(5, 63, 18), // "PrimaryRectChanged"
QT_MOC_LITERAL(6, 82, 19), // "__propertyChanged__"
QT_MOC_LITERAL(7, 102, 12), // "QDBusMessage"
QT_MOC_LITERAL(8, 115, 3), // "msg"
QT_MOC_LITERAL(9, 119, 12), // "ApplyChanges"
QT_MOC_LITERAL(10, 132, 19), // "QDBusPendingReply<>"
QT_MOC_LITERAL(11, 152, 14), // "AssociateTouch"
QT_MOC_LITERAL(12, 167, 10), // "outputName"
QT_MOC_LITERAL(13, 178, 11), // "touchSerial"
QT_MOC_LITERAL(14, 190, 20), // "AssociateTouchByUUID"
QT_MOC_LITERAL(15, 211, 9), // "touchUUID"
QT_MOC_LITERAL(16, 221, 9), // "CanRotate"
QT_MOC_LITERAL(17, 231, 23), // "QDBusPendingReply<bool>"
QT_MOC_LITERAL(18, 255, 16), // "CanSetBrightness"
QT_MOC_LITERAL(19, 272, 16), // "ChangeBrightness"
QT_MOC_LITERAL(20, 289, 6), // "raised"
QT_MOC_LITERAL(21, 296, 16), // "DeleteCustomMode"
QT_MOC_LITERAL(22, 313, 4), // "name"
QT_MOC_LITERAL(23, 318, 17), // "GetBuiltinMonitor"
QT_MOC_LITERAL(24, 336, 42), // "QDBusPendingReply<QString,QDB..."
QT_MOC_LITERAL(25, 379, 19), // "QDBusReply<QString>"
QT_MOC_LITERAL(26, 399, 16), // "QDBusObjectPath&"
QT_MOC_LITERAL(27, 416, 7), // "outArg1"
QT_MOC_LITERAL(28, 424, 18), // "GetRealDisplayMode"
QT_MOC_LITERAL(29, 443, 24), // "QDBusPendingReply<uchar>"
QT_MOC_LITERAL(30, 468, 15), // "ListOutputNames"
QT_MOC_LITERAL(31, 484, 30), // "QDBusPendingReply<QStringList>"
QT_MOC_LITERAL(32, 515, 16), // "ModifyConfigName"
QT_MOC_LITERAL(33, 532, 7), // "newName"
QT_MOC_LITERAL(34, 540, 17), // "RefreshBrightness"
QT_MOC_LITERAL(35, 558, 5), // "Reset"
QT_MOC_LITERAL(36, 564, 12), // "ResetChanges"
QT_MOC_LITERAL(37, 577, 4), // "Save"
QT_MOC_LITERAL(38, 582, 20), // "SetAndSaveBrightness"
QT_MOC_LITERAL(39, 603, 5), // "value"
QT_MOC_LITERAL(40, 609, 13), // "SetBrightness"
QT_MOC_LITERAL(41, 623, 19), // "SetColorTemperature"
QT_MOC_LITERAL(42, 643, 18), // "SetMethodAdjustCCT"
QT_MOC_LITERAL(43, 662, 12), // "adjustMethod"
QT_MOC_LITERAL(44, 675, 10), // "SetPrimary"
QT_MOC_LITERAL(45, 686, 10), // "SwitchMode"
QT_MOC_LITERAL(46, 697, 4), // "mode"
QT_MOC_LITERAL(47, 702, 11), // "DisplayMode"
QT_MOC_LITERAL(48, 714, 10), // "HasChanged"
QT_MOC_LITERAL(49, 725, 8), // "Monitors"
QT_MOC_LITERAL(50, 734, 22), // "QList<QDBusObjectPath>"
QT_MOC_LITERAL(51, 757, 7), // "Primary"
QT_MOC_LITERAL(52, 765, 11), // "PrimaryRect"
QT_MOC_LITERAL(53, 777, 11), // "DisplayRect"
QT_MOC_LITERAL(54, 789, 12), // "ScreenHeight"
QT_MOC_LITERAL(55, 802, 11) // "ScreenWidth"

    },
    "DBusDisplay\0MonitorsChanged\0\0"
    "PrimaryChanged\0DisplayModeChanged\0"
    "PrimaryRectChanged\0__propertyChanged__\0"
    "QDBusMessage\0msg\0ApplyChanges\0"
    "QDBusPendingReply<>\0AssociateTouch\0"
    "outputName\0touchSerial\0AssociateTouchByUUID\0"
    "touchUUID\0CanRotate\0QDBusPendingReply<bool>\0"
    "CanSetBrightness\0ChangeBrightness\0"
    "raised\0DeleteCustomMode\0name\0"
    "GetBuiltinMonitor\0"
    "QDBusPendingReply<QString,QDBusObjectPath>\0"
    "QDBusReply<QString>\0QDBusObjectPath&\0"
    "outArg1\0GetRealDisplayMode\0"
    "QDBusPendingReply<uchar>\0ListOutputNames\0"
    "QDBusPendingReply<QStringList>\0"
    "ModifyConfigName\0newName\0RefreshBrightness\0"
    "Reset\0ResetChanges\0Save\0SetAndSaveBrightness\0"
    "value\0SetBrightness\0SetColorTemperature\0"
    "SetMethodAdjustCCT\0adjustMethod\0"
    "SetPrimary\0SwitchMode\0mode\0DisplayMode\0"
    "HasChanged\0Monitors\0QList<QDBusObjectPath>\0"
    "Primary\0PrimaryRect\0DisplayRect\0"
    "ScreenHeight\0ScreenWidth"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DBusDisplay[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      27,   14, // methods
       7,  216, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  149,    2, 0x06 /* Public */,
       3,    0,  150,    2, 0x06 /* Public */,
       4,    0,  151,    2, 0x06 /* Public */,
       5,    0,  152,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,  153,    2, 0x08 /* Private */,
       9,    0,  156,    2, 0x0a /* Public */,
      11,    2,  157,    2, 0x0a /* Public */,
      14,    2,  162,    2, 0x0a /* Public */,
      16,    0,  167,    2, 0x0a /* Public */,
      18,    1,  168,    2, 0x0a /* Public */,
      19,    1,  171,    2, 0x0a /* Public */,
      21,    1,  174,    2, 0x0a /* Public */,
      23,    0,  177,    2, 0x0a /* Public */,
      23,    1,  178,    2, 0x0a /* Public */,
      28,    0,  181,    2, 0x0a /* Public */,
      30,    0,  182,    2, 0x0a /* Public */,
      32,    2,  183,    2, 0x0a /* Public */,
      34,    0,  188,    2, 0x0a /* Public */,
      35,    0,  189,    2, 0x0a /* Public */,
      36,    0,  190,    2, 0x0a /* Public */,
      37,    0,  191,    2, 0x0a /* Public */,
      38,    2,  192,    2, 0x0a /* Public */,
      40,    2,  197,    2, 0x0a /* Public */,
      41,    1,  202,    2, 0x0a /* Public */,
      42,    1,  205,    2, 0x0a /* Public */,
      44,    1,  208,    2, 0x0a /* Public */,
      45,    2,  211,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7,    8,
    0x80000000 | 10,
    0x80000000 | 10, QMetaType::QString, QMetaType::QString,   12,   13,
    0x80000000 | 10, QMetaType::QString, QMetaType::QString,   12,   15,
    0x80000000 | 17,
    0x80000000 | 17, QMetaType::QString,   12,
    0x80000000 | 10, QMetaType::Bool,   20,
    0x80000000 | 10, QMetaType::QString,   22,
    0x80000000 | 24,
    0x80000000 | 25, 0x80000000 | 26,   27,
    0x80000000 | 29,
    0x80000000 | 31,
    0x80000000 | 10, QMetaType::QString, QMetaType::QString,   22,   33,
    0x80000000 | 10,
    0x80000000 | 10,
    0x80000000 | 10,
    0x80000000 | 10,
    0x80000000 | 10, QMetaType::QString, QMetaType::Double,   12,   39,
    0x80000000 | 10, QMetaType::QString, QMetaType::Double,   12,   39,
    0x80000000 | 10, QMetaType::Int,   39,
    0x80000000 | 10, QMetaType::Int,   43,
    0x80000000 | 10, QMetaType::QString,   12,
    0x80000000 | 10, QMetaType::UChar, QMetaType::QString,   46,   22,

 // properties: name, type, flags
      47, QMetaType::UChar, 0x00495001,
      48, QMetaType::Bool, 0x00095001,
      49, 0x80000000 | 50, 0x00495009,
      51, QMetaType::QString, 0x00495001,
      52, 0x80000000 | 53, 0x00495009,
      54, QMetaType::UShort, 0x00095001,
      55, QMetaType::UShort, 0x00095001,

 // properties: notify_signal_id
       2,
       0,
       0,
       1,
       3,
       0,
       0,

       0        // eod
};

void DBusDisplay::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DBusDisplay *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->MonitorsChanged(); break;
        case 1: _t->PrimaryChanged(); break;
        case 2: _t->DisplayModeChanged(); break;
        case 3: _t->PrimaryRectChanged(); break;
        case 4: _t->__propertyChanged__((*reinterpret_cast< const QDBusMessage(*)>(_a[1]))); break;
        case 5: { QDBusPendingReply<> _r = _t->ApplyChanges();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->AssociateTouch((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->AssociateTouchByUUID((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<bool> _r = _t->CanRotate();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<bool> _r = _t->CanSetBrightness((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<> _r = _t->ChangeBrightness((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<> _r = _t->DeleteCustomMode((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<QString,QDBusObjectPath> _r = _t->GetBuiltinMonitor();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString,QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusReply<QString> _r = _t->GetBuiltinMonitor((*reinterpret_cast< QDBusObjectPath(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<uchar> _r = _t->GetRealDisplayMode();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<uchar>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<QStringList> _r = _t->ListOutputNames();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 16: { QDBusPendingReply<> _r = _t->ModifyConfigName((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 17: { QDBusPendingReply<> _r = _t->RefreshBrightness();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 18: { QDBusPendingReply<> _r = _t->Reset();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 19: { QDBusPendingReply<> _r = _t->ResetChanges();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 20: { QDBusPendingReply<> _r = _t->Save();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 21: { QDBusPendingReply<> _r = _t->SetAndSaveBrightness((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 22: { QDBusPendingReply<> _r = _t->SetBrightness((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 23: { QDBusPendingReply<> _r = _t->SetColorTemperature((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 24: { QDBusPendingReply<> _r = _t->SetMethodAdjustCCT((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 25: { QDBusPendingReply<> _r = _t->SetPrimary((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 26: { QDBusPendingReply<> _r = _t->SwitchMode((*reinterpret_cast< uchar(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusMessage >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DBusDisplay::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusDisplay::MonitorsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DBusDisplay::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusDisplay::PrimaryChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DBusDisplay::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusDisplay::DisplayModeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DBusDisplay::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusDisplay::PrimaryRectChanged)) {
                *result = 3;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< DisplayRect >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QDBusObjectPath> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DBusDisplay *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< uchar*>(_v) = _t->displayMode(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->hasChanged(); break;
        case 2: *reinterpret_cast< QList<QDBusObjectPath>*>(_v) = _t->monitors(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->primary(); break;
        case 4: *reinterpret_cast< DisplayRect*>(_v) = _t->primaryRect(); break;
        case 5: *reinterpret_cast< ushort*>(_v) = _t->screenHeight(); break;
        case 6: *reinterpret_cast< ushort*>(_v) = _t->screenWidth(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject DBusDisplay::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_DBusDisplay.data,
    qt_meta_data_DBusDisplay,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DBusDisplay::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DBusDisplay::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DBusDisplay.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int DBusDisplay::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void DBusDisplay::MonitorsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DBusDisplay::PrimaryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DBusDisplay::DisplayModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void DBusDisplay::PrimaryRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
