/****************************************************************************
** Meta object code from reading C++ file 'videopreviewplugin.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../../dde-file-manager/src/apps/dde-file-manager-preview/pluginpreviews/video-preview/videopreviewplugin.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'videopreviewplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_plugin_filepreview__VideoPreviewPlugin_t {
    QByteArrayData data[5];
    char stringdata0[81];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_plugin_filepreview__VideoPreviewPlugin_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_plugin_filepreview__VideoPreviewPlugin_t qt_meta_stringdata_plugin_filepreview__VideoPreviewPlugin = {
    {
QT_MOC_LITERAL(0, 0, 38), // "plugin_filepreview::VideoPrev..."
QT_MOC_LITERAL(1, 39, 6), // "create"
QT_MOC_LITERAL(2, 46, 29), // "dfmbase::AbstractBasePreview*"
QT_MOC_LITERAL(3, 76, 0), // ""
QT_MOC_LITERAL(4, 77, 3) // "key"

    },
    "plugin_filepreview::VideoPreviewPlugin\0"
    "create\0dfmbase::AbstractBasePreview*\0"
    "\0key"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_plugin_filepreview__VideoPreviewPlugin[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    3, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 2, QMetaType::QString,    4,

       0        // eod
};

void plugin_filepreview::VideoPreviewPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<VideoPreviewPlugin *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { dfmbase::AbstractBasePreview* _r = _t->create((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< dfmbase::AbstractBasePreview**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject plugin_filepreview::VideoPreviewPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<dfmbase::AbstractFilePreviewPlugin::staticMetaObject>(),
    qt_meta_stringdata_plugin_filepreview__VideoPreviewPlugin.data,
    qt_meta_data_plugin_filepreview__VideoPreviewPlugin,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *plugin_filepreview::VideoPreviewPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *plugin_filepreview::VideoPreviewPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_plugin_filepreview__VideoPreviewPlugin.stringdata0))
        return static_cast<void*>(this);
    return dfmbase::AbstractFilePreviewPlugin::qt_metacast(_clname);
}

int plugin_filepreview::VideoPreviewPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = dfmbase::AbstractFilePreviewPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

QT_PLUGIN_METADATA_SECTION
static constexpr unsigned char qt_pluginMetaData[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x36,  'c',  'o',  'm',  '.',  'd', 
    'e',  'e',  'p',  'i',  'n',  '.',  'f',  'i', 
    'l',  'e',  'm',  'a',  'n',  'a',  'g',  'e', 
    'r',  '.',  'F',  'i',  'l',  'e',  'P',  'r', 
    'e',  'v',  'i',  'e',  'w',  'F',  'a',  'c', 
    't',  'o',  'r',  'y',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '_',  'i',  'i', 
    'd', 
    // "className"
    0x03,  0x72,  'V',  'i',  'd',  'e',  'o',  'P', 
    'r',  'e',  'v',  'i',  'e',  'w',  'P',  'l', 
    'u',  'g',  'i',  'n', 
    // "MetaData"
    0x04,  0xa1,  0x64,  'K',  'e',  'y',  's',  0x82, 
    0x67,  'v',  'i',  'd',  'e',  'o',  '/',  '*', 
    0x78,  0x21,  'a',  'p',  'p',  'l',  'i',  'c', 
    'a',  't',  'i',  'o',  'n',  '/',  'v',  'n', 
    'd',  '.',  'a',  'd',  'o',  'b',  'e',  '.', 
    'f',  'l',  'a',  's',  'h',  '.',  'm',  'o', 
    'v',  'i',  'e', 
    0xff, 
};
using namespace plugin_filepreview;
QT_MOC_EXPORT_PLUGIN(plugin_filepreview::VideoPreviewPlugin, VideoPreviewPlugin)

QT_WARNING_POP
QT_END_MOC_NAMESPACE
