/****************************************************************************
** Meta object code from reading C++ file 'global_server_defines.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../dde-file-manager/src/dfm-base/dbusservice/global_server_defines.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'global_server_defines.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_GlobalServerDefines_t {
    QByteArrayData data[11];
    char stringdata0[129];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_GlobalServerDefines_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_GlobalServerDefines_t qt_meta_stringdata_GlobalServerDefines = {
    {
QT_MOC_LITERAL(0, 0, 19), // "GlobalServerDefines"
QT_MOC_LITERAL(1, 20, 17), // "DeviceQueryOption"
QT_MOC_LITERAL(2, 38, 12), // "kNoCondition"
QT_MOC_LITERAL(3, 51, 10), // "kMountable"
QT_MOC_LITERAL(4, 62, 8), // "kMounted"
QT_MOC_LITERAL(5, 71, 10), // "kRemovable"
QT_MOC_LITERAL(6, 82, 11), // "kNotIgnored"
QT_MOC_LITERAL(7, 94, 11), // "kNotMounted"
QT_MOC_LITERAL(8, 106, 8), // "kOptical"
QT_MOC_LITERAL(9, 115, 7), // "kSystem"
QT_MOC_LITERAL(10, 123, 5) // "kLoop"

    },
    "GlobalServerDefines\0DeviceQueryOption\0"
    "kNoCondition\0kMountable\0kMounted\0"
    "kRemovable\0kNotIgnored\0kNotMounted\0"
    "kOptical\0kSystem\0kLoop"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_GlobalServerDefines[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       1,   14, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // enums: name, alias, flags, count, data
       1,    1, 0x0,    9,   19,

 // enum data: key, value
       2, uint(GlobalServerDefines::kNoCondition),
       3, uint(GlobalServerDefines::kMountable),
       4, uint(GlobalServerDefines::kMounted),
       5, uint(GlobalServerDefines::kRemovable),
       6, uint(GlobalServerDefines::kNotIgnored),
       7, uint(GlobalServerDefines::kNotMounted),
       8, uint(GlobalServerDefines::kOptical),
       9, uint(GlobalServerDefines::kSystem),
      10, uint(GlobalServerDefines::kLoop),

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject GlobalServerDefines::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_GlobalServerDefines.data,
    qt_meta_data_GlobalServerDefines,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
