/****************************************************************************
** Meta object code from reading C++ file 'devicewatcher.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../dde-file-manager/src/dfm-base/base/device/private/devicewatcher.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'devicewatcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmbase__DeviceWatcher_t {
    QByteArrayData data[20];
    char stringdata0[313];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__DeviceWatcher_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__DeviceWatcher_t qt_meta_stringdata_dfmbase__DeviceWatcher = {
    {
QT_MOC_LITERAL(0, 0, 22), // "dfmbase::DeviceWatcher"
QT_MOC_LITERAL(1, 23, 13), // "onBlkDevAdded"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 2), // "id"
QT_MOC_LITERAL(4, 41, 15), // "onBlkDevRemoved"
QT_MOC_LITERAL(5, 57, 15), // "onBlkDevMounted"
QT_MOC_LITERAL(6, 73, 3), // "mpt"
QT_MOC_LITERAL(7, 77, 17), // "onBlkDevUnmounted"
QT_MOC_LITERAL(8, 95, 14), // "onBlkDevLocked"
QT_MOC_LITERAL(9, 110, 16), // "onBlkDevUnlocked"
QT_MOC_LITERAL(10, 127, 11), // "cleartextId"
QT_MOC_LITERAL(11, 139, 15), // "onBlkDevFsAdded"
QT_MOC_LITERAL(12, 155, 17), // "onBlkDevFsRemoved"
QT_MOC_LITERAL(13, 173, 25), // "onBlkDevPropertiesChanged"
QT_MOC_LITERAL(14, 199, 33), // "QMap<dfmmount::Property,QVari..."
QT_MOC_LITERAL(15, 233, 7), // "changes"
QT_MOC_LITERAL(16, 241, 15), // "onProtoDevAdded"
QT_MOC_LITERAL(17, 257, 17), // "onProtoDevRemoved"
QT_MOC_LITERAL(18, 275, 17), // "onProtoDevMounted"
QT_MOC_LITERAL(19, 293, 19) // "onProtoDevUnmounted"

    },
    "dfmbase::DeviceWatcher\0onBlkDevAdded\0"
    "\0id\0onBlkDevRemoved\0onBlkDevMounted\0"
    "mpt\0onBlkDevUnmounted\0onBlkDevLocked\0"
    "onBlkDevUnlocked\0cleartextId\0"
    "onBlkDevFsAdded\0onBlkDevFsRemoved\0"
    "onBlkDevPropertiesChanged\0"
    "QMap<dfmmount::Property,QVariant>\0"
    "changes\0onProtoDevAdded\0onProtoDevRemoved\0"
    "onProtoDevMounted\0onProtoDevUnmounted"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__DeviceWatcher[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x08 /* Private */,
       4,    1,   82,    2, 0x08 /* Private */,
       5,    2,   85,    2, 0x08 /* Private */,
       7,    1,   90,    2, 0x08 /* Private */,
       8,    1,   93,    2, 0x08 /* Private */,
       9,    2,   96,    2, 0x08 /* Private */,
      11,    1,  101,    2, 0x08 /* Private */,
      12,    1,  104,    2, 0x08 /* Private */,
      13,    2,  107,    2, 0x08 /* Private */,
      16,    1,  112,    2, 0x08 /* Private */,
      17,    1,  115,    2, 0x08 /* Private */,
      18,    2,  118,    2, 0x08 /* Private */,
      19,    1,  123,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    6,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,   10,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 14,    3,   15,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    6,
    QMetaType::Void, QMetaType::QString,    3,

       0        // eod
};

void dfmbase::DeviceWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DeviceWatcher *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onBlkDevAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->onBlkDevRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->onBlkDevMounted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->onBlkDevUnmounted((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->onBlkDevLocked((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->onBlkDevUnlocked((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->onBlkDevFsAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->onBlkDevFsRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->onBlkDevPropertiesChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QMap<dfmmount::Property,QVariant>(*)>(_a[2]))); break;
        case 9: _t->onProtoDevAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->onProtoDevRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->onProtoDevMounted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 12: _t->onProtoDevUnmounted((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::DeviceWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__DeviceWatcher.data,
    qt_meta_data_dfmbase__DeviceWatcher,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::DeviceWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::DeviceWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__DeviceWatcher.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmbase::DeviceWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
