/****************************************************************************
** Meta object code from reading C++ file 'filedialogmanagerdbus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filedialog/filedialogplugin-core/dbus/filedialogmanagerdbus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filedialogmanagerdbus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FileDialogManagerDBus_t {
    QByteArrayData data[22];
    char stringdata0[315];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FileDialogManagerDBus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FileDialogManagerDBus_t qt_meta_stringdata_FileDialogManagerDBus = {
    {
QT_MOC_LITERAL(0, 0, 21), // "FileDialogManagerDBus"
QT_MOC_LITERAL(1, 22, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 38, 33), // "com.deepin.filemanager.filedi..."
QT_MOC_LITERAL(3, 72, 12), // "createDialog"
QT_MOC_LITERAL(4, 85, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(5, 101, 0), // ""
QT_MOC_LITERAL(6, 102, 3), // "key"
QT_MOC_LITERAL(7, 106, 13), // "destroyDialog"
QT_MOC_LITERAL(8, 120, 4), // "path"
QT_MOC_LITERAL(9, 125, 7), // "dialogs"
QT_MOC_LITERAL(10, 133, 22), // "QList<QDBusObjectPath>"
QT_MOC_LITERAL(11, 156, 11), // "errorString"
QT_MOC_LITERAL(12, 168, 22), // "isUseFileChooserDialog"
QT_MOC_LITERAL(13, 191, 23), // "canUseFileChooserDialog"
QT_MOC_LITERAL(14, 215, 5), // "group"
QT_MOC_LITERAL(15, 221, 18), // "executableFileName"
QT_MOC_LITERAL(16, 240, 19), // "globPatternsForMime"
QT_MOC_LITERAL(17, 260, 8), // "mimeType"
QT_MOC_LITERAL(18, 269, 12), // "monitorFiles"
QT_MOC_LITERAL(19, 282, 24), // "showBluetoothTransDialog"
QT_MOC_LITERAL(20, 307, 2), // "id"
QT_MOC_LITERAL(21, 310, 4) // "URIs"

    },
    "FileDialogManagerDBus\0D-Bus Interface\0"
    "com.deepin.filemanager.filedialog\0"
    "createDialog\0QDBusObjectPath\0\0key\0"
    "destroyDialog\0path\0dialogs\0"
    "QList<QDBusObjectPath>\0errorString\0"
    "isUseFileChooserDialog\0canUseFileChooserDialog\0"
    "group\0executableFileName\0globPatternsForMime\0"
    "mimeType\0monitorFiles\0showBluetoothTransDialog\0"
    "id\0URIs"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FileDialogManagerDBus[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       9,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // slots: name, argc, parameters, tag, flags
       3,    1,   61,    5, 0x0a /* Public */,
       7,    1,   64,    5, 0x0a /* Public */,
       9,    0,   67,    5, 0x0a /* Public */,
      11,    0,   68,    5, 0x0a /* Public */,
      12,    0,   69,    5, 0x0a /* Public */,
      13,    2,   70,    5, 0x0a /* Public */,
      16,    1,   75,    5, 0x0a /* Public */,
      18,    0,   78,    5, 0x0a /* Public */,
      19,    2,   79,    5, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 4, QMetaType::QString,    6,
    QMetaType::Void, 0x80000000 | 4,    8,
    0x80000000 | 10,
    QMetaType::QString,
    QMetaType::Bool,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString,   14,   15,
    QMetaType::QStringList, QMetaType::QString,   17,
    QMetaType::QStringList,
    QMetaType::Void, QMetaType::QString, QMetaType::QStringList,   20,   21,

       0        // eod
};

void FileDialogManagerDBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileDialogManagerDBus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QDBusObjectPath _r = _t->createDialog((*reinterpret_cast< QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusObjectPath*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->destroyDialog((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1]))); break;
        case 2: { QList<QDBusObjectPath> _r = _t->dialogs();
            if (_a[0]) *reinterpret_cast< QList<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QString _r = _t->errorString();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->isUseFileChooserDialog();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->canUseFileChooserDialog((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { QStringList _r = _t->globPatternsForMime((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 7: { QStringList _r = _t->monitorFiles();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->showBluetoothTransDialog((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FileDialogManagerDBus::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_FileDialogManagerDBus.data,
    qt_meta_data_FileDialogManagerDBus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FileDialogManagerDBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileDialogManagerDBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FileDialogManagerDBus.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int FileDialogManagerDBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
