/****************************************************************************
** Meta object code from reading C++ file 'addressbar_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-titlebar/views/private/addressbar_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'addressbar_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_titlebar__AddressBarPrivate_t {
    QByteArrayData data[17];
    char stringdata0[307];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_titlebar__AddressBarPrivate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_titlebar__AddressBarPrivate_t qt_meta_stringdata_dfmplugin_titlebar__AddressBarPrivate = {
    {
QT_MOC_LITERAL(0, 0, 37), // "dfmplugin_titlebar::AddressBa..."
QT_MOC_LITERAL(1, 38, 12), // "startSpinner"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 11), // "stopSpinner"
QT_MOC_LITERAL(4, 64, 12), // "onTextEdited"
QT_MOC_LITERAL(5, 77, 4), // "text"
QT_MOC_LITERAL(6, 82, 15), // "onReturnPressed"
QT_MOC_LITERAL(7, 98, 16), // "insertCompletion"
QT_MOC_LITERAL(8, 115, 10), // "completion"
QT_MOC_LITERAL(9, 126, 23), // "onCompletionHighlighted"
QT_MOC_LITERAL(10, 150, 21), // "highlightedCompletion"
QT_MOC_LITERAL(11, 172, 19), // "updateIndicatorIcon"
QT_MOC_LITERAL(12, 192, 29), // "onCompletionModelCountChanged"
QT_MOC_LITERAL(13, 222, 22), // "appendToCompleterModel"
QT_MOC_LITERAL(14, 245, 10), // "stringList"
QT_MOC_LITERAL(15, 256, 30), // "onTravelCompletionListFinished"
QT_MOC_LITERAL(16, 287, 19) // "onIndicatorTriggerd"

    },
    "dfmplugin_titlebar::AddressBarPrivate\0"
    "startSpinner\0\0stopSpinner\0onTextEdited\0"
    "text\0onReturnPressed\0insertCompletion\0"
    "completion\0onCompletionHighlighted\0"
    "highlightedCompletion\0updateIndicatorIcon\0"
    "onCompletionModelCountChanged\0"
    "appendToCompleterModel\0stringList\0"
    "onTravelCompletionListFinished\0"
    "onIndicatorTriggerd"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_titlebar__AddressBarPrivate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x0a /* Public */,
       3,    0,   70,    2, 0x0a /* Public */,
       4,    1,   71,    2, 0x0a /* Public */,
       6,    0,   74,    2, 0x0a /* Public */,
       7,    1,   75,    2, 0x0a /* Public */,
       9,    1,   78,    2, 0x0a /* Public */,
      11,    0,   81,    2, 0x0a /* Public */,
      12,    0,   82,    2, 0x0a /* Public */,
      13,    1,   83,    2, 0x0a /* Public */,
      15,    0,   86,    2, 0x0a /* Public */,
      16,    0,   87,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QStringList,   14,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_titlebar::AddressBarPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AddressBarPrivate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->startSpinner(); break;
        case 1: _t->stopSpinner(); break;
        case 2: _t->onTextEdited((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->onReturnPressed(); break;
        case 4: _t->insertCompletion((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->onCompletionHighlighted((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->updateIndicatorIcon(); break;
        case 7: _t->onCompletionModelCountChanged(); break;
        case 8: _t->appendToCompleterModel((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 9: _t->onTravelCompletionListFinished(); break;
        case 10: _t->onIndicatorTriggerd(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_titlebar::AddressBarPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_titlebar__AddressBarPrivate.data,
    qt_meta_data_dfmplugin_titlebar__AddressBarPrivate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_titlebar::AddressBarPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_titlebar::AddressBarPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_titlebar__AddressBarPrivate.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_titlebar::AddressBarPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
