// SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "selectionhookinterface.h"

using namespace ddplugin_canvas;

SelectionHookInterface::SelectionHookInterface()
{

}

SelectionHookInterface::~SelectionHookInterface()
{

}

void SelectionHookInterface::clear() const
{

}
