// SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef DAEMONPLUGIN_CORE_GLOBAL_H
#define DAEMONPLUGIN_CORE_GLOBAL_H

#include <dfm-base/dfm_log_defines.h>

#define DAEMONPCORE_NAMESPACE daemonplugin_core

#define DAEMONPCORE_BEGIN_NAMESPACE namespace DAEMONPCORE_NAMESPACE {
#define DAEMONPCORE_END_NAMESPACE }
#define DAEMONPCORE_USE_NAMESPACE using namespace DAEMONPCORE_NAMESPACE;

DAEMONPCORE_BEGIN_NAMESPACE
DFM_LOG_USE_CATEGORY(DAEMONPCORE_NAMESPACE)
DAEMONPCORE_END_NAMESPACE

#endif   // DAEMONPLUGIN_CORE_GLOBAL_H
