/****************************************************************************
** Meta object code from reading C++ file 'reportlogmanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-utils/reportlog/reportlogmanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'reportlogmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_utils__ReportLogManager_t {
    QByteArrayData data[20];
    char stringdata0[248];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_utils__ReportLogManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_utils__ReportLogManager_t qt_meta_stringdata_dfmplugin_utils__ReportLogManager = {
    {
QT_MOC_LITERAL(0, 0, 33), // "dfmplugin_utils::ReportLogMan..."
QT_MOC_LITERAL(1, 34, 16), // "requestCommitLog"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 4), // "type"
QT_MOC_LITERAL(4, 57, 4), // "args"
QT_MOC_LITERAL(5, 62, 21), // "requestReportMenuData"
QT_MOC_LITERAL(6, 84, 4), // "name"
QT_MOC_LITERAL(7, 89, 11), // "QList<QUrl>"
QT_MOC_LITERAL(8, 101, 7), // "urlList"
QT_MOC_LITERAL(9, 109, 29), // "requestReportNetworkMountData"
QT_MOC_LITERAL(10, 139, 3), // "ret"
QT_MOC_LITERAL(11, 143, 21), // "dfmmount::DeviceError"
QT_MOC_LITERAL(12, 165, 3), // "err"
QT_MOC_LITERAL(13, 169, 3), // "msg"
QT_MOC_LITERAL(14, 173, 27), // "requestReportBlockMountData"
QT_MOC_LITERAL(15, 201, 2), // "id"
QT_MOC_LITERAL(16, 204, 6), // "result"
QT_MOC_LITERAL(17, 211, 27), // "requestReportDesktopStartUp"
QT_MOC_LITERAL(18, 239, 3), // "key"
QT_MOC_LITERAL(19, 243, 4) // "data"

    },
    "dfmplugin_utils::ReportLogManager\0"
    "requestCommitLog\0\0type\0args\0"
    "requestReportMenuData\0name\0QList<QUrl>\0"
    "urlList\0requestReportNetworkMountData\0"
    "ret\0dfmmount::DeviceError\0err\0msg\0"
    "requestReportBlockMountData\0id\0result\0"
    "requestReportDesktopStartUp\0key\0data"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_utils__ReportLogManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   39,    2, 0x06 /* Public */,
       5,    2,   44,    2, 0x06 /* Public */,
       9,    3,   49,    2, 0x06 /* Public */,
      14,    2,   56,    2, 0x06 /* Public */,
      17,    2,   61,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QVariantMap,    3,    4,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 7,    6,    8,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 11, QMetaType::QString,   10,   12,   13,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,   15,   16,
    QMetaType::Void, QMetaType::QString, QMetaType::QVariant,   18,   19,

       0        // eod
};

void dfmplugin_utils::ReportLogManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ReportLogManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestCommitLog((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2]))); break;
        case 1: _t->requestReportMenuData((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2]))); break;
        case 2: _t->requestReportNetworkMountData((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< dfmmount::DeviceError(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 3: _t->requestReportBlockMountData((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 4: _t->requestReportDesktopStartUp((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ReportLogManager::*)(const QString & , const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ReportLogManager::requestCommitLog)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ReportLogManager::*)(const QString & , const QList<QUrl> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ReportLogManager::requestReportMenuData)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ReportLogManager::*)(bool , dfmmount::DeviceError , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ReportLogManager::requestReportNetworkMountData)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ReportLogManager::*)(const QString & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ReportLogManager::requestReportBlockMountData)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ReportLogManager::*)(const QString & , const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ReportLogManager::requestReportDesktopStartUp)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_utils::ReportLogManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_utils__ReportLogManager.data,
    qt_meta_data_dfmplugin_utils__ReportLogManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_utils::ReportLogManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_utils::ReportLogManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_utils__ReportLogManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_utils::ReportLogManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_utils::ReportLogManager::requestCommitLog(const QString & _t1, const QVariantMap & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_utils::ReportLogManager::requestReportMenuData(const QString & _t1, const QList<QUrl> & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_utils::ReportLogManager::requestReportNetworkMountData(bool _t1, dfmmount::DeviceError _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_utils::ReportLogManager::requestReportBlockMountData(const QString & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_utils::ReportLogManager::requestReportDesktopStartUp(const QString & _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
