/****************************************************************************
** Meta object code from reading C++ file 'bluetoothmanager_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-utils/bluetooth/private/bluetoothmanager_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bluetoothmanager_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_utils__BluetoothManagerPrivate_t {
    QByteArrayData data[26];
    char stringdata0[401];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_utils__BluetoothManagerPrivate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_utils__BluetoothManagerPrivate_t qt_meta_stringdata_dfmplugin_utils__BluetoothManagerPrivate = {
    {
QT_MOC_LITERAL(0, 0, 40), // "dfmplugin_utils::BluetoothMan..."
QT_MOC_LITERAL(1, 41, 21), // "onServiceValidChanged"
QT_MOC_LITERAL(2, 63, 0), // ""
QT_MOC_LITERAL(3, 64, 5), // "valid"
QT_MOC_LITERAL(4, 70, 14), // "onAdapterAdded"
QT_MOC_LITERAL(5, 85, 4), // "json"
QT_MOC_LITERAL(6, 90, 16), // "onAdapterRemoved"
QT_MOC_LITERAL(7, 107, 26), // "onAdapterPropertiesChanged"
QT_MOC_LITERAL(8, 134, 13), // "onDeviceAdded"
QT_MOC_LITERAL(9, 148, 15), // "onDeviceRemoved"
QT_MOC_LITERAL(10, 164, 25), // "onDevicePropertiesChanged"
QT_MOC_LITERAL(11, 190, 17), // "onTransferCreated"
QT_MOC_LITERAL(12, 208, 4), // "file"
QT_MOC_LITERAL(13, 213, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(14, 229, 12), // "transferPath"
QT_MOC_LITERAL(15, 242, 11), // "sessionPath"
QT_MOC_LITERAL(16, 254, 17), // "onTransferRemoved"
QT_MOC_LITERAL(17, 272, 4), // "done"
QT_MOC_LITERAL(18, 277, 20), // "onObexSessionCreated"
QT_MOC_LITERAL(19, 298, 20), // "onObexSessionRemoved"
QT_MOC_LITERAL(20, 319, 21), // "onObexSessionProgress"
QT_MOC_LITERAL(21, 341, 9), // "totalSize"
QT_MOC_LITERAL(22, 351, 11), // "transferred"
QT_MOC_LITERAL(23, 363, 12), // "currentIndex"
QT_MOC_LITERAL(24, 376, 16), // "onTransferFailed"
QT_MOC_LITERAL(25, 393, 7) // "errInfo"

    },
    "dfmplugin_utils::BluetoothManagerPrivate\0"
    "onServiceValidChanged\0\0valid\0"
    "onAdapterAdded\0json\0onAdapterRemoved\0"
    "onAdapterPropertiesChanged\0onDeviceAdded\0"
    "onDeviceRemoved\0onDevicePropertiesChanged\0"
    "onTransferCreated\0file\0QDBusObjectPath\0"
    "transferPath\0sessionPath\0onTransferRemoved\0"
    "done\0onObexSessionCreated\0"
    "onObexSessionRemoved\0onObexSessionProgress\0"
    "totalSize\0transferred\0currentIndex\0"
    "onTransferFailed\0errInfo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_utils__BluetoothManagerPrivate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x0a /* Public */,
       4,    1,   82,    2, 0x0a /* Public */,
       6,    1,   85,    2, 0x0a /* Public */,
       7,    1,   88,    2, 0x0a /* Public */,
       8,    1,   91,    2, 0x0a /* Public */,
       9,    1,   94,    2, 0x0a /* Public */,
      10,    1,   97,    2, 0x0a /* Public */,
      11,    3,  100,    2, 0x0a /* Public */,
      16,    4,  107,    2, 0x0a /* Public */,
      18,    1,  116,    2, 0x0a /* Public */,
      19,    1,  119,    2, 0x0a /* Public */,
      20,    4,  122,    2, 0x0a /* Public */,
      24,    3,  131,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 13, 0x80000000 | 13,   12,   14,   15,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 13, 0x80000000 | 13, QMetaType::Bool,   12,   14,   15,   17,
    QMetaType::Void, 0x80000000 | 13,   15,
    QMetaType::Void, 0x80000000 | 13,   15,
    QMetaType::Void, 0x80000000 | 13, QMetaType::ULongLong, QMetaType::ULongLong, QMetaType::Int,   15,   21,   22,   23,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 13, QMetaType::QString,   12,   15,   25,

       0        // eod
};

void dfmplugin_utils::BluetoothManagerPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BluetoothManagerPrivate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onServiceValidChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onAdapterAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->onAdapterRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->onAdapterPropertiesChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->onDeviceAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->onDeviceRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->onDevicePropertiesChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->onTransferCreated((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QDBusObjectPath(*)>(_a[2])),(*reinterpret_cast< const QDBusObjectPath(*)>(_a[3]))); break;
        case 8: _t->onTransferRemoved((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QDBusObjectPath(*)>(_a[2])),(*reinterpret_cast< const QDBusObjectPath(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4]))); break;
        case 9: _t->onObexSessionCreated((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1]))); break;
        case 10: _t->onObexSessionRemoved((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1]))); break;
        case 11: _t->onObexSessionProgress((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< qulonglong(*)>(_a[2])),(*reinterpret_cast< qulonglong(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 12: _t->onTransferFailed((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QDBusObjectPath(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_utils::BluetoothManagerPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_utils__BluetoothManagerPrivate.data,
    qt_meta_data_dfmplugin_utils__BluetoothManagerPrivate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_utils::BluetoothManagerPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_utils::BluetoothManagerPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_utils__BluetoothManagerPrivate.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_utils::BluetoothManagerPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
