/****************************************************************************
** Meta object code from reading C++ file 'filedialoghandledbus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filedialog/filedialogplugin-core/dbus/filedialoghandledbus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filedialoghandledbus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FileDialogHandleDBus_t {
    QByteArrayData data[43];
    char stringdata0[486];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FileDialogHandleDBus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FileDialogHandleDBus_t qt_meta_stringdata_FileDialogHandleDBus = {
    {
QT_MOC_LITERAL(0, 0, 20), // "FileDialogHandleDBus"
QT_MOC_LITERAL(1, 21, 19), // "windowActiveChanged"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 16), // "directoryChanged"
QT_MOC_LITERAL(4, 59, 19), // "directoryUrlChanged"
QT_MOC_LITERAL(5, 79, 9), // "directory"
QT_MOC_LITERAL(6, 89, 15), // "setDirectoryUrl"
QT_MOC_LITERAL(7, 105, 12), // "directoryUrl"
QT_MOC_LITERAL(8, 118, 9), // "selectUrl"
QT_MOC_LITERAL(9, 128, 3), // "url"
QT_MOC_LITERAL(10, 132, 12), // "selectedUrls"
QT_MOC_LITERAL(11, 145, 6), // "filter"
QT_MOC_LITERAL(12, 152, 9), // "setFilter"
QT_MOC_LITERAL(13, 162, 7), // "filters"
QT_MOC_LITERAL(14, 170, 11), // "setViewMode"
QT_MOC_LITERAL(15, 182, 4), // "mode"
QT_MOC_LITERAL(16, 187, 8), // "viewMode"
QT_MOC_LITERAL(17, 196, 11), // "setFileMode"
QT_MOC_LITERAL(18, 208, 13), // "setAcceptMode"
QT_MOC_LITERAL(19, 222, 10), // "acceptMode"
QT_MOC_LITERAL(20, 233, 12), // "setLabelText"
QT_MOC_LITERAL(21, 246, 5), // "label"
QT_MOC_LITERAL(22, 252, 4), // "text"
QT_MOC_LITERAL(23, 257, 9), // "labelText"
QT_MOC_LITERAL(24, 267, 10), // "setOptions"
QT_MOC_LITERAL(25, 278, 7), // "options"
QT_MOC_LITERAL(26, 286, 9), // "setOption"
QT_MOC_LITERAL(27, 296, 6), // "option"
QT_MOC_LITERAL(28, 303, 2), // "on"
QT_MOC_LITERAL(29, 306, 10), // "testOption"
QT_MOC_LITERAL(30, 317, 5), // "winId"
QT_MOC_LITERAL(31, 323, 14), // "setWindowTitle"
QT_MOC_LITERAL(32, 338, 5), // "title"
QT_MOC_LITERAL(33, 344, 12), // "windowActive"
QT_MOC_LITERAL(34, 357, 14), // "activateWindow"
QT_MOC_LITERAL(35, 372, 17), // "heartbeatInterval"
QT_MOC_LITERAL(36, 390, 13), // "makeHeartbeat"
QT_MOC_LITERAL(37, 404, 11), // "windowFlags"
QT_MOC_LITERAL(38, 416, 20), // "setHeartbeatInterval"
QT_MOC_LITERAL(39, 437, 8), // "interval"
QT_MOC_LITERAL(40, 446, 14), // "setWindowFlags"
QT_MOC_LITERAL(41, 461, 11), // "nameFilters"
QT_MOC_LITERAL(42, 473, 12) // "hideOnAccept"

    },
    "FileDialogHandleDBus\0windowActiveChanged\0"
    "\0directoryChanged\0directoryUrlChanged\0"
    "directory\0setDirectoryUrl\0directoryUrl\0"
    "selectUrl\0url\0selectedUrls\0filter\0"
    "setFilter\0filters\0setViewMode\0mode\0"
    "viewMode\0setFileMode\0setAcceptMode\0"
    "acceptMode\0setLabelText\0label\0text\0"
    "labelText\0setOptions\0options\0setOption\0"
    "option\0on\0testOption\0winId\0setWindowTitle\0"
    "title\0windowActive\0activateWindow\0"
    "heartbeatInterval\0makeHeartbeat\0"
    "windowFlags\0setHeartbeatInterval\0"
    "interval\0setWindowFlags\0nameFilters\0"
    "hideOnAccept"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FileDialogHandleDBus[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      31,   14, // methods
      10,  234, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  169,    2, 0x06 /* Public */,
       3,    0,  170,    2, 0x06 /* Public */,
       4,    0,  171,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,  172,    2, 0x0a /* Public */,
       6,    1,  173,    2, 0x0a /* Public */,
       7,    0,  176,    2, 0x0a /* Public */,
       8,    1,  177,    2, 0x0a /* Public */,
      10,    0,  180,    2, 0x0a /* Public */,
      11,    0,  181,    2, 0x0a /* Public */,
      12,    1,  182,    2, 0x0a /* Public */,
      14,    1,  185,    2, 0x0a /* Public */,
      16,    0,  188,    2, 0x0a /* Public */,
      17,    1,  189,    2, 0x0a /* Public */,
      18,    1,  192,    2, 0x0a /* Public */,
      19,    0,  195,    2, 0x0a /* Public */,
      20,    2,  196,    2, 0x0a /* Public */,
      23,    1,  201,    2, 0x0a /* Public */,
      24,    1,  204,    2, 0x0a /* Public */,
      26,    2,  207,    2, 0x0a /* Public */,
      26,    1,  212,    2, 0x2a /* Public | MethodCloned */,
      25,    0,  215,    2, 0x0a /* Public */,
      29,    1,  216,    2, 0x0a /* Public */,
      30,    0,  219,    2, 0x0a /* Public */,
      31,    1,  220,    2, 0x0a /* Public */,
      33,    0,  223,    2, 0x0a /* Public */,
      34,    0,  224,    2, 0x0a /* Public */,
      35,    0,  225,    2, 0x0a /* Public */,
      36,    0,  226,    2, 0x0a /* Public */,
      37,    0,  227,    2, 0x0a /* Public */,
      38,    1,  228,    2, 0x0a /* Public */,
      40,    1,  231,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::QStringList,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   21,   22,
    QMetaType::QString, QMetaType::Int,   21,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,   27,   28,
    QMetaType::Void, QMetaType::Int,   27,
    QMetaType::Int,
    QMetaType::Bool, QMetaType::Int,   27,
    QMetaType::ULongLong,
    QMetaType::Void, QMetaType::QString,   32,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::UInt,
    QMetaType::Void, QMetaType::Int,   39,
    QMetaType::Void, QMetaType::UInt,   37,

 // properties: name, type, flags
       5, QMetaType::QString, 0x00495103,
       7, QMetaType::QString, 0x00495103,
      41, QMetaType::QStringList, 0x00095103,
      11, QMetaType::Int, 0x00095103,
      16, QMetaType::Int, 0x00095103,
      19, QMetaType::Int, 0x00095103,
      33, QMetaType::Bool, 0x00495001,
      35, QMetaType::Int, 0x00095103,
      37, QMetaType::UInt, 0x00095103,
      42, QMetaType::Bool, 0x00095103,

 // properties: notify_signal_id
       1,
       2,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

       0        // eod
};

void FileDialogHandleDBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileDialogHandleDBus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->windowActiveChanged(); break;
        case 1: _t->directoryChanged(); break;
        case 2: _t->directoryUrlChanged(); break;
        case 3: { QString _r = _t->directory();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->setDirectoryUrl((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: { QString _r = _t->directoryUrl();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->selectUrl((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: { QStringList _r = _t->selectedUrls();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 8: { int _r = _t->filter();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->setFilter((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->setViewMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: { int _r = _t->viewMode();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->setFileMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->setAcceptMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: { int _r = _t->acceptMode();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->setLabelText((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 16: { QString _r = _t->labelText((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->setOptions((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->setOption((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 19: _t->setOption((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: { int _r = _t->options();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->testOption((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { qulonglong _r = _t->winId();
            if (_a[0]) *reinterpret_cast< qulonglong*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->setWindowTitle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 24: { bool _r = _t->windowActive();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->activateWindow(); break;
        case 26: { int _r = _t->heartbeatInterval();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->makeHeartbeat(); break;
        case 28: { quint32 _r = _t->windowFlags();
            if (_a[0]) *reinterpret_cast< quint32*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->setHeartbeatInterval((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 30: _t->setWindowFlags((*reinterpret_cast< quint32(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileDialogHandleDBus::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialogHandleDBus::windowActiveChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileDialogHandleDBus::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialogHandleDBus::directoryChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileDialogHandleDBus::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialogHandleDBus::directoryUrlChanged)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<FileDialogHandleDBus *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->directory(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->directoryUrl(); break;
        case 2: *reinterpret_cast< QStringList*>(_v) = _t->nameFilters(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->filter(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->viewMode(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->acceptMode(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->windowActive(); break;
        case 7: *reinterpret_cast< int*>(_v) = _t->heartbeatInterval(); break;
        case 8: *reinterpret_cast< quint32*>(_v) = _t->windowFlags(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->hideOnAccept(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<FileDialogHandleDBus *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDirectory(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setDirectoryUrl(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setNameFilters(*reinterpret_cast< QStringList*>(_v)); break;
        case 3: _t->setFilter(*reinterpret_cast< int*>(_v)); break;
        case 4: _t->setViewMode(*reinterpret_cast< int*>(_v)); break;
        case 5: _t->setAcceptMode(*reinterpret_cast< int*>(_v)); break;
        case 7: _t->setHeartbeatInterval(*reinterpret_cast< int*>(_v)); break;
        case 8: _t->setWindowFlags(*reinterpret_cast< quint32*>(_v)); break;
        case 9: _t->setHideOnAccept(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject FileDialogHandleDBus::staticMetaObject = { {
    QMetaObject::SuperData::link<FileDialogHandle::staticMetaObject>(),
    qt_meta_stringdata_FileDialogHandleDBus.data,
    qt_meta_data_FileDialogHandleDBus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FileDialogHandleDBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileDialogHandleDBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FileDialogHandleDBus.stringdata0))
        return static_cast<void*>(this);
    return FileDialogHandle::qt_metacast(_clname);
}

int FileDialogHandleDBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileDialogHandle::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 31;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 10;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void FileDialogHandleDBus::windowActiveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void FileDialogHandleDBus::directoryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void FileDialogHandleDBus::directoryUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
