/****************************************************************************
** Meta object code from reading C++ file 'tagdbhandler.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/daemon/daemonplugin-tag/tagdbhandler.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tagdbhandler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_daemonplugin_tag__TagDbHandler_t {
    QByteArrayData data[13];
    char stringdata0[185];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_daemonplugin_tag__TagDbHandler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_daemonplugin_tag__TagDbHandler_t qt_meta_stringdata_daemonplugin_tag__TagDbHandler = {
    {
QT_MOC_LITERAL(0, 0, 30), // "daemonplugin_tag::TagDbHandler"
QT_MOC_LITERAL(1, 31, 12), // "newTagsAdded"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 7), // "newTags"
QT_MOC_LITERAL(4, 53, 11), // "tagsDeleted"
QT_MOC_LITERAL(5, 65, 13), // "beDeletedTags"
QT_MOC_LITERAL(6, 79, 16), // "tagsColorChanged"
QT_MOC_LITERAL(7, 96, 14), // "oldAndNewColor"
QT_MOC_LITERAL(8, 111, 15), // "tagsNameChanged"
QT_MOC_LITERAL(9, 127, 13), // "oldAndNewName"
QT_MOC_LITERAL(10, 141, 15), // "filesWereTagged"
QT_MOC_LITERAL(11, 157, 13), // "filesUntagged"
QT_MOC_LITERAL(12, 171, 13) // "delTagsOfFile"

    },
    "daemonplugin_tag::TagDbHandler\0"
    "newTagsAdded\0\0newTags\0tagsDeleted\0"
    "beDeletedTags\0tagsColorChanged\0"
    "oldAndNewColor\0tagsNameChanged\0"
    "oldAndNewName\0filesWereTagged\0"
    "filesUntagged\0delTagsOfFile"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_daemonplugin_tag__TagDbHandler[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       4,    1,   47,    2, 0x06 /* Public */,
       6,    1,   50,    2, 0x06 /* Public */,
       8,    1,   53,    2, 0x06 /* Public */,
      10,    1,   56,    2, 0x06 /* Public */,
      11,    1,   59,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QVariantMap,    3,
    QMetaType::Void, QMetaType::QStringList,    5,
    QMetaType::Void, QMetaType::QVariantMap,    7,
    QMetaType::Void, QMetaType::QVariantMap,    9,
    QMetaType::Void, QMetaType::QVariantMap,   10,
    QMetaType::Void, QMetaType::QVariantMap,   12,

       0        // eod
};

void daemonplugin_tag::TagDbHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TagDbHandler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->newTagsAdded((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 1: _t->tagsDeleted((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 2: _t->tagsColorChanged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 3: _t->tagsNameChanged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 4: _t->filesWereTagged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 5: _t->filesUntagged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TagDbHandler::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagDbHandler::newTagsAdded)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TagDbHandler::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagDbHandler::tagsDeleted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TagDbHandler::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagDbHandler::tagsColorChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TagDbHandler::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagDbHandler::tagsNameChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TagDbHandler::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagDbHandler::filesWereTagged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (TagDbHandler::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagDbHandler::filesUntagged)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject daemonplugin_tag::TagDbHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_daemonplugin_tag__TagDbHandler.data,
    qt_meta_data_daemonplugin_tag__TagDbHandler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *daemonplugin_tag::TagDbHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *daemonplugin_tag::TagDbHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_daemonplugin_tag__TagDbHandler.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int daemonplugin_tag::TagDbHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void daemonplugin_tag::TagDbHandler::newTagsAdded(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void daemonplugin_tag::TagDbHandler::tagsDeleted(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void daemonplugin_tag::TagDbHandler::tagsColorChanged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void daemonplugin_tag::TagDbHandler::tagsNameChanged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void daemonplugin_tag::TagDbHandler::filesWereTagged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void daemonplugin_tag::TagDbHandler::filesUntagged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
