/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a mountcontroladaptor -i mountcontroldbus.h -l MountControlDBus /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.MountControl.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "mountcontroladaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class MountControlAdaptor
 */

MountControlAdaptor::MountControlAdaptor(MountControlDBus *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

MountControlAdaptor::~MountControlAdaptor()
{
    // destructor
}

QVariantMap MountControlAdaptor::Mount(const QString &path, const QVariantMap &opts)
{
    // handle method call org.deepin.Filemanager.MountControl.Mount
    return parent()->Mount(path, opts);
}

QStringList MountControlAdaptor::SupportedFileSystems()
{
    // handle method call org.deepin.Filemanager.MountControl.SupportedFileSystems
    return parent()->SupportedFileSystems();
}

QVariantMap MountControlAdaptor::Unmount(const QString &path, const QVariantMap &opts)
{
    // handle method call org.deepin.Filemanager.MountControl.Unmount
    return parent()->Unmount(path, opts);
}


#include "mountcontroladaptor.moc"
