/****************************************************************************
** Meta object code from reading C++ file 'addressbar.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-titlebar/views/addressbar.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'addressbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_titlebar__AddressBar_t {
    QByteArrayData data[7];
    char stringdata0[105];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_titlebar__AddressBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_titlebar__AddressBar_t qt_meta_stringdata_dfmplugin_titlebar__AddressBar = {
    {
QT_MOC_LITERAL(0, 0, 30), // "dfmplugin_titlebar::AddressBar"
QT_MOC_LITERAL(1, 31, 9), // "lostFocus"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 13), // "escKeyPressed"
QT_MOC_LITERAL(4, 56, 18), // "clearButtonPressed"
QT_MOC_LITERAL(5, 75, 18), // "pauseButtonClicked"
QT_MOC_LITERAL(6, 94, 10) // "urlChanged"

    },
    "dfmplugin_titlebar::AddressBar\0lostFocus\0"
    "\0escKeyPressed\0clearButtonPressed\0"
    "pauseButtonClicked\0urlChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_titlebar__AddressBar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x06 /* Public */,
       3,    0,   40,    2, 0x06 /* Public */,
       4,    0,   41,    2, 0x06 /* Public */,
       5,    0,   42,    2, 0x06 /* Public */,
       6,    0,   43,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_titlebar::AddressBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AddressBar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->lostFocus(); break;
        case 1: _t->escKeyPressed(); break;
        case 2: _t->clearButtonPressed(); break;
        case 3: _t->pauseButtonClicked(); break;
        case 4: _t->urlChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AddressBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AddressBar::lostFocus)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AddressBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AddressBar::escKeyPressed)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (AddressBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AddressBar::clearButtonPressed)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (AddressBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AddressBar::pauseButtonClicked)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (AddressBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AddressBar::urlChanged)) {
                *result = 4;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_titlebar::AddressBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QLineEdit::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_titlebar__AddressBar.data,
    qt_meta_data_dfmplugin_titlebar__AddressBar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_titlebar::AddressBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_titlebar::AddressBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_titlebar__AddressBar.stringdata0))
        return static_cast<void*>(this);
    return QLineEdit::qt_metacast(_clname);
}

int dfmplugin_titlebar::AddressBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_titlebar::AddressBar::lostFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dfmplugin_titlebar::AddressBar::escKeyPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dfmplugin_titlebar::AddressBar::clearButtonPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void dfmplugin_titlebar::AddressBar::pauseButtonClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void dfmplugin_titlebar::AddressBar::urlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
