// SPDX-FileCopyrightText: 2021 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "listitemdelegate_p.h"

using namespace dfmplugin_workspace;

ListItemDelegatePrivate::ListItemDelegatePrivate(ListItemDelegate *qq)
    : BaseItemDelegatePrivate(qq)
{
}

ListItemDelegatePrivate::~ListItemDelegatePrivate()
{
}
