/****************************************************************************
** Meta object code from reading C++ file 'tageventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-tag/events/tageventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tageventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_tag__TagEventReceiver_t {
    QByteArrayData data[22];
    char stringdata0[313];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_tag__TagEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_tag__TagEventReceiver_t qt_meta_stringdata_dfmplugin_tag__TagEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 31), // "dfmplugin_tag::TagEventReceiver"
QT_MOC_LITERAL(1, 32, 21), // "handleHideFilesResult"
QT_MOC_LITERAL(2, 54, 0), // ""
QT_MOC_LITERAL(3, 55, 5), // "winId"
QT_MOC_LITERAL(4, 61, 11), // "QList<QUrl>"
QT_MOC_LITERAL(5, 73, 4), // "urls"
QT_MOC_LITERAL(6, 78, 2), // "ok"
QT_MOC_LITERAL(7, 81, 19), // "handleFileCutResult"
QT_MOC_LITERAL(8, 101, 7), // "srcUrls"
QT_MOC_LITERAL(9, 109, 8), // "destUrls"
QT_MOC_LITERAL(10, 118, 6), // "errMsg"
QT_MOC_LITERAL(11, 125, 22), // "handleFileRemoveResult"
QT_MOC_LITERAL(12, 148, 22), // "handleFileRenameResult"
QT_MOC_LITERAL(13, 171, 15), // "QMap<QUrl,QUrl>"
QT_MOC_LITERAL(14, 187, 11), // "renamedUrls"
QT_MOC_LITERAL(15, 199, 22), // "handleWindowUrlChanged"
QT_MOC_LITERAL(16, 222, 3), // "url"
QT_MOC_LITERAL(17, 226, 28), // "handleRestoreFromTrashResult"
QT_MOC_LITERAL(18, 255, 11), // "customInfos"
QT_MOC_LITERAL(19, 267, 13), // "handleGetTags"
QT_MOC_LITERAL(20, 281, 25), // "handleSidebarOrderChanged"
QT_MOC_LITERAL(21, 307, 5) // "group"

    },
    "dfmplugin_tag::TagEventReceiver\0"
    "handleHideFilesResult\0\0winId\0QList<QUrl>\0"
    "urls\0ok\0handleFileCutResult\0srcUrls\0"
    "destUrls\0errMsg\0handleFileRemoveResult\0"
    "handleFileRenameResult\0QMap<QUrl,QUrl>\0"
    "renamedUrls\0handleWindowUrlChanged\0"
    "url\0handleRestoreFromTrashResult\0"
    "customInfos\0handleGetTags\0"
    "handleSidebarOrderChanged\0group"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_tag__TagEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   54,    2, 0x0a /* Public */,
       7,    4,   61,    2, 0x0a /* Public */,
      11,    3,   70,    2, 0x0a /* Public */,
      12,    4,   77,    2, 0x0a /* Public */,
      15,    2,   86,    2, 0x0a /* Public */,
      17,    5,   91,    2, 0x0a /* Public */,
      19,    1,  102,    2, 0x0a /* Public */,
      20,    2,  105,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::Bool,    3,    5,    6,
    QMetaType::Void, 0x80000000 | 4, 0x80000000 | 4, QMetaType::Bool, QMetaType::QString,    8,    9,    6,   10,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Bool, QMetaType::QString,    8,    6,   10,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 13, QMetaType::Bool, QMetaType::QString,    3,   14,    6,   10,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,    3,   16,
    QMetaType::Void, 0x80000000 | 4, 0x80000000 | 4, QMetaType::QVariantList, QMetaType::Bool, QMetaType::QString,    8,    9,   18,    6,   10,
    QMetaType::QStringList, QMetaType::QUrl,   16,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QString,    3,   21,

       0        // eod
};

void dfmplugin_tag::TagEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TagEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleHideFilesResult((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 1: _t->handleFileCutResult((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 2: _t->handleFileRemoveResult((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 3: _t->handleFileRenameResult((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QMap<QUrl,QUrl>(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 4: _t->handleWindowUrlChanged((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 5: _t->handleRestoreFromTrashResult((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QVariantList(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4])),(*reinterpret_cast< const QString(*)>(_a[5]))); break;
        case 6: { QStringList _r = _t->handleGetTags((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->handleSidebarOrderChanged((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_tag::TagEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_tag__TagEventReceiver.data,
    qt_meta_data_dfmplugin_tag__TagEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_tag::TagEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_tag::TagEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_tag__TagEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_tag::TagEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
