/****************************************************************************
** Meta object code from reading C++ file 'dfmplugin_propertydialog_global.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-propertydialog/dfmplugin_propertydialog_global.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dfmplugin_propertydialog_global.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_propertydialog_t {
    QByteArrayData data[25];
    char stringdata0[390];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_propertydialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_propertydialog_t qt_meta_stringdata_dfmplugin_propertydialog = {
    {
QT_MOC_LITERAL(0, 0, 24), // "dfmplugin_propertydialog"
QT_MOC_LITERAL(1, 25, 18), // "PropertyFilterType"
QT_MOC_LITERAL(2, 44, 10), // "kNotFilter"
QT_MOC_LITERAL(3, 55, 10), // "kIconTitle"
QT_MOC_LITERAL(4, 66, 10), // "kBasisInfo"
QT_MOC_LITERAL(5, 77, 11), // "kPermission"
QT_MOC_LITERAL(6, 89, 14), // "kFileSizeFiled"
QT_MOC_LITERAL(7, 104, 15), // "kFileCountFiled"
QT_MOC_LITERAL(8, 120, 14), // "kFileTypeFiled"
QT_MOC_LITERAL(9, 135, 18), // "kFilePositionFiled"
QT_MOC_LITERAL(10, 154, 20), // "kFileCreateTimeFiled"
QT_MOC_LITERAL(11, 175, 22), // "kFileAccessedTimeFiled"
QT_MOC_LITERAL(12, 198, 22), // "kFileModifiedTimeFiled"
QT_MOC_LITERAL(13, 221, 20), // "BasicFieldExpandEnum"
QT_MOC_LITERAL(14, 242, 7), // "kNotAll"
QT_MOC_LITERAL(15, 250, 9), // "kFileSize"
QT_MOC_LITERAL(16, 260, 10), // "kFileCount"
QT_MOC_LITERAL(17, 271, 9), // "kFileType"
QT_MOC_LITERAL(18, 281, 13), // "kFilePosition"
QT_MOC_LITERAL(19, 295, 15), // "kFileCreateTime"
QT_MOC_LITERAL(20, 311, 17), // "kFileAccessedTime"
QT_MOC_LITERAL(21, 329, 17), // "kFileModifiedTime"
QT_MOC_LITERAL(22, 347, 15), // "BasicExpandType"
QT_MOC_LITERAL(23, 363, 12), // "kFieldInsert"
QT_MOC_LITERAL(24, 376, 13) // "kFieldReplace"

    },
    "dfmplugin_propertydialog\0PropertyFilterType\0"
    "kNotFilter\0kIconTitle\0kBasisInfo\0"
    "kPermission\0kFileSizeFiled\0kFileCountFiled\0"
    "kFileTypeFiled\0kFilePositionFiled\0"
    "kFileCreateTimeFiled\0kFileAccessedTimeFiled\0"
    "kFileModifiedTimeFiled\0BasicFieldExpandEnum\0"
    "kNotAll\0kFileSize\0kFileCount\0kFileType\0"
    "kFilePosition\0kFileCreateTime\0"
    "kFileAccessedTime\0kFileModifiedTime\0"
    "BasicExpandType\0kFieldInsert\0kFieldReplace"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_propertydialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       3,   14, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // enums: name, alias, flags, count, data
       1,    1, 0x0,   11,   29,
      13,   13, 0x0,    8,   51,
      22,   22, 0x0,    2,   67,

 // enum data: key, value
       2, uint(dfmplugin_propertydialog::kNotFilter),
       3, uint(dfmplugin_propertydialog::kIconTitle),
       4, uint(dfmplugin_propertydialog::kBasisInfo),
       5, uint(dfmplugin_propertydialog::kPermission),
       6, uint(dfmplugin_propertydialog::kFileSizeFiled),
       7, uint(dfmplugin_propertydialog::kFileCountFiled),
       8, uint(dfmplugin_propertydialog::kFileTypeFiled),
       9, uint(dfmplugin_propertydialog::kFilePositionFiled),
      10, uint(dfmplugin_propertydialog::kFileCreateTimeFiled),
      11, uint(dfmplugin_propertydialog::kFileAccessedTimeFiled),
      12, uint(dfmplugin_propertydialog::kFileModifiedTimeFiled),
      14, uint(dfmplugin_propertydialog::kNotAll),
      15, uint(dfmplugin_propertydialog::kFileSize),
      16, uint(dfmplugin_propertydialog::kFileCount),
      17, uint(dfmplugin_propertydialog::kFileType),
      18, uint(dfmplugin_propertydialog::kFilePosition),
      19, uint(dfmplugin_propertydialog::kFileCreateTime),
      20, uint(dfmplugin_propertydialog::kFileAccessedTime),
      21, uint(dfmplugin_propertydialog::kFileModifiedTime),
      23, uint(dfmplugin_propertydialog::kFieldInsert),
      24, uint(dfmplugin_propertydialog::kFieldReplace),

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject dfmplugin_propertydialog::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_dfmplugin_propertydialog.data,
    qt_meta_data_dfmplugin_propertydialog,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
