/****************************************************************************
** Meta object code from reading C++ file 'encryptworker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../dde-file-manager/src/services/diskencrypt/encrypt/encryptworker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'encryptworker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_daemonplugin_file_encrypt__Worker_t {
    QByteArrayData data[1];
    char stringdata0[34];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_daemonplugin_file_encrypt__Worker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_daemonplugin_file_encrypt__Worker_t qt_meta_stringdata_daemonplugin_file_encrypt__Worker = {
    {
QT_MOC_LITERAL(0, 0, 33) // "daemonplugin_file_encrypt::Wo..."

    },
    "daemonplugin_file_encrypt::Worker"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_daemonplugin_file_encrypt__Worker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void daemonplugin_file_encrypt::Worker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject daemonplugin_file_encrypt::Worker::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_daemonplugin_file_encrypt__Worker.data,
    qt_meta_data_daemonplugin_file_encrypt__Worker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *daemonplugin_file_encrypt::Worker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *daemonplugin_file_encrypt::Worker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_daemonplugin_file_encrypt__Worker.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int daemonplugin_file_encrypt::Worker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_daemonplugin_file_encrypt__PrencryptWorker_t {
    QByteArrayData data[1];
    char stringdata0[43];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_daemonplugin_file_encrypt__PrencryptWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_daemonplugin_file_encrypt__PrencryptWorker_t qt_meta_stringdata_daemonplugin_file_encrypt__PrencryptWorker = {
    {
QT_MOC_LITERAL(0, 0, 42) // "daemonplugin_file_encrypt::Pr..."

    },
    "daemonplugin_file_encrypt::PrencryptWorker"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_daemonplugin_file_encrypt__PrencryptWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void daemonplugin_file_encrypt::PrencryptWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject daemonplugin_file_encrypt::PrencryptWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<Worker::staticMetaObject>(),
    qt_meta_stringdata_daemonplugin_file_encrypt__PrencryptWorker.data,
    qt_meta_data_daemonplugin_file_encrypt__PrencryptWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *daemonplugin_file_encrypt::PrencryptWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *daemonplugin_file_encrypt::PrencryptWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_daemonplugin_file_encrypt__PrencryptWorker.stringdata0))
        return static_cast<void*>(this);
    return Worker::qt_metacast(_clname);
}

int daemonplugin_file_encrypt::PrencryptWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Worker::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_daemonplugin_file_encrypt__ReencryptWorkerV2_t {
    QByteArrayData data[8];
    char stringdata0[115];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_daemonplugin_file_encrypt__ReencryptWorkerV2_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_daemonplugin_file_encrypt__ReencryptWorkerV2_t qt_meta_stringdata_daemonplugin_file_encrypt__ReencryptWorkerV2 = {
    {
QT_MOC_LITERAL(0, 0, 44), // "daemonplugin_file_encrypt::Re..."
QT_MOC_LITERAL(1, 45, 20), // "requestEncryptParams"
QT_MOC_LITERAL(2, 66, 0), // ""
QT_MOC_LITERAL(3, 67, 7), // "encConf"
QT_MOC_LITERAL(4, 75, 21), // "deviceReencryptResult"
QT_MOC_LITERAL(5, 97, 6), // "device"
QT_MOC_LITERAL(6, 104, 6), // "result"
QT_MOC_LITERAL(7, 111, 3) // "msg"

    },
    "daemonplugin_file_encrypt::ReencryptWorkerV2\0"
    "requestEncryptParams\0\0encConf\0"
    "deviceReencryptResult\0device\0result\0"
    "msg"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_daemonplugin_file_encrypt__ReencryptWorkerV2[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,
       4,    3,   27,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QVariantMap,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::QString,    5,    6,    7,

       0        // eod
};

void daemonplugin_file_encrypt::ReencryptWorkerV2::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ReencryptWorkerV2 *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestEncryptParams((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 1: _t->deviceReencryptResult((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ReencryptWorkerV2::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ReencryptWorkerV2::requestEncryptParams)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ReencryptWorkerV2::*)(const QString & , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ReencryptWorkerV2::deviceReencryptResult)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject daemonplugin_file_encrypt::ReencryptWorkerV2::staticMetaObject = { {
    QMetaObject::SuperData::link<Worker::staticMetaObject>(),
    qt_meta_stringdata_daemonplugin_file_encrypt__ReencryptWorkerV2.data,
    qt_meta_data_daemonplugin_file_encrypt__ReencryptWorkerV2,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *daemonplugin_file_encrypt::ReencryptWorkerV2::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *daemonplugin_file_encrypt::ReencryptWorkerV2::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_daemonplugin_file_encrypt__ReencryptWorkerV2.stringdata0))
        return static_cast<void*>(this);
    return Worker::qt_metacast(_clname);
}

int daemonplugin_file_encrypt::ReencryptWorkerV2::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Worker::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void daemonplugin_file_encrypt::ReencryptWorkerV2::requestEncryptParams(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void daemonplugin_file_encrypt::ReencryptWorkerV2::deviceReencryptResult(const QString & _t1, int _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_daemonplugin_file_encrypt__DecryptWorker_t {
    QByteArrayData data[1];
    char stringdata0[41];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_daemonplugin_file_encrypt__DecryptWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_daemonplugin_file_encrypt__DecryptWorker_t qt_meta_stringdata_daemonplugin_file_encrypt__DecryptWorker = {
    {
QT_MOC_LITERAL(0, 0, 40) // "daemonplugin_file_encrypt::De..."

    },
    "daemonplugin_file_encrypt::DecryptWorker"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_daemonplugin_file_encrypt__DecryptWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void daemonplugin_file_encrypt::DecryptWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject daemonplugin_file_encrypt::DecryptWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<Worker::staticMetaObject>(),
    qt_meta_stringdata_daemonplugin_file_encrypt__DecryptWorker.data,
    qt_meta_data_daemonplugin_file_encrypt__DecryptWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *daemonplugin_file_encrypt::DecryptWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *daemonplugin_file_encrypt::DecryptWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_daemonplugin_file_encrypt__DecryptWorker.stringdata0))
        return static_cast<void*>(this);
    return Worker::qt_metacast(_clname);
}

int daemonplugin_file_encrypt::DecryptWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Worker::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_daemonplugin_file_encrypt__ChgPassWorker_t {
    QByteArrayData data[1];
    char stringdata0[41];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_daemonplugin_file_encrypt__ChgPassWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_daemonplugin_file_encrypt__ChgPassWorker_t qt_meta_stringdata_daemonplugin_file_encrypt__ChgPassWorker = {
    {
QT_MOC_LITERAL(0, 0, 40) // "daemonplugin_file_encrypt::Ch..."

    },
    "daemonplugin_file_encrypt::ChgPassWorker"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_daemonplugin_file_encrypt__ChgPassWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void daemonplugin_file_encrypt::ChgPassWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject daemonplugin_file_encrypt::ChgPassWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<Worker::staticMetaObject>(),
    qt_meta_stringdata_daemonplugin_file_encrypt__ChgPassWorker.data,
    qt_meta_data_daemonplugin_file_encrypt__ChgPassWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *daemonplugin_file_encrypt::ChgPassWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *daemonplugin_file_encrypt::ChgPassWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_daemonplugin_file_encrypt__ChgPassWorker.stringdata0))
        return static_cast<void*>(this);
    return Worker::qt_metacast(_clname);
}

int daemonplugin_file_encrypt::ChgPassWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Worker::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
