/****************************************************************************
** Meta object code from reading C++ file 'packetwritingjob.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-burn/utils/packetwritingjob.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'packetwritingjob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_burn__PacketWritingScheduler_t {
    QByteArrayData data[3];
    char stringdata0[50];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_burn__PacketWritingScheduler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_burn__PacketWritingScheduler_t qt_meta_stringdata_dfmplugin_burn__PacketWritingScheduler = {
    {
QT_MOC_LITERAL(0, 0, 38), // "dfmplugin_burn::PacketWriting..."
QT_MOC_LITERAL(1, 39, 9), // "onTimeout"
QT_MOC_LITERAL(2, 49, 0) // ""

    },
    "dfmplugin_burn::PacketWritingScheduler\0"
    "onTimeout\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_burn__PacketWritingScheduler[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void dfmplugin_burn::PacketWritingScheduler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PacketWritingScheduler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onTimeout(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_burn::PacketWritingScheduler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_burn__PacketWritingScheduler.data,
    qt_meta_data_dfmplugin_burn__PacketWritingScheduler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_burn::PacketWritingScheduler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_burn::PacketWritingScheduler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_burn__PacketWritingScheduler.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_burn::PacketWritingScheduler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_dfmplugin_burn__AbstractPacketWritingJob_t {
    QByteArrayData data[1];
    char stringdata0[41];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_burn__AbstractPacketWritingJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_burn__AbstractPacketWritingJob_t qt_meta_stringdata_dfmplugin_burn__AbstractPacketWritingJob = {
    {
QT_MOC_LITERAL(0, 0, 40) // "dfmplugin_burn::AbstractPacke..."

    },
    "dfmplugin_burn::AbstractPacketWritingJob"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_burn__AbstractPacketWritingJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void dfmplugin_burn::AbstractPacketWritingJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_burn::AbstractPacketWritingJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_burn__AbstractPacketWritingJob.data,
    qt_meta_data_dfmplugin_burn__AbstractPacketWritingJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_burn::AbstractPacketWritingJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_burn::AbstractPacketWritingJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_burn__AbstractPacketWritingJob.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int dfmplugin_burn::AbstractPacketWritingJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_dfmplugin_burn__PutPacketWritingJob_t {
    QByteArrayData data[3];
    char stringdata0[60];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_burn__PutPacketWritingJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_burn__PutPacketWritingJob_t qt_meta_stringdata_dfmplugin_burn__PutPacketWritingJob = {
    {
QT_MOC_LITERAL(0, 0, 35), // "dfmplugin_burn::PutPacketWrit..."
QT_MOC_LITERAL(1, 36, 11), // "pendingUrls"
QT_MOC_LITERAL(2, 48, 11) // "QList<QUrl>"

    },
    "dfmplugin_burn::PutPacketWritingJob\0"
    "pendingUrls\0QList<QUrl>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_burn__PutPacketWritingJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       1,   14, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // properties: name, type, flags
       1, 0x80000000 | 2, 0x0009510b,

       0        // eod
};

void dfmplugin_burn::PutPacketWritingJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<PutPacketWritingJob *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QList<QUrl>*>(_v) = _t->getPendingUrls(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<PutPacketWritingJob *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPendingUrls(*reinterpret_cast< QList<QUrl>*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_burn::PutPacketWritingJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractPacketWritingJob::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_burn__PutPacketWritingJob.data,
    qt_meta_data_dfmplugin_burn__PutPacketWritingJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_burn::PutPacketWritingJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_burn::PutPacketWritingJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_burn__PutPacketWritingJob.stringdata0))
        return static_cast<void*>(this);
    return AbstractPacketWritingJob::qt_metacast(_clname);
}

int dfmplugin_burn::PutPacketWritingJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractPacketWritingJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_dfmplugin_burn__RemovePacketWritingJob_t {
    QByteArrayData data[3];
    char stringdata0[63];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_burn__RemovePacketWritingJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_burn__RemovePacketWritingJob_t qt_meta_stringdata_dfmplugin_burn__RemovePacketWritingJob = {
    {
QT_MOC_LITERAL(0, 0, 38), // "dfmplugin_burn::RemovePacketW..."
QT_MOC_LITERAL(1, 39, 11), // "pendingUrls"
QT_MOC_LITERAL(2, 51, 11) // "QList<QUrl>"

    },
    "dfmplugin_burn::RemovePacketWritingJob\0"
    "pendingUrls\0QList<QUrl>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_burn__RemovePacketWritingJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       1,   14, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // properties: name, type, flags
       1, 0x80000000 | 2, 0x0009510b,

       0        // eod
};

void dfmplugin_burn::RemovePacketWritingJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<RemovePacketWritingJob *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QList<QUrl>*>(_v) = _t->getPendingUrls(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<RemovePacketWritingJob *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPendingUrls(*reinterpret_cast< QList<QUrl>*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_burn::RemovePacketWritingJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractPacketWritingJob::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_burn__RemovePacketWritingJob.data,
    qt_meta_data_dfmplugin_burn__RemovePacketWritingJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_burn::RemovePacketWritingJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_burn::RemovePacketWritingJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_burn__RemovePacketWritingJob.stringdata0))
        return static_cast<void*>(this);
    return AbstractPacketWritingJob::qt_metacast(_clname);
}

int dfmplugin_burn::RemovePacketWritingJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractPacketWritingJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_dfmplugin_burn__RenamePacketWritingJob_t {
    QByteArrayData data[3];
    char stringdata0[54];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_burn__RenamePacketWritingJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_burn__RenamePacketWritingJob_t qt_meta_stringdata_dfmplugin_burn__RenamePacketWritingJob = {
    {
QT_MOC_LITERAL(0, 0, 38), // "dfmplugin_burn::RenamePacketW..."
QT_MOC_LITERAL(1, 39, 6), // "srcUrl"
QT_MOC_LITERAL(2, 46, 7) // "destUrl"

    },
    "dfmplugin_burn::RenamePacketWritingJob\0"
    "srcUrl\0destUrl"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_burn__RenamePacketWritingJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       2,   14, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // properties: name, type, flags
       1, QMetaType::QUrl, 0x00095103,
       2, QMetaType::QUrl, 0x00095103,

       0        // eod
};

void dfmplugin_burn::RenamePacketWritingJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<RenamePacketWritingJob *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QUrl*>(_v) = _t->getSrcUrl(); break;
        case 1: *reinterpret_cast< QUrl*>(_v) = _t->getDestUrl(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<RenamePacketWritingJob *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSrcUrl(*reinterpret_cast< QUrl*>(_v)); break;
        case 1: _t->setDestUrl(*reinterpret_cast< QUrl*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_burn::RenamePacketWritingJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractPacketWritingJob::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_burn__RenamePacketWritingJob.data,
    qt_meta_data_dfmplugin_burn__RenamePacketWritingJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_burn::RenamePacketWritingJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_burn::RenamePacketWritingJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_burn__RenamePacketWritingJob.stringdata0))
        return static_cast<void*>(this);
    return AbstractPacketWritingJob::qt_metacast(_clname);
}

int dfmplugin_burn::RenamePacketWritingJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractPacketWritingJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
