// SPDX-FileCopyrightText: 2021 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef DFMPLUGIN_TRASH_CORE_GLOBAL_H
#define DFMPLUGIN_TRASH_CORE_GLOBAL_H

#include <dfm-base/dfm_log_defines.h>

#define DPTRASHCORE_NAMESPACE dfmplugin_trashcore

#define DPTRASHCORE_BEGIN_NAMESPACE namespace DPTRASHCORE_NAMESPACE {
#define DPTRASHCORE_END_NAMESPACE }
#define DPTRASHCORE_USE_NAMESPACE using namespace DPTRASHCORE_NAMESPACE;

DPTRASHCORE_BEGIN_NAMESPACE
DFM_LOG_USE_CATEGORY(DPTRASHCORE_NAMESPACE)
DPTRASHCORE_END_NAMESPACE

#endif   // DFMPLUGIN_TRASH_CORE_GLOBAL_H
