/****************************************************************************
** Meta object code from reading C++ file 'canvasorganizer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-organizer/mode/canvasorganizer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvasorganizer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_organizer__CanvasOrganizer_t {
    QByteArrayData data[30];
    char stringdata0[360];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_organizer__CanvasOrganizer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_organizer__CanvasOrganizer_t qt_meta_stringdata_ddplugin_organizer__CanvasOrganizer = {
    {
QT_MOC_LITERAL(0, 0, 35), // "ddplugin_organizer::CanvasOrg..."
QT_MOC_LITERAL(1, 36, 17), // "collectionChanged"
QT_MOC_LITERAL(2, 54, 0), // ""
QT_MOC_LITERAL(3, 55, 17), // "hideAllKeyPressed"
QT_MOC_LITERAL(4, 73, 16), // "filterDataRested"
QT_MOC_LITERAL(5, 90, 12), // "QList<QUrl>*"
QT_MOC_LITERAL(6, 103, 4), // "urls"
QT_MOC_LITERAL(7, 108, 18), // "filterDataInserted"
QT_MOC_LITERAL(8, 127, 3), // "url"
QT_MOC_LITERAL(9, 131, 17), // "filterDataRenamed"
QT_MOC_LITERAL(10, 149, 6), // "oldUrl"
QT_MOC_LITERAL(11, 156, 6), // "newUrl"
QT_MOC_LITERAL(12, 163, 14), // "filterDropData"
QT_MOC_LITERAL(13, 178, 9), // "viewIndex"
QT_MOC_LITERAL(14, 188, 16), // "const QMimeData*"
QT_MOC_LITERAL(15, 205, 8), // "mimeData"
QT_MOC_LITERAL(16, 214, 9), // "viewPoint"
QT_MOC_LITERAL(17, 224, 7), // "extData"
QT_MOC_LITERAL(18, 232, 22), // "filterShortcutkeyPress"
QT_MOC_LITERAL(19, 255, 3), // "key"
QT_MOC_LITERAL(20, 259, 9), // "modifiers"
QT_MOC_LITERAL(21, 269, 14), // "filterKeyPress"
QT_MOC_LITERAL(22, 284, 11), // "filterWheel"
QT_MOC_LITERAL(23, 296, 10), // "angleDelta"
QT_MOC_LITERAL(24, 307, 4), // "ctrl"
QT_MOC_LITERAL(25, 312, 17), // "filterContextMenu"
QT_MOC_LITERAL(26, 330, 3), // "dir"
QT_MOC_LITERAL(27, 334, 11), // "QList<QUrl>"
QT_MOC_LITERAL(28, 346, 5), // "files"
QT_MOC_LITERAL(29, 352, 7) // "viewPos"

    },
    "ddplugin_organizer::CanvasOrganizer\0"
    "collectionChanged\0\0hideAllKeyPressed\0"
    "filterDataRested\0QList<QUrl>*\0urls\0"
    "filterDataInserted\0url\0filterDataRenamed\0"
    "oldUrl\0newUrl\0filterDropData\0viewIndex\0"
    "const QMimeData*\0mimeData\0viewPoint\0"
    "extData\0filterShortcutkeyPress\0key\0"
    "modifiers\0filterKeyPress\0filterWheel\0"
    "angleDelta\0ctrl\0filterContextMenu\0dir\0"
    "QList<QUrl>\0files\0viewPos"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_organizer__CanvasOrganizer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x06 /* Public */,
       3,    0,   65,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   66,    2, 0x09 /* Protected */,
       7,    1,   69,    2, 0x09 /* Protected */,
       9,    2,   72,    2, 0x09 /* Protected */,
      12,    4,   77,    2, 0x09 /* Protected */,
      18,    3,   86,    2, 0x09 /* Protected */,
      21,    3,   93,    2, 0x09 /* Protected */,
      22,    3,  100,    2, 0x09 /* Protected */,
      25,    4,  107,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Bool, 0x80000000 | 5,    6,
    QMetaType::Bool, QMetaType::QUrl,    8,
    QMetaType::Bool, QMetaType::QUrl, QMetaType::QUrl,   10,   11,
    QMetaType::Bool, QMetaType::Int, 0x80000000 | 14, QMetaType::QPoint, QMetaType::VoidStar,   13,   15,   16,   17,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int,   13,   19,   20,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int,   13,   19,   20,
    QMetaType::Bool, QMetaType::Int, QMetaType::QPoint, QMetaType::Bool,   13,   23,   24,
    QMetaType::Bool, QMetaType::Int, QMetaType::QUrl, 0x80000000 | 27, QMetaType::QPoint,   13,   26,   28,   29,

       0        // eod
};

void ddplugin_organizer::CanvasOrganizer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CanvasOrganizer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->collectionChanged(); break;
        case 1: _t->hideAllKeyPressed(); break;
        case 2: { bool _r = _t->filterDataRested((*reinterpret_cast< QList<QUrl>*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->filterDataInserted((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->filterDataRenamed((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->filterDropData((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QMimeData*(*)>(_a[2])),(*reinterpret_cast< const QPoint(*)>(_a[3])),(*reinterpret_cast< void*(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->filterShortcutkeyPress((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->filterKeyPress((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->filterWheel((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QPoint(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->filterContextMenu((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[3])),(*reinterpret_cast< const QPoint(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CanvasOrganizer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CanvasOrganizer::collectionChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CanvasOrganizer::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CanvasOrganizer::hideAllKeyPressed)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_organizer::CanvasOrganizer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_organizer__CanvasOrganizer.data,
    qt_meta_data_ddplugin_organizer__CanvasOrganizer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_organizer::CanvasOrganizer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_organizer::CanvasOrganizer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_organizer__CanvasOrganizer.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_organizer::CanvasOrganizer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void ddplugin_organizer::CanvasOrganizer::collectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ddplugin_organizer::CanvasOrganizer::hideAllKeyPressed()const
{
    QMetaObject::activate(const_cast< ddplugin_organizer::CanvasOrganizer *>(this), &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
