/****************************************************************************
** Meta object code from reading C++ file 'abstractjobhandler.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../dde-file-manager/include/dfm-base/interfaces/abstractjobhandler.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstractjobhandler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmbase__AbstractJobHandler_t {
    QByteArrayData data[151];
    char stringdata0[2393];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__AbstractJobHandler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__AbstractJobHandler_t qt_meta_stringdata_dfmbase__AbstractJobHandler = {
    {
QT_MOC_LITERAL(0, 0, 27), // "dfmbase::AbstractJobHandler"
QT_MOC_LITERAL(1, 28, 21), // "proccessChangedNotify"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 14), // "JobInfoPointer"
QT_MOC_LITERAL(4, 66, 7), // "jobInfo"
QT_MOC_LITERAL(5, 74, 18), // "stateChangedNotify"
QT_MOC_LITERAL(6, 93, 17), // "currentTaskNotify"
QT_MOC_LITERAL(7, 111, 14), // "finishedNotify"
QT_MOC_LITERAL(8, 126, 11), // "errorNotify"
QT_MOC_LITERAL(9, 138, 18), // "speedUpdatedNotify"
QT_MOC_LITERAL(10, 157, 21), // "requestShowTipsDialog"
QT_MOC_LITERAL(11, 179, 43), // "dfmbase::AbstractJobHandler::..."
QT_MOC_LITERAL(12, 223, 4), // "type"
QT_MOC_LITERAL(13, 228, 11), // "QList<QUrl>"
QT_MOC_LITERAL(14, 240, 4), // "list"
QT_MOC_LITERAL(15, 245, 12), // "workerFinish"
QT_MOC_LITERAL(16, 258, 23), // "requestRemoveTaskWidget"
QT_MOC_LITERAL(17, 282, 24), // "requestSaveRedoOperation"
QT_MOC_LITERAL(18, 307, 5), // "token"
QT_MOC_LITERAL(19, 313, 19), // "deleteFirstFileSize"
QT_MOC_LITERAL(20, 333, 17), // "requestTaskDailog"
QT_MOC_LITERAL(21, 351, 10), // "userAction"
QT_MOC_LITERAL(22, 362, 14), // "SupportActions"
QT_MOC_LITERAL(23, 377, 7), // "actions"
QT_MOC_LITERAL(24, 385, 14), // "operateTaskJob"
QT_MOC_LITERAL(25, 400, 17), // "onProccessChanged"
QT_MOC_LITERAL(26, 418, 14), // "onStateChanged"
QT_MOC_LITERAL(27, 433, 13), // "onCurrentTask"
QT_MOC_LITERAL(28, 447, 7), // "onError"
QT_MOC_LITERAL(29, 455, 10), // "onFinished"
QT_MOC_LITERAL(30, 466, 14), // "onSpeedUpdated"
QT_MOC_LITERAL(31, 481, 7), // "JobFlag"
QT_MOC_LITERAL(32, 489, 7), // "kNoHint"
QT_MOC_LITERAL(33, 497, 18), // "kCopyFollowSymlink"
QT_MOC_LITERAL(34, 516, 15), // "kCopyAttributes"
QT_MOC_LITERAL(35, 532, 19), // "kCopyAttributesOnly"
QT_MOC_LITERAL(36, 552, 11), // "kCopyToSelf"
QT_MOC_LITERAL(37, 564, 22), // "kCopyRemoveDestination"
QT_MOC_LITERAL(38, 587, 26), // "kCopyResizeDestinationFile"
QT_MOC_LITERAL(39, 614, 22), // "kCopyIntegrityChecking"
QT_MOC_LITERAL(40, 637, 22), // "kDeleteForceDeleteFile"
QT_MOC_LITERAL(41, 660, 19), // "kDontFormatFileName"
QT_MOC_LITERAL(42, 680, 11), // "kRevocation"
QT_MOC_LITERAL(43, 692, 11), // "kCopyRemote"
QT_MOC_LITERAL(44, 704, 5), // "kRedo"
QT_MOC_LITERAL(45, 710, 23), // "kCountProgressCustomize"
QT_MOC_LITERAL(46, 734, 8), // "JobState"
QT_MOC_LITERAL(47, 743, 11), // "kStartState"
QT_MOC_LITERAL(48, 755, 13), // "kRunningState"
QT_MOC_LITERAL(49, 769, 11), // "kPauseState"
QT_MOC_LITERAL(50, 781, 10), // "kStopState"
QT_MOC_LITERAL(51, 792, 12), // "kUnknowState"
QT_MOC_LITERAL(52, 805, 14), // "StatisticState"
QT_MOC_LITERAL(53, 820, 8), // "kNoState"
QT_MOC_LITERAL(54, 829, 12), // "JobErrorType"
QT_MOC_LITERAL(55, 842, 8), // "kNoError"
QT_MOC_LITERAL(56, 851, 14), // "kNoSourceError"
QT_MOC_LITERAL(57, 866, 12), // "kCancelError"
QT_MOC_LITERAL(58, 879, 16), // "kPermissionError"
QT_MOC_LITERAL(59, 896, 17), // "kSpecialFileError"
QT_MOC_LITERAL(60, 914, 16), // "kFileExistsError"
QT_MOC_LITERAL(61, 931, 21), // "kDirectoryExistsError"
QT_MOC_LITERAL(62, 953, 10), // "kOpenError"
QT_MOC_LITERAL(63, 964, 10), // "kReadError"
QT_MOC_LITERAL(64, 975, 11), // "kWriteError"
QT_MOC_LITERAL(65, 987, 13), // "kSymlinkError"
QT_MOC_LITERAL(66, 1001, 11), // "kMkdirError"
QT_MOC_LITERAL(67, 1013, 12), // "kResizeError"
QT_MOC_LITERAL(68, 1026, 12), // "kRemoveError"
QT_MOC_LITERAL(69, 1039, 12), // "kRenameError"
QT_MOC_LITERAL(70, 1052, 19), // "kSymlinkToGvfsError"
QT_MOC_LITERAL(71, 1072, 15), // "kUnknowUrlError"
QT_MOC_LITERAL(72, 1088, 18), // "kNonexistenceError"
QT_MOC_LITERAL(73, 1107, 23), // "kIntegrityCheckingError"
QT_MOC_LITERAL(74, 1131, 20), // "kFileSizeTooBigError"
QT_MOC_LITERAL(75, 1152, 20), // "kNotEnoughSpaceError"
QT_MOC_LITERAL(76, 1173, 20), // "kTargetReadOnlyError"
QT_MOC_LITERAL(77, 1194, 18), // "kTargetIsSelfError"
QT_MOC_LITERAL(78, 1213, 18), // "kNotSupportedError"
QT_MOC_LITERAL(79, 1232, 22), // "kPermissionDeniedError"
QT_MOC_LITERAL(80, 1255, 10), // "kSeekError"
QT_MOC_LITERAL(81, 1266, 15), // "kProrogramError"
QT_MOC_LITERAL(82, 1282, 11), // "kDfmIoError"
QT_MOC_LITERAL(83, 1294, 16), // "kDeleteFileError"
QT_MOC_LITERAL(84, 1311, 23), // "kMakeStandardTrashError"
QT_MOC_LITERAL(85, 1335, 20), // "kGetRestorePathError"
QT_MOC_LITERAL(86, 1356, 21), // "kDeleteTrashFileError"
QT_MOC_LITERAL(87, 1378, 20), // "kIsNotTrashFileError"
QT_MOC_LITERAL(88, 1399, 21), // "kCreateParentDirError"
QT_MOC_LITERAL(89, 1421, 22), // "kFailedParseUrlOfTrash"
QT_MOC_LITERAL(90, 1444, 30), // "kFailedObtainTrashOriginalFile"
QT_MOC_LITERAL(91, 1475, 21), // "kFileMoveToTrashError"
QT_MOC_LITERAL(92, 1497, 23), // "kRetryReadOrWriteFailed"
QT_MOC_LITERAL(93, 1521, 17), // "kCanNotAccessFile"
QT_MOC_LITERAL(94, 1539, 12), // "kUnknowError"
QT_MOC_LITERAL(95, 1552, 7), // "JobType"
QT_MOC_LITERAL(96, 1560, 9), // "kCopyType"
QT_MOC_LITERAL(97, 1570, 8), // "kCutType"
QT_MOC_LITERAL(98, 1579, 11), // "kDeleteType"
QT_MOC_LITERAL(99, 1591, 16), // "kMoveToTrashType"
QT_MOC_LITERAL(100, 1608, 12), // "kRestoreType"
QT_MOC_LITERAL(101, 1621, 15), // "kCleanTrashType"
QT_MOC_LITERAL(102, 1637, 7), // "kUnknow"
QT_MOC_LITERAL(103, 1645, 13), // "SupportAction"
QT_MOC_LITERAL(104, 1659, 9), // "kNoAction"
QT_MOC_LITERAL(105, 1669, 12), // "kRetryAction"
QT_MOC_LITERAL(106, 1682, 14), // "kReplaceAction"
QT_MOC_LITERAL(107, 1697, 12), // "kMergeAction"
QT_MOC_LITERAL(108, 1710, 11), // "kSkipAction"
QT_MOC_LITERAL(109, 1722, 14), // "kCoexistAction"
QT_MOC_LITERAL(110, 1737, 13), // "kCancelAction"
QT_MOC_LITERAL(111, 1751, 14), // "kEnforceAction"
QT_MOC_LITERAL(112, 1766, 15), // "kRememberAction"
QT_MOC_LITERAL(113, 1782, 12), // "kPauseAction"
QT_MOC_LITERAL(114, 1795, 11), // "kStopAction"
QT_MOC_LITERAL(115, 1807, 12), // "kStartAction"
QT_MOC_LITERAL(116, 1820, 12), // "kResumAction"
QT_MOC_LITERAL(117, 1833, 13), // "NotifyInfoKey"
QT_MOC_LITERAL(118, 1847, 11), // "kJobtypeKey"
QT_MOC_LITERAL(119, 1859, 19), // "kCurrentProgressKey"
QT_MOC_LITERAL(120, 1879, 13), // "kTotalSizeKey"
QT_MOC_LITERAL(121, 1893, 18), // "kStatisticStateKey"
QT_MOC_LITERAL(122, 1912, 12), // "kJobStateKey"
QT_MOC_LITERAL(123, 1925, 16), // "kJobStateHideKey"
QT_MOC_LITERAL(124, 1942, 13), // "kSourceUrlKey"
QT_MOC_LITERAL(125, 1956, 13), // "kTargetUrlKey"
QT_MOC_LITERAL(126, 1970, 13), // "kErrorTypeKey"
QT_MOC_LITERAL(127, 1984, 13), // "kSourceMsgKey"
QT_MOC_LITERAL(128, 1998, 13), // "kTargetMsgKey"
QT_MOC_LITERAL(129, 2012, 12), // "kErrorMsgKey"
QT_MOC_LITERAL(130, 2025, 11), // "kActionsKey"
QT_MOC_LITERAL(131, 2037, 9), // "kSpeedKey"
QT_MOC_LITERAL(132, 2047, 14), // "kRemindTimeKey"
QT_MOC_LITERAL(133, 2062, 17), // "kCompleteFilesKey"
QT_MOC_LITERAL(134, 2080, 23), // "kCompleteTargetFilesKey"
QT_MOC_LITERAL(135, 2104, 23), // "kCompleteCustomInfosKey"
QT_MOC_LITERAL(136, 2128, 17), // "kJobHandlePointer"
QT_MOC_LITERAL(137, 2146, 14), // "kWorkerPointer"
QT_MOC_LITERAL(138, 2161, 10), // "NotifyType"
QT_MOC_LITERAL(139, 2172, 25), // "kNotifyProccessChangedKey"
QT_MOC_LITERAL(140, 2198, 22), // "kNotifyStateChangedKey"
QT_MOC_LITERAL(141, 2221, 21), // "kNotifyCurrentTaskKey"
QT_MOC_LITERAL(142, 2243, 18), // "kNotifyFinishedKey"
QT_MOC_LITERAL(143, 2262, 26), // "kNotifySpeedUpdatedTaskKey"
QT_MOC_LITERAL(144, 2289, 19), // "kNotifyErrorTaskKey"
QT_MOC_LITERAL(145, 2309, 15), // "FileNameAddFlag"
QT_MOC_LITERAL(146, 2325, 7), // "kPrefix"
QT_MOC_LITERAL(147, 2333, 7), // "kSuffix"
QT_MOC_LITERAL(148, 2341, 22), // "DeleteDialogNoticeType"
QT_MOC_LITERAL(149, 2364, 11), // "kEmptyTrash"
QT_MOC_LITERAL(150, 2376, 16) // "kDeleteTashFiles"

    },
    "dfmbase::AbstractJobHandler\0"
    "proccessChangedNotify\0\0JobInfoPointer\0"
    "jobInfo\0stateChangedNotify\0currentTaskNotify\0"
    "finishedNotify\0errorNotify\0"
    "speedUpdatedNotify\0requestShowTipsDialog\0"
    "dfmbase::AbstractJobHandler::ShowDialogType\0"
    "type\0QList<QUrl>\0list\0workerFinish\0"
    "requestRemoveTaskWidget\0"
    "requestSaveRedoOperation\0token\0"
    "deleteFirstFileSize\0requestTaskDailog\0"
    "userAction\0SupportActions\0actions\0"
    "operateTaskJob\0onProccessChanged\0"
    "onStateChanged\0onCurrentTask\0onError\0"
    "onFinished\0onSpeedUpdated\0JobFlag\0"
    "kNoHint\0kCopyFollowSymlink\0kCopyAttributes\0"
    "kCopyAttributesOnly\0kCopyToSelf\0"
    "kCopyRemoveDestination\0"
    "kCopyResizeDestinationFile\0"
    "kCopyIntegrityChecking\0kDeleteForceDeleteFile\0"
    "kDontFormatFileName\0kRevocation\0"
    "kCopyRemote\0kRedo\0kCountProgressCustomize\0"
    "JobState\0kStartState\0kRunningState\0"
    "kPauseState\0kStopState\0kUnknowState\0"
    "StatisticState\0kNoState\0JobErrorType\0"
    "kNoError\0kNoSourceError\0kCancelError\0"
    "kPermissionError\0kSpecialFileError\0"
    "kFileExistsError\0kDirectoryExistsError\0"
    "kOpenError\0kReadError\0kWriteError\0"
    "kSymlinkError\0kMkdirError\0kResizeError\0"
    "kRemoveError\0kRenameError\0kSymlinkToGvfsError\0"
    "kUnknowUrlError\0kNonexistenceError\0"
    "kIntegrityCheckingError\0kFileSizeTooBigError\0"
    "kNotEnoughSpaceError\0kTargetReadOnlyError\0"
    "kTargetIsSelfError\0kNotSupportedError\0"
    "kPermissionDeniedError\0kSeekError\0"
    "kProrogramError\0kDfmIoError\0"
    "kDeleteFileError\0kMakeStandardTrashError\0"
    "kGetRestorePathError\0kDeleteTrashFileError\0"
    "kIsNotTrashFileError\0kCreateParentDirError\0"
    "kFailedParseUrlOfTrash\0"
    "kFailedObtainTrashOriginalFile\0"
    "kFileMoveToTrashError\0kRetryReadOrWriteFailed\0"
    "kCanNotAccessFile\0kUnknowError\0JobType\0"
    "kCopyType\0kCutType\0kDeleteType\0"
    "kMoveToTrashType\0kRestoreType\0"
    "kCleanTrashType\0kUnknow\0SupportAction\0"
    "kNoAction\0kRetryAction\0kReplaceAction\0"
    "kMergeAction\0kSkipAction\0kCoexistAction\0"
    "kCancelAction\0kEnforceAction\0"
    "kRememberAction\0kPauseAction\0kStopAction\0"
    "kStartAction\0kResumAction\0NotifyInfoKey\0"
    "kJobtypeKey\0kCurrentProgressKey\0"
    "kTotalSizeKey\0kStatisticStateKey\0"
    "kJobStateKey\0kJobStateHideKey\0"
    "kSourceUrlKey\0kTargetUrlKey\0kErrorTypeKey\0"
    "kSourceMsgKey\0kTargetMsgKey\0kErrorMsgKey\0"
    "kActionsKey\0kSpeedKey\0kRemindTimeKey\0"
    "kCompleteFilesKey\0kCompleteTargetFilesKey\0"
    "kCompleteCustomInfosKey\0kJobHandlePointer\0"
    "kWorkerPointer\0NotifyType\0"
    "kNotifyProccessChangedKey\0"
    "kNotifyStateChangedKey\0kNotifyCurrentTaskKey\0"
    "kNotifyFinishedKey\0kNotifySpeedUpdatedTaskKey\0"
    "kNotifyErrorTaskKey\0FileNameAddFlag\0"
    "kPrefix\0kSuffix\0DeleteDialogNoticeType\0"
    "kEmptyTrash\0kDeleteTashFiles"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__AbstractJobHandler[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
      10,  164, // enums/sets
       0,    0, // constructors
       0,       // flags
      12,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  109,    2, 0x06 /* Public */,
       5,    1,  112,    2, 0x06 /* Public */,
       6,    1,  115,    2, 0x06 /* Public */,
       7,    1,  118,    2, 0x06 /* Public */,
       8,    1,  121,    2, 0x06 /* Public */,
       9,    1,  124,    2, 0x06 /* Public */,
      10,    2,  127,    2, 0x06 /* Public */,
      15,    0,  132,    2, 0x06 /* Public */,
      16,    0,  133,    2, 0x06 /* Public */,
      17,    2,  134,    2, 0x06 /* Public */,
      20,    0,  139,    2, 0x06 /* Public */,
      21,    1,  140,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      24,    1,  143,    2, 0x0a /* Public */,
      25,    1,  146,    2, 0x0a /* Public */,
      26,    1,  149,    2, 0x0a /* Public */,
      27,    1,  152,    2, 0x0a /* Public */,
      28,    1,  155,    2, 0x0a /* Public */,
      29,    1,  158,    2, 0x0a /* Public */,
      30,    1,  161,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 11, 0x80000000 | 13,   12,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::LongLong,   18,   19,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 22,   23,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 22,   23,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,

 // enums: name, alias, flags, count, data
      31,   31, 0x2,   14,  214,
      46,   46, 0x2,    5,  242,
      52,   52, 0x2,    3,  252,
      54,   54, 0x2,   40,  258,
      95,   95, 0x2,    7,  338,
     103,  103, 0x2,   13,  352,
     117,  117, 0x0,   20,  378,
     138,  138, 0x2,    6,  418,
     145,  145, 0x0,    2,  430,
     148,  148, 0x2,    2,  434,

 // enum data: key, value
      32, uint(dfmbase::AbstractJobHandler::JobFlag::kNoHint),
      33, uint(dfmbase::AbstractJobHandler::JobFlag::kCopyFollowSymlink),
      34, uint(dfmbase::AbstractJobHandler::JobFlag::kCopyAttributes),
      35, uint(dfmbase::AbstractJobHandler::JobFlag::kCopyAttributesOnly),
      36, uint(dfmbase::AbstractJobHandler::JobFlag::kCopyToSelf),
      37, uint(dfmbase::AbstractJobHandler::JobFlag::kCopyRemoveDestination),
      38, uint(dfmbase::AbstractJobHandler::JobFlag::kCopyResizeDestinationFile),
      39, uint(dfmbase::AbstractJobHandler::JobFlag::kCopyIntegrityChecking),
      40, uint(dfmbase::AbstractJobHandler::JobFlag::kDeleteForceDeleteFile),
      41, uint(dfmbase::AbstractJobHandler::JobFlag::kDontFormatFileName),
      42, uint(dfmbase::AbstractJobHandler::JobFlag::kRevocation),
      43, uint(dfmbase::AbstractJobHandler::JobFlag::kCopyRemote),
      44, uint(dfmbase::AbstractJobHandler::JobFlag::kRedo),
      45, uint(dfmbase::AbstractJobHandler::JobFlag::kCountProgressCustomize),
      47, uint(dfmbase::AbstractJobHandler::JobState::kStartState),
      48, uint(dfmbase::AbstractJobHandler::JobState::kRunningState),
      49, uint(dfmbase::AbstractJobHandler::JobState::kPauseState),
      50, uint(dfmbase::AbstractJobHandler::JobState::kStopState),
      51, uint(dfmbase::AbstractJobHandler::JobState::kUnknowState),
      53, uint(dfmbase::AbstractJobHandler::StatisticState::kNoState),
      48, uint(dfmbase::AbstractJobHandler::StatisticState::kRunningState),
      50, uint(dfmbase::AbstractJobHandler::StatisticState::kStopState),
      55, uint(dfmbase::AbstractJobHandler::JobErrorType::kNoError),
      56, uint(dfmbase::AbstractJobHandler::JobErrorType::kNoSourceError),
      57, uint(dfmbase::AbstractJobHandler::JobErrorType::kCancelError),
      58, uint(dfmbase::AbstractJobHandler::JobErrorType::kPermissionError),
      59, uint(dfmbase::AbstractJobHandler::JobErrorType::kSpecialFileError),
      60, uint(dfmbase::AbstractJobHandler::JobErrorType::kFileExistsError),
      61, uint(dfmbase::AbstractJobHandler::JobErrorType::kDirectoryExistsError),
      62, uint(dfmbase::AbstractJobHandler::JobErrorType::kOpenError),
      63, uint(dfmbase::AbstractJobHandler::JobErrorType::kReadError),
      64, uint(dfmbase::AbstractJobHandler::JobErrorType::kWriteError),
      65, uint(dfmbase::AbstractJobHandler::JobErrorType::kSymlinkError),
      66, uint(dfmbase::AbstractJobHandler::JobErrorType::kMkdirError),
      67, uint(dfmbase::AbstractJobHandler::JobErrorType::kResizeError),
      68, uint(dfmbase::AbstractJobHandler::JobErrorType::kRemoveError),
      69, uint(dfmbase::AbstractJobHandler::JobErrorType::kRenameError),
      70, uint(dfmbase::AbstractJobHandler::JobErrorType::kSymlinkToGvfsError),
      71, uint(dfmbase::AbstractJobHandler::JobErrorType::kUnknowUrlError),
      72, uint(dfmbase::AbstractJobHandler::JobErrorType::kNonexistenceError),
      73, uint(dfmbase::AbstractJobHandler::JobErrorType::kIntegrityCheckingError),
      74, uint(dfmbase::AbstractJobHandler::JobErrorType::kFileSizeTooBigError),
      75, uint(dfmbase::AbstractJobHandler::JobErrorType::kNotEnoughSpaceError),
      76, uint(dfmbase::AbstractJobHandler::JobErrorType::kTargetReadOnlyError),
      77, uint(dfmbase::AbstractJobHandler::JobErrorType::kTargetIsSelfError),
      78, uint(dfmbase::AbstractJobHandler::JobErrorType::kNotSupportedError),
      79, uint(dfmbase::AbstractJobHandler::JobErrorType::kPermissionDeniedError),
      80, uint(dfmbase::AbstractJobHandler::JobErrorType::kSeekError),
      81, uint(dfmbase::AbstractJobHandler::JobErrorType::kProrogramError),
      82, uint(dfmbase::AbstractJobHandler::JobErrorType::kDfmIoError),
      83, uint(dfmbase::AbstractJobHandler::JobErrorType::kDeleteFileError),
      84, uint(dfmbase::AbstractJobHandler::JobErrorType::kMakeStandardTrashError),
      85, uint(dfmbase::AbstractJobHandler::JobErrorType::kGetRestorePathError),
      86, uint(dfmbase::AbstractJobHandler::JobErrorType::kDeleteTrashFileError),
      87, uint(dfmbase::AbstractJobHandler::JobErrorType::kIsNotTrashFileError),
      88, uint(dfmbase::AbstractJobHandler::JobErrorType::kCreateParentDirError),
      89, uint(dfmbase::AbstractJobHandler::JobErrorType::kFailedParseUrlOfTrash),
      90, uint(dfmbase::AbstractJobHandler::JobErrorType::kFailedObtainTrashOriginalFile),
      91, uint(dfmbase::AbstractJobHandler::JobErrorType::kFileMoveToTrashError),
      92, uint(dfmbase::AbstractJobHandler::JobErrorType::kRetryReadOrWriteFailed),
      93, uint(dfmbase::AbstractJobHandler::JobErrorType::kCanNotAccessFile),
      94, uint(dfmbase::AbstractJobHandler::JobErrorType::kUnknowError),
      96, uint(dfmbase::AbstractJobHandler::JobType::kCopyType),
      97, uint(dfmbase::AbstractJobHandler::JobType::kCutType),
      98, uint(dfmbase::AbstractJobHandler::JobType::kDeleteType),
      99, uint(dfmbase::AbstractJobHandler::JobType::kMoveToTrashType),
     100, uint(dfmbase::AbstractJobHandler::JobType::kRestoreType),
     101, uint(dfmbase::AbstractJobHandler::JobType::kCleanTrashType),
     102, uint(dfmbase::AbstractJobHandler::JobType::kUnknow),
     104, uint(dfmbase::AbstractJobHandler::SupportAction::kNoAction),
     105, uint(dfmbase::AbstractJobHandler::SupportAction::kRetryAction),
     106, uint(dfmbase::AbstractJobHandler::SupportAction::kReplaceAction),
     107, uint(dfmbase::AbstractJobHandler::SupportAction::kMergeAction),
     108, uint(dfmbase::AbstractJobHandler::SupportAction::kSkipAction),
     109, uint(dfmbase::AbstractJobHandler::SupportAction::kCoexistAction),
     110, uint(dfmbase::AbstractJobHandler::SupportAction::kCancelAction),
     111, uint(dfmbase::AbstractJobHandler::SupportAction::kEnforceAction),
     112, uint(dfmbase::AbstractJobHandler::SupportAction::kRememberAction),
     113, uint(dfmbase::AbstractJobHandler::SupportAction::kPauseAction),
     114, uint(dfmbase::AbstractJobHandler::SupportAction::kStopAction),
     115, uint(dfmbase::AbstractJobHandler::SupportAction::kStartAction),
     116, uint(dfmbase::AbstractJobHandler::SupportAction::kResumAction),
     118, uint(dfmbase::AbstractJobHandler::kJobtypeKey),
     119, uint(dfmbase::AbstractJobHandler::kCurrentProgressKey),
     120, uint(dfmbase::AbstractJobHandler::kTotalSizeKey),
     121, uint(dfmbase::AbstractJobHandler::kStatisticStateKey),
     122, uint(dfmbase::AbstractJobHandler::kJobStateKey),
     123, uint(dfmbase::AbstractJobHandler::kJobStateHideKey),
     124, uint(dfmbase::AbstractJobHandler::kSourceUrlKey),
     125, uint(dfmbase::AbstractJobHandler::kTargetUrlKey),
     126, uint(dfmbase::AbstractJobHandler::kErrorTypeKey),
     127, uint(dfmbase::AbstractJobHandler::kSourceMsgKey),
     128, uint(dfmbase::AbstractJobHandler::kTargetMsgKey),
     129, uint(dfmbase::AbstractJobHandler::kErrorMsgKey),
     130, uint(dfmbase::AbstractJobHandler::kActionsKey),
     131, uint(dfmbase::AbstractJobHandler::kSpeedKey),
     132, uint(dfmbase::AbstractJobHandler::kRemindTimeKey),
     133, uint(dfmbase::AbstractJobHandler::kCompleteFilesKey),
     134, uint(dfmbase::AbstractJobHandler::kCompleteTargetFilesKey),
     135, uint(dfmbase::AbstractJobHandler::kCompleteCustomInfosKey),
     136, uint(dfmbase::AbstractJobHandler::kJobHandlePointer),
     137, uint(dfmbase::AbstractJobHandler::kWorkerPointer),
     139, uint(dfmbase::AbstractJobHandler::NotifyType::kNotifyProccessChangedKey),
     140, uint(dfmbase::AbstractJobHandler::NotifyType::kNotifyStateChangedKey),
     141, uint(dfmbase::AbstractJobHandler::NotifyType::kNotifyCurrentTaskKey),
     142, uint(dfmbase::AbstractJobHandler::NotifyType::kNotifyFinishedKey),
     143, uint(dfmbase::AbstractJobHandler::NotifyType::kNotifySpeedUpdatedTaskKey),
     144, uint(dfmbase::AbstractJobHandler::NotifyType::kNotifyErrorTaskKey),
     146, uint(dfmbase::AbstractJobHandler::kPrefix),
     147, uint(dfmbase::AbstractJobHandler::kSuffix),
     149, uint(dfmbase::AbstractJobHandler::DeleteDialogNoticeType::kEmptyTrash),
     150, uint(dfmbase::AbstractJobHandler::DeleteDialogNoticeType::kDeleteTashFiles),

       0        // eod
};

void dfmbase::AbstractJobHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AbstractJobHandler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->proccessChangedNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 1: _t->stateChangedNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 2: _t->currentTaskNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 3: _t->finishedNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 4: _t->errorNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 5: _t->speedUpdatedNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 6: _t->requestShowTipsDialog((*reinterpret_cast< dfmbase::AbstractJobHandler::ShowDialogType(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2]))); break;
        case 7: _t->workerFinish(); break;
        case 8: _t->requestRemoveTaskWidget(); break;
        case 9: _t->requestSaveRedoOperation((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const qint64(*)>(_a[2]))); break;
        case 10: _t->requestTaskDailog(); break;
        case 11: _t->userAction((*reinterpret_cast< SupportActions(*)>(_a[1]))); break;
        case 12: _t->operateTaskJob((*reinterpret_cast< SupportActions(*)>(_a[1]))); break;
        case 13: _t->onProccessChanged((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 14: _t->onStateChanged((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 15: _t->onCurrentTask((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 16: _t->onError((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 17: _t->onFinished((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 18: _t->onSpeedUpdated((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::ShowDialogType >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AbstractJobHandler::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJobHandler::proccessChangedNotify)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AbstractJobHandler::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJobHandler::stateChangedNotify)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (AbstractJobHandler::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJobHandler::currentTaskNotify)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (AbstractJobHandler::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJobHandler::finishedNotify)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (AbstractJobHandler::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJobHandler::errorNotify)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (AbstractJobHandler::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJobHandler::speedUpdatedNotify)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (AbstractJobHandler::*)(dfmbase::AbstractJobHandler::ShowDialogType , const QList<QUrl> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJobHandler::requestShowTipsDialog)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (AbstractJobHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJobHandler::workerFinish)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (AbstractJobHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJobHandler::requestRemoveTaskWidget)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (AbstractJobHandler::*)(const QString & , const qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJobHandler::requestSaveRedoOperation)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (AbstractJobHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJobHandler::requestTaskDailog)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (AbstractJobHandler::*)(SupportActions );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractJobHandler::userAction)) {
                *result = 11;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::AbstractJobHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__AbstractJobHandler.data,
    qt_meta_data_dfmbase__AbstractJobHandler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::AbstractJobHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::AbstractJobHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__AbstractJobHandler.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmbase::AbstractJobHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void dfmbase::AbstractJobHandler::proccessChangedNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmbase::AbstractJobHandler::stateChangedNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmbase::AbstractJobHandler::currentTaskNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmbase::AbstractJobHandler::finishedNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmbase::AbstractJobHandler::errorNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmbase::AbstractJobHandler::speedUpdatedNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void dfmbase::AbstractJobHandler::requestShowTipsDialog(dfmbase::AbstractJobHandler::ShowDialogType _t1, const QList<QUrl> _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void dfmbase::AbstractJobHandler::workerFinish()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void dfmbase::AbstractJobHandler::requestRemoveTaskWidget()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void dfmbase::AbstractJobHandler::requestSaveRedoOperation(const QString & _t1, const qint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void dfmbase::AbstractJobHandler::requestTaskDailog()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void dfmbase::AbstractJobHandler::userAction(SupportActions _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
