/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a sharecontroladaptor -i sharecontroldbus.h -l ShareControlDBus /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.ShareControl.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "sharecontroladaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class UserShareManagerAdaptor
 */

UserShareManagerAdaptor::UserShareManagerAdaptor(ShareControlDBus *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

UserShareManagerAdaptor::~UserShareManagerAdaptor()
{
    // destructor
}

bool UserShareManagerAdaptor::CloseSmbShareByShareName(const QString &name, bool show)
{
    // handle method call org.deepin.Filemanager.UserShareManager.CloseSmbShareByShareName
    return parent()->CloseSmbShareByShareName(name, show);
}

bool UserShareManagerAdaptor::EnableSmbServices()
{
    // handle method call org.deepin.Filemanager.UserShareManager.EnableSmbServices
    return parent()->EnableSmbServices();
}

bool UserShareManagerAdaptor::IsUserSharePasswordSet(const QString &username)
{
    // handle method call org.deepin.Filemanager.UserShareManager.IsUserSharePasswordSet
    return parent()->IsUserSharePasswordSet(username);
}

bool UserShareManagerAdaptor::SetUserSharePassword(const QString &name, const QString &passwd)
{
    // handle method call org.deepin.Filemanager.UserShareManager.SetUserSharePassword
    return parent()->SetUserSharePassword(name, passwd);
}


#include "sharecontroladaptor.moc"
