/****************************************************************************
** Meta object code from reading C++ file 'core.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-core/core.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'core.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_core__EventHandle_t {
    QByteArrayData data[32];
    char stringdata0[584];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_core__EventHandle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_core__EventHandle_t qt_meta_stringdata_ddplugin_core__EventHandle = {
    {
QT_MOC_LITERAL(0, 0, 26), // "ddplugin_core::EventHandle"
QT_MOC_LITERAL(1, 27, 19), // "screenProxyInstance"
QT_MOC_LITERAL(2, 47, 29), // "dfmbase::AbstractScreenProxy*"
QT_MOC_LITERAL(3, 77, 0), // ""
QT_MOC_LITERAL(4, 78, 13), // "primaryScreen"
QT_MOC_LITERAL(5, 92, 22), // "dfmbase::ScreenPointer"
QT_MOC_LITERAL(6, 115, 7), // "screens"
QT_MOC_LITERAL(7, 123, 29), // "QList<dfmbase::ScreenPointer>"
QT_MOC_LITERAL(8, 153, 12), // "logicScreens"
QT_MOC_LITERAL(9, 166, 6), // "screen"
QT_MOC_LITERAL(10, 173, 4), // "name"
QT_MOC_LITERAL(11, 178, 16), // "devicePixelRatio"
QT_MOC_LITERAL(12, 195, 11), // "displayMode"
QT_MOC_LITERAL(13, 207, 15), // "lastChangedMode"
QT_MOC_LITERAL(14, 223, 5), // "reset"
QT_MOC_LITERAL(15, 229, 12), // "desktopFrame"
QT_MOC_LITERAL(16, 242, 30), // "dfmbase::AbstractDesktopFrame*"
QT_MOC_LITERAL(17, 273, 11), // "rootWindows"
QT_MOC_LITERAL(18, 285, 15), // "QList<QWidget*>"
QT_MOC_LITERAL(19, 301, 12), // "layoutWidget"
QT_MOC_LITERAL(20, 314, 12), // "screensInUse"
QT_MOC_LITERAL(21, 327, 12), // "QStringList*"
QT_MOC_LITERAL(22, 340, 3), // "out"
QT_MOC_LITERAL(23, 344, 20), // "publishScreenChanged"
QT_MOC_LITERAL(24, 365, 25), // "publishDisplayModeChanged"
QT_MOC_LITERAL(25, 391, 28), // "publishScreenGeometryChanged"
QT_MOC_LITERAL(26, 420, 37), // "publishScreenAvailableGeometr..."
QT_MOC_LITERAL(27, 458, 29), // "publishWindowAboutToBeBuilded"
QT_MOC_LITERAL(28, 488, 20), // "publishWindowBuilded"
QT_MOC_LITERAL(29, 509, 19), // "publishWindowShowed"
QT_MOC_LITERAL(30, 529, 22), // "publishGeometryChanged"
QT_MOC_LITERAL(31, 552, 31) // "publishAvailableGeometryChanged"

    },
    "ddplugin_core::EventHandle\0"
    "screenProxyInstance\0dfmbase::AbstractScreenProxy*\0"
    "\0primaryScreen\0dfmbase::ScreenPointer\0"
    "screens\0QList<dfmbase::ScreenPointer>\0"
    "logicScreens\0screen\0name\0devicePixelRatio\0"
    "displayMode\0lastChangedMode\0reset\0"
    "desktopFrame\0dfmbase::AbstractDesktopFrame*\0"
    "rootWindows\0QList<QWidget*>\0layoutWidget\0"
    "screensInUse\0QStringList*\0out\0"
    "publishScreenChanged\0publishDisplayModeChanged\0"
    "publishScreenGeometryChanged\0"
    "publishScreenAvailableGeometryChanged\0"
    "publishWindowAboutToBeBuilded\0"
    "publishWindowBuilded\0publishWindowShowed\0"
    "publishGeometryChanged\0"
    "publishAvailableGeometryChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_core__EventHandle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      22,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  124,    3, 0x0a /* Public */,
       4,    0,  125,    3, 0x0a /* Public */,
       6,    0,  126,    3, 0x0a /* Public */,
       8,    0,  127,    3, 0x0a /* Public */,
       9,    1,  128,    3, 0x0a /* Public */,
      11,    0,  131,    3, 0x0a /* Public */,
      12,    0,  132,    3, 0x0a /* Public */,
      13,    0,  133,    3, 0x0a /* Public */,
      14,    0,  134,    3, 0x0a /* Public */,
      15,    0,  135,    3, 0x0a /* Public */,
      17,    0,  136,    3, 0x0a /* Public */,
      19,    0,  137,    3, 0x0a /* Public */,
      20,    1,  138,    3, 0x0a /* Public */,
      23,    0,  141,    3, 0x08 /* Private */,
      24,    0,  142,    3, 0x08 /* Private */,
      25,    0,  143,    3, 0x08 /* Private */,
      26,    0,  144,    3, 0x08 /* Private */,
      27,    0,  145,    3, 0x08 /* Private */,
      28,    0,  146,    3, 0x08 /* Private */,
      29,    0,  147,    3, 0x08 /* Private */,
      30,    0,  148,    3, 0x08 /* Private */,
      31,    0,  149,    3, 0x08 /* Private */,

 // slots: parameters
    0x80000000 | 2,
    0x80000000 | 5,
    0x80000000 | 7,
    0x80000000 | 7,
    0x80000000 | 5, QMetaType::QString,   10,
    QMetaType::QReal,
    QMetaType::Int,
    QMetaType::Int,
    QMetaType::Void,
    0x80000000 | 16,
    0x80000000 | 18,
    QMetaType::Void,
    QMetaType::Bool, 0x80000000 | 21,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ddplugin_core::EventHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EventHandle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { dfmbase::AbstractScreenProxy* _r = _t->screenProxyInstance();
            if (_a[0]) *reinterpret_cast< dfmbase::AbstractScreenProxy**>(_a[0]) = std::move(_r); }  break;
        case 1: { dfmbase::ScreenPointer _r = _t->primaryScreen();
            if (_a[0]) *reinterpret_cast< dfmbase::ScreenPointer*>(_a[0]) = std::move(_r); }  break;
        case 2: { QList<dfmbase::ScreenPointer> _r = _t->screens();
            if (_a[0]) *reinterpret_cast< QList<dfmbase::ScreenPointer>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QList<dfmbase::ScreenPointer> _r = _t->logicScreens();
            if (_a[0]) *reinterpret_cast< QList<dfmbase::ScreenPointer>*>(_a[0]) = std::move(_r); }  break;
        case 4: { dfmbase::ScreenPointer _r = _t->screen((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< dfmbase::ScreenPointer*>(_a[0]) = std::move(_r); }  break;
        case 5: { qreal _r = _t->devicePixelRatio();
            if (_a[0]) *reinterpret_cast< qreal*>(_a[0]) = std::move(_r); }  break;
        case 6: { int _r = _t->displayMode();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 7: { int _r = _t->lastChangedMode();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->reset(); break;
        case 9: { dfmbase::AbstractDesktopFrame* _r = _t->desktopFrame();
            if (_a[0]) *reinterpret_cast< dfmbase::AbstractDesktopFrame**>(_a[0]) = std::move(_r); }  break;
        case 10: { QList<QWidget*> _r = _t->rootWindows();
            if (_a[0]) *reinterpret_cast< QList<QWidget*>*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->layoutWidget(); break;
        case 12: { bool _r = _t->screensInUse((*reinterpret_cast< QStringList*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->publishScreenChanged(); break;
        case 14: _t->publishDisplayModeChanged(); break;
        case 15: _t->publishScreenGeometryChanged(); break;
        case 16: _t->publishScreenAvailableGeometryChanged(); break;
        case 17: _t->publishWindowAboutToBeBuilded(); break;
        case 18: _t->publishWindowBuilded(); break;
        case 19: _t->publishWindowShowed(); break;
        case 20: _t->publishGeometryChanged(); break;
        case 21: _t->publishAvailableGeometryChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_core::EventHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_core__EventHandle.data,
    qt_meta_data_ddplugin_core__EventHandle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_core::EventHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_core::EventHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_core__EventHandle.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_core::EventHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 22;
    }
    return _id;
}
struct qt_meta_stringdata_ddplugin_core__Core_t {
    QByteArrayData data[7];
    char stringdata0[89];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_core__Core_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_core__Core_t qt_meta_stringdata_ddplugin_core__Core = {
    {
QT_MOC_LITERAL(0, 0, 19), // "ddplugin_core::Core"
QT_MOC_LITERAL(1, 20, 7), // "onStart"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 12), // "onFrameReady"
QT_MOC_LITERAL(4, 42, 17), // "handleLoadPlugins"
QT_MOC_LITERAL(5, 60, 5), // "names"
QT_MOC_LITERAL(6, 66, 22) // "initializeAfterPainted"

    },
    "ddplugin_core::Core\0onStart\0\0onFrameReady\0"
    "handleLoadPlugins\0names\0initializeAfterPainted"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_core__Core[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x09 /* Protected */,
       3,    0,   35,    2, 0x09 /* Protected */,
       4,    1,   36,    2, 0x09 /* Protected */,

 // methods: name, argc, parameters, tag, flags
       6,    0,   39,    2, 0x02 /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QStringList,    5,

 // methods: parameters
    QMetaType::Void,

       0        // eod
};

void ddplugin_core::Core::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Core *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onStart(); break;
        case 1: _t->onFrameReady(); break;
        case 2: _t->handleLoadPlugins((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 3: _t->initializeAfterPainted(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_core::Core::staticMetaObject = { {
    QMetaObject::SuperData::link<dpf::Plugin::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_core__Core.data,
    qt_meta_data_ddplugin_core__Core,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_core::Core::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_core::Core::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_core__Core.stringdata0))
        return static_cast<void*>(this);
    return dpf::Plugin::qt_metacast(_clname);
}

int ddplugin_core::Core::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = dpf::Plugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

QT_PLUGIN_METADATA_SECTION
static constexpr unsigned char qt_pluginMetaData[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x19,  'o',  'r',  'g',  '.',  'd', 
    'e',  'e',  'p',  'i',  'n',  '.',  'p',  'l', 
    'u',  'g',  'i',  'n',  '.',  'd',  'e',  's', 
    'k',  't',  'o',  'p', 
    // "className"
    0x03,  0x64,  'C',  'o',  'r',  'e', 
    // "MetaData"
    0x04,  0xaa,  0x68,  'C',  'a',  't',  'e',  'g', 
    'o',  'r',  'y',  0x60,  0x6d,  'C',  'o',  'm', 
    'p',  'a',  't',  'V',  'e',  'r',  's',  'i', 
    'o',  'n',  0x65,  '1',  '.',  '0',  '.',  '0', 
    0x69,  'C',  'o',  'p',  'y',  'r',  'i',  'g', 
    'h',  't',  0x78,  0x41,  'C',  'o',  'p',  'y', 
    'r',  'i',  'g',  'h',  't',  ' ',  '(',  'C', 
    ')',  ' ',  '2',  '0',  '2',  '0',  ' ',  '~', 
    ' ',  '2',  '0',  '2',  '1',  ' ',  'U',  'n', 
    'i',  'o',  'n',  't',  'e',  'c',  'h',  ' ', 
    'S',  'o',  'f',  't',  'w',  'a',  'r',  'e', 
    ' ',  'T',  'e',  'c',  'h',  'n',  'o',  'l', 
    'o',  'g',  'y',  ' ',  'C',  'o',  '.',  ',', 
    ' ',  'L',  't',  'd',  '.',  0x67,  'D',  'e', 
    'p',  'e',  'n',  'd',  's',  0x80,  0x6b,  'D', 
    'e',  's',  'c',  'r',  'i',  'p',  't',  'i', 
    'o',  'n',  0x78,  0x20,  'T',  'h',  'e',  ' ', 
    'c',  'o',  'r',  'e',  ' ',  'p',  'l',  'u', 
    'g',  'i',  'n',  ' ',  'f',  'o',  'r',  ' ', 
    't',  'h',  'e',  ' ',  'd',  'e',  's',  'k', 
    't',  'o',  'p',  '.',  0x67,  'L',  'i',  'c', 
    'e',  'n',  's',  'e',  0x80,  0x64,  'N',  'a', 
    'm',  'e',  0x6d,  'd',  'd',  'p',  'l',  'u', 
    'g',  'i',  'n',  '-',  'c',  'o',  'r',  'e', 
    0x67,  'U',  'r',  'l',  'L',  'i',  'n',  'k', 
    0x78,  0x19,  'h',  't',  't',  'p',  's',  ':', 
    '/',  '/',  'w',  'w',  'w',  '.',  'u',  'n', 
    'i',  'o',  'n',  't',  'e',  'c',  'h',  '.', 
    'c',  'o',  'm',  0x66,  'V',  'e',  'n',  'd', 
    'o',  'r',  0x78,  0x2b,  'T',  'h',  'e',  ' ', 
    'U',  'n',  'i',  'o',  'n',  't',  'e',  'c', 
    'h',  ' ',  'S',  'o',  'f',  't',  'w',  'a', 
    'r',  'e',  ' ',  'T',  'e',  'c',  'h',  'n', 
    'o',  'l',  'o',  'g',  'y',  ' ',  'C',  'o', 
    '.',  ',',  ' ',  'L',  't',  'd',  '.',  0x67, 
    'V',  'e',  'r',  's',  'i',  'o',  'n',  0x65, 
    '1',  '.',  '0',  '.',  '0', 
    0xff, 
};
using namespace ddplugin_core;
QT_MOC_EXPORT_PLUGIN(ddplugin_core::Core, Core)

QT_WARNING_POP
QT_END_MOC_NAMESPACE
