/****************************************************************************
** Meta object code from reading C++ file 'fileencrypthandle.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-vault/utils/fileencrypthandle.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileencrypthandle.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_vault__FileEncryptHandle_t {
    QByteArrayData data[12];
    char stringdata0[166];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_vault__FileEncryptHandle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_vault__FileEncryptHandle_t qt_meta_stringdata_dfmplugin_vault__FileEncryptHandle = {
    {
QT_MOC_LITERAL(0, 0, 34), // "dfmplugin_vault::FileEncryptH..."
QT_MOC_LITERAL(1, 35, 15), // "signalReadError"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 5), // "error"
QT_MOC_LITERAL(4, 58, 16), // "signalReadOutput"
QT_MOC_LITERAL(5, 75, 3), // "msg"
QT_MOC_LITERAL(6, 79, 17), // "signalCreateVault"
QT_MOC_LITERAL(7, 97, 5), // "state"
QT_MOC_LITERAL(8, 103, 17), // "signalUnlockVault"
QT_MOC_LITERAL(9, 121, 15), // "signalLockVault"
QT_MOC_LITERAL(10, 137, 13), // "slotReadError"
QT_MOC_LITERAL(11, 151, 14) // "slotReadOutput"

    },
    "dfmplugin_vault::FileEncryptHandle\0"
    "signalReadError\0\0error\0signalReadOutput\0"
    "msg\0signalCreateVault\0state\0"
    "signalUnlockVault\0signalLockVault\0"
    "slotReadError\0slotReadOutput"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_vault__FileEncryptHandle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       4,    1,   52,    2, 0x06 /* Public */,
       6,    1,   55,    2, 0x06 /* Public */,
       8,    1,   58,    2, 0x06 /* Public */,
       9,    1,   61,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,   64,    2, 0x0a /* Public */,
      11,    0,   65,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_vault::FileEncryptHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileEncryptHandle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalReadError((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->signalReadOutput((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 2: _t->signalCreateVault((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->signalUnlockVault((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->signalLockVault((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->slotReadError(); break;
        case 6: _t->slotReadOutput(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileEncryptHandle::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileEncryptHandle::signalReadError)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileEncryptHandle::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileEncryptHandle::signalReadOutput)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileEncryptHandle::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileEncryptHandle::signalCreateVault)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FileEncryptHandle::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileEncryptHandle::signalUnlockVault)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FileEncryptHandle::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileEncryptHandle::signalLockVault)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_vault::FileEncryptHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_vault__FileEncryptHandle.data,
    qt_meta_data_dfmplugin_vault__FileEncryptHandle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_vault::FileEncryptHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_vault::FileEncryptHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_vault__FileEncryptHandle.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_vault::FileEncryptHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_vault::FileEncryptHandle::signalReadError(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_vault::FileEncryptHandle::signalReadOutput(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_vault::FileEncryptHandle::signalCreateVault(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_vault::FileEncryptHandle::signalUnlockVault(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_vault::FileEncryptHandle::signalLockVault(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
