/****************************************************************************
** Meta object code from reading C++ file 'opticaleventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-optical/events/opticaleventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'opticaleventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_optical__OpticalEventReceiver_t {
    QByteArrayData data[28];
    char stringdata0[402];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_optical__OpticalEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_optical__OpticalEventReceiver_t qt_meta_stringdata_dfmplugin_optical__OpticalEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 39), // "dfmplugin_optical::OpticalEve..."
QT_MOC_LITERAL(1, 40, 25), // "handleDeleteFilesShortcut"
QT_MOC_LITERAL(2, 66, 0), // ""
QT_MOC_LITERAL(3, 67, 11), // "QList<QUrl>"
QT_MOC_LITERAL(4, 79, 4), // "urls"
QT_MOC_LITERAL(5, 84, 25), // "handleCheckDragDropAction"
QT_MOC_LITERAL(6, 110, 5), // "urlTo"
QT_MOC_LITERAL(7, 116, 15), // "Qt::DropAction*"
QT_MOC_LITERAL(8, 132, 6), // "action"
QT_MOC_LITERAL(9, 139, 25), // "handleMoveToTrashShortcut"
QT_MOC_LITERAL(10, 165, 5), // "winId"
QT_MOC_LITERAL(11, 171, 7), // "rootUrl"
QT_MOC_LITERAL(12, 179, 22), // "handleCutFilesShortcut"
QT_MOC_LITERAL(13, 202, 24), // "handlePasteFilesShortcut"
QT_MOC_LITERAL(14, 227, 8), // "fromUrls"
QT_MOC_LITERAL(15, 236, 2), // "to"
QT_MOC_LITERAL(16, 239, 19), // "sepateTitlebarCrumb"
QT_MOC_LITERAL(17, 259, 3), // "url"
QT_MOC_LITERAL(18, 263, 19), // "QList<QVariantMap>*"
QT_MOC_LITERAL(19, 283, 8), // "mapGroup"
QT_MOC_LITERAL(20, 292, 15), // "handleDropFiles"
QT_MOC_LITERAL(21, 308, 5), // "toUrl"
QT_MOC_LITERAL(22, 314, 24), // "handleBlockShortcutPaste"
QT_MOC_LITERAL(23, 339, 14), // "detailViewIcon"
QT_MOC_LITERAL(24, 354, 8), // "QString*"
QT_MOC_LITERAL(25, 363, 8), // "iconName"
QT_MOC_LITERAL(26, 372, 18), // "handleTabCloseable"
QT_MOC_LITERAL(27, 391, 10) // "currentUrl"

    },
    "dfmplugin_optical::OpticalEventReceiver\0"
    "handleDeleteFilesShortcut\0\0QList<QUrl>\0"
    "urls\0handleCheckDragDropAction\0urlTo\0"
    "Qt::DropAction*\0action\0handleMoveToTrashShortcut\0"
    "winId\0rootUrl\0handleCutFilesShortcut\0"
    "handlePasteFilesShortcut\0fromUrls\0to\0"
    "sepateTitlebarCrumb\0url\0QList<QVariantMap>*\0"
    "mapGroup\0handleDropFiles\0toUrl\0"
    "handleBlockShortcutPaste\0detailViewIcon\0"
    "QString*\0iconName\0handleTabCloseable\0"
    "currentUrl"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_optical__OpticalEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   64,    2, 0x0a /* Public */,
       5,    3,   71,    2, 0x0a /* Public */,
       9,    3,   78,    2, 0x0a /* Public */,
      12,    3,   85,    2, 0x0a /* Public */,
      13,    3,   92,    2, 0x0a /* Public */,
      16,    2,   99,    2, 0x0a /* Public */,
      20,    2,  104,    2, 0x0a /* Public */,
      22,    3,  109,    2, 0x0a /* Public */,
      23,    2,  116,    2, 0x0a /* Public */,
      26,    2,  121,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 3, QMetaType::QUrl,    2,    4,    2,
    QMetaType::Bool, 0x80000000 | 3, QMetaType::QUrl, 0x80000000 | 7,    4,    6,    8,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 3, QMetaType::QUrl,   10,    4,   11,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 3, QMetaType::QUrl,   10,    4,   11,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 3, QMetaType::QUrl,   10,   14,   15,
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 18,   17,   19,
    QMetaType::Bool, 0x80000000 | 3, QMetaType::QUrl,   14,   21,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 3, QMetaType::QUrl,   10,   14,   15,
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 24,   17,   25,
    QMetaType::Bool, QMetaType::QUrl, QMetaType::QUrl,   27,   11,

       0        // eod
};

void dfmplugin_optical::OpticalEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OpticalEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->handleDeleteFilesShortcut((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->handleCheckDragDropAction((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< Qt::DropAction*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->handleMoveToTrashShortcut((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->handleCutFilesShortcut((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->handlePasteFilesShortcut((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->sepateTitlebarCrumb((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QList<QVariantMap>*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->handleDropFiles((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->handleBlockShortcutPaste((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->detailViewIcon((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QString*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->handleTabCloseable((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_optical::OpticalEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_optical__OpticalEventReceiver.data,
    qt_meta_data_dfmplugin_optical__OpticalEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_optical::OpticalEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_optical::OpticalEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_optical__OpticalEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_optical::OpticalEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
