// This file is autogenerated. Changes will be overwritten.
#include "MJDKUFM4QZ/moc_filetaginfo.cpp"
#include "MJDKUFM4QZ/moc_tagproperty.cpp"
#include "EWIEGA46WW/moc_tagdaemon.cpp"
#include "EWIEGA46WW/moc_tagdbhandler.cpp"
#include "EWIEGA46WW/moc_tagmanagerdbus.cpp"
