/****************************************************************************
** Meta object code from reading C++ file 'normalizedmode.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-organizer/mode/normalizedmode.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'normalizedmode.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_organizer__NormalizedMode_t {
    QByteArrayData data[49];
    char stringdata0[621];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_organizer__NormalizedMode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_organizer__NormalizedMode_t qt_meta_stringdata_ddplugin_organizer__NormalizedMode = {
    {
QT_MOC_LITERAL(0, 0, 34), // "ddplugin_organizer::Normalize..."
QT_MOC_LITERAL(1, 35, 7), // "rebuild"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 10), // "reorganize"
QT_MOC_LITERAL(4, 55, 13), // "onFileRenamed"
QT_MOC_LITERAL(5, 69, 6), // "oldUrl"
QT_MOC_LITERAL(6, 76, 6), // "newUrl"
QT_MOC_LITERAL(7, 83, 14), // "onFileInserted"
QT_MOC_LITERAL(8, 98, 11), // "QModelIndex"
QT_MOC_LITERAL(9, 110, 6), // "parent"
QT_MOC_LITERAL(10, 117, 5), // "first"
QT_MOC_LITERAL(11, 123, 4), // "last"
QT_MOC_LITERAL(12, 128, 22), // "onFileAboutToBeRemoved"
QT_MOC_LITERAL(13, 151, 17), // "onFileDataChanged"
QT_MOC_LITERAL(14, 169, 7), // "topLeft"
QT_MOC_LITERAL(15, 177, 11), // "bottomRight"
QT_MOC_LITERAL(16, 189, 12), // "QVector<int>"
QT_MOC_LITERAL(17, 202, 5), // "roles"
QT_MOC_LITERAL(18, 208, 19), // "onReorganizeDesktop"
QT_MOC_LITERAL(19, 228, 17), // "releaseCollection"
QT_MOC_LITERAL(20, 246, 8), // "category"
QT_MOC_LITERAL(21, 255, 14), // "filterDropData"
QT_MOC_LITERAL(22, 270, 9), // "viewIndex"
QT_MOC_LITERAL(23, 280, 16), // "const QMimeData*"
QT_MOC_LITERAL(24, 297, 8), // "mimeData"
QT_MOC_LITERAL(25, 306, 9), // "viewPoint"
QT_MOC_LITERAL(26, 316, 7), // "extData"
QT_MOC_LITERAL(27, 324, 16), // "filterDataRested"
QT_MOC_LITERAL(28, 341, 12), // "QList<QUrl>*"
QT_MOC_LITERAL(29, 354, 4), // "urls"
QT_MOC_LITERAL(30, 359, 18), // "filterDataInserted"
QT_MOC_LITERAL(31, 378, 3), // "url"
QT_MOC_LITERAL(32, 382, 17), // "filterDataRenamed"
QT_MOC_LITERAL(33, 400, 22), // "filterShortcutkeyPress"
QT_MOC_LITERAL(34, 423, 3), // "key"
QT_MOC_LITERAL(35, 427, 9), // "modifiers"
QT_MOC_LITERAL(36, 437, 14), // "filterKeyPress"
QT_MOC_LITERAL(37, 452, 17), // "filterContextMenu"
QT_MOC_LITERAL(38, 470, 3), // "dir"
QT_MOC_LITERAL(39, 474, 11), // "QList<QUrl>"
QT_MOC_LITERAL(40, 486, 5), // "files"
QT_MOC_LITERAL(41, 492, 7), // "viewPos"
QT_MOC_LITERAL(42, 500, 29), // "onCollectionEditStatusChanged"
QT_MOC_LITERAL(43, 530, 7), // "editing"
QT_MOC_LITERAL(44, 538, 23), // "changeCollectionSurface"
QT_MOC_LITERAL(45, 562, 10), // "screenName"
QT_MOC_LITERAL(46, 573, 21), // "deactiveAllPredictors"
QT_MOC_LITERAL(47, 595, 18), // "onCollectionMoving"
QT_MOC_LITERAL(48, 614, 6) // "moving"

    },
    "ddplugin_organizer::NormalizedMode\0"
    "rebuild\0\0reorganize\0onFileRenamed\0"
    "oldUrl\0newUrl\0onFileInserted\0QModelIndex\0"
    "parent\0first\0last\0onFileAboutToBeRemoved\0"
    "onFileDataChanged\0topLeft\0bottomRight\0"
    "QVector<int>\0roles\0onReorganizeDesktop\0"
    "releaseCollection\0category\0filterDropData\0"
    "viewIndex\0const QMimeData*\0mimeData\0"
    "viewPoint\0extData\0filterDataRested\0"
    "QList<QUrl>*\0urls\0filterDataInserted\0"
    "url\0filterDataRenamed\0filterShortcutkeyPress\0"
    "key\0modifiers\0filterKeyPress\0"
    "filterContextMenu\0dir\0QList<QUrl>\0"
    "files\0viewPos\0onCollectionEditStatusChanged\0"
    "editing\0changeCollectionSurface\0"
    "screenName\0deactiveAllPredictors\0"
    "onCollectionMoving\0moving"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_organizer__NormalizedMode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  109,    2, 0x0a /* Public */,
       1,    0,  112,    2, 0x2a /* Public | MethodCloned */,
       4,    2,  113,    2, 0x0a /* Public */,
       7,    3,  118,    2, 0x0a /* Public */,
      12,    3,  125,    2, 0x0a /* Public */,
      13,    3,  132,    2, 0x0a /* Public */,
      18,    0,  139,    2, 0x0a /* Public */,
      19,    1,  140,    2, 0x0a /* Public */,
      21,    4,  143,    2, 0x09 /* Protected */,
      27,    1,  152,    2, 0x09 /* Protected */,
      30,    1,  155,    2, 0x09 /* Protected */,
      32,    2,  158,    2, 0x09 /* Protected */,
      33,    3,  163,    2, 0x09 /* Protected */,
      36,    3,  170,    2, 0x09 /* Protected */,
      37,    4,  177,    2, 0x09 /* Protected */,
      42,    1,  186,    2, 0x09 /* Protected */,
      44,    1,  189,    2, 0x09 /* Protected */,
      46,    0,  192,    2, 0x09 /* Protected */,
      47,    1,  193,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,    5,    6,
    QMetaType::Void, 0x80000000 | 8, QMetaType::Int, QMetaType::Int,    9,   10,   11,
    QMetaType::Void, 0x80000000 | 8, QMetaType::Int, QMetaType::Int,    9,   10,   11,
    QMetaType::Void, 0x80000000 | 8, 0x80000000 | 8, 0x80000000 | 16,   14,   15,   17,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Bool, QMetaType::Int, 0x80000000 | 23, QMetaType::QPoint, QMetaType::VoidStar,   22,   24,   25,   26,
    QMetaType::Bool, 0x80000000 | 28,   29,
    QMetaType::Bool, QMetaType::QUrl,   31,
    QMetaType::Bool, QMetaType::QUrl, QMetaType::QUrl,    5,    6,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int,   22,   34,   35,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int,   22,   34,   35,
    QMetaType::Bool, QMetaType::Int, QMetaType::QUrl, 0x80000000 | 39, QMetaType::QPoint,   22,   38,   40,   41,
    QMetaType::Void, QMetaType::Bool,   43,
    QMetaType::Void, QMetaType::QString,   45,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   48,

       0        // eod
};

void ddplugin_organizer::NormalizedMode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NormalizedMode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->rebuild((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->rebuild(); break;
        case 2: _t->onFileRenamed((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 3: _t->onFileInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 4: _t->onFileAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 5: _t->onFileDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2])),(*reinterpret_cast< const QVector<int>(*)>(_a[3]))); break;
        case 6: _t->onReorganizeDesktop(); break;
        case 7: _t->releaseCollection((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: { bool _r = _t->filterDropData((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QMimeData*(*)>(_a[2])),(*reinterpret_cast< const QPoint(*)>(_a[3])),(*reinterpret_cast< void*(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->filterDataRested((*reinterpret_cast< QList<QUrl>*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->filterDataInserted((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->filterDataRenamed((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->filterShortcutkeyPress((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->filterKeyPress((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->filterContextMenu((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[3])),(*reinterpret_cast< const QPoint(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->onCollectionEditStatusChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->changeCollectionSurface((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->deactiveAllPredictors(); break;
        case 18: _t->onCollectionMoving((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<int> >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_organizer::NormalizedMode::staticMetaObject = { {
    QMetaObject::SuperData::link<CanvasOrganizer::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_organizer__NormalizedMode.data,
    qt_meta_data_ddplugin_organizer__NormalizedMode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_organizer::NormalizedMode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_organizer::NormalizedMode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_organizer__NormalizedMode.stringdata0))
        return static_cast<void*>(this);
    return CanvasOrganizer::qt_metacast(_clname);
}

int ddplugin_organizer::NormalizedMode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CanvasOrganizer::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
