// SPDX-FileCopyrightText: 2022 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include <dfm-base/interfaces/abstractfilepreviewplugin.h>

using namespace dfmbase;
AbstractFilePreviewPlugin::AbstractFilePreviewPlugin(QObject *parent)
    : QObject(parent)
{
}
