/****************************************************************************
** Meta object code from reading C++ file 'fileoperatorproxy.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-canvas/view/operator/fileoperatorproxy.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileoperatorproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_canvas__FileOperatorProxy_t {
    QByteArrayData data[20];
    char stringdata0[328];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_canvas__FileOperatorProxy_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_canvas__FileOperatorProxy_t qt_meta_stringdata_ddplugin_canvas__FileOperatorProxy = {
    {
QT_MOC_LITERAL(0, 0, 34), // "ddplugin_canvas::FileOperator..."
QT_MOC_LITERAL(1, 35, 18), // "filePastedCallback"
QT_MOC_LITERAL(2, 54, 0), // ""
QT_MOC_LITERAL(3, 55, 19), // "fileTouchedCallback"
QT_MOC_LITERAL(4, 75, 19), // "fileRenamedCallback"
QT_MOC_LITERAL(5, 95, 10), // "renameFile"
QT_MOC_LITERAL(6, 106, 3), // "wid"
QT_MOC_LITERAL(7, 110, 6), // "oldUrl"
QT_MOC_LITERAL(8, 117, 6), // "newUrl"
QT_MOC_LITERAL(9, 124, 13), // "touchFileData"
QT_MOC_LITERAL(10, 138, 33), // "QPair<QString,QPair<int,QPoin..."
QT_MOC_LITERAL(11, 172, 18), // "clearTouchFileData"
QT_MOC_LITERAL(12, 191, 14), // "renameFileData"
QT_MOC_LITERAL(13, 206, 16), // "QHash<QUrl,QUrl>"
QT_MOC_LITERAL(14, 223, 20), // "removeRenameFileData"
QT_MOC_LITERAL(15, 244, 19), // "clearRenameFileData"
QT_MOC_LITERAL(16, 264, 13), // "pasteFileData"
QT_MOC_LITERAL(17, 278, 10), // "QSet<QUrl>"
QT_MOC_LITERAL(18, 289, 19), // "removePasteFileData"
QT_MOC_LITERAL(19, 309, 18) // "clearPasteFileData"

    },
    "ddplugin_canvas::FileOperatorProxy\0"
    "filePastedCallback\0\0fileTouchedCallback\0"
    "fileRenamedCallback\0renameFile\0wid\0"
    "oldUrl\0newUrl\0touchFileData\0"
    "QPair<QString,QPair<int,QPoint> >\0"
    "clearTouchFileData\0renameFileData\0"
    "QHash<QUrl,QUrl>\0removeRenameFileData\0"
    "clearRenameFileData\0pasteFileData\0"
    "QSet<QUrl>\0removePasteFileData\0"
    "clearPasteFileData"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_canvas__FileOperatorProxy[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x06 /* Public */,
       3,    0,   75,    2, 0x06 /* Public */,
       4,    0,   76,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       5,    3,   77,    2, 0x02 /* Public */,
       9,    0,   84,    2, 0x02 /* Public */,
      11,    0,   85,    2, 0x02 /* Public */,
      12,    0,   86,    2, 0x02 /* Public */,
      14,    1,   87,    2, 0x02 /* Public */,
      15,    0,   90,    2, 0x02 /* Public */,
      16,    0,   91,    2, 0x02 /* Public */,
      18,    1,   92,    2, 0x02 /* Public */,
      19,    0,   95,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::QUrl, QMetaType::QUrl,    6,    7,    8,
    0x80000000 | 10,
    QMetaType::Void,
    0x80000000 | 13,
    QMetaType::Void, QMetaType::QUrl,    7,
    QMetaType::Void,
    0x80000000 | 17,
    QMetaType::Void, QMetaType::QUrl,    7,
    QMetaType::Void,

       0        // eod
};

void ddplugin_canvas::FileOperatorProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileOperatorProxy *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->filePastedCallback(); break;
        case 1: _t->fileTouchedCallback(); break;
        case 2: _t->fileRenamedCallback(); break;
        case 3: _t->renameFile((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3]))); break;
        case 4: { QPair<QString,QPair<int,QPoint> > _r = _t->touchFileData();
            if (_a[0]) *reinterpret_cast< QPair<QString,QPair<int,QPoint> >*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->clearTouchFileData(); break;
        case 6: { QHash<QUrl,QUrl> _r = _t->renameFileData();
            if (_a[0]) *reinterpret_cast< QHash<QUrl,QUrl>*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->removeRenameFileData((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 8: _t->clearRenameFileData(); break;
        case 9: { QSet<QUrl> _r = _t->pasteFileData();
            if (_a[0]) *reinterpret_cast< QSet<QUrl>*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->removePasteFileData((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 11: _t->clearPasteFileData(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileOperatorProxy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileOperatorProxy::filePastedCallback)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileOperatorProxy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileOperatorProxy::fileTouchedCallback)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileOperatorProxy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileOperatorProxy::fileRenamedCallback)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_canvas::FileOperatorProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_canvas__FileOperatorProxy.data,
    qt_meta_data_ddplugin_canvas__FileOperatorProxy,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_canvas::FileOperatorProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_canvas::FileOperatorProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_canvas__FileOperatorProxy.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_canvas::FileOperatorProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void ddplugin_canvas::FileOperatorProxy::filePastedCallback()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ddplugin_canvas::FileOperatorProxy::fileTouchedCallback()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ddplugin_canvas::FileOperatorProxy::fileRenamedCallback()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
