/****************************************************************************
** Meta object code from reading C++ file 'screensaver_interface.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-wallpapersetting/dbus/screensaver_interface.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'screensaver_interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ComDeepinScreenSaverInterface_t {
    QByteArrayData data[23];
    char stringdata0[391];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ComDeepinScreenSaverInterface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ComDeepinScreenSaverInterface_t qt_meta_stringdata_ComDeepinScreenSaverInterface = {
    {
QT_MOC_LITERAL(0, 0, 29), // "ComDeepinScreenSaverInterface"
QT_MOC_LITERAL(1, 30, 17), // "ConfigurableItems"
QT_MOC_LITERAL(2, 48, 30), // "QDBusPendingReply<QStringList>"
QT_MOC_LITERAL(3, 79, 0), // ""
QT_MOC_LITERAL(4, 80, 19), // "GetScreenSaverCover"
QT_MOC_LITERAL(5, 100, 26), // "QDBusPendingReply<QString>"
QT_MOC_LITERAL(6, 127, 4), // "name"
QT_MOC_LITERAL(7, 132, 14), // "IsConfigurable"
QT_MOC_LITERAL(8, 147, 23), // "QDBusPendingReply<bool>"
QT_MOC_LITERAL(9, 171, 7), // "Preview"
QT_MOC_LITERAL(10, 179, 7), // "staysOn"
QT_MOC_LITERAL(11, 187, 22), // "RefreshScreenSaverList"
QT_MOC_LITERAL(12, 210, 19), // "QDBusPendingReply<>"
QT_MOC_LITERAL(13, 230, 5), // "Start"
QT_MOC_LITERAL(14, 236, 17), // "StartCustomConfig"
QT_MOC_LITERAL(15, 254, 4), // "Stop"
QT_MOC_LITERAL(16, 259, 14), // "allScreenSaver"
QT_MOC_LITERAL(17, 274, 25), // "batteryScreenSaverTimeout"
QT_MOC_LITERAL(18, 300, 18), // "currentScreenSaver"
QT_MOC_LITERAL(19, 319, 9), // "isRunning"
QT_MOC_LITERAL(20, 329, 27), // "linePowerScreenSaverTimeout"
QT_MOC_LITERAL(21, 357, 17), // "lockScreenAtAwake"
QT_MOC_LITERAL(22, 375, 15) // "lockScreenDelay"

    },
    "ComDeepinScreenSaverInterface\0"
    "ConfigurableItems\0QDBusPendingReply<QStringList>\0"
    "\0GetScreenSaverCover\0QDBusPendingReply<QString>\0"
    "name\0IsConfigurable\0QDBusPendingReply<bool>\0"
    "Preview\0staysOn\0RefreshScreenSaverList\0"
    "QDBusPendingReply<>\0Start\0StartCustomConfig\0"
    "Stop\0allScreenSaver\0batteryScreenSaverTimeout\0"
    "currentScreenSaver\0isRunning\0"
    "linePowerScreenSaverTimeout\0"
    "lockScreenAtAwake\0lockScreenDelay"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ComDeepinScreenSaverInterface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       7,   72, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   54,    3, 0x0a /* Public */,
       4,    1,   55,    3, 0x0a /* Public */,
       7,    1,   58,    3, 0x0a /* Public */,
       9,    2,   61,    3, 0x0a /* Public */,
      11,    0,   66,    3, 0x0a /* Public */,
      13,    0,   67,    3, 0x0a /* Public */,
      14,    1,   68,    3, 0x0a /* Public */,
      15,    0,   71,    3, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 2,
    0x80000000 | 5, QMetaType::QString,    6,
    0x80000000 | 8, QMetaType::QString,    6,
    0x80000000 | 8, QMetaType::QString, QMetaType::Int,    6,   10,
    0x80000000 | 12,
    0x80000000 | 12,
    0x80000000 | 8, QMetaType::QString,    6,
    0x80000000 | 12,

 // properties: name, type, flags
      16, QMetaType::QStringList, 0x00095001,
      17, QMetaType::Int, 0x00095103,
      18, QMetaType::QString, 0x00095103,
      19, QMetaType::Bool, 0x00095001,
      20, QMetaType::Int, 0x00095103,
      21, QMetaType::Bool, 0x00095103,
      22, QMetaType::Int, 0x00095103,

       0        // eod
};

void ComDeepinScreenSaverInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ComDeepinScreenSaverInterface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QDBusPendingReply<QStringList> _r = _t->ConfigurableItems();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusPendingReply<QString> _r = _t->GetScreenSaverCover((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<bool> _r = _t->IsConfigurable((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<bool> _r = _t->Preview((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<> _r = _t->RefreshScreenSaverList();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->Start();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<bool> _r = _t->StartCustomConfig((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->Stop();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<ComDeepinScreenSaverInterface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QStringList*>(_v) = _t->allScreenSaver(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->batteryScreenSaverTimeout(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->currentScreenSaver(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->isRunning(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->linePowerScreenSaverTimeout(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->lockScreenAtAwake(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->lockScreenDelay(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<ComDeepinScreenSaverInterface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setBatteryScreenSaverTimeout(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setCurrentScreenSaver(*reinterpret_cast< QString*>(_v)); break;
        case 4: _t->setLinePowerScreenSaverTimeout(*reinterpret_cast< int*>(_v)); break;
        case 5: _t->setLockScreenAtAwake(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setLockScreenDelay(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject ComDeepinScreenSaverInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_ComDeepinScreenSaverInterface.data,
    qt_meta_data_ComDeepinScreenSaverInterface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ComDeepinScreenSaverInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ComDeepinScreenSaverInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ComDeepinScreenSaverInterface.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int ComDeepinScreenSaverInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
