/****************************************************************************
** Meta object code from reading C++ file 'connecttoserverdialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-titlebar/dialogs/connecttoserverdialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'connecttoserverdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_titlebar__ConnectToServerDialog_t {
    QByteArrayData data[16];
    char stringdata0[250];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_titlebar__ConnectToServerDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_titlebar__ConnectToServerDialog_t qt_meta_stringdata_dfmplugin_titlebar__ConnectToServerDialog = {
    {
QT_MOC_LITERAL(0, 0, 41), // "dfmplugin_titlebar::ConnectTo..."
QT_MOC_LITERAL(1, 42, 17), // "collectionOperate"
QT_MOC_LITERAL(2, 60, 0), // ""
QT_MOC_LITERAL(3, 61, 15), // "onButtonClicked"
QT_MOC_LITERAL(4, 77, 5), // "index"
QT_MOC_LITERAL(5, 83, 20), // "onCurrentTextChanged"
QT_MOC_LITERAL(6, 104, 6), // "string"
QT_MOC_LITERAL(7, 111, 18), // "doDeleteCollection"
QT_MOC_LITERAL(8, 130, 4), // "text"
QT_MOC_LITERAL(9, 135, 3), // "row"
QT_MOC_LITERAL(10, 139, 21), // "onCurrentInputChanged"
QT_MOC_LITERAL(11, 161, 23), // "onCollectionViewClicked"
QT_MOC_LITERAL(12, 185, 11), // "QModelIndex"
QT_MOC_LITERAL(13, 197, 20), // "onCompleterActivated"
QT_MOC_LITERAL(14, 218, 17), // "initUiForSizeMode"
QT_MOC_LITERAL(15, 236, 13) // "updateUiState"

    },
    "dfmplugin_titlebar::ConnectToServerDialog\0"
    "collectionOperate\0\0onButtonClicked\0"
    "index\0onCurrentTextChanged\0string\0"
    "doDeleteCollection\0text\0row\0"
    "onCurrentInputChanged\0onCollectionViewClicked\0"
    "QModelIndex\0onCompleterActivated\0"
    "initUiForSizeMode\0updateUiState"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_titlebar__ConnectToServerDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x0a /* Public */,
       3,    1,   65,    2, 0x0a /* Public */,
       5,    1,   68,    2, 0x0a /* Public */,
       7,    2,   71,    2, 0x0a /* Public */,
       7,    1,   76,    2, 0x2a /* Public | MethodCloned */,
      10,    1,   79,    2, 0x0a /* Public */,
      11,    1,   82,    2, 0x0a /* Public */,
      13,    1,   85,    2, 0x0a /* Public */,
      14,    0,   88,    2, 0x08 /* Private */,
      15,    0,   89,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    8,    9,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, 0x80000000 | 12,    4,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_titlebar::ConnectToServerDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ConnectToServerDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->collectionOperate(); break;
        case 1: _t->onButtonClicked((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 2: _t->onCurrentTextChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->doDeleteCollection((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->doDeleteCollection((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->onCurrentInputChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->onCollectionViewClicked((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 7: _t->onCompleterActivated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->initUiForSizeMode(); break;
        case 9: _t->updateUiState(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_titlebar::ConnectToServerDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<Dtk::Widget::DDialog::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_titlebar__ConnectToServerDialog.data,
    qt_meta_data_dfmplugin_titlebar__ConnectToServerDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_titlebar::ConnectToServerDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_titlebar::ConnectToServerDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_titlebar__ConnectToServerDialog.stringdata0))
        return static_cast<void*>(this);
    return Dtk::Widget::DDialog::qt_metacast(_clname);
}

int dfmplugin_titlebar::ConnectToServerDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Dtk::Widget::DDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
