/****************************************************************************
** Meta object code from reading C++ file 'filemanagerwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../dde-file-manager/include/dfm-base/widgets/filemanagerwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filemanagerwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmbase__FileManagerWindow_t {
    QByteArrayData data[35];
    char stringdata0[556];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__FileManagerWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__FileManagerWindow_t qt_meta_stringdata_dfmbase__FileManagerWindow = {
    {
QT_MOC_LITERAL(0, 0, 26), // "dfmbase::FileManagerWindow"
QT_MOC_LITERAL(1, 27, 11), // "aboutToOpen"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 12), // "aboutToClose"
QT_MOC_LITERAL(4, 53, 15), // "positionChanged"
QT_MOC_LITERAL(5, 69, 3), // "pos"
QT_MOC_LITERAL(6, 73, 17), // "currentUrlChanged"
QT_MOC_LITERAL(7, 91, 3), // "url"
QT_MOC_LITERAL(8, 95, 23), // "currentViewStateChanged"
QT_MOC_LITERAL(9, 119, 16), // "selectUrlChanged"
QT_MOC_LITERAL(10, 136, 11), // "QList<QUrl>"
QT_MOC_LITERAL(11, 148, 7), // "urlList"
QT_MOC_LITERAL(12, 156, 28), // "aboutToPlaySplitterAnimation"
QT_MOC_LITERAL(13, 185, 10), // "startValue"
QT_MOC_LITERAL(14, 196, 8), // "endValue"
QT_MOC_LITERAL(15, 205, 26), // "windowSplitterWidthChanged"
QT_MOC_LITERAL(16, 232, 5), // "value"
QT_MOC_LITERAL(17, 238, 10), // "reqRefresh"
QT_MOC_LITERAL(18, 249, 18), // "reqActivateNextTab"
QT_MOC_LITERAL(19, 268, 22), // "reqActivatePreviousTab"
QT_MOC_LITERAL(20, 291, 14), // "reqSearchCtrlF"
QT_MOC_LITERAL(21, 306, 14), // "reqSearchCtrlL"
QT_MOC_LITERAL(22, 321, 7), // "reqBack"
QT_MOC_LITERAL(23, 329, 10), // "reqForward"
QT_MOC_LITERAL(24, 340, 18), // "reqCloseCurrentTab"
QT_MOC_LITERAL(25, 359, 12), // "reqCreateTab"
QT_MOC_LITERAL(26, 372, 15), // "reqCreateWindow"
QT_MOC_LITERAL(27, 388, 23), // "reqTriggerActionByIndex"
QT_MOC_LITERAL(28, 412, 5), // "index"
QT_MOC_LITERAL(29, 418, 21), // "reqActivateTabByIndex"
QT_MOC_LITERAL(30, 440, 17), // "reqShowHotkeyHelp"
QT_MOC_LITERAL(31, 458, 23), // "titleBarInstallFinished"
QT_MOC_LITERAL(32, 482, 22), // "sideBarInstallFinished"
QT_MOC_LITERAL(33, 505, 24), // "workspaceInstallFinished"
QT_MOC_LITERAL(34, 530, 25) // "detailViewInstallFinished"

    },
    "dfmbase::FileManagerWindow\0aboutToOpen\0"
    "\0aboutToClose\0positionChanged\0pos\0"
    "currentUrlChanged\0url\0currentViewStateChanged\0"
    "selectUrlChanged\0QList<QUrl>\0urlList\0"
    "aboutToPlaySplitterAnimation\0startValue\0"
    "endValue\0windowSplitterWidthChanged\0"
    "value\0reqRefresh\0reqActivateNextTab\0"
    "reqActivatePreviousTab\0reqSearchCtrlF\0"
    "reqSearchCtrlL\0reqBack\0reqForward\0"
    "reqCloseCurrentTab\0reqCreateTab\0"
    "reqCreateWindow\0reqTriggerActionByIndex\0"
    "index\0reqActivateTabByIndex\0"
    "reqShowHotkeyHelp\0titleBarInstallFinished\0"
    "sideBarInstallFinished\0workspaceInstallFinished\0"
    "detailViewInstallFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__FileManagerWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      25,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      25,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  139,    2, 0x06 /* Public */,
       3,    0,  140,    2, 0x06 /* Public */,
       4,    1,  141,    2, 0x06 /* Public */,
       6,    1,  144,    2, 0x06 /* Public */,
       8,    0,  147,    2, 0x06 /* Public */,
       9,    1,  148,    2, 0x06 /* Public */,
      12,    2,  151,    2, 0x06 /* Public */,
      15,    1,  156,    2, 0x06 /* Public */,
      17,    0,  159,    2, 0x06 /* Public */,
      18,    0,  160,    2, 0x06 /* Public */,
      19,    0,  161,    2, 0x06 /* Public */,
      20,    0,  162,    2, 0x06 /* Public */,
      21,    0,  163,    2, 0x06 /* Public */,
      22,    0,  164,    2, 0x06 /* Public */,
      23,    0,  165,    2, 0x06 /* Public */,
      24,    0,  166,    2, 0x06 /* Public */,
      25,    0,  167,    2, 0x06 /* Public */,
      26,    0,  168,    2, 0x06 /* Public */,
      27,    1,  169,    2, 0x06 /* Public */,
      29,    1,  172,    2, 0x06 /* Public */,
      30,    0,  175,    2, 0x06 /* Public */,
      31,    0,  176,    2, 0x06 /* Public */,
      32,    0,  177,    2, 0x06 /* Public */,
      33,    0,  178,    2, 0x06 /* Public */,
      34,    0,  179,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,    5,
    QMetaType::Void, QMetaType::QUrl,    7,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   13,   14,
    QMetaType::Void, QMetaType::QVariant,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   28,
    QMetaType::Void, QMetaType::Int,   28,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void dfmbase::FileManagerWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileManagerWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->aboutToOpen(); break;
        case 1: _t->aboutToClose(); break;
        case 2: _t->positionChanged((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 3: _t->currentUrlChanged((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 4: _t->currentViewStateChanged(); break;
        case 5: _t->selectUrlChanged((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 6: _t->aboutToPlaySplitterAnimation((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 7: _t->windowSplitterWidthChanged((*reinterpret_cast< const QVariant(*)>(_a[1]))); break;
        case 8: _t->reqRefresh(); break;
        case 9: _t->reqActivateNextTab(); break;
        case 10: _t->reqActivatePreviousTab(); break;
        case 11: _t->reqSearchCtrlF(); break;
        case 12: _t->reqSearchCtrlL(); break;
        case 13: _t->reqBack(); break;
        case 14: _t->reqForward(); break;
        case 15: _t->reqCloseCurrentTab(); break;
        case 16: _t->reqCreateTab(); break;
        case 17: _t->reqCreateWindow(); break;
        case 18: _t->reqTriggerActionByIndex((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->reqActivateTabByIndex((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->reqShowHotkeyHelp(); break;
        case 21: _t->titleBarInstallFinished(); break;
        case 22: _t->sideBarInstallFinished(); break;
        case 23: _t->workspaceInstallFinished(); break;
        case 24: _t->detailViewInstallFinished(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::aboutToOpen)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::aboutToClose)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)(const QPoint & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::positionChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::currentUrlChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::currentViewStateChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)(const QList<QUrl> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::selectUrlChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::aboutToPlaySplitterAnimation)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)(const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::windowSplitterWidthChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqRefresh)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqActivateNextTab)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqActivatePreviousTab)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqSearchCtrlF)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqSearchCtrlL)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqBack)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqForward)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqCloseCurrentTab)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqCreateTab)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqCreateWindow)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqTriggerActionByIndex)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqActivateTabByIndex)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::reqShowHotkeyHelp)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::titleBarInstallFinished)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::sideBarInstallFinished)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::workspaceInstallFinished)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (FileManagerWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileManagerWindow::detailViewInstallFinished)) {
                *result = 24;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::FileManagerWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<DMainWindow::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__FileManagerWindow.data,
    qt_meta_data_dfmbase__FileManagerWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::FileManagerWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::FileManagerWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__FileManagerWindow.stringdata0))
        return static_cast<void*>(this);
    return DMainWindow::qt_metacast(_clname);
}

int dfmbase::FileManagerWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void dfmbase::FileManagerWindow::aboutToOpen()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dfmbase::FileManagerWindow::aboutToClose()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dfmbase::FileManagerWindow::positionChanged(const QPoint & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmbase::FileManagerWindow::currentUrlChanged(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmbase::FileManagerWindow::currentViewStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void dfmbase::FileManagerWindow::selectUrlChanged(const QList<QUrl> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void dfmbase::FileManagerWindow::aboutToPlaySplitterAnimation(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void dfmbase::FileManagerWindow::windowSplitterWidthChanged(const QVariant & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void dfmbase::FileManagerWindow::reqRefresh()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void dfmbase::FileManagerWindow::reqActivateNextTab()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void dfmbase::FileManagerWindow::reqActivatePreviousTab()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void dfmbase::FileManagerWindow::reqSearchCtrlF()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void dfmbase::FileManagerWindow::reqSearchCtrlL()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void dfmbase::FileManagerWindow::reqBack()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void dfmbase::FileManagerWindow::reqForward()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void dfmbase::FileManagerWindow::reqCloseCurrentTab()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void dfmbase::FileManagerWindow::reqCreateTab()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void dfmbase::FileManagerWindow::reqCreateWindow()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void dfmbase::FileManagerWindow::reqTriggerActionByIndex(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void dfmbase::FileManagerWindow::reqActivateTabByIndex(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void dfmbase::FileManagerWindow::reqShowHotkeyHelp()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void dfmbase::FileManagerWindow::titleBarInstallFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void dfmbase::FileManagerWindow::sideBarInstallFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void dfmbase::FileManagerWindow::workspaceInstallFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void dfmbase::FileManagerWindow::detailViewInstallFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
