/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -p devicemanager_interface /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.Daemon.DeviceManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "devicemanager_interface.h"

/*
 * Implementation of interface class OrgDeepinFilemanagerDaemonDeviceManagerInterface
 */

OrgDeepinFilemanagerDaemonDeviceManagerInterface::OrgDeepinFilemanagerDaemonDeviceManagerInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

OrgDeepinFilemanagerDaemonDeviceManagerInterface::~OrgDeepinFilemanagerDaemonDeviceManagerInterface()
{
}


#include "devicemanager_interface.moc"
