// SPDX-FileCopyrightText: 2024 - 2025 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef DFMEXTWINDOWPROXYPRIVATE_H
#define DFMEXTWINDOWPROXYPRIVATE_H

#include <dfm-extension/dfm-extension-global.h>

BEGEN_DFMEXT_NAMESPACE

// TODO: impl me

class DFMExtWindowProxyPrivate
{
public:
    DFMExtWindowProxyPrivate();
};

END_DFMEXT_NAMESPACE

#endif   // DFMEXTWINDOWPROXYPRIVATE_H
