/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a operationsstackmanageradaptor -i operationsstackmanagerdbus.h -l OperationsStackManagerDbus /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.Daemon.OperationsStackManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "operationsstackmanageradaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class OperationsStackManagerAdaptor
 */

OperationsStackManagerAdaptor::OperationsStackManagerAdaptor(OperationsStackManagerDbus *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

OperationsStackManagerAdaptor::~OperationsStackManagerAdaptor()
{
    // destructor
}

void OperationsStackManagerAdaptor::CleanOperations()
{
    // handle method call org.deepin.Filemanager.Daemon.OperationsStackManager.CleanOperations
    parent()->CleanOperations();
}

void OperationsStackManagerAdaptor::CleanOperationsByUrl(const QStringList &urls)
{
    // handle method call org.deepin.Filemanager.Daemon.OperationsStackManager.CleanOperationsByUrl
    parent()->CleanOperationsByUrl(urls);
}

QVariantMap OperationsStackManagerAdaptor::RevocationOperations()
{
    // handle method call org.deepin.Filemanager.Daemon.OperationsStackManager.RevocationOperations
    return parent()->RevocationOperations();
}

QVariantMap OperationsStackManagerAdaptor::RevocationRedoOperations()
{
    // handle method call org.deepin.Filemanager.Daemon.OperationsStackManager.RevocationRedoOperations
    return parent()->RevocationRedoOperations();
}

void OperationsStackManagerAdaptor::SaveOperations(const QVariantMap &values)
{
    // handle method call org.deepin.Filemanager.Daemon.OperationsStackManager.SaveOperations
    parent()->SaveOperations(values);
}

void OperationsStackManagerAdaptor::SaveRedoOperations(const QVariantMap &values)
{
    // handle method call org.deepin.Filemanager.Daemon.OperationsStackManager.SaveRedoOperations
    parent()->SaveRedoOperations(values);
}


#include "operationsstackmanageradaptor.moc"
