/****************************************************************************
** Meta object code from reading C++ file 'deviceproxymanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../dde-file-manager/src/dfm-base/base/device/deviceproxymanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'deviceproxymanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmbase__DeviceProxyManager_t {
    QByteArrayData data[28];
    char stringdata0[402];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__DeviceProxyManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__DeviceProxyManager_t qt_meta_stringdata_dfmbase__DeviceProxyManager = {
    {
QT_MOC_LITERAL(0, 0, 27), // "dfmbase::DeviceProxyManager"
QT_MOC_LITERAL(1, 28, 14), // "devSizeChanged"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 2), // "id"
QT_MOC_LITERAL(4, 47, 5), // "total"
QT_MOC_LITERAL(5, 53, 4), // "avai"
QT_MOC_LITERAL(6, 58, 23), // "blockDevPropertyChanged"
QT_MOC_LITERAL(7, 82, 8), // "property"
QT_MOC_LITERAL(8, 91, 3), // "val"
QT_MOC_LITERAL(9, 95, 15), // "blockDriveAdded"
QT_MOC_LITERAL(10, 111, 17), // "blockDriveRemoved"
QT_MOC_LITERAL(11, 129, 13), // "blockDevAdded"
QT_MOC_LITERAL(12, 143, 15), // "blockDevRemoved"
QT_MOC_LITERAL(13, 159, 6), // "oldMpt"
QT_MOC_LITERAL(14, 166, 15), // "blockDevMounted"
QT_MOC_LITERAL(15, 182, 3), // "mpt"
QT_MOC_LITERAL(16, 186, 17), // "blockDevUnmounted"
QT_MOC_LITERAL(17, 204, 14), // "blockDevLocked"
QT_MOC_LITERAL(18, 219, 16), // "blockDevUnlocked"
QT_MOC_LITERAL(19, 236, 11), // "cleartextId"
QT_MOC_LITERAL(20, 248, 15), // "blockDevFsAdded"
QT_MOC_LITERAL(21, 264, 17), // "blockDevFsRemoved"
QT_MOC_LITERAL(22, 282, 16), // "protocolDevAdded"
QT_MOC_LITERAL(23, 299, 18), // "protocolDevRemoved"
QT_MOC_LITERAL(24, 318, 18), // "protocolDevMounted"
QT_MOC_LITERAL(25, 337, 20), // "protocolDevUnmounted"
QT_MOC_LITERAL(26, 358, 20), // "devMngDBusRegistered"
QT_MOC_LITERAL(27, 379, 22) // "devMngDBusUnregistered"

    },
    "dfmbase::DeviceProxyManager\0devSizeChanged\0"
    "\0id\0total\0avai\0blockDevPropertyChanged\0"
    "property\0val\0blockDriveAdded\0"
    "blockDriveRemoved\0blockDevAdded\0"
    "blockDevRemoved\0oldMpt\0blockDevMounted\0"
    "mpt\0blockDevUnmounted\0blockDevLocked\0"
    "blockDevUnlocked\0cleartextId\0"
    "blockDevFsAdded\0blockDevFsRemoved\0"
    "protocolDevAdded\0protocolDevRemoved\0"
    "protocolDevMounted\0protocolDevUnmounted\0"
    "devMngDBusRegistered\0devMngDBusUnregistered"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__DeviceProxyManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      18,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,  104,    2, 0x06 /* Public */,
       6,    3,  111,    2, 0x06 /* Public */,
       9,    0,  118,    2, 0x06 /* Public */,
      10,    0,  119,    2, 0x06 /* Public */,
      11,    1,  120,    2, 0x06 /* Public */,
      12,    2,  123,    2, 0x06 /* Public */,
      14,    2,  128,    2, 0x06 /* Public */,
      16,    2,  133,    2, 0x06 /* Public */,
      17,    1,  138,    2, 0x06 /* Public */,
      18,    2,  141,    2, 0x06 /* Public */,
      20,    1,  146,    2, 0x06 /* Public */,
      21,    1,  149,    2, 0x06 /* Public */,
      22,    1,  152,    2, 0x06 /* Public */,
      23,    2,  155,    2, 0x06 /* Public */,
      24,    2,  160,    2, 0x06 /* Public */,
      25,    2,  165,    2, 0x06 /* Public */,
      26,    0,  170,    2, 0x06 /* Public */,
      27,    0,  171,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::LongLong, QMetaType::LongLong,    3,    4,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QVariant,    3,    7,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,   13,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,   15,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,   13,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,   19,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,   13,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,   15,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,   13,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void dfmbase::DeviceProxyManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DeviceProxyManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->devSizeChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2])),(*reinterpret_cast< qint64(*)>(_a[3]))); break;
        case 1: _t->blockDevPropertyChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3]))); break;
        case 2: _t->blockDriveAdded(); break;
        case 3: _t->blockDriveRemoved(); break;
        case 4: _t->blockDevAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->blockDevRemoved((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->blockDevMounted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 7: _t->blockDevUnmounted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 8: _t->blockDevLocked((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->blockDevUnlocked((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 10: _t->blockDevFsAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->blockDevFsRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->protocolDevAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->protocolDevRemoved((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 14: _t->protocolDevMounted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 15: _t->protocolDevUnmounted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 16: _t->devMngDBusRegistered(); break;
        case 17: _t->devMngDBusUnregistered(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DeviceProxyManager::*)(const QString & , qint64 , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::devSizeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & , const QString & , const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::blockDevPropertyChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::blockDriveAdded)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::blockDriveRemoved)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::blockDevAdded)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::blockDevRemoved)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::blockDevMounted)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::blockDevUnmounted)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::blockDevLocked)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::blockDevUnlocked)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::blockDevFsAdded)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::blockDevFsRemoved)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::protocolDevAdded)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::protocolDevRemoved)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::protocolDevMounted)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::protocolDevUnmounted)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::devMngDBusRegistered)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (DeviceProxyManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceProxyManager::devMngDBusUnregistered)) {
                *result = 17;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::DeviceProxyManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__DeviceProxyManager.data,
    qt_meta_data_dfmbase__DeviceProxyManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::DeviceProxyManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::DeviceProxyManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__DeviceProxyManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmbase::DeviceProxyManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void dfmbase::DeviceProxyManager::devSizeChanged(const QString & _t1, qint64 _t2, qint64 _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmbase::DeviceProxyManager::blockDevPropertyChanged(const QString & _t1, const QString & _t2, const QVariant & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmbase::DeviceProxyManager::blockDriveAdded()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void dfmbase::DeviceProxyManager::blockDriveRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void dfmbase::DeviceProxyManager::blockDevAdded(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmbase::DeviceProxyManager::blockDevRemoved(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void dfmbase::DeviceProxyManager::blockDevMounted(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void dfmbase::DeviceProxyManager::blockDevUnmounted(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void dfmbase::DeviceProxyManager::blockDevLocked(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void dfmbase::DeviceProxyManager::blockDevUnlocked(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void dfmbase::DeviceProxyManager::blockDevFsAdded(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void dfmbase::DeviceProxyManager::blockDevFsRemoved(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void dfmbase::DeviceProxyManager::protocolDevAdded(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void dfmbase::DeviceProxyManager::protocolDevRemoved(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void dfmbase::DeviceProxyManager::protocolDevMounted(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void dfmbase::DeviceProxyManager::protocolDevUnmounted(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void dfmbase::DeviceProxyManager::devMngDBusRegistered()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void dfmbase::DeviceProxyManager::devMngDBusUnregistered()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
