/****************************************************************************
** Meta object code from reading C++ file 'menu.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-menu/menu.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'menu.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_menu__MenuHandle_t {
    QByteArrayData data[19];
    char stringdata0[253];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_menu__MenuHandle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_menu__MenuHandle_t qt_meta_stringdata_dfmplugin_menu__MenuHandle = {
    {
QT_MOC_LITERAL(0, 0, 26), // "dfmplugin_menu::MenuHandle"
QT_MOC_LITERAL(1, 27, 8), // "contains"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 4), // "name"
QT_MOC_LITERAL(4, 42, 13), // "registerScene"
QT_MOC_LITERAL(5, 56, 30), // "dfmbase::AbstractSceneCreator*"
QT_MOC_LITERAL(6, 87, 7), // "creator"
QT_MOC_LITERAL(7, 95, 15), // "unregisterScene"
QT_MOC_LITERAL(8, 111, 4), // "bind"
QT_MOC_LITERAL(9, 116, 6), // "parent"
QT_MOC_LITERAL(10, 123, 6), // "unbind"
QT_MOC_LITERAL(11, 130, 11), // "createScene"
QT_MOC_LITERAL(12, 142, 27), // "dfmbase::AbstractMenuScene*"
QT_MOC_LITERAL(13, 170, 17), // "perfectMenuParams"
QT_MOC_LITERAL(14, 188, 6), // "params"
QT_MOC_LITERAL(15, 195, 13), // "isMenuDisable"
QT_MOC_LITERAL(16, 209, 17), // "publishSceneAdded"
QT_MOC_LITERAL(17, 227, 5), // "scene"
QT_MOC_LITERAL(18, 233, 19) // "publishSceneRemoved"

    },
    "dfmplugin_menu::MenuHandle\0contains\0"
    "\0name\0registerScene\0dfmbase::AbstractSceneCreator*\0"
    "creator\0unregisterScene\0bind\0parent\0"
    "unbind\0createScene\0dfmbase::AbstractMenuScene*\0"
    "perfectMenuParams\0params\0isMenuDisable\0"
    "publishSceneAdded\0scene\0publishSceneRemoved"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_menu__MenuHandle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x0a /* Public */,
       4,    2,   72,    2, 0x0a /* Public */,
       7,    1,   77,    2, 0x0a /* Public */,
       8,    2,   80,    2, 0x0a /* Public */,
      10,    2,   85,    2, 0x0a /* Public */,
      10,    1,   90,    2, 0x2a /* Public | MethodCloned */,
      11,    1,   93,    2, 0x0a /* Public */,
      13,    1,   96,    2, 0x0a /* Public */,
      15,    1,   99,    2, 0x0a /* Public */,
      16,    1,  102,    2, 0x08 /* Private */,
      18,    1,  105,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Bool, QMetaType::QString,    3,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 5,    3,    6,
    0x80000000 | 5, QMetaType::QString,    3,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString,    3,    9,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    9,
    QMetaType::Void, QMetaType::QString,    3,
    0x80000000 | 12, QMetaType::QString,    3,
    QMetaType::QVariantHash, QMetaType::QVariantHash,   14,
    QMetaType::Bool, QMetaType::QVariantHash,   14,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void, QMetaType::QString,   17,

       0        // eod
};

void dfmplugin_menu::MenuHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MenuHandle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->contains((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->registerScene((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< dfmbase::AbstractSceneCreator*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { dfmbase::AbstractSceneCreator* _r = _t->unregisterScene((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< dfmbase::AbstractSceneCreator**>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->bind((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->unbind((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 5: _t->unbind((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: { dfmbase::AbstractMenuScene* _r = _t->createScene((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< dfmbase::AbstractMenuScene**>(_a[0]) = std::move(_r); }  break;
        case 7: { QVariantHash _r = _t->perfectMenuParams((*reinterpret_cast< const QVariantHash(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QVariantHash*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->isMenuDisable((*reinterpret_cast< const QVariantHash(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->publishSceneAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->publishSceneRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractSceneCreator* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_menu::MenuHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_menu__MenuHandle.data,
    qt_meta_data_dfmplugin_menu__MenuHandle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_menu::MenuHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_menu::MenuHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_menu__MenuHandle.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_menu::MenuHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}
struct qt_meta_stringdata_dfmplugin_menu__Menu_t {
    QByteArrayData data[1];
    char stringdata0[21];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_menu__Menu_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_menu__Menu_t qt_meta_stringdata_dfmplugin_menu__Menu = {
    {
QT_MOC_LITERAL(0, 0, 20) // "dfmplugin_menu::Menu"

    },
    "dfmplugin_menu::Menu"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_menu__Menu[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void dfmplugin_menu::Menu::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_menu::Menu::staticMetaObject = { {
    QMetaObject::SuperData::link<dpf::Plugin::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_menu__Menu.data,
    qt_meta_data_dfmplugin_menu__Menu,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_menu::Menu::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_menu::Menu::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_menu__Menu.stringdata0))
        return static_cast<void*>(this);
    return dpf::Plugin::qt_metacast(_clname);
}

int dfmplugin_menu::Menu::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = dpf::Plugin::qt_metacall(_c, _id, _a);
    return _id;
}

QT_PLUGIN_METADATA_SECTION
static constexpr unsigned char qt_pluginMetaData[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x18,  'o',  'r',  'g',  '.',  'd', 
    'e',  'e',  'p',  'i',  'n',  '.',  'p',  'l', 
    'u',  'g',  'i',  'n',  '.',  'c',  'o',  'm', 
    'm',  'o',  'n', 
    // "className"
    0x03,  0x64,  'M',  'e',  'n',  'u', 
    // "MetaData"
    0x04,  0xaa,  0x68,  'C',  'a',  't',  'e',  'g', 
    'o',  'r',  'y',  0x60,  0x6d,  'C',  'o',  'm', 
    'p',  'a',  't',  'V',  'e',  'r',  's',  'i', 
    'o',  'n',  0x65,  '1',  '.',  '0',  '.',  '0', 
    0x69,  'C',  'o',  'p',  'y',  'r',  'i',  'g', 
    'h',  't',  0x78,  0x3a,  'C',  'o',  'p',  'y', 
    'r',  'i',  'g',  'h',  't',  ' ',  '(',  'C', 
    ')',  ' ',  '2',  '0',  '2',  '2',  ' ',  'U', 
    'n',  'i',  'o',  'n',  't',  'e',  'c',  'h', 
    ' ',  'S',  'o',  'f',  't',  'w',  'a',  'r', 
    'e',  ' ',  'T',  'e',  'c',  'h',  'n',  'o', 
    'l',  'o',  'g',  'y',  ' ',  'C',  'o',  '.', 
    ',',  ' ',  'L',  't',  'd',  '.',  0x67,  'D', 
    'e',  'p',  'e',  'n',  'd',  's',  0x80,  0x6b, 
    'D',  'e',  's',  'c',  'r',  'i',  'p',  't', 
    'i',  'o',  'n',  0x78,  0x32,  'T',  'h',  'e', 
    ' ',  'c',  'o',  'm',  'm',  'o',  'n',  ' ', 
    'p',  'l',  'u',  'g',  'i',  'n',  ' ',  'f', 
    'o',  'r',  ' ',  't',  'h',  'e',  ' ',  'f', 
    'i',  'l',  'e',  'm',  'a',  'n',  'a',  'g', 
    'e',  'r',  ' ',  'a',  'n',  'd',  ' ',  'd', 
    'e',  's',  'k',  't',  'o',  'p',  '.',  0x67, 
    'L',  'i',  'c',  'e',  'n',  's',  'e',  0x80, 
    0x64,  'N',  'a',  'm',  'e',  0x6e,  'd',  'f', 
    'm',  'p',  'l',  'u',  'g',  'i',  'n',  '-', 
    'm',  'e',  'n',  'u',  0x67,  'U',  'r',  'l', 
    'L',  'i',  'n',  'k',  0x78,  0x19,  'h',  't', 
    't',  'p',  's',  ':',  '/',  '/',  'w',  'w', 
    'w',  '.',  'u',  'n',  'i',  'o',  'n',  't', 
    'e',  'c',  'h',  '.',  'c',  'o',  'm',  0x66, 
    'V',  'e',  'n',  'd',  'o',  'r',  0x78,  0x2b, 
    'T',  'h',  'e',  ' ',  'U',  'n',  'i',  'o', 
    'n',  't',  'e',  'c',  'h',  ' ',  'S',  'o', 
    'f',  't',  'w',  'a',  'r',  'e',  ' ',  'T', 
    'e',  'c',  'h',  'n',  'o',  'l',  'o',  'g', 
    'y',  ' ',  'C',  'o',  '.',  ',',  ' ',  'L', 
    't',  'd',  '.',  0x67,  'V',  'e',  'r',  's', 
    'i',  'o',  'n',  0x65,  '1',  '.',  '0',  '.', 
    '0', 
    0xff, 
};
using namespace dfmplugin_menu;
QT_MOC_EXPORT_PLUGIN(dfmplugin_menu::Menu, Menu)

QT_WARNING_POP
QT_END_MOC_NAMESPACE
