/****************************************************************************
** Meta object code from reading C++ file 'accesscontroldbus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../dde-file-manager/src/services/accesscontrol/accesscontroldbus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'accesscontroldbus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_AccessControlDBus_t {
    QByteArrayData data[17];
    char stringdata0[227];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AccessControlDBus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AccessControlDBus_t qt_meta_stringdata_AccessControlDBus = {
    {
QT_MOC_LITERAL(0, 0, 17), // "AccessControlDBus"
QT_MOC_LITERAL(1, 18, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 34, 43), // "org.deepin.Filemanager.Access..."
QT_MOC_LITERAL(3, 78, 19), // "DiskPasswordChecked"
QT_MOC_LITERAL(4, 98, 0), // ""
QT_MOC_LITERAL(5, 99, 4), // "code"
QT_MOC_LITERAL(6, 104, 19), // "DiskPasswordChanged"
QT_MOC_LITERAL(7, 124, 18), // "ChangeDiskPassword"
QT_MOC_LITERAL(8, 143, 6), // "oldPwd"
QT_MOC_LITERAL(9, 150, 6), // "newPwd"
QT_MOC_LITERAL(10, 157, 5), // "Chmod"
QT_MOC_LITERAL(11, 163, 4), // "path"
QT_MOC_LITERAL(12, 168, 4), // "mode"
QT_MOC_LITERAL(13, 173, 15), // "onBlockDevAdded"
QT_MOC_LITERAL(14, 189, 8), // "deviceId"
QT_MOC_LITERAL(15, 198, 17), // "onBlockDevMounted"
QT_MOC_LITERAL(16, 216, 10) // "mountPoint"

    },
    "AccessControlDBus\0D-Bus Interface\0"
    "org.deepin.Filemanager.AccessControlManager\0"
    "DiskPasswordChecked\0\0code\0DiskPasswordChanged\0"
    "ChangeDiskPassword\0oldPwd\0newPwd\0Chmod\0"
    "path\0mode\0onBlockDevAdded\0deviceId\0"
    "onBlockDevMounted\0mountPoint"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AccessControlDBus[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       6,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   46,    4, 0x06 /* Public */,
       6,    1,   49,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    2,   52,    4, 0x0a /* Public */,
      10,    2,   57,    4, 0x0a /* Public */,
      13,    1,   62,    4, 0x08 /* Private */,
      15,    2,   65,    4, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    8,    9,
    QMetaType::Bool, QMetaType::QString, QMetaType::UInt,   11,   12,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   14,   16,

       0        // eod
};

void AccessControlDBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AccessControlDBus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->DiskPasswordChecked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->DiskPasswordChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->ChangeDiskPassword((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: { bool _r = _t->Chmod((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->onBlockDevAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->onBlockDevMounted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AccessControlDBus::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AccessControlDBus::DiskPasswordChecked)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AccessControlDBus::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AccessControlDBus::DiskPasswordChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject AccessControlDBus::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_AccessControlDBus.data,
    qt_meta_data_AccessControlDBus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AccessControlDBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AccessControlDBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AccessControlDBus.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int AccessControlDBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void AccessControlDBus::DiskPasswordChecked(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void AccessControlDBus::DiskPasswordChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
