/****************************************************************************
** Meta object code from reading C++ file 'infocache.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../dde-file-manager/include/dfm-base/utils/infocache.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'infocache.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmbase__CacheWorker_t {
    QByteArrayData data[12];
    char stringdata0[138];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__CacheWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__CacheWorker_t qt_meta_stringdata_dfmbase__CacheWorker = {
    {
QT_MOC_LITERAL(0, 0, 20), // "dfmbase::CacheWorker"
QT_MOC_LITERAL(1, 21, 9), // "cacheInfo"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 3), // "url"
QT_MOC_LITERAL(4, 36, 15), // "FileInfoPointer"
QT_MOC_LITERAL(5, 52, 4), // "info"
QT_MOC_LITERAL(6, 57, 12), // "removeCaches"
QT_MOC_LITERAL(7, 70, 11), // "QList<QUrl>"
QT_MOC_LITERAL(8, 82, 4), // "urls"
QT_MOC_LITERAL(9, 87, 17), // "disconnectWatcher"
QT_MOC_LITERAL(10, 105, 26), // "QMap<QUrl,FileInfoPointer>"
QT_MOC_LITERAL(11, 132, 5) // "infos"

    },
    "dfmbase::CacheWorker\0cacheInfo\0\0url\0"
    "FileInfoPointer\0info\0removeCaches\0"
    "QList<QUrl>\0urls\0disconnectWatcher\0"
    "QMap<QUrl,FileInfoPointer>\0infos"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__CacheWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   29,    2, 0x0a /* Public */,
       6,    1,   34,    2, 0x0a /* Public */,
       9,    1,   37,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 4,    3,    5,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 10,   11,

       0        // eod
};

void dfmbase::CacheWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CacheWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cacheInfo((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const FileInfoPointer(*)>(_a[2]))); break;
        case 1: _t->removeCaches((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 2: _t->disconnectWatcher((*reinterpret_cast< const QMap<QUrl,FileInfoPointer>(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< FileInfoPointer >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::CacheWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__CacheWorker.data,
    qt_meta_data_dfmbase__CacheWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::CacheWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::CacheWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__CacheWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmbase::CacheWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
struct qt_meta_stringdata_dfmbase__TimeToUpdateCache_t {
    QByteArrayData data[9];
    char stringdata0[101];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__TimeToUpdateCache_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__TimeToUpdateCache_t qt_meta_stringdata_dfmbase__TimeToUpdateCache = {
    {
QT_MOC_LITERAL(0, 0, 26), // "dfmbase::TimeToUpdateCache"
QT_MOC_LITERAL(1, 27, 14), // "updateInfoTime"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 3), // "url"
QT_MOC_LITERAL(4, 47, 14), // "dealRemoveInfo"
QT_MOC_LITERAL(5, 62, 17), // "updateWatcherTime"
QT_MOC_LITERAL(6, 80, 11), // "QList<QUrl>"
QT_MOC_LITERAL(7, 92, 4), // "urls"
QT_MOC_LITERAL(8, 97, 3) // "add"

    },
    "dfmbase::TimeToUpdateCache\0updateInfoTime\0"
    "\0url\0dealRemoveInfo\0updateWatcherTime\0"
    "QList<QUrl>\0urls\0add"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__TimeToUpdateCache[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x0a /* Public */,
       4,    0,   32,    2, 0x0a /* Public */,
       5,    2,   33,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl,    3,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6, QMetaType::Bool,    7,    8,

       0        // eod
};

void dfmbase::TimeToUpdateCache::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TimeToUpdateCache *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateInfoTime((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 1: _t->dealRemoveInfo(); break;
        case 2: _t->updateWatcherTime((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::TimeToUpdateCache::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__TimeToUpdateCache.data,
    qt_meta_data_dfmbase__TimeToUpdateCache,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::TimeToUpdateCache::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::TimeToUpdateCache::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__TimeToUpdateCache.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmbase::TimeToUpdateCache::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
struct qt_meta_stringdata_dfmbase__InfoCache_t {
    QByteArrayData data[13];
    char stringdata0[183];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__InfoCache_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__InfoCache_t qt_meta_stringdata_dfmbase__InfoCache = {
    {
QT_MOC_LITERAL(0, 0, 18), // "dfmbase::InfoCache"
QT_MOC_LITERAL(1, 19, 17), // "cacheRemoveCaches"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 11), // "QList<QUrl>"
QT_MOC_LITERAL(4, 50, 3), // "key"
QT_MOC_LITERAL(5, 54, 22), // "cacheDisconnectWatcher"
QT_MOC_LITERAL(6, 77, 26), // "QMap<QUrl,FileInfoPointer>"
QT_MOC_LITERAL(7, 104, 5), // "infos"
QT_MOC_LITERAL(8, 110, 19), // "cacheUpdateInfoTime"
QT_MOC_LITERAL(9, 130, 3), // "url"
QT_MOC_LITERAL(10, 134, 20), // "fileAttributeChanged"
QT_MOC_LITERAL(11, 155, 11), // "removeCache"
QT_MOC_LITERAL(12, 167, 15) // "refreshFileInfo"

    },
    "dfmbase::InfoCache\0cacheRemoveCaches\0"
    "\0QList<QUrl>\0key\0cacheDisconnectWatcher\0"
    "QMap<QUrl,FileInfoPointer>\0infos\0"
    "cacheUpdateInfoTime\0url\0fileAttributeChanged\0"
    "removeCache\0refreshFileInfo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__InfoCache[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       5,    1,   47,    2, 0x06 /* Public */,
       8,    1,   50,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,   53,    2, 0x08 /* Private */,
      11,    1,   56,    2, 0x08 /* Private */,
      12,    1,   59,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::QUrl,    9,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl,    9,
    QMetaType::Void, QMetaType::QUrl,    9,
    QMetaType::Void, QMetaType::QUrl,    9,

       0        // eod
};

void dfmbase::InfoCache::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<InfoCache *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cacheRemoveCaches((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 1: _t->cacheDisconnectWatcher((*reinterpret_cast< const QMap<QUrl,FileInfoPointer>(*)>(_a[1]))); break;
        case 2: _t->cacheUpdateInfoTime((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 3: _t->fileAttributeChanged((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 4: _t->removeCache((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 5: _t->refreshFileInfo((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (InfoCache::*)(const QList<QUrl> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InfoCache::cacheRemoveCaches)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (InfoCache::*)(const QMap<QUrl,FileInfoPointer> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InfoCache::cacheDisconnectWatcher)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (InfoCache::*)(const QUrl );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InfoCache::cacheUpdateInfoTime)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::InfoCache::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__InfoCache.data,
    qt_meta_data_dfmbase__InfoCache,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::InfoCache::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::InfoCache::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__InfoCache.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmbase::InfoCache::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void dfmbase::InfoCache::cacheRemoveCaches(const QList<QUrl> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmbase::InfoCache::cacheDisconnectWatcher(const QMap<QUrl,FileInfoPointer> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmbase::InfoCache::cacheUpdateInfoTime(const QUrl _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
struct qt_meta_stringdata_dfmbase__InfoCacheController_t {
    QByteArrayData data[9];
    char stringdata0[106];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__InfoCacheController_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__InfoCacheController_t qt_meta_stringdata_dfmbase__InfoCacheController = {
    {
QT_MOC_LITERAL(0, 0, 28), // "dfmbase::InfoCacheController"
QT_MOC_LITERAL(1, 29, 13), // "cacheFileInfo"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 3), // "url"
QT_MOC_LITERAL(4, 48, 15), // "FileInfoPointer"
QT_MOC_LITERAL(5, 64, 4), // "info"
QT_MOC_LITERAL(6, 69, 19), // "removeCacheFileInfo"
QT_MOC_LITERAL(7, 89, 11), // "QList<QUrl>"
QT_MOC_LITERAL(8, 101, 4) // "urls"

    },
    "dfmbase::InfoCacheController\0cacheFileInfo\0"
    "\0url\0FileInfoPointer\0info\0removeCacheFileInfo\0"
    "QList<QUrl>\0urls"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__InfoCacheController[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x06 /* Public */,
       6,    1,   29,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 4,    3,    5,
    QMetaType::Void, 0x80000000 | 7,    8,

       0        // eod
};

void dfmbase::InfoCacheController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<InfoCacheController *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cacheFileInfo((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const FileInfoPointer(*)>(_a[2]))); break;
        case 1: _t->removeCacheFileInfo((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< FileInfoPointer >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (InfoCacheController::*)(const QUrl , const FileInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InfoCacheController::cacheFileInfo)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (InfoCacheController::*)(const QList<QUrl> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InfoCacheController::removeCacheFileInfo)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::InfoCacheController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__InfoCacheController.data,
    qt_meta_data_dfmbase__InfoCacheController,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::InfoCacheController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::InfoCacheController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__InfoCacheController.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmbase::InfoCacheController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void dfmbase::InfoCacheController::cacheFileInfo(const QUrl _t1, const FileInfoPointer _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmbase::InfoCacheController::removeCacheFileInfo(const QList<QUrl> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
