/****************************************************************************
** Meta object code from reading C++ file 'taskwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../dde-file-manager/src/dfm-base/dialogs/taskdialog/taskwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'taskwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmbase__ElidedLable_t {
    QByteArrayData data[1];
    char stringdata0[21];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__ElidedLable_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__ElidedLable_t qt_meta_stringdata_dfmbase__ElidedLable = {
    {
QT_MOC_LITERAL(0, 0, 20) // "dfmbase::ElidedLable"

    },
    "dfmbase::ElidedLable"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__ElidedLable[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void dfmbase::ElidedLable::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::ElidedLable::staticMetaObject = { {
    QMetaObject::SuperData::link<QLabel::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__ElidedLable.data,
    qt_meta_data_dfmbase__ElidedLable,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::ElidedLable::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::ElidedLable::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__ElidedLable.stringdata0))
        return static_cast<void*>(this);
    return QLabel::qt_metacast(_clname);
}

int dfmbase::ElidedLable::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLabel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_dfmbase__TaskWidget_t {
    QByteArrayData data[21];
    char stringdata0[297];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__TaskWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__TaskWidget_t qt_meta_stringdata_dfmbase__TaskWidget = {
    {
QT_MOC_LITERAL(0, 0, 19), // "dfmbase::TaskWidget"
QT_MOC_LITERAL(1, 20, 13), // "buttonClicked"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 34), // "AbstractJobHandler::SupportAc..."
QT_MOC_LITERAL(4, 70, 7), // "actions"
QT_MOC_LITERAL(5, 78, 13), // "heightChanged"
QT_MOC_LITERAL(6, 92, 6), // "height"
QT_MOC_LITERAL(7, 99, 11), // "parentClose"
QT_MOC_LITERAL(8, 111, 15), // "onButtonClicked"
QT_MOC_LITERAL(9, 127, 12), // "onShowErrors"
QT_MOC_LITERAL(10, 140, 14), // "JobInfoPointer"
QT_MOC_LITERAL(11, 155, 7), // "JobInfo"
QT_MOC_LITERAL(12, 163, 18), // "onShowConflictInfo"
QT_MOC_LITERAL(13, 182, 6), // "source"
QT_MOC_LITERAL(14, 189, 6), // "target"
QT_MOC_LITERAL(15, 196, 6), // "action"
QT_MOC_LITERAL(16, 203, 24), // "onHandlerTaskStateChange"
QT_MOC_LITERAL(17, 228, 14), // "onShowTaskInfo"
QT_MOC_LITERAL(18, 243, 18), // "onShowTaskProccess"
QT_MOC_LITERAL(19, 262, 22), // "onShowSpeedUpdatedInfo"
QT_MOC_LITERAL(20, 285, 11) // "onInfoTimer"

    },
    "dfmbase::TaskWidget\0buttonClicked\0\0"
    "AbstractJobHandler::SupportActions\0"
    "actions\0heightChanged\0height\0parentClose\0"
    "onButtonClicked\0onShowErrors\0"
    "JobInfoPointer\0JobInfo\0onShowConflictInfo\0"
    "source\0target\0action\0onHandlerTaskStateChange\0"
    "onShowTaskInfo\0onShowTaskProccess\0"
    "onShowSpeedUpdatedInfo\0onInfoTimer"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__TaskWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,
       5,    1,   72,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,   75,    2, 0x0a /* Public */,
       8,    0,   76,    2, 0x08 /* Private */,
       9,    1,   77,    2, 0x08 /* Private */,
      12,    3,   80,    2, 0x08 /* Private */,
      16,    1,   87,    2, 0x08 /* Private */,
      17,    1,   90,    2, 0x08 /* Private */,
      18,    1,   93,    2, 0x08 /* Private */,
      19,    1,   96,    2, 0x08 /* Private */,
      20,    0,   99,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Int,    6,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl, 0x80000000 | 3,   13,   14,   15,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,

       0        // eod
};

void dfmbase::TaskWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TaskWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->buttonClicked((*reinterpret_cast< AbstractJobHandler::SupportActions(*)>(_a[1]))); break;
        case 1: _t->heightChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->parentClose(); break;
        case 3: _t->onButtonClicked(); break;
        case 4: _t->onShowErrors((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 5: _t->onShowConflictInfo((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const AbstractJobHandler::SupportActions(*)>(_a[3]))); break;
        case 6: _t->onHandlerTaskStateChange((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 7: _t->onShowTaskInfo((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 8: _t->onShowTaskProccess((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 9: _t->onShowSpeedUpdatedInfo((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 10: _t->onInfoTimer(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TaskWidget::*)(AbstractJobHandler::SupportActions );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TaskWidget::buttonClicked)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TaskWidget::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TaskWidget::heightChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::TaskWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__TaskWidget.data,
    qt_meta_data_dfmbase__TaskWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::TaskWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::TaskWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__TaskWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int dfmbase::TaskWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void dfmbase::TaskWidget::buttonClicked(AbstractJobHandler::SupportActions _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmbase::TaskWidget::heightChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
