/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a textindexadaptor -i textindexdbus.h -l TextIndexDBus /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.TextIndex.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "textindexadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class TextIndexAdaptor
 */

TextIndexAdaptor::TextIndexAdaptor(TextIndexDBus *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

TextIndexAdaptor::~TextIndexAdaptor()
{
    // destructor
}

bool TextIndexAdaptor::CreateIndexTask(const QString &path)
{
    // handle method call org.deepin.Filemanager.TextIndex.CreateIndexTask
    return parent()->CreateIndexTask(path);
}

QString TextIndexAdaptor::GetLastUpdateTime()
{
    // handle method call org.deepin.Filemanager.TextIndex.GetLastUpdateTime
    return parent()->GetLastUpdateTime();
}

bool TextIndexAdaptor::HasRunningTask()
{
    // handle method call org.deepin.Filemanager.TextIndex.HasRunningTask
    return parent()->HasRunningTask();
}

bool TextIndexAdaptor::IndexDatabaseExists()
{
    // handle method call org.deepin.Filemanager.TextIndex.IndexDatabaseExists
    return parent()->IndexDatabaseExists();
}

bool TextIndexAdaptor::RemoveIndexTask(const QStringList &paths)
{
    // handle method call org.deepin.Filemanager.TextIndex.RemoveIndexTask
    return parent()->RemoveIndexTask(paths);
}

bool TextIndexAdaptor::StopCurrentTask()
{
    // handle method call org.deepin.Filemanager.TextIndex.StopCurrentTask
    return parent()->StopCurrentTask();
}

bool TextIndexAdaptor::UpdateIndexTask(const QString &path)
{
    // handle method call org.deepin.Filemanager.TextIndex.UpdateIndexTask
    return parent()->UpdateIndexTask(path);
}


#include "textindexadaptor.moc"
