/****************************************************************************
** Meta object code from reading C++ file 'typedefines.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-smbbrowser/typedefines.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'typedefines.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_smbbrowser_t {
    QByteArrayData data[4];
    char stringdata0[59];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_smbbrowser_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_smbbrowser_t qt_meta_stringdata_dfmplugin_smbbrowser = {
    {
QT_MOC_LITERAL(0, 0, 20), // "dfmplugin_smbbrowser"
QT_MOC_LITERAL(1, 21, 14), // "SmbDisplayMode"
QT_MOC_LITERAL(2, 36, 9), // "kSeperate"
QT_MOC_LITERAL(3, 46, 12) // "kAggregation"

    },
    "dfmplugin_smbbrowser\0SmbDisplayMode\0"
    "kSeperate\0kAggregation"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_smbbrowser[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       1,   14, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // enums: name, alias, flags, count, data
       1,    1, 0x0,    2,   19,

 // enum data: key, value
       2, uint(dfmplugin_smbbrowser::kSeperate),
       3, uint(dfmplugin_smbbrowser::kAggregation),

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject dfmplugin_smbbrowser::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_dfmplugin_smbbrowser.data,
    qt_meta_data_dfmplugin_smbbrowser,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
