/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a accesscontroladaptor -i accesscontroldbus.h -l AccessControlDBus /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.AccessControl.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "accesscontroladaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class AccessControlManagerAdaptor
 */

AccessControlManagerAdaptor::AccessControlManagerAdaptor(AccessControlDBus *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

AccessControlManagerAdaptor::~AccessControlManagerAdaptor()
{
    // destructor
}

void AccessControlManagerAdaptor::ChangeDiskPassword(const QString &oldPwd, const QString &newPwd)
{
    // handle method call org.deepin.Filemanager.AccessControlManager.ChangeDiskPassword
    parent()->ChangeDiskPassword(oldPwd, newPwd);
}

bool AccessControlManagerAdaptor::Chmod(const QString &path, uint mode)
{
    // handle method call org.deepin.Filemanager.AccessControlManager.Chmod
    return parent()->Chmod(path, mode);
}


#include "accesscontroladaptor.moc"
