/****************************************************************************
** Meta object code from reading C++ file 'application.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../dde-file-manager/src/dfm-base/base/application/application.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'application.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmbase__Application_t {
    QByteArrayData data[69];
    char stringdata0[1273];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__Application_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__Application_t qt_meta_stringdata_dfmbase__Application = {
    {
QT_MOC_LITERAL(0, 0, 20), // "dfmbase::Application"
QT_MOC_LITERAL(1, 21, 19), // "appAttributeChanged"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 20), // "ApplicationAttribute"
QT_MOC_LITERAL(4, 63, 2), // "aa"
QT_MOC_LITERAL(5, 66, 5), // "value"
QT_MOC_LITERAL(6, 72, 23), // "genericAttributeChanged"
QT_MOC_LITERAL(7, 96, 16), // "GenericAttribute"
QT_MOC_LITERAL(8, 113, 2), // "ga"
QT_MOC_LITERAL(9, 116, 18), // "appAttributeEdited"
QT_MOC_LITERAL(10, 135, 22), // "genericAttributeEdited"
QT_MOC_LITERAL(11, 158, 20), // "iconSizeLevelChanged"
QT_MOC_LITERAL(12, 179, 5), // "level"
QT_MOC_LITERAL(13, 185, 23), // "gridDensityLevelChanged"
QT_MOC_LITERAL(14, 209, 22), // "listHeightLevelChanged"
QT_MOC_LITERAL(15, 232, 15), // "viewModeChanged"
QT_MOC_LITERAL(16, 248, 4), // "mode"
QT_MOC_LITERAL(17, 253, 26), // "previewCompressFileChanged"
QT_MOC_LITERAL(18, 280, 6), // "enable"
QT_MOC_LITERAL(19, 287, 23), // "showedFileSuffixChanged"
QT_MOC_LITERAL(20, 311, 23), // "previewAttributeChanged"
QT_MOC_LITERAL(21, 335, 24), // "showedHiddenFilesChanged"
QT_MOC_LITERAL(22, 360, 32), // "csdClickableAreaAttributeChanged"
QT_MOC_LITERAL(23, 393, 7), // "enabled"
QT_MOC_LITERAL(24, 401, 26), // "indexFullTextSearchChanged"
QT_MOC_LITERAL(25, 428, 21), // "genericSettingCreated"
QT_MOC_LITERAL(26, 450, 9), // "Settings*"
QT_MOC_LITERAL(27, 460, 8), // "settings"
QT_MOC_LITERAL(28, 469, 17), // "appSettingCreated"
QT_MOC_LITERAL(29, 487, 18), // "clearSearchHistory"
QT_MOC_LITERAL(30, 506, 5), // "winId"
QT_MOC_LITERAL(31, 512, 22), // "kAllwayOpenOnNewWindow"
QT_MOC_LITERAL(32, 535, 14), // "kIconSizeLevel"
QT_MOC_LITERAL(33, 550, 9), // "kViewMode"
QT_MOC_LITERAL(34, 560, 19), // "kViewSizeAdjustable"
QT_MOC_LITERAL(35, 580, 17), // "kViewComppactMode"
QT_MOC_LITERAL(36, 598, 16), // "kViewAutoCompace"
QT_MOC_LITERAL(37, 615, 13), // "kOpenFileMode"
QT_MOC_LITERAL(38, 629, 15), // "kUrlOfNewWindow"
QT_MOC_LITERAL(39, 645, 12), // "kUrlOfNewTab"
QT_MOC_LITERAL(40, 658, 10), // "kThemeName"
QT_MOC_LITERAL(41, 669, 20), // "kFileAndDirMixedSort"
QT_MOC_LITERAL(42, 690, 19), // "kListItemExpandable"
QT_MOC_LITERAL(43, 710, 17), // "kGridDensityLevel"
QT_MOC_LITERAL(44, 728, 16), // "kListHeightLevel"
QT_MOC_LITERAL(45, 745, 20), // "kPreviewCompressFile"
QT_MOC_LITERAL(46, 766, 16), // "kPreviewTextFile"
QT_MOC_LITERAL(47, 783, 20), // "kPreviewDocumentFile"
QT_MOC_LITERAL(48, 804, 13), // "kPreviewImage"
QT_MOC_LITERAL(49, 818, 13), // "kPreviewVideo"
QT_MOC_LITERAL(50, 832, 13), // "kPreviewAudio"
QT_MOC_LITERAL(51, 846, 10), // "kAutoMount"
QT_MOC_LITERAL(52, 857, 17), // "kAutoMountAndOpen"
QT_MOC_LITERAL(53, 875, 35), // "kAlwaysShowOfflineRemoteConne..."
QT_MOC_LITERAL(54, 911, 36), // "kMergeTheEntriesOfSambaShared..."
QT_MOC_LITERAL(55, 948, 26), // "kOverrideFileChooserDialog"
QT_MOC_LITERAL(56, 975, 21), // "kShowedHiddenOnSearch"
QT_MOC_LITERAL(57, 997, 18), // "kShowedHiddenFiles"
QT_MOC_LITERAL(58, 1016, 17), // "kShowedFileSuffix"
QT_MOC_LITERAL(59, 1034, 33), // "kDisableNonRemovableDeviceUnm..."
QT_MOC_LITERAL(60, 1068, 22), // "kHiddenSystemPartition"
QT_MOC_LITERAL(61, 1091, 29), // "kShowCsdCrumbBarClickableArea"
QT_MOC_LITERAL(62, 1121, 28), // "kShowFileSystemTagOnDiskIcon"
QT_MOC_LITERAL(63, 1150, 24), // "kShowDeleteConfirmDialog"
QT_MOC_LITERAL(64, 1175, 19), // "kHideLoopPartitions"
QT_MOC_LITERAL(65, 1195, 23), // "kShowThunmbnailInRemote"
QT_MOC_LITERAL(66, 1219, 16), // "TriggerAttribute"
QT_MOC_LITERAL(67, 1236, 16), // "kRestoreViewMode"
QT_MOC_LITERAL(68, 1253, 19) // "kClearSearchHistory"

    },
    "dfmbase::Application\0appAttributeChanged\0"
    "\0ApplicationAttribute\0aa\0value\0"
    "genericAttributeChanged\0GenericAttribute\0"
    "ga\0appAttributeEdited\0genericAttributeEdited\0"
    "iconSizeLevelChanged\0level\0"
    "gridDensityLevelChanged\0listHeightLevelChanged\0"
    "viewModeChanged\0mode\0previewCompressFileChanged\0"
    "enable\0showedFileSuffixChanged\0"
    "previewAttributeChanged\0"
    "showedHiddenFilesChanged\0"
    "csdClickableAreaAttributeChanged\0"
    "enabled\0indexFullTextSearchChanged\0"
    "genericSettingCreated\0Settings*\0"
    "settings\0appSettingCreated\0"
    "clearSearchHistory\0winId\0"
    "kAllwayOpenOnNewWindow\0kIconSizeLevel\0"
    "kViewMode\0kViewSizeAdjustable\0"
    "kViewComppactMode\0kViewAutoCompace\0"
    "kOpenFileMode\0kUrlOfNewWindow\0"
    "kUrlOfNewTab\0kThemeName\0kFileAndDirMixedSort\0"
    "kListItemExpandable\0kGridDensityLevel\0"
    "kListHeightLevel\0kPreviewCompressFile\0"
    "kPreviewTextFile\0kPreviewDocumentFile\0"
    "kPreviewImage\0kPreviewVideo\0kPreviewAudio\0"
    "kAutoMount\0kAutoMountAndOpen\0"
    "kAlwaysShowOfflineRemoteConnections\0"
    "kMergeTheEntriesOfSambaSharedFolders\0"
    "kOverrideFileChooserDialog\0"
    "kShowedHiddenOnSearch\0kShowedHiddenFiles\0"
    "kShowedFileSuffix\0kDisableNonRemovableDeviceUnmount\0"
    "kHiddenSystemPartition\0"
    "kShowCsdCrumbBarClickableArea\0"
    "kShowFileSystemTagOnDiskIcon\0"
    "kShowDeleteConfirmDialog\0kHideLoopPartitions\0"
    "kShowThunmbnailInRemote\0TriggerAttribute\0"
    "kRestoreViewMode\0kClearSearchHistory"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__Application[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       3,  160, // enums/sets
       0,    0, // constructors
       0,       // flags
      17,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   99,    2, 0x06 /* Public */,
       6,    2,  104,    2, 0x06 /* Public */,
       9,    2,  109,    2, 0x06 /* Public */,
      10,    2,  114,    2, 0x06 /* Public */,
      11,    1,  119,    2, 0x06 /* Public */,
      13,    1,  122,    2, 0x06 /* Public */,
      14,    1,  125,    2, 0x06 /* Public */,
      15,    1,  128,    2, 0x06 /* Public */,
      17,    1,  131,    2, 0x06 /* Public */,
      19,    1,  134,    2, 0x06 /* Public */,
      20,    2,  137,    2, 0x06 /* Public */,
      21,    1,  142,    2, 0x06 /* Public */,
      22,    1,  145,    2, 0x06 /* Public */,
      24,    1,  148,    2, 0x06 /* Public */,
      25,    1,  151,    2, 0x06 /* Public */,
      28,    1,  154,    2, 0x06 /* Public */,
      29,    1,  157,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QVariant,    4,    5,
    QMetaType::Void, 0x80000000 | 7, QMetaType::QVariant,    8,    5,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QVariant,    4,    5,
    QMetaType::Void, 0x80000000 | 7, QMetaType::QVariant,    8,    5,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Bool,    8,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void, 0x80000000 | 26,   27,
    QMetaType::Void, 0x80000000 | 26,   27,
    QMetaType::Void, QMetaType::ULongLong,   30,

 // enums: name, alias, flags, count, data
       3,    3, 0x0,   14,  175,
       7,    7, 0x0,   21,  203,
      66,   66, 0x0,    2,  245,

 // enum data: key, value
      31, uint(dfmbase::Application::kAllwayOpenOnNewWindow),
      32, uint(dfmbase::Application::kIconSizeLevel),
      33, uint(dfmbase::Application::kViewMode),
      34, uint(dfmbase::Application::kViewSizeAdjustable),
      35, uint(dfmbase::Application::kViewComppactMode),
      36, uint(dfmbase::Application::kViewAutoCompace),
      37, uint(dfmbase::Application::kOpenFileMode),
      38, uint(dfmbase::Application::kUrlOfNewWindow),
      39, uint(dfmbase::Application::kUrlOfNewTab),
      40, uint(dfmbase::Application::kThemeName),
      41, uint(dfmbase::Application::kFileAndDirMixedSort),
      42, uint(dfmbase::Application::kListItemExpandable),
      43, uint(dfmbase::Application::kGridDensityLevel),
      44, uint(dfmbase::Application::kListHeightLevel),
      45, uint(dfmbase::Application::kPreviewCompressFile),
      46, uint(dfmbase::Application::kPreviewTextFile),
      47, uint(dfmbase::Application::kPreviewDocumentFile),
      48, uint(dfmbase::Application::kPreviewImage),
      49, uint(dfmbase::Application::kPreviewVideo),
      50, uint(dfmbase::Application::kPreviewAudio),
      51, uint(dfmbase::Application::kAutoMount),
      52, uint(dfmbase::Application::kAutoMountAndOpen),
      53, uint(dfmbase::Application::kAlwaysShowOfflineRemoteConnections),
      54, uint(dfmbase::Application::kMergeTheEntriesOfSambaSharedFolders),
      55, uint(dfmbase::Application::kOverrideFileChooserDialog),
      56, uint(dfmbase::Application::kShowedHiddenOnSearch),
      57, uint(dfmbase::Application::kShowedHiddenFiles),
      58, uint(dfmbase::Application::kShowedFileSuffix),
      59, uint(dfmbase::Application::kDisableNonRemovableDeviceUnmount),
      60, uint(dfmbase::Application::kHiddenSystemPartition),
      61, uint(dfmbase::Application::kShowCsdCrumbBarClickableArea),
      62, uint(dfmbase::Application::kShowFileSystemTagOnDiskIcon),
      63, uint(dfmbase::Application::kShowDeleteConfirmDialog),
      64, uint(dfmbase::Application::kHideLoopPartitions),
      65, uint(dfmbase::Application::kShowThunmbnailInRemote),
      67, uint(dfmbase::Application::kRestoreViewMode),
      68, uint(dfmbase::Application::kClearSearchHistory),

       0        // eod
};

void dfmbase::Application::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Application *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->appAttributeChanged((*reinterpret_cast< ApplicationAttribute(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 1: _t->genericAttributeChanged((*reinterpret_cast< GenericAttribute(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 2: _t->appAttributeEdited((*reinterpret_cast< ApplicationAttribute(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 3: _t->genericAttributeEdited((*reinterpret_cast< GenericAttribute(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 4: _t->iconSizeLevelChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->gridDensityLevelChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->listHeightLevelChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->viewModeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->previewCompressFileChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->showedFileSuffixChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->previewAttributeChanged((*reinterpret_cast< GenericAttribute(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 11: _t->showedHiddenFilesChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->csdClickableAreaAttributeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->indexFullTextSearchChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->genericSettingCreated((*reinterpret_cast< Settings*(*)>(_a[1]))); break;
        case 15: _t->appSettingCreated((*reinterpret_cast< Settings*(*)>(_a[1]))); break;
        case 16: _t->clearSearchHistory((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Application::*)(ApplicationAttribute , const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::appAttributeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Application::*)(GenericAttribute , const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::genericAttributeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Application::*)(ApplicationAttribute , const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::appAttributeEdited)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Application::*)(GenericAttribute , const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::genericAttributeEdited)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Application::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::iconSizeLevelChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Application::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::gridDensityLevelChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Application::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::listHeightLevelChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Application::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::viewModeChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Application::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::previewCompressFileChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Application::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::showedFileSuffixChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (Application::*)(GenericAttribute , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::previewAttributeChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (Application::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::showedHiddenFilesChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (Application::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::csdClickableAreaAttributeChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (Application::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::indexFullTextSearchChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (Application::*)(Settings * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::genericSettingCreated)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (Application::*)(Settings * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::appSettingCreated)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (Application::*)(quint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Application::clearSearchHistory)) {
                *result = 16;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::Application::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__Application.data,
    qt_meta_data_dfmbase__Application,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::Application::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::Application::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__Application.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmbase::Application::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void dfmbase::Application::appAttributeChanged(ApplicationAttribute _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmbase::Application::genericAttributeChanged(GenericAttribute _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmbase::Application::appAttributeEdited(ApplicationAttribute _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmbase::Application::genericAttributeEdited(GenericAttribute _t1, const QVariant & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmbase::Application::iconSizeLevelChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmbase::Application::gridDensityLevelChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void dfmbase::Application::listHeightLevelChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void dfmbase::Application::viewModeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void dfmbase::Application::previewCompressFileChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void dfmbase::Application::showedFileSuffixChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void dfmbase::Application::previewAttributeChanged(GenericAttribute _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void dfmbase::Application::showedHiddenFilesChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void dfmbase::Application::csdClickableAreaAttributeChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void dfmbase::Application::indexFullTextSearchChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void dfmbase::Application::genericSettingCreated(Settings * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void dfmbase::Application::appSettingCreated(Settings * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void dfmbase::Application::clearSearchHistory(quint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
