/****************************************************************************
** Meta object code from reading C++ file 'bluetoothmanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-utils/bluetooth/private/bluetoothmanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bluetoothmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_utils__BluetoothManager_t {
    QByteArrayData data[24];
    char stringdata0[357];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_utils__BluetoothManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_utils__BluetoothManager_t qt_meta_stringdata_dfmplugin_utils__BluetoothManager = {
    {
QT_MOC_LITERAL(0, 0, 33), // "dfmplugin_utils::BluetoothMan..."
QT_MOC_LITERAL(1, 34, 23), // "transferProgressUpdated"
QT_MOC_LITERAL(2, 58, 0), // ""
QT_MOC_LITERAL(3, 59, 11), // "sessionPath"
QT_MOC_LITERAL(4, 71, 5), // "total"
QT_MOC_LITERAL(5, 77, 11), // "transferred"
QT_MOC_LITERAL(6, 89, 13), // "currFileIndex"
QT_MOC_LITERAL(7, 103, 23), // "transferCancledByRemote"
QT_MOC_LITERAL(8, 127, 20), // "fileTransferFinished"
QT_MOC_LITERAL(9, 148, 8), // "filePath"
QT_MOC_LITERAL(10, 157, 14), // "transferFailed"
QT_MOC_LITERAL(11, 172, 6), // "errMsg"
QT_MOC_LITERAL(12, 179, 23), // "transferEstablishFinish"
QT_MOC_LITERAL(13, 203, 11), // "senderToken"
QT_MOC_LITERAL(14, 215, 12), // "adapterAdded"
QT_MOC_LITERAL(15, 228, 23), // "const BluetoothAdapter*"
QT_MOC_LITERAL(16, 252, 7), // "adapter"
QT_MOC_LITERAL(17, 260, 14), // "adapterRemoved"
QT_MOC_LITERAL(18, 275, 7), // "refresh"
QT_MOC_LITERAL(19, 283, 21), // "showBluetoothSettings"
QT_MOC_LITERAL(20, 305, 9), // "sendFiles"
QT_MOC_LITERAL(21, 315, 2), // "id"
QT_MOC_LITERAL(22, 318, 14), // "cancelTransfer"
QT_MOC_LITERAL(23, 333, 23) // "canSendBluetoothRequest"

    },
    "dfmplugin_utils::BluetoothManager\0"
    "transferProgressUpdated\0\0sessionPath\0"
    "total\0transferred\0currFileIndex\0"
    "transferCancledByRemote\0fileTransferFinished\0"
    "filePath\0transferFailed\0errMsg\0"
    "transferEstablishFinish\0senderToken\0"
    "adapterAdded\0const BluetoothAdapter*\0"
    "adapter\0adapterRemoved\0refresh\0"
    "showBluetoothSettings\0sendFiles\0id\0"
    "cancelTransfer\0canSendBluetoothRequest"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_utils__BluetoothManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    4,   74,    2, 0x06 /* Public */,
       7,    1,   83,    2, 0x06 /* Public */,
       8,    2,   86,    2, 0x06 /* Public */,
      10,    3,   91,    2, 0x06 /* Public */,
      12,    3,   98,    2, 0x06 /* Public */,
      14,    1,  105,    2, 0x06 /* Public */,
      17,    1,  108,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    0,  111,    2, 0x0a /* Public */,
      19,    0,  112,    2, 0x0a /* Public */,
      20,    3,  113,    2, 0x0a /* Public */,
      22,    1,  120,    2, 0x0a /* Public */,
      23,    0,  123,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::ULongLong, QMetaType::ULongLong, QMetaType::Int,    3,    4,    5,    6,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    9,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString,    3,    9,   11,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString,    3,   11,   13,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, 0x80000000 | 15,   16,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QStringList, QMetaType::QString,   21,    9,   13,
    QMetaType::Bool, QMetaType::QString,    3,
    QMetaType::Bool,

       0        // eod
};

void dfmplugin_utils::BluetoothManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BluetoothManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->transferProgressUpdated((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qulonglong(*)>(_a[2])),(*reinterpret_cast< qulonglong(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 1: _t->transferCancledByRemote((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->fileTransferFinished((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->transferFailed((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 4: _t->transferEstablishFinish((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 5: _t->adapterAdded((*reinterpret_cast< const BluetoothAdapter*(*)>(_a[1]))); break;
        case 6: _t->adapterRemoved((*reinterpret_cast< const BluetoothAdapter*(*)>(_a[1]))); break;
        case 7: _t->refresh(); break;
        case 8: _t->showBluetoothSettings(); break;
        case 9: _t->sendFiles((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 10: { bool _r = _t->cancelTransfer((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->canSendBluetoothRequest();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (BluetoothManager::*)(const QString & , qulonglong , qulonglong , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothManager::transferProgressUpdated)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (BluetoothManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothManager::transferCancledByRemote)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (BluetoothManager::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothManager::fileTransferFinished)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (BluetoothManager::*)(const QString & , const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothManager::transferFailed)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (BluetoothManager::*)(const QString & , const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothManager::transferEstablishFinish)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (BluetoothManager::*)(const BluetoothAdapter * ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothManager::adapterAdded)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (BluetoothManager::*)(const BluetoothAdapter * ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothManager::adapterRemoved)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_utils::BluetoothManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_utils__BluetoothManager.data,
    qt_meta_data_dfmplugin_utils__BluetoothManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_utils::BluetoothManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_utils::BluetoothManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_utils__BluetoothManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_utils::BluetoothManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_utils::BluetoothManager::transferProgressUpdated(const QString & _t1, qulonglong _t2, qulonglong _t3, int _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_utils::BluetoothManager::transferCancledByRemote(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_utils::BluetoothManager::fileTransferFinished(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_utils::BluetoothManager::transferFailed(const QString & _t1, const QString & _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_utils::BluetoothManager::transferEstablishFinish(const QString & _t1, const QString & _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmplugin_utils::BluetoothManager::adapterAdded(const BluetoothAdapter * _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< dfmplugin_utils::BluetoothManager *>(this), &staticMetaObject, 5, _a);
}

// SIGNAL 6
void dfmplugin_utils::BluetoothManager::adapterRemoved(const BluetoothAdapter * _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< dfmplugin_utils::BluetoothManager *>(this), &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
