/****************************************************************************
** Meta object code from reading C++ file 'titlebareventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-titlebar/events/titlebareventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'titlebareventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_titlebar__TitleBarEventReceiver_t {
    QByteArrayData data[23];
    char stringdata0[371];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_titlebar__TitleBarEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_titlebar__TitleBarEventReceiver_t qt_meta_stringdata_dfmplugin_titlebar__TitleBarEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 41), // "dfmplugin_titlebar::TitleBarE..."
QT_MOC_LITERAL(1, 42, 20), // "handleCustomRegister"
QT_MOC_LITERAL(2, 63, 0), // ""
QT_MOC_LITERAL(3, 64, 6), // "scheme"
QT_MOC_LITERAL(4, 71, 10), // "properties"
QT_MOC_LITERAL(5, 82, 18), // "handleStartSpinner"
QT_MOC_LITERAL(6, 101, 8), // "windowId"
QT_MOC_LITERAL(7, 110, 17), // "handleStopSpinner"
QT_MOC_LITERAL(8, 128, 22), // "handleShowFilterButton"
QT_MOC_LITERAL(9, 151, 7), // "visible"
QT_MOC_LITERAL(10, 159, 21), // "handleViewModeChanged"
QT_MOC_LITERAL(11, 181, 4), // "mode"
QT_MOC_LITERAL(12, 186, 30), // "handleSetNewWindowAndTabEnable"
QT_MOC_LITERAL(13, 217, 6), // "enable"
QT_MOC_LITERAL(14, 224, 19), // "handleWindowForward"
QT_MOC_LITERAL(15, 244, 20), // "handleWindowBackward"
QT_MOC_LITERAL(16, 265, 19), // "handleRemoveHistory"
QT_MOC_LITERAL(17, 285, 3), // "url"
QT_MOC_LITERAL(18, 289, 16), // "handleTabAddable"
QT_MOC_LITERAL(19, 306, 15), // "handleCloseTabs"
QT_MOC_LITERAL(20, 322, 17), // "handleSetTabAlias"
QT_MOC_LITERAL(21, 340, 4), // "name"
QT_MOC_LITERAL(22, 345, 25) // "handleOpenNewTabTriggered"

    },
    "dfmplugin_titlebar::TitleBarEventReceiver\0"
    "handleCustomRegister\0\0scheme\0properties\0"
    "handleStartSpinner\0windowId\0"
    "handleStopSpinner\0handleShowFilterButton\0"
    "visible\0handleViewModeChanged\0mode\0"
    "handleSetNewWindowAndTabEnable\0enable\0"
    "handleWindowForward\0handleWindowBackward\0"
    "handleRemoveHistory\0url\0handleTabAddable\0"
    "handleCloseTabs\0handleSetTabAlias\0"
    "name\0handleOpenNewTabTriggered"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_titlebar__TitleBarEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   79,    2, 0x0a /* Public */,
       5,    1,   84,    2, 0x0a /* Public */,
       7,    1,   87,    2, 0x0a /* Public */,
       8,    2,   90,    2, 0x0a /* Public */,
      10,    2,   95,    2, 0x0a /* Public */,
      12,    1,  100,    2, 0x0a /* Public */,
      14,    1,  103,    2, 0x0a /* Public */,
      15,    1,  106,    2, 0x0a /* Public */,
      16,    2,  109,    2, 0x0a /* Public */,
      18,    1,  114,    2, 0x0a /* Public */,
      19,    1,  117,    2, 0x0a /* Public */,
      20,    2,  120,    2, 0x0a /* Public */,
      22,    2,  125,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Bool, QMetaType::QString, QMetaType::QVariantMap,    3,    4,
    QMetaType::Void, QMetaType::ULongLong,    6,
    QMetaType::Void, QMetaType::ULongLong,    6,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::Bool,    6,    9,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::Int,    6,   11,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Void, QMetaType::ULongLong,    6,
    QMetaType::Void, QMetaType::ULongLong,    6,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,    6,   17,
    QMetaType::Bool, QMetaType::ULongLong,    6,
    QMetaType::Void, QMetaType::QUrl,   17,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QString,   17,   21,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,    6,   17,

       0        // eod
};

void dfmplugin_titlebar::TitleBarEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TitleBarEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->handleCustomRegister((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->handleStartSpinner((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 2: _t->handleStopSpinner((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 3: _t->handleShowFilterButton((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 4: _t->handleViewModeChanged((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->handleSetNewWindowAndTabEnable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->handleWindowForward((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 7: _t->handleWindowBackward((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 8: _t->handleRemoveHistory((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 9: { bool _r = _t->handleTabAddable((*reinterpret_cast< quint64(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->handleCloseTabs((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 11: _t->handleSetTabAlias((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 12: _t->handleOpenNewTabTriggered((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_titlebar::TitleBarEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_titlebar__TitleBarEventReceiver.data,
    qt_meta_data_dfmplugin_titlebar__TitleBarEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_titlebar::TitleBarEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_titlebar::TitleBarEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_titlebar__TitleBarEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_titlebar::TitleBarEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
