/****************************************************************************
** Meta object code from reading C++ file 'fileinfoasycworker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../dde-file-manager/src/dfm-base/utils/fileinfoasycworker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileinfoasycworker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmbase__FileInfoAsycWorker_t {
    QByteArrayData data[19];
    char stringdata0[266];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__FileInfoAsycWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__FileInfoAsycWorker_t qt_meta_stringdata_dfmbase__FileInfoAsycWorker = {
    {
QT_MOC_LITERAL(0, 0, 27), // "dfmbase::FileInfoAsycWorker"
QT_MOC_LITERAL(1, 28, 20), // "fileConutAsyncFinish"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 3), // "url"
QT_MOC_LITERAL(4, 54, 5), // "files"
QT_MOC_LITERAL(5, 60, 20), // "fileMimeTypeFinished"
QT_MOC_LITERAL(6, 81, 9), // "QMimeType"
QT_MOC_LITERAL(7, 91, 4), // "type"
QT_MOC_LITERAL(8, 96, 14), // "fileConutAsync"
QT_MOC_LITERAL(9, 111, 39), // "QSharedPointer<FileInfoHelper..."
QT_MOC_LITERAL(10, 151, 4), // "data"
QT_MOC_LITERAL(11, 156, 12), // "fileMimeType"
QT_MOC_LITERAL(12, 169, 24), // "QMimeDatabase::MatchMode"
QT_MOC_LITERAL(13, 194, 4), // "mode"
QT_MOC_LITERAL(14, 199, 4), // "inod"
QT_MOC_LITERAL(15, 204, 6), // "isGvfs"
QT_MOC_LITERAL(16, 211, 11), // "fileRefresh"
QT_MOC_LITERAL(17, 223, 32), // "QSharedPointer<dfmio::DFileInfo>"
QT_MOC_LITERAL(18, 256, 9) // "dfileInfo"

    },
    "dfmbase::FileInfoAsycWorker\0"
    "fileConutAsyncFinish\0\0url\0files\0"
    "fileMimeTypeFinished\0QMimeType\0type\0"
    "fileConutAsync\0QSharedPointer<FileInfoHelperUeserData>\0"
    "data\0fileMimeType\0QMimeDatabase::MatchMode\0"
    "mode\0inod\0isGvfs\0fileRefresh\0"
    "QSharedPointer<dfmio::DFileInfo>\0"
    "dfileInfo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__FileInfoAsycWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   39,    2, 0x06 /* Public */,
       5,    2,   44,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    2,   49,    2, 0x08 /* Private */,
      11,    5,   54,    2, 0x08 /* Private */,
      16,    2,   65,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl, QMetaType::Int,    3,    4,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 6,    3,    7,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 9,    3,   10,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 12, QMetaType::QString, QMetaType::Bool, 0x80000000 | 9,    3,   13,   14,   15,   10,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 17,    3,   18,

       0        // eod
};

void dfmbase::FileInfoAsycWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileInfoAsycWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->fileConutAsyncFinish((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 1: _t->fileMimeTypeFinished((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QMimeType(*)>(_a[2]))); break;
        case 2: _t->fileConutAsync((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QSharedPointer<FileInfoHelperUeserData>(*)>(_a[2]))); break;
        case 3: _t->fileMimeType((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QMimeDatabase::MatchMode(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const bool(*)>(_a[4])),(*reinterpret_cast< const QSharedPointer<FileInfoHelperUeserData>(*)>(_a[5]))); break;
        case 4: _t->fileRefresh((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QSharedPointer<dfmio::DFileInfo>(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileInfoAsycWorker::*)(const QUrl & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileInfoAsycWorker::fileConutAsyncFinish)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileInfoAsycWorker::*)(const QUrl & , const QMimeType & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileInfoAsycWorker::fileMimeTypeFinished)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::FileInfoAsycWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__FileInfoAsycWorker.data,
    qt_meta_data_dfmbase__FileInfoAsycWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::FileInfoAsycWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::FileInfoAsycWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__FileInfoAsycWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmbase::FileInfoAsycWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void dfmbase::FileInfoAsycWorker::fileConutAsyncFinish(const QUrl & _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmbase::FileInfoAsycWorker::fileMimeTypeFinished(const QUrl & _t1, const QMimeType & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
