// SPDX-FileCopyrightText: 2022 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef SERVICE_SHARECONTROL_GLOBAL_H
#define SERVICE_SHARECONTROL_GLOBAL_H

#include <dfm-base/dfm_log_defines.h>

#define SERVICESHARECONTROL_NAMESPACE service_sharecontrol

#define SERVICESHARECONTROL_BEGIN_NAMESPACE namespace SERVICESHARECONTROL_NAMESPACE {
#define SERVICESHARECONTROL_END_NAMESPACE }
#define SERVICESHARECONTROL_USE_NAMESPACE using namespace SERVICESHARECONTROL_NAMESPACE;

SERVICESHARECONTROL_BEGIN_NAMESPACE
DFM_LOG_USE_CATEGORY(SERVICESHARECONTROL_NAMESPACE)
SERVICESHARECONTROL_END_NAMESPACE

#endif   // SERVICE_SHARECONTROL_GLOBAL_H
