/****************************************************************************
** Meta object code from reading C++ file 'searcheditwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-titlebar/views/searcheditwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'searcheditwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_titlebar__SearchEditWidget_t {
    QByteArrayData data[23];
    char stringdata0[360];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_titlebar__SearchEditWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_titlebar__SearchEditWidget_t qt_meta_stringdata_dfmplugin_titlebar__SearchEditWidget = {
    {
QT_MOC_LITERAL(0, 0, 36), // "dfmplugin_titlebar::SearchEdi..."
QT_MOC_LITERAL(1, 37, 10), // "searchQuit"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 12), // "onUrlChanged"
QT_MOC_LITERAL(4, 62, 3), // "url"
QT_MOC_LITERAL(5, 66, 20), // "onPauseButtonClicked"
QT_MOC_LITERAL(6, 87, 23), // "onAdvancedButtonClicked"
QT_MOC_LITERAL(7, 111, 15), // "onReturnPressed"
QT_MOC_LITERAL(8, 127, 12), // "onTextEdited"
QT_MOC_LITERAL(9, 140, 4), // "text"
QT_MOC_LITERAL(10, 145, 20), // "onClearSearchHistory"
QT_MOC_LITERAL(11, 166, 5), // "winId"
QT_MOC_LITERAL(12, 172, 21), // "onDConfigValueChanged"
QT_MOC_LITERAL(13, 194, 6), // "config"
QT_MOC_LITERAL(14, 201, 3), // "key"
QT_MOC_LITERAL(15, 205, 16), // "insertCompletion"
QT_MOC_LITERAL(16, 222, 10), // "completion"
QT_MOC_LITERAL(17, 233, 23), // "onCompletionHighlighted"
QT_MOC_LITERAL(18, 257, 21), // "highlightedCompletion"
QT_MOC_LITERAL(19, 279, 29), // "onCompletionModelCountChanged"
QT_MOC_LITERAL(20, 309, 22), // "appendToCompleterModel"
QT_MOC_LITERAL(21, 332, 10), // "stringList"
QT_MOC_LITERAL(22, 343, 16) // "expandSearchEdit"

    },
    "dfmplugin_titlebar::SearchEditWidget\0"
    "searchQuit\0\0onUrlChanged\0url\0"
    "onPauseButtonClicked\0onAdvancedButtonClicked\0"
    "onReturnPressed\0onTextEdited\0text\0"
    "onClearSearchHistory\0winId\0"
    "onDConfigValueChanged\0config\0key\0"
    "insertCompletion\0completion\0"
    "onCompletionHighlighted\0highlightedCompletion\0"
    "onCompletionModelCountChanged\0"
    "appendToCompleterModel\0stringList\0"
    "expandSearchEdit"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_titlebar__SearchEditWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   79,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   80,    2, 0x0a /* Public */,
       5,    0,   83,    2, 0x0a /* Public */,
       6,    0,   84,    2, 0x0a /* Public */,
       7,    0,   85,    2, 0x0a /* Public */,
       8,    1,   86,    2, 0x0a /* Public */,
      10,    1,   89,    2, 0x0a /* Public */,
      12,    2,   92,    2, 0x0a /* Public */,
      15,    1,   97,    2, 0x0a /* Public */,
      17,    1,  100,    2, 0x0a /* Public */,
      19,    0,  103,    2, 0x0a /* Public */,
      20,    1,  104,    2, 0x0a /* Public */,
      22,    0,  107,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::ULongLong,   11,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   13,   14,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void, QMetaType::QString,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QStringList,   21,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_titlebar::SearchEditWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SearchEditWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->searchQuit(); break;
        case 1: _t->onUrlChanged((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 2: _t->onPauseButtonClicked(); break;
        case 3: _t->onAdvancedButtonClicked(); break;
        case 4: _t->onReturnPressed(); break;
        case 5: _t->onTextEdited((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->onClearSearchHistory((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 7: _t->onDConfigValueChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 8: _t->insertCompletion((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->onCompletionHighlighted((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->onCompletionModelCountChanged(); break;
        case 11: _t->appendToCompleterModel((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 12: _t->expandSearchEdit(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SearchEditWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SearchEditWidget::searchQuit)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_titlebar::SearchEditWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_titlebar__SearchEditWidget.data,
    qt_meta_data_dfmplugin_titlebar__SearchEditWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_titlebar::SearchEditWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_titlebar::SearchEditWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_titlebar__SearchEditWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int dfmplugin_titlebar::SearchEditWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_titlebar::SearchEditWidget::searchQuit()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
