/****************************************************************************
** Meta object code from reading C++ file 'thumbnailfactory.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../dde-file-manager/src/dfm-base/utils/thumbnail/thumbnailfactory.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'thumbnailfactory.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmbase__ThumbnailFactory_t {
    QByteArrayData data[16];
    char stringdata0[212];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__ThumbnailFactory_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__ThumbnailFactory_t qt_meta_stringdata_dfmbase__ThumbnailFactory = {
    {
QT_MOC_LITERAL(0, 0, 25), // "dfmbase::ThumbnailFactory"
QT_MOC_LITERAL(1, 26, 15), // "produceFinished"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 3), // "src"
QT_MOC_LITERAL(4, 47, 5), // "thumb"
QT_MOC_LITERAL(5, 53, 13), // "produceFailed"
QT_MOC_LITERAL(6, 67, 7), // "addTask"
QT_MOC_LITERAL(7, 75, 33), // "ThumbnailWorker::ThumbnailTas..."
QT_MOC_LITERAL(8, 109, 7), // "taskMap"
QT_MOC_LITERAL(9, 117, 12), // "thumbnailJob"
QT_MOC_LITERAL(10, 130, 3), // "url"
QT_MOC_LITERAL(11, 134, 30), // "dfmbase::Global::ThumbnailSize"
QT_MOC_LITERAL(12, 165, 4), // "size"
QT_MOC_LITERAL(13, 170, 13), // "onAboutToQuit"
QT_MOC_LITERAL(14, 184, 8), // "pushTask"
QT_MOC_LITERAL(15, 193, 18) // "doJoinThumbnailJob"

    },
    "dfmbase::ThumbnailFactory\0produceFinished\0"
    "\0src\0thumb\0produceFailed\0addTask\0"
    "ThumbnailWorker::ThumbnailTaskMap\0"
    "taskMap\0thumbnailJob\0url\0"
    "dfmbase::Global::ThumbnailSize\0size\0"
    "onAboutToQuit\0pushTask\0doJoinThumbnailJob"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__ThumbnailFactory[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   49,    2, 0x06 /* Public */,
       5,    1,   54,    2, 0x06 /* Public */,
       6,    1,   57,    2, 0x06 /* Public */,
       9,    2,   60,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    0,   65,    2, 0x08 /* Private */,
      14,    0,   66,    2, 0x08 /* Private */,
      15,    2,   67,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl, QMetaType::QString,    3,    4,
    QMetaType::Void, QMetaType::QUrl,    3,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 11,   10,   12,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 11,   10,   12,

       0        // eod
};

void dfmbase::ThumbnailFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ThumbnailFactory *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->produceFinished((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->produceFailed((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 2: _t->addTask((*reinterpret_cast< const ThumbnailWorker::ThumbnailTaskMap(*)>(_a[1]))); break;
        case 3: _t->thumbnailJob((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< dfmbase::Global::ThumbnailSize(*)>(_a[2]))); break;
        case 4: _t->onAboutToQuit(); break;
        case 5: _t->pushTask(); break;
        case 6: _t->doJoinThumbnailJob((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< dfmbase::Global::ThumbnailSize(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::Global::ThumbnailSize >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::Global::ThumbnailSize >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ThumbnailFactory::*)(const QUrl & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ThumbnailFactory::produceFinished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ThumbnailFactory::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ThumbnailFactory::produceFailed)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ThumbnailFactory::*)(const ThumbnailWorker::ThumbnailTaskMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ThumbnailFactory::addTask)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ThumbnailFactory::*)(const QUrl & , dfmbase::Global::ThumbnailSize );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ThumbnailFactory::thumbnailJob)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::ThumbnailFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__ThumbnailFactory.data,
    qt_meta_data_dfmbase__ThumbnailFactory,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::ThumbnailFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::ThumbnailFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__ThumbnailFactory.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmbase::ThumbnailFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void dfmbase::ThumbnailFactory::produceFinished(const QUrl & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmbase::ThumbnailFactory::produceFailed(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmbase::ThumbnailFactory::addTask(const ThumbnailWorker::ThumbnailTaskMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmbase::ThumbnailFactory::thumbnailJob(const QUrl & _t1, dfmbase::Global::ThumbnailSize _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
