// SPDX-FileCopyrightText: 2021 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include <dfm-base/interfaces/screen/abstractscreen.h>

namespace dfmbase {

AbstractScreen::AbstractScreen(QObject *parent) : QObject(parent)
{

}

}




