/****************************************************************************
** Meta object code from reading C++ file 'computeritemwatcher.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-computer/watcher/computeritemwatcher.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'computeritemwatcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_computer__ComputerItemWatcher_t {
    QByteArrayData data[50];
    char stringdata0[747];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_computer__ComputerItemWatcher_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_computer__ComputerItemWatcher_t qt_meta_stringdata_dfmplugin_computer__ComputerItemWatcher = {
    {
QT_MOC_LITERAL(0, 0, 39), // "dfmplugin_computer::ComputerI..."
QT_MOC_LITERAL(1, 40, 17), // "itemQueryFinished"
QT_MOC_LITERAL(2, 58, 0), // ""
QT_MOC_LITERAL(3, 59, 16), // "ComputerDataList"
QT_MOC_LITERAL(4, 76, 7), // "results"
QT_MOC_LITERAL(5, 84, 9), // "itemAdded"
QT_MOC_LITERAL(6, 94, 16), // "ComputerItemData"
QT_MOC_LITERAL(7, 111, 4), // "item"
QT_MOC_LITERAL(8, 116, 11), // "itemRemoved"
QT_MOC_LITERAL(9, 128, 3), // "url"
QT_MOC_LITERAL(10, 132, 11), // "itemUpdated"
QT_MOC_LITERAL(11, 144, 19), // "itemPropertyChanged"
QT_MOC_LITERAL(12, 164, 8), // "property"
QT_MOC_LITERAL(13, 173, 3), // "var"
QT_MOC_LITERAL(14, 177, 15), // "itemSizeChanged"
QT_MOC_LITERAL(15, 193, 17), // "hideFileSystemTag"
QT_MOC_LITERAL(16, 211, 4), // "hide"
QT_MOC_LITERAL(17, 216, 24), // "updatePartitionsVisiable"
QT_MOC_LITERAL(18, 241, 13), // "onViewRefresh"
QT_MOC_LITERAL(19, 255, 13), // "onDeviceAdded"
QT_MOC_LITERAL(20, 269, 6), // "devUrl"
QT_MOC_LITERAL(21, 276, 7), // "groupId"
QT_MOC_LITERAL(22, 284, 27), // "ComputerItemData::ShapeType"
QT_MOC_LITERAL(23, 312, 5), // "shape"
QT_MOC_LITERAL(24, 318, 15), // "needSidebarItem"
QT_MOC_LITERAL(25, 334, 27), // "onDevicePropertyChangedQVar"
QT_MOC_LITERAL(26, 362, 2), // "id"
QT_MOC_LITERAL(27, 365, 12), // "propertyName"
QT_MOC_LITERAL(28, 378, 31), // "onDevicePropertyChangedQDBusVar"
QT_MOC_LITERAL(29, 410, 12), // "QDBusVariant"
QT_MOC_LITERAL(30, 423, 21), // "onGenAttributeChanged"
QT_MOC_LITERAL(31, 445, 29), // "Application::GenericAttribute"
QT_MOC_LITERAL(32, 475, 2), // "ga"
QT_MOC_LITERAL(33, 478, 5), // "value"
QT_MOC_LITERAL(34, 484, 16), // "onDConfigChanged"
QT_MOC_LITERAL(35, 501, 3), // "cfg"
QT_MOC_LITERAL(36, 505, 6), // "cfgKey"
QT_MOC_LITERAL(37, 512, 18), // "onBlockDeviceAdded"
QT_MOC_LITERAL(38, 531, 20), // "onBlockDeviceRemoved"
QT_MOC_LITERAL(39, 552, 20), // "onBlockDeviceMounted"
QT_MOC_LITERAL(40, 573, 7), // "mntPath"
QT_MOC_LITERAL(41, 581, 22), // "onBlockDeviceUnmounted"
QT_MOC_LITERAL(42, 604, 19), // "onBlockDeviceLocked"
QT_MOC_LITERAL(43, 624, 17), // "onUpdateBlockItem"
QT_MOC_LITERAL(44, 642, 23), // "onProtocolDeviceMounted"
QT_MOC_LITERAL(45, 666, 25), // "onProtocolDeviceUnmounted"
QT_MOC_LITERAL(46, 692, 19), // "onDeviceSizeChanged"
QT_MOC_LITERAL(47, 712, 5), // "total"
QT_MOC_LITERAL(48, 718, 4), // "free"
QT_MOC_LITERAL(49, 723, 23) // "onProtocolDeviceRemoved"

    },
    "dfmplugin_computer::ComputerItemWatcher\0"
    "itemQueryFinished\0\0ComputerDataList\0"
    "results\0itemAdded\0ComputerItemData\0"
    "item\0itemRemoved\0url\0itemUpdated\0"
    "itemPropertyChanged\0property\0var\0"
    "itemSizeChanged\0hideFileSystemTag\0"
    "hide\0updatePartitionsVisiable\0"
    "onViewRefresh\0onDeviceAdded\0devUrl\0"
    "groupId\0ComputerItemData::ShapeType\0"
    "shape\0needSidebarItem\0onDevicePropertyChangedQVar\0"
    "id\0propertyName\0onDevicePropertyChangedQDBusVar\0"
    "QDBusVariant\0onGenAttributeChanged\0"
    "Application::GenericAttribute\0ga\0value\0"
    "onDConfigChanged\0cfg\0cfgKey\0"
    "onBlockDeviceAdded\0onBlockDeviceRemoved\0"
    "onBlockDeviceMounted\0mntPath\0"
    "onBlockDeviceUnmounted\0onBlockDeviceLocked\0"
    "onUpdateBlockItem\0onProtocolDeviceMounted\0"
    "onProtocolDeviceUnmounted\0onDeviceSizeChanged\0"
    "total\0free\0onProtocolDeviceRemoved"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_computer__ComputerItemWatcher[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      26,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  144,    2, 0x06 /* Public */,
       5,    1,  147,    2, 0x06 /* Public */,
       8,    1,  150,    2, 0x06 /* Public */,
      10,    1,  153,    2, 0x06 /* Public */,
      11,    3,  156,    2, 0x06 /* Public */,
      14,    3,  163,    2, 0x06 /* Public */,
      15,    1,  170,    2, 0x06 /* Public */,
      17,    0,  173,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    0,  174,    2, 0x0a /* Public */,
      19,    4,  175,    2, 0x09 /* Protected */,
      19,    3,  184,    2, 0x29 /* Protected | MethodCloned */,
      19,    2,  191,    2, 0x29 /* Protected | MethodCloned */,
      25,    3,  196,    2, 0x09 /* Protected */,
      28,    3,  203,    2, 0x09 /* Protected */,
      30,    2,  210,    2, 0x09 /* Protected */,
      34,    2,  215,    2, 0x09 /* Protected */,
      37,    1,  220,    2, 0x09 /* Protected */,
      38,    1,  223,    2, 0x09 /* Protected */,
      39,    2,  226,    2, 0x09 /* Protected */,
      41,    1,  231,    2, 0x09 /* Protected */,
      42,    1,  234,    2, 0x09 /* Protected */,
      43,    1,  237,    2, 0x09 /* Protected */,
      44,    2,  240,    2, 0x09 /* Protected */,
      45,    1,  245,    2, 0x09 /* Protected */,
      46,    3,  248,    2, 0x09 /* Protected */,
      49,    1,  255,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::QUrl,    9,
    QMetaType::Void, QMetaType::QUrl,    9,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QString, QMetaType::QVariant,    9,   12,   13,
    QMetaType::Void, QMetaType::QUrl, QMetaType::LongLong, QMetaType::LongLong,    9,    2,    2,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl, QMetaType::Int, 0x80000000 | 22, QMetaType::Bool,   20,   21,   23,   24,
    QMetaType::Void, QMetaType::QUrl, QMetaType::Int, 0x80000000 | 22,   20,   21,   23,
    QMetaType::Void, QMetaType::QUrl, QMetaType::Int,   20,   21,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QVariant,   26,   27,   13,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, 0x80000000 | 29,   26,   27,   13,
    QMetaType::Void, 0x80000000 | 31, QMetaType::QVariant,   32,   33,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   35,   36,
    QMetaType::Void, QMetaType::QString,   26,
    QMetaType::Void, QMetaType::QString,   26,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   26,   40,
    QMetaType::Void, QMetaType::QString,   26,
    QMetaType::Void, QMetaType::QString,   26,
    QMetaType::Void, QMetaType::QString,   26,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   26,   40,
    QMetaType::Void, QMetaType::QString,   26,
    QMetaType::Void, QMetaType::QString, QMetaType::LongLong, QMetaType::LongLong,   26,   47,   48,
    QMetaType::Void, QMetaType::QString,   26,

       0        // eod
};

void dfmplugin_computer::ComputerItemWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ComputerItemWatcher *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->itemQueryFinished((*reinterpret_cast< const ComputerDataList(*)>(_a[1]))); break;
        case 1: _t->itemAdded((*reinterpret_cast< const ComputerItemData(*)>(_a[1]))); break;
        case 2: _t->itemRemoved((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 3: _t->itemUpdated((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 4: _t->itemPropertyChanged((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3]))); break;
        case 5: _t->itemSizeChanged((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< qlonglong(*)>(_a[2])),(*reinterpret_cast< qlonglong(*)>(_a[3]))); break;
        case 6: _t->hideFileSystemTag((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->updatePartitionsVisiable(); break;
        case 8: _t->onViewRefresh(); break;
        case 9: _t->onDeviceAdded((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< ComputerItemData::ShapeType(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4]))); break;
        case 10: _t->onDeviceAdded((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< ComputerItemData::ShapeType(*)>(_a[3]))); break;
        case 11: _t->onDeviceAdded((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 12: _t->onDevicePropertyChangedQVar((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3]))); break;
        case 13: _t->onDevicePropertyChangedQDBusVar((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QDBusVariant(*)>(_a[3]))); break;
        case 14: _t->onGenAttributeChanged((*reinterpret_cast< Application::GenericAttribute(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 15: _t->onDConfigChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 16: _t->onBlockDeviceAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->onBlockDeviceRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 18: _t->onBlockDeviceMounted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 19: _t->onBlockDeviceUnmounted((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 20: _t->onBlockDeviceLocked((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 21: _t->onUpdateBlockItem((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 22: _t->onProtocolDeviceMounted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 23: _t->onProtocolDeviceUnmounted((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 24: _t->onDeviceSizeChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qlonglong(*)>(_a[2])),(*reinterpret_cast< qlonglong(*)>(_a[3]))); break;
        case 25: _t->onProtocolDeviceRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusVariant >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ComputerItemWatcher::*)(const ComputerDataList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComputerItemWatcher::itemQueryFinished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ComputerItemWatcher::*)(const ComputerItemData & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComputerItemWatcher::itemAdded)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ComputerItemWatcher::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComputerItemWatcher::itemRemoved)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ComputerItemWatcher::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComputerItemWatcher::itemUpdated)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ComputerItemWatcher::*)(const QUrl & , const QString & , const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComputerItemWatcher::itemPropertyChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ComputerItemWatcher::*)(const QUrl & , qlonglong , qlonglong );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComputerItemWatcher::itemSizeChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (ComputerItemWatcher::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComputerItemWatcher::hideFileSystemTag)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (ComputerItemWatcher::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComputerItemWatcher::updatePartitionsVisiable)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_computer::ComputerItemWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_computer__ComputerItemWatcher.data,
    qt_meta_data_dfmplugin_computer__ComputerItemWatcher,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_computer::ComputerItemWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_computer::ComputerItemWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_computer__ComputerItemWatcher.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_computer::ComputerItemWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_computer::ComputerItemWatcher::itemQueryFinished(const ComputerDataList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_computer::ComputerItemWatcher::itemAdded(const ComputerItemData & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_computer::ComputerItemWatcher::itemRemoved(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_computer::ComputerItemWatcher::itemUpdated(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_computer::ComputerItemWatcher::itemPropertyChanged(const QUrl & _t1, const QString & _t2, const QVariant & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmplugin_computer::ComputerItemWatcher::itemSizeChanged(const QUrl & _t1, qlonglong _t2, qlonglong _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void dfmplugin_computer::ComputerItemWatcher::hideFileSystemTag(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void dfmplugin_computer::ComputerItemWatcher::updatePartitionsVisiable()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
