/****************************************************************************
** Meta object code from reading C++ file 'computereventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-computer/events/computereventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'computereventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_computer__ComputerEventReceiver_t {
    QByteArrayData data[21];
    char stringdata0[268];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_computer__ComputerEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_computer__ComputerEventReceiver_t qt_meta_stringdata_dfmplugin_computer__ComputerEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 41), // "dfmplugin_computer::ComputerE..."
QT_MOC_LITERAL(1, 42, 15), // "handleItemEject"
QT_MOC_LITERAL(2, 58, 0), // ""
QT_MOC_LITERAL(3, 59, 3), // "url"
QT_MOC_LITERAL(4, 63, 25), // "handleSepateTitlebarCrumb"
QT_MOC_LITERAL(5, 89, 19), // "QList<QVariantMap>*"
QT_MOC_LITERAL(6, 109, 8), // "mapGroup"
QT_MOC_LITERAL(7, 118, 14), // "handleSortItem"
QT_MOC_LITERAL(8, 133, 5), // "group"
QT_MOC_LITERAL(9, 139, 8), // "subGroup"
QT_MOC_LITERAL(10, 148, 1), // "a"
QT_MOC_LITERAL(11, 150, 1), // "b"
QT_MOC_LITERAL(12, 152, 16), // "handleSetTabName"
QT_MOC_LITERAL(13, 169, 8), // "QString*"
QT_MOC_LITERAL(14, 178, 7), // "tabName"
QT_MOC_LITERAL(15, 186, 20), // "setContextMenuEnable"
QT_MOC_LITERAL(16, 207, 6), // "enable"
QT_MOC_LITERAL(17, 214, 19), // "dirAccessPrehandler"
QT_MOC_LITERAL(18, 234, 5), // "winId"
QT_MOC_LITERAL(19, 240, 21), // "std::function<void()>"
QT_MOC_LITERAL(20, 262, 5) // "after"

    },
    "dfmplugin_computer::ComputerEventReceiver\0"
    "handleItemEject\0\0url\0handleSepateTitlebarCrumb\0"
    "QList<QVariantMap>*\0mapGroup\0"
    "handleSortItem\0group\0subGroup\0a\0b\0"
    "handleSetTabName\0QString*\0tabName\0"
    "setContextMenuEnable\0enable\0"
    "dirAccessPrehandler\0winId\0"
    "std::function<void()>\0after"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_computer__ComputerEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x0a /* Public */,
       4,    2,   47,    2, 0x0a /* Public */,
       7,    4,   52,    2, 0x0a /* Public */,
      12,    2,   61,    2, 0x0a /* Public */,
      15,    1,   66,    2, 0x0a /* Public */,
      17,    3,   69,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl,    3,
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 5,    3,    6,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::QUrl, QMetaType::QUrl,    8,    9,   10,   11,
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 13,    3,   14,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl, 0x80000000 | 19,   18,    3,   20,

       0        // eod
};

void dfmplugin_computer::ComputerEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ComputerEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleItemEject((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 1: { bool _r = _t->handleSepateTitlebarCrumb((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QList<QVariantMap>*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->handleSortItem((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const QUrl(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->handleSetTabName((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QString*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->setContextMenuEnable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->dirAccessPrehandler((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< std::function<void()>(*)>(_a[3]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_computer::ComputerEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_computer__ComputerEventReceiver.data,
    qt_meta_data_dfmplugin_computer__ComputerEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_computer::ComputerEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_computer::ComputerEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_computer__ComputerEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_computer::ComputerEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
