/****************************************************************************
** Meta object code from reading C++ file 'fileoperationseventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-fileoperations/fileoperationsevent/fileoperationseventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileoperationseventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_fileoperations__FileOperationsEventReceiver_t {
    QByteArrayData data[78];
    char stringdata0[1413];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_fileoperations__FileOperationsEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_fileoperations__FileOperationsEventReceiver_t qt_meta_stringdata_dfmplugin_fileoperations__FileOperationsEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 53), // "dfmplugin_fileoperations::Fil..."
QT_MOC_LITERAL(1, 54, 19), // "handleOperationCopy"
QT_MOC_LITERAL(2, 74, 0), // ""
QT_MOC_LITERAL(3, 75, 8), // "windowId"
QT_MOC_LITERAL(4, 84, 11), // "QList<QUrl>"
QT_MOC_LITERAL(5, 96, 7), // "sources"
QT_MOC_LITERAL(6, 104, 6), // "target"
QT_MOC_LITERAL(7, 111, 36), // "dfmbase::AbstractJobHandler::..."
QT_MOC_LITERAL(8, 148, 5), // "flags"
QT_MOC_LITERAL(9, 154, 51), // "dfmbase::AbstractJobHandler::..."
QT_MOC_LITERAL(10, 206, 6), // "handle"
QT_MOC_LITERAL(11, 213, 6), // "custom"
QT_MOC_LITERAL(12, 220, 45), // "dfmbase::AbstractJobHandler::..."
QT_MOC_LITERAL(13, 266, 8), // "callback"
QT_MOC_LITERAL(14, 275, 18), // "handleOperationCut"
QT_MOC_LITERAL(15, 294, 22), // "handleOperationDeletes"
QT_MOC_LITERAL(16, 317, 24), // "handleOperationOpenFiles"
QT_MOC_LITERAL(17, 342, 4), // "urls"
QT_MOC_LITERAL(18, 347, 5), // "bool*"
QT_MOC_LITERAL(19, 353, 2), // "ok"
QT_MOC_LITERAL(20, 356, 29), // "handleOperationOpenFilesByApp"
QT_MOC_LITERAL(21, 386, 14), // "QList<QString>"
QT_MOC_LITERAL(22, 401, 4), // "apps"
QT_MOC_LITERAL(23, 406, 25), // "handleOperationRenameFile"
QT_MOC_LITERAL(24, 432, 6), // "oldUrl"
QT_MOC_LITERAL(25, 439, 6), // "newUrl"
QT_MOC_LITERAL(26, 446, 26), // "handleOperationRenameFiles"
QT_MOC_LITERAL(27, 473, 22), // "QPair<QString,QString>"
QT_MOC_LITERAL(28, 496, 4), // "pair"
QT_MOC_LITERAL(29, 501, 7), // "replace"
QT_MOC_LITERAL(30, 509, 59), // "QPair<QString,dfmbase::Abstra..."
QT_MOC_LITERAL(31, 569, 20), // "handleOperationMkdir"
QT_MOC_LITERAL(32, 590, 3), // "url"
QT_MOC_LITERAL(33, 594, 24), // "handleOperationTouchFile"
QT_MOC_LITERAL(34, 619, 31), // "dfmbase::Global::CreateFileType"
QT_MOC_LITERAL(35, 651, 8), // "fileType"
QT_MOC_LITERAL(36, 660, 6), // "suffix"
QT_MOC_LITERAL(37, 667, 7), // "tempUrl"
QT_MOC_LITERAL(38, 675, 19), // "callbackImmediately"
QT_MOC_LITERAL(39, 695, 23), // "handleOperationLinkFile"
QT_MOC_LITERAL(40, 719, 4), // "link"
QT_MOC_LITERAL(41, 724, 5), // "force"
QT_MOC_LITERAL(42, 730, 7), // "silence"
QT_MOC_LITERAL(43, 738, 28), // "handleOperationSetPermission"
QT_MOC_LITERAL(44, 767, 24), // "QFileDevice::Permissions"
QT_MOC_LITERAL(45, 792, 11), // "permissions"
QT_MOC_LITERAL(46, 804, 8), // "callBack"
QT_MOC_LITERAL(47, 813, 31), // "handleOperationWriteToClipboard"
QT_MOC_LITERAL(48, 845, 35), // "dfmbase::ClipBoard::Clipboard..."
QT_MOC_LITERAL(49, 881, 6), // "action"
QT_MOC_LITERAL(50, 888, 35), // "handleOperationWriteDataToCli..."
QT_MOC_LITERAL(51, 924, 10), // "QMimeData*"
QT_MOC_LITERAL(52, 935, 4), // "data"
QT_MOC_LITERAL(53, 940, 29), // "handleOperationOpenInTerminal"
QT_MOC_LITERAL(54, 970, 29), // "handleOperationSaveOperations"
QT_MOC_LITERAL(55, 1000, 6), // "values"
QT_MOC_LITERAL(56, 1007, 39), // "handleOperationCleanSaveOpera..."
QT_MOC_LITERAL(57, 1047, 25), // "handleOperationRevocation"
QT_MOC_LITERAL(58, 1073, 24), // "handleOperationHideFiles"
QT_MOC_LITERAL(59, 1098, 14), // "handleShortCut"
QT_MOC_LITERAL(60, 1113, 7), // "rootUrl"
QT_MOC_LITERAL(61, 1121, 19), // "handleShortCutPaste"
QT_MOC_LITERAL(62, 1141, 33), // "handleOperationSaveRedoOperat..."
QT_MOC_LITERAL(63, 1175, 26), // "handleOperationCleanByUrls"
QT_MOC_LITERAL(64, 1202, 35), // "handleRecoveryOperationRedoRe..."
QT_MOC_LITERAL(65, 1238, 17), // "handleSaveRedoOpt"
QT_MOC_LITERAL(66, 1256, 5), // "token"
QT_MOC_LITERAL(67, 1262, 8), // "fileSize"
QT_MOC_LITERAL(68, 1271, 26), // "handleOperationUndoDeletes"
QT_MOC_LITERAL(69, 1298, 14), // "handleCallback"
QT_MOC_LITERAL(70, 1313, 2), // "op"
QT_MOC_LITERAL(71, 1316, 22), // "handleOperationUndoCut"
QT_MOC_LITERAL(72, 1339, 27), // "handleOperationFilesPreview"
QT_MOC_LITERAL(73, 1367, 10), // "selectUrls"
QT_MOC_LITERAL(74, 1378, 7), // "dirUrls"
QT_MOC_LITERAL(75, 1386, 15), // "handleIsSubFile"
QT_MOC_LITERAL(76, 1402, 6), // "parent"
QT_MOC_LITERAL(77, 1409, 3) // "sub"

    },
    "dfmplugin_fileoperations::FileOperationsEventReceiver\0"
    "handleOperationCopy\0\0windowId\0QList<QUrl>\0"
    "sources\0target\0dfmbase::AbstractJobHandler::JobFlag\0"
    "flags\0dfmbase::AbstractJobHandler::OperatorHandleCallback\0"
    "handle\0custom\0"
    "dfmbase::AbstractJobHandler::OperatorCallback\0"
    "callback\0handleOperationCut\0"
    "handleOperationDeletes\0handleOperationOpenFiles\0"
    "urls\0bool*\0ok\0handleOperationOpenFilesByApp\0"
    "QList<QString>\0apps\0handleOperationRenameFile\0"
    "oldUrl\0newUrl\0handleOperationRenameFiles\0"
    "QPair<QString,QString>\0pair\0replace\0"
    "QPair<QString,dfmbase::AbstractJobHandler::FileNameAddFlag>\0"
    "handleOperationMkdir\0url\0"
    "handleOperationTouchFile\0"
    "dfmbase::Global::CreateFileType\0"
    "fileType\0suffix\0tempUrl\0callbackImmediately\0"
    "handleOperationLinkFile\0link\0force\0"
    "silence\0handleOperationSetPermission\0"
    "QFileDevice::Permissions\0permissions\0"
    "callBack\0handleOperationWriteToClipboard\0"
    "dfmbase::ClipBoard::ClipboardAction\0"
    "action\0handleOperationWriteDataToClipboard\0"
    "QMimeData*\0data\0handleOperationOpenInTerminal\0"
    "handleOperationSaveOperations\0values\0"
    "handleOperationCleanSaveOperationsStack\0"
    "handleOperationRevocation\0"
    "handleOperationHideFiles\0handleShortCut\0"
    "rootUrl\0handleShortCutPaste\0"
    "handleOperationSaveRedoOperations\0"
    "handleOperationCleanByUrls\0"
    "handleRecoveryOperationRedoRecovery\0"
    "handleSaveRedoOpt\0token\0fileSize\0"
    "handleOperationUndoDeletes\0handleCallback\0"
    "op\0handleOperationUndoCut\0"
    "handleOperationFilesPreview\0selectUrls\0"
    "dirUrls\0handleIsSubFile\0parent\0sub"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_fileoperations__FileOperationsEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      45,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    5,  239,    2, 0x0a /* Public */,
       1,    7,  250,    2, 0x0a /* Public */,
      14,    5,  265,    2, 0x0a /* Public */,
      14,    7,  276,    2, 0x0a /* Public */,
      15,    4,  291,    2, 0x0a /* Public */,
      15,    6,  300,    2, 0x0a /* Public */,
      16,    2,  313,    2, 0x0a /* Public */,
      16,    3,  318,    2, 0x0a /* Public */,
      16,    4,  325,    2, 0x0a /* Public */,
      20,    3,  334,    2, 0x0a /* Public */,
      20,    5,  341,    2, 0x0a /* Public */,
      23,    4,  352,    2, 0x0a /* Public */,
      23,    6,  361,    2, 0x0a /* Public */,
      26,    4,  374,    2, 0x0a /* Public */,
      26,    6,  383,    2, 0x0a /* Public */,
      26,    3,  396,    2, 0x0a /* Public */,
      26,    5,  403,    2, 0x0a /* Public */,
      31,    2,  414,    2, 0x0a /* Public */,
      31,    4,  419,    2, 0x0a /* Public */,
      33,    4,  428,    2, 0x0a /* Public */,
      33,    4,  437,    2, 0x0a /* Public */,
      33,    6,  446,    2, 0x0a /* Public */,
      33,    6,  459,    2, 0x0a /* Public */,
      39,    5,  472,    2, 0x0a /* Public */,
      39,    7,  483,    2, 0x0a /* Public */,
      43,    3,  498,    2, 0x0a /* Public */,
      43,    5,  505,    2, 0x0a /* Public */,
      47,    3,  516,    2, 0x0a /* Public */,
      50,    2,  523,    2, 0x0a /* Public */,
      53,    2,  528,    2, 0x0a /* Public */,
      54,    1,  533,    2, 0x0a /* Public */,
      56,    0,  536,    2, 0x0a /* Public */,
      57,    2,  537,    2, 0x0a /* Public */,
      58,    2,  542,    2, 0x0a /* Public */,
      58,    4,  547,    2, 0x0a /* Public */,
      59,    3,  556,    2, 0x0a /* Public */,
      61,    3,  563,    2, 0x0a /* Public */,
      62,    1,  570,    2, 0x0a /* Public */,
      63,    1,  573,    2, 0x0a /* Public */,
      64,    2,  576,    2, 0x0a /* Public */,
      65,    2,  581,    2, 0x0a /* Public */,
      68,    5,  586,    2, 0x0a /* Public */,
      71,    6,  597,    2, 0x0a /* Public */,
      72,    3,  610,    2, 0x0a /* Public */,
      75,    2,  617,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QUrl, 0x80000000 | 7, 0x80000000 | 9,    3,    5,    6,    8,   10,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QUrl, 0x80000000 | 7, 0x80000000 | 9, QMetaType::QVariant, 0x80000000 | 12,    3,    5,    6,    8,   10,   11,   13,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QUrl, 0x80000000 | 7, 0x80000000 | 9,    3,    5,    6,    8,   10,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QUrl, 0x80000000 | 7, 0x80000000 | 9, QMetaType::QVariant, 0x80000000 | 12,    3,    5,    6,    8,   10,   11,   13,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 7, 0x80000000 | 9,    3,    5,    8,   10,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 7, 0x80000000 | 9, QMetaType::QVariant, 0x80000000 | 12,    3,    5,    8,   10,   11,   13,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 4,    3,   17,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 18,    3,   17,   19,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QVariant, 0x80000000 | 12,    3,   17,   11,   13,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 21,    3,   17,   22,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 21, QMetaType::QVariant, 0x80000000 | 12,    3,   17,   22,   11,   13,
    QMetaType::Bool, QMetaType::ULongLong, QMetaType::QUrl, QMetaType::QUrl, 0x80000000 | 7,    3,   24,   25,    8,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl, QMetaType::QUrl, 0x80000000 | 7, QMetaType::QVariant, 0x80000000 | 12,    3,   24,   25,    8,   11,   13,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 27, QMetaType::Bool,    3,   17,   28,   29,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 27, QMetaType::Bool, QMetaType::QVariant, 0x80000000 | 12,    3,   17,   28,   29,   11,   13,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 30,    3,   17,   28,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 30, QMetaType::QVariant, 0x80000000 | 12,    3,   17,   28,   11,   13,
    QMetaType::Bool, QMetaType::ULongLong, QMetaType::QUrl,    3,   32,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl, QMetaType::QVariant, 0x80000000 | 12,    3,   32,   11,   13,
    QMetaType::QString, QMetaType::ULongLong, QMetaType::QUrl, 0x80000000 | 34, QMetaType::QString,    3,   32,   35,   36,
    QMetaType::QString, QMetaType::ULongLong, QMetaType::QUrl, QMetaType::QUrl, QMetaType::QString,    3,   32,   37,   36,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl, 0x80000000 | 34, QMetaType::QString, QMetaType::QVariant, 0x80000000 | 12,    3,   32,   35,   36,   11,   38,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl, QMetaType::QUrl, QMetaType::QString, QMetaType::QVariant, 0x80000000 | 12,    3,   32,   37,   36,   11,   38,
    QMetaType::Bool, QMetaType::ULongLong, QMetaType::QUrl, QMetaType::QUrl, QMetaType::Bool, QMetaType::Bool,    3,   32,   40,   41,   42,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl, QMetaType::QUrl, QMetaType::Bool, QMetaType::Bool, QMetaType::QVariant, 0x80000000 | 12,    3,   32,   40,   41,   42,   11,   13,
    QMetaType::Bool, QMetaType::ULongLong, QMetaType::QUrl, 0x80000000 | 44,    3,   32,   45,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl, 0x80000000 | 44, QMetaType::QVariant, 0x80000000 | 12,    3,   32,   45,   11,   46,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 48, 0x80000000 | 4,    3,   49,   17,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 51,    3,   52,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 4,    3,   17,
    QMetaType::Void, QMetaType::QVariantMap,   55,
    QMetaType::Void,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 9,    3,   10,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 4,    3,   17,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QVariant, 0x80000000 | 12,    3,   17,   11,   13,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QUrl,    2,   17,   60,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QUrl,    2,    2,    6,
    QMetaType::Void, QMetaType::QVariantMap,   55,
    QMetaType::Void, 0x80000000 | 4,   17,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 9,    3,   10,
    QMetaType::Void, QMetaType::QString, QMetaType::LongLong,   66,   67,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 7, 0x80000000 | 9, QMetaType::QVariantMap,    3,    5,    8,   69,   70,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QUrl, 0x80000000 | 7, 0x80000000 | 9, QMetaType::QVariantMap,    3,    5,    6,    8,   69,   70,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 4,    3,   73,   74,
    QMetaType::Bool, QMetaType::QUrl, QMetaType::QUrl,   76,   77,

       0        // eod
};

void dfmplugin_fileoperations::FileOperationsEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileOperationsEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleOperationCopy((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[5]))); break;
        case 1: _t->handleOperationCopy((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[5])),(*reinterpret_cast< const QVariant(*)>(_a[6])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[7]))); break;
        case 2: _t->handleOperationCut((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[5]))); break;
        case 3: _t->handleOperationCut((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[5])),(*reinterpret_cast< const QVariant(*)>(_a[6])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[7]))); break;
        case 4: _t->handleOperationDeletes((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[3])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[4]))); break;
        case 5: _t->handleOperationDeletes((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[3])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[4])),(*reinterpret_cast< const QVariant(*)>(_a[5])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[6]))); break;
        case 6: { bool _r = _t->handleOperationOpenFiles((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->handleOperationOpenFiles((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< bool*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->handleOperationOpenFiles((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[4]))); break;
        case 9: { bool _r = _t->handleOperationOpenFilesByApp((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QList<QString>(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->handleOperationOpenFilesByApp((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QList<QString>(*)>(_a[3])),(*reinterpret_cast< const QVariant(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[5]))); break;
        case 11: { bool _r = _t->handleOperationRenameFile((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->handleOperationRenameFile((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[4])),(*reinterpret_cast< const QVariant(*)>(_a[5])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[6]))); break;
        case 13: { bool _r = _t->handleOperationRenameFiles((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QPair<QString,QString>(*)>(_a[3])),(*reinterpret_cast< const bool(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->handleOperationRenameFiles((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QPair<QString,QString>(*)>(_a[3])),(*reinterpret_cast< const bool(*)>(_a[4])),(*reinterpret_cast< const QVariant(*)>(_a[5])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[6]))); break;
        case 15: { bool _r = _t->handleOperationRenameFiles((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QPair<QString,dfmbase::AbstractJobHandler::FileNameAddFlag>(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->handleOperationRenameFiles((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QPair<QString,dfmbase::AbstractJobHandler::FileNameAddFlag>(*)>(_a[3])),(*reinterpret_cast< const QVariant(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[5]))); break;
        case 17: { bool _r = _t->handleOperationMkdir((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->handleOperationMkdir((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[4]))); break;
        case 19: { QString _r = _t->handleOperationTouchFile((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const dfmbase::Global::CreateFileType(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 20: { QString _r = _t->handleOperationTouchFile((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->handleOperationTouchFile((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const dfmbase::Global::CreateFileType(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< const QVariant(*)>(_a[5])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[6]))); break;
        case 22: _t->handleOperationTouchFile((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< const QVariant(*)>(_a[5])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[6]))); break;
        case 23: { bool _r = _t->handleOperationLinkFile((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const bool(*)>(_a[4])),(*reinterpret_cast< const bool(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->handleOperationLinkFile((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const bool(*)>(_a[4])),(*reinterpret_cast< const bool(*)>(_a[5])),(*reinterpret_cast< const QVariant(*)>(_a[6])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[7]))); break;
        case 25: { bool _r = _t->handleOperationSetPermission((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QFileDevice::Permissions(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->handleOperationSetPermission((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QFileDevice::Permissions(*)>(_a[3])),(*reinterpret_cast< const QVariant(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[5]))); break;
        case 27: { bool _r = _t->handleOperationWriteToClipboard((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const dfmbase::ClipBoard::ClipboardAction(*)>(_a[2])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->handleOperationWriteDataToClipboard((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< QMimeData*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 29: { bool _r = _t->handleOperationOpenInTerminal((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->handleOperationSaveOperations((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 31: _t->handleOperationCleanSaveOperationsStack(); break;
        case 32: _t->handleOperationRevocation((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[2]))); break;
        case 33: { bool _r = _t->handleOperationHideFiles((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 34: _t->handleOperationHideFiles((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[4]))); break;
        case 35: { bool _r = _t->handleShortCut((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 36: { bool _r = _t->handleShortCutPaste((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 37: _t->handleOperationSaveRedoOperations((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 38: _t->handleOperationCleanByUrls((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 39: _t->handleRecoveryOperationRedoRecovery((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[2]))); break;
        case 40: _t->handleSaveRedoOpt((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const qint64(*)>(_a[2]))); break;
        case 41: _t->handleOperationUndoDeletes((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[3])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[4])),(*reinterpret_cast< const QVariantMap(*)>(_a[5]))); break;
        case 42: _t->handleOperationUndoCut((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[5])),(*reinterpret_cast< const QVariantMap(*)>(_a[6]))); break;
        case 43: _t->handleOperationFilesPreview((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[3]))); break;
        case 44: { bool _r = _t->handleIsSubFile((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 6:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 6:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 5:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QString> >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QString> >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 5:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 5:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::Global::CreateFileType >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 5:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::Global::CreateFileType >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 5:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 6:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QFileDevice::Permissions >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QFileDevice::Permissions >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            }
            break;
        case 27:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::ClipBoard::ClipboardAction >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            }
            break;
        case 35:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 41:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 42:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 43:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_fileoperations::FileOperationsEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_fileoperations__FileOperationsEventReceiver.data,
    qt_meta_data_dfmplugin_fileoperations__FileOperationsEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_fileoperations::FileOperationsEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_fileoperations::FileOperationsEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_fileoperations__FileOperationsEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_fileoperations::FileOperationsEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
