/****************************************************************************
** Meta object code from reading C++ file 'filecopymovejob.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-fileoperations/fileoperations/filecopymovejob.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filecopymovejob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_fileoperations__FileCopyMoveJob_t {
    QByteArrayData data[7];
    char stringdata0[124];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_fileoperations__FileCopyMoveJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_fileoperations__FileCopyMoveJob_t qt_meta_stringdata_dfmplugin_fileoperations__FileCopyMoveJob = {
    {
QT_MOC_LITERAL(0, 0, 41), // "dfmplugin_fileoperations::Fil..."
QT_MOC_LITERAL(1, 42, 15), // "onHandleAddTask"
QT_MOC_LITERAL(2, 58, 0), // ""
QT_MOC_LITERAL(3, 59, 23), // "onHandleAddTaskWithArgs"
QT_MOC_LITERAL(4, 83, 14), // "JobInfoPointer"
QT_MOC_LITERAL(5, 98, 4), // "info"
QT_MOC_LITERAL(6, 103, 20) // "onHandleTaskFinished"

    },
    "dfmplugin_fileoperations::FileCopyMoveJob\0"
    "onHandleAddTask\0\0onHandleAddTaskWithArgs\0"
    "JobInfoPointer\0info\0onHandleTaskFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_fileoperations__FileCopyMoveJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x08 /* Private */,
       3,    1,   30,    2, 0x08 /* Private */,
       6,    1,   33,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    5,

       0        // eod
};

void dfmplugin_fileoperations::FileCopyMoveJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileCopyMoveJob *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onHandleAddTask(); break;
        case 1: _t->onHandleAddTaskWithArgs((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 2: _t->onHandleTaskFinished((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_fileoperations::FileCopyMoveJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_fileoperations__FileCopyMoveJob.data,
    qt_meta_data_dfmplugin_fileoperations__FileCopyMoveJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_fileoperations::FileCopyMoveJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_fileoperations::FileCopyMoveJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_fileoperations__FileCopyMoveJob.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_fileoperations::FileCopyMoveJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
