/****************************************************************************
** Meta object code from reading C++ file 'pagerenderthread.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../../dde-file-manager/src/apps/dde-file-manager-preview/pluginpreviews/pdf-preview/pagerenderthread.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pagerenderthread.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_plugin_filepreview__PageRenderThread_t {
    QByteArrayData data[19];
    char stringdata0[317];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_plugin_filepreview__PageRenderThread_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_plugin_filepreview__PageRenderThread_t qt_meta_stringdata_plugin_filepreview__PageRenderThread = {
    {
QT_MOC_LITERAL(0, 0, 36), // "plugin_filepreview::PageRende..."
QT_MOC_LITERAL(1, 37, 33), // "sigDocPageNormalImageTaskFini..."
QT_MOC_LITERAL(2, 71, 0), // ""
QT_MOC_LITERAL(3, 72, 22), // "DocPageNormalImageTask"
QT_MOC_LITERAL(4, 95, 31), // "sigDocPageThumbnailTaskFinished"
QT_MOC_LITERAL(5, 127, 20), // "DocPageThumbnailTask"
QT_MOC_LITERAL(6, 148, 14), // "sigDocOpenTask"
QT_MOC_LITERAL(7, 163, 11), // "DocOpenTask"
QT_MOC_LITERAL(8, 175, 15), // "Document::Error"
QT_MOC_LITERAL(9, 191, 9), // "Document*"
QT_MOC_LITERAL(10, 201, 12), // "QList<Page*>"
QT_MOC_LITERAL(11, 214, 32), // "onDocPageNormalImageTaskFinished"
QT_MOC_LITERAL(12, 247, 4), // "task"
QT_MOC_LITERAL(13, 252, 6), // "pixmap"
QT_MOC_LITERAL(14, 259, 22), // "onDocPageThumbnailTask"
QT_MOC_LITERAL(15, 282, 13), // "onDocOpenTask"
QT_MOC_LITERAL(16, 296, 5), // "error"
QT_MOC_LITERAL(17, 302, 8), // "document"
QT_MOC_LITERAL(18, 311, 5) // "pages"

    },
    "plugin_filepreview::PageRenderThread\0"
    "sigDocPageNormalImageTaskFinished\0\0"
    "DocPageNormalImageTask\0"
    "sigDocPageThumbnailTaskFinished\0"
    "DocPageThumbnailTask\0sigDocOpenTask\0"
    "DocOpenTask\0Document::Error\0Document*\0"
    "QList<Page*>\0onDocPageNormalImageTaskFinished\0"
    "task\0pixmap\0onDocPageThumbnailTask\0"
    "onDocOpenTask\0error\0document\0pages"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_plugin_filepreview__PageRenderThread[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   44,    2, 0x06 /* Public */,
       4,    2,   49,    2, 0x06 /* Public */,
       6,    4,   54,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    2,   63,    2, 0x08 /* Private */,
      14,    2,   68,    2, 0x08 /* Private */,
      15,    4,   73,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QPixmap,    2,    2,
    QMetaType::Void, 0x80000000 | 5, QMetaType::QPixmap,    2,    2,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 8, 0x80000000 | 9, 0x80000000 | 10,    2,    2,    2,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QPixmap,   12,   13,
    QMetaType::Void, 0x80000000 | 5, QMetaType::QPixmap,   12,   13,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 8, 0x80000000 | 9, 0x80000000 | 10,   12,   16,   17,   18,

       0        // eod
};

void plugin_filepreview::PageRenderThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PageRenderThread *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sigDocPageNormalImageTaskFinished((*reinterpret_cast< DocPageNormalImageTask(*)>(_a[1])),(*reinterpret_cast< QPixmap(*)>(_a[2]))); break;
        case 1: _t->sigDocPageThumbnailTaskFinished((*reinterpret_cast< DocPageThumbnailTask(*)>(_a[1])),(*reinterpret_cast< QPixmap(*)>(_a[2]))); break;
        case 2: _t->sigDocOpenTask((*reinterpret_cast< DocOpenTask(*)>(_a[1])),(*reinterpret_cast< Document::Error(*)>(_a[2])),(*reinterpret_cast< Document*(*)>(_a[3])),(*reinterpret_cast< QList<Page*>(*)>(_a[4]))); break;
        case 3: _t->onDocPageNormalImageTaskFinished((*reinterpret_cast< DocPageNormalImageTask(*)>(_a[1])),(*reinterpret_cast< QPixmap(*)>(_a[2]))); break;
        case 4: _t->onDocPageThumbnailTask((*reinterpret_cast< DocPageThumbnailTask(*)>(_a[1])),(*reinterpret_cast< QPixmap(*)>(_a[2]))); break;
        case 5: _t->onDocOpenTask((*reinterpret_cast< DocOpenTask(*)>(_a[1])),(*reinterpret_cast< Document::Error(*)>(_a[2])),(*reinterpret_cast< Document*(*)>(_a[3])),(*reinterpret_cast< QList<Page*>(*)>(_a[4]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Document* >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<Page*> >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Document* >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<Page*> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PageRenderThread::*)(DocPageNormalImageTask , QPixmap );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageRenderThread::sigDocPageNormalImageTaskFinished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PageRenderThread::*)(DocPageThumbnailTask , QPixmap );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageRenderThread::sigDocPageThumbnailTaskFinished)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PageRenderThread::*)(DocOpenTask , Document::Error , Document * , QList<Page*> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageRenderThread::sigDocOpenTask)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject plugin_filepreview::PageRenderThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_plugin_filepreview__PageRenderThread.data,
    qt_meta_data_plugin_filepreview__PageRenderThread,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *plugin_filepreview::PageRenderThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *plugin_filepreview::PageRenderThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_plugin_filepreview__PageRenderThread.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int plugin_filepreview::PageRenderThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void plugin_filepreview::PageRenderThread::sigDocPageNormalImageTaskFinished(DocPageNormalImageTask _t1, QPixmap _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void plugin_filepreview::PageRenderThread::sigDocPageThumbnailTaskFinished(DocPageThumbnailTask _t1, QPixmap _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void plugin_filepreview::PageRenderThread::sigDocOpenTask(DocOpenTask _t1, Document::Error _t2, Document * _t3, QList<Page*> _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
