// This file was generated by dtk-settings-tools version 0.1.2 

#include <DSettings>

void global_setting_template_pro_trans()
{
    auto advance_dialog_default_chooser_dialogText = QObject::tr("Use the file chooser dialog of File Manager");
    auto advance_dialog_delete_confirmation_dialogText = QObject::tr("Ask for my confirmation when deleting files");
    auto advance_index_index_externalText = QObject::tr("Index external storage device after connected to computer");
    auto advance_index_index_internalText = QObject::tr("Auto index internal disk");
    auto advance_index_index_searchText = QObject::tr("Full-Text search");
    auto advance_items_in_sidebar_builtinText = QObject::tr("Built-in disks");
    auto advance_items_in_sidebar_computerText = QObject::tr("Computer");
    auto advance_items_in_sidebar_computers_in_lanText = QObject::tr("Computers in LAN");
    auto advance_items_in_sidebar_desktopText = QObject::tr("Desktop");
    auto advance_items_in_sidebar_documentsText = QObject::tr("Documents");
    auto advance_items_in_sidebar_downloadsText = QObject::tr("Downloads");
    auto advance_items_in_sidebar_homeText = QObject::tr("Home");
    auto advance_items_in_sidebar_loopText = QObject::tr("Loop partitions");
    auto advance_items_in_sidebar_mounted_share_dirsText = QObject::tr("Mounted sharing folders");
    auto advance_items_in_sidebar_musicText = QObject::tr("Music");
    auto advance_items_in_sidebar_my_sharesText = QObject::tr("My shares");
    auto advance_items_in_sidebar_network_splittersName = QObject::tr("Network");
    auto advance_items_in_sidebar_other_disksText = QObject::tr("Mounted partitions and discs");
    auto advance_items_in_sidebar_partitions_splitterName = QObject::tr("Partitions");
    auto advance_items_in_sidebar_picturesText = QObject::tr("Pictures");
    auto advance_items_in_sidebar_quick_access_splitterName = QObject::tr("Quick access");
    auto advance_items_in_sidebar_recentText = QObject::tr("Recent");
    auto advance_items_in_sidebar_tag_splitterName = QObject::tr("Tag");
    auto advance_items_in_sidebar_tagsText = QObject::tr("Added tags");
    auto advance_items_in_sidebar_trashText = QObject::tr("Trash");
    auto advance_items_in_sidebar_vaultText = QObject::tr("Vault");
    auto advance_items_in_sidebar_videosText = QObject::tr("Videos");
    auto advance_mount_always_show_offline_remote_connectionText = QObject::tr("Keep showing the mounted Samba shares");
    auto advance_mount_auto_mountText = QObject::tr("Auto mount");
    auto advance_mount_auto_mount_and_openText = QObject::tr("Open after auto mount");
    auto advance_mount_merge_the_entries_of_samba_shared_foldersText = QObject::tr("Merge the entries of Samba shared folders");
    auto advance_mount_mtp_show_bottom_infoText = QObject::tr("Show item counts and sizes in the path of mounted MTP devices");
    auto advance_other_extend_file_nameText = QObject::tr("Extend filename characters");
    auto advance_other_hide_builtin_partitionText = QObject::tr("Hide built-in disks on the Computer page");
    auto advance_other_hide_loop_partitionsText = QObject::tr("Hide loop partitions on the Computer page");
    auto advance_other_show_crumbbar_clickable_areaText = QObject::tr("Show crumb bar clickable area");
    auto advance_other_show_filesystemtag_on_diskiconText = QObject::tr("Show file system on disk icon");
    auto advance_preview_audio_file_previewText = QObject::tr("Music preview");
    auto advance_preview_document_file_previewText = QObject::tr("Document preview");
    auto advance_preview_image_file_previewText = QObject::tr("Image preview");
    auto advance_preview_remote_env_file_previewText = QObject::tr("The remote environment shows thumbnail previews");
    auto advance_preview_text_file_previewText = QObject::tr("Text preview");
    auto advance_preview_video_file_previewText = QObject::tr("Video preview");
    auto advance_search_show_hiddenText = QObject::tr("Show hidden files in search results");
    auto base_default_view_icon_sizeName = QObject::tr("Default icon size:");
    auto base_default_view_icon_sizeText0 = QObject::tr("Extra small");
    auto base_default_view_icon_sizeText1 = QObject::tr("Small");
    auto base_default_view_icon_sizeText2 = QObject::tr("Medium");
    auto base_default_view_icon_sizeText3 = QObject::tr("Large");
    auto base_default_view_icon_sizeText4 = QObject::tr("Extra large");
    auto base_default_view_mixed_sortText = QObject::tr("Mix sorting of files and folders");
    auto base_default_view_view_modeName = QObject::tr("Default view:");
    auto base_hidden_files_show_hiddenText = QObject::tr("Show hidden files");
    auto base_hidden_files_show_suffixText = QObject::tr("Show file extensions");
    auto base_new_tab_windows_default_window_pathName = QObject::tr("Open from default window:");
    auto base_new_tab_windows_new_tab_pathName = QObject::tr("Open in new tab:");
    auto base_open_action_allways_open_on_new_windowText = QObject::tr("Always open folder in new window");
    auto base_open_action_open_file_actionName = QObject::tr("Open file:");
    auto base_open_action_open_file_actionText0 = QObject::tr("Click");
    auto base_open_action_open_file_actionText1 = QObject::tr("Double click");
    auto group_advanceName = QObject::tr("Advanced");
    auto group_advance_dialogName = QObject::tr("Dialog");
    auto group_advance_indexName = QObject::tr("Index");
    auto group_advance_items_in_sidebarName = QObject::tr("Items on sidebar pane");
    auto group_advance_mountName = QObject::tr("Mount");
    auto group_advance_otherName = QObject::tr("Other");
    auto group_advance_previewName = QObject::tr("Preview");
    auto group_baseName = QObject::tr("Basic");
    auto group_base_default_viewName = QObject::tr("View");
    auto group_base_hidden_filesName = QObject::tr("Hidden files");
    auto group_base_new_tab_windowsName = QObject::tr("New window and tab");
    auto group_base_open_actionName = QObject::tr("Open behavior");
}
