/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a vaultmanageradaptor -i dbus/vaultmanagerdbus.h -l VaultManagerDBus /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.Daemon.VaultManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "vaultmanageradaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class VaultManagerAdaptor
 */

VaultManagerAdaptor::VaultManagerAdaptor(VaultManagerDBus *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

VaultManagerAdaptor::~VaultManagerAdaptor()
{
    // destructor
}

void VaultManagerAdaptor::ClearLockEvent()
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.ClearLockEvent
    parent()->ClearLockEvent();
}

void VaultManagerAdaptor::ComputerSleep(bool bSleep)
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.ComputerSleep
    parent()->ComputerSleep(bSleep);
}

qulonglong VaultManagerAdaptor::GetLastestTime()
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.GetLastestTime
    return parent()->GetLastestTime();
}

int VaultManagerAdaptor::GetLeftoverErrorInputTimes(int userID)
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.GetLeftoverErrorInputTimes
    return parent()->GetLeftoverErrorInputTimes(userID);
}

int VaultManagerAdaptor::GetNeedWaitMinutes(int userID)
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.GetNeedWaitMinutes
    return parent()->GetNeedWaitMinutes(userID);
}

qulonglong VaultManagerAdaptor::GetSelfTime()
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.GetSelfTime
    return parent()->GetSelfTime();
}

bool VaultManagerAdaptor::IsLockEventTriggered()
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.IsLockEventTriggered
    return parent()->IsLockEventTriggered();
}

void VaultManagerAdaptor::LeftoverErrorInputTimesMinusOne(int userID)
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.LeftoverErrorInputTimesMinusOne
    parent()->LeftoverErrorInputTimesMinusOne(userID);
}

void VaultManagerAdaptor::RestoreLeftoverErrorInputTimes(int userID)
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.RestoreLeftoverErrorInputTimes
    parent()->RestoreLeftoverErrorInputTimes(userID);
}

void VaultManagerAdaptor::RestoreNeedWaitMinutes(int userID)
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.RestoreNeedWaitMinutes
    parent()->RestoreNeedWaitMinutes(userID);
}

void VaultManagerAdaptor::SetRefreshTime(qulonglong time)
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.SetRefreshTime
    parent()->SetRefreshTime(time);
}

void VaultManagerAdaptor::StartTimerOfRestorePasswordInput(int userID)
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.StartTimerOfRestorePasswordInput
    parent()->StartTimerOfRestorePasswordInput(userID);
}

void VaultManagerAdaptor::SysUserChanged(const QString &curUser)
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.SysUserChanged
    parent()->SysUserChanged(curUser);
}

void VaultManagerAdaptor::TriggerLockEvent()
{
    // handle method call org.deepin.Filemanager.Daemon.VaultManager.TriggerLockEvent
    parent()->TriggerLockEvent();
}


#include "vaultmanageradaptor.moc"
