/****************************************************************************
** Meta object code from reading C++ file 'headerview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-workspace/views/headerview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'headerview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_workspace__HeaderView_t {
    QByteArrayData data[12];
    char stringdata0[149];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_workspace__HeaderView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_workspace__HeaderView_t qt_meta_stringdata_dfmplugin_workspace__HeaderView = {
    {
QT_MOC_LITERAL(0, 0, 31), // "dfmplugin_workspace::HeaderView"
QT_MOC_LITERAL(1, 32, 12), // "mousePressed"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 13), // "mouseReleased"
QT_MOC_LITERAL(4, 60, 11), // "viewResized"
QT_MOC_LITERAL(5, 72, 20), // "hiddenSectionChanged"
QT_MOC_LITERAL(6, 93, 8), // "roleName"
QT_MOC_LITERAL(7, 102, 7), // "checked"
QT_MOC_LITERAL(8, 110, 15), // "onActionClicked"
QT_MOC_LITERAL(9, 126, 6), // "column"
QT_MOC_LITERAL(10, 133, 8), // "QAction*"
QT_MOC_LITERAL(11, 142, 6) // "action"

    },
    "dfmplugin_workspace::HeaderView\0"
    "mousePressed\0\0mouseReleased\0viewResized\0"
    "hiddenSectionChanged\0roleName\0checked\0"
    "onActionClicked\0column\0QAction*\0action"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_workspace__HeaderView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x06 /* Public */,
       3,    0,   40,    2, 0x06 /* Public */,
       4,    0,   41,    2, 0x06 /* Public */,
       5,    2,   42,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    2,   47,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    6,    7,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, 0x80000000 | 10,    9,   11,

       0        // eod
};

void dfmplugin_workspace::HeaderView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HeaderView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->mousePressed(); break;
        case 1: _t->mouseReleased(); break;
        case 2: _t->viewResized(); break;
        case 3: _t->hiddenSectionChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        case 4: _t->onActionClicked((*reinterpret_cast< const int(*)>(_a[1])),(*reinterpret_cast< QAction*(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HeaderView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HeaderView::mousePressed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (HeaderView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HeaderView::mouseReleased)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (HeaderView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HeaderView::viewResized)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (HeaderView::*)(const QString & , const bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HeaderView::hiddenSectionChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_workspace::HeaderView::staticMetaObject = { {
    QMetaObject::SuperData::link<QHeaderView::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_workspace__HeaderView.data,
    qt_meta_data_dfmplugin_workspace__HeaderView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_workspace::HeaderView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_workspace::HeaderView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_workspace__HeaderView.stringdata0))
        return static_cast<void*>(this);
    return QHeaderView::qt_metacast(_clname);
}

int dfmplugin_workspace::HeaderView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QHeaderView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_workspace::HeaderView::mousePressed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dfmplugin_workspace::HeaderView::mouseReleased()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dfmplugin_workspace::HeaderView::viewResized()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void dfmplugin_workspace::HeaderView::hiddenSectionChanged(const QString & _t1, const bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
