/****************************************************************************
** Meta object code from reading C++ file 'abstractworker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-fileoperations/fileoperations/fileoperationutils/abstractworker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstractworker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_fileoperations__AbstractWorker_t {
    QByteArrayData data[30];
    char stringdata0[496];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_fileoperations__AbstractWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_fileoperations__AbstractWorker_t qt_meta_stringdata_dfmplugin_fileoperations__AbstractWorker = {
    {
QT_MOC_LITERAL(0, 0, 40), // "dfmplugin_fileoperations::Abs..."
QT_MOC_LITERAL(1, 41, 21), // "progressChangedNotify"
QT_MOC_LITERAL(2, 63, 0), // ""
QT_MOC_LITERAL(3, 64, 14), // "JobInfoPointer"
QT_MOC_LITERAL(4, 79, 7), // "jobInfo"
QT_MOC_LITERAL(5, 87, 18), // "stateChangedNotify"
QT_MOC_LITERAL(6, 106, 17), // "currentTaskNotify"
QT_MOC_LITERAL(7, 124, 14), // "finishedNotify"
QT_MOC_LITERAL(8, 139, 18), // "speedUpdatedNotify"
QT_MOC_LITERAL(9, 158, 16), // "removeTaskWidget"
QT_MOC_LITERAL(10, 175, 21), // "requestShowTipsDialog"
QT_MOC_LITERAL(11, 197, 43), // "dfmbase::AbstractJobHandler::..."
QT_MOC_LITERAL(12, 241, 4), // "type"
QT_MOC_LITERAL(13, 246, 11), // "QList<QUrl>"
QT_MOC_LITERAL(14, 258, 4), // "list"
QT_MOC_LITERAL(15, 263, 12), // "workerFinish"
QT_MOC_LITERAL(16, 276, 24), // "requestSaveRedoOperation"
QT_MOC_LITERAL(17, 301, 5), // "token"
QT_MOC_LITERAL(18, 307, 19), // "deleteFirstFileSize"
QT_MOC_LITERAL(19, 327, 24), // "startUpdateProgressTimer"
QT_MOC_LITERAL(20, 352, 9), // "startWork"
QT_MOC_LITERAL(21, 362, 11), // "errorNotify"
QT_MOC_LITERAL(22, 374, 15), // "retryErrSuccess"
QT_MOC_LITERAL(23, 390, 2), // "id"
QT_MOC_LITERAL(24, 393, 17), // "requestTaskDailog"
QT_MOC_LITERAL(25, 411, 6), // "doWork"
QT_MOC_LITERAL(26, 418, 16), // "onUpdateProgress"
QT_MOC_LITERAL(27, 435, 27), // "onStatisticsFilesSizeFinish"
QT_MOC_LITERAL(28, 463, 27), // "onStatisticsFilesSizeUpdate"
QT_MOC_LITERAL(29, 491, 4) // "size"

    },
    "dfmplugin_fileoperations::AbstractWorker\0"
    "progressChangedNotify\0\0JobInfoPointer\0"
    "jobInfo\0stateChangedNotify\0currentTaskNotify\0"
    "finishedNotify\0speedUpdatedNotify\0"
    "removeTaskWidget\0requestShowTipsDialog\0"
    "dfmbase::AbstractJobHandler::ShowDialogType\0"
    "type\0QList<QUrl>\0list\0workerFinish\0"
    "requestSaveRedoOperation\0token\0"
    "deleteFirstFileSize\0startUpdateProgressTimer\0"
    "startWork\0errorNotify\0retryErrSuccess\0"
    "id\0requestTaskDailog\0doWork\0"
    "onUpdateProgress\0onStatisticsFilesSizeFinish\0"
    "onStatisticsFilesSizeUpdate\0size"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_fileoperations__AbstractWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      14,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  104,    2, 0x06 /* Public */,
       5,    1,  107,    2, 0x06 /* Public */,
       6,    1,  110,    2, 0x06 /* Public */,
       7,    1,  113,    2, 0x06 /* Public */,
       8,    1,  116,    2, 0x06 /* Public */,
       9,    0,  119,    2, 0x06 /* Public */,
      10,    2,  120,    2, 0x06 /* Public */,
      15,    0,  125,    2, 0x06 /* Public */,
      16,    2,  126,    2, 0x06 /* Public */,
      19,    0,  131,    2, 0x06 /* Public */,
      20,    0,  132,    2, 0x06 /* Public */,
      21,    1,  133,    2, 0x06 /* Public */,
      22,    1,  136,    2, 0x06 /* Public */,
      24,    0,  139,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      25,    0,  140,    2, 0x09 /* Protected */,
      26,    0,  141,    2, 0x09 /* Protected */,
      27,    0,  142,    2, 0x09 /* Protected */,
      28,    1,  143,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 11, 0x80000000 | 13,   12,   14,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::LongLong,   17,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::ULongLong,   23,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::LongLong,   29,

       0        // eod
};

void dfmplugin_fileoperations::AbstractWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AbstractWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->progressChangedNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 1: _t->stateChangedNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 2: _t->currentTaskNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 3: _t->finishedNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 4: _t->speedUpdatedNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 5: _t->removeTaskWidget(); break;
        case 6: _t->requestShowTipsDialog((*reinterpret_cast< dfmbase::AbstractJobHandler::ShowDialogType(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2]))); break;
        case 7: _t->workerFinish(); break;
        case 8: _t->requestSaveRedoOperation((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const qint64(*)>(_a[2]))); break;
        case 9: _t->startUpdateProgressTimer(); break;
        case 10: _t->startWork(); break;
        case 11: _t->errorNotify((*reinterpret_cast< const JobInfoPointer(*)>(_a[1]))); break;
        case 12: _t->retryErrSuccess((*reinterpret_cast< const quint64(*)>(_a[1]))); break;
        case 13: _t->requestTaskDailog(); break;
        case 14: { bool _r = _t->doWork();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->onUpdateProgress(); break;
        case 16: _t->onStatisticsFilesSizeFinish(); break;
        case 17: _t->onStatisticsFilesSizeUpdate((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::ShowDialogType >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< JobInfoPointer >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AbstractWorker::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::progressChangedNotify)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::stateChangedNotify)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::currentTaskNotify)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::finishedNotify)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::speedUpdatedNotify)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::removeTaskWidget)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)(dfmbase::AbstractJobHandler::ShowDialogType , const QList<QUrl> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::requestShowTipsDialog)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::workerFinish)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)(const QString & , const qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::requestSaveRedoOperation)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::startUpdateProgressTimer)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::startWork)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)(const JobInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::errorNotify)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)(const quint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::retryErrSuccess)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (AbstractWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractWorker::requestTaskDailog)) {
                *result = 13;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_fileoperations::AbstractWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_fileoperations__AbstractWorker.data,
    qt_meta_data_dfmplugin_fileoperations__AbstractWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_fileoperations::AbstractWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_fileoperations::AbstractWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_fileoperations__AbstractWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_fileoperations::AbstractWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_fileoperations::AbstractWorker::progressChangedNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_fileoperations::AbstractWorker::stateChangedNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_fileoperations::AbstractWorker::currentTaskNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_fileoperations::AbstractWorker::finishedNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_fileoperations::AbstractWorker::speedUpdatedNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmplugin_fileoperations::AbstractWorker::removeTaskWidget()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void dfmplugin_fileoperations::AbstractWorker::requestShowTipsDialog(dfmbase::AbstractJobHandler::ShowDialogType _t1, const QList<QUrl> & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void dfmplugin_fileoperations::AbstractWorker::workerFinish()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void dfmplugin_fileoperations::AbstractWorker::requestSaveRedoOperation(const QString & _t1, const qint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void dfmplugin_fileoperations::AbstractWorker::startUpdateProgressTimer()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void dfmplugin_fileoperations::AbstractWorker::startWork()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void dfmplugin_fileoperations::AbstractWorker::errorNotify(const JobInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void dfmplugin_fileoperations::AbstractWorker::retryErrSuccess(const quint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void dfmplugin_fileoperations::AbstractWorker::requestTaskDailog()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
