/****************************************************************************
** Meta object code from reading C++ file 'rootinfo.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-workspace/models/rootinfo.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'rootinfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_workspace__RootInfo_t {
    QByteArrayData data[56];
    char stringdata0[834];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_workspace__RootInfo_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_workspace__RootInfo_t qt_meta_stringdata_dfmplugin_workspace__RootInfo = {
    {
QT_MOC_LITERAL(0, 0, 29), // "dfmplugin_workspace::RootInfo"
QT_MOC_LITERAL(1, 30, 9), // "itemAdded"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 18), // "iteratorLocalFiles"
QT_MOC_LITERAL(4, 60, 3), // "key"
QT_MOC_LITERAL(5, 64, 22), // "QList<SortInfoPointer>"
QT_MOC_LITERAL(6, 87, 8), // "children"
QT_MOC_LITERAL(7, 96, 39), // "dfmio::DEnumerator::SortRoleC..."
QT_MOC_LITERAL(8, 136, 8), // "sortRole"
QT_MOC_LITERAL(9, 145, 13), // "Qt::SortOrder"
QT_MOC_LITERAL(10, 159, 9), // "sortOrder"
QT_MOC_LITERAL(11, 169, 15), // "isMixDirAndFile"
QT_MOC_LITERAL(12, 185, 15), // "iteratorAddFile"
QT_MOC_LITERAL(13, 201, 15), // "SortInfoPointer"
QT_MOC_LITERAL(14, 217, 8), // "sortInfo"
QT_MOC_LITERAL(15, 226, 15), // "FileInfoPointer"
QT_MOC_LITERAL(16, 242, 4), // "info"
QT_MOC_LITERAL(17, 247, 16), // "iteratorAddFiles"
QT_MOC_LITERAL(18, 264, 9), // "sortInfos"
QT_MOC_LITERAL(19, 274, 22), // "QList<FileInfoPointer>"
QT_MOC_LITERAL(20, 297, 5), // "infos"
QT_MOC_LITERAL(21, 303, 15), // "watcherAddFiles"
QT_MOC_LITERAL(22, 319, 18), // "watcherRemoveFiles"
QT_MOC_LITERAL(23, 338, 17), // "traversalFinished"
QT_MOC_LITERAL(24, 356, 11), // "sourceDatas"
QT_MOC_LITERAL(25, 368, 10), // "isFinished"
QT_MOC_LITERAL(26, 379, 17), // "watcherUpdateFile"
QT_MOC_LITERAL(27, 397, 18), // "watcherUpdateFiles"
QT_MOC_LITERAL(28, 416, 21), // "watcherUpdateHideFile"
QT_MOC_LITERAL(29, 438, 6), // "hidUrl"
QT_MOC_LITERAL(30, 445, 11), // "requestSort"
QT_MOC_LITERAL(31, 457, 6), // "dirUrl"
QT_MOC_LITERAL(32, 464, 15), // "requestCloseTab"
QT_MOC_LITERAL(33, 480, 3), // "url"
QT_MOC_LITERAL(34, 484, 18), // "requestTreeSortDir"
QT_MOC_LITERAL(35, 503, 6), // "parent"
QT_MOC_LITERAL(36, 510, 24), // "renameFileProcessStarted"
QT_MOC_LITERAL(37, 535, 16), // "requestClearRoot"
QT_MOC_LITERAL(38, 552, 13), // "doFileDeleted"
QT_MOC_LITERAL(39, 566, 11), // "dofileMoved"
QT_MOC_LITERAL(40, 578, 7), // "fromUrl"
QT_MOC_LITERAL(41, 586, 5), // "toUrl"
QT_MOC_LITERAL(42, 592, 13), // "dofileCreated"
QT_MOC_LITERAL(43, 606, 13), // "doFileUpdated"
QT_MOC_LITERAL(44, 620, 14), // "doWatcherEvent"
QT_MOC_LITERAL(45, 635, 20), // "doThreadWatcherEvent"
QT_MOC_LITERAL(46, 656, 21), // "handleTraversalResult"
QT_MOC_LITERAL(47, 678, 5), // "child"
QT_MOC_LITERAL(48, 684, 11), // "travseToken"
QT_MOC_LITERAL(49, 696, 22), // "handleTraversalResults"
QT_MOC_LITERAL(50, 719, 26), // "handleTraversalLocalResult"
QT_MOC_LITERAL(51, 746, 21), // "handleTraversalFinish"
QT_MOC_LITERAL(52, 768, 19), // "handleTraversalSort"
QT_MOC_LITERAL(53, 788, 19), // "handleGetSourceData"
QT_MOC_LITERAL(54, 808, 12), // "currentToken"
QT_MOC_LITERAL(55, 821, 12) // "startWatcher"

    },
    "dfmplugin_workspace::RootInfo\0itemAdded\0"
    "\0iteratorLocalFiles\0key\0QList<SortInfoPointer>\0"
    "children\0dfmio::DEnumerator::SortRoleCompareFlag\0"
    "sortRole\0Qt::SortOrder\0sortOrder\0"
    "isMixDirAndFile\0iteratorAddFile\0"
    "SortInfoPointer\0sortInfo\0FileInfoPointer\0"
    "info\0iteratorAddFiles\0sortInfos\0"
    "QList<FileInfoPointer>\0infos\0"
    "watcherAddFiles\0watcherRemoveFiles\0"
    "traversalFinished\0sourceDatas\0isFinished\0"
    "watcherUpdateFile\0watcherUpdateFiles\0"
    "watcherUpdateHideFile\0hidUrl\0requestSort\0"
    "dirUrl\0requestCloseTab\0url\0"
    "requestTreeSortDir\0parent\0"
    "renameFileProcessStarted\0requestClearRoot\0"
    "doFileDeleted\0dofileMoved\0fromUrl\0"
    "toUrl\0dofileCreated\0doFileUpdated\0"
    "doWatcherEvent\0doThreadWatcherEvent\0"
    "handleTraversalResult\0child\0travseToken\0"
    "handleTraversalResults\0"
    "handleTraversalLocalResult\0"
    "handleTraversalFinish\0handleTraversalSort\0"
    "handleGetSourceData\0currentToken\0"
    "startWatcher"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_workspace__RootInfo[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      29,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      16,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  159,    2, 0x06 /* Public */,
       3,    5,  160,    2, 0x06 /* Public */,
      12,    3,  171,    2, 0x06 /* Public */,
      17,    3,  178,    2, 0x06 /* Public */,
      21,    1,  185,    2, 0x06 /* Public */,
      22,    1,  188,    2, 0x06 /* Public */,
      23,    1,  191,    2, 0x06 /* Public */,
      24,    6,  194,    2, 0x06 /* Public */,
      26,    1,  207,    2, 0x06 /* Public */,
      27,    1,  210,    2, 0x06 /* Public */,
      28,    1,  213,    2, 0x06 /* Public */,
      30,    2,  216,    2, 0x06 /* Public */,
      32,    1,  221,    2, 0x06 /* Public */,
      34,    2,  224,    2, 0x06 /* Public */,
      36,    0,  229,    2, 0x06 /* Public */,
      37,    1,  230,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      38,    1,  233,    2, 0x0a /* Public */,
      39,    2,  236,    2, 0x0a /* Public */,
      42,    1,  241,    2, 0x0a /* Public */,
      43,    1,  244,    2, 0x0a /* Public */,
      44,    0,  247,    2, 0x0a /* Public */,
      45,    0,  248,    2, 0x0a /* Public */,
      46,    2,  249,    2, 0x0a /* Public */,
      49,    2,  254,    2, 0x0a /* Public */,
      50,    5,  259,    2, 0x0a /* Public */,
      51,    1,  270,    2, 0x0a /* Public */,
      52,    1,  273,    2, 0x0a /* Public */,
      53,    1,  276,    2, 0x0a /* Public */,
      55,    0,  279,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 5, 0x80000000 | 7, 0x80000000 | 9, QMetaType::Bool,    4,    6,    8,   10,   11,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 13, 0x80000000 | 15,    4,   14,   16,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 5, 0x80000000 | 19,    4,   18,   20,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 5, 0x80000000 | 7, 0x80000000 | 9, QMetaType::Bool, QMetaType::Bool,    4,    6,    8,   10,   11,   25,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 5,   18,
    QMetaType::Void, QMetaType::QUrl,   29,
    QMetaType::Void, QMetaType::QString, QMetaType::QUrl,    4,   31,
    QMetaType::Void, QMetaType::QUrl,   33,
    QMetaType::Void, QMetaType::QString, QMetaType::QUrl,    4,   35,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,   33,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl,   33,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,   40,   41,
    QMetaType::Void, QMetaType::QUrl,   33,
    QMetaType::Void, QMetaType::QUrl,   33,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15, QMetaType::QString,   47,   48,
    QMetaType::Void, 0x80000000 | 19, QMetaType::QString,    6,   48,
    QMetaType::Void, 0x80000000 | 5, 0x80000000 | 7, 0x80000000 | 9, QMetaType::Bool, QMetaType::QString,    6,    8,   10,   11,   48,
    QMetaType::Void, QMetaType::QString,   48,
    QMetaType::Void, QMetaType::QString,   48,
    QMetaType::Void, QMetaType::QString,   54,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_workspace::RootInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RootInfo *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->itemAdded(); break;
        case 1: _t->iteratorLocalFiles((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[2])),(*reinterpret_cast< const dfmio::DEnumerator::SortRoleCompareFlag(*)>(_a[3])),(*reinterpret_cast< const Qt::SortOrder(*)>(_a[4])),(*reinterpret_cast< const bool(*)>(_a[5]))); break;
        case 2: _t->iteratorAddFile((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const SortInfoPointer(*)>(_a[2])),(*reinterpret_cast< const FileInfoPointer(*)>(_a[3]))); break;
        case 3: _t->iteratorAddFiles((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[2])),(*reinterpret_cast< const QList<FileInfoPointer>(*)>(_a[3]))); break;
        case 4: _t->watcherAddFiles((*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[1]))); break;
        case 5: _t->watcherRemoveFiles((*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[1]))); break;
        case 6: _t->traversalFinished((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->sourceDatas((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[2])),(*reinterpret_cast< const dfmio::DEnumerator::SortRoleCompareFlag(*)>(_a[3])),(*reinterpret_cast< const Qt::SortOrder(*)>(_a[4])),(*reinterpret_cast< const bool(*)>(_a[5])),(*reinterpret_cast< const bool(*)>(_a[6]))); break;
        case 8: _t->watcherUpdateFile((*reinterpret_cast< const SortInfoPointer(*)>(_a[1]))); break;
        case 9: _t->watcherUpdateFiles((*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[1]))); break;
        case 10: _t->watcherUpdateHideFile((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 11: _t->requestSort((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 12: _t->requestCloseTab((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 13: _t->requestTreeSortDir((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 14: _t->renameFileProcessStarted(); break;
        case 15: _t->requestClearRoot((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 16: _t->doFileDeleted((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 17: _t->dofileMoved((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 18: _t->dofileCreated((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 19: _t->doFileUpdated((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 20: _t->doWatcherEvent(); break;
        case 21: _t->doThreadWatcherEvent(); break;
        case 22: _t->handleTraversalResult((*reinterpret_cast< const FileInfoPointer(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 23: _t->handleTraversalResults((*reinterpret_cast< const QList<FileInfoPointer>(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 24: _t->handleTraversalLocalResult((*reinterpret_cast< QList<SortInfoPointer>(*)>(_a[1])),(*reinterpret_cast< dfmio::DEnumerator::SortRoleCompareFlag(*)>(_a[2])),(*reinterpret_cast< Qt::SortOrder(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4])),(*reinterpret_cast< const QString(*)>(_a[5]))); break;
        case 25: _t->handleTraversalFinish((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 26: _t->handleTraversalSort((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 27: _t->handleGetSourceData((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 28: _t->startWatcher(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmio::DEnumerator::SortRoleCompareFlag >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< FileInfoPointer >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< SortInfoPointer >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<FileInfoPointer> >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmio::DEnumerator::SortRoleCompareFlag >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< SortInfoPointer >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< FileInfoPointer >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<FileInfoPointer> >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmio::DEnumerator::SortRoleCompareFlag >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RootInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::itemAdded)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QString & , const QList<SortInfoPointer> , const dfmio::DEnumerator::SortRoleCompareFlag , const Qt::SortOrder , const bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::iteratorLocalFiles)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QString & , const SortInfoPointer , const FileInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::iteratorAddFile)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QString & , const QList<SortInfoPointer> , const QList<FileInfoPointer> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::iteratorAddFiles)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QList<SortInfoPointer> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::watcherAddFiles)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QList<SortInfoPointer> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::watcherRemoveFiles)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::traversalFinished)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QString & , const QList<SortInfoPointer> , const dfmio::DEnumerator::SortRoleCompareFlag , const Qt::SortOrder , const bool , const bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::sourceDatas)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const SortInfoPointer );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::watcherUpdateFile)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QList<SortInfoPointer> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::watcherUpdateFiles)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::watcherUpdateHideFile)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QString & , const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::requestSort)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::requestCloseTab)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QString & , const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::requestTreeSortDir)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::renameFileProcessStarted)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (RootInfo::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RootInfo::requestClearRoot)) {
                *result = 15;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_workspace::RootInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_workspace__RootInfo.data,
    qt_meta_data_dfmplugin_workspace__RootInfo,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_workspace::RootInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_workspace::RootInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_workspace__RootInfo.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_workspace::RootInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_workspace::RootInfo::itemAdded()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dfmplugin_workspace::RootInfo::iteratorLocalFiles(const QString & _t1, const QList<SortInfoPointer> _t2, const dfmio::DEnumerator::SortRoleCompareFlag _t3, const Qt::SortOrder _t4, const bool _t5)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_workspace::RootInfo::iteratorAddFile(const QString & _t1, const SortInfoPointer _t2, const FileInfoPointer _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_workspace::RootInfo::iteratorAddFiles(const QString & _t1, const QList<SortInfoPointer> _t2, const QList<FileInfoPointer> _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_workspace::RootInfo::watcherAddFiles(const QList<SortInfoPointer> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmplugin_workspace::RootInfo::watcherRemoveFiles(const QList<SortInfoPointer> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void dfmplugin_workspace::RootInfo::traversalFinished(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void dfmplugin_workspace::RootInfo::sourceDatas(const QString & _t1, const QList<SortInfoPointer> _t2, const dfmio::DEnumerator::SortRoleCompareFlag _t3, const Qt::SortOrder _t4, const bool _t5, const bool _t6)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t6))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void dfmplugin_workspace::RootInfo::watcherUpdateFile(const SortInfoPointer _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void dfmplugin_workspace::RootInfo::watcherUpdateFiles(const QList<SortInfoPointer> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void dfmplugin_workspace::RootInfo::watcherUpdateHideFile(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void dfmplugin_workspace::RootInfo::requestSort(const QString & _t1, const QUrl & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void dfmplugin_workspace::RootInfo::requestCloseTab(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void dfmplugin_workspace::RootInfo::requestTreeSortDir(const QString & _t1, const QUrl & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void dfmplugin_workspace::RootInfo::renameFileProcessStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void dfmplugin_workspace::RootInfo::requestClearRoot(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
