/****************************************************************************
** Meta object code from reading C++ file 'recentiterateworker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/daemon/daemon-recentdaemon/recentiterateworker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'recentiterateworker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_serverplugin_recentmanager__RecentIterateWorker_t {
    QByteArrayData data[18];
    char stringdata0[245];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_serverplugin_recentmanager__RecentIterateWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_serverplugin_recentmanager__RecentIterateWorker_t qt_meta_stringdata_serverplugin_recentmanager__RecentIterateWorker = {
    {
QT_MOC_LITERAL(0, 0, 47), // "serverplugin_recentmanager::R..."
QT_MOC_LITERAL(1, 48, 14), // "reloadFinished"
QT_MOC_LITERAL(2, 63, 0), // ""
QT_MOC_LITERAL(3, 64, 9), // "timestamp"
QT_MOC_LITERAL(4, 74, 13), // "purgeFinished"
QT_MOC_LITERAL(5, 88, 9), // "itemAdded"
QT_MOC_LITERAL(6, 98, 4), // "path"
QT_MOC_LITERAL(7, 103, 10), // "RecentItem"
QT_MOC_LITERAL(8, 114, 4), // "item"
QT_MOC_LITERAL(9, 119, 12), // "itemsRemoved"
QT_MOC_LITERAL(10, 132, 5), // "paths"
QT_MOC_LITERAL(11, 138, 11), // "itemChanged"
QT_MOC_LITERAL(12, 150, 15), // "onRequestReload"
QT_MOC_LITERAL(13, 166, 8), // "xbelPath"
QT_MOC_LITERAL(14, 175, 22), // "onRequestAddRecentItem"
QT_MOC_LITERAL(15, 198, 20), // "onRequestRemoveItems"
QT_MOC_LITERAL(16, 219, 5), // "hrefs"
QT_MOC_LITERAL(17, 225, 19) // "onRequestPurgeItems"

    },
    "serverplugin_recentmanager::RecentIterateWorker\0"
    "reloadFinished\0\0timestamp\0purgeFinished\0"
    "itemAdded\0path\0RecentItem\0item\0"
    "itemsRemoved\0paths\0itemChanged\0"
    "onRequestReload\0xbelPath\0"
    "onRequestAddRecentItem\0onRequestRemoveItems\0"
    "hrefs\0onRequestPurgeItems"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_serverplugin_recentmanager__RecentIterateWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,
       4,    0,   62,    2, 0x06 /* Public */,
       5,    2,   63,    2, 0x06 /* Public */,
       9,    1,   68,    2, 0x06 /* Public */,
      11,    2,   71,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    2,   76,    2, 0x0a /* Public */,
      14,    1,   81,    2, 0x0a /* Public */,
      15,    1,   84,    2, 0x0a /* Public */,
      17,    1,   87,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::LongLong,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 7,    6,    8,
    QMetaType::Void, QMetaType::QStringList,   10,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 7,    6,    8,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::LongLong,   13,    3,
    QMetaType::Void, QMetaType::QVariantMap,    8,
    QMetaType::Void, QMetaType::QStringList,   16,
    QMetaType::Void, QMetaType::QString,   13,

       0        // eod
};

void serverplugin_recentmanager::RecentIterateWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RecentIterateWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->reloadFinished((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 1: _t->purgeFinished(); break;
        case 2: _t->itemAdded((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const RecentItem(*)>(_a[2]))); break;
        case 3: _t->itemsRemoved((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 4: _t->itemChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const RecentItem(*)>(_a[2]))); break;
        case 5: _t->onRequestReload((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2]))); break;
        case 6: _t->onRequestAddRecentItem((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 7: _t->onRequestRemoveItems((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 8: _t->onRequestPurgeItems((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RecentIterateWorker::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentIterateWorker::reloadFinished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RecentIterateWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentIterateWorker::purgeFinished)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (RecentIterateWorker::*)(const QString & , const RecentItem & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentIterateWorker::itemAdded)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (RecentIterateWorker::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentIterateWorker::itemsRemoved)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (RecentIterateWorker::*)(const QString & , const RecentItem & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentIterateWorker::itemChanged)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject serverplugin_recentmanager::RecentIterateWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_serverplugin_recentmanager__RecentIterateWorker.data,
    qt_meta_data_serverplugin_recentmanager__RecentIterateWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *serverplugin_recentmanager::RecentIterateWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *serverplugin_recentmanager::RecentIterateWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_serverplugin_recentmanager__RecentIterateWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int serverplugin_recentmanager::RecentIterateWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void serverplugin_recentmanager::RecentIterateWorker::reloadFinished(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void serverplugin_recentmanager::RecentIterateWorker::purgeFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void serverplugin_recentmanager::RecentIterateWorker::itemAdded(const QString & _t1, const RecentItem & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void serverplugin_recentmanager::RecentIterateWorker::itemsRemoved(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void serverplugin_recentmanager::RecentIterateWorker::itemChanged(const QString & _t1, const RecentItem & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
