/****************************************************************************
** Meta object code from reading C++ file 'vaulthelper.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-vault/utils/vaulthelper.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vaulthelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_vault__VaultHelper_t {
    QByteArrayData data[25];
    char stringdata0[309];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_vault__VaultHelper_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_vault__VaultHelper_t qt_meta_stringdata_dfmplugin_vault__VaultHelper = {
    {
QT_MOC_LITERAL(0, 0, 28), // "dfmplugin_vault::VaultHelper"
QT_MOC_LITERAL(1, 29, 14), // "sigCreateVault"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 5), // "state"
QT_MOC_LITERAL(4, 51, 11), // "sigUnlocked"
QT_MOC_LITERAL(5, 63, 9), // "sigLocked"
QT_MOC_LITERAL(6, 73, 14), // "sigCloseWindow"
QT_MOC_LITERAL(7, 88, 13), // "slotlockVault"
QT_MOC_LITERAL(8, 102, 11), // "createVault"
QT_MOC_LITERAL(9, 114, 8), // "QString&"
QT_MOC_LITERAL(10, 123, 8), // "password"
QT_MOC_LITERAL(11, 132, 11), // "unlockVault"
QT_MOC_LITERAL(12, 144, 9), // "lockVault"
QT_MOC_LITERAL(13, 154, 8), // "isForced"
QT_MOC_LITERAL(14, 163, 15), // "defaultCdAction"
QT_MOC_LITERAL(15, 179, 8), // "windowId"
QT_MOC_LITERAL(16, 188, 3), // "url"
QT_MOC_LITERAL(17, 192, 13), // "openNewWindow"
QT_MOC_LITERAL(18, 206, 17), // "createVaultDialog"
QT_MOC_LITERAL(19, 224, 17), // "unlockVaultDialog"
QT_MOC_LITERAL(20, 242, 21), // "showRemoveVaultDialog"
QT_MOC_LITERAL(21, 264, 10), // "openWindow"
QT_MOC_LITERAL(22, 275, 13), // "openWidWindow"
QT_MOC_LITERAL(23, 289, 5), // "winID"
QT_MOC_LITERAL(24, 295, 13) // "newOpenWindow"

    },
    "dfmplugin_vault::VaultHelper\0"
    "sigCreateVault\0\0state\0sigUnlocked\0"
    "sigLocked\0sigCloseWindow\0slotlockVault\0"
    "createVault\0QString&\0password\0unlockVault\0"
    "lockVault\0isForced\0defaultCdAction\0"
    "windowId\0url\0openNewWindow\0createVaultDialog\0"
    "unlockVaultDialog\0showRemoveVaultDialog\0"
    "openWindow\0openWidWindow\0winID\0"
    "newOpenWindow"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_vault__VaultHelper[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   94,    2, 0x06 /* Public */,
       4,    1,   97,    2, 0x06 /* Public */,
       5,    1,  100,    2, 0x06 /* Public */,
       6,    0,  103,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,  104,    2, 0x0a /* Public */,
       8,    1,  107,    2, 0x0a /* Public */,
      11,    1,  110,    2, 0x0a /* Public */,
      12,    1,  113,    2, 0x0a /* Public */,
      14,    2,  116,    2, 0x0a /* Public */,
      17,    1,  121,    2, 0x0a /* Public */,
      18,    0,  124,    2, 0x0a /* Public */,
      19,    0,  125,    2, 0x0a /* Public */,
      20,    0,  126,    2, 0x0a /* Public */,
      21,    0,  127,    2, 0x0a /* Public */,
      22,    2,  128,    2, 0x0a /* Public */,
      24,    0,  133,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Bool, QMetaType::QString,   10,
    QMetaType::Bool, QMetaType::Bool,   13,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,   15,   16,
    QMetaType::Void, QMetaType::QUrl,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,   23,   16,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_vault::VaultHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<VaultHelper *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sigCreateVault((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->sigUnlocked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->sigLocked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->sigCloseWindow(); break;
        case 4: _t->slotlockVault((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->createVault((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 6: { bool _r = _t->unlockVault((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->lockVault((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->defaultCdAction((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 9: _t->openNewWindow((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 10: _t->createVaultDialog(); break;
        case 11: _t->unlockVaultDialog(); break;
        case 12: _t->showRemoveVaultDialog(); break;
        case 13: _t->openWindow(); break;
        case 14: _t->openWidWindow((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 15: _t->newOpenWindow(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (VaultHelper::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VaultHelper::sigCreateVault)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (VaultHelper::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VaultHelper::sigUnlocked)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (VaultHelper::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VaultHelper::sigLocked)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (VaultHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VaultHelper::sigCloseWindow)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_vault::VaultHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_vault__VaultHelper.data,
    qt_meta_data_dfmplugin_vault__VaultHelper,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_vault::VaultHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_vault::VaultHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_vault__VaultHelper.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_vault::VaultHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_vault::VaultHelper::sigCreateVault(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_vault::VaultHelper::sigUnlocked(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_vault::VaultHelper::sigLocked(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_vault::VaultHelper::sigCloseWindow()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
