/****************************************************************************
** Meta object code from reading C++ file 'detailspace.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-detailspace/detailspace.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'detailspace.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_detailspace__DetailSpace_t {
    QByteArrayData data[4];
    char stringdata0[58];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_detailspace__DetailSpace_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_detailspace__DetailSpace_t qt_meta_stringdata_dfmplugin_detailspace__DetailSpace = {
    {
QT_MOC_LITERAL(0, 0, 34), // "dfmplugin_detailspace::Detail..."
QT_MOC_LITERAL(1, 35, 14), // "onWindowClosed"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 6) // "windId"

    },
    "dfmplugin_detailspace::DetailSpace\0"
    "onWindowClosed\0\0windId"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_detailspace__DetailSpace[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::ULongLong,    3,

       0        // eod
};

void dfmplugin_detailspace::DetailSpace::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DetailSpace *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onWindowClosed((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_detailspace::DetailSpace::staticMetaObject = { {
    QMetaObject::SuperData::link<dpf::Plugin::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_detailspace__DetailSpace.data,
    qt_meta_data_dfmplugin_detailspace__DetailSpace,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_detailspace::DetailSpace::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_detailspace::DetailSpace::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_detailspace__DetailSpace.stringdata0))
        return static_cast<void*>(this);
    return dpf::Plugin::qt_metacast(_clname);
}

int dfmplugin_detailspace::DetailSpace::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = dpf::Plugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

QT_PLUGIN_METADATA_SECTION
static constexpr unsigned char qt_pluginMetaData[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'o',  'r',  'g',  '.',  'd', 
    'e',  'e',  'p',  'i',  'n',  '.',  'p',  'l', 
    'u',  'g',  'i',  'n',  '.',  'f',  'i',  'l', 
    'e',  'm',  'a',  'n',  'a',  'g',  'e',  'r', 
    // "className"
    0x03,  0x6b,  'D',  'e',  't',  'a',  'i',  'l', 
    'S',  'p',  'a',  'c',  'e', 
    // "MetaData"
    0x04,  0xaa,  0x68,  'C',  'a',  't',  'e',  'g', 
    'o',  'r',  'y',  0x60,  0x6d,  'C',  'o',  'm', 
    'p',  'a',  't',  'V',  'e',  'r',  's',  'i', 
    'o',  'n',  0x65,  '1',  '.',  '0',  '.',  '0', 
    0x69,  'C',  'o',  'p',  'y',  'r',  'i',  'g', 
    'h',  't',  0x78,  0x3a,  'C',  'o',  'p',  'y', 
    'r',  'i',  'g',  'h',  't',  ' ',  '(',  'C', 
    ')',  ' ',  '2',  '0',  '2',  '1',  ' ',  'U', 
    'n',  'i',  'o',  'n',  't',  'e',  'c',  'h', 
    ' ',  'S',  'o',  'f',  't',  'w',  'a',  'r', 
    'e',  ' ',  'T',  'e',  'c',  'h',  'n',  'o', 
    'l',  'o',  'g',  'y',  ' ',  'C',  'o',  '.', 
    ',',  ' ',  'L',  't',  'd',  '.',  0x67,  'D', 
    'e',  'p',  'e',  'n',  'd',  's',  0x81,  0xa2, 
    0x64,  'N',  'a',  'm',  'e',  0x6e,  'd',  'f', 
    'm',  'p',  'l',  'u',  'g',  'i',  'n',  '-', 
    'c',  'o',  'r',  'e',  0x67,  'V',  'e',  'r', 
    's',  'i',  'o',  'n',  0x65,  '1',  '.',  '0', 
    '.',  '0',  0x6b,  'D',  'e',  's',  'c',  'r', 
    'i',  'p',  't',  'i',  'o',  'n',  0x78,  0x2b, 
    'T',  'h',  'e',  ' ',  'd',  'e',  't',  'a', 
    'i',  'l',  's',  'p',  'a',  'c',  'e',  ' ', 
    'p',  'l',  'u',  'g',  'i',  'n',  ' ',  'f', 
    'o',  'r',  ' ',  't',  'h',  'e',  ' ',  'f', 
    'i',  'l',  'e',  'm',  'a',  'n',  'a',  'g', 
    'e',  'r',  '.',  0x67,  'L',  'i',  'c',  'e', 
    'n',  's',  'e',  0x80,  0x64,  'N',  'a',  'm', 
    'e',  0x75,  'd',  'f',  'm',  'p',  'l',  'u', 
    'g',  'i',  'n',  '-',  'd',  'e',  't',  'a', 
    'i',  'l',  's',  'p',  'a',  'c',  'e',  0x67, 
    'U',  'r',  'l',  'L',  'i',  'n',  'k',  0x78, 
    0x19,  'h',  't',  't',  'p',  's',  ':',  '/', 
    '/',  'w',  'w',  'w',  '.',  'u',  'n',  'i', 
    'o',  'n',  't',  'e',  'c',  'h',  '.',  'c', 
    'o',  'm',  0x66,  'V',  'e',  'n',  'd',  'o', 
    'r',  0x78,  0x2b,  'T',  'h',  'e',  ' ',  'U', 
    'n',  'i',  'o',  'n',  't',  'e',  'c',  'h', 
    ' ',  'S',  'o',  'f',  't',  'w',  'a',  'r', 
    'e',  ' ',  'T',  'e',  'c',  'h',  'n',  'o', 
    'l',  'o',  'g',  'y',  ' ',  'C',  'o',  '.', 
    ',',  ' ',  'L',  't',  'd',  '.',  0x67,  'V', 
    'e',  'r',  's',  'i',  'o',  'n',  0x65,  '1', 
    '.',  '0',  '.',  '0', 
    0xff, 
};
using namespace dfmplugin_detailspace;
QT_MOC_EXPORT_PLUGIN(dfmplugin_detailspace::DetailSpace, DetailSpace)

QT_WARNING_POP
QT_END_MOC_NAMESPACE
