// SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "workerdata.h"

DPFILEOPERATIONS_USE_NAMESPACE
WorkerData::WorkerData()
{
}
