/****************************************************************************
** Meta object code from reading C++ file 'appearance_interface.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-background/appearance_interface.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'appearance_interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_background__Appearance_Interface_t {
    QByteArrayData data[54];
    char stringdata0[819];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_background__Appearance_Interface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_background__Appearance_Interface_t qt_meta_stringdata_ddplugin_background__Appearance_Interface = {
    {
QT_MOC_LITERAL(0, 0, 41), // "ddplugin_background::Appearan..."
QT_MOC_LITERAL(1, 42, 7), // "Changed"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 2), // "ty"
QT_MOC_LITERAL(4, 54, 5), // "value"
QT_MOC_LITERAL(5, 60, 9), // "Refreshed"
QT_MOC_LITERAL(6, 70, 4), // "type"
QT_MOC_LITERAL(7, 75, 6), // "Delete"
QT_MOC_LITERAL(8, 82, 19), // "QDBusPendingReply<>"
QT_MOC_LITERAL(9, 102, 4), // "name"
QT_MOC_LITERAL(10, 107, 29), // "GetCurrentWorkspaceBackground"
QT_MOC_LITERAL(11, 137, 26), // "QDBusPendingReply<QString>"
QT_MOC_LITERAL(12, 164, 39), // "GetCurrentWorkspaceBackground..."
QT_MOC_LITERAL(13, 204, 14), // "strMonitorName"
QT_MOC_LITERAL(14, 219, 14), // "GetScaleFactor"
QT_MOC_LITERAL(15, 234, 25), // "QDBusPendingReply<double>"
QT_MOC_LITERAL(16, 260, 21), // "GetScreenScaleFactors"
QT_MOC_LITERAL(17, 282, 31), // "QDBusPendingReply<ScaleFactors>"
QT_MOC_LITERAL(18, 314, 21), // "GetWallpaperSlideShow"
QT_MOC_LITERAL(19, 336, 11), // "monitorName"
QT_MOC_LITERAL(20, 348, 32), // "GetWorkspaceBackgroundForMonitor"
QT_MOC_LITERAL(21, 381, 5), // "index"
QT_MOC_LITERAL(22, 387, 4), // "List"
QT_MOC_LITERAL(23, 392, 5), // "Reset"
QT_MOC_LITERAL(24, 398, 3), // "Set"
QT_MOC_LITERAL(25, 402, 29), // "SetCurrentWorkspaceBackground"
QT_MOC_LITERAL(26, 432, 3), // "uri"
QT_MOC_LITERAL(27, 436, 39), // "SetCurrentWorkspaceBackground..."
QT_MOC_LITERAL(28, 476, 20), // "SetMonitorBackground"
QT_MOC_LITERAL(29, 497, 9), // "imageGile"
QT_MOC_LITERAL(30, 507, 14), // "SetScaleFactor"
QT_MOC_LITERAL(31, 522, 5), // "scale"
QT_MOC_LITERAL(32, 528, 21), // "SetScreenScaleFactors"
QT_MOC_LITERAL(33, 550, 12), // "ScaleFactors"
QT_MOC_LITERAL(34, 563, 11), // "scaleFactor"
QT_MOC_LITERAL(35, 575, 21), // "SetWallpaperSlideShow"
QT_MOC_LITERAL(36, 597, 9), // "slideShow"
QT_MOC_LITERAL(37, 607, 32), // "SetWorkspaceBackgroundForMonitor"
QT_MOC_LITERAL(38, 640, 4), // "Show"
QT_MOC_LITERAL(39, 645, 5), // "names"
QT_MOC_LITERAL(40, 651, 9), // "Thumbnail"
QT_MOC_LITERAL(41, 661, 10), // "Background"
QT_MOC_LITERAL(42, 672, 11), // "CursorTheme"
QT_MOC_LITERAL(43, 684, 8), // "FontSize"
QT_MOC_LITERAL(44, 693, 11), // "GlobalTheme"
QT_MOC_LITERAL(45, 705, 8), // "GtkTheme"
QT_MOC_LITERAL(46, 714, 9), // "IconTheme"
QT_MOC_LITERAL(47, 724, 13), // "MonospaceFont"
QT_MOC_LITERAL(48, 738, 7), // "Opacity"
QT_MOC_LITERAL(49, 746, 13), // "QtActiveColor"
QT_MOC_LITERAL(50, 760, 12), // "StandardFont"
QT_MOC_LITERAL(51, 773, 18), // "WallpaperSlideShow"
QT_MOC_LITERAL(52, 792, 13), // "WallpaperURls"
QT_MOC_LITERAL(53, 806, 12) // "WindowRadius"

    },
    "ddplugin_background::Appearance_Interface\0"
    "Changed\0\0ty\0value\0Refreshed\0type\0"
    "Delete\0QDBusPendingReply<>\0name\0"
    "GetCurrentWorkspaceBackground\0"
    "QDBusPendingReply<QString>\0"
    "GetCurrentWorkspaceBackgroundForMonitor\0"
    "strMonitorName\0GetScaleFactor\0"
    "QDBusPendingReply<double>\0"
    "GetScreenScaleFactors\0"
    "QDBusPendingReply<ScaleFactors>\0"
    "GetWallpaperSlideShow\0monitorName\0"
    "GetWorkspaceBackgroundForMonitor\0index\0"
    "List\0Reset\0Set\0SetCurrentWorkspaceBackground\0"
    "uri\0SetCurrentWorkspaceBackgroundForMonitor\0"
    "SetMonitorBackground\0imageGile\0"
    "SetScaleFactor\0scale\0SetScreenScaleFactors\0"
    "ScaleFactors\0scaleFactor\0SetWallpaperSlideShow\0"
    "slideShow\0SetWorkspaceBackgroundForMonitor\0"
    "Show\0names\0Thumbnail\0Background\0"
    "CursorTheme\0FontSize\0GlobalTheme\0"
    "GtkTheme\0IconTheme\0MonospaceFont\0"
    "Opacity\0QtActiveColor\0StandardFont\0"
    "WallpaperSlideShow\0WallpaperURls\0"
    "WindowRadius"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_background__Appearance_Interface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
      13,  196, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,  119,    2, 0x06 /* Public */,
       5,    1,  124,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    2,  127,    2, 0x0a /* Public */,
      10,    0,  132,    2, 0x0a /* Public */,
      12,    1,  133,    2, 0x0a /* Public */,
      14,    0,  136,    2, 0x0a /* Public */,
      16,    0,  137,    2, 0x0a /* Public */,
      18,    1,  138,    2, 0x0a /* Public */,
      20,    2,  141,    2, 0x0a /* Public */,
      22,    1,  146,    2, 0x0a /* Public */,
      23,    0,  149,    2, 0x0a /* Public */,
      24,    2,  150,    2, 0x0a /* Public */,
      25,    1,  155,    2, 0x0a /* Public */,
      27,    2,  158,    2, 0x0a /* Public */,
      28,    2,  163,    2, 0x0a /* Public */,
      30,    1,  168,    2, 0x0a /* Public */,
      32,    1,  171,    2, 0x0a /* Public */,
      35,    2,  174,    2, 0x0a /* Public */,
      37,    3,  179,    2, 0x0a /* Public */,
      38,    2,  186,    2, 0x0a /* Public */,
      40,    2,  191,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    4,
    QMetaType::Void, QMetaType::QString,    6,

 // slots: parameters
    0x80000000 | 8, QMetaType::QString, QMetaType::QString,    3,    9,
    0x80000000 | 11,
    0x80000000 | 11, QMetaType::QString,   13,
    0x80000000 | 15,
    0x80000000 | 17,
    0x80000000 | 11, QMetaType::QString,   19,
    0x80000000 | 11, QMetaType::Int, QMetaType::QString,   21,   13,
    0x80000000 | 11, QMetaType::QString,    3,
    0x80000000 | 8,
    0x80000000 | 8, QMetaType::QString, QMetaType::QString,    3,    4,
    0x80000000 | 8, QMetaType::QString,   26,
    0x80000000 | 8, QMetaType::QString, QMetaType::QString,   26,   13,
    0x80000000 | 8, QMetaType::QString, QMetaType::QString,   19,   29,
    0x80000000 | 8, QMetaType::Double,   31,
    0x80000000 | 8, 0x80000000 | 33,   34,
    0x80000000 | 8, QMetaType::QString, QMetaType::QString,   19,   36,
    0x80000000 | 8, QMetaType::Int, QMetaType::QString, QMetaType::QString,   21,   13,   26,
    0x80000000 | 11, QMetaType::QString, QMetaType::QStringList,    3,   39,
    0x80000000 | 11, QMetaType::QString, QMetaType::QString,    3,    9,

 // properties: name, type, flags
      41, QMetaType::QString, 0x00095001,
      42, QMetaType::QString, 0x00095001,
      43, QMetaType::Double, 0x00095103,
      44, QMetaType::QString, 0x00095001,
      45, QMetaType::QString, 0x00095001,
      46, QMetaType::QString, 0x00095001,
      47, QMetaType::QString, 0x00095001,
      48, QMetaType::Double, 0x00095103,
      49, QMetaType::QString, 0x00095103,
      50, QMetaType::QString, 0x00095001,
      51, QMetaType::QString, 0x00095103,
      52, QMetaType::QString, 0x00095001,
      53, QMetaType::Int, 0x00095103,

       0        // eod
};

void ddplugin_background::Appearance_Interface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Appearance_Interface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->Changed((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->Refreshed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: { QDBusPendingReply<> _r = _t->Delete((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<QString> _r = _t->GetCurrentWorkspaceBackground();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<QString> _r = _t->GetCurrentWorkspaceBackgroundForMonitor((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<double> _r = _t->GetScaleFactor();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<double>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<ScaleFactors> _r = _t->GetScreenScaleFactors();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<ScaleFactors>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<QString> _r = _t->GetWallpaperSlideShow((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<QString> _r = _t->GetWorkspaceBackgroundForMonitor((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<QString> _r = _t->List((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<> _r = _t->Reset();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<> _r = _t->Set((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<> _r = _t->SetCurrentWorkspaceBackground((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusPendingReply<> _r = _t->SetCurrentWorkspaceBackgroundForMonitor((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<> _r = _t->SetMonitorBackground((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<> _r = _t->SetScaleFactor((*reinterpret_cast< double(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 16: { QDBusPendingReply<> _r = _t->SetScreenScaleFactors((*reinterpret_cast< ScaleFactors(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 17: { QDBusPendingReply<> _r = _t->SetWallpaperSlideShow((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 18: { QDBusPendingReply<> _r = _t->SetWorkspaceBackgroundForMonitor((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 19: { QDBusPendingReply<QString> _r = _t->Show((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 20: { QDBusPendingReply<QString> _r = _t->Thumbnail((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Appearance_Interface::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Appearance_Interface::Changed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Appearance_Interface::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Appearance_Interface::Refreshed)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Appearance_Interface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->background(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->cursorTheme(); break;
        case 2: *reinterpret_cast< double*>(_v) = _t->fontSize(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->globalTheme(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->gtkTheme(); break;
        case 5: *reinterpret_cast< QString*>(_v) = _t->iconTheme(); break;
        case 6: *reinterpret_cast< QString*>(_v) = _t->monospaceFont(); break;
        case 7: *reinterpret_cast< double*>(_v) = _t->opacity(); break;
        case 8: *reinterpret_cast< QString*>(_v) = _t->qtActiveColor(); break;
        case 9: *reinterpret_cast< QString*>(_v) = _t->standardFont(); break;
        case 10: *reinterpret_cast< QString*>(_v) = _t->wallpaperSlideShow(); break;
        case 11: *reinterpret_cast< QString*>(_v) = _t->wallpaperURls(); break;
        case 12: *reinterpret_cast< int*>(_v) = _t->windowRadius(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Appearance_Interface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setFontSize(*reinterpret_cast< double*>(_v)); break;
        case 7: _t->setOpacity(*reinterpret_cast< double*>(_v)); break;
        case 8: _t->setQtActiveColor(*reinterpret_cast< QString*>(_v)); break;
        case 10: _t->setWallpaperSlideShow(*reinterpret_cast< QString*>(_v)); break;
        case 12: _t->setWindowRadius(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_background::Appearance_Interface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_background__Appearance_Interface.data,
    qt_meta_data_ddplugin_background__Appearance_Interface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_background::Appearance_Interface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_background::Appearance_Interface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_background__Appearance_Interface.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int ddplugin_background::Appearance_Interface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 13;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void ddplugin_background::Appearance_Interface::Changed(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ddplugin_background::Appearance_Interface::Refreshed(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
