/****************************************************************************
** Meta object code from reading C++ file 'editstackedwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-propertydialog/views/editstackedwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'editstackedwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_propertydialog__NameTextEdit_t {
    QByteArrayData data[8];
    char stringdata0[113];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_propertydialog__NameTextEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_propertydialog__NameTextEdit_t qt_meta_stringdata_dfmplugin_propertydialog__NameTextEdit = {
    {
QT_MOC_LITERAL(0, 0, 38), // "dfmplugin_propertydialog::Nam..."
QT_MOC_LITERAL(1, 39, 12), // "editFinished"
QT_MOC_LITERAL(2, 52, 0), // ""
QT_MOC_LITERAL(3, 53, 12), // "setPlainText"
QT_MOC_LITERAL(4, 66, 4), // "text"
QT_MOC_LITERAL(5, 71, 15), // "slotTextChanged"
QT_MOC_LITERAL(6, 87, 16), // "showAlertMessage"
QT_MOC_LITERAL(7, 104, 8) // "duration"

    },
    "dfmplugin_propertydialog::NameTextEdit\0"
    "editFinished\0\0setPlainText\0text\0"
    "slotTextChanged\0showAlertMessage\0"
    "duration"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_propertydialog__NameTextEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   40,    2, 0x0a /* Public */,
       5,    0,   43,    2, 0x0a /* Public */,
       6,    2,   44,    2, 0x0a /* Public */,
       6,    1,   49,    2, 0x2a /* Public | MethodCloned */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    4,    7,
    QMetaType::Void, QMetaType::QString,    4,

       0        // eod
};

void dfmplugin_propertydialog::NameTextEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NameTextEdit *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editFinished(); break;
        case 1: _t->setPlainText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->slotTextChanged(); break;
        case 3: _t->showAlertMessage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->showAlertMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (NameTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NameTextEdit::editFinished)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_propertydialog::NameTextEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<Dtk::Widget::DTextEdit::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_propertydialog__NameTextEdit.data,
    qt_meta_data_dfmplugin_propertydialog__NameTextEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_propertydialog::NameTextEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_propertydialog::NameTextEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_propertydialog__NameTextEdit.stringdata0))
        return static_cast<void*>(this);
    return Dtk::Widget::DTextEdit::qt_metacast(_clname);
}

int dfmplugin_propertydialog::NameTextEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Dtk::Widget::DTextEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_propertydialog::NameTextEdit::editFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_dfmplugin_propertydialog__EditStackedWidget_t {
    QByteArrayData data[10];
    char stringdata0[138];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_propertydialog__EditStackedWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_propertydialog__EditStackedWidget_t qt_meta_stringdata_dfmplugin_propertydialog__EditStackedWidget = {
    {
QT_MOC_LITERAL(0, 0, 43), // "dfmplugin_propertydialog::Edi..."
QT_MOC_LITERAL(1, 44, 16), // "selectUrlRenamed"
QT_MOC_LITERAL(2, 61, 0), // ""
QT_MOC_LITERAL(3, 62, 3), // "url"
QT_MOC_LITERAL(4, 66, 10), // "renameFile"
QT_MOC_LITERAL(5, 77, 17), // "showTextShowFrame"
QT_MOC_LITERAL(6, 95, 10), // "selectFile"
QT_MOC_LITERAL(7, 106, 12), // "mouseProcess"
QT_MOC_LITERAL(8, 119, 12), // "QMouseEvent*"
QT_MOC_LITERAL(9, 132, 5) // "event"

    },
    "dfmplugin_propertydialog::EditStackedWidget\0"
    "selectUrlRenamed\0\0url\0renameFile\0"
    "showTextShowFrame\0selectFile\0mouseProcess\0"
    "QMouseEvent*\0event"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_propertydialog__EditStackedWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   42,    2, 0x0a /* Public */,
       5,    0,   43,    2, 0x0a /* Public */,
       6,    1,   44,    2, 0x0a /* Public */,
       7,    1,   47,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,    3,
    QMetaType::Void, 0x80000000 | 8,    9,

       0        // eod
};

void dfmplugin_propertydialog::EditStackedWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EditStackedWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->selectUrlRenamed((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 1: _t->renameFile(); break;
        case 2: _t->showTextShowFrame(); break;
        case 3: _t->selectFile((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 4: _t->mouseProcess((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (EditStackedWidget::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditStackedWidget::selectUrlRenamed)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_propertydialog::EditStackedWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QStackedWidget::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_propertydialog__EditStackedWidget.data,
    qt_meta_data_dfmplugin_propertydialog__EditStackedWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_propertydialog::EditStackedWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_propertydialog::EditStackedWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_propertydialog__EditStackedWidget.stringdata0))
        return static_cast<void*>(this);
    return QStackedWidget::qt_metacast(_clname);
}

int dfmplugin_propertydialog::EditStackedWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStackedWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_propertydialog::EditStackedWidget::selectUrlRenamed(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
