/****************************************************************************
** Meta object code from reading C++ file 'custommode.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-organizer/mode/custommode.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'custommode.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_organizer__CustomMode_t {
    QByteArrayData data[34];
    char stringdata0[404];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_organizer__CustomMode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_organizer__CustomMode_t qt_meta_stringdata_ddplugin_organizer__CustomMode = {
    {
QT_MOC_LITERAL(0, 0, 30), // "ddplugin_organizer::CustomMode"
QT_MOC_LITERAL(1, 31, 7), // "rebuild"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 13), // "onFileRenamed"
QT_MOC_LITERAL(4, 54, 6), // "oldUrl"
QT_MOC_LITERAL(5, 61, 6), // "newUrl"
QT_MOC_LITERAL(6, 68, 14), // "onFileInserted"
QT_MOC_LITERAL(7, 83, 11), // "QModelIndex"
QT_MOC_LITERAL(8, 95, 6), // "parent"
QT_MOC_LITERAL(9, 102, 5), // "first"
QT_MOC_LITERAL(10, 108, 4), // "last"
QT_MOC_LITERAL(11, 113, 22), // "onFileAboutToBeRemoved"
QT_MOC_LITERAL(12, 136, 17), // "onFileDataChanged"
QT_MOC_LITERAL(13, 154, 7), // "topLeft"
QT_MOC_LITERAL(14, 162, 11), // "bottomRight"
QT_MOC_LITERAL(15, 174, 12), // "QVector<int>"
QT_MOC_LITERAL(16, 187, 5), // "roles"
QT_MOC_LITERAL(17, 193, 16), // "filterDataRested"
QT_MOC_LITERAL(18, 210, 12), // "QList<QUrl>*"
QT_MOC_LITERAL(19, 223, 4), // "urls"
QT_MOC_LITERAL(20, 228, 18), // "filterDataInserted"
QT_MOC_LITERAL(21, 247, 3), // "url"
QT_MOC_LITERAL(22, 251, 17), // "filterDataRenamed"
QT_MOC_LITERAL(23, 269, 14), // "filterDropData"
QT_MOC_LITERAL(24, 284, 9), // "viewIndex"
QT_MOC_LITERAL(25, 294, 16), // "const QMimeData*"
QT_MOC_LITERAL(26, 311, 8), // "mimeData"
QT_MOC_LITERAL(27, 320, 9), // "viewPoint"
QT_MOC_LITERAL(28, 330, 7), // "extData"
QT_MOC_LITERAL(29, 338, 15), // "onNewCollection"
QT_MOC_LITERAL(30, 354, 11), // "QList<QUrl>"
QT_MOC_LITERAL(31, 366, 18), // "onDeleteCollection"
QT_MOC_LITERAL(32, 385, 3), // "key"
QT_MOC_LITERAL(33, 389, 14) // "onItemsChanged"

    },
    "ddplugin_organizer::CustomMode\0rebuild\0"
    "\0onFileRenamed\0oldUrl\0newUrl\0"
    "onFileInserted\0QModelIndex\0parent\0"
    "first\0last\0onFileAboutToBeRemoved\0"
    "onFileDataChanged\0topLeft\0bottomRight\0"
    "QVector<int>\0roles\0filterDataRested\0"
    "QList<QUrl>*\0urls\0filterDataInserted\0"
    "url\0filterDataRenamed\0filterDropData\0"
    "viewIndex\0const QMimeData*\0mimeData\0"
    "viewPoint\0extData\0onNewCollection\0"
    "QList<QUrl>\0onDeleteCollection\0key\0"
    "onItemsChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_organizer__CustomMode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x0a /* Public */,
       3,    2,   75,    2, 0x0a /* Public */,
       6,    3,   80,    2, 0x0a /* Public */,
      11,    3,   87,    2, 0x0a /* Public */,
      12,    3,   94,    2, 0x0a /* Public */,
      17,    1,  101,    2, 0x09 /* Protected */,
      20,    1,  104,    2, 0x09 /* Protected */,
      22,    2,  107,    2, 0x09 /* Protected */,
      23,    4,  112,    2, 0x09 /* Protected */,
      29,    1,  121,    2, 0x08 /* Private */,
      31,    1,  124,    2, 0x08 /* Private */,
      33,    0,  127,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,    4,    5,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Int, QMetaType::Int,    8,    9,   10,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Int, QMetaType::Int,    8,    9,   10,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 7, 0x80000000 | 15,   13,   14,   16,
    QMetaType::Bool, 0x80000000 | 18,   19,
    QMetaType::Bool, QMetaType::QUrl,   21,
    QMetaType::Bool, QMetaType::QUrl, QMetaType::QUrl,    4,    5,
    QMetaType::Bool, QMetaType::Int, 0x80000000 | 25, QMetaType::QPoint, QMetaType::VoidStar,   24,   26,   27,   28,
    QMetaType::Void, 0x80000000 | 30,    2,
    QMetaType::Void, QMetaType::QString,   32,
    QMetaType::Void,

       0        // eod
};

void ddplugin_organizer::CustomMode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CustomMode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->rebuild(); break;
        case 1: _t->onFileRenamed((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 2: _t->onFileInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 3: _t->onFileAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 4: _t->onFileDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2])),(*reinterpret_cast< const QVector<int>(*)>(_a[3]))); break;
        case 5: { bool _r = _t->filterDataRested((*reinterpret_cast< QList<QUrl>*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->filterDataInserted((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->filterDataRenamed((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->filterDropData((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QMimeData*(*)>(_a[2])),(*reinterpret_cast< const QPoint(*)>(_a[3])),(*reinterpret_cast< void*(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->onNewCollection((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 10: _t->onDeleteCollection((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->onItemsChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<int> >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_organizer::CustomMode::staticMetaObject = { {
    QMetaObject::SuperData::link<CanvasOrganizer::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_organizer__CustomMode.data,
    qt_meta_data_ddplugin_organizer__CustomMode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_organizer::CustomMode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_organizer::CustomMode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_organizer__CustomMode.stringdata0))
        return static_cast<void*>(this);
    return CanvasOrganizer::qt_metacast(_clname);
}

int ddplugin_organizer::CustomMode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CanvasOrganizer::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
