/****************************************************************************
** Meta object code from reading C++ file 'tagdaemon.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/daemon/daemonplugin-tag/tagdaemon.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tagdaemon.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_daemonplugin_tag__TagDBusWorker_t {
    QByteArrayData data[3];
    char stringdata0[47];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_daemonplugin_tag__TagDBusWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_daemonplugin_tag__TagDBusWorker_t qt_meta_stringdata_daemonplugin_tag__TagDBusWorker = {
    {
QT_MOC_LITERAL(0, 0, 31), // "daemonplugin_tag::TagDBusWorker"
QT_MOC_LITERAL(1, 32, 13), // "launchService"
QT_MOC_LITERAL(2, 46, 0) // ""

    },
    "daemonplugin_tag::TagDBusWorker\0"
    "launchService\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_daemonplugin_tag__TagDBusWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void daemonplugin_tag::TagDBusWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TagDBusWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->launchService(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject daemonplugin_tag::TagDBusWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_daemonplugin_tag__TagDBusWorker.data,
    qt_meta_data_daemonplugin_tag__TagDBusWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *daemonplugin_tag::TagDBusWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *daemonplugin_tag::TagDBusWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_daemonplugin_tag__TagDBusWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int daemonplugin_tag::TagDBusWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_daemonplugin_tag__TagDaemon_t {
    QByteArrayData data[3];
    char stringdata0[43];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_daemonplugin_tag__TagDaemon_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_daemonplugin_tag__TagDaemon_t qt_meta_stringdata_daemonplugin_tag__TagDaemon = {
    {
QT_MOC_LITERAL(0, 0, 27), // "daemonplugin_tag::TagDaemon"
QT_MOC_LITERAL(1, 28, 13), // "requestLaunch"
QT_MOC_LITERAL(2, 42, 0) // ""

    },
    "daemonplugin_tag::TagDaemon\0requestLaunch\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_daemonplugin_tag__TagDaemon[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void daemonplugin_tag::TagDaemon::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TagDaemon *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestLaunch(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TagDaemon::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagDaemon::requestLaunch)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject daemonplugin_tag::TagDaemon::staticMetaObject = { {
    QMetaObject::SuperData::link<dpf::Plugin::staticMetaObject>(),
    qt_meta_stringdata_daemonplugin_tag__TagDaemon.data,
    qt_meta_data_daemonplugin_tag__TagDaemon,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *daemonplugin_tag::TagDaemon::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *daemonplugin_tag::TagDaemon::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_daemonplugin_tag__TagDaemon.stringdata0))
        return static_cast<void*>(this);
    return dpf::Plugin::qt_metacast(_clname);
}

int daemonplugin_tag::TagDaemon::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = dpf::Plugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void daemonplugin_tag::TagDaemon::requestLaunch()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

QT_PLUGIN_METADATA_SECTION
static constexpr unsigned char qt_pluginMetaData[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x18,  'o',  'r',  'g',  '.',  'd', 
    'e',  'e',  'p',  'i',  'n',  '.',  'p',  'l', 
    'u',  'g',  'i',  'n',  '.',  'd',  'a',  'e', 
    'm',  'o',  'n', 
    // "className"
    0x03,  0x69,  'T',  'a',  'g',  'D',  'a',  'e', 
    'm',  'o',  'n', 
    // "MetaData"
    0x04,  0xa4,  0x67,  'D',  'e',  'p',  'e',  'n', 
    'd',  's',  0x80,  0x6b,  'D',  'e',  's',  'c', 
    'r',  'i',  'p',  't',  'i',  'o',  'n',  0x78, 
    0x30,  'T',  'h',  'e',  ' ',  'c',  'o',  'r', 
    'e',  ' ',  'p',  'l',  'u',  'g',  'i',  'n', 
    ' ',  'f',  'o',  'r',  ' ',  't',  'h',  'e', 
    ' ',  'd',  'd',  'e',  '-',  'f',  'i',  'l', 
    'e',  '-',  'm',  'a',  'n',  'a',  'g',  'e', 
    'r',  '-',  'd',  'a',  'e',  'm',  'o',  'n', 
    '.',  0x64,  'N',  'a',  'm',  'e',  0x70,  'd', 
    'a',  'e',  'm',  'o',  'n',  'p',  'l',  'u', 
    'g',  'i',  'n',  '-',  't',  'a',  'g',  0x67, 
    'V',  'e',  'r',  's',  'i',  'o',  'n',  0x65, 
    '1',  '.',  '0',  '.',  '0', 
    0xff, 
};
using namespace daemonplugin_tag;
QT_MOC_EXPORT_PLUGIN(daemonplugin_tag::TagDaemon, TagDaemon)

QT_WARNING_POP
QT_END_MOC_NAMESPACE
