/****************************************************************************
** Meta object code from reading C++ file 'filedialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filedialog/filedialogplugin-core/views/filedialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filedialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_filedialog_core__FileDialog_t {
    QByteArrayData data[31];
    char stringdata0[430];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_filedialog_core__FileDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_filedialog_core__FileDialog_t qt_meta_stringdata_filedialog_core__FileDialog = {
    {
QT_MOC_LITERAL(0, 0, 27), // "filedialog_core::FileDialog"
QT_MOC_LITERAL(1, 28, 8), // "finished"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 6), // "result"
QT_MOC_LITERAL(4, 45, 8), // "accepted"
QT_MOC_LITERAL(5, 54, 8), // "rejected"
QT_MOC_LITERAL(6, 63, 21), // "selectionFilesChanged"
QT_MOC_LITERAL(7, 85, 25), // "selectedNameFilterChanged"
QT_MOC_LITERAL(8, 111, 11), // "initialized"
QT_MOC_LITERAL(9, 123, 6), // "accept"
QT_MOC_LITERAL(10, 130, 4), // "done"
QT_MOC_LITERAL(11, 135, 1), // "r"
QT_MOC_LITERAL(12, 137, 4), // "exec"
QT_MOC_LITERAL(13, 142, 4), // "open"
QT_MOC_LITERAL(14, 147, 6), // "reject"
QT_MOC_LITERAL(15, 154, 21), // "onAcceptButtonClicked"
QT_MOC_LITERAL(16, 176, 21), // "onRejectButtonClicked"
QT_MOC_LITERAL(17, 198, 25), // "onCurrentInputNameChanged"
QT_MOC_LITERAL(18, 224, 23), // "updateAcceptButtonState"
QT_MOC_LITERAL(19, 248, 18), // "handleEnterPressed"
QT_MOC_LITERAL(20, 267, 16), // "handleUrlChanged"
QT_MOC_LITERAL(21, 284, 3), // "url"
QT_MOC_LITERAL(22, 288, 22), // "onViewSelectionChanged"
QT_MOC_LITERAL(23, 311, 8), // "windowID"
QT_MOC_LITERAL(24, 320, 14), // "QItemSelection"
QT_MOC_LITERAL(25, 335, 8), // "selected"
QT_MOC_LITERAL(26, 344, 10), // "deselected"
QT_MOC_LITERAL(27, 355, 17), // "onViewItemClicked"
QT_MOC_LITERAL(28, 373, 4), // "data"
QT_MOC_LITERAL(29, 378, 26), // "handleRenameStartAcceptBtn"
QT_MOC_LITERAL(30, 405, 24) // "handleRenameEndAcceptBtn"

    },
    "filedialog_core::FileDialog\0finished\0"
    "\0result\0accepted\0rejected\0"
    "selectionFilesChanged\0selectedNameFilterChanged\0"
    "initialized\0accept\0done\0r\0exec\0open\0"
    "reject\0onAcceptButtonClicked\0"
    "onRejectButtonClicked\0onCurrentInputNameChanged\0"
    "updateAcceptButtonState\0handleEnterPressed\0"
    "handleUrlChanged\0url\0onViewSelectionChanged\0"
    "windowID\0QItemSelection\0selected\0"
    "deselected\0onViewItemClicked\0data\0"
    "handleRenameStartAcceptBtn\0"
    "handleRenameEndAcceptBtn"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_filedialog_core__FileDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  119,    2, 0x06 /* Public */,
       4,    0,  122,    2, 0x06 /* Public */,
       5,    0,  123,    2, 0x06 /* Public */,
       6,    0,  124,    2, 0x06 /* Public */,
       7,    0,  125,    2, 0x06 /* Public */,
       8,    0,  126,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,  127,    2, 0x0a /* Public */,
      10,    1,  128,    2, 0x0a /* Public */,
      12,    0,  131,    2, 0x0a /* Public */,
      13,    0,  132,    2, 0x0a /* Public */,
      14,    0,  133,    2, 0x0a /* Public */,
      15,    0,  134,    2, 0x0a /* Public */,
      16,    0,  135,    2, 0x08 /* Private */,
      17,    0,  136,    2, 0x08 /* Private */,
      18,    0,  137,    2, 0x08 /* Private */,
      19,    0,  138,    2, 0x08 /* Private */,
      20,    1,  139,    2, 0x08 /* Private */,
      22,    3,  142,    2, 0x08 /* Private */,
      27,    1,  149,    2, 0x08 /* Private */,
      29,    2,  152,    2, 0x08 /* Private */,
      30,    2,  157,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,   21,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 24, 0x80000000 | 24,   23,   25,   26,
    QMetaType::Void, QMetaType::QVariantMap,   28,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,   23,   21,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,   23,   21,

       0        // eod
};

void filedialog_core::FileDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->finished((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->accepted(); break;
        case 2: _t->rejected(); break;
        case 3: _t->selectionFilesChanged(); break;
        case 4: _t->selectedNameFilterChanged(); break;
        case 5: _t->initialized(); break;
        case 6: _t->accept(); break;
        case 7: _t->done((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: { int _r = _t->exec();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->open(); break;
        case 10: _t->reject(); break;
        case 11: _t->onAcceptButtonClicked(); break;
        case 12: _t->onRejectButtonClicked(); break;
        case 13: _t->onCurrentInputNameChanged(); break;
        case 14: _t->updateAcceptButtonState(); break;
        case 15: _t->handleEnterPressed(); break;
        case 16: _t->handleUrlChanged((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 17: _t->onViewSelectionChanged((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2])),(*reinterpret_cast< const QItemSelection(*)>(_a[3]))); break;
        case 18: _t->onViewItemClicked((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 19: _t->handleRenameStartAcceptBtn((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 20: _t->handleRenameEndAcceptBtn((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileDialog::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialog::finished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialog::accepted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialog::rejected)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FileDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialog::selectionFilesChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FileDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialog::selectedNameFilterChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (FileDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialog::initialized)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject filedialog_core::FileDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<dfmbase::FileManagerWindow::staticMetaObject>(),
    qt_meta_stringdata_filedialog_core__FileDialog.data,
    qt_meta_data_filedialog_core__FileDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *filedialog_core::FileDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *filedialog_core::FileDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_filedialog_core__FileDialog.stringdata0))
        return static_cast<void*>(this);
    return dfmbase::FileManagerWindow::qt_metacast(_clname);
}

int filedialog_core::FileDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = dfmbase::FileManagerWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void filedialog_core::FileDialog::finished(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void filedialog_core::FileDialog::accepted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void filedialog_core::FileDialog::rejected()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void filedialog_core::FileDialog::selectionFilesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void filedialog_core::FileDialog::selectedNameFilterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void filedialog_core::FileDialog::initialized()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
