/****************************************************************************
** Meta object code from reading C++ file 'vaultentryfileentity.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-vault/utils/vaultentryfileentity.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vaultentryfileentity.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_vault__VaultEntryFileEntity_t {
    QByteArrayData data[20];
    char stringdata0[283];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_vault__VaultEntryFileEntity_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_vault__VaultEntryFileEntity_t qt_meta_stringdata_dfmplugin_vault__VaultEntryFileEntity = {
    {
QT_MOC_LITERAL(0, 0, 37), // "dfmplugin_vault::VaultEntryFi..."
QT_MOC_LITERAL(1, 38, 21), // "slotFileDirSizeChange"
QT_MOC_LITERAL(2, 60, 0), // ""
QT_MOC_LITERAL(3, 61, 4), // "size"
QT_MOC_LITERAL(4, 66, 10), // "filesCount"
QT_MOC_LITERAL(5, 77, 14), // "directoryCount"
QT_MOC_LITERAL(6, 92, 18), // "slotFinishedThread"
QT_MOC_LITERAL(7, 111, 11), // "displayName"
QT_MOC_LITERAL(8, 123, 4), // "icon"
QT_MOC_LITERAL(9, 128, 6), // "exists"
QT_MOC_LITERAL(10, 135, 12), // "showProgress"
QT_MOC_LITERAL(11, 148, 13), // "showTotalSize"
QT_MOC_LITERAL(12, 162, 13), // "showUsageSize"
QT_MOC_LITERAL(13, 176, 5), // "order"
QT_MOC_LITERAL(14, 182, 44), // "dfmbase::AbstractEntryFileEnt..."
QT_MOC_LITERAL(15, 227, 7), // "refresh"
QT_MOC_LITERAL(16, 235, 9), // "sizeTotal"
QT_MOC_LITERAL(17, 245, 9), // "targetUrl"
QT_MOC_LITERAL(18, 255, 20), // "VaultEntryFileEntity"
QT_MOC_LITERAL(19, 276, 6) // "parent"

    },
    "dfmplugin_vault::VaultEntryFileEntity\0"
    "slotFileDirSizeChange\0\0size\0filesCount\0"
    "directoryCount\0slotFinishedThread\0"
    "displayName\0icon\0exists\0showProgress\0"
    "showTotalSize\0showUsageSize\0order\0"
    "dfmbase::AbstractEntryFileEntity::EntryOrder\0"
    "refresh\0sizeTotal\0targetUrl\0"
    "VaultEntryFileEntity\0parent"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_vault__VaultEntryFileEntity[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       2,   96, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   74,    2, 0x0a /* Public */,
       6,    0,   81,    2, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
       7,    0,   82,    2, 0x02 /* Public */,
       8,    0,   83,    2, 0x02 /* Public */,
       9,    0,   84,    2, 0x02 /* Public */,
      10,    0,   85,    2, 0x02 /* Public */,
      11,    0,   86,    2, 0x02 /* Public */,
      12,    0,   87,    2, 0x02 /* Public */,
      13,    0,   88,    2, 0x02 /* Public */,
      15,    0,   89,    2, 0x02 /* Public */,
      16,    0,   90,    2, 0x02 /* Public */,
      17,    0,   91,    2, 0x02 /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::LongLong, QMetaType::Int, QMetaType::Int,    3,    4,    5,
    QMetaType::Void,

 // methods: parameters
    QMetaType::QString,
    QMetaType::QIcon,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    0x80000000 | 14,
    QMetaType::Void,
    QMetaType::ULongLong,
    QMetaType::QUrl,

 // constructors: parameters
    0x80000000 | 2, QMetaType::QObjectStar,   19,
    0x80000000 | 2,

 // constructors: name, argc, parameters, tag, flags
      18,    1,   92,    2, 0x0e /* Public */,
      18,    0,   95,    2, 0x2e /* Public | MethodCloned */,

       0        // eod
};

void dfmplugin_vault::VaultEntryFileEntity::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::CreateInstance) {
        switch (_id) {
        case 0: { VaultEntryFileEntity *_r = new VaultEntryFileEntity((*reinterpret_cast< QObject*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 1: { VaultEntryFileEntity *_r = new VaultEntryFileEntity();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        default: break;
        }
    } else if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<VaultEntryFileEntity *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotFileDirSizeChange((*reinterpret_cast< qint64(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 1: _t->slotFinishedThread(); break;
        case 2: { QString _r = _t->displayName();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 3: { QIcon _r = _t->icon();
            if (_a[0]) *reinterpret_cast< QIcon*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->exists();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->showProgress();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->showTotalSize();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->showUsageSize();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { dfmbase::AbstractEntryFileEntity::EntryOrder _r = _t->order();
            if (_a[0]) *reinterpret_cast< dfmbase::AbstractEntryFileEntity::EntryOrder*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->refresh(); break;
        case 10: { quint64 _r = _t->sizeTotal();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 11: { QUrl _r = _t->targetUrl();
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_vault::VaultEntryFileEntity::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_vault__VaultEntryFileEntity.data,
    qt_meta_data_dfmplugin_vault__VaultEntryFileEntity,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_vault::VaultEntryFileEntity::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_vault::VaultEntryFileEntity::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_vault__VaultEntryFileEntity.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_vault::VaultEntryFileEntity::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
