/****************************************************************************
** Meta object code from reading C++ file 'filestatisticsjob.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../dde-file-manager/src/dfm-base/utils/filestatisticsjob.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filestatisticsjob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmbase__FileStatisticsJob_t {
    QByteArrayData data[32];
    char stringdata0[423];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmbase__FileStatisticsJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmbase__FileStatisticsJob_t qt_meta_stringdata_dfmbase__FileStatisticsJob = {
    {
QT_MOC_LITERAL(0, 0, 26), // "dfmbase::FileStatisticsJob"
QT_MOC_LITERAL(1, 27, 12), // "stateChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 5), // "State"
QT_MOC_LITERAL(4, 47, 5), // "state"
QT_MOC_LITERAL(5, 53, 11), // "sizeChanged"
QT_MOC_LITERAL(6, 65, 4), // "size"
QT_MOC_LITERAL(7, 70, 10), // "dataNotify"
QT_MOC_LITERAL(8, 81, 10), // "filesCount"
QT_MOC_LITERAL(9, 92, 14), // "directoryCount"
QT_MOC_LITERAL(10, 107, 5), // "start"
QT_MOC_LITERAL(11, 113, 11), // "QList<QUrl>"
QT_MOC_LITERAL(12, 125, 10), // "sourceUrls"
QT_MOC_LITERAL(13, 136, 4), // "stop"
QT_MOC_LITERAL(14, 141, 11), // "togglePause"
QT_MOC_LITERAL(15, 153, 12), // "setFileHints"
QT_MOC_LITERAL(16, 166, 9), // "FileHints"
QT_MOC_LITERAL(17, 176, 9), // "fileHints"
QT_MOC_LITERAL(18, 186, 13), // "kStoppedState"
QT_MOC_LITERAL(19, 200, 13), // "kRunningState"
QT_MOC_LITERAL(20, 214, 12), // "kPausedState"
QT_MOC_LITERAL(21, 227, 8), // "FileHint"
QT_MOC_LITERAL(22, 236, 7), // "kNoHint"
QT_MOC_LITERAL(23, 244, 16), // "kNoFollowSymlink"
QT_MOC_LITERAL(24, 261, 18), // "kExcludeSourceFile"
QT_MOC_LITERAL(25, 280, 12), // "kSingleDepth"
QT_MOC_LITERAL(26, 293, 21), // "kDontSkipAVFSDStorage"
QT_MOC_LITERAL(27, 315, 20), // "kDontSkipPROCStorage"
QT_MOC_LITERAL(28, 336, 23), // "kDontSkipCharDeviceFile"
QT_MOC_LITERAL(29, 360, 24), // "kDontSkipBlockDeviceFile"
QT_MOC_LITERAL(30, 385, 17), // "kDontSkipFIFOFile"
QT_MOC_LITERAL(31, 403, 19) // "kDontSkipSocketFile"

    },
    "dfmbase::FileStatisticsJob\0stateChanged\0"
    "\0State\0state\0sizeChanged\0size\0dataNotify\0"
    "filesCount\0directoryCount\0start\0"
    "QList<QUrl>\0sourceUrls\0stop\0togglePause\0"
    "setFileHints\0FileHints\0fileHints\0"
    "kStoppedState\0kRunningState\0kPausedState\0"
    "FileHint\0kNoHint\0kNoFollowSymlink\0"
    "kExcludeSourceFile\0kSingleDepth\0"
    "kDontSkipAVFSDStorage\0kDontSkipPROCStorage\0"
    "kDontSkipCharDeviceFile\0"
    "kDontSkipBlockDeviceFile\0kDontSkipFIFOFile\0"
    "kDontSkipSocketFile"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmbase__FileStatisticsJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       2,   70, // properties
       2,   78, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       5,    1,   52,    2, 0x06 /* Public */,
       7,    3,   55,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,   62,    2, 0x0a /* Public */,
      13,    0,   65,    2, 0x0a /* Public */,
      14,    0,   66,    2, 0x0a /* Public */,
      15,    1,   67,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::LongLong,    6,
    QMetaType::Void, QMetaType::LongLong, QMetaType::Int, QMetaType::Int,    6,    8,    9,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16,   17,

 // properties: name, type, flags
       4, 0x80000000 | 3, 0x00495009,
      17, 0x80000000 | 16, 0x0009510b,

 // properties: notify_signal_id
       0,
       0,

 // enums: name, alias, flags, count, data
       3,    3, 0x0,    3,   88,
      21,   21, 0x0,   10,   94,

 // enum data: key, value
      18, uint(dfmbase::FileStatisticsJob::kStoppedState),
      19, uint(dfmbase::FileStatisticsJob::kRunningState),
      20, uint(dfmbase::FileStatisticsJob::kPausedState),
      22, uint(dfmbase::FileStatisticsJob::kNoHint),
      23, uint(dfmbase::FileStatisticsJob::kNoFollowSymlink),
      24, uint(dfmbase::FileStatisticsJob::kExcludeSourceFile),
      25, uint(dfmbase::FileStatisticsJob::kSingleDepth),
      26, uint(dfmbase::FileStatisticsJob::kDontSkipAVFSDStorage),
      27, uint(dfmbase::FileStatisticsJob::kDontSkipPROCStorage),
      28, uint(dfmbase::FileStatisticsJob::kDontSkipCharDeviceFile),
      29, uint(dfmbase::FileStatisticsJob::kDontSkipBlockDeviceFile),
      30, uint(dfmbase::FileStatisticsJob::kDontSkipFIFOFile),
      31, uint(dfmbase::FileStatisticsJob::kDontSkipSocketFile),

       0        // eod
};

void dfmbase::FileStatisticsJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileStatisticsJob *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->stateChanged((*reinterpret_cast< State(*)>(_a[1]))); break;
        case 1: _t->sizeChanged((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 2: _t->dataNotify((*reinterpret_cast< qint64(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 3: _t->start((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 4: _t->stop(); break;
        case 5: _t->togglePause(); break;
        case 6: _t->setFileHints((*reinterpret_cast< FileHints(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileStatisticsJob::*)(State );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileStatisticsJob::stateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileStatisticsJob::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileStatisticsJob::sizeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileStatisticsJob::*)(qint64 , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileStatisticsJob::dataNotify)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<FileStatisticsJob *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< State*>(_v) = _t->state(); break;
        case 1: *reinterpret_cast< FileHints*>(_v) = _t->fileHints(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<FileStatisticsJob *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setFileHints(*reinterpret_cast< FileHints*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject dfmbase::FileStatisticsJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_dfmbase__FileStatisticsJob.data,
    qt_meta_data_dfmbase__FileStatisticsJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmbase::FileStatisticsJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmbase::FileStatisticsJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmbase__FileStatisticsJob.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int dfmbase::FileStatisticsJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void dfmbase::FileStatisticsJob::stateChanged(State _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmbase::FileStatisticsJob::sizeChanged(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmbase::FileStatisticsJob::dataNotify(qint64 _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
