/****************************************************************************
** Meta object code from reading C++ file 'collectionframe.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-organizer/view/collectionframe.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'collectionframe.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_organizer__CollectionFrame_t {
    QByteArrayData data[28];
    char stringdata0[524];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_organizer__CollectionFrame_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_organizer__CollectionFrame_t qt_meta_stringdata_ddplugin_organizer__CollectionFrame = {
    {
QT_MOC_LITERAL(0, 0, 35), // "ddplugin_organizer::Collectio..."
QT_MOC_LITERAL(1, 36, 15), // "sizeModeChanged"
QT_MOC_LITERAL(2, 52, 0), // ""
QT_MOC_LITERAL(3, 53, 19), // "CollectionFrameSize"
QT_MOC_LITERAL(4, 73, 4), // "size"
QT_MOC_LITERAL(5, 78, 15), // "geometryChanged"
QT_MOC_LITERAL(6, 94, 20), // "editingStatusChanged"
QT_MOC_LITERAL(7, 115, 7), // "editing"
QT_MOC_LITERAL(8, 123, 20), // "requestChangeSurface"
QT_MOC_LITERAL(9, 144, 16), // "cursorScreenName"
QT_MOC_LITERAL(10, 161, 13), // "oldScreenName"
QT_MOC_LITERAL(11, 175, 14), // "surfaceChanged"
QT_MOC_LITERAL(12, 190, 8), // "QWidget*"
QT_MOC_LITERAL(13, 199, 7), // "surface"
QT_MOC_LITERAL(14, 207, 28), // "requestDeactiveAllPredictors"
QT_MOC_LITERAL(15, 236, 16), // "moveStateChanged"
QT_MOC_LITERAL(16, 253, 6), // "moving"
QT_MOC_LITERAL(17, 260, 14), // "adjustSizeMode"
QT_MOC_LITERAL(18, 275, 23), // "CollectionFrameFeatures"
QT_MOC_LITERAL(19, 299, 22), // "CollectionFrameFeature"
QT_MOC_LITERAL(20, 322, 25), // "NoCollectionFrameFeatures"
QT_MOC_LITERAL(21, 348, 23), // "CollectionFrameClosable"
QT_MOC_LITERAL(22, 372, 22), // "CollectionFrameMovable"
QT_MOC_LITERAL(23, 395, 24), // "CollectionFrameFloatable"
QT_MOC_LITERAL(24, 420, 23), // "CollectionFrameHiddable"
QT_MOC_LITERAL(25, 444, 25), // "CollectionFrameAdjustable"
QT_MOC_LITERAL(26, 470, 26), // "CollectionFrameStretchable"
QT_MOC_LITERAL(27, 497, 26) // "CollectionGroupFeatureMask"

    },
    "ddplugin_organizer::CollectionFrame\0"
    "sizeModeChanged\0\0CollectionFrameSize\0"
    "size\0geometryChanged\0editingStatusChanged\0"
    "editing\0requestChangeSurface\0"
    "cursorScreenName\0oldScreenName\0"
    "surfaceChanged\0QWidget*\0surface\0"
    "requestDeactiveAllPredictors\0"
    "moveStateChanged\0moving\0adjustSizeMode\0"
    "CollectionFrameFeatures\0CollectionFrameFeature\0"
    "NoCollectionFrameFeatures\0"
    "CollectionFrameClosable\0CollectionFrameMovable\0"
    "CollectionFrameFloatable\0"
    "CollectionFrameHiddable\0"
    "CollectionFrameAdjustable\0"
    "CollectionFrameStretchable\0"
    "CollectionGroupFeatureMask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_organizer__CollectionFrame[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       1,   76, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       5,    0,   57,    2, 0x06 /* Public */,
       6,    1,   58,    2, 0x06 /* Public */,
       8,    2,   61,    2, 0x06 /* Public */,
      11,    1,   66,    2, 0x06 /* Public */,
      14,    0,   69,    2, 0x06 /* Public */,
      15,    1,   70,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      17,    1,   73,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    9,   10,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   16,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // enums: name, alias, flags, count, data
      18,   19, 0x1,    8,   81,

 // enum data: key, value
      20, uint(ddplugin_organizer::CollectionFrame::NoCollectionFrameFeatures),
      21, uint(ddplugin_organizer::CollectionFrame::CollectionFrameClosable),
      22, uint(ddplugin_organizer::CollectionFrame::CollectionFrameMovable),
      23, uint(ddplugin_organizer::CollectionFrame::CollectionFrameFloatable),
      24, uint(ddplugin_organizer::CollectionFrame::CollectionFrameHiddable),
      25, uint(ddplugin_organizer::CollectionFrame::CollectionFrameAdjustable),
      26, uint(ddplugin_organizer::CollectionFrame::CollectionFrameStretchable),
      27, uint(ddplugin_organizer::CollectionFrame::CollectionGroupFeatureMask),

       0        // eod
};

void ddplugin_organizer::CollectionFrame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CollectionFrame *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sizeModeChanged((*reinterpret_cast< const CollectionFrameSize(*)>(_a[1]))); break;
        case 1: _t->geometryChanged(); break;
        case 2: _t->editingStatusChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->requestChangeSurface((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 4: _t->surfaceChanged((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 5: _t->requestDeactiveAllPredictors(); break;
        case 6: _t->moveStateChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->adjustSizeMode((*reinterpret_cast< const CollectionFrameSize(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CollectionFrame::*)(const CollectionFrameSize & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CollectionFrame::sizeModeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CollectionFrame::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CollectionFrame::geometryChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CollectionFrame::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CollectionFrame::editingStatusChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CollectionFrame::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CollectionFrame::requestChangeSurface)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (CollectionFrame::*)(QWidget * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CollectionFrame::surfaceChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (CollectionFrame::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CollectionFrame::requestDeactiveAllPredictors)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (CollectionFrame::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CollectionFrame::moveStateChanged)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_organizer::CollectionFrame::staticMetaObject = { {
    QMetaObject::SuperData::link<Dtk::Widget::DFrame::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_organizer__CollectionFrame.data,
    qt_meta_data_ddplugin_organizer__CollectionFrame,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_organizer::CollectionFrame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_organizer::CollectionFrame::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_organizer__CollectionFrame.stringdata0))
        return static_cast<void*>(this);
    return Dtk::Widget::DFrame::qt_metacast(_clname);
}

int ddplugin_organizer::CollectionFrame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Dtk::Widget::DFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void ddplugin_organizer::CollectionFrame::sizeModeChanged(const CollectionFrameSize & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ddplugin_organizer::CollectionFrame::geometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ddplugin_organizer::CollectionFrame::editingStatusChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ddplugin_organizer::CollectionFrame::requestChangeSurface(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ddplugin_organizer::CollectionFrame::surfaceChanged(QWidget * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void ddplugin_organizer::CollectionFrame::requestDeactiveAllPredictors()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void ddplugin_organizer::CollectionFrame::moveStateChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
