/****************************************************************************
** Meta object code from reading C++ file 'tabbar.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-titlebar/views/tabbar.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tabbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_titlebar__TabBar_t {
    QByteArrayData data[26];
    char stringdata0[342];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_titlebar__TabBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_titlebar__TabBar_t qt_meta_stringdata_dfmplugin_titlebar__TabBar = {
    {
QT_MOC_LITERAL(0, 0, 26), // "dfmplugin_titlebar::TabBar"
QT_MOC_LITERAL(1, 27, 14), // "currentChanged"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 5), // "index"
QT_MOC_LITERAL(4, 49, 17), // "tabCloseRequested"
QT_MOC_LITERAL(5, 67, 11), // "remainState"
QT_MOC_LITERAL(6, 79, 16), // "requestNewWindow"
QT_MOC_LITERAL(7, 96, 3), // "url"
QT_MOC_LITERAL(8, 100, 13), // "newTabCreated"
QT_MOC_LITERAL(9, 114, 10), // "tabRemoved"
QT_MOC_LITERAL(10, 125, 8), // "tabMoved"
QT_MOC_LITERAL(11, 134, 4), // "from"
QT_MOC_LITERAL(12, 139, 2), // "to"
QT_MOC_LITERAL(13, 142, 19), // "tabAddButtonClicked"
QT_MOC_LITERAL(14, 162, 23), // "onTabCloseButtonClicked"
QT_MOC_LITERAL(15, 186, 10), // "onMoveNext"
QT_MOC_LITERAL(16, 197, 4), // "Tab*"
QT_MOC_LITERAL(17, 202, 3), // "tab"
QT_MOC_LITERAL(18, 206, 13), // "onMovePrevius"
QT_MOC_LITERAL(19, 220, 18), // "onRequestNewWindow"
QT_MOC_LITERAL(20, 239, 18), // "onAboutToNewWindow"
QT_MOC_LITERAL(21, 258, 12), // "onTabClicked"
QT_MOC_LITERAL(22, 271, 17), // "onTabDragFinished"
QT_MOC_LITERAL(23, 289, 16), // "onTabDragStarted"
QT_MOC_LITERAL(24, 306, 15), // "activateNextTab"
QT_MOC_LITERAL(25, 322, 19) // "activatePreviousTab"

    },
    "dfmplugin_titlebar::TabBar\0currentChanged\0"
    "\0index\0tabCloseRequested\0remainState\0"
    "requestNewWindow\0url\0newTabCreated\0"
    "tabRemoved\0tabMoved\0from\0to\0"
    "tabAddButtonClicked\0onTabCloseButtonClicked\0"
    "onMoveNext\0Tab*\0tab\0onMovePrevius\0"
    "onRequestNewWindow\0onAboutToNewWindow\0"
    "onTabClicked\0onTabDragFinished\0"
    "onTabDragStarted\0activateNextTab\0"
    "activatePreviousTab"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_titlebar__TabBar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  104,    2, 0x06 /* Public */,
       4,    2,  107,    2, 0x06 /* Public */,
       4,    1,  112,    2, 0x26 /* Public | MethodCloned */,
       6,    1,  115,    2, 0x06 /* Public */,
       8,    0,  118,    2, 0x06 /* Public */,
       9,    1,  119,    2, 0x06 /* Public */,
      10,    2,  122,    2, 0x06 /* Public */,
      13,    0,  127,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    0,  128,    2, 0x0a /* Public */,
      15,    1,  129,    2, 0x0a /* Public */,
      18,    1,  132,    2, 0x0a /* Public */,
      19,    1,  135,    2, 0x0a /* Public */,
      20,    1,  138,    2, 0x0a /* Public */,
      21,    0,  141,    2, 0x0a /* Public */,
      22,    0,  142,    2, 0x0a /* Public */,
      23,    0,  143,    2, 0x0a /* Public */,
      24,    0,  144,    2, 0x0a /* Public */,
      25,    0,  145,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    3,    5,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::QUrl,    7,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   11,   12,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, QMetaType::QUrl,    7,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_titlebar::TabBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TabBar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentChanged((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 1: _t->tabCloseRequested((*reinterpret_cast< const int(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        case 2: _t->tabCloseRequested((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 3: _t->requestNewWindow((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 4: _t->newTabCreated(); break;
        case 5: _t->tabRemoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->tabMoved((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 7: _t->tabAddButtonClicked(); break;
        case 8: _t->onTabCloseButtonClicked(); break;
        case 9: _t->onMoveNext((*reinterpret_cast< Tab*(*)>(_a[1]))); break;
        case 10: _t->onMovePrevius((*reinterpret_cast< Tab*(*)>(_a[1]))); break;
        case 11: _t->onRequestNewWindow((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 12: _t->onAboutToNewWindow((*reinterpret_cast< Tab*(*)>(_a[1]))); break;
        case 13: _t->onTabClicked(); break;
        case 14: _t->onTabDragFinished(); break;
        case 15: _t->onTabDragStarted(); break;
        case 16: _t->activateNextTab(); break;
        case 17: _t->activatePreviousTab(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TabBar::*)(const int & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TabBar::currentChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TabBar::*)(const int & , const bool & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TabBar::tabCloseRequested)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TabBar::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TabBar::requestNewWindow)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TabBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TabBar::newTabCreated)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (TabBar::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TabBar::tabRemoved)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (TabBar::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TabBar::tabMoved)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (TabBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TabBar::tabAddButtonClicked)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_titlebar::TabBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_titlebar__TabBar.data,
    qt_meta_data_dfmplugin_titlebar__TabBar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_titlebar::TabBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_titlebar::TabBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_titlebar__TabBar.stringdata0))
        return static_cast<void*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int dfmplugin_titlebar::TabBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_titlebar::TabBar::currentChanged(const int & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_titlebar::TabBar::tabCloseRequested(const int & _t1, const bool & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 3
void dfmplugin_titlebar::TabBar::requestNewWindow(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_titlebar::TabBar::newTabCreated()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void dfmplugin_titlebar::TabBar::tabRemoved(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void dfmplugin_titlebar::TabBar::tabMoved(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void dfmplugin_titlebar::TabBar::tabAddButtonClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
