/****************************************************************************
** Meta object code from reading C++ file 'cusmediaplayer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../../dde-file-manager/src/apps/dde-file-manager-preview/pluginpreviews/music-preview/cusmediaplayer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cusmediaplayer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_plugin_filepreview__CusMediaPlayer_t {
    QByteArrayData data[19];
    char stringdata0[259];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_plugin_filepreview__CusMediaPlayer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_plugin_filepreview__CusMediaPlayer_t qt_meta_stringdata_plugin_filepreview__CusMediaPlayer = {
    {
QT_MOC_LITERAL(0, 0, 34), // "plugin_filepreview::CusMediaP..."
QT_MOC_LITERAL(1, 35, 15), // "sigStateChanged"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 19), // "QMediaPlayer::State"
QT_MOC_LITERAL(4, 72, 8), // "newState"
QT_MOC_LITERAL(5, 81, 16), // "sigStatusChanged"
QT_MOC_LITERAL(6, 98, 25), // "QMediaPlayer::MediaStatus"
QT_MOC_LITERAL(7, 124, 6), // "status"
QT_MOC_LITERAL(8, 131, 18), // "sigDurationChanged"
QT_MOC_LITERAL(9, 150, 8), // "duration"
QT_MOC_LITERAL(10, 159, 18), // "sigPositionChanged"
QT_MOC_LITERAL(11, 178, 3), // "pos"
QT_MOC_LITERAL(12, 182, 20), // "sigCreateMediaPlayer"
QT_MOC_LITERAL(13, 203, 11), // "sigSetMedia"
QT_MOC_LITERAL(14, 215, 3), // "url"
QT_MOC_LITERAL(15, 219, 14), // "sigSetPosition"
QT_MOC_LITERAL(16, 234, 7), // "sigPlay"
QT_MOC_LITERAL(17, 242, 8), // "sigPause"
QT_MOC_LITERAL(18, 251, 7) // "sigStop"

    },
    "plugin_filepreview::CusMediaPlayer\0"
    "sigStateChanged\0\0QMediaPlayer::State\0"
    "newState\0sigStatusChanged\0"
    "QMediaPlayer::MediaStatus\0status\0"
    "sigDurationChanged\0duration\0"
    "sigPositionChanged\0pos\0sigCreateMediaPlayer\0"
    "sigSetMedia\0url\0sigSetPosition\0sigPlay\0"
    "sigPause\0sigStop"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_plugin_filepreview__CusMediaPlayer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x06 /* Public */,
       5,    1,   67,    2, 0x06 /* Public */,
       8,    1,   70,    2, 0x06 /* Public */,
      10,    1,   73,    2, 0x06 /* Public */,
      12,    0,   76,    2, 0x06 /* Public */,
      13,    1,   77,    2, 0x06 /* Public */,
      15,    1,   80,    2, 0x06 /* Public */,
      16,    0,   83,    2, 0x06 /* Public */,
      17,    0,   84,    2, 0x06 /* Public */,
      18,    0,   85,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::LongLong,    9,
    QMetaType::Void, QMetaType::LongLong,   11,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,   14,
    QMetaType::Void, QMetaType::LongLong,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void plugin_filepreview::CusMediaPlayer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CusMediaPlayer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sigStateChanged((*reinterpret_cast< QMediaPlayer::State(*)>(_a[1]))); break;
        case 1: _t->sigStatusChanged((*reinterpret_cast< QMediaPlayer::MediaStatus(*)>(_a[1]))); break;
        case 2: _t->sigDurationChanged((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 3: _t->sigPositionChanged((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 4: _t->sigCreateMediaPlayer(); break;
        case 5: _t->sigSetMedia((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 6: _t->sigSetPosition((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 7: _t->sigPlay(); break;
        case 8: _t->sigPause(); break;
        case 9: _t->sigStop(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaPlayer::State >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaPlayer::MediaStatus >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CusMediaPlayer::*)(QMediaPlayer::State );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CusMediaPlayer::sigStateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CusMediaPlayer::*)(QMediaPlayer::MediaStatus );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CusMediaPlayer::sigStatusChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CusMediaPlayer::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CusMediaPlayer::sigDurationChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CusMediaPlayer::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CusMediaPlayer::sigPositionChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (CusMediaPlayer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CusMediaPlayer::sigCreateMediaPlayer)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (CusMediaPlayer::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CusMediaPlayer::sigSetMedia)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (CusMediaPlayer::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CusMediaPlayer::sigSetPosition)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (CusMediaPlayer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CusMediaPlayer::sigPlay)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (CusMediaPlayer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CusMediaPlayer::sigPause)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (CusMediaPlayer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CusMediaPlayer::sigStop)) {
                *result = 9;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject plugin_filepreview::CusMediaPlayer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_plugin_filepreview__CusMediaPlayer.data,
    qt_meta_data_plugin_filepreview__CusMediaPlayer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *plugin_filepreview::CusMediaPlayer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *plugin_filepreview::CusMediaPlayer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_plugin_filepreview__CusMediaPlayer.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int plugin_filepreview::CusMediaPlayer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void plugin_filepreview::CusMediaPlayer::sigStateChanged(QMediaPlayer::State _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void plugin_filepreview::CusMediaPlayer::sigStatusChanged(QMediaPlayer::MediaStatus _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void plugin_filepreview::CusMediaPlayer::sigDurationChanged(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void plugin_filepreview::CusMediaPlayer::sigPositionChanged(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void plugin_filepreview::CusMediaPlayer::sigCreateMediaPlayer()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void plugin_filepreview::CusMediaPlayer::sigSetMedia(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void plugin_filepreview::CusMediaPlayer::sigSetPosition(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void plugin_filepreview::CusMediaPlayer::sigPlay()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void plugin_filepreview::CusMediaPlayer::sigPause()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void plugin_filepreview::CusMediaPlayer::sigStop()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
