/****************************************************************************
** Meta object code from reading C++ file 'extensionemblemmanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-utils/extensionimpl/emblemimpl/extensionemblemmanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'extensionemblemmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_utils__ExtensionEmblemManager_t {
    QByteArrayData data[16];
    char stringdata0[244];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_utils__ExtensionEmblemManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_utils__ExtensionEmblemManager_t qt_meta_stringdata_dfmplugin_utils__ExtensionEmblemManager = {
    {
QT_MOC_LITERAL(0, 0, 39), // "dfmplugin_utils::ExtensionEmb..."
QT_MOC_LITERAL(1, 40, 22), // "requestFetchEmblemIcon"
QT_MOC_LITERAL(2, 63, 0), // ""
QT_MOC_LITERAL(3, 64, 26), // "QList<QPair<QString,int> >"
QT_MOC_LITERAL(4, 91, 10), // "localPaths"
QT_MOC_LITERAL(5, 102, 17), // "requestClearCache"
QT_MOC_LITERAL(6, 120, 20), // "onFetchCustomEmblems"
QT_MOC_LITERAL(7, 141, 3), // "url"
QT_MOC_LITERAL(8, 145, 13), // "QList<QIcon>*"
QT_MOC_LITERAL(9, 159, 7), // "emblems"
QT_MOC_LITERAL(10, 167, 19), // "onEmblemIconChanged"
QT_MOC_LITERAL(11, 187, 4), // "path"
QT_MOC_LITERAL(12, 192, 5), // "group"
QT_MOC_LITERAL(13, 198, 23), // "onAllPluginsInitialized"
QT_MOC_LITERAL(14, 222, 12), // "onUrlChanged"
QT_MOC_LITERAL(15, 235, 8) // "windowId"

    },
    "dfmplugin_utils::ExtensionEmblemManager\0"
    "requestFetchEmblemIcon\0\0"
    "QList<QPair<QString,int> >\0localPaths\0"
    "requestClearCache\0onFetchCustomEmblems\0"
    "url\0QList<QIcon>*\0emblems\0onEmblemIconChanged\0"
    "path\0group\0onAllPluginsInitialized\0"
    "onUrlChanged\0windowId"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_utils__ExtensionEmblemManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       5,    0,   47,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    2,   48,    2, 0x0a /* Public */,
      10,    2,   53,    2, 0x0a /* Public */,
      13,    0,   58,    2, 0x0a /* Public */,
      14,    2,   59,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 8,    7,    9,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 3,   11,   12,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::ULongLong, QMetaType::QUrl,   15,    7,

       0        // eod
};

void dfmplugin_utils::ExtensionEmblemManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ExtensionEmblemManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestFetchEmblemIcon((*reinterpret_cast< const QList<QPair<QString,int> >(*)>(_a[1]))); break;
        case 1: _t->requestClearCache(); break;
        case 2: { bool _r = _t->onFetchCustomEmblems((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QList<QIcon>*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->onEmblemIconChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<QPair<QString,int> >(*)>(_a[2]))); break;
        case 4: _t->onAllPluginsInitialized(); break;
        case 5: { bool _r = _t->onUrlChanged((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ExtensionEmblemManager::*)(const QList<QPair<QString,int>> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ExtensionEmblemManager::requestFetchEmblemIcon)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ExtensionEmblemManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ExtensionEmblemManager::requestClearCache)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_utils::ExtensionEmblemManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_utils__ExtensionEmblemManager.data,
    qt_meta_data_dfmplugin_utils__ExtensionEmblemManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_utils::ExtensionEmblemManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_utils::ExtensionEmblemManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_utils__ExtensionEmblemManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_utils::ExtensionEmblemManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_utils::ExtensionEmblemManager::requestFetchEmblemIcon(const QList<QPair<QString,int>> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_utils::ExtensionEmblemManager::requestClearCache()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
