/****************************************************************************
** Meta object code from reading C++ file 'docopyfileworker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-fileoperations/fileoperations/fileoperationutils/docopyfileworker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'docopyfileworker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_fileoperations__DoCopyFileWorker_t {
    QByteArrayData data[20];
    char stringdata0[237];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_fileoperations__DoCopyFileWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_fileoperations__DoCopyFileWorker_t qt_meta_stringdata_dfmplugin_fileoperations__DoCopyFileWorker = {
    {
QT_MOC_LITERAL(0, 0, 42), // "dfmplugin_fileoperations::DoC..."
QT_MOC_LITERAL(1, 43, 13), // "ErrorFinished"
QT_MOC_LITERAL(2, 57, 0), // ""
QT_MOC_LITERAL(3, 58, 12), // "CompleteSize"
QT_MOC_LITERAL(4, 71, 4), // "size"
QT_MOC_LITERAL(5, 76, 11), // "currentTask"
QT_MOC_LITERAL(6, 88, 6), // "source"
QT_MOC_LITERAL(7, 95, 6), // "target"
QT_MOC_LITERAL(8, 102, 11), // "errorNotify"
QT_MOC_LITERAL(9, 114, 4), // "from"
QT_MOC_LITERAL(10, 119, 2), // "to"
QT_MOC_LITERAL(11, 122, 32), // "AbstractJobHandler::JobErrorType"
QT_MOC_LITERAL(12, 155, 5), // "error"
QT_MOC_LITERAL(13, 161, 4), // "isTo"
QT_MOC_LITERAL(14, 166, 2), // "id"
QT_MOC_LITERAL(15, 169, 8), // "errorMsg"
QT_MOC_LITERAL(16, 178, 13), // "allUsErrorMsg"
QT_MOC_LITERAL(17, 192, 15), // "retryErrSuccess"
QT_MOC_LITERAL(18, 208, 20), // "skipCopyLocalBigFile"
QT_MOC_LITERAL(19, 229, 7) // "fromUrl"

    },
    "dfmplugin_fileoperations::DoCopyFileWorker\0"
    "ErrorFinished\0\0CompleteSize\0size\0"
    "currentTask\0source\0target\0errorNotify\0"
    "from\0to\0AbstractJobHandler::JobErrorType\0"
    "error\0isTo\0id\0errorMsg\0allUsErrorMsg\0"
    "retryErrSuccess\0skipCopyLocalBigFile\0"
    "fromUrl"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_fileoperations__DoCopyFileWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,
       3,    1,   45,    2, 0x06 /* Public */,
       5,    2,   48,    2, 0x06 /* Public */,
       8,    7,   53,    2, 0x06 /* Public */,
      17,    1,   68,    2, 0x06 /* Public */,
      18,    1,   71,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,    6,    7,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl, 0x80000000 | 11, QMetaType::Bool, QMetaType::ULongLong, QMetaType::QString, QMetaType::Bool,    9,   10,   12,   13,   14,   15,   16,
    QMetaType::Void, QMetaType::ULongLong,   14,
    QMetaType::Void, QMetaType::QUrl,   19,

       0        // eod
};

void dfmplugin_fileoperations::DoCopyFileWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DoCopyFileWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->ErrorFinished(); break;
        case 1: _t->CompleteSize((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 2: _t->currentTask((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 3: _t->errorNotify((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const AbstractJobHandler::JobErrorType(*)>(_a[3])),(*reinterpret_cast< const bool(*)>(_a[4])),(*reinterpret_cast< const quint64(*)>(_a[5])),(*reinterpret_cast< const QString(*)>(_a[6])),(*reinterpret_cast< const bool(*)>(_a[7]))); break;
        case 4: _t->retryErrSuccess((*reinterpret_cast< const quint64(*)>(_a[1]))); break;
        case 5: _t->skipCopyLocalBigFile((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DoCopyFileWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DoCopyFileWorker::ErrorFinished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DoCopyFileWorker::*)(const int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DoCopyFileWorker::CompleteSize)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DoCopyFileWorker::*)(const QUrl , const QUrl );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DoCopyFileWorker::currentTask)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DoCopyFileWorker::*)(const QUrl & , const QUrl & , const AbstractJobHandler::JobErrorType & , const bool , const quint64 , const QString & , const bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DoCopyFileWorker::errorNotify)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DoCopyFileWorker::*)(const quint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DoCopyFileWorker::retryErrSuccess)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DoCopyFileWorker::*)(const QUrl );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DoCopyFileWorker::skipCopyLocalBigFile)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_fileoperations::DoCopyFileWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_fileoperations__DoCopyFileWorker.data,
    qt_meta_data_dfmplugin_fileoperations__DoCopyFileWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_fileoperations::DoCopyFileWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_fileoperations::DoCopyFileWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_fileoperations__DoCopyFileWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_fileoperations::DoCopyFileWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_fileoperations::DoCopyFileWorker::ErrorFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dfmplugin_fileoperations::DoCopyFileWorker::CompleteSize(const int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_fileoperations::DoCopyFileWorker::currentTask(const QUrl _t1, const QUrl _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_fileoperations::DoCopyFileWorker::errorNotify(const QUrl & _t1, const QUrl & _t2, const AbstractJobHandler::JobErrorType & _t3, const bool _t4, const quint64 _t5, const QString & _t6, const bool _t7)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t6))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t7))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_fileoperations::DoCopyFileWorker::retryErrSuccess(const quint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmplugin_fileoperations::DoCopyFileWorker::skipCopyLocalBigFile(const QUrl _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
