/****************************************************************************
** Meta object code from reading C++ file 'fileoperator.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-organizer/utils/fileoperator.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileoperator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_organizer__FileOperator_t {
    QByteArrayData data[24];
    char stringdata0[340];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_organizer__FileOperator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_organizer__FileOperator_t qt_meta_stringdata_ddplugin_organizer__FileOperator = {
    {
QT_MOC_LITERAL(0, 0, 32), // "ddplugin_organizer::FileOperator"
QT_MOC_LITERAL(1, 33, 17), // "requestSelectFile"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 12), // "QList<QUrl>&"
QT_MOC_LITERAL(4, 65, 4), // "urls"
QT_MOC_LITERAL(5, 70, 4), // "flag"
QT_MOC_LITERAL(6, 75, 21), // "requestClearSelection"
QT_MOC_LITERAL(7, 97, 15), // "requestDropFile"
QT_MOC_LITERAL(8, 113, 10), // "collection"
QT_MOC_LITERAL(9, 124, 19), // "onCanvasPastedFiles"
QT_MOC_LITERAL(10, 144, 10), // "renameFile"
QT_MOC_LITERAL(11, 155, 3), // "wid"
QT_MOC_LITERAL(12, 159, 6), // "oldUrl"
QT_MOC_LITERAL(13, 166, 6), // "newUrl"
QT_MOC_LITERAL(14, 173, 13), // "touchFileData"
QT_MOC_LITERAL(15, 187, 18), // "clearTouchFileData"
QT_MOC_LITERAL(16, 206, 13), // "pasteFileData"
QT_MOC_LITERAL(17, 220, 10), // "QSet<QUrl>"
QT_MOC_LITERAL(18, 231, 19), // "removePasteFileData"
QT_MOC_LITERAL(19, 251, 18), // "clearPasteFileData"
QT_MOC_LITERAL(20, 270, 12), // "dropFileData"
QT_MOC_LITERAL(21, 283, 19), // "QHash<QUrl,QString>"
QT_MOC_LITERAL(22, 303, 18), // "removeDropFileData"
QT_MOC_LITERAL(23, 322, 17) // "clearDropFileData"

    },
    "ddplugin_organizer::FileOperator\0"
    "requestSelectFile\0\0QList<QUrl>&\0urls\0"
    "flag\0requestClearSelection\0requestDropFile\0"
    "collection\0onCanvasPastedFiles\0"
    "renameFile\0wid\0oldUrl\0newUrl\0touchFileData\0"
    "clearTouchFileData\0pasteFileData\0"
    "QSet<QUrl>\0removePasteFileData\0"
    "clearPasteFileData\0dropFileData\0"
    "QHash<QUrl,QString>\0removeDropFileData\0"
    "clearDropFileData"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_organizer__FileOperator[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   79,    2, 0x06 /* Public */,
       6,    0,   84,    2, 0x06 /* Public */,
       7,    2,   85,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,   90,    2, 0x09 /* Protected */,

 // methods: name, argc, parameters, tag, flags
      10,    3,   91,    2, 0x02 /* Public */,
      14,    0,   98,    2, 0x02 /* Public */,
      15,    0,   99,    2, 0x02 /* Public */,
      16,    0,  100,    2, 0x02 /* Public */,
      18,    1,  101,    2, 0x02 /* Public */,
      19,    0,  104,    2, 0x02 /* Public */,
      20,    0,  105,    2, 0x02 /* Public */,
      22,    1,  106,    2, 0x02 /* Public */,
      23,    0,  109,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int,    4,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 3,    8,    4,

 // slots: parameters
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::QUrl, QMetaType::QUrl,   11,   12,   13,
    QMetaType::QUrl,
    QMetaType::Void,
    0x80000000 | 17,
    QMetaType::Void, QMetaType::QUrl,   12,
    QMetaType::Void,
    0x80000000 | 21,
    QMetaType::Void, QMetaType::QUrl,   12,
    QMetaType::Void,

       0        // eod
};

void ddplugin_organizer::FileOperator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileOperator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestSelectFile((*reinterpret_cast< QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 1: _t->requestClearSelection(); break;
        case 2: _t->requestDropFile((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QList<QUrl>(*)>(_a[2]))); break;
        case 3: _t->onCanvasPastedFiles(); break;
        case 4: _t->renameFile((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3]))); break;
        case 5: { QUrl _r = _t->touchFileData();
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->clearTouchFileData(); break;
        case 7: { QSet<QUrl> _r = _t->pasteFileData();
            if (_a[0]) *reinterpret_cast< QSet<QUrl>*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->removePasteFileData((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 9: _t->clearPasteFileData(); break;
        case 10: { QHash<QUrl,QString> _r = _t->dropFileData();
            if (_a[0]) *reinterpret_cast< QHash<QUrl,QString>*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->removeDropFileData((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 12: _t->clearDropFileData(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileOperator::*)(QList<QUrl> & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileOperator::requestSelectFile)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileOperator::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileOperator::requestClearSelection)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileOperator::*)(const QString & , QList<QUrl> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileOperator::requestDropFile)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_organizer::FileOperator::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_organizer__FileOperator.data,
    qt_meta_data_ddplugin_organizer__FileOperator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_organizer::FileOperator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_organizer::FileOperator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_organizer__FileOperator.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_organizer::FileOperator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void ddplugin_organizer::FileOperator::requestSelectFile(QList<QUrl> & _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ddplugin_organizer::FileOperator::requestClearSelection()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ddplugin_organizer::FileOperator::requestDropFile(const QString & _t1, QList<QUrl> & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
