// SPDX-FileCopyrightText: 2022 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef CHINESE_2_PINYIN_H
#define CHINESE_2_PINYIN_H

#include <QString>

namespace Pinyin {
QString Chinese2Pinyin(const QString& words);
};

#endif  // CHINESE_2_PINYIN_H
