// SPDX-FileCopyrightText: 2022 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "upgradeunit.h"

using namespace dfm_upgrade;

UpgradeUnit::UpgradeUnit()
{

}

UpgradeUnit::~UpgradeUnit()
{

}

void UpgradeUnit::completed()
{

}
