/****************************************************************************
** Meta object code from reading C++ file 'titlebarwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-titlebar/views/titlebarwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'titlebarwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_titlebar__TitleBarWidget_t {
    QByteArrayData data[26];
    char stringdata0[409];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_titlebar__TitleBarWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_titlebar__TitleBarWidget_t qt_meta_stringdata_dfmplugin_titlebar__TitleBarWidget = {
    {
QT_MOC_LITERAL(0, 0, 34), // "dfmplugin_titlebar::TitleBarW..."
QT_MOC_LITERAL(1, 35, 17), // "currentUrlChanged"
QT_MOC_LITERAL(2, 53, 0), // ""
QT_MOC_LITERAL(3, 54, 3), // "url"
QT_MOC_LITERAL(4, 58, 17), // "handleHotkeyCtrlF"
QT_MOC_LITERAL(5, 76, 17), // "handleHotkeyCtrlL"
QT_MOC_LITERAL(6, 94, 26), // "handleHotketSwitchViewMode"
QT_MOC_LITERAL(7, 121, 4), // "mode"
QT_MOC_LITERAL(8, 126, 19), // "handleHotketNextTab"
QT_MOC_LITERAL(9, 146, 23), // "handleHotketPreviousTab"
QT_MOC_LITERAL(10, 170, 27), // "handleHotketCloseCurrentTab"
QT_MOC_LITERAL(11, 198, 24), // "handleHotketCreateNewTab"
QT_MOC_LITERAL(12, 223, 23), // "handleHotketActivateTab"
QT_MOC_LITERAL(13, 247, 5), // "index"
QT_MOC_LITERAL(14, 253, 16), // "onAddressBarJump"
QT_MOC_LITERAL(15, 270, 12), // "onTabCreated"
QT_MOC_LITERAL(16, 283, 12), // "onTabRemoved"
QT_MOC_LITERAL(17, 296, 10), // "onTabMoved"
QT_MOC_LITERAL(18, 307, 4), // "from"
QT_MOC_LITERAL(19, 312, 2), // "to"
QT_MOC_LITERAL(20, 315, 19), // "onTabCurrentChanged"
QT_MOC_LITERAL(21, 335, 8), // "tabIndex"
QT_MOC_LITERAL(22, 344, 19), // "onTabCloseRequested"
QT_MOC_LITERAL(23, 364, 11), // "remainState"
QT_MOC_LITERAL(24, 376, 21), // "onTabAddButtonClicked"
QT_MOC_LITERAL(25, 398, 10) // "quitSearch"

    },
    "dfmplugin_titlebar::TitleBarWidget\0"
    "currentUrlChanged\0\0url\0handleHotkeyCtrlF\0"
    "handleHotkeyCtrlL\0handleHotketSwitchViewMode\0"
    "mode\0handleHotketNextTab\0"
    "handleHotketPreviousTab\0"
    "handleHotketCloseCurrentTab\0"
    "handleHotketCreateNewTab\0"
    "handleHotketActivateTab\0index\0"
    "onAddressBarJump\0onTabCreated\0"
    "onTabRemoved\0onTabMoved\0from\0to\0"
    "onTabCurrentChanged\0tabIndex\0"
    "onTabCloseRequested\0remainState\0"
    "onTabAddButtonClicked\0quitSearch"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_titlebar__TitleBarWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   99,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  102,    2, 0x0a /* Public */,
       5,    0,  103,    2, 0x0a /* Public */,
       6,    1,  104,    2, 0x0a /* Public */,
       8,    0,  107,    2, 0x0a /* Public */,
       9,    0,  108,    2, 0x0a /* Public */,
      10,    0,  109,    2, 0x0a /* Public */,
      11,    0,  110,    2, 0x0a /* Public */,
      12,    1,  111,    2, 0x0a /* Public */,
      14,    0,  114,    2, 0x08 /* Private */,
      15,    0,  115,    2, 0x08 /* Private */,
      16,    1,  116,    2, 0x08 /* Private */,
      17,    2,  119,    2, 0x08 /* Private */,
      20,    1,  124,    2, 0x08 /* Private */,
      22,    2,  127,    2, 0x08 /* Private */,
      24,    0,  132,    2, 0x08 /* Private */,
      25,    0,  133,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   18,   19,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,   13,   23,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_titlebar::TitleBarWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TitleBarWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentUrlChanged((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 1: _t->handleHotkeyCtrlF(); break;
        case 2: _t->handleHotkeyCtrlL(); break;
        case 3: _t->handleHotketSwitchViewMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->handleHotketNextTab(); break;
        case 5: _t->handleHotketPreviousTab(); break;
        case 6: _t->handleHotketCloseCurrentTab(); break;
        case 7: _t->handleHotketCreateNewTab(); break;
        case 8: _t->handleHotketActivateTab((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 9: _t->onAddressBarJump(); break;
        case 10: _t->onTabCreated(); break;
        case 11: _t->onTabRemoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->onTabMoved((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 13: _t->onTabCurrentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->onTabCloseRequested((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 15: _t->onTabAddButtonClicked(); break;
        case 16: _t->quitSearch(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TitleBarWidget::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TitleBarWidget::currentUrlChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_titlebar::TitleBarWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<dfmbase::AbstractFrame::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_titlebar__TitleBarWidget.data,
    qt_meta_data_dfmplugin_titlebar__TitleBarWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_titlebar::TitleBarWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_titlebar::TitleBarWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_titlebar__TitleBarWidget.stringdata0))
        return static_cast<void*>(this);
    return dfmbase::AbstractFrame::qt_metacast(_clname);
}

int dfmplugin_titlebar::TitleBarWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = dfmbase::AbstractFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_titlebar::TitleBarWidget::currentUrlChanged(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
