/****************************************************************************
** Meta object code from reading C++ file 'globaltypesdefine.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../dde-file-manager/src/services/diskencrypt/globaltypesdefine.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'globaltypesdefine.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_disk_encrypt_t {
    QByteArrayData data[10];
    char stringdata0[159];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_disk_encrypt_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_disk_encrypt_t qt_meta_stringdata_disk_encrypt = {
    {
QT_MOC_LITERAL(0, 0, 12), // "disk_encrypt"
QT_MOC_LITERAL(1, 13, 12), // "EncryptState"
QT_MOC_LITERAL(2, 26, 19), // "kStatusNotEncrypted"
QT_MOC_LITERAL(3, 46, 15), // "kStatusFinished"
QT_MOC_LITERAL(4, 62, 14), // "kStatusOffline"
QT_MOC_LITERAL(5, 77, 13), // "kStatusOnline"
QT_MOC_LITERAL(6, 91, 14), // "kStatusEncrypt"
QT_MOC_LITERAL(7, 106, 14), // "kStatusDecrypt"
QT_MOC_LITERAL(8, 121, 22), // "kStatusNoEncryptConfig"
QT_MOC_LITERAL(9, 144, 14) // "kStatusUnknown"

    },
    "disk_encrypt\0EncryptState\0kStatusNotEncrypted\0"
    "kStatusFinished\0kStatusOffline\0"
    "kStatusOnline\0kStatusEncrypt\0"
    "kStatusDecrypt\0kStatusNoEncryptConfig\0"
    "kStatusUnknown"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_disk_encrypt[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       1,   14, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // enums: name, alias, flags, count, data
       1,    1, 0x0,    8,   19,

 // enum data: key, value
       2, uint(disk_encrypt::kStatusNotEncrypted),
       3, uint(disk_encrypt::kStatusFinished),
       4, uint(disk_encrypt::kStatusOffline),
       5, uint(disk_encrypt::kStatusOnline),
       6, uint(disk_encrypt::kStatusEncrypt),
       7, uint(disk_encrypt::kStatusDecrypt),
       8, uint(disk_encrypt::kStatusNoEncryptConfig),
       9, uint(disk_encrypt::kStatusUnknown),

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject disk_encrypt::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_disk_encrypt.data,
    qt_meta_data_disk_encrypt,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
