/****************************************************************************
** Meta object code from reading C++ file 'diskmountplugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../dde-file-manager/src/external/dde-dock-plugins/disk-mount/diskmountplugin.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'diskmountplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15DiskMountPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto DiskMountPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN15DiskMountPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DiskMountPlugin",
        "setDockEntryVisible",
        "",
        "visible"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setDockEntryVisible'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DiskMountPlugin, qt_meta_tag_ZN15DiskMountPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DiskMountPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DiskMountPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DiskMountPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DiskMountPluginE_t>.metaTypes,
    nullptr
} };

void DiskMountPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DiskMountPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setDockEntryVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *DiskMountPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DiskMountPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DiskMountPluginE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "PluginsItemInterfaceV2"))
        return static_cast< PluginsItemInterfaceV2*>(this);
    if (!strcmp(_clname, "com.deepin.dock.PluginsItemInterface_V2"))
        return static_cast< PluginsItemInterfaceV2*>(this);
    return QObject::qt_metacast(_clname);
}

int DiskMountPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_DiskMountPlugin[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x27,  'c',  'o',  'm',  '.',  'd', 
    'e',  'e',  'p',  'i',  'n',  '.',  'd',  'o', 
    'c',  'k',  '.',  'P',  'l',  'u',  'g',  'i', 
    'n',  's',  'I',  't',  'e',  'm',  'I',  'n', 
    't',  'e',  'r',  'f',  'a',  'c',  'e',  '_', 
    'V',  '2', 
    // "className"
    0x03,  0x6f,  'D',  'i',  's',  'k',  'M',  'o', 
    'u',  'n',  't',  'P',  'l',  'u',  'g',  'i', 
    'n', 
    // "MetaData"
    0x04,  0xa2,  0x63,  'a',  'p',  'i',  0x65,  '2', 
    '.',  '0',  '.',  '0',  0x78,  0x1b,  'd',  'e', 
    'p',  'e',  'n',  'd',  's',  '-',  'd',  'a', 
    'e',  'm',  'o',  'n',  '-',  'd',  'b',  'u', 
    's',  '-',  's',  'e',  'r',  'v',  'i',  'c', 
    'e',  0x78,  0x1d,  'o',  'r',  'g',  '.',  'd', 
    'e',  'e',  'p',  'i',  'n',  '.',  'F',  'i', 
    'l',  'e',  'm',  'a',  'n',  'a',  'g',  'e', 
    'r',  '.',  'D',  'a',  'e',  'm',  'o',  'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(DiskMountPlugin, DiskMountPlugin, qt_pluginMetaDataV2_DiskMountPlugin)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_DiskMountPlugin[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x27,  'c',  'o',  'm',  '.',  'd', 
    'e',  'e',  'p',  'i',  'n',  '.',  'd',  'o', 
    'c',  'k',  '.',  'P',  'l',  'u',  'g',  'i', 
    'n',  's',  'I',  't',  'e',  'm',  'I',  'n', 
    't',  'e',  'r',  'f',  'a',  'c',  'e',  '_', 
    'V',  '2', 
    // "className"
    0x03,  0x6f,  'D',  'i',  's',  'k',  'M',  'o', 
    'u',  'n',  't',  'P',  'l',  'u',  'g',  'i', 
    'n', 
    // "MetaData"
    0x04,  0xa2,  0x63,  'a',  'p',  'i',  0x65,  '2', 
    '.',  '0',  '.',  '0',  0x78,  0x1b,  'd',  'e', 
    'p',  'e',  'n',  'd',  's',  '-',  'd',  'a', 
    'e',  'm',  'o',  'n',  '-',  'd',  'b',  'u', 
    's',  '-',  's',  'e',  'r',  'v',  'i',  'c', 
    'e',  0x78,  0x1d,  'o',  'r',  'g',  '.',  'd', 
    'e',  'e',  'p',  'i',  'n',  '.',  'F',  'i', 
    'l',  'e',  'm',  'a',  'n',  'a',  'g',  'e', 
    'r',  '.',  'D',  'a',  'e',  'm',  'o',  'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(DiskMountPlugin, DiskMountPlugin)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
