/****************************************************************************
** Meta object code from reading C++ file 'configpresenter.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-organizer/config/configpresenter.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'configpresenter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_organizer__ConfigPresenter_t {
    QByteArrayData data[20];
    char stringdata0[298];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_organizer__ConfigPresenter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_organizer__ConfigPresenter_t qt_meta_stringdata_ddplugin_organizer__ConfigPresenter = {
    {
QT_MOC_LITERAL(0, 0, 35), // "ddplugin_organizer::ConfigPre..."
QT_MOC_LITERAL(1, 36, 17), // "changeEnableState"
QT_MOC_LITERAL(2, 54, 0), // ""
QT_MOC_LITERAL(3, 55, 1), // "e"
QT_MOC_LITERAL(4, 57, 27), // "changeEnableVisibilityState"
QT_MOC_LITERAL(5, 85, 1), // "v"
QT_MOC_LITERAL(6, 87, 24), // "changeHideAllKeySequence"
QT_MOC_LITERAL(7, 112, 18), // "switchToNormalized"
QT_MOC_LITERAL(8, 131, 17), // "releaseCollection"
QT_MOC_LITERAL(9, 149, 8), // "category"
QT_MOC_LITERAL(10, 158, 14), // "switchToCustom"
QT_MOC_LITERAL(11, 173, 17), // "changeDisplaySize"
QT_MOC_LITERAL(12, 191, 13), // "newCollection"
QT_MOC_LITERAL(13, 205, 11), // "QList<QUrl>"
QT_MOC_LITERAL(14, 217, 16), // "showOptionWindow"
QT_MOC_LITERAL(15, 234, 17), // "reorganizeDesktop"
QT_MOC_LITERAL(16, 252, 20), // "optimizeStateChanged"
QT_MOC_LITERAL(17, 273, 16), // "onDConfigChanged"
QT_MOC_LITERAL(18, 290, 3), // "cfg"
QT_MOC_LITERAL(19, 294, 3) // "key"

    },
    "ddplugin_organizer::ConfigPresenter\0"
    "changeEnableState\0\0e\0changeEnableVisibilityState\0"
    "v\0changeHideAllKeySequence\0"
    "switchToNormalized\0releaseCollection\0"
    "category\0switchToCustom\0changeDisplaySize\0"
    "newCollection\0QList<QUrl>\0showOptionWindow\0"
    "reorganizeDesktop\0optimizeStateChanged\0"
    "onDConfigChanged\0cfg\0key"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_organizer__ConfigPresenter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       4,    1,   77,    2, 0x06 /* Public */,
       6,    1,   80,    2, 0x06 /* Public */,
       7,    1,   83,    2, 0x06 /* Public */,
       8,    1,   86,    2, 0x06 /* Public */,
      10,    0,   89,    2, 0x06 /* Public */,
      11,    1,   90,    2, 0x06 /* Public */,
      12,    1,   93,    2, 0x06 /* Public */,
      14,    0,   96,    2, 0x06 /* Public */,
      15,    0,   97,    2, 0x06 /* Public */,
      16,    1,   98,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      17,    2,  101,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::QKeySequence,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 13,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   18,   19,

       0        // eod
};

void ddplugin_organizer::ConfigPresenter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ConfigPresenter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->changeEnableState((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->changeEnableVisibilityState((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->changeHideAllKeySequence((*reinterpret_cast< const QKeySequence(*)>(_a[1]))); break;
        case 3: _t->switchToNormalized((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->releaseCollection((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->switchToCustom(); break;
        case 6: _t->changeDisplaySize((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->newCollection((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 8: _t->showOptionWindow(); break;
        case 9: _t->reorganizeDesktop(); break;
        case 10: _t->optimizeStateChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->onDConfigChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ConfigPresenter::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfigPresenter::changeEnableState)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ConfigPresenter::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfigPresenter::changeEnableVisibilityState)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ConfigPresenter::*)(const QKeySequence & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfigPresenter::changeHideAllKeySequence)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ConfigPresenter::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfigPresenter::switchToNormalized)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ConfigPresenter::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfigPresenter::releaseCollection)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ConfigPresenter::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfigPresenter::switchToCustom)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (ConfigPresenter::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfigPresenter::changeDisplaySize)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (ConfigPresenter::*)(const QList<QUrl> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfigPresenter::newCollection)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (ConfigPresenter::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfigPresenter::showOptionWindow)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (ConfigPresenter::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfigPresenter::reorganizeDesktop)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (ConfigPresenter::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConfigPresenter::optimizeStateChanged)) {
                *result = 10;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_organizer::ConfigPresenter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_organizer__ConfigPresenter.data,
    qt_meta_data_ddplugin_organizer__ConfigPresenter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_organizer::ConfigPresenter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_organizer::ConfigPresenter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_organizer__ConfigPresenter.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_organizer::ConfigPresenter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void ddplugin_organizer::ConfigPresenter::changeEnableState(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ddplugin_organizer::ConfigPresenter::changeEnableVisibilityState(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ddplugin_organizer::ConfigPresenter::changeHideAllKeySequence(const QKeySequence & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ddplugin_organizer::ConfigPresenter::switchToNormalized(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ddplugin_organizer::ConfigPresenter::releaseCollection(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void ddplugin_organizer::ConfigPresenter::switchToCustom()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void ddplugin_organizer::ConfigPresenter::changeDisplaySize(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void ddplugin_organizer::ConfigPresenter::newCollection(const QList<QUrl> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void ddplugin_organizer::ConfigPresenter::showOptionWindow()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void ddplugin_organizer::ConfigPresenter::reorganizeDesktop()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void ddplugin_organizer::ConfigPresenter::optimizeStateChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
