/****************************************************************************
** Meta object code from reading C++ file 'fileinfomodelbroker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-canvas/broker/fileinfomodelbroker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileinfomodelbroker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_canvas__FileInfoModelBroker_t {
    QByteArrayData data[20];
    char stringdata0[205];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_canvas__FileInfoModelBroker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_canvas__FileInfoModelBroker_t qt_meta_stringdata_ddplugin_canvas__FileInfoModelBroker = {
    {
QT_MOC_LITERAL(0, 0, 36), // "ddplugin_canvas::FileInfoMode..."
QT_MOC_LITERAL(1, 37, 7), // "rootUrl"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 9), // "rootIndex"
QT_MOC_LITERAL(4, 56, 11), // "QModelIndex"
QT_MOC_LITERAL(5, 68, 8), // "urlIndex"
QT_MOC_LITERAL(6, 77, 3), // "url"
QT_MOC_LITERAL(7, 81, 8), // "indexUrl"
QT_MOC_LITERAL(8, 90, 5), // "index"
QT_MOC_LITERAL(9, 96, 5), // "files"
QT_MOC_LITERAL(10, 102, 11), // "QList<QUrl>"
QT_MOC_LITERAL(11, 114, 8), // "fileInfo"
QT_MOC_LITERAL(12, 123, 15), // "FileInfoPointer"
QT_MOC_LITERAL(13, 139, 7), // "refresh"
QT_MOC_LITERAL(14, 147, 6), // "parent"
QT_MOC_LITERAL(15, 154, 10), // "modelState"
QT_MOC_LITERAL(16, 165, 10), // "updateFile"
QT_MOC_LITERAL(17, 176, 14), // "onDataReplaced"
QT_MOC_LITERAL(18, 191, 6), // "oldUrl"
QT_MOC_LITERAL(19, 198, 6) // "newUrl"

    },
    "ddplugin_canvas::FileInfoModelBroker\0"
    "rootUrl\0\0rootIndex\0QModelIndex\0urlIndex\0"
    "url\0indexUrl\0index\0files\0QList<QUrl>\0"
    "fileInfo\0FileInfoPointer\0refresh\0"
    "parent\0modelState\0updateFile\0"
    "onDataReplaced\0oldUrl\0newUrl"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_canvas__FileInfoModelBroker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x0a /* Public */,
       3,    0,   65,    2, 0x0a /* Public */,
       5,    1,   66,    2, 0x0a /* Public */,
       7,    1,   69,    2, 0x0a /* Public */,
       9,    0,   72,    2, 0x0a /* Public */,
      11,    1,   73,    2, 0x0a /* Public */,
      13,    1,   76,    2, 0x0a /* Public */,
      15,    0,   79,    2, 0x0a /* Public */,
      16,    1,   80,    2, 0x0a /* Public */,
      17,    2,   83,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::QUrl,
    0x80000000 | 4,
    0x80000000 | 4, QMetaType::QUrl,    6,
    QMetaType::QUrl, 0x80000000 | 4,    8,
    0x80000000 | 10,
    0x80000000 | 12, 0x80000000 | 4,    8,
    QMetaType::Void, 0x80000000 | 4,   14,
    QMetaType::Int,
    QMetaType::Void, QMetaType::QUrl,    6,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,   18,   19,

       0        // eod
};

void ddplugin_canvas::FileInfoModelBroker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileInfoModelBroker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QUrl _r = _t->rootUrl();
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = std::move(_r); }  break;
        case 1: { QModelIndex _r = _t->rootIndex();
            if (_a[0]) *reinterpret_cast< QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 2: { QModelIndex _r = _t->urlIndex((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 3: { QUrl _r = _t->indexUrl((*reinterpret_cast< const QModelIndex(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = std::move(_r); }  break;
        case 4: { QList<QUrl> _r = _t->files();
            if (_a[0]) *reinterpret_cast< QList<QUrl>*>(_a[0]) = std::move(_r); }  break;
        case 5: { FileInfoPointer _r = _t->fileInfo((*reinterpret_cast< const QModelIndex(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< FileInfoPointer*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->refresh((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 7: { int _r = _t->modelState();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->updateFile((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 9: _t->onDataReplaced((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_canvas::FileInfoModelBroker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_canvas__FileInfoModelBroker.data,
    qt_meta_data_ddplugin_canvas__FileInfoModelBroker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_canvas::FileInfoModelBroker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_canvas::FileInfoModelBroker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_canvas__FileInfoModelBroker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_canvas::FileInfoModelBroker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
