/****************************************************************************
** Meta object code from reading C++ file 'sheetbrowser.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../../dde-file-manager/src/apps/dde-file-manager-preview/pluginpreviews/pdf-preview/sheetbrowser.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sheetbrowser.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_plugin_filepreview__SheetBrowser_t {
    QByteArrayData data[13];
    char stringdata0[206];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_plugin_filepreview__SheetBrowser_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_plugin_filepreview__SheetBrowser_t qt_meta_stringdata_plugin_filepreview__SheetBrowser = {
    {
QT_MOC_LITERAL(0, 0, 32), // "plugin_filepreview::SheetBrowser"
QT_MOC_LITERAL(1, 33, 14), // "sigPageChanged"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 4), // "page"
QT_MOC_LITERAL(4, 54, 16), // "sigNeedPageFirst"
QT_MOC_LITERAL(5, 71, 15), // "sigNeedPagePrev"
QT_MOC_LITERAL(6, 87, 15), // "sigNeedPageNext"
QT_MOC_LITERAL(7, 103, 15), // "sigNeedPageLast"
QT_MOC_LITERAL(8, 119, 6), // "onInit"
QT_MOC_LITERAL(9, 126, 31), // "onVerticalScrollBarValueChanged"
QT_MOC_LITERAL(10, 158, 5), // "value"
QT_MOC_LITERAL(11, 164, 17), // "onViewportChanged"
QT_MOC_LITERAL(12, 182, 23) // "onRemoveDocSlideGesture"

    },
    "plugin_filepreview::SheetBrowser\0"
    "sigPageChanged\0\0page\0sigNeedPageFirst\0"
    "sigNeedPagePrev\0sigNeedPageNext\0"
    "sigNeedPageLast\0onInit\0"
    "onVerticalScrollBarValueChanged\0value\0"
    "onViewportChanged\0onRemoveDocSlideGesture"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_plugin_filepreview__SheetBrowser[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,
       4,    0,   62,    2, 0x06 /* Public */,
       5,    0,   63,    2, 0x06 /* Public */,
       6,    0,   64,    2, 0x06 /* Public */,
       7,    0,   65,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    0,   66,    2, 0x08 /* Private */,
       9,    1,   67,    2, 0x08 /* Private */,
      11,    0,   70,    2, 0x08 /* Private */,
      12,    0,   71,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void plugin_filepreview::SheetBrowser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SheetBrowser *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sigPageChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->sigNeedPageFirst(); break;
        case 2: _t->sigNeedPagePrev(); break;
        case 3: _t->sigNeedPageNext(); break;
        case 4: _t->sigNeedPageLast(); break;
        case 5: _t->onInit(); break;
        case 6: _t->onVerticalScrollBarValueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->onViewportChanged(); break;
        case 8: _t->onRemoveDocSlideGesture(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SheetBrowser::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SheetBrowser::sigPageChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SheetBrowser::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SheetBrowser::sigNeedPageFirst)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (SheetBrowser::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SheetBrowser::sigNeedPagePrev)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (SheetBrowser::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SheetBrowser::sigNeedPageNext)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (SheetBrowser::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SheetBrowser::sigNeedPageLast)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject plugin_filepreview::SheetBrowser::staticMetaObject = { {
    QMetaObject::SuperData::link<Dtk::Widget::DGraphicsView::staticMetaObject>(),
    qt_meta_stringdata_plugin_filepreview__SheetBrowser.data,
    qt_meta_data_plugin_filepreview__SheetBrowser,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *plugin_filepreview::SheetBrowser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *plugin_filepreview::SheetBrowser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_plugin_filepreview__SheetBrowser.stringdata0))
        return static_cast<void*>(this);
    return Dtk::Widget::DGraphicsView::qt_metacast(_clname);
}

int plugin_filepreview::SheetBrowser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Dtk::Widget::DGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void plugin_filepreview::SheetBrowser::sigPageChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void plugin_filepreview::SheetBrowser::sigNeedPageFirst()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void plugin_filepreview::SheetBrowser::sigNeedPagePrev()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void plugin_filepreview::SheetBrowser::sigNeedPageNext()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void plugin_filepreview::SheetBrowser::sigNeedPageLast()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
