/****************************************************************************
** Meta object code from reading C++ file 'trashfileeventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-fileoperations/fileoperationsevent/trashfileeventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'trashfileeventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_fileoperations__TrashFileEventReceiver_t {
    QByteArrayData data[29];
    char stringdata0[604];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_fileoperations__TrashFileEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_fileoperations__TrashFileEventReceiver_t qt_meta_stringdata_dfmplugin_fileoperations__TrashFileEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 48), // "dfmplugin_fileoperations::Tra..."
QT_MOC_LITERAL(1, 49, 14), // "cleanTrashUrls"
QT_MOC_LITERAL(2, 64, 0), // ""
QT_MOC_LITERAL(3, 65, 8), // "windowId"
QT_MOC_LITERAL(4, 74, 11), // "QList<QUrl>"
QT_MOC_LITERAL(5, 86, 7), // "sources"
QT_MOC_LITERAL(6, 94, 51), // "dfmbase::AbstractJobHandler::..."
QT_MOC_LITERAL(7, 146, 16), // "deleteNoticeType"
QT_MOC_LITERAL(8, 163, 51), // "dfmbase::AbstractJobHandler::..."
QT_MOC_LITERAL(9, 215, 14), // "handleCallback"
QT_MOC_LITERAL(10, 230, 26), // "handleOperationMoveToTrash"
QT_MOC_LITERAL(11, 257, 36), // "dfmbase::AbstractJobHandler::..."
QT_MOC_LITERAL(12, 294, 5), // "flags"
QT_MOC_LITERAL(13, 300, 6), // "handle"
QT_MOC_LITERAL(14, 307, 6), // "custom"
QT_MOC_LITERAL(15, 314, 45), // "dfmbase::AbstractJobHandler::..."
QT_MOC_LITERAL(16, 360, 8), // "callback"
QT_MOC_LITERAL(17, 369, 31), // "handleOperationRestoreFromTrash"
QT_MOC_LITERAL(18, 401, 6), // "target"
QT_MOC_LITERAL(19, 408, 25), // "handleOperationCleanTrash"
QT_MOC_LITERAL(20, 434, 28), // "handleOperationCopyFromTrash"
QT_MOC_LITERAL(21, 463, 17), // "handleSaveRedoOpt"
QT_MOC_LITERAL(22, 481, 5), // "token"
QT_MOC_LITERAL(23, 487, 12), // "moreThanZero"
QT_MOC_LITERAL(24, 500, 30), // "handleOperationUndoMoveToTrash"
QT_MOC_LITERAL(25, 531, 2), // "op"
QT_MOC_LITERAL(26, 534, 35), // "handleOperationUndoRestoreFro..."
QT_MOC_LITERAL(27, 570, 16), // "onCleanTrashUrls"
QT_MOC_LITERAL(28, 587, 16) // "JobHandlePointer"

    },
    "dfmplugin_fileoperations::TrashFileEventReceiver\0"
    "cleanTrashUrls\0\0windowId\0QList<QUrl>\0"
    "sources\0dfmbase::AbstractJobHandler::DeleteDialogNoticeType\0"
    "deleteNoticeType\0"
    "dfmbase::AbstractJobHandler::OperatorHandleCallback\0"
    "handleCallback\0handleOperationMoveToTrash\0"
    "dfmbase::AbstractJobHandler::JobFlag\0"
    "flags\0handle\0custom\0"
    "dfmbase::AbstractJobHandler::OperatorCallback\0"
    "callback\0handleOperationRestoreFromTrash\0"
    "target\0handleOperationCleanTrash\0"
    "handleOperationCopyFromTrash\0"
    "handleSaveRedoOpt\0token\0moreThanZero\0"
    "handleOperationUndoMoveToTrash\0op\0"
    "handleOperationUndoRestoreFromTrash\0"
    "onCleanTrashUrls\0JobHandlePointer"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_fileoperations__TrashFileEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    4,   79,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    4,   88,    2, 0x0a /* Public */,
      10,    6,   97,    2, 0x0a /* Public */,
      17,    5,  110,    2, 0x0a /* Public */,
      17,    7,  121,    2, 0x0a /* Public */,
      19,    4,  136,    2, 0x0a /* Public */,
      19,    5,  145,    2, 0x0a /* Public */,
      20,    5,  156,    2, 0x0a /* Public */,
      20,    7,  167,    2, 0x0a /* Public */,
      21,    2,  182,    2, 0x0a /* Public */,
      24,    5,  187,    2, 0x0a /* Public */,
      26,    6,  198,    2, 0x0a /* Public */,
      27,    4,  211,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 6, 0x80000000 | 8,    3,    5,    7,    9,

 // slots: parameters
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 11, 0x80000000 | 8,    3,    5,   12,   13,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 11, 0x80000000 | 8, QMetaType::QVariant, 0x80000000 | 15,    3,    5,   12,   13,   14,   16,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QUrl, 0x80000000 | 11, 0x80000000 | 8,    3,    5,   18,   12,   13,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QUrl, 0x80000000 | 11, 0x80000000 | 8, QMetaType::QVariant, 0x80000000 | 15,    3,    5,   18,   12,   13,   14,   16,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 6, 0x80000000 | 8,    3,    5,    7,   13,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 8, QMetaType::QVariant, 0x80000000 | 15,    3,    5,   13,   14,   16,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QUrl, 0x80000000 | 11, 0x80000000 | 8,    3,    5,   18,   12,   13,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QUrl, 0x80000000 | 11, 0x80000000 | 8, QMetaType::QVariant, 0x80000000 | 15,    3,    5,   18,   12,   13,   14,   16,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,   22,   23,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 11, 0x80000000 | 8, QMetaType::QVariantMap,    3,    5,   12,    9,   25,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 4, QMetaType::QUrl, 0x80000000 | 11, 0x80000000 | 8, QMetaType::QVariantMap,    3,    5,   18,   12,    9,   25,
    0x80000000 | 28, QMetaType::ULongLong, 0x80000000 | 4, 0x80000000 | 6, 0x80000000 | 8,    3,    5,    7,    9,

       0        // eod
};

void dfmplugin_fileoperations::TrashFileEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TrashFileEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cleanTrashUrls((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::DeleteDialogNoticeType(*)>(_a[3])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[4]))); break;
        case 1: _t->handleOperationMoveToTrash((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[3])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[4]))); break;
        case 2: _t->handleOperationMoveToTrash((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[3])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[4])),(*reinterpret_cast< const QVariant(*)>(_a[5])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[6]))); break;
        case 3: _t->handleOperationRestoreFromTrash((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[5]))); break;
        case 4: _t->handleOperationRestoreFromTrash((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[5])),(*reinterpret_cast< const QVariant(*)>(_a[6])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[7]))); break;
        case 5: _t->handleOperationCleanTrash((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::DeleteDialogNoticeType(*)>(_a[3])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[4]))); break;
        case 6: _t->handleOperationCleanTrash((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[3])),(*reinterpret_cast< const QVariant(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[5]))); break;
        case 7: _t->handleOperationCopyFromTrash((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[5]))); break;
        case 8: _t->handleOperationCopyFromTrash((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[5])),(*reinterpret_cast< const QVariant(*)>(_a[6])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorCallback(*)>(_a[7]))); break;
        case 9: _t->handleSaveRedoOpt((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        case 10: _t->handleOperationUndoMoveToTrash((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[3])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[4])),(*reinterpret_cast< const QVariantMap(*)>(_a[5]))); break;
        case 11: _t->handleOperationUndoRestoreFromTrash((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::JobFlag(*)>(_a[4])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[5])),(*reinterpret_cast< const QVariantMap(*)>(_a[6]))); break;
        case 12: { JobHandlePointer _r = _t->onCleanTrashUrls((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const dfmbase::AbstractJobHandler::DeleteDialogNoticeType(*)>(_a[3])),(*reinterpret_cast< dfmbase::AbstractJobHandler::OperatorHandleCallback(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< JobHandlePointer*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 5:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 6:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 6:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorCallback >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::JobFlag >(); break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::AbstractJobHandler::OperatorHandleCallback >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TrashFileEventReceiver::*)(const quint64 , const QList<QUrl> , const dfmbase::AbstractJobHandler::DeleteDialogNoticeType , dfmbase::AbstractJobHandler::OperatorHandleCallback );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TrashFileEventReceiver::cleanTrashUrls)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_fileoperations::TrashFileEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_fileoperations__TrashFileEventReceiver.data,
    qt_meta_data_dfmplugin_fileoperations__TrashFileEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_fileoperations::TrashFileEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_fileoperations::TrashFileEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_fileoperations__TrashFileEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_fileoperations::TrashFileEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_fileoperations::TrashFileEventReceiver::cleanTrashUrls(const quint64 _t1, const QList<QUrl> _t2, const dfmbase::AbstractJobHandler::DeleteDialogNoticeType _t3, dfmbase::AbstractJobHandler::OperatorHandleCallback _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
