/****************************************************************************
** Meta object code from reading C++ file 'filedialoghandle.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filedialog/filedialogplugin-core/dbus/filedialoghandle.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filedialoghandle.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FileDialogHandle_t {
    QByteArrayData data[83];
    char stringdata0[998];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FileDialogHandle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FileDialogHandle_t qt_meta_stringdata_FileDialogHandle = {
    {
QT_MOC_LITERAL(0, 0, 16), // "FileDialogHandle"
QT_MOC_LITERAL(1, 17, 8), // "finished"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 6), // "result"
QT_MOC_LITERAL(4, 34, 8), // "accepted"
QT_MOC_LITERAL(5, 43, 8), // "rejected"
QT_MOC_LITERAL(6, 52, 21), // "selectionFilesChanged"
QT_MOC_LITERAL(7, 74, 17), // "currentUrlChanged"
QT_MOC_LITERAL(8, 92, 25), // "selectedNameFilterChanged"
QT_MOC_LITERAL(9, 118, 9), // "setParent"
QT_MOC_LITERAL(10, 128, 8), // "QWidget*"
QT_MOC_LITERAL(11, 137, 6), // "parent"
QT_MOC_LITERAL(12, 144, 6), // "widget"
QT_MOC_LITERAL(13, 151, 12), // "setDirectory"
QT_MOC_LITERAL(14, 164, 9), // "directory"
QT_MOC_LITERAL(15, 174, 4), // "QDir"
QT_MOC_LITERAL(16, 179, 15), // "setDirectoryUrl"
QT_MOC_LITERAL(17, 195, 12), // "directoryUrl"
QT_MOC_LITERAL(18, 208, 10), // "selectFile"
QT_MOC_LITERAL(19, 219, 8), // "filename"
QT_MOC_LITERAL(20, 228, 13), // "selectedFiles"
QT_MOC_LITERAL(21, 242, 9), // "selectUrl"
QT_MOC_LITERAL(22, 252, 3), // "url"
QT_MOC_LITERAL(23, 256, 12), // "selectedUrls"
QT_MOC_LITERAL(24, 269, 11), // "QList<QUrl>"
QT_MOC_LITERAL(25, 281, 19), // "addDisableUrlScheme"
QT_MOC_LITERAL(26, 301, 6), // "scheme"
QT_MOC_LITERAL(27, 308, 14), // "setNameFilters"
QT_MOC_LITERAL(28, 323, 7), // "filters"
QT_MOC_LITERAL(29, 331, 11), // "nameFilters"
QT_MOC_LITERAL(30, 343, 16), // "selectNameFilter"
QT_MOC_LITERAL(31, 360, 6), // "filter"
QT_MOC_LITERAL(32, 367, 18), // "selectedNameFilter"
QT_MOC_LITERAL(33, 386, 23), // "selectNameFilterByIndex"
QT_MOC_LITERAL(34, 410, 5), // "index"
QT_MOC_LITERAL(35, 416, 23), // "selectedNameFilterIndex"
QT_MOC_LITERAL(36, 440, 5), // "winId"
QT_MOC_LITERAL(37, 446, 13), // "QDir::Filters"
QT_MOC_LITERAL(38, 460, 9), // "setFilter"
QT_MOC_LITERAL(39, 470, 11), // "setViewMode"
QT_MOC_LITERAL(40, 482, 21), // "QFileDialog::ViewMode"
QT_MOC_LITERAL(41, 504, 4), // "mode"
QT_MOC_LITERAL(42, 509, 8), // "viewMode"
QT_MOC_LITERAL(43, 518, 11), // "setFileMode"
QT_MOC_LITERAL(44, 530, 21), // "QFileDialog::FileMode"
QT_MOC_LITERAL(45, 552, 13), // "setAcceptMode"
QT_MOC_LITERAL(46, 566, 23), // "QFileDialog::AcceptMode"
QT_MOC_LITERAL(47, 590, 10), // "acceptMode"
QT_MOC_LITERAL(48, 601, 12), // "setLabelText"
QT_MOC_LITERAL(49, 614, 24), // "QFileDialog::DialogLabel"
QT_MOC_LITERAL(50, 639, 5), // "label"
QT_MOC_LITERAL(51, 645, 4), // "text"
QT_MOC_LITERAL(52, 650, 9), // "labelText"
QT_MOC_LITERAL(53, 660, 10), // "setOptions"
QT_MOC_LITERAL(54, 671, 20), // "QFileDialog::Options"
QT_MOC_LITERAL(55, 692, 7), // "options"
QT_MOC_LITERAL(56, 700, 9), // "setOption"
QT_MOC_LITERAL(57, 710, 19), // "QFileDialog::Option"
QT_MOC_LITERAL(58, 730, 6), // "option"
QT_MOC_LITERAL(59, 737, 2), // "on"
QT_MOC_LITERAL(60, 740, 10), // "testOption"
QT_MOC_LITERAL(61, 751, 19), // "setCurrentInputName"
QT_MOC_LITERAL(62, 771, 4), // "name"
QT_MOC_LITERAL(63, 776, 15), // "addCustomWidget"
QT_MOC_LITERAL(64, 792, 4), // "type"
QT_MOC_LITERAL(65, 797, 4), // "data"
QT_MOC_LITERAL(66, 802, 20), // "getCustomWidgetValue"
QT_MOC_LITERAL(67, 823, 12), // "QDBusVariant"
QT_MOC_LITERAL(68, 836, 21), // "allCustomWidgetsValue"
QT_MOC_LITERAL(69, 858, 20), // "beginAddCustomWidget"
QT_MOC_LITERAL(70, 879, 18), // "endAddCustomWidget"
QT_MOC_LITERAL(71, 898, 22), // "setAllowMixedSelection"
QT_MOC_LITERAL(72, 921, 15), // "setHideOnAccept"
QT_MOC_LITERAL(73, 937, 6), // "enable"
QT_MOC_LITERAL(74, 944, 12), // "hideOnAccept"
QT_MOC_LITERAL(75, 957, 4), // "show"
QT_MOC_LITERAL(76, 962, 4), // "hide"
QT_MOC_LITERAL(77, 967, 6), // "accept"
QT_MOC_LITERAL(78, 974, 4), // "done"
QT_MOC_LITERAL(79, 979, 1), // "r"
QT_MOC_LITERAL(80, 981, 4), // "exec"
QT_MOC_LITERAL(81, 986, 4), // "open"
QT_MOC_LITERAL(82, 991, 6) // "reject"

    },
    "FileDialogHandle\0finished\0\0result\0"
    "accepted\0rejected\0selectionFilesChanged\0"
    "currentUrlChanged\0selectedNameFilterChanged\0"
    "setParent\0QWidget*\0parent\0widget\0"
    "setDirectory\0directory\0QDir\0setDirectoryUrl\0"
    "directoryUrl\0selectFile\0filename\0"
    "selectedFiles\0selectUrl\0url\0selectedUrls\0"
    "QList<QUrl>\0addDisableUrlScheme\0scheme\0"
    "setNameFilters\0filters\0nameFilters\0"
    "selectNameFilter\0filter\0selectedNameFilter\0"
    "selectNameFilterByIndex\0index\0"
    "selectedNameFilterIndex\0winId\0"
    "QDir::Filters\0setFilter\0setViewMode\0"
    "QFileDialog::ViewMode\0mode\0viewMode\0"
    "setFileMode\0QFileDialog::FileMode\0"
    "setAcceptMode\0QFileDialog::AcceptMode\0"
    "acceptMode\0setLabelText\0"
    "QFileDialog::DialogLabel\0label\0text\0"
    "labelText\0setOptions\0QFileDialog::Options\0"
    "options\0setOption\0QFileDialog::Option\0"
    "option\0on\0testOption\0setCurrentInputName\0"
    "name\0addCustomWidget\0type\0data\0"
    "getCustomWidgetValue\0QDBusVariant\0"
    "allCustomWidgetsValue\0beginAddCustomWidget\0"
    "endAddCustomWidget\0setAllowMixedSelection\0"
    "setHideOnAccept\0enable\0hideOnAccept\0"
    "show\0hide\0accept\0done\0r\0exec\0open\0"
    "reject"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FileDialogHandle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      55,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  289,    2, 0x06 /* Public */,
       4,    0,  292,    2, 0x06 /* Public */,
       5,    0,  293,    2, 0x06 /* Public */,
       6,    0,  294,    2, 0x06 /* Public */,
       7,    0,  295,    2, 0x06 /* Public */,
       8,    0,  296,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    1,  297,    2, 0x0a /* Public */,
      12,    0,  300,    2, 0x0a /* Public */,
      13,    1,  301,    2, 0x0a /* Public */,
      13,    1,  304,    2, 0x0a /* Public */,
      14,    0,  307,    2, 0x0a /* Public */,
      16,    1,  308,    2, 0x0a /* Public */,
      17,    0,  311,    2, 0x0a /* Public */,
      18,    1,  312,    2, 0x0a /* Public */,
      20,    0,  315,    2, 0x0a /* Public */,
      21,    1,  316,    2, 0x0a /* Public */,
      23,    0,  319,    2, 0x0a /* Public */,
      25,    1,  320,    2, 0x0a /* Public */,
      27,    1,  323,    2, 0x0a /* Public */,
      29,    0,  326,    2, 0x0a /* Public */,
      30,    1,  327,    2, 0x0a /* Public */,
      32,    0,  330,    2, 0x0a /* Public */,
      33,    1,  331,    2, 0x0a /* Public */,
      35,    0,  334,    2, 0x0a /* Public */,
      36,    0,  335,    2, 0x0a /* Public */,
      31,    0,  336,    2, 0x0a /* Public */,
      38,    1,  337,    2, 0x0a /* Public */,
      39,    1,  340,    2, 0x0a /* Public */,
      42,    0,  343,    2, 0x0a /* Public */,
      43,    1,  344,    2, 0x0a /* Public */,
      45,    1,  347,    2, 0x0a /* Public */,
      47,    0,  350,    2, 0x0a /* Public */,
      48,    2,  351,    2, 0x0a /* Public */,
      52,    1,  356,    2, 0x0a /* Public */,
      53,    1,  359,    2, 0x0a /* Public */,
      56,    2,  362,    2, 0x0a /* Public */,
      56,    1,  367,    2, 0x2a /* Public | MethodCloned */,
      55,    0,  370,    2, 0x0a /* Public */,
      60,    1,  371,    2, 0x0a /* Public */,
      61,    1,  374,    2, 0x0a /* Public */,
      63,    2,  377,    2, 0x0a /* Public */,
      66,    2,  382,    2, 0x0a /* Public */,
      68,    1,  387,    2, 0x0a /* Public */,
      69,    0,  390,    2, 0x0a /* Public */,
      70,    0,  391,    2, 0x0a /* Public */,
      71,    1,  392,    2, 0x0a /* Public */,
      72,    1,  395,    2, 0x0a /* Public */,
      74,    0,  398,    2, 0x0a /* Public */,
      75,    0,  399,    2, 0x0a /* Public */,
      76,    0,  400,    2, 0x0a /* Public */,
      77,    0,  401,    2, 0x0a /* Public */,
      78,    1,  402,    2, 0x0a /* Public */,
      80,    0,  405,    2, 0x0a /* Public */,
      81,    0,  406,    2, 0x0a /* Public */,
      82,    0,  407,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 10,   11,
    0x80000000 | 10,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void, 0x80000000 | 15,   14,
    0x80000000 | 15,
    QMetaType::Void, QMetaType::QUrl,   14,
    QMetaType::QUrl,
    QMetaType::Void, QMetaType::QString,   19,
    QMetaType::QStringList,
    QMetaType::Void, QMetaType::QUrl,   22,
    0x80000000 | 24,
    QMetaType::Void, QMetaType::QString,   26,
    QMetaType::Void, QMetaType::QStringList,   28,
    QMetaType::QStringList,
    QMetaType::Void, QMetaType::QString,   31,
    QMetaType::QString,
    QMetaType::Void, QMetaType::Int,   34,
    QMetaType::Int,
    QMetaType::ULongLong,
    0x80000000 | 37,
    QMetaType::Void, 0x80000000 | 37,   28,
    QMetaType::Void, 0x80000000 | 40,   41,
    0x80000000 | 40,
    QMetaType::Void, 0x80000000 | 44,   41,
    QMetaType::Void, 0x80000000 | 46,   41,
    0x80000000 | 46,
    QMetaType::Void, 0x80000000 | 49, QMetaType::QString,   50,   51,
    QMetaType::QString, 0x80000000 | 49,   50,
    QMetaType::Void, 0x80000000 | 54,   55,
    QMetaType::Void, 0x80000000 | 57, QMetaType::Bool,   58,   59,
    QMetaType::Void, 0x80000000 | 57,   58,
    0x80000000 | 54,
    QMetaType::Bool, 0x80000000 | 57,   58,
    QMetaType::Void, QMetaType::QString,   62,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   64,   65,
    0x80000000 | 67, QMetaType::Int, QMetaType::QString,   64,   51,
    QMetaType::QVariantMap, QMetaType::Int,   64,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   59,
    QMetaType::Void, QMetaType::Bool,   73,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   79,
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void FileDialogHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileDialogHandle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->finished((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->accepted(); break;
        case 2: _t->rejected(); break;
        case 3: _t->selectionFilesChanged(); break;
        case 4: _t->currentUrlChanged(); break;
        case 5: _t->selectedNameFilterChanged(); break;
        case 6: _t->setParent((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 7: { QWidget* _r = _t->widget();
            if (_a[0]) *reinterpret_cast< QWidget**>(_a[0]) = std::move(_r); }  break;
        case 8: _t->setDirectory((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->setDirectory((*reinterpret_cast< const QDir(*)>(_a[1]))); break;
        case 10: { QDir _r = _t->directory();
            if (_a[0]) *reinterpret_cast< QDir*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->setDirectoryUrl((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 12: { QUrl _r = _t->directoryUrl();
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->selectFile((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: { QStringList _r = _t->selectedFiles();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->selectUrl((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 16: { QList<QUrl> _r = _t->selectedUrls();
            if (_a[0]) *reinterpret_cast< QList<QUrl>*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->addDisableUrlScheme((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 18: _t->setNameFilters((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 19: { QStringList _r = _t->nameFilters();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->selectNameFilter((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 21: { QString _r = _t->selectedNameFilter();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 22: _t->selectNameFilterByIndex((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: { int _r = _t->selectedNameFilterIndex();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 24: { qulonglong _r = _t->winId();
            if (_a[0]) *reinterpret_cast< qulonglong*>(_a[0]) = std::move(_r); }  break;
        case 25: { QDir::Filters _r = _t->filter();
            if (_a[0]) *reinterpret_cast< QDir::Filters*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->setFilter((*reinterpret_cast< QDir::Filters(*)>(_a[1]))); break;
        case 27: _t->setViewMode((*reinterpret_cast< QFileDialog::ViewMode(*)>(_a[1]))); break;
        case 28: { QFileDialog::ViewMode _r = _t->viewMode();
            if (_a[0]) *reinterpret_cast< QFileDialog::ViewMode*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->setFileMode((*reinterpret_cast< QFileDialog::FileMode(*)>(_a[1]))); break;
        case 30: _t->setAcceptMode((*reinterpret_cast< QFileDialog::AcceptMode(*)>(_a[1]))); break;
        case 31: { QFileDialog::AcceptMode _r = _t->acceptMode();
            if (_a[0]) *reinterpret_cast< QFileDialog::AcceptMode*>(_a[0]) = std::move(_r); }  break;
        case 32: _t->setLabelText((*reinterpret_cast< QFileDialog::DialogLabel(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 33: { QString _r = _t->labelText((*reinterpret_cast< QFileDialog::DialogLabel(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 34: _t->setOptions((*reinterpret_cast< QFileDialog::Options(*)>(_a[1]))); break;
        case 35: _t->setOption((*reinterpret_cast< QFileDialog::Option(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 36: _t->setOption((*reinterpret_cast< QFileDialog::Option(*)>(_a[1]))); break;
        case 37: { QFileDialog::Options _r = _t->options();
            if (_a[0]) *reinterpret_cast< QFileDialog::Options*>(_a[0]) = std::move(_r); }  break;
        case 38: { bool _r = _t->testOption((*reinterpret_cast< QFileDialog::Option(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 39: _t->setCurrentInputName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 40: _t->addCustomWidget((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 41: { QDBusVariant _r = _t->getCustomWidgetValue((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusVariant*>(_a[0]) = std::move(_r); }  break;
        case 42: { QVariantMap _r = _t->allCustomWidgetsValue((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 43: _t->beginAddCustomWidget(); break;
        case 44: _t->endAddCustomWidget(); break;
        case 45: _t->setAllowMixedSelection((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 46: _t->setHideOnAccept((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 47: { bool _r = _t->hideOnAccept();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 48: _t->show(); break;
        case 49: _t->hide(); break;
        case 50: _t->accept(); break;
        case 51: _t->done((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 52: { int _r = _t->exec();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 53: _t->open(); break;
        case 54: _t->reject(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileDialogHandle::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialogHandle::finished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileDialogHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialogHandle::accepted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileDialogHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialogHandle::rejected)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FileDialogHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialogHandle::selectionFilesChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FileDialogHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialogHandle::currentUrlChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (FileDialogHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDialogHandle::selectedNameFilterChanged)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FileDialogHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_FileDialogHandle.data,
    qt_meta_data_FileDialogHandle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FileDialogHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileDialogHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FileDialogHandle.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int FileDialogHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 55)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 55;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 55)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 55;
    }
    return _id;
}

// SIGNAL 0
void FileDialogHandle::finished(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void FileDialogHandle::accepted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void FileDialogHandle::rejected()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void FileDialogHandle::selectionFilesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void FileDialogHandle::currentUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void FileDialogHandle::selectedNameFilterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
