/****************************************************************************
** Meta object code from reading C++ file 'recenteventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-recent/events/recenteventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'recenteventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_recent__RecentEventReceiver_t {
    QByteArrayData data[41];
    char stringdata0[593];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_recent__RecentEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_recent__RecentEventReceiver_t qt_meta_stringdata_dfmplugin_recent__RecentEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 37), // "dfmplugin_recent::RecentEvent..."
QT_MOC_LITERAL(1, 38, 22), // "handleWindowUrlChanged"
QT_MOC_LITERAL(2, 61, 0), // ""
QT_MOC_LITERAL(3, 62, 5), // "winId"
QT_MOC_LITERAL(4, 68, 3), // "url"
QT_MOC_LITERAL(5, 72, 23), // "handleRemoveFilesResult"
QT_MOC_LITERAL(6, 96, 11), // "QList<QUrl>"
QT_MOC_LITERAL(7, 108, 4), // "urls"
QT_MOC_LITERAL(8, 113, 2), // "ok"
QT_MOC_LITERAL(9, 116, 6), // "errMsg"
QT_MOC_LITERAL(10, 123, 22), // "handleFileRenameResult"
QT_MOC_LITERAL(11, 146, 15), // "QMap<QUrl,QUrl>"
QT_MOC_LITERAL(12, 162, 11), // "renamedUrls"
QT_MOC_LITERAL(13, 174, 19), // "handleFileCutResult"
QT_MOC_LITERAL(14, 194, 7), // "srcUrls"
QT_MOC_LITERAL(15, 202, 8), // "destUrls"
QT_MOC_LITERAL(16, 211, 16), // "customColumnRole"
QT_MOC_LITERAL(17, 228, 7), // "rootUrl"
QT_MOC_LITERAL(18, 236, 34), // "QList<dfmbase::Global::ItemRo..."
QT_MOC_LITERAL(19, 271, 8), // "roleList"
QT_MOC_LITERAL(20, 280, 21), // "customRoleDisplayName"
QT_MOC_LITERAL(21, 302, 26), // "dfmbase::Global::ItemRoles"
QT_MOC_LITERAL(22, 329, 4), // "role"
QT_MOC_LITERAL(23, 334, 8), // "QString*"
QT_MOC_LITERAL(24, 343, 11), // "displayName"
QT_MOC_LITERAL(25, 355, 14), // "detailViewIcon"
QT_MOC_LITERAL(26, 370, 8), // "iconName"
QT_MOC_LITERAL(27, 379, 19), // "sepateTitlebarCrumb"
QT_MOC_LITERAL(28, 399, 19), // "QList<QVariantMap>*"
QT_MOC_LITERAL(29, 419, 8), // "mapGroup"
QT_MOC_LITERAL(30, 428, 13), // "isTransparent"
QT_MOC_LITERAL(31, 442, 35), // "dfmbase::Global::TransparentS..."
QT_MOC_LITERAL(32, 478, 6), // "status"
QT_MOC_LITERAL(33, 485, 19), // "checkDragDropAction"
QT_MOC_LITERAL(34, 505, 5), // "urlTo"
QT_MOC_LITERAL(35, 511, 15), // "Qt::DropAction*"
QT_MOC_LITERAL(36, 527, 6), // "action"
QT_MOC_LITERAL(37, 534, 15), // "handleDropFiles"
QT_MOC_LITERAL(38, 550, 8), // "fromUrls"
QT_MOC_LITERAL(39, 559, 5), // "toUrl"
QT_MOC_LITERAL(40, 565, 27) // "handlePropertydialogDisable"

    },
    "dfmplugin_recent::RecentEventReceiver\0"
    "handleWindowUrlChanged\0\0winId\0url\0"
    "handleRemoveFilesResult\0QList<QUrl>\0"
    "urls\0ok\0errMsg\0handleFileRenameResult\0"
    "QMap<QUrl,QUrl>\0renamedUrls\0"
    "handleFileCutResult\0srcUrls\0destUrls\0"
    "customColumnRole\0rootUrl\0"
    "QList<dfmbase::Global::ItemRoles>*\0"
    "roleList\0customRoleDisplayName\0"
    "dfmbase::Global::ItemRoles\0role\0"
    "QString*\0displayName\0detailViewIcon\0"
    "iconName\0sepateTitlebarCrumb\0"
    "QList<QVariantMap>*\0mapGroup\0isTransparent\0"
    "dfmbase::Global::TransparentStatus*\0"
    "status\0checkDragDropAction\0urlTo\0"
    "Qt::DropAction*\0action\0handleDropFiles\0"
    "fromUrls\0toUrl\0handlePropertydialogDisable"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_recent__RecentEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   74,    2, 0x0a /* Public */,
       5,    3,   79,    2, 0x0a /* Public */,
      10,    4,   86,    2, 0x0a /* Public */,
      13,    4,   95,    2, 0x0a /* Public */,
      16,    2,  104,    2, 0x0a /* Public */,
      20,    3,  109,    2, 0x0a /* Public */,
      25,    2,  116,    2, 0x0a /* Public */,
      27,    2,  121,    2, 0x0a /* Public */,
      30,    2,  126,    2, 0x0a /* Public */,
      33,    3,  131,    2, 0x0a /* Public */,
      37,    2,  138,    2, 0x0a /* Public */,
      40,    1,  143,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,    3,    4,
    QMetaType::Void, 0x80000000 | 6, QMetaType::Bool, QMetaType::QString,    7,    8,    9,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 11, QMetaType::Bool, QMetaType::QString,    3,   12,    8,    9,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 6, QMetaType::Bool, QMetaType::QString,   14,   15,    8,    9,
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 18,   17,   19,
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 21, 0x80000000 | 23,    4,   22,   24,
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 23,    4,   26,
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 28,    4,   29,
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 31,    4,   32,
    QMetaType::Bool, 0x80000000 | 6, QMetaType::QUrl, 0x80000000 | 35,    7,   34,   36,
    QMetaType::Bool, 0x80000000 | 6, QMetaType::QUrl,   38,   39,
    QMetaType::Bool, QMetaType::QUrl,    4,

       0        // eod
};

void dfmplugin_recent::RecentEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RecentEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleWindowUrlChanged((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 1: _t->handleRemoveFilesResult((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 2: _t->handleFileRenameResult((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QMap<QUrl,QUrl>(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 3: _t->handleFileCutResult((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 4: { bool _r = _t->customColumnRole((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QList<dfmbase::Global::ItemRoles>*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->customRoleDisplayName((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const dfmbase::Global::ItemRoles(*)>(_a[2])),(*reinterpret_cast< QString*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->detailViewIcon((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QString*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->sepateTitlebarCrumb((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QList<QVariantMap>*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->isTransparent((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< dfmbase::Global::TransparentStatus*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->checkDragDropAction((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< Qt::DropAction*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->handleDropFiles((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->handlePropertydialogDisable((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<dfmbase::Global::ItemRoles>* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::Global::ItemRoles >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::Global::TransparentStatus* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_recent::RecentEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_recent__RecentEventReceiver.data,
    qt_meta_data_dfmplugin_recent__RecentEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_recent::RecentEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_recent::RecentEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_recent__RecentEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_recent::RecentEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
