// SPDX-FileCopyrightText: 2021 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "iconitemdelegate_p.h"

using namespace dfmplugin_workspace;

IconItemDelegatePrivate::IconItemDelegatePrivate(IconItemDelegate *qq)
    : BaseItemDelegatePrivate(qq)
{
}

IconItemDelegatePrivate::~IconItemDelegatePrivate()
{
}
