/****************************************************************************
** Meta object code from reading C++ file 'canvasmanagerbroker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-canvas/broker/canvasmanagerbroker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvasmanagerbroker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_canvas__CanvasManagerBroker_t {
    QByteArrayData data[18];
    char stringdata0[212];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_canvas__CanvasManagerBroker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_canvas__CanvasManagerBroker_t qt_meta_stringdata_ddplugin_canvas__CanvasManagerBroker = {
    {
QT_MOC_LITERAL(0, 0, 36), // "ddplugin_canvas::CanvasManage..."
QT_MOC_LITERAL(1, 37, 6), // "update"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 4), // "edit"
QT_MOC_LITERAL(4, 50, 3), // "url"
QT_MOC_LITERAL(5, 54, 9), // "iconLevel"
QT_MOC_LITERAL(6, 64, 12), // "setIconLevel"
QT_MOC_LITERAL(7, 77, 2), // "lv"
QT_MOC_LITERAL(8, 80, 11), // "autoArrange"
QT_MOC_LITERAL(9, 92, 14), // "setAutoArrange"
QT_MOC_LITERAL(10, 107, 2), // "on"
QT_MOC_LITERAL(11, 110, 13), // "fileInfoModel"
QT_MOC_LITERAL(12, 124, 19), // "QAbstractItemModel*"
QT_MOC_LITERAL(13, 144, 4), // "view"
QT_MOC_LITERAL(14, 149, 18), // "QAbstractItemView*"
QT_MOC_LITERAL(15, 168, 7), // "viewIdx"
QT_MOC_LITERAL(16, 176, 14), // "selectionModel"
QT_MOC_LITERAL(17, 191, 20) // "QItemSelectionModel*"

    },
    "ddplugin_canvas::CanvasManagerBroker\0"
    "update\0\0edit\0url\0iconLevel\0setIconLevel\0"
    "lv\0autoArrange\0setAutoArrange\0on\0"
    "fileInfoModel\0QAbstractItemModel*\0"
    "view\0QAbstractItemView*\0viewIdx\0"
    "selectionModel\0QItemSelectionModel*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_canvas__CanvasManagerBroker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x0a /* Public */,
       3,    1,   60,    2, 0x0a /* Public */,
       5,    0,   63,    2, 0x0a /* Public */,
       6,    1,   64,    2, 0x0a /* Public */,
       8,    0,   67,    2, 0x0a /* Public */,
       9,    1,   68,    2, 0x0a /* Public */,
      11,    0,   71,    2, 0x0a /* Public */,
      13,    1,   72,    2, 0x0a /* Public */,
      16,    0,   75,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,    4,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   10,
    0x80000000 | 12,
    0x80000000 | 14, QMetaType::Int,   15,
    0x80000000 | 17,

       0        // eod
};

void ddplugin_canvas::CanvasManagerBroker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CanvasManagerBroker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->update(); break;
        case 1: _t->edit((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 2: { int _r = _t->iconLevel();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->setIconLevel((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: { bool _r = _t->autoArrange();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->setAutoArrange((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: { QAbstractItemModel* _r = _t->fileInfoModel();
            if (_a[0]) *reinterpret_cast< QAbstractItemModel**>(_a[0]) = std::move(_r); }  break;
        case 7: { QAbstractItemView* _r = _t->view((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QAbstractItemView**>(_a[0]) = std::move(_r); }  break;
        case 8: { QItemSelectionModel* _r = _t->selectionModel();
            if (_a[0]) *reinterpret_cast< QItemSelectionModel**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_canvas::CanvasManagerBroker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_canvas__CanvasManagerBroker.data,
    qt_meta_data_ddplugin_canvas__CanvasManagerBroker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_canvas::CanvasManagerBroker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_canvas::CanvasManagerBroker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_canvas__CanvasManagerBroker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_canvas::CanvasManagerBroker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
