/****************************************************************************
** Meta object code from reading C++ file 'reportlogworker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-utils/reportlog/reportlogworker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'reportlogworker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_utils__ReportLogWorker_t {
    QByteArrayData data[20];
    char stringdata0[218];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_utils__ReportLogWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_utils__ReportLogWorker_t qt_meta_stringdata_dfmplugin_utils__ReportLogWorker = {
    {
QT_MOC_LITERAL(0, 0, 32), // "dfmplugin_utils::ReportLogWorker"
QT_MOC_LITERAL(1, 33, 9), // "commitLog"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 4), // "type"
QT_MOC_LITERAL(4, 49, 4), // "args"
QT_MOC_LITERAL(5, 54, 14), // "handleMenuData"
QT_MOC_LITERAL(6, 69, 4), // "name"
QT_MOC_LITERAL(7, 74, 11), // "QList<QUrl>"
QT_MOC_LITERAL(8, 86, 7), // "urlList"
QT_MOC_LITERAL(9, 94, 20), // "handleBlockMountData"
QT_MOC_LITERAL(10, 115, 2), // "id"
QT_MOC_LITERAL(11, 118, 6), // "result"
QT_MOC_LITERAL(12, 125, 24), // "handleDesktopStartUpData"
QT_MOC_LITERAL(13, 150, 3), // "key"
QT_MOC_LITERAL(14, 154, 4), // "data"
QT_MOC_LITERAL(15, 159, 24), // "handleMountNetworkResult"
QT_MOC_LITERAL(16, 184, 3), // "ret"
QT_MOC_LITERAL(17, 188, 21), // "dfmmount::DeviceError"
QT_MOC_LITERAL(18, 210, 3), // "err"
QT_MOC_LITERAL(19, 214, 3) // "msg"

    },
    "dfmplugin_utils::ReportLogWorker\0"
    "commitLog\0\0type\0args\0handleMenuData\0"
    "name\0QList<QUrl>\0urlList\0handleBlockMountData\0"
    "id\0result\0handleDesktopStartUpData\0"
    "key\0data\0handleMountNetworkResult\0ret\0"
    "dfmmount::DeviceError\0err\0msg"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_utils__ReportLogWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   39,    2, 0x0a /* Public */,
       5,    2,   44,    2, 0x0a /* Public */,
       9,    2,   49,    2, 0x0a /* Public */,
      12,    2,   54,    2, 0x0a /* Public */,
      15,    3,   59,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QVariantMap,    3,    4,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 7,    6,    8,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,   10,   11,
    QMetaType::Void, QMetaType::QString, QMetaType::QVariant,   13,   14,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 17, QMetaType::QString,   16,   18,   19,

       0        // eod
};

void dfmplugin_utils::ReportLogWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ReportLogWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->commitLog((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2]))); break;
        case 1: _t->handleMenuData((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2]))); break;
        case 2: _t->handleBlockMountData((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: _t->handleDesktopStartUpData((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 4: _t->handleMountNetworkResult((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< dfmmount::DeviceError(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_utils::ReportLogWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_utils__ReportLogWorker.data,
    qt_meta_data_dfmplugin_utils__ReportLogWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_utils::ReportLogWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_utils::ReportLogWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_utils__ReportLogWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_utils::ReportLogWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
