/****************************************************************************
** Meta object code from reading C++ file 'toolbarframe.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../../dde-file-manager/src/apps/dde-file-manager-preview/pluginpreviews/music-preview/toolbarframe.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'toolbarframe.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_plugin_filepreview__ToolBarFrame_t {
    QByteArrayData data[17];
    char stringdata0[245];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_plugin_filepreview__ToolBarFrame_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_plugin_filepreview__ToolBarFrame_t qt_meta_stringdata_plugin_filepreview__ToolBarFrame = {
    {
QT_MOC_LITERAL(0, 0, 32), // "plugin_filepreview::ToolBarFrame"
QT_MOC_LITERAL(1, 33, 18), // "onPlayStateChanged"
QT_MOC_LITERAL(2, 52, 0), // ""
QT_MOC_LITERAL(3, 53, 19), // "QMediaPlayer::State"
QT_MOC_LITERAL(4, 73, 5), // "state"
QT_MOC_LITERAL(5, 79, 19), // "onPlayStatusChanged"
QT_MOC_LITERAL(6, 99, 25), // "QMediaPlayer::MediaStatus"
QT_MOC_LITERAL(7, 125, 6), // "status"
QT_MOC_LITERAL(8, 132, 21), // "onPlayDurationChanged"
QT_MOC_LITERAL(9, 154, 8), // "duration"
QT_MOC_LITERAL(10, 163, 21), // "onPlayPositionChanged"
QT_MOC_LITERAL(11, 185, 3), // "pos"
QT_MOC_LITERAL(12, 189, 26), // "onPlayControlButtonClicked"
QT_MOC_LITERAL(13, 216, 12), // "seekPosition"
QT_MOC_LITERAL(14, 229, 4), // "play"
QT_MOC_LITERAL(15, 234, 5), // "pause"
QT_MOC_LITERAL(16, 240, 4) // "stop"

    },
    "plugin_filepreview::ToolBarFrame\0"
    "onPlayStateChanged\0\0QMediaPlayer::State\0"
    "state\0onPlayStatusChanged\0"
    "QMediaPlayer::MediaStatus\0status\0"
    "onPlayDurationChanged\0duration\0"
    "onPlayPositionChanged\0pos\0"
    "onPlayControlButtonClicked\0seekPosition\0"
    "play\0pause\0stop"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_plugin_filepreview__ToolBarFrame[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x0a /* Public */,
       5,    1,   62,    2, 0x0a /* Public */,
       8,    1,   65,    2, 0x0a /* Public */,
      10,    1,   68,    2, 0x0a /* Public */,
      12,    0,   71,    2, 0x0a /* Public */,
      13,    1,   72,    2, 0x0a /* Public */,
      14,    0,   75,    2, 0x0a /* Public */,
      15,    0,   76,    2, 0x0a /* Public */,
      16,    0,   77,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::LongLong,    9,
    QMetaType::Void, QMetaType::LongLong,   11,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void plugin_filepreview::ToolBarFrame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolBarFrame *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onPlayStateChanged((*reinterpret_cast< const QMediaPlayer::State(*)>(_a[1]))); break;
        case 1: _t->onPlayStatusChanged((*reinterpret_cast< const QMediaPlayer::MediaStatus(*)>(_a[1]))); break;
        case 2: _t->onPlayDurationChanged((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 3: _t->onPlayPositionChanged((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 4: _t->onPlayControlButtonClicked(); break;
        case 5: _t->seekPosition((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 6: _t->play(); break;
        case 7: _t->pause(); break;
        case 8: _t->stop(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaPlayer::State >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaPlayer::MediaStatus >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject plugin_filepreview::ToolBarFrame::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_plugin_filepreview__ToolBarFrame.data,
    qt_meta_data_plugin_filepreview__ToolBarFrame,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *plugin_filepreview::ToolBarFrame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *plugin_filepreview::ToolBarFrame::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_plugin_filepreview__ToolBarFrame.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int plugin_filepreview::ToolBarFrame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
