/****************************************************************************
** Meta object code from reading C++ file 'filetagcache.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-tag/utils/filetagcache.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filetagcache.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_tag__FileTagCacheWorker_t {
    QByteArrayData data[13];
    char stringdata0[194];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_tag__FileTagCacheWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_tag__FileTagCacheWorker_t qt_meta_stringdata_dfmplugin_tag__FileTagCacheWorker = {
    {
QT_MOC_LITERAL(0, 0, 33), // "dfmplugin_tag::FileTagCacheWo..."
QT_MOC_LITERAL(1, 34, 24), // "loadFileTagsFromDatabase"
QT_MOC_LITERAL(2, 59, 0), // ""
QT_MOC_LITERAL(3, 60, 10), // "onTagAdded"
QT_MOC_LITERAL(4, 71, 4), // "tags"
QT_MOC_LITERAL(5, 76, 12), // "onTagDeleted"
QT_MOC_LITERAL(6, 89, 18), // "onTagsColorChanged"
QT_MOC_LITERAL(7, 108, 15), // "tagAndColorName"
QT_MOC_LITERAL(8, 124, 17), // "onTagsNameChanged"
QT_MOC_LITERAL(9, 142, 9), // "oldAndNew"
QT_MOC_LITERAL(10, 152, 13), // "onFilesTagged"
QT_MOC_LITERAL(11, 166, 11), // "fileAndTags"
QT_MOC_LITERAL(12, 178, 15) // "onFilesUntagged"

    },
    "dfmplugin_tag::FileTagCacheWorker\0"
    "loadFileTagsFromDatabase\0\0onTagAdded\0"
    "tags\0onTagDeleted\0onTagsColorChanged\0"
    "tagAndColorName\0onTagsNameChanged\0"
    "oldAndNew\0onFilesTagged\0fileAndTags\0"
    "onFilesUntagged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_tag__FileTagCacheWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x0a /* Public */,
       3,    1,   50,    2, 0x0a /* Public */,
       5,    1,   53,    2, 0x0a /* Public */,
       6,    1,   56,    2, 0x0a /* Public */,
       8,    1,   59,    2, 0x0a /* Public */,
      10,    1,   62,    2, 0x0a /* Public */,
      12,    1,   65,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QVariantMap,    4,
    QMetaType::Void, QMetaType::QVariant,    4,
    QMetaType::Void, QMetaType::QVariantMap,    7,
    QMetaType::Void, QMetaType::QVariantMap,    9,
    QMetaType::Void, QMetaType::QVariantMap,   11,
    QMetaType::Void, QMetaType::QVariantMap,   11,

       0        // eod
};

void dfmplugin_tag::FileTagCacheWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileTagCacheWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->loadFileTagsFromDatabase(); break;
        case 1: _t->onTagAdded((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 2: _t->onTagDeleted((*reinterpret_cast< const QVariant(*)>(_a[1]))); break;
        case 3: _t->onTagsColorChanged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 4: _t->onTagsNameChanged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 5: _t->onFilesTagged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 6: _t->onFilesUntagged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_tag::FileTagCacheWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_tag__FileTagCacheWorker.data,
    qt_meta_data_dfmplugin_tag__FileTagCacheWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_tag::FileTagCacheWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_tag::FileTagCacheWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_tag__FileTagCacheWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_tag::FileTagCacheWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}
struct qt_meta_stringdata_dfmplugin_tag__FileTagCache_t {
    QByteArrayData data[1];
    char stringdata0[28];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_tag__FileTagCache_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_tag__FileTagCache_t qt_meta_stringdata_dfmplugin_tag__FileTagCache = {
    {
QT_MOC_LITERAL(0, 0, 27) // "dfmplugin_tag::FileTagCache"

    },
    "dfmplugin_tag::FileTagCache"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_tag__FileTagCache[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void dfmplugin_tag::FileTagCache::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_tag::FileTagCache::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_tag__FileTagCache.data,
    qt_meta_data_dfmplugin_tag__FileTagCache,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_tag::FileTagCache::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_tag::FileTagCache::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_tag__FileTagCache.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_tag::FileTagCache::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_dfmplugin_tag__FileTagCacheController_t {
    QByteArrayData data[12];
    char stringdata0[167];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_tag__FileTagCacheController_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_tag__FileTagCacheController_t qt_meta_stringdata_dfmplugin_tag__FileTagCacheController = {
    {
QT_MOC_LITERAL(0, 0, 37), // "dfmplugin_tag::FileTagCacheCo..."
QT_MOC_LITERAL(1, 38, 16), // "initLoadTagInfos"
QT_MOC_LITERAL(2, 55, 0), // ""
QT_MOC_LITERAL(3, 56, 11), // "filesTagged"
QT_MOC_LITERAL(4, 68, 11), // "fileAndTags"
QT_MOC_LITERAL(5, 80, 13), // "filesUntagged"
QT_MOC_LITERAL(6, 94, 12), // "newTagsAdded"
QT_MOC_LITERAL(7, 107, 4), // "tags"
QT_MOC_LITERAL(8, 112, 16), // "tagsColorChanged"
QT_MOC_LITERAL(9, 129, 9), // "oldAndNew"
QT_MOC_LITERAL(10, 139, 11), // "tagsDeleted"
QT_MOC_LITERAL(11, 151, 15) // "tagsNameChanged"

    },
    "dfmplugin_tag::FileTagCacheController\0"
    "initLoadTagInfos\0\0filesTagged\0fileAndTags\0"
    "filesUntagged\0newTagsAdded\0tags\0"
    "tagsColorChanged\0oldAndNew\0tagsDeleted\0"
    "tagsNameChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_tag__FileTagCacheController[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    1,   50,    2, 0x06 /* Public */,
       5,    1,   53,    2, 0x06 /* Public */,
       6,    1,   56,    2, 0x06 /* Public */,
       8,    1,   59,    2, 0x06 /* Public */,
      10,    1,   62,    2, 0x06 /* Public */,
      11,    1,   65,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QVariantMap,    4,
    QMetaType::Void, QMetaType::QVariantMap,    4,
    QMetaType::Void, QMetaType::QVariantMap,    7,
    QMetaType::Void, QMetaType::QVariantMap,    9,
    QMetaType::Void, QMetaType::QStringList,    7,
    QMetaType::Void, QMetaType::QVariantMap,    9,

       0        // eod
};

void dfmplugin_tag::FileTagCacheController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileTagCacheController *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->initLoadTagInfos(); break;
        case 1: _t->filesTagged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 2: _t->filesUntagged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 3: _t->newTagsAdded((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 4: _t->tagsColorChanged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 5: _t->tagsDeleted((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 6: _t->tagsNameChanged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileTagCacheController::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileTagCacheController::initLoadTagInfos)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileTagCacheController::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileTagCacheController::filesTagged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileTagCacheController::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileTagCacheController::filesUntagged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FileTagCacheController::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileTagCacheController::newTagsAdded)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FileTagCacheController::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileTagCacheController::tagsColorChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (FileTagCacheController::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileTagCacheController::tagsDeleted)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (FileTagCacheController::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileTagCacheController::tagsNameChanged)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_tag::FileTagCacheController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_tag__FileTagCacheController.data,
    qt_meta_data_dfmplugin_tag__FileTagCacheController,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_tag::FileTagCacheController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_tag::FileTagCacheController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_tag__FileTagCacheController.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_tag::FileTagCacheController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_tag::FileTagCacheController::initLoadTagInfos()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dfmplugin_tag::FileTagCacheController::filesTagged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_tag::FileTagCacheController::filesUntagged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_tag::FileTagCacheController::newTagsAdded(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_tag::FileTagCacheController::tagsColorChanged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmplugin_tag::FileTagCacheController::tagsDeleted(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void dfmplugin_tag::FileTagCacheController::tagsNameChanged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
