/****************************************************************************
** Meta object code from reading C++ file 'tagmanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-tag/utils/tagmanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tagmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_tag__TagManager_t {
    QByteArrayData data[17];
    char stringdata0[216];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_tag__TagManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_tag__TagManager_t qt_meta_stringdata_dfmplugin_tag__TagManager = {
    {
QT_MOC_LITERAL(0, 0, 25), // "dfmplugin_tag::TagManager"
QT_MOC_LITERAL(1, 26, 10), // "tagDeleted"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 7), // "tagName"
QT_MOC_LITERAL(4, 46, 11), // "filesTagged"
QT_MOC_LITERAL(5, 58, 11), // "fileAndTags"
QT_MOC_LITERAL(6, 70, 13), // "filesUntagged"
QT_MOC_LITERAL(7, 84, 11), // "filesHidden"
QT_MOC_LITERAL(8, 96, 10), // "onTagAdded"
QT_MOC_LITERAL(9, 107, 4), // "tags"
QT_MOC_LITERAL(10, 112, 12), // "onTagDeleted"
QT_MOC_LITERAL(11, 125, 17), // "onTagColorChanged"
QT_MOC_LITERAL(12, 143, 15), // "tagAndColorName"
QT_MOC_LITERAL(13, 159, 16), // "onTagNameChanged"
QT_MOC_LITERAL(14, 176, 9), // "oldAndNew"
QT_MOC_LITERAL(15, 186, 13), // "onFilesTagged"
QT_MOC_LITERAL(16, 200, 15) // "onFilesUntagged"

    },
    "dfmplugin_tag::TagManager\0tagDeleted\0"
    "\0tagName\0filesTagged\0fileAndTags\0"
    "filesUntagged\0filesHidden\0onTagAdded\0"
    "tags\0onTagDeleted\0onTagColorChanged\0"
    "tagAndColorName\0onTagNameChanged\0"
    "oldAndNew\0onFilesTagged\0onFilesUntagged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_tag__TagManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x06 /* Public */,
       4,    1,   67,    2, 0x06 /* Public */,
       6,    1,   70,    2, 0x06 /* Public */,
       7,    1,   73,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    1,   76,    2, 0x0a /* Public */,
      10,    1,   79,    2, 0x0a /* Public */,
      11,    1,   82,    2, 0x0a /* Public */,
      13,    1,   85,    2, 0x0a /* Public */,
      15,    1,   88,    2, 0x0a /* Public */,
      16,    1,   91,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QVariantMap,    5,
    QMetaType::Void, QMetaType::QVariantMap,    5,
    QMetaType::Void, QMetaType::QVariantMap,    5,

 // slots: parameters
    QMetaType::Void, QMetaType::QVariantMap,    9,
    QMetaType::Void, QMetaType::QStringList,    9,
    QMetaType::Void, QMetaType::QVariantMap,   12,
    QMetaType::Void, QMetaType::QVariantMap,   14,
    QMetaType::Void, QMetaType::QVariantMap,    5,
    QMetaType::Void, QMetaType::QVariantMap,    5,

       0        // eod
};

void dfmplugin_tag::TagManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TagManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->tagDeleted((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->filesTagged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 2: _t->filesUntagged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 3: _t->filesHidden((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 4: _t->onTagAdded((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 5: _t->onTagDeleted((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 6: _t->onTagColorChanged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 7: _t->onTagNameChanged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 8: _t->onFilesTagged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 9: _t->onFilesUntagged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TagManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagManager::tagDeleted)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TagManager::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagManager::filesTagged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TagManager::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagManager::filesUntagged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TagManager::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagManager::filesHidden)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_tag::TagManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_tag__TagManager.data,
    qt_meta_data_dfmplugin_tag__TagManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_tag::TagManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_tag::TagManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_tag__TagManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_tag::TagManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_tag::TagManager::tagDeleted(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_tag::TagManager::filesTagged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_tag::TagManager::filesUntagged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_tag::TagManager::filesHidden(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
