/****************************************************************************
** Meta object code from reading C++ file 'filesortworker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-workspace/utils/filesortworker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filesortworker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_workspace__FileSortWorker_t {
    QByteArrayData data[83];
    char stringdata0[1287];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_workspace__FileSortWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_workspace__FileSortWorker_t qt_meta_stringdata_dfmplugin_workspace__FileSortWorker = {
    {
QT_MOC_LITERAL(0, 0, 35), // "dfmplugin_workspace::FileSort..."
QT_MOC_LITERAL(1, 36, 10), // "insertRows"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 5), // "first"
QT_MOC_LITERAL(4, 54, 5), // "count"
QT_MOC_LITERAL(5, 60, 12), // "insertFinish"
QT_MOC_LITERAL(6, 73, 10), // "removeRows"
QT_MOC_LITERAL(7, 84, 12), // "removeFinish"
QT_MOC_LITERAL(8, 97, 16), // "requestFetchMore"
QT_MOC_LITERAL(9, 114, 17), // "selectAndEditFile"
QT_MOC_LITERAL(10, 132, 3), // "url"
QT_MOC_LITERAL(11, 136, 11), // "dataChanged"
QT_MOC_LITERAL(12, 148, 4), // "last"
QT_MOC_LITERAL(13, 153, 14), // "requestSetIdel"
QT_MOC_LITERAL(14, 168, 13), // "visiableCount"
QT_MOC_LITERAL(15, 182, 10), // "totalCount"
QT_MOC_LITERAL(16, 193, 9), // "updateRow"
QT_MOC_LITERAL(17, 203, 3), // "row"
QT_MOC_LITERAL(18, 207, 13), // "getSourceData"
QT_MOC_LITERAL(19, 221, 3), // "key"
QT_MOC_LITERAL(20, 225, 17), // "requestUpdateView"
QT_MOC_LITERAL(21, 243, 17), // "requestCursorWait"
QT_MOC_LITERAL(22, 261, 18), // "reqUestCloseCursor"
QT_MOC_LITERAL(23, 280, 23), // "requestUpdateTimerStart"
QT_MOC_LITERAL(24, 304, 21), // "requestSortByMimeType"
QT_MOC_LITERAL(25, 326, 27), // "handleIteratorLocalChildren"
QT_MOC_LITERAL(26, 354, 22), // "QList<SortInfoPointer>"
QT_MOC_LITERAL(27, 377, 8), // "children"
QT_MOC_LITERAL(28, 386, 39), // "dfmio::DEnumerator::SortRoleC..."
QT_MOC_LITERAL(29, 426, 8), // "sortRole"
QT_MOC_LITERAL(30, 435, 13), // "Qt::SortOrder"
QT_MOC_LITERAL(31, 449, 9), // "sortOrder"
QT_MOC_LITERAL(32, 459, 15), // "isMixDirAndFile"
QT_MOC_LITERAL(33, 475, 20), // "handleSourceChildren"
QT_MOC_LITERAL(34, 496, 10), // "isFinished"
QT_MOC_LITERAL(35, 507, 22), // "handleIteratorChildren"
QT_MOC_LITERAL(36, 530, 22), // "QList<FileInfoPointer>"
QT_MOC_LITERAL(37, 553, 5), // "infos"
QT_MOC_LITERAL(38, 559, 21), // "handleTraversalFinish"
QT_MOC_LITERAL(39, 581, 13), // "handleSortDir"
QT_MOC_LITERAL(40, 595, 6), // "parent"
QT_MOC_LITERAL(41, 602, 24), // "handleModelGetSourceData"
QT_MOC_LITERAL(42, 627, 13), // "handleFilters"
QT_MOC_LITERAL(43, 641, 13), // "QDir::Filters"
QT_MOC_LITERAL(44, 655, 7), // "filters"
QT_MOC_LITERAL(45, 663, 17), // "HandleNameFilters"
QT_MOC_LITERAL(46, 681, 16), // "handleFilterData"
QT_MOC_LITERAL(47, 698, 4), // "data"
QT_MOC_LITERAL(48, 703, 20), // "handleFilterCallFunc"
QT_MOC_LITERAL(49, 724, 22), // "FileViewFilterCallback"
QT_MOC_LITERAL(50, 747, 8), // "callback"
QT_MOC_LITERAL(51, 756, 19), // "onToggleHiddenFiles"
QT_MOC_LITERAL(52, 776, 23), // "onShowHiddenFileChanged"
QT_MOC_LITERAL(53, 800, 6), // "isShow"
QT_MOC_LITERAL(54, 807, 24), // "handleWatcherAddChildren"
QT_MOC_LITERAL(55, 832, 27), // "handleWatcherRemoveChildren"
QT_MOC_LITERAL(56, 860, 23), // "handleWatcherUpdateFile"
QT_MOC_LITERAL(57, 884, 15), // "SortInfoPointer"
QT_MOC_LITERAL(58, 900, 5), // "child"
QT_MOC_LITERAL(59, 906, 24), // "handleWatcherUpdateFiles"
QT_MOC_LITERAL(60, 931, 27), // "handleWatcherUpdateHideFile"
QT_MOC_LITERAL(61, 959, 6), // "hidUrl"
QT_MOC_LITERAL(62, 966, 12), // "handleResort"
QT_MOC_LITERAL(63, 979, 5), // "order"
QT_MOC_LITERAL(64, 985, 17), // "Global::ItemRoles"
QT_MOC_LITERAL(65, 1003, 21), // "onAppAttributeChanged"
QT_MOC_LITERAL(66, 1025, 33), // "Application::ApplicationAttri..."
QT_MOC_LITERAL(67, 1059, 2), // "aa"
QT_MOC_LITERAL(68, 1062, 5), // "value"
QT_MOC_LITERAL(69, 1068, 16), // "handleUpdateFile"
QT_MOC_LITERAL(70, 1085, 17), // "handleUpdateFiles"
QT_MOC_LITERAL(71, 1103, 11), // "QList<QUrl>"
QT_MOC_LITERAL(72, 1115, 4), // "urls"
QT_MOC_LITERAL(73, 1120, 13), // "handleRefresh"
QT_MOC_LITERAL(74, 1134, 20), // "handleClearThumbnail"
QT_MOC_LITERAL(75, 1155, 21), // "handleFileInfoUpdated"
QT_MOC_LITERAL(76, 1177, 7), // "infoPtr"
QT_MOC_LITERAL(77, 1185, 9), // "isLinkOrg"
QT_MOC_LITERAL(78, 1195, 24), // "handleUpdateRefreshFiles"
QT_MOC_LITERAL(79, 1220, 20), // "handleSortByMimeType"
QT_MOC_LITERAL(80, 1241, 17), // "handleCloseExpand"
QT_MOC_LITERAL(81, 1259, 20), // "handleSwitchTreeView"
QT_MOC_LITERAL(82, 1280, 6) // "isTree"

    },
    "dfmplugin_workspace::FileSortWorker\0"
    "insertRows\0\0first\0count\0insertFinish\0"
    "removeRows\0removeFinish\0requestFetchMore\0"
    "selectAndEditFile\0url\0dataChanged\0"
    "last\0requestSetIdel\0visiableCount\0"
    "totalCount\0updateRow\0row\0getSourceData\0"
    "key\0requestUpdateView\0requestCursorWait\0"
    "reqUestCloseCursor\0requestUpdateTimerStart\0"
    "requestSortByMimeType\0handleIteratorLocalChildren\0"
    "QList<SortInfoPointer>\0children\0"
    "dfmio::DEnumerator::SortRoleCompareFlag\0"
    "sortRole\0Qt::SortOrder\0sortOrder\0"
    "isMixDirAndFile\0handleSourceChildren\0"
    "isFinished\0handleIteratorChildren\0"
    "QList<FileInfoPointer>\0infos\0"
    "handleTraversalFinish\0handleSortDir\0"
    "parent\0handleModelGetSourceData\0"
    "handleFilters\0QDir::Filters\0filters\0"
    "HandleNameFilters\0handleFilterData\0"
    "data\0handleFilterCallFunc\0"
    "FileViewFilterCallback\0callback\0"
    "onToggleHiddenFiles\0onShowHiddenFileChanged\0"
    "isShow\0handleWatcherAddChildren\0"
    "handleWatcherRemoveChildren\0"
    "handleWatcherUpdateFile\0SortInfoPointer\0"
    "child\0handleWatcherUpdateFiles\0"
    "handleWatcherUpdateHideFile\0hidUrl\0"
    "handleResort\0order\0Global::ItemRoles\0"
    "onAppAttributeChanged\0"
    "Application::ApplicationAttribute\0aa\0"
    "value\0handleUpdateFile\0handleUpdateFiles\0"
    "QList<QUrl>\0urls\0handleRefresh\0"
    "handleClearThumbnail\0handleFileInfoUpdated\0"
    "infoPtr\0isLinkOrg\0handleUpdateRefreshFiles\0"
    "handleSortByMimeType\0handleCloseExpand\0"
    "handleSwitchTreeView\0isTree"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_workspace__FileSortWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      43,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      15,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,  229,    2, 0x06 /* Public */,
       5,    0,  234,    2, 0x06 /* Public */,
       6,    2,  235,    2, 0x06 /* Public */,
       7,    0,  240,    2, 0x06 /* Public */,
       8,    0,  241,    2, 0x06 /* Public */,
       9,    1,  242,    2, 0x06 /* Public */,
      11,    2,  245,    2, 0x06 /* Public */,
      13,    2,  250,    2, 0x06 /* Public */,
      16,    1,  255,    2, 0x06 /* Public */,
      18,    1,  258,    2, 0x06 /* Public */,
      20,    0,  261,    2, 0x06 /* Public */,
      21,    0,  262,    2, 0x06 /* Public */,
      22,    0,  263,    2, 0x06 /* Public */,
      23,    0,  264,    2, 0x06 /* Public */,
      24,    0,  265,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      25,    5,  266,    2, 0x0a /* Public */,
      33,    6,  277,    2, 0x0a /* Public */,
      35,    3,  290,    2, 0x0a /* Public */,
      38,    1,  297,    2, 0x0a /* Public */,
      39,    2,  300,    2, 0x0a /* Public */,
      41,    0,  305,    2, 0x0a /* Public */,
      42,    1,  306,    2, 0x0a /* Public */,
      45,    1,  309,    2, 0x0a /* Public */,
      46,    1,  312,    2, 0x0a /* Public */,
      48,    1,  315,    2, 0x0a /* Public */,
      51,    0,  318,    2, 0x0a /* Public */,
      52,    1,  319,    2, 0x0a /* Public */,
      54,    1,  322,    2, 0x0a /* Public */,
      55,    1,  325,    2, 0x0a /* Public */,
      56,    1,  328,    2, 0x0a /* Public */,
      59,    1,  331,    2, 0x0a /* Public */,
      60,    1,  334,    2, 0x0a /* Public */,
      62,    3,  337,    2, 0x0a /* Public */,
      65,    2,  344,    2, 0x0a /* Public */,
      69,    1,  349,    2, 0x0a /* Public */,
      70,    1,  352,    2, 0x0a /* Public */,
      73,    0,  355,    2, 0x0a /* Public */,
      74,    0,  356,    2, 0x0a /* Public */,
      75,    3,  357,    2, 0x0a /* Public */,
      78,    0,  364,    2, 0x0a /* Public */,
      79,    0,  365,    2, 0x0a /* Public */,
      80,    2,  366,    2, 0x0a /* Public */,
      81,    1,  371,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    3,    4,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    3,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,   10,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    3,   12,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   14,   15,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::QString,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, 0x80000000 | 26, 0x80000000 | 28, 0x80000000 | 30, QMetaType::Bool,   19,   27,   29,   31,   32,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 26, 0x80000000 | 28, 0x80000000 | 30, QMetaType::Bool, QMetaType::Bool,   19,   27,   29,   31,   32,   34,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 26, 0x80000000 | 36,   19,   27,   37,
    QMetaType::Void, QMetaType::QString,   19,
    QMetaType::Void, QMetaType::QString, QMetaType::QUrl,   19,   40,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 43,   44,
    QMetaType::Void, QMetaType::QStringList,   44,
    QMetaType::Void, QMetaType::QVariant,   47,
    QMetaType::Void, 0x80000000 | 49,   50,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   53,
    QMetaType::Void, 0x80000000 | 26,   27,
    QMetaType::Void, 0x80000000 | 26,   27,
    QMetaType::Bool, 0x80000000 | 57,   58,
    QMetaType::Void, 0x80000000 | 26,   27,
    QMetaType::Void, QMetaType::QUrl,   61,
    QMetaType::Void, 0x80000000 | 30, 0x80000000 | 64, QMetaType::Bool,   63,   29,   32,
    QMetaType::Void, 0x80000000 | 66, QMetaType::QVariant,   67,   68,
    QMetaType::Bool, QMetaType::QUrl,   10,
    QMetaType::Void, 0x80000000 | 71,   72,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QString, QMetaType::Bool,   10,   76,   77,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QUrl,   19,   40,
    QMetaType::Void, QMetaType::Bool,   82,

       0        // eod
};

void dfmplugin_workspace::FileSortWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileSortWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->insertRows((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 1: _t->insertFinish(); break;
        case 2: _t->removeRows((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 3: _t->removeFinish(); break;
        case 4: _t->requestFetchMore(); break;
        case 5: _t->selectAndEditFile((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 6: _t->dataChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 7: _t->requestSetIdel((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 8: _t->updateRow((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->getSourceData((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->requestUpdateView(); break;
        case 11: _t->requestCursorWait(); break;
        case 12: _t->reqUestCloseCursor(); break;
        case 13: _t->requestUpdateTimerStart(); break;
        case 14: _t->requestSortByMimeType(); break;
        case 15: _t->handleIteratorLocalChildren((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[2])),(*reinterpret_cast< const dfmio::DEnumerator::SortRoleCompareFlag(*)>(_a[3])),(*reinterpret_cast< const Qt::SortOrder(*)>(_a[4])),(*reinterpret_cast< const bool(*)>(_a[5]))); break;
        case 16: _t->handleSourceChildren((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[2])),(*reinterpret_cast< const dfmio::DEnumerator::SortRoleCompareFlag(*)>(_a[3])),(*reinterpret_cast< const Qt::SortOrder(*)>(_a[4])),(*reinterpret_cast< const bool(*)>(_a[5])),(*reinterpret_cast< const bool(*)>(_a[6]))); break;
        case 17: _t->handleIteratorChildren((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[2])),(*reinterpret_cast< const QList<FileInfoPointer>(*)>(_a[3]))); break;
        case 18: _t->handleTraversalFinish((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 19: _t->handleSortDir((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 20: _t->handleModelGetSourceData(); break;
        case 21: _t->handleFilters((*reinterpret_cast< QDir::Filters(*)>(_a[1]))); break;
        case 22: _t->HandleNameFilters((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 23: _t->handleFilterData((*reinterpret_cast< const QVariant(*)>(_a[1]))); break;
        case 24: _t->handleFilterCallFunc((*reinterpret_cast< FileViewFilterCallback(*)>(_a[1]))); break;
        case 25: _t->onToggleHiddenFiles(); break;
        case 26: _t->onShowHiddenFileChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 27: _t->handleWatcherAddChildren((*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[1]))); break;
        case 28: _t->handleWatcherRemoveChildren((*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[1]))); break;
        case 29: { bool _r = _t->handleWatcherUpdateFile((*reinterpret_cast< const SortInfoPointer(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->handleWatcherUpdateFiles((*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[1]))); break;
        case 31: _t->handleWatcherUpdateHideFile((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 32: _t->handleResort((*reinterpret_cast< const Qt::SortOrder(*)>(_a[1])),(*reinterpret_cast< const Global::ItemRoles(*)>(_a[2])),(*reinterpret_cast< const bool(*)>(_a[3]))); break;
        case 33: _t->onAppAttributeChanged((*reinterpret_cast< Application::ApplicationAttribute(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 34: { bool _r = _t->handleUpdateFile((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->handleUpdateFiles((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 36: _t->handleRefresh(); break;
        case 37: _t->handleClearThumbnail(); break;
        case 38: _t->handleFileInfoUpdated((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const bool(*)>(_a[3]))); break;
        case 39: _t->handleUpdateRefreshFiles(); break;
        case 40: _t->handleSortByMimeType(); break;
        case 41: _t->handleCloseExpand((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 42: _t->handleSwitchTreeView((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmio::DEnumerator::SortRoleCompareFlag >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmio::DEnumerator::SortRoleCompareFlag >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<FileInfoPointer> >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDir::Filters >(); break;
            }
            break;
        case 27:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< SortInfoPointer >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            }
            break;
        case 35:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileSortWorker::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::insertRows)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::insertFinish)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::removeRows)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::removeFinish)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::requestFetchMore)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::selectAndEditFile)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::dataChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::requestSetIdel)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::updateRow)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::getSourceData)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::requestUpdateView)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::requestCursorWait)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::reqUestCloseCursor)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::requestUpdateTimerStart)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (FileSortWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileSortWorker::requestSortByMimeType)) {
                *result = 14;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_workspace::FileSortWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_workspace__FileSortWorker.data,
    qt_meta_data_dfmplugin_workspace__FileSortWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_workspace::FileSortWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_workspace::FileSortWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_workspace__FileSortWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_workspace::FileSortWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_workspace::FileSortWorker::insertRows(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_workspace::FileSortWorker::insertFinish()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dfmplugin_workspace::FileSortWorker::removeRows(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_workspace::FileSortWorker::removeFinish()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void dfmplugin_workspace::FileSortWorker::requestFetchMore()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void dfmplugin_workspace::FileSortWorker::selectAndEditFile(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void dfmplugin_workspace::FileSortWorker::dataChanged(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void dfmplugin_workspace::FileSortWorker::requestSetIdel(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void dfmplugin_workspace::FileSortWorker::updateRow(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void dfmplugin_workspace::FileSortWorker::getSourceData(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void dfmplugin_workspace::FileSortWorker::requestUpdateView()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void dfmplugin_workspace::FileSortWorker::requestCursorWait()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void dfmplugin_workspace::FileSortWorker::reqUestCloseCursor()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void dfmplugin_workspace::FileSortWorker::requestUpdateTimerStart()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void dfmplugin_workspace::FileSortWorker::requestSortByMimeType()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
