/****************************************************************************
** Meta object code from reading C++ file 'bluetoothadapter.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-utils/bluetooth/private/bluetoothadapter.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bluetoothadapter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_utils__BluetoothAdapter_t {
    QByteArrayData data[13];
    char stringdata0[163];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_utils__BluetoothAdapter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_utils__BluetoothAdapter_t qt_meta_stringdata_dfmplugin_utils__BluetoothAdapter = {
    {
QT_MOC_LITERAL(0, 0, 33), // "dfmplugin_utils::BluetoothAda..."
QT_MOC_LITERAL(1, 34, 11), // "deviceAdded"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 22), // "const BluetoothDevice*"
QT_MOC_LITERAL(4, 70, 6), // "device"
QT_MOC_LITERAL(5, 77, 13), // "deviceRemoved"
QT_MOC_LITERAL(6, 91, 8), // "deviceId"
QT_MOC_LITERAL(7, 100, 11), // "nameChanged"
QT_MOC_LITERAL(8, 112, 4), // "name"
QT_MOC_LITERAL(9, 117, 14), // "poweredChanged"
QT_MOC_LITERAL(10, 132, 7), // "powered"
QT_MOC_LITERAL(11, 140, 9), // "addDevice"
QT_MOC_LITERAL(12, 150, 12) // "removeDevice"

    },
    "dfmplugin_utils::BluetoothAdapter\0"
    "deviceAdded\0\0const BluetoothDevice*\0"
    "device\0deviceRemoved\0deviceId\0nameChanged\0"
    "name\0poweredChanged\0powered\0addDevice\0"
    "removeDevice"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_utils__BluetoothAdapter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       5,    1,   47,    2, 0x06 /* Public */,
       7,    1,   50,    2, 0x06 /* Public */,
       9,    1,   53,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    1,   56,    2, 0x0a /* Public */,
      12,    1,   59,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::Bool,   10,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QString,    6,

       0        // eod
};

void dfmplugin_utils::BluetoothAdapter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BluetoothAdapter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->deviceAdded((*reinterpret_cast< const BluetoothDevice*(*)>(_a[1]))); break;
        case 1: _t->deviceRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->nameChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->poweredChanged((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 4: _t->addDevice((*reinterpret_cast< const BluetoothDevice*(*)>(_a[1]))); break;
        case 5: _t->removeDevice((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (BluetoothAdapter::*)(const BluetoothDevice * ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothAdapter::deviceAdded)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (BluetoothAdapter::*)(const QString & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothAdapter::deviceRemoved)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (BluetoothAdapter::*)(const QString & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothAdapter::nameChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (BluetoothAdapter::*)(const bool & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothAdapter::poweredChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_utils::BluetoothAdapter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_utils__BluetoothAdapter.data,
    qt_meta_data_dfmplugin_utils__BluetoothAdapter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_utils::BluetoothAdapter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_utils::BluetoothAdapter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_utils__BluetoothAdapter.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_utils::BluetoothAdapter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_utils::BluetoothAdapter::deviceAdded(const BluetoothDevice * _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< dfmplugin_utils::BluetoothAdapter *>(this), &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_utils::BluetoothAdapter::deviceRemoved(const QString & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< dfmplugin_utils::BluetoothAdapter *>(this), &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_utils::BluetoothAdapter::nameChanged(const QString & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< dfmplugin_utils::BluetoothAdapter *>(this), &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_utils::BluetoothAdapter::poweredChanged(const bool & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< dfmplugin_utils::BluetoothAdapter *>(this), &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
