/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a devicemanageradaptor -i devicemanagerdbus.h -l DeviceManagerDBus /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.Daemon.DeviceManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "devicemanageradaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class DeviceManagerAdaptor
 */

DeviceManagerAdaptor::DeviceManagerAdaptor(DeviceManagerDBus *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

DeviceManagerAdaptor::~DeviceManagerAdaptor()
{
    // destructor
}

void DeviceManagerAdaptor::DetachAllMountedDevices()
{
    // handle method call org.deepin.Filemanager.Daemon.DeviceManager.DetachAllMountedDevices
    parent()->DetachAllMountedDevices();
}

void DeviceManagerAdaptor::DetachBlockDevice(const QString &id)
{
    // handle method call org.deepin.Filemanager.Daemon.DeviceManager.DetachBlockDevice
    parent()->DetachBlockDevice(id);
}

void DeviceManagerAdaptor::DetachProtocolDevice(const QString &id)
{
    // handle method call org.deepin.Filemanager.Daemon.DeviceManager.DetachProtocolDevice
    parent()->DetachProtocolDevice(id);
}

QStringList DeviceManagerAdaptor::GetBlockDevicesIdList(int opts)
{
    // handle method call org.deepin.Filemanager.Daemon.DeviceManager.GetBlockDevicesIdList
    return parent()->GetBlockDevicesIdList(opts);
}

QStringList DeviceManagerAdaptor::GetProtocolDevicesIdList()
{
    // handle method call org.deepin.Filemanager.Daemon.DeviceManager.GetProtocolDevicesIdList
    return parent()->GetProtocolDevicesIdList();
}

bool DeviceManagerAdaptor::IsMonotorWorking()
{
    // handle method call org.deepin.Filemanager.Daemon.DeviceManager.IsMonotorWorking
    return parent()->IsMonotorWorking();
}

QVariantMap DeviceManagerAdaptor::QueryBlockDeviceInfo(const QString &id, bool reload)
{
    // handle method call org.deepin.Filemanager.Daemon.DeviceManager.QueryBlockDeviceInfo
    return parent()->QueryBlockDeviceInfo(id, reload);
}

QVariantMap DeviceManagerAdaptor::QueryProtocolDeviceInfo(const QString &id, bool reload)
{
    // handle method call org.deepin.Filemanager.Daemon.DeviceManager.QueryProtocolDeviceInfo
    return parent()->QueryProtocolDeviceInfo(id, reload);
}


#include "devicemanageradaptor.moc"
