/****************************************************************************
** Meta object code from reading C++ file 'operationsstackmanagerdbus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/daemon/daemonplugin-core/operationsstackmanagerdbus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'operationsstackmanagerdbus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OperationsStackManagerDbus_t {
    QByteArrayData data[12];
    char stringdata0[226];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OperationsStackManagerDbus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OperationsStackManagerDbus_t qt_meta_stringdata_OperationsStackManagerDbus = {
    {
QT_MOC_LITERAL(0, 0, 26), // "OperationsStackManagerDbus"
QT_MOC_LITERAL(1, 27, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 43, 52), // "org.deepin.Filemanager.Daemon..."
QT_MOC_LITERAL(3, 96, 14), // "SaveOperations"
QT_MOC_LITERAL(4, 111, 0), // ""
QT_MOC_LITERAL(5, 112, 6), // "values"
QT_MOC_LITERAL(6, 119, 15), // "CleanOperations"
QT_MOC_LITERAL(7, 135, 20), // "RevocationOperations"
QT_MOC_LITERAL(8, 156, 18), // "SaveRedoOperations"
QT_MOC_LITERAL(9, 175, 24), // "RevocationRedoOperations"
QT_MOC_LITERAL(10, 200, 20), // "CleanOperationsByUrl"
QT_MOC_LITERAL(11, 221, 4) // "urls"

    },
    "OperationsStackManagerDbus\0D-Bus Interface\0"
    "org.deepin.Filemanager.Daemon.OperationsStackManager\0"
    "SaveOperations\0\0values\0CleanOperations\0"
    "RevocationOperations\0SaveRedoOperations\0"
    "RevocationRedoOperations\0CleanOperationsByUrl\0"
    "urls"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OperationsStackManagerDbus[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       6,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // slots: name, argc, parameters, tag, flags
       3,    1,   46,    4, 0x0a /* Public */,
       6,    0,   49,    4, 0x0a /* Public */,
       7,    0,   50,    4, 0x0a /* Public */,
       8,    1,   51,    4, 0x0a /* Public */,
       9,    0,   54,    4, 0x0a /* Public */,
      10,    1,   55,    4, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::QVariantMap,    5,
    QMetaType::Void,
    QMetaType::QVariantMap,
    QMetaType::Void, QMetaType::QVariantMap,    5,
    QMetaType::QVariantMap,
    QMetaType::Void, QMetaType::QStringList,   11,

       0        // eod
};

void OperationsStackManagerDbus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OperationsStackManagerDbus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->SaveOperations((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 1: _t->CleanOperations(); break;
        case 2: { QVariantMap _r = _t->RevocationOperations();
            if (_a[0]) *reinterpret_cast< QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->SaveRedoOperations((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 4: { QVariantMap _r = _t->RevocationRedoOperations();
            if (_a[0]) *reinterpret_cast< QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->CleanOperationsByUrl((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject OperationsStackManagerDbus::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_OperationsStackManagerDbus.data,
    qt_meta_data_OperationsStackManagerDbus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OperationsStackManagerDbus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OperationsStackManagerDbus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OperationsStackManagerDbus.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OperationsStackManagerDbus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
