/****************************************************************************
** Meta object code from reading C++ file 'diskencryptdbus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../dde-file-manager/src/services/diskencrypt/diskencryptdbus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'diskencryptdbus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DiskEncryptDBus_t {
    QByteArrayData data[37];
    char stringdata0[536];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DiskEncryptDBus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DiskEncryptDBus_t qt_meta_stringdata_DiskEncryptDBus = {
    {
QT_MOC_LITERAL(0, 0, 15), // "DiskEncryptDBus"
QT_MOC_LITERAL(1, 16, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 32, 34), // "org.deepin.Filemanager.DiskEn..."
QT_MOC_LITERAL(3, 67, 24), // "PrepareEncryptDiskResult"
QT_MOC_LITERAL(4, 92, 0), // ""
QT_MOC_LITERAL(5, 93, 6), // "device"
QT_MOC_LITERAL(6, 100, 7), // "devName"
QT_MOC_LITERAL(7, 108, 5), // "jobID"
QT_MOC_LITERAL(8, 114, 7), // "errCode"
QT_MOC_LITERAL(9, 122, 17), // "EncryptDiskResult"
QT_MOC_LITERAL(10, 140, 3), // "msg"
QT_MOC_LITERAL(11, 144, 17), // "DecryptDiskResult"
QT_MOC_LITERAL(12, 162, 22), // "ChangePassphressResult"
QT_MOC_LITERAL(13, 185, 15), // "EncryptProgress"
QT_MOC_LITERAL(14, 201, 8), // "progress"
QT_MOC_LITERAL(15, 210, 15), // "DecryptProgress"
QT_MOC_LITERAL(16, 226, 20), // "RequestEncryptParams"
QT_MOC_LITERAL(17, 247, 9), // "encConfig"
QT_MOC_LITERAL(18, 257, 18), // "PrepareEncryptDisk"
QT_MOC_LITERAL(19, 276, 6), // "params"
QT_MOC_LITERAL(20, 283, 11), // "DecryptDisk"
QT_MOC_LITERAL(21, 295, 23), // "ChangeEncryptPassphress"
QT_MOC_LITERAL(22, 319, 18), // "IgnoreParamRequest"
QT_MOC_LITERAL(23, 338, 16), // "ResumeEncryption"
QT_MOC_LITERAL(24, 355, 16), // "SetEncryptParams"
QT_MOC_LITERAL(25, 372, 13), // "QueryTPMToken"
QT_MOC_LITERAL(26, 386, 13), // "EncryptStatus"
QT_MOC_LITERAL(27, 400, 14), // "HasPendingTask"
QT_MOC_LITERAL(28, 415, 15), // "IsWorkerRunning"
QT_MOC_LITERAL(29, 431, 20), // "UnfinishedDecryptJob"
QT_MOC_LITERAL(30, 452, 29), // "onFstabDiskEncProgressUpdated"
QT_MOC_LITERAL(31, 482, 3), // "dev"
QT_MOC_LITERAL(32, 486, 6), // "offset"
QT_MOC_LITERAL(33, 493, 5), // "total"
QT_MOC_LITERAL(34, 499, 22), // "onFstabDiskEncFinished"
QT_MOC_LITERAL(35, 522, 6), // "result"
QT_MOC_LITERAL(36, 529, 6) // "errstr"

    },
    "DiskEncryptDBus\0D-Bus Interface\0"
    "org.deepin.Filemanager.DiskEncrypt\0"
    "PrepareEncryptDiskResult\0\0device\0"
    "devName\0jobID\0errCode\0EncryptDiskResult\0"
    "msg\0DecryptDiskResult\0ChangePassphressResult\0"
    "EncryptProgress\0progress\0DecryptProgress\0"
    "RequestEncryptParams\0encConfig\0"
    "PrepareEncryptDisk\0params\0DecryptDisk\0"
    "ChangeEncryptPassphress\0IgnoreParamRequest\0"
    "ResumeEncryption\0SetEncryptParams\0"
    "QueryTPMToken\0EncryptStatus\0HasPendingTask\0"
    "IsWorkerRunning\0UnfinishedDecryptJob\0"
    "onFstabDiskEncProgressUpdated\0dev\0"
    "offset\0total\0onFstabDiskEncFinished\0"
    "result\0errstr"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DiskEncryptDBus[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      20,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    4,  116,    4, 0x06 /* Public */,
       9,    4,  125,    4, 0x06 /* Public */,
      11,    4,  134,    4, 0x06 /* Public */,
      12,    4,  143,    4, 0x06 /* Public */,
      13,    3,  152,    4, 0x06 /* Public */,
      15,    3,  159,    4, 0x06 /* Public */,
      16,    1,  166,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    1,  169,    4, 0x0a /* Public */,
      20,    1,  172,    4, 0x0a /* Public */,
      21,    1,  175,    4, 0x0a /* Public */,
      22,    0,  178,    4, 0x0a /* Public */,
      23,    1,  179,    4, 0x0a /* Public */,
      24,    1,  182,    4, 0x0a /* Public */,
      25,    1,  185,    4, 0x0a /* Public */,
      26,    1,  188,    4, 0x0a /* Public */,
      27,    0,  191,    4, 0x0a /* Public */,
      28,    0,  192,    4, 0x0a /* Public */,
      29,    0,  193,    4, 0x0a /* Public */,
      30,    3,  194,    4, 0x08 /* Private */,
      34,    3,  201,    4, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::Int,    5,    6,    7,    8,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int, QMetaType::QString,    5,    6,    8,   10,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::Int,    5,    6,    7,    8,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::Int,    5,    6,    7,    8,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Double,    5,    6,   14,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Double,    5,    6,   14,
    QMetaType::Void, QMetaType::QVariantMap,   17,

 // slots: parameters
    QMetaType::QString, QMetaType::QVariantMap,   19,
    QMetaType::QString, QMetaType::QVariantMap,   19,
    QMetaType::QString, QMetaType::QVariantMap,   19,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QVariantMap,   19,
    QMetaType::QString, QMetaType::QString,    5,
    QMetaType::Int, QMetaType::QString,    5,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString, QMetaType::LongLong, QMetaType::LongLong,   31,   32,   33,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::QString,   31,   35,   36,

       0        // eod
};

void DiskEncryptDBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DiskEncryptDBus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->PrepareEncryptDiskResult((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 1: _t->EncryptDiskResult((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 2: _t->DecryptDiskResult((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 3: _t->ChangePassphressResult((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 4: _t->EncryptProgress((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 5: _t->DecryptProgress((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 6: _t->RequestEncryptParams((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 7: { QString _r = _t->PrepareEncryptDisk((*reinterpret_cast< const QVariantMap(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 8: { QString _r = _t->DecryptDisk((*reinterpret_cast< const QVariantMap(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 9: { QString _r = _t->ChangeEncryptPassphress((*reinterpret_cast< const QVariantMap(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->IgnoreParamRequest(); break;
        case 11: _t->ResumeEncryption((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->SetEncryptParams((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 13: { QString _r = _t->QueryTPMToken((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 14: { int _r = _t->EncryptStatus((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->HasPendingTask();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->IsWorkerRunning();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { QString _r = _t->UnfinishedDecryptJob();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->onFstabDiskEncProgressUpdated((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2])),(*reinterpret_cast< qint64(*)>(_a[3]))); break;
        case 19: _t->onFstabDiskEncFinished((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DiskEncryptDBus::*)(const QString & , const QString & , const QString & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DiskEncryptDBus::PrepareEncryptDiskResult)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DiskEncryptDBus::*)(const QString & , const QString & , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DiskEncryptDBus::EncryptDiskResult)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DiskEncryptDBus::*)(const QString & , const QString & , const QString & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DiskEncryptDBus::DecryptDiskResult)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DiskEncryptDBus::*)(const QString & , const QString & , const QString & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DiskEncryptDBus::ChangePassphressResult)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DiskEncryptDBus::*)(const QString & , const QString & , double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DiskEncryptDBus::EncryptProgress)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DiskEncryptDBus::*)(const QString & , const QString & , double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DiskEncryptDBus::DecryptProgress)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DiskEncryptDBus::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DiskEncryptDBus::RequestEncryptParams)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DiskEncryptDBus::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusService::staticMetaObject>(),
    qt_meta_stringdata_DiskEncryptDBus.data,
    qt_meta_data_DiskEncryptDBus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DiskEncryptDBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DiskEncryptDBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DiskEncryptDBus.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QDBusService::qt_metacast(_clname);
}

int DiskEncryptDBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusService::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void DiskEncryptDBus::PrepareEncryptDiskResult(const QString & _t1, const QString & _t2, const QString & _t3, int _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DiskEncryptDBus::EncryptDiskResult(const QString & _t1, const QString & _t2, int _t3, const QString & _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DiskEncryptDBus::DecryptDiskResult(const QString & _t1, const QString & _t2, const QString & _t3, int _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void DiskEncryptDBus::ChangePassphressResult(const QString & _t1, const QString & _t2, const QString & _t3, int _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void DiskEncryptDBus::EncryptProgress(const QString & _t1, const QString & _t2, double _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void DiskEncryptDBus::DecryptProgress(const QString & _t1, const QString & _t2, double _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void DiskEncryptDBus::RequestEncryptParams(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
