/****************************************************************************
** Meta object code from reading C++ file 'textindexdbus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../dde-file-manager/src/services/textindex/textindexdbus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textindexdbus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TextIndexDBus_t {
    QByteArrayData data[18];
    char stringdata0[244];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TextIndexDBus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TextIndexDBus_t qt_meta_stringdata_TextIndexDBus = {
    {
QT_MOC_LITERAL(0, 0, 13), // "TextIndexDBus"
QT_MOC_LITERAL(1, 14, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 30, 32), // "org.deepin.Filemanager.TextIndex"
QT_MOC_LITERAL(3, 63, 12), // "TaskFinished"
QT_MOC_LITERAL(4, 76, 0), // ""
QT_MOC_LITERAL(5, 77, 4), // "type"
QT_MOC_LITERAL(6, 82, 4), // "path"
QT_MOC_LITERAL(7, 87, 7), // "success"
QT_MOC_LITERAL(8, 95, 19), // "TaskProgressChanged"
QT_MOC_LITERAL(9, 115, 5), // "count"
QT_MOC_LITERAL(10, 121, 15), // "CreateIndexTask"
QT_MOC_LITERAL(11, 137, 15), // "UpdateIndexTask"
QT_MOC_LITERAL(12, 153, 15), // "RemoveIndexTask"
QT_MOC_LITERAL(13, 169, 5), // "paths"
QT_MOC_LITERAL(14, 175, 15), // "StopCurrentTask"
QT_MOC_LITERAL(15, 191, 14), // "HasRunningTask"
QT_MOC_LITERAL(16, 206, 19), // "IndexDatabaseExists"
QT_MOC_LITERAL(17, 226, 17) // "GetLastUpdateTime"

    },
    "TextIndexDBus\0D-Bus Interface\0"
    "org.deepin.Filemanager.TextIndex\0"
    "TaskFinished\0\0type\0path\0success\0"
    "TaskProgressChanged\0count\0CreateIndexTask\0"
    "UpdateIndexTask\0RemoveIndexTask\0paths\0"
    "StopCurrentTask\0HasRunningTask\0"
    "IndexDatabaseExists\0GetLastUpdateTime"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TextIndexDBus[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       9,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    3,   61,    4, 0x06 /* Public */,
       8,    3,   68,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,   75,    4, 0x0a /* Public */,
      11,    1,   78,    4, 0x0a /* Public */,
      12,    1,   81,    4, 0x0a /* Public */,
      14,    0,   84,    4, 0x0a /* Public */,
      15,    0,   85,    4, 0x0a /* Public */,
      16,    0,   86,    4, 0x0a /* Public */,
      17,    0,   87,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Bool,    5,    6,    7,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::LongLong,    5,    6,    9,

 // slots: parameters
    QMetaType::Bool, QMetaType::QString,    6,
    QMetaType::Bool, QMetaType::QString,    6,
    QMetaType::Bool, QMetaType::QStringList,   13,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::QString,

       0        // eod
};

void TextIndexDBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TextIndexDBus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->TaskFinished((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 1: _t->TaskProgressChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< qint64(*)>(_a[3]))); break;
        case 2: { bool _r = _t->CreateIndexTask((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->UpdateIndexTask((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->RemoveIndexTask((*reinterpret_cast< const QStringList(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->StopCurrentTask();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->HasRunningTask();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->IndexDatabaseExists();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { QString _r = _t->GetLastUpdateTime();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TextIndexDBus::*)(const QString & , const QString & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextIndexDBus::TaskFinished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TextIndexDBus::*)(const QString & , const QString & , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextIndexDBus::TaskProgressChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TextIndexDBus::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TextIndexDBus.data,
    qt_meta_data_TextIndexDBus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TextIndexDBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextIndexDBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TextIndexDBus.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int TextIndexDBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void TextIndexDBus::TaskFinished(const QString & _t1, const QString & _t2, bool _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TextIndexDBus::TaskProgressChanged(const QString & _t1, const QString & _t2, qint64 _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
