/****************************************************************************
** Meta object code from reading C++ file 'vaultclock.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/daemon/daemonplugin-vault/dbus/private/vaultclock.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vaultclock.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_VaultClock_t {
    QByteArrayData data[12];
    char stringdata0[137];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_VaultClock_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_VaultClock_t qt_meta_stringdata_VaultClock = {
    {
QT_MOC_LITERAL(0, 0, 10), // "VaultClock"
QT_MOC_LITERAL(1, 11, 14), // "setRefreshTime"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 4), // "time"
QT_MOC_LITERAL(4, 32, 14), // "getLastestTime"
QT_MOC_LITERAL(5, 47, 11), // "getSelfTime"
QT_MOC_LITERAL(6, 59, 20), // "isLockEventTriggered"
QT_MOC_LITERAL(7, 80, 16), // "triggerLockEvent"
QT_MOC_LITERAL(8, 97, 14), // "clearLockEvent"
QT_MOC_LITERAL(9, 112, 11), // "addTickTime"
QT_MOC_LITERAL(10, 124, 7), // "seconds"
QT_MOC_LITERAL(11, 132, 4) // "tick"

    },
    "VaultClock\0setRefreshTime\0\0time\0"
    "getLastestTime\0getSelfTime\0"
    "isLockEventTriggered\0triggerLockEvent\0"
    "clearLockEvent\0addTickTime\0seconds\0"
    "tick"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_VaultClock[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x0a /* Public */,
       4,    0,   57,    2, 0x0a /* Public */,
       5,    0,   58,    2, 0x0a /* Public */,
       6,    0,   59,    2, 0x0a /* Public */,
       7,    0,   60,    2, 0x0a /* Public */,
       8,    0,   61,    2, 0x0a /* Public */,
       9,    1,   62,    2, 0x0a /* Public */,
      11,    0,   65,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::ULongLong,    3,
    QMetaType::ULongLong,
    QMetaType::ULongLong,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::LongLong,   10,
    QMetaType::Void,

       0        // eod
};

void VaultClock::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<VaultClock *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setRefreshTime((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 1: { quint64 _r = _t->getLastestTime();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 2: { quint64 _r = _t->getSelfTime();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->isLockEventTriggered();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->triggerLockEvent(); break;
        case 5: _t->clearLockEvent(); break;
        case 6: _t->addTickTime((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 7: _t->tick(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject VaultClock::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_VaultClock.data,
    qt_meta_data_VaultClock,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *VaultClock::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VaultClock::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_VaultClock.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int VaultClock::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
