/****************************************************************************
** Meta object code from reading C++ file 'canvasmanager_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-canvas/private/canvasmanager_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvasmanager_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_canvas__CanvasManagerPrivate_t {
    QByteArrayData data[20];
    char stringdata0[261];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_canvas__CanvasManagerPrivate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_canvas__CanvasManagerPrivate_t qt_meta_stringdata_ddplugin_canvas__CanvasManagerPrivate = {
    {
QT_MOC_LITERAL(0, 0, 37), // "ddplugin_canvas::CanvasManage..."
QT_MOC_LITERAL(1, 38, 20), // "onHiddenFlagsChanged"
QT_MOC_LITERAL(2, 59, 0), // ""
QT_MOC_LITERAL(3, 60, 13), // "onFileRenamed"
QT_MOC_LITERAL(4, 74, 6), // "oldUrl"
QT_MOC_LITERAL(5, 81, 6), // "newUrl"
QT_MOC_LITERAL(6, 88, 14), // "onFileInserted"
QT_MOC_LITERAL(7, 103, 11), // "QModelIndex"
QT_MOC_LITERAL(8, 115, 6), // "parent"
QT_MOC_LITERAL(9, 122, 5), // "first"
QT_MOC_LITERAL(10, 128, 4), // "last"
QT_MOC_LITERAL(11, 133, 22), // "onFileAboutToBeRemoved"
QT_MOC_LITERAL(12, 156, 17), // "onFileDataChanged"
QT_MOC_LITERAL(13, 174, 7), // "topLeft"
QT_MOC_LITERAL(14, 182, 11), // "bottomRight"
QT_MOC_LITERAL(15, 194, 12), // "QVector<int>"
QT_MOC_LITERAL(16, 207, 5), // "roles"
QT_MOC_LITERAL(17, 213, 16), // "onFileModelReset"
QT_MOC_LITERAL(18, 230, 17), // "onAboutToFileSort"
QT_MOC_LITERAL(19, 248, 12) // "onFileSorted"

    },
    "ddplugin_canvas::CanvasManagerPrivate\0"
    "onHiddenFlagsChanged\0\0onFileRenamed\0"
    "oldUrl\0newUrl\0onFileInserted\0QModelIndex\0"
    "parent\0first\0last\0onFileAboutToBeRemoved\0"
    "onFileDataChanged\0topLeft\0bottomRight\0"
    "QVector<int>\0roles\0onFileModelReset\0"
    "onAboutToFileSort\0onFileSorted"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_canvas__CanvasManagerPrivate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x0a /* Public */,
       3,    2,   57,    2, 0x0a /* Public */,
       6,    3,   62,    2, 0x0a /* Public */,
      11,    3,   69,    2, 0x0a /* Public */,
      12,    3,   76,    2, 0x0a /* Public */,
      17,    0,   83,    2, 0x0a /* Public */,
      18,    0,   84,    2, 0x0a /* Public */,
      19,    0,   85,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,    4,    5,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Int, QMetaType::Int,    8,    9,   10,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Int, QMetaType::Int,    8,    9,   10,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 7, 0x80000000 | 15,   13,   14,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ddplugin_canvas::CanvasManagerPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CanvasManagerPrivate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onHiddenFlagsChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onFileRenamed((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 2: _t->onFileInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 3: _t->onFileAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 4: _t->onFileDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2])),(*reinterpret_cast< const QVector<int>(*)>(_a[3]))); break;
        case 5: _t->onFileModelReset(); break;
        case 6: _t->onAboutToFileSort(); break;
        case 7: _t->onFileSorted(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<int> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_canvas::CanvasManagerPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_canvas__CanvasManagerPrivate.data,
    qt_meta_data_ddplugin_canvas__CanvasManagerPrivate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_canvas::CanvasManagerPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_canvas::CanvasManagerPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_canvas__CanvasManagerPrivate.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_canvas::CanvasManagerPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
