/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -p textindex_interface /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.TextIndex.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef TEXTINDEX_INTERFACE_H
#define TEXTINDEX_INTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.deepin.Filemanager.TextIndex
 */
class OrgDeepinFilemanagerTextIndexInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.deepin.Filemanager.TextIndex"; }

public:
    OrgDeepinFilemanagerTextIndexInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgDeepinFilemanagerTextIndexInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<bool> CreateIndexTask(const QString &path)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(path);
        return asyncCallWithArgumentList(QStringLiteral("CreateIndexTask"), argumentList);
    }

    inline QDBusPendingReply<QString> GetLastUpdateTime()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("GetLastUpdateTime"), argumentList);
    }

    inline QDBusPendingReply<bool> HasRunningTask()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("HasRunningTask"), argumentList);
    }

    inline QDBusPendingReply<bool> IndexDatabaseExists()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("IndexDatabaseExists"), argumentList);
    }

    inline QDBusPendingReply<bool> RemoveIndexTask(const QStringList &paths)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(paths);
        return asyncCallWithArgumentList(QStringLiteral("RemoveIndexTask"), argumentList);
    }

    inline QDBusPendingReply<bool> StopCurrentTask()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("StopCurrentTask"), argumentList);
    }

    inline QDBusPendingReply<bool> UpdateIndexTask(const QString &path)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(path);
        return asyncCallWithArgumentList(QStringLiteral("UpdateIndexTask"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void TaskFinished(const QString &type, const QString &path, bool success);
    void TaskProgressChanged(const QString &type, const QString &path, qlonglong count);
};

namespace org {
  namespace deepin {
    namespace Filemanager {
      typedef ::OrgDeepinFilemanagerTextIndexInterface TextIndex;
    }
  }
}
#endif
