/****************************************************************************
** Meta object code from reading C++ file 'normalizedmode_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-organizer/mode/normalized/normalizedmode_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'normalizedmode_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_organizer__NormalizedModePrivate_t {
    QByteArrayData data[21];
    char stringdata0[280];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_organizer__NormalizedModePrivate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_organizer__NormalizedModePrivate_t qt_meta_stringdata_ddplugin_organizer__NormalizedModePrivate = {
    {
QT_MOC_LITERAL(0, 0, 41), // "ddplugin_organizer::Normalize..."
QT_MOC_LITERAL(1, 42, 12), // "onSelectFile"
QT_MOC_LITERAL(2, 55, 0), // ""
QT_MOC_LITERAL(3, 56, 12), // "QList<QUrl>&"
QT_MOC_LITERAL(4, 69, 4), // "urls"
QT_MOC_LITERAL(5, 74, 4), // "flag"
QT_MOC_LITERAL(6, 79, 16), // "onClearSelection"
QT_MOC_LITERAL(7, 96, 10), // "onDropFile"
QT_MOC_LITERAL(8, 107, 10), // "collection"
QT_MOC_LITERAL(9, 118, 17), // "onIconSizeChanged"
QT_MOC_LITERAL(10, 136, 13), // "onFontChanged"
QT_MOC_LITERAL(11, 150, 12), // "refreshViews"
QT_MOC_LITERAL(12, 163, 7), // "silence"
QT_MOC_LITERAL(13, 171, 24), // "updateHolderSurfaceIndex"
QT_MOC_LITERAL(14, 196, 8), // "QWidget*"
QT_MOC_LITERAL(15, 205, 7), // "surface"
QT_MOC_LITERAL(16, 213, 16), // "batchRenameFiles"
QT_MOC_LITERAL(17, 230, 17), // "moveFilesToCanvas"
QT_MOC_LITERAL(18, 248, 9), // "viewIndex"
QT_MOC_LITERAL(19, 258, 11), // "QList<QUrl>"
QT_MOC_LITERAL(20, 270, 9) // "viewPoint"

    },
    "ddplugin_organizer::NormalizedModePrivate\0"
    "onSelectFile\0\0QList<QUrl>&\0urls\0flag\0"
    "onClearSelection\0onDropFile\0collection\0"
    "onIconSizeChanged\0onFontChanged\0"
    "refreshViews\0silence\0updateHolderSurfaceIndex\0"
    "QWidget*\0surface\0batchRenameFiles\0"
    "moveFilesToCanvas\0viewIndex\0QList<QUrl>\0"
    "viewPoint"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_organizer__NormalizedModePrivate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   59,    2, 0x0a /* Public */,
       6,    0,   64,    2, 0x0a /* Public */,
       7,    2,   65,    2, 0x0a /* Public */,
       9,    0,   70,    2, 0x0a /* Public */,
      10,    0,   71,    2, 0x0a /* Public */,
      11,    1,   72,    2, 0x0a /* Public */,
      13,    1,   75,    2, 0x0a /* Public */,
      16,    0,   78,    2, 0x0a /* Public */,
      17,    3,   79,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int,    4,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 3,    8,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Bool,
    QMetaType::Bool, QMetaType::Int, 0x80000000 | 19, QMetaType::QPoint,   18,    4,   20,

       0        // eod
};

void ddplugin_organizer::NormalizedModePrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NormalizedModePrivate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onSelectFile((*reinterpret_cast< QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 1: _t->onClearSelection(); break;
        case 2: _t->onDropFile((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QList<QUrl>(*)>(_a[2]))); break;
        case 3: _t->onIconSizeChanged(); break;
        case 4: _t->onFontChanged(); break;
        case 5: _t->refreshViews((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->updateHolderSurfaceIndex((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 7: { bool _r = _t->batchRenameFiles();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->moveFilesToCanvas((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QPoint(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_organizer::NormalizedModePrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_organizer__NormalizedModePrivate.data,
    qt_meta_data_ddplugin_organizer__NormalizedModePrivate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_organizer::NormalizedModePrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_organizer::NormalizedModePrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_organizer__NormalizedModePrivate.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_organizer::NormalizedModePrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
