/****************************************************************************
** Meta object code from reading C++ file 'detailspaceeventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-detailspace/events/detailspaceeventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'detailspaceeventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_detailspace__DetailSpaceEventReceiver_t {
    QByteArrayData data[24];
    char stringdata0[411];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_detailspace__DetailSpaceEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_detailspace__DetailSpaceEventReceiver_t qt_meta_stringdata_dfmplugin_detailspace__DetailSpaceEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 47), // "dfmplugin_detailspace::Detail..."
QT_MOC_LITERAL(1, 48, 27), // "handleTileBarShowDetailView"
QT_MOC_LITERAL(2, 76, 0), // ""
QT_MOC_LITERAL(3, 77, 8), // "windowId"
QT_MOC_LITERAL(4, 86, 7), // "checked"
QT_MOC_LITERAL(5, 94, 15), // "handleSetSelect"
QT_MOC_LITERAL(6, 110, 3), // "url"
QT_MOC_LITERAL(7, 114, 27), // "handleViewExtensionRegister"
QT_MOC_LITERAL(8, 142, 23), // "CustomViewExtensionView"
QT_MOC_LITERAL(9, 166, 4), // "view"
QT_MOC_LITERAL(10, 171, 5), // "index"
QT_MOC_LITERAL(11, 177, 32), // "handleBasicViewExtensionRegister"
QT_MOC_LITERAL(12, 210, 18), // "BasicViewFieldFunc"
QT_MOC_LITERAL(13, 229, 4), // "func"
QT_MOC_LITERAL(14, 234, 6), // "scheme"
QT_MOC_LITERAL(15, 241, 36), // "handleBasicViewExtensionRootR..."
QT_MOC_LITERAL(16, 278, 25), // "handleBasicFiledFilterAdd"
QT_MOC_LITERAL(17, 304, 5), // "enums"
QT_MOC_LITERAL(18, 310, 29), // "handleBasicFiledFilterRootAdd"
QT_MOC_LITERAL(19, 340, 26), // "handleViewSelectionChanged"
QT_MOC_LITERAL(20, 367, 8), // "windowID"
QT_MOC_LITERAL(21, 376, 14), // "QItemSelection"
QT_MOC_LITERAL(22, 391, 8), // "selected"
QT_MOC_LITERAL(23, 400, 10) // "deselected"

    },
    "dfmplugin_detailspace::DetailSpaceEventReceiver\0"
    "handleTileBarShowDetailView\0\0windowId\0"
    "checked\0handleSetSelect\0url\0"
    "handleViewExtensionRegister\0"
    "CustomViewExtensionView\0view\0index\0"
    "handleBasicViewExtensionRegister\0"
    "BasicViewFieldFunc\0func\0scheme\0"
    "handleBasicViewExtensionRootRegister\0"
    "handleBasicFiledFilterAdd\0enums\0"
    "handleBasicFiledFilterRootAdd\0"
    "handleViewSelectionChanged\0windowID\0"
    "QItemSelection\0selected\0deselected"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_detailspace__DetailSpaceEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   54,    2, 0x0a /* Public */,
       5,    2,   59,    2, 0x0a /* Public */,
       7,    2,   64,    2, 0x0a /* Public */,
      11,    2,   69,    2, 0x0a /* Public */,
      15,    2,   74,    2, 0x0a /* Public */,
      16,    2,   79,    2, 0x0a /* Public */,
      18,    2,   84,    2, 0x0a /* Public */,
      19,    3,   89,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::ULongLong, QMetaType::Bool,    3,    4,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,    3,    6,
    QMetaType::Bool, 0x80000000 | 8, QMetaType::Int,    9,   10,
    QMetaType::Bool, 0x80000000 | 12, QMetaType::QString,   13,   14,
    QMetaType::Bool, 0x80000000 | 12, QMetaType::QString,   13,   14,
    QMetaType::Bool, QMetaType::QString, QMetaType::QStringList,   14,   17,
    QMetaType::Bool, QMetaType::QString, QMetaType::QStringList,   14,   17,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 21, 0x80000000 | 21,   20,   22,   23,

       0        // eod
};

void dfmplugin_detailspace::DetailSpaceEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DetailSpaceEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleTileBarShowDetailView((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->handleSetSelect((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 2: { bool _r = _t->handleViewExtensionRegister((*reinterpret_cast< CustomViewExtensionView(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->handleBasicViewExtensionRegister((*reinterpret_cast< BasicViewFieldFunc(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->handleBasicViewExtensionRootRegister((*reinterpret_cast< BasicViewFieldFunc(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->handleBasicFiledFilterAdd((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->handleBasicFiledFilterRootAdd((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->handleViewSelectionChanged((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2])),(*reinterpret_cast< const QItemSelection(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_detailspace::DetailSpaceEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_detailspace__DetailSpaceEventReceiver.data,
    qt_meta_data_dfmplugin_detailspace__DetailSpaceEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_detailspace::DetailSpaceEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_detailspace::DetailSpaceEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_detailspace__DetailSpaceEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_detailspace::DetailSpaceEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
