/****************************************************************************
** Meta object code from reading C++ file 'fileview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-workspace/views/fileview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_workspace__FileView_t {
    QByteArrayData data[69];
    char stringdata0[1018];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_workspace__FileView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_workspace__FileView_t qt_meta_stringdata_dfmplugin_workspace__FileView = {
    {
QT_MOC_LITERAL(0, 0, 29), // "dfmplugin_workspace::FileView"
QT_MOC_LITERAL(1, 30, 16), // "reqOpenNewWindow"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 11), // "QList<QUrl>"
QT_MOC_LITERAL(4, 60, 4), // "urls"
QT_MOC_LITERAL(5, 65, 16), // "viewStateChanged"
QT_MOC_LITERAL(6, 82, 16), // "selectUrlChanged"
QT_MOC_LITERAL(7, 99, 24), // "onHeaderViewMousePressed"
QT_MOC_LITERAL(8, 124, 25), // "onHeaderViewMouseReleased"
QT_MOC_LITERAL(9, 150, 22), // "onHeaderSectionResized"
QT_MOC_LITERAL(10, 173, 12), // "logicalIndex"
QT_MOC_LITERAL(11, 186, 7), // "oldSize"
QT_MOC_LITERAL(12, 194, 7), // "newSize"
QT_MOC_LITERAL(13, 202, 20), // "onHeaderSectionMoved"
QT_MOC_LITERAL(14, 223, 14), // "oldVisualIndex"
QT_MOC_LITERAL(15, 238, 14), // "newVisualIndex"
QT_MOC_LITERAL(16, 253, 21), // "onHeaderHiddenChanged"
QT_MOC_LITERAL(17, 275, 8), // "roleName"
QT_MOC_LITERAL(18, 284, 8), // "isHidden"
QT_MOC_LITERAL(19, 293, 22), // "onSortIndicatorChanged"
QT_MOC_LITERAL(20, 316, 13), // "Qt::SortOrder"
QT_MOC_LITERAL(21, 330, 5), // "order"
QT_MOC_LITERAL(22, 336, 28), // "onSectionHandleDoubleClicked"
QT_MOC_LITERAL(23, 365, 9), // "onClicked"
QT_MOC_LITERAL(24, 375, 11), // "QModelIndex"
QT_MOC_LITERAL(25, 387, 5), // "index"
QT_MOC_LITERAL(26, 393, 15), // "onDoubleClicked"
QT_MOC_LITERAL(27, 409, 21), // "onScalingValueChanged"
QT_MOC_LITERAL(28, 431, 5), // "value"
QT_MOC_LITERAL(29, 437, 15), // "viewModeChanged"
QT_MOC_LITERAL(30, 453, 8), // "windowId"
QT_MOC_LITERAL(31, 462, 8), // "viewMode"
QT_MOC_LITERAL(32, 471, 17), // "onRowCountChanged"
QT_MOC_LITERAL(33, 489, 17), // "trashStateChanged"
QT_MOC_LITERAL(34, 507, 26), // "onHeaderViewSectionChanged"
QT_MOC_LITERAL(35, 534, 3), // "url"
QT_MOC_LITERAL(36, 538, 21), // "onAppAttributeChanged"
QT_MOC_LITERAL(37, 560, 5), // "group"
QT_MOC_LITERAL(38, 566, 3), // "key"
QT_MOC_LITERAL(39, 570, 15), // "onSelectAndEdit"
QT_MOC_LITERAL(40, 586, 4), // "edit"
QT_MOC_LITERAL(41, 591, 11), // "EditTrigger"
QT_MOC_LITERAL(42, 603, 7), // "trigger"
QT_MOC_LITERAL(43, 611, 7), // "QEvent*"
QT_MOC_LITERAL(44, 619, 5), // "event"
QT_MOC_LITERAL(45, 625, 18), // "currentDirOpenMode"
QT_MOC_LITERAL(46, 644, 11), // "DirOpenMode"
QT_MOC_LITERAL(47, 656, 14), // "onWidgetUpdate"
QT_MOC_LITERAL(48, 671, 22), // "onRenameProcessStarted"
QT_MOC_LITERAL(49, 694, 13), // "loadViewState"
QT_MOC_LITERAL(50, 708, 17), // "saveViewModeState"
QT_MOC_LITERAL(51, 726, 19), // "onModelStateChanged"
QT_MOC_LITERAL(52, 746, 22), // "setIconSizeBySizeIndex"
QT_MOC_LITERAL(53, 769, 9), // "sizeIndex"
QT_MOC_LITERAL(54, 779, 23), // "onShowFileSuffixChanged"
QT_MOC_LITERAL(55, 803, 6), // "isShow"
QT_MOC_LITERAL(56, 810, 22), // "updateHorizontalOffset"
QT_MOC_LITERAL(57, 833, 10), // "updateView"
QT_MOC_LITERAL(58, 844, 13), // "updateOneView"
QT_MOC_LITERAL(59, 858, 18), // "onSelectionChanged"
QT_MOC_LITERAL(60, 877, 14), // "QItemSelection"
QT_MOC_LITERAL(61, 892, 11), // "indexInRect"
QT_MOC_LITERAL(62, 904, 10), // "deselected"
QT_MOC_LITERAL(63, 915, 24), // "onDefaultViewModeChanged"
QT_MOC_LITERAL(64, 940, 4), // "mode"
QT_MOC_LITERAL(65, 945, 17), // "onIconSizeChanged"
QT_MOC_LITERAL(66, 963, 23), // "onItemWidthLevelChanged"
QT_MOC_LITERAL(67, 987, 5), // "level"
QT_MOC_LITERAL(68, 993, 24) // "onItemHeightLevelChanged"

    },
    "dfmplugin_workspace::FileView\0"
    "reqOpenNewWindow\0\0QList<QUrl>\0urls\0"
    "viewStateChanged\0selectUrlChanged\0"
    "onHeaderViewMousePressed\0"
    "onHeaderViewMouseReleased\0"
    "onHeaderSectionResized\0logicalIndex\0"
    "oldSize\0newSize\0onHeaderSectionMoved\0"
    "oldVisualIndex\0newVisualIndex\0"
    "onHeaderHiddenChanged\0roleName\0isHidden\0"
    "onSortIndicatorChanged\0Qt::SortOrder\0"
    "order\0onSectionHandleDoubleClicked\0"
    "onClicked\0QModelIndex\0index\0onDoubleClicked\0"
    "onScalingValueChanged\0value\0viewModeChanged\0"
    "windowId\0viewMode\0onRowCountChanged\0"
    "trashStateChanged\0onHeaderViewSectionChanged\0"
    "url\0onAppAttributeChanged\0group\0key\0"
    "onSelectAndEdit\0edit\0EditTrigger\0"
    "trigger\0QEvent*\0event\0currentDirOpenMode\0"
    "DirOpenMode\0onWidgetUpdate\0"
    "onRenameProcessStarted\0loadViewState\0"
    "saveViewModeState\0onModelStateChanged\0"
    "setIconSizeBySizeIndex\0sizeIndex\0"
    "onShowFileSuffixChanged\0isShow\0"
    "updateHorizontalOffset\0updateView\0"
    "updateOneView\0onSelectionChanged\0"
    "QItemSelection\0indexInRect\0deselected\0"
    "onDefaultViewModeChanged\0mode\0"
    "onIconSizeChanged\0onItemWidthLevelChanged\0"
    "level\0onItemHeightLevelChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_workspace__FileView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      36,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  194,    2, 0x06 /* Public */,
       5,    0,  197,    2, 0x06 /* Public */,
       6,    1,  198,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,  201,    2, 0x0a /* Public */,
       8,    0,  202,    2, 0x0a /* Public */,
       9,    3,  203,    2, 0x0a /* Public */,
      13,    3,  210,    2, 0x0a /* Public */,
      16,    2,  217,    2, 0x0a /* Public */,
      19,    2,  222,    2, 0x0a /* Public */,
      22,    1,  227,    2, 0x0a /* Public */,
      23,    1,  230,    2, 0x0a /* Public */,
      26,    1,  233,    2, 0x0a /* Public */,
      27,    1,  236,    2, 0x0a /* Public */,
      29,    2,  239,    2, 0x0a /* Public */,
      32,    0,  244,    2, 0x0a /* Public */,
      33,    0,  245,    2, 0x0a /* Public */,
      34,    1,  246,    2, 0x0a /* Public */,
      36,    3,  249,    2, 0x0a /* Public */,
      39,    1,  256,    2, 0x0a /* Public */,
      40,    3,  259,    2, 0x0a /* Public */,
      45,    0,  266,    2, 0x0a /* Public */,
      47,    0,  267,    2, 0x0a /* Public */,
      48,    0,  268,    2, 0x0a /* Public */,
      49,    1,  269,    2, 0x08 /* Private */,
      50,    0,  272,    2, 0x08 /* Private */,
      51,    0,  273,    2, 0x08 /* Private */,
      52,    1,  274,    2, 0x08 /* Private */,
      54,    1,  277,    2, 0x08 /* Private */,
      56,    0,  280,    2, 0x08 /* Private */,
      57,    0,  281,    2, 0x08 /* Private */,
      58,    1,  282,    2, 0x08 /* Private */,
      59,    2,  285,    2, 0x08 /* Private */,
      63,    1,  290,    2, 0x08 /* Private */,
      65,    1,  293,    2, 0x08 /* Private */,
      66,    1,  296,    2, 0x08 /* Private */,
      68,    1,  299,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,   10,   11,   12,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,   10,   14,   15,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,   17,   18,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 20,   10,   21,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, 0x80000000 | 24,   25,
    QMetaType::Void, 0x80000000 | 24,   25,
    QMetaType::Void, QMetaType::Int,   28,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::Int,   30,   31,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,   35,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QVariant,   37,   38,   28,
    QMetaType::Void, QMetaType::QUrl,   35,
    QMetaType::Bool, 0x80000000 | 24, 0x80000000 | 41, 0x80000000 | 43,   25,   42,   44,
    0x80000000 | 46,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,   35,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   53,
    QMetaType::Void, QMetaType::Bool,   55,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 24,   25,
    QMetaType::Void, 0x80000000 | 60, 0x80000000 | 60,   61,   62,
    QMetaType::Void, QMetaType::Int,   64,
    QMetaType::Void, QMetaType::Int,   53,
    QMetaType::Void, QMetaType::Int,   67,
    QMetaType::Void, QMetaType::Int,   67,

       0        // eod
};

void dfmplugin_workspace::FileView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->reqOpenNewWindow((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 1: _t->viewStateChanged(); break;
        case 2: _t->selectUrlChanged((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 3: _t->onHeaderViewMousePressed(); break;
        case 4: _t->onHeaderViewMouseReleased(); break;
        case 5: _t->onHeaderSectionResized((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 6: _t->onHeaderSectionMoved((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 7: _t->onHeaderHiddenChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        case 8: _t->onSortIndicatorChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< Qt::SortOrder(*)>(_a[2]))); break;
        case 9: _t->onSectionHandleDoubleClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->onClicked((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 11: _t->onDoubleClicked((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 12: _t->onScalingValueChanged((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 13: _t->viewModeChanged((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 14: _t->onRowCountChanged(); break;
        case 15: _t->trashStateChanged(); break;
        case 16: _t->onHeaderViewSectionChanged((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 17: _t->onAppAttributeChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3]))); break;
        case 18: _t->onSelectAndEdit((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 19: { bool _r = _t->edit((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< EditTrigger(*)>(_a[2])),(*reinterpret_cast< QEvent*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { DirOpenMode _r = _t->currentDirOpenMode();
            if (_a[0]) *reinterpret_cast< DirOpenMode*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->onWidgetUpdate(); break;
        case 22: _t->onRenameProcessStarted(); break;
        case 23: _t->loadViewState((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 24: _t->saveViewModeState(); break;
        case 25: _t->onModelStateChanged(); break;
        case 26: _t->setIconSizeBySizeIndex((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 27: _t->onShowFileSuffixChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 28: _t->updateHorizontalOffset(); break;
        case 29: _t->updateView(); break;
        case 30: _t->updateOneView((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 31: _t->onSelectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 32: _t->onDefaultViewModeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 33: _t->onIconSizeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 34: _t->onItemWidthLevelChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 35: _t->onItemHeightLevelChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileView::*)(const QList<QUrl> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileView::reqOpenNewWindow)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileView::viewStateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileView::*)(const QList<QUrl> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileView::selectUrlChanged)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_workspace::FileView::staticMetaObject = { {
    QMetaObject::SuperData::link<DListView::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_workspace__FileView.data,
    qt_meta_data_dfmplugin_workspace__FileView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_workspace::FileView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_workspace::FileView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_workspace__FileView.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "dfmbase::AbstractBaseView"))
        return static_cast< dfmbase::AbstractBaseView*>(this);
    return DListView::qt_metacast(_clname);
}

int dfmplugin_workspace::FileView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DListView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_workspace::FileView::reqOpenNewWindow(const QList<QUrl> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_workspace::FileView::viewStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dfmplugin_workspace::FileView::selectUrlChanged(const QList<QUrl> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
