/****************************************************************************
** Meta object code from reading C++ file 'canvasmodelshell.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-organizer/interface/canvasmodelshell.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvasmodelshell.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_organizer__CanvasModelShell_t {
    QByteArrayData data[14];
    char stringdata0[187];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_organizer__CanvasModelShell_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_organizer__CanvasModelShell_t qt_meta_stringdata_ddplugin_organizer__CanvasModelShell = {
    {
QT_MOC_LITERAL(0, 0, 36), // "ddplugin_organizer::CanvasMod..."
QT_MOC_LITERAL(1, 37, 16), // "filterDataRested"
QT_MOC_LITERAL(2, 54, 0), // ""
QT_MOC_LITERAL(3, 55, 12), // "QList<QUrl>*"
QT_MOC_LITERAL(4, 68, 4), // "urls"
QT_MOC_LITERAL(5, 73, 18), // "filterDataInserted"
QT_MOC_LITERAL(6, 92, 3), // "url"
QT_MOC_LITERAL(7, 96, 17), // "filterDataRenamed"
QT_MOC_LITERAL(8, 114, 6), // "oldUrl"
QT_MOC_LITERAL(9, 121, 6), // "newUrl"
QT_MOC_LITERAL(10, 128, 15), // "eventDataRested"
QT_MOC_LITERAL(11, 144, 7), // "extData"
QT_MOC_LITERAL(12, 152, 17), // "eventDataInserted"
QT_MOC_LITERAL(13, 170, 16) // "eventDataRenamed"

    },
    "ddplugin_organizer::CanvasModelShell\0"
    "filterDataRested\0\0QList<QUrl>*\0urls\0"
    "filterDataInserted\0url\0filterDataRenamed\0"
    "oldUrl\0newUrl\0eventDataRested\0extData\0"
    "eventDataInserted\0eventDataRenamed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_organizer__CanvasModelShell[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       5,    1,   47,    2, 0x06 /* Public */,
       7,    2,   50,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    2,   55,    2, 0x08 /* Private */,
      12,    2,   60,    2, 0x08 /* Private */,
      13,    3,   65,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Bool, 0x80000000 | 3,    4,
    QMetaType::Bool, QMetaType::QUrl,    6,
    QMetaType::Bool, QMetaType::QUrl, QMetaType::QUrl,    8,    9,

 // slots: parameters
    QMetaType::Bool, 0x80000000 | 3, QMetaType::VoidStar,    4,   11,
    QMetaType::Bool, QMetaType::QUrl, QMetaType::VoidStar,    6,   11,
    QMetaType::Bool, QMetaType::QUrl, QMetaType::QUrl, QMetaType::VoidStar,    8,    9,   11,

       0        // eod
};

void ddplugin_organizer::CanvasModelShell::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CanvasModelShell *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->filterDataRested((*reinterpret_cast< QList<QUrl>*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->filterDataInserted((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->filterDataRenamed((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->eventDataRested((*reinterpret_cast< QList<QUrl>*(*)>(_a[1])),(*reinterpret_cast< void*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->eventDataInserted((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< void*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->eventDataRenamed((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< void*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = bool (CanvasModelShell::*)(QList<QUrl> * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CanvasModelShell::filterDataRested)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = bool (CanvasModelShell::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CanvasModelShell::filterDataInserted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = bool (CanvasModelShell::*)(const QUrl & , const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CanvasModelShell::filterDataRenamed)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_organizer::CanvasModelShell::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_organizer__CanvasModelShell.data,
    qt_meta_data_ddplugin_organizer__CanvasModelShell,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_organizer::CanvasModelShell::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_organizer::CanvasModelShell::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_organizer__CanvasModelShell.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_organizer::CanvasModelShell::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
bool ddplugin_organizer::CanvasModelShell::filterDataRested(QList<QUrl> * _t1)
{
    bool _t0{};
    void *_a[] = { const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t0))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
    return _t0;
}

// SIGNAL 1
bool ddplugin_organizer::CanvasModelShell::filterDataInserted(const QUrl & _t1)
{
    bool _t0{};
    void *_a[] = { const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t0))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
    return _t0;
}

// SIGNAL 2
bool ddplugin_organizer::CanvasModelShell::filterDataRenamed(const QUrl & _t1, const QUrl & _t2)
{
    bool _t0{};
    void *_a[] = { const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t0))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
    return _t0;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
