/****************************************************************************
** Meta object code from reading C++ file 'burneventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-burn/events/burneventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'burneventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_burn__BurnEventReceiver_t {
    QByteArrayData data[28];
    char stringdata0[343];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_burn__BurnEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_burn__BurnEventReceiver_t qt_meta_stringdata_dfmplugin_burn__BurnEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 33), // "dfmplugin_burn::BurnEventRece..."
QT_MOC_LITERAL(1, 34, 17), // "handleShowBurnDlg"
QT_MOC_LITERAL(2, 52, 0), // ""
QT_MOC_LITERAL(3, 53, 3), // "dev"
QT_MOC_LITERAL(4, 57, 14), // "isSupportedUDF"
QT_MOC_LITERAL(5, 72, 8), // "QWidget*"
QT_MOC_LITERAL(6, 81, 6), // "parent"
QT_MOC_LITERAL(7, 88, 20), // "handleShowDumpISODlg"
QT_MOC_LITERAL(8, 109, 5), // "devId"
QT_MOC_LITERAL(9, 115, 11), // "handleErase"
QT_MOC_LITERAL(10, 127, 13), // "handlePasteTo"
QT_MOC_LITERAL(11, 141, 11), // "QList<QUrl>"
QT_MOC_LITERAL(12, 153, 4), // "urls"
QT_MOC_LITERAL(13, 158, 4), // "dest"
QT_MOC_LITERAL(14, 163, 6), // "isCopy"
QT_MOC_LITERAL(15, 170, 16), // "handleMountImage"
QT_MOC_LITERAL(16, 187, 5), // "winId"
QT_MOC_LITERAL(17, 193, 6), // "isoUrl"
QT_MOC_LITERAL(18, 200, 21), // "handleCopyFilesResult"
QT_MOC_LITERAL(19, 222, 7), // "srcUrls"
QT_MOC_LITERAL(20, 230, 8), // "destUrls"
QT_MOC_LITERAL(21, 239, 2), // "ok"
QT_MOC_LITERAL(22, 242, 6), // "errMsg"
QT_MOC_LITERAL(23, 249, 19), // "handleFileCutResult"
QT_MOC_LITERAL(24, 269, 22), // "handleFileRemoveResult"
QT_MOC_LITERAL(25, 292, 22), // "handleFileRenameResult"
QT_MOC_LITERAL(26, 315, 15), // "QMap<QUrl,QUrl>"
QT_MOC_LITERAL(27, 331, 11) // "renamedUrls"

    },
    "dfmplugin_burn::BurnEventReceiver\0"
    "handleShowBurnDlg\0\0dev\0isSupportedUDF\0"
    "QWidget*\0parent\0handleShowDumpISODlg\0"
    "devId\0handleErase\0handlePasteTo\0"
    "QList<QUrl>\0urls\0dest\0isCopy\0"
    "handleMountImage\0winId\0isoUrl\0"
    "handleCopyFilesResult\0srcUrls\0destUrls\0"
    "ok\0errMsg\0handleFileCutResult\0"
    "handleFileRemoveResult\0handleFileRenameResult\0"
    "QMap<QUrl,QUrl>\0renamedUrls"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_burn__BurnEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   59,    2, 0x0a /* Public */,
       7,    1,   66,    2, 0x0a /* Public */,
       9,    1,   69,    2, 0x0a /* Public */,
      10,    3,   72,    2, 0x0a /* Public */,
      15,    2,   79,    2, 0x0a /* Public */,
      18,    4,   84,    2, 0x0a /* Public */,
      23,    4,   93,    2, 0x0a /* Public */,
      24,    3,  102,    2, 0x0a /* Public */,
      25,    4,  109,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::Bool, 0x80000000 | 5,    3,    4,    6,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, 0x80000000 | 11, QMetaType::QUrl, QMetaType::Bool,   12,   13,   14,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,   16,   17,
    QMetaType::Void, 0x80000000 | 11, 0x80000000 | 11, QMetaType::Bool, QMetaType::QString,   19,   20,   21,   22,
    QMetaType::Void, 0x80000000 | 11, 0x80000000 | 11, QMetaType::Bool, QMetaType::QString,   19,   20,   21,   22,
    QMetaType::Void, 0x80000000 | 11, QMetaType::Bool, QMetaType::QString,   19,   21,   22,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 26, QMetaType::Bool, QMetaType::QString,   16,   27,   21,   22,

       0        // eod
};

void dfmplugin_burn::BurnEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BurnEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleShowBurnDlg((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< QWidget*(*)>(_a[3]))); break;
        case 1: _t->handleShowDumpISODlg((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->handleErase((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->handlePasteTo((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 4: _t->handleMountImage((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 5: _t->handleCopyFilesResult((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 6: _t->handleFileCutResult((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 7: _t->handleFileRemoveResult((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 8: _t->handleFileRenameResult((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QMap<QUrl,QUrl>(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_burn::BurnEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_burn__BurnEventReceiver.data,
    qt_meta_data_dfmplugin_burn__BurnEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_burn::BurnEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_burn::BurnEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_burn__BurnEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_burn::BurnEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
