/****************************************************************************
** Meta object code from reading C++ file 'wallpaperlist.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-wallpapersetting/wallpaperlist.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wallpaperlist.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperList_t {
    QByteArrayData data[13];
    char stringdata0[176];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperList_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperList_t qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperList = {
    {
QT_MOC_LITERAL(0, 0, 40), // "ddplugin_wallpapersetting::Wa..."
QT_MOC_LITERAL(1, 41, 11), // "itemPressed"
QT_MOC_LITERAL(2, 53, 0), // ""
QT_MOC_LITERAL(3, 54, 4), // "data"
QT_MOC_LITERAL(4, 59, 20), // "mouseOverItemChanged"
QT_MOC_LITERAL(5, 80, 3), // "pos"
QT_MOC_LITERAL(6, 84, 8), // "prevPage"
QT_MOC_LITERAL(7, 93, 8), // "nextPage"
QT_MOC_LITERAL(8, 102, 15), // "updateItemThumb"
QT_MOC_LITERAL(9, 118, 13), // "onItemPressed"
QT_MOC_LITERAL(10, 132, 14), // "WallpaperItem*"
QT_MOC_LITERAL(11, 147, 13), // "onItemHoverIn"
QT_MOC_LITERAL(12, 161, 14) // "onItemHoverOut"

    },
    "ddplugin_wallpapersetting::WallpaperList\0"
    "itemPressed\0\0data\0mouseOverItemChanged\0"
    "pos\0prevPage\0nextPage\0updateItemThumb\0"
    "onItemPressed\0WallpaperItem*\0onItemHoverIn\0"
    "onItemHoverOut"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_wallpapersetting__WallpaperList[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       4,    2,   57,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,   62,    2, 0x0a /* Public */,
       7,    0,   63,    2, 0x0a /* Public */,
       8,    0,   64,    2, 0x0a /* Public */,
       9,    1,   65,    2, 0x08 /* Private */,
      11,    1,   68,    2, 0x08 /* Private */,
      12,    1,   71,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QPoint,    3,    5,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,    2,
    QMetaType::Void, 0x80000000 | 10,    2,
    QMetaType::Void, 0x80000000 | 10,    2,

       0        // eod
};

void ddplugin_wallpapersetting::WallpaperList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<WallpaperList *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->itemPressed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->mouseOverItemChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QPoint(*)>(_a[2]))); break;
        case 2: _t->prevPage(); break;
        case 3: _t->nextPage(); break;
        case 4: _t->updateItemThumb(); break;
        case 5: _t->onItemPressed((*reinterpret_cast< WallpaperItem*(*)>(_a[1]))); break;
        case 6: _t->onItemHoverIn((*reinterpret_cast< WallpaperItem*(*)>(_a[1]))); break;
        case 7: _t->onItemHoverOut((*reinterpret_cast< WallpaperItem*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WallpaperItem* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WallpaperItem* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WallpaperItem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (WallpaperList::*)(const QString & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WallpaperList::itemPressed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (WallpaperList::*)(const QString & , QPoint ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WallpaperList::mouseOverItemChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_wallpapersetting::WallpaperList::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperList.data,
    qt_meta_data_ddplugin_wallpapersetting__WallpaperList,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_wallpapersetting::WallpaperList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_wallpapersetting::WallpaperList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperList.stringdata0))
        return static_cast<void*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int ddplugin_wallpapersetting::WallpaperList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void ddplugin_wallpapersetting::WallpaperList::itemPressed(const QString & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< ddplugin_wallpapersetting::WallpaperList *>(this), &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ddplugin_wallpapersetting::WallpaperList::mouseOverItemChanged(const QString & _t1, QPoint _t2)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(const_cast< ddplugin_wallpapersetting::WallpaperList *>(this), &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
