/****************************************************************************
** Meta object code from reading C++ file 'organizerbroker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-organizer/broker/organizerbroker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'organizerbroker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_organizer__OrganizerBroker_t {
    QByteArrayData data[17];
    char stringdata0[162];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_organizer__OrganizerBroker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_organizer__OrganizerBroker_t qt_meta_stringdata_ddplugin_organizer__OrganizerBroker = {
    {
QT_MOC_LITERAL(0, 0, 35), // "ddplugin_organizer::Organizer..."
QT_MOC_LITERAL(1, 36, 12), // "refreshModel"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 6), // "global"
QT_MOC_LITERAL(4, 57, 2), // "ms"
QT_MOC_LITERAL(5, 60, 4), // "file"
QT_MOC_LITERAL(6, 65, 9), // "gridPoint"
QT_MOC_LITERAL(7, 75, 4), // "item"
QT_MOC_LITERAL(8, 80, 7), // "QPoint*"
QT_MOC_LITERAL(9, 88, 5), // "point"
QT_MOC_LITERAL(10, 94, 10), // "visualRect"
QT_MOC_LITERAL(11, 105, 2), // "id"
QT_MOC_LITERAL(12, 108, 4), // "view"
QT_MOC_LITERAL(13, 113, 18), // "QAbstractItemView*"
QT_MOC_LITERAL(14, 132, 8), // "iconRect"
QT_MOC_LITERAL(15, 141, 5), // "vrect"
QT_MOC_LITERAL(16, 147, 14) // "selectAllItems"

    },
    "ddplugin_organizer::OrganizerBroker\0"
    "refreshModel\0\0global\0ms\0file\0gridPoint\0"
    "item\0QPoint*\0point\0visualRect\0id\0view\0"
    "QAbstractItemView*\0iconRect\0vrect\0"
    "selectAllItems"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_organizer__OrganizerBroker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   44,    2, 0x0a /* Public */,
       6,    2,   51,    2, 0x0a /* Public */,
      10,    2,   56,    2, 0x0a /* Public */,
      12,    1,   61,    2, 0x0a /* Public */,
      14,    2,   64,    2, 0x0a /* Public */,
      16,    0,   69,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool, QMetaType::Int, QMetaType::Bool,    3,    4,    5,
    QMetaType::QString, QMetaType::QUrl, 0x80000000 | 8,    7,    9,
    QMetaType::QRect, QMetaType::QString, QMetaType::QUrl,   11,    7,
    0x80000000 | 13, QMetaType::QString,   11,
    QMetaType::QRect, QMetaType::QString, QMetaType::QRect,   11,   15,
    QMetaType::Bool,

       0        // eod
};

void ddplugin_organizer::OrganizerBroker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OrganizerBroker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->refreshModel((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 1: { QString _r = _t->gridPoint((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QPoint*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { QRect _r = _t->visualRect((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QRect*>(_a[0]) = std::move(_r); }  break;
        case 3: { QAbstractItemView* _r = _t->view((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QAbstractItemView**>(_a[0]) = std::move(_r); }  break;
        case 4: { QRect _r = _t->iconRect((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QRect(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QRect*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->selectAllItems();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_organizer::OrganizerBroker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_organizer__OrganizerBroker.data,
    qt_meta_data_ddplugin_organizer__OrganizerBroker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_organizer::OrganizerBroker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_organizer::OrganizerBroker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_organizer__OrganizerBroker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_organizer::OrganizerBroker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
