/****************************************************************************
** Meta object code from reading C++ file 'devicemanager_interface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "devicemanager_interface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'devicemanager_interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN48OrgDeepinFilemanagerDaemonDeviceManagerInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgDeepinFilemanagerDaemonDeviceManagerInterface::qt_create_metaobjectdata<qt_meta_tag_ZN48OrgDeepinFilemanagerDaemonDeviceManagerInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgDeepinFilemanagerDaemonDeviceManagerInterface",
        "BlockDeviceAdded",
        "",
        "id",
        "BlockDeviceFilesystemAdded",
        "BlockDeviceFilesystemRemoved",
        "BlockDeviceLocked",
        "BlockDeviceMounted",
        "mountPoint",
        "BlockDevicePropertyChanged",
        "property",
        "QDBusVariant",
        "value",
        "BlockDeviceRemoved",
        "oldMpt",
        "BlockDeviceUnlocked",
        "clearDeviceId",
        "BlockDeviceUnmounted",
        "BlockDriveAdded",
        "BlockDriveRemoved",
        "NotifyDeviceBusy",
        "action",
        "ProtocolDeviceAdded",
        "ProtocolDeviceMounted",
        "ProtocolDeviceRemoved",
        "ProtocolDeviceUnmounted",
        "SizeUsedChanged",
        "total",
        "free",
        "DetachAllMountedDevices",
        "QDBusPendingReply<>",
        "DetachBlockDevice",
        "DetachProtocolDevice",
        "GetBlockDevicesIdList",
        "QDBusPendingReply<QStringList>",
        "opts",
        "GetProtocolDevicesIdList",
        "IsMonotorWorking",
        "QDBusPendingReply<bool>",
        "QueryBlockDeviceInfo",
        "QDBusPendingReply<QVariantMap>",
        "reload",
        "QueryProtocolDeviceInfo"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'BlockDeviceAdded'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'BlockDeviceFilesystemAdded'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'BlockDeviceFilesystemRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'BlockDeviceLocked'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'BlockDeviceMounted'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'BlockDevicePropertyChanged'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QDBusVariant &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Signal 'BlockDeviceRemoved'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 14 },
        }}),
        // Signal 'BlockDeviceUnlocked'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 16 },
        }}),
        // Signal 'BlockDeviceUnmounted'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 14 },
        }}),
        // Signal 'BlockDriveAdded'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'BlockDriveRemoved'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'NotifyDeviceBusy'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 21 },
        }}),
        // Signal 'ProtocolDeviceAdded'
        QtMocHelpers::SignalData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'ProtocolDeviceMounted'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'ProtocolDeviceRemoved'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 14 },
        }}),
        // Signal 'ProtocolDeviceUnmounted'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 14 },
        }}),
        // Signal 'SizeUsedChanged'
        QtMocHelpers::SignalData<void(const QString &, qlonglong, qlonglong)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::LongLong, 27 }, { QMetaType::LongLong, 28 },
        }}),
        // Slot 'DetachAllMountedDevices'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(29, 2, QMC::AccessPublic, 0x80000000 | 30),
        // Slot 'DetachBlockDevice'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(31, 2, QMC::AccessPublic, 0x80000000 | 30, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'DetachProtocolDevice'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(32, 2, QMC::AccessPublic, 0x80000000 | 30, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'GetBlockDevicesIdList'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>(int)>(33, 2, QMC::AccessPublic, 0x80000000 | 34, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'GetProtocolDevicesIdList'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>()>(36, 2, QMC::AccessPublic, 0x80000000 | 34),
        // Slot 'IsMonotorWorking'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>()>(37, 2, QMC::AccessPublic, 0x80000000 | 38),
        // Slot 'QueryBlockDeviceInfo'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantMap>(const QString &, bool)>(39, 2, QMC::AccessPublic, 0x80000000 | 40, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 41 },
        }}),
        // Slot 'QueryProtocolDeviceInfo'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantMap>(const QString &, bool)>(42, 2, QMC::AccessPublic, 0x80000000 | 40, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 41 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgDeepinFilemanagerDaemonDeviceManagerInterface, qt_meta_tag_ZN48OrgDeepinFilemanagerDaemonDeviceManagerInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgDeepinFilemanagerDaemonDeviceManagerInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN48OrgDeepinFilemanagerDaemonDeviceManagerInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN48OrgDeepinFilemanagerDaemonDeviceManagerInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN48OrgDeepinFilemanagerDaemonDeviceManagerInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgDeepinFilemanagerDaemonDeviceManagerInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgDeepinFilemanagerDaemonDeviceManagerInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->BlockDeviceAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->BlockDeviceFilesystemAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->BlockDeviceFilesystemRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->BlockDeviceLocked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->BlockDeviceMounted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->BlockDevicePropertyChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[3]))); break;
        case 6: _t->BlockDeviceRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->BlockDeviceUnlocked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->BlockDeviceUnmounted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->BlockDriveAdded(); break;
        case 10: _t->BlockDriveRemoved(); break;
        case 11: _t->NotifyDeviceBusy((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 12: _t->ProtocolDeviceAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->ProtocolDeviceMounted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->ProtocolDeviceRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 15: _t->ProtocolDeviceUnmounted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 16: _t->SizeUsedChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[3]))); break;
        case 17: { QDBusPendingReply<> _r = _t->DetachAllMountedDevices();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 18: { QDBusPendingReply<> _r = _t->DetachBlockDevice((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 19: { QDBusPendingReply<> _r = _t->DetachProtocolDevice((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 20: { QDBusPendingReply<QStringList> _r = _t->GetBlockDevicesIdList((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 21: { QDBusPendingReply<QStringList> _r = _t->GetProtocolDevicesIdList();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 22: { QDBusPendingReply<bool> _r = _t->IsMonotorWorking();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 23: { QDBusPendingReply<QVariantMap> _r = _t->QueryBlockDeviceInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 24: { QDBusPendingReply<QVariantMap> _r = _t->QueryProtocolDeviceInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceFilesystemAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceFilesystemRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceLocked, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & , const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceMounted, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & , const QString & , const QDBusVariant & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDevicePropertyChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & , const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceRemoved, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & , const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceUnlocked, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & , const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceUnmounted, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)()>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDriveAdded, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)()>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDriveRemoved, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & , const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::NotifyDeviceBusy, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::ProtocolDeviceAdded, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & , const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::ProtocolDeviceMounted, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & , const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::ProtocolDeviceRemoved, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & , const QString & )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::ProtocolDeviceUnmounted, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinFilemanagerDaemonDeviceManagerInterface::*)(const QString & , qlonglong , qlonglong )>(_a, &OrgDeepinFilemanagerDaemonDeviceManagerInterface::SizeUsedChanged, 16))
            return;
    }
}

const QMetaObject *OrgDeepinFilemanagerDaemonDeviceManagerInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgDeepinFilemanagerDaemonDeviceManagerInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN48OrgDeepinFilemanagerDaemonDeviceManagerInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgDeepinFilemanagerDaemonDeviceManagerInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceFilesystemAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceFilesystemRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceLocked(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceMounted(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDevicePropertyChanged(const QString & _t1, const QString & _t2, const QDBusVariant & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceRemoved(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceUnlocked(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDeviceUnmounted(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDriveAdded()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::BlockDriveRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::NotifyDeviceBusy(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::ProtocolDeviceAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::ProtocolDeviceMounted(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2);
}

// SIGNAL 14
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::ProtocolDeviceRemoved(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1, _t2);
}

// SIGNAL 15
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::ProtocolDeviceUnmounted(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1, _t2);
}

// SIGNAL 16
void OrgDeepinFilemanagerDaemonDeviceManagerInterface::SizeUsedChanged(const QString & _t1, qlonglong _t2, qlonglong _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
