// SPDX-FileCopyrightText: 2024 - 2025 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "dfmextwindowprivate.h"

BEGEN_DFMEXT_NAMESPACE

DFMExtWindowPrivate::DFMExtWindowPrivate()
{
}

END_DFMEXT_NAMESPACE
