/****************************************************************************
** Meta object code from reading C++ file 'avfsbrowser.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-avfsbrowser/avfsbrowser.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'avfsbrowser.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_avfsbrowser__AvfsBrowser_t {
    QByteArrayData data[1];
    char stringdata0[35];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_avfsbrowser__AvfsBrowser_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_avfsbrowser__AvfsBrowser_t qt_meta_stringdata_dfmplugin_avfsbrowser__AvfsBrowser = {
    {
QT_MOC_LITERAL(0, 0, 34) // "dfmplugin_avfsbrowser::AvfsBr..."

    },
    "dfmplugin_avfsbrowser::AvfsBrowser"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_avfsbrowser__AvfsBrowser[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void dfmplugin_avfsbrowser::AvfsBrowser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_avfsbrowser::AvfsBrowser::staticMetaObject = { {
    QMetaObject::SuperData::link<dpf::Plugin::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_avfsbrowser__AvfsBrowser.data,
    qt_meta_data_dfmplugin_avfsbrowser__AvfsBrowser,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_avfsbrowser::AvfsBrowser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_avfsbrowser::AvfsBrowser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_avfsbrowser__AvfsBrowser.stringdata0))
        return static_cast<void*>(this);
    return dpf::Plugin::qt_metacast(_clname);
}

int dfmplugin_avfsbrowser::AvfsBrowser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = dpf::Plugin::qt_metacall(_c, _id, _a);
    return _id;
}

QT_PLUGIN_METADATA_SECTION
static constexpr unsigned char qt_pluginMetaData[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'o',  'r',  'g',  '.',  'd', 
    'e',  'e',  'p',  'i',  'n',  '.',  'p',  'l', 
    'u',  'g',  'i',  'n',  '.',  'f',  'i',  'l', 
    'e',  'm',  'a',  'n',  'a',  'g',  'e',  'r', 
    // "className"
    0x03,  0x6b,  'A',  'v',  'f',  's',  'B',  'r', 
    'o',  'w',  's',  'e',  'r', 
    // "MetaData"
    0x04,  0xaa,  0x68,  'C',  'a',  't',  'e',  'g', 
    'o',  'r',  'y',  0x60,  0x6d,  'C',  'o',  'm', 
    'p',  'a',  't',  'V',  'e',  'r',  's',  'i', 
    'o',  'n',  0x65,  '4',  '.',  '8',  '.',  '0', 
    0x69,  'C',  'o',  'p',  'y',  'r',  'i',  'g', 
    'h',  't',  0x78,  0x41,  'C',  'o',  'p',  'y', 
    'r',  'i',  'g',  'h',  't',  ' ',  '(',  'C', 
    ')',  ' ',  '2',  '0',  '2',  '0',  ' ',  '~', 
    ' ',  '2',  '0',  '2',  '1',  ' ',  'U',  'n', 
    'i',  'o',  'n',  't',  'e',  'c',  'h',  ' ', 
    'S',  'o',  'f',  't',  'w',  'a',  'r',  'e', 
    ' ',  'T',  'e',  'c',  'h',  'n',  'o',  'l', 
    'o',  'g',  'y',  ' ',  'C',  'o',  '.',  ',', 
    ' ',  'L',  't',  'd',  '.',  0x67,  'D',  'e', 
    'p',  'e',  'n',  'd',  's',  0x80,  0x6b,  'D', 
    'e',  's',  'c',  'r',  'i',  'p',  't',  'i', 
    'o',  'n',  0x78,  0x28,  'T',  'h',  'e',  ' ', 
    'c',  'o',  'm',  'p',  'u',  't',  'e',  'r', 
    ' ',  'p',  'l',  'u',  'g',  'i',  'n',  ' ', 
    'f',  'o',  'r',  ' ',  't',  'h',  'e',  ' ', 
    'f',  'i',  'l',  'e',  'm',  'a',  'n',  'a', 
    'g',  'e',  'r',  '.',  0x67,  'L',  'i',  'c', 
    'e',  'n',  's',  'e',  0x80,  0x64,  'N',  'a', 
    'm',  'e',  0x75,  'd',  'f',  'm',  'p',  'l', 
    'u',  'g',  'i',  'n',  '-',  'a',  'v',  'f', 
    's',  'b',  'r',  'o',  'w',  's',  'e',  'r', 
    0x67,  'U',  'r',  'l',  'L',  'i',  'n',  'k', 
    0x78,  0x19,  'h',  't',  't',  'p',  's',  ':', 
    '/',  '/',  'w',  'w',  'w',  '.',  'u',  'n', 
    'i',  'o',  'n',  't',  'e',  'c',  'h',  '.', 
    'c',  'o',  'm',  0x66,  'V',  'e',  'n',  'd', 
    'o',  'r',  0x78,  0x2b,  'T',  'h',  'e',  ' ', 
    'U',  'n',  'i',  'o',  'n',  't',  'e',  'c', 
    'h',  ' ',  'S',  'o',  'f',  't',  'w',  'a', 
    'r',  'e',  ' ',  'T',  'e',  'c',  'h',  'n', 
    'o',  'l',  'o',  'g',  'y',  ' ',  'C',  'o', 
    '.',  ',',  ' ',  'L',  't',  'd',  '.',  0x67, 
    'V',  'e',  'r',  's',  'i',  'o',  'n',  0x65, 
    '4',  '.',  '8',  '.',  '2', 
    0xff, 
};
using namespace dfmplugin_avfsbrowser;
QT_MOC_EXPORT_PLUGIN(dfmplugin_avfsbrowser::AvfsBrowser, AvfsBrowser)

QT_WARNING_POP
QT_END_MOC_NAMESPACE
