/****************************************************************************
** Meta object code from reading C++ file 'devicemanagerdbus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/daemon/daemonplugin-core/devicemanagerdbus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'devicemanagerdbus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DeviceManagerDBus_t {
    QByteArrayData data[41];
    char stringdata0[696];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DeviceManagerDBus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DeviceManagerDBus_t qt_meta_stringdata_DeviceManagerDBus = {
    {
QT_MOC_LITERAL(0, 0, 17), // "DeviceManagerDBus"
QT_MOC_LITERAL(1, 18, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 34, 43), // "org.deepin.Filemanager.Daemon..."
QT_MOC_LITERAL(3, 78, 15), // "SizeUsedChanged"
QT_MOC_LITERAL(4, 94, 0), // ""
QT_MOC_LITERAL(5, 95, 2), // "id"
QT_MOC_LITERAL(6, 98, 5), // "total"
QT_MOC_LITERAL(7, 104, 4), // "free"
QT_MOC_LITERAL(8, 109, 16), // "NotifyDeviceBusy"
QT_MOC_LITERAL(9, 126, 6), // "action"
QT_MOC_LITERAL(10, 133, 15), // "BlockDriveAdded"
QT_MOC_LITERAL(11, 149, 17), // "BlockDriveRemoved"
QT_MOC_LITERAL(12, 167, 16), // "BlockDeviceAdded"
QT_MOC_LITERAL(13, 184, 18), // "BlockDeviceRemoved"
QT_MOC_LITERAL(14, 203, 6), // "oldMpt"
QT_MOC_LITERAL(15, 210, 26), // "BlockDeviceFilesystemAdded"
QT_MOC_LITERAL(16, 237, 28), // "BlockDeviceFilesystemRemoved"
QT_MOC_LITERAL(17, 266, 18), // "BlockDeviceMounted"
QT_MOC_LITERAL(18, 285, 10), // "mountPoint"
QT_MOC_LITERAL(19, 296, 20), // "BlockDeviceUnmounted"
QT_MOC_LITERAL(20, 317, 26), // "BlockDevicePropertyChanged"
QT_MOC_LITERAL(21, 344, 8), // "property"
QT_MOC_LITERAL(22, 353, 12), // "QDBusVariant"
QT_MOC_LITERAL(23, 366, 5), // "value"
QT_MOC_LITERAL(24, 372, 19), // "BlockDeviceUnlocked"
QT_MOC_LITERAL(25, 392, 13), // "clearDeviceId"
QT_MOC_LITERAL(26, 406, 17), // "BlockDeviceLocked"
QT_MOC_LITERAL(27, 424, 19), // "ProtocolDeviceAdded"
QT_MOC_LITERAL(28, 444, 21), // "ProtocolDeviceRemoved"
QT_MOC_LITERAL(29, 466, 21), // "ProtocolDeviceMounted"
QT_MOC_LITERAL(30, 488, 23), // "ProtocolDeviceUnmounted"
QT_MOC_LITERAL(31, 512, 16), // "IsMonotorWorking"
QT_MOC_LITERAL(32, 529, 17), // "DetachBlockDevice"
QT_MOC_LITERAL(33, 547, 20), // "DetachProtocolDevice"
QT_MOC_LITERAL(34, 568, 23), // "DetachAllMountedDevices"
QT_MOC_LITERAL(35, 592, 21), // "GetBlockDevicesIdList"
QT_MOC_LITERAL(36, 614, 4), // "opts"
QT_MOC_LITERAL(37, 619, 20), // "QueryBlockDeviceInfo"
QT_MOC_LITERAL(38, 640, 6), // "reload"
QT_MOC_LITERAL(39, 647, 24), // "GetProtocolDevicesIdList"
QT_MOC_LITERAL(40, 672, 23) // "QueryProtocolDeviceInfo"

    },
    "DeviceManagerDBus\0D-Bus Interface\0"
    "org.deepin.Filemanager.Daemon.DeviceManager\0"
    "SizeUsedChanged\0\0id\0total\0free\0"
    "NotifyDeviceBusy\0action\0BlockDriveAdded\0"
    "BlockDriveRemoved\0BlockDeviceAdded\0"
    "BlockDeviceRemoved\0oldMpt\0"
    "BlockDeviceFilesystemAdded\0"
    "BlockDeviceFilesystemRemoved\0"
    "BlockDeviceMounted\0mountPoint\0"
    "BlockDeviceUnmounted\0BlockDevicePropertyChanged\0"
    "property\0QDBusVariant\0value\0"
    "BlockDeviceUnlocked\0clearDeviceId\0"
    "BlockDeviceLocked\0ProtocolDeviceAdded\0"
    "ProtocolDeviceRemoved\0ProtocolDeviceMounted\0"
    "ProtocolDeviceUnmounted\0IsMonotorWorking\0"
    "DetachBlockDevice\0DetachProtocolDevice\0"
    "DetachAllMountedDevices\0GetBlockDevicesIdList\0"
    "opts\0QueryBlockDeviceInfo\0reload\0"
    "GetProtocolDevicesIdList\0"
    "QueryProtocolDeviceInfo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DeviceManagerDBus[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      25,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      17,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    3,  141,    4, 0x06 /* Public */,
       8,    2,  148,    4, 0x06 /* Public */,
      10,    0,  153,    4, 0x06 /* Public */,
      11,    0,  154,    4, 0x06 /* Public */,
      12,    1,  155,    4, 0x06 /* Public */,
      13,    2,  158,    4, 0x06 /* Public */,
      15,    1,  163,    4, 0x06 /* Public */,
      16,    1,  166,    4, 0x06 /* Public */,
      17,    2,  169,    4, 0x06 /* Public */,
      19,    2,  174,    4, 0x06 /* Public */,
      20,    3,  179,    4, 0x06 /* Public */,
      24,    2,  186,    4, 0x06 /* Public */,
      26,    1,  191,    4, 0x06 /* Public */,
      27,    1,  194,    4, 0x06 /* Public */,
      28,    2,  197,    4, 0x06 /* Public */,
      29,    2,  202,    4, 0x06 /* Public */,
      30,    2,  207,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      31,    0,  212,    4, 0x0a /* Public */,
      32,    1,  213,    4, 0x0a /* Public */,
      33,    1,  216,    4, 0x0a /* Public */,
      34,    0,  219,    4, 0x0a /* Public */,
      35,    1,  220,    4, 0x0a /* Public */,
      37,    2,  223,    4, 0x0a /* Public */,
      39,    0,  228,    4, 0x0a /* Public */,
      40,    2,  229,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::LongLong, QMetaType::LongLong,    5,    6,    7,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    5,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    5,   14,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    5,   18,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    5,   14,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, 0x80000000 | 22,    5,   21,   23,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    5,   25,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    5,   14,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    5,   18,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    5,   14,

 // slots: parameters
    QMetaType::Bool,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void,
    QMetaType::QStringList, QMetaType::Int,   36,
    QMetaType::QVariantMap, QMetaType::QString, QMetaType::Bool,    5,   38,
    QMetaType::QStringList,
    QMetaType::QVariantMap, QMetaType::QString, QMetaType::Bool,    5,   38,

       0        // eod
};

void DeviceManagerDBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DeviceManagerDBus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->SizeUsedChanged((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2])),(*reinterpret_cast< qint64(*)>(_a[3]))); break;
        case 1: _t->NotifyDeviceBusy((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 2: _t->BlockDriveAdded(); break;
        case 3: _t->BlockDriveRemoved(); break;
        case 4: _t->BlockDeviceAdded((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 5: _t->BlockDeviceRemoved((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->BlockDeviceFilesystemAdded((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 7: _t->BlockDeviceFilesystemRemoved((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 8: _t->BlockDeviceMounted((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 9: _t->BlockDeviceUnmounted((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 10: _t->BlockDevicePropertyChanged((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< QDBusVariant(*)>(_a[3]))); break;
        case 11: _t->BlockDeviceUnlocked((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 12: _t->BlockDeviceLocked((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 13: _t->ProtocolDeviceAdded((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 14: _t->ProtocolDeviceRemoved((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 15: _t->ProtocolDeviceMounted((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 16: _t->ProtocolDeviceUnmounted((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 17: { bool _r = _t->IsMonotorWorking();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->DetachBlockDevice((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 19: _t->DetachProtocolDevice((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 20: _t->DetachAllMountedDevices(); break;
        case 21: { QStringList _r = _t->GetBlockDevicesIdList((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 22: { QVariantMap _r = _t->QueryBlockDeviceInfo((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 23: { QStringList _r = _t->GetProtocolDevicesIdList();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 24: { QVariantMap _r = _t->QueryProtocolDeviceInfo((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariantMap*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusVariant >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DeviceManagerDBus::*)(QString , qint64 , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::SizeUsedChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::NotifyDeviceBusy)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::BlockDriveAdded)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::BlockDriveRemoved)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::BlockDeviceAdded)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::BlockDeviceRemoved)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::BlockDeviceFilesystemAdded)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::BlockDeviceFilesystemRemoved)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::BlockDeviceMounted)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::BlockDeviceUnmounted)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString , QString , QDBusVariant );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::BlockDevicePropertyChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::BlockDeviceUnlocked)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::BlockDeviceLocked)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::ProtocolDeviceAdded)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::ProtocolDeviceRemoved)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::ProtocolDeviceMounted)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDBus::*)(QString , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDBus::ProtocolDeviceUnmounted)) {
                *result = 16;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DeviceManagerDBus::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_DeviceManagerDBus.data,
    qt_meta_data_DeviceManagerDBus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DeviceManagerDBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeviceManagerDBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DeviceManagerDBus.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DeviceManagerDBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void DeviceManagerDBus::SizeUsedChanged(QString _t1, qint64 _t2, qint64 _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DeviceManagerDBus::NotifyDeviceBusy(QString _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DeviceManagerDBus::BlockDriveAdded()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void DeviceManagerDBus::BlockDriveRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void DeviceManagerDBus::BlockDeviceAdded(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void DeviceManagerDBus::BlockDeviceRemoved(QString _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void DeviceManagerDBus::BlockDeviceFilesystemAdded(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void DeviceManagerDBus::BlockDeviceFilesystemRemoved(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void DeviceManagerDBus::BlockDeviceMounted(QString _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void DeviceManagerDBus::BlockDeviceUnmounted(QString _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void DeviceManagerDBus::BlockDevicePropertyChanged(QString _t1, QString _t2, QDBusVariant _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void DeviceManagerDBus::BlockDeviceUnlocked(QString _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void DeviceManagerDBus::BlockDeviceLocked(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void DeviceManagerDBus::ProtocolDeviceAdded(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void DeviceManagerDBus::ProtocolDeviceRemoved(QString _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void DeviceManagerDBus::ProtocolDeviceMounted(QString _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void DeviceManagerDBus::ProtocolDeviceUnmounted(QString _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
