/****************************************************************************
** Meta object code from reading C++ file 'canvasproxymodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-canvas/model/canvasproxymodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvasproxymodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_canvas__CanvasProxyModel_t {
    QByteArrayData data[21];
    char stringdata0[203];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_canvas__CanvasProxyModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_canvas__CanvasProxyModel_t qt_meta_stringdata_ddplugin_canvas__CanvasProxyModel = {
    {
QT_MOC_LITERAL(0, 0, 33), // "ddplugin_canvas::CanvasProxyM..."
QT_MOC_LITERAL(1, 34, 12), // "dataReplaced"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 6), // "oldUrl"
QT_MOC_LITERAL(4, 55, 6), // "newUrl"
QT_MOC_LITERAL(5, 62, 4), // "sort"
QT_MOC_LITERAL(6, 67, 7), // "refresh"
QT_MOC_LITERAL(7, 75, 11), // "QModelIndex"
QT_MOC_LITERAL(8, 87, 6), // "parent"
QT_MOC_LITERAL(9, 94, 6), // "global"
QT_MOC_LITERAL(10, 101, 2), // "ms"
QT_MOC_LITERAL(11, 104, 10), // "updateFile"
QT_MOC_LITERAL(12, 115, 18), // "setShowHiddenFiles"
QT_MOC_LITERAL(13, 134, 4), // "show"
QT_MOC_LITERAL(14, 139, 5), // "fetch"
QT_MOC_LITERAL(15, 145, 3), // "url"
QT_MOC_LITERAL(16, 149, 4), // "take"
QT_MOC_LITERAL(17, 154, 11), // "mapToSource"
QT_MOC_LITERAL(18, 166, 10), // "proxyIndex"
QT_MOC_LITERAL(19, 177, 13), // "mapFromSource"
QT_MOC_LITERAL(20, 191, 11) // "sourceIndex"

    },
    "ddplugin_canvas::CanvasProxyModel\0"
    "dataReplaced\0\0oldUrl\0newUrl\0sort\0"
    "refresh\0QModelIndex\0parent\0global\0ms\0"
    "updateFile\0setShowHiddenFiles\0show\0"
    "fetch\0url\0take\0mapToSource\0proxyIndex\0"
    "mapFromSource\0sourceIndex"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_canvas__CanvasProxyModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   69,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   74,    2, 0x0a /* Public */,
       6,    4,   75,    2, 0x0a /* Public */,
       6,    3,   84,    2, 0x2a /* Public | MethodCloned */,
       6,    2,   91,    2, 0x2a /* Public | MethodCloned */,
       6,    1,   96,    2, 0x2a /* Public | MethodCloned */,
      12,    1,   99,    2, 0x0a /* Public */,
      14,    1,  102,    2, 0x0a /* Public */,
      16,    1,  105,    2, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
      17,    1,  108,    2, 0x02 /* Public */,
      19,    1,  111,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,    3,    4,

 // slots: parameters
    QMetaType::Bool,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Bool, QMetaType::Int, QMetaType::Bool,    8,    9,   10,   11,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Bool, QMetaType::Int,    8,    9,   10,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Bool,    8,    9,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Bool, QMetaType::QUrl,   15,
    QMetaType::Bool, QMetaType::QUrl,   15,

 // methods: parameters
    0x80000000 | 7, 0x80000000 | 7,   18,
    0x80000000 | 7, 0x80000000 | 7,   20,

       0        // eod
};

void ddplugin_canvas::CanvasProxyModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CanvasProxyModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->dataReplaced((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 1: { bool _r = _t->sort();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->refresh((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4]))); break;
        case 3: _t->refresh((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 4: _t->refresh((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 5: _t->refresh((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 6: _t->setShowHiddenFiles((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: { bool _r = _t->fetch((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->take((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { QModelIndex _r = _t->mapToSource((*reinterpret_cast< const QModelIndex(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 10: { QModelIndex _r = _t->mapFromSource((*reinterpret_cast< const QModelIndex(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QModelIndex*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CanvasProxyModel::*)(const QUrl & , const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CanvasProxyModel::dataReplaced)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_canvas::CanvasProxyModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractProxyModel::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_canvas__CanvasProxyModel.data,
    qt_meta_data_ddplugin_canvas__CanvasProxyModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_canvas::CanvasProxyModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_canvas::CanvasProxyModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_canvas__CanvasProxyModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractProxyModel::qt_metacast(_clname);
}

int ddplugin_canvas::CanvasProxyModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void ddplugin_canvas::CanvasProxyModel::dataReplaced(const QUrl & _t1, const QUrl & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
