// SPDX-FileCopyrightText: 2022 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef DFMPLUGIN_OPTICAL_GLOBAL_H
#define DFMPLUGIN_OPTICAL_GLOBAL_H

#include <dfm-base/dfm_log_defines.h>

#define DPOPTICAL_NAMESPACE dfmplugin_optical

#define DPOPTICAL_BEGIN_NAMESPACE namespace DPOPTICAL_NAMESPACE {
#define DPOPTICAL_END_NAMESPACE }
#define DPOPTICAL_USE_NAMESPACE using namespace DPOPTICAL_NAMESPACE;

DPOPTICAL_BEGIN_NAMESPACE
DFM_LOG_USE_CATEGORY(DPOPTICAL_NAMESPACE)
DPOPTICAL_END_NAMESPACE

#endif   // DFMPLUGIN_OPTICAL_GLOBAL_H
