/****************************************************************************
** Meta object code from reading C++ file 'vaulteventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-vault/events/vaulteventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vaulteventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_vault__VaultEventReceiver_t {
    QByteArrayData data[31];
    char stringdata0[434];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_vault__VaultEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_vault__VaultEventReceiver_t qt_meta_stringdata_dfmplugin_vault__VaultEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 35), // "dfmplugin_vault::VaultEventRe..."
QT_MOC_LITERAL(1, 36, 16), // "computerOpenItem"
QT_MOC_LITERAL(2, 53, 0), // ""
QT_MOC_LITERAL(3, 54, 5), // "winId"
QT_MOC_LITERAL(4, 60, 3), // "url"
QT_MOC_LITERAL(5, 64, 30), // "handleNotAllowedAppendCompress"
QT_MOC_LITERAL(6, 95, 11), // "QList<QUrl>"
QT_MOC_LITERAL(7, 107, 8), // "fromUrls"
QT_MOC_LITERAL(8, 116, 5), // "toUrl"
QT_MOC_LITERAL(9, 122, 23), // "handleCurrentUrlChanged"
QT_MOC_LITERAL(10, 146, 29), // "handleSideBarItemDragMoveData"
QT_MOC_LITERAL(11, 176, 4), // "urls"
QT_MOC_LITERAL(12, 181, 15), // "Qt::DropAction*"
QT_MOC_LITERAL(13, 197, 6), // "action"
QT_MOC_LITERAL(14, 204, 24), // "handleShortCutPasteFiles"
QT_MOC_LITERAL(15, 229, 2), // "to"
QT_MOC_LITERAL(16, 232, 20), // "changeUrlEventFilter"
QT_MOC_LITERAL(17, 253, 8), // "windowId"
QT_MOC_LITERAL(18, 262, 19), // "handlePathtoVirtual"
QT_MOC_LITERAL(19, 282, 5), // "files"
QT_MOC_LITERAL(20, 288, 12), // "QList<QUrl>*"
QT_MOC_LITERAL(21, 301, 12), // "virtualFiles"
QT_MOC_LITERAL(22, 314, 14), // "detailViewIcon"
QT_MOC_LITERAL(23, 329, 8), // "QString*"
QT_MOC_LITERAL(24, 338, 8), // "iconName"
QT_MOC_LITERAL(25, 347, 24), // "fileDropHandleWithAction"
QT_MOC_LITERAL(26, 372, 23), // "handlePermissionViewAsh"
QT_MOC_LITERAL(27, 396, 5), // "bool*"
QT_MOC_LITERAL(28, 402, 5), // "isAsh"
QT_MOC_LITERAL(29, 408, 18), // "handleFileCanTaged"
QT_MOC_LITERAL(30, 427, 6) // "canTag"

    },
    "dfmplugin_vault::VaultEventReceiver\0"
    "computerOpenItem\0\0winId\0url\0"
    "handleNotAllowedAppendCompress\0"
    "QList<QUrl>\0fromUrls\0toUrl\0"
    "handleCurrentUrlChanged\0"
    "handleSideBarItemDragMoveData\0urls\0"
    "Qt::DropAction*\0action\0handleShortCutPasteFiles\0"
    "to\0changeUrlEventFilter\0windowId\0"
    "handlePathtoVirtual\0files\0QList<QUrl>*\0"
    "virtualFiles\0detailViewIcon\0QString*\0"
    "iconName\0fileDropHandleWithAction\0"
    "handlePermissionViewAsh\0bool*\0isAsh\0"
    "handleFileCanTaged\0canTag"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_vault__VaultEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   69,    2, 0x0a /* Public */,
       5,    2,   74,    2, 0x0a /* Public */,
       9,    2,   79,    2, 0x0a /* Public */,
      10,    3,   84,    2, 0x0a /* Public */,
      14,    3,   91,    2, 0x0a /* Public */,
      16,    2,   98,    2, 0x0a /* Public */,
      18,    2,  103,    2, 0x0a /* Public */,
      22,    2,  108,    2, 0x0a /* Public */,
      25,    3,  113,    2, 0x0a /* Public */,
      26,    2,  120,    2, 0x0a /* Public */,
      29,    2,  125,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,    3,    4,
    QMetaType::Bool, 0x80000000 | 6, QMetaType::QUrl,    7,    8,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,    3,    4,
    QMetaType::Bool, 0x80000000 | 6, QMetaType::QUrl, 0x80000000 | 12,   11,    4,   13,
    QMetaType::Bool, QMetaType::ULongLong, 0x80000000 | 6, QMetaType::QUrl,    3,    7,   15,
    QMetaType::Bool, QMetaType::ULongLong, QMetaType::QUrl,   17,    4,
    QMetaType::Bool, 0x80000000 | 6, 0x80000000 | 20,   19,   21,
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 23,    4,   24,
    QMetaType::Bool, 0x80000000 | 6, QMetaType::QUrl, 0x80000000 | 12,    7,    8,   13,
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 27,    4,   28,
    QMetaType::Bool, QMetaType::QUrl, 0x80000000 | 27,    4,   30,

       0        // eod
};

void dfmplugin_vault::VaultEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<VaultEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->computerOpenItem((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 1: { bool _r = _t->handleNotAllowedAppendCompress((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->handleCurrentUrlChanged((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 3: { bool _r = _t->handleSideBarItemDragMoveData((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< Qt::DropAction*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->handleShortCutPasteFiles((*reinterpret_cast< const quint64(*)>(_a[1])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->changeUrlEventFilter((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->handlePathtoVirtual((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< QList<QUrl>*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->detailViewIcon((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QString*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->fileDropHandleWithAction((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< Qt::DropAction*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->handlePermissionViewAsh((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< bool*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->handleFileCanTaged((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< bool*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_vault::VaultEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_vault__VaultEventReceiver.data,
    qt_meta_data_dfmplugin_vault__VaultEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_vault::VaultEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_vault::VaultEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_vault__VaultEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_vault::VaultEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
