/****************************************************************************
** Meta object code from reading C++ file 'fileinfomodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-canvas/model/fileinfomodel_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileinfomodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_canvas__FileInfoModelPrivate_t {
    QByteArrayData data[16];
    char stringdata0[170];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_canvas__FileInfoModelPrivate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_canvas__FileInfoModelPrivate_t qt_meta_stringdata_ddplugin_canvas__FileInfoModelPrivate = {
    {
QT_MOC_LITERAL(0, 0, 37), // "ddplugin_canvas::FileInfoMode..."
QT_MOC_LITERAL(1, 38, 9), // "resetData"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 11), // "QList<QUrl>"
QT_MOC_LITERAL(4, 61, 4), // "urls"
QT_MOC_LITERAL(5, 66, 10), // "insertData"
QT_MOC_LITERAL(6, 77, 3), // "url"
QT_MOC_LITERAL(7, 81, 10), // "removeData"
QT_MOC_LITERAL(8, 92, 11), // "replaceData"
QT_MOC_LITERAL(9, 104, 6), // "oldUrl"
QT_MOC_LITERAL(10, 111, 6), // "newUrl"
QT_MOC_LITERAL(11, 118, 10), // "updateData"
QT_MOC_LITERAL(12, 129, 11), // "dataUpdated"
QT_MOC_LITERAL(13, 141, 9), // "isLinkOrg"
QT_MOC_LITERAL(14, 151, 12), // "thumbUpdated"
QT_MOC_LITERAL(15, 164, 5) // "thumb"

    },
    "ddplugin_canvas::FileInfoModelPrivate\0"
    "resetData\0\0QList<QUrl>\0urls\0insertData\0"
    "url\0removeData\0replaceData\0oldUrl\0"
    "newUrl\0updateData\0dataUpdated\0isLinkOrg\0"
    "thumbUpdated\0thumb"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_canvas__FileInfoModelPrivate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x0a /* Public */,
       5,    1,   52,    2, 0x0a /* Public */,
       7,    1,   55,    2, 0x0a /* Public */,
       8,    2,   58,    2, 0x0a /* Public */,
      11,    1,   63,    2, 0x0a /* Public */,
      12,    2,   66,    2, 0x0a /* Public */,
      14,    2,   71,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QUrl,    6,
    QMetaType::Void, QMetaType::QUrl,    6,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,    9,   10,
    QMetaType::Void, QMetaType::QUrl,    6,
    QMetaType::Void, QMetaType::QUrl, QMetaType::Bool,    6,   13,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QString,    6,   15,

       0        // eod
};

void ddplugin_canvas::FileInfoModelPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileInfoModelPrivate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->resetData((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 1: _t->insertData((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 2: _t->removeData((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 3: _t->replaceData((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 4: _t->updateData((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 5: _t->dataUpdated((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        case 6: _t->thumbUpdated((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_canvas::FileInfoModelPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_canvas__FileInfoModelPrivate.data,
    qt_meta_data_ddplugin_canvas__FileInfoModelPrivate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_canvas::FileInfoModelPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_canvas::FileInfoModelPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_canvas__FileInfoModelPrivate.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_canvas::FileInfoModelPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
