/****************************************************************************
** Meta object code from reading C++ file 'filebaseinfoview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-detailspace/views/filebaseinfoview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filebaseinfoview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_detailspace__FileBaseInfoView_t {
    QByteArrayData data[14];
    char stringdata0[263];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_detailspace__FileBaseInfoView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_detailspace__FileBaseInfoView_t qt_meta_stringdata_dfmplugin_detailspace__FileBaseInfoView = {
    {
QT_MOC_LITERAL(0, 0, 39), // "dfmplugin_detailspace::FileBa..."
QT_MOC_LITERAL(1, 40, 17), // "sigImageExtenInfo"
QT_MOC_LITERAL(2, 58, 0), // ""
QT_MOC_LITERAL(3, 59, 10), // "properties"
QT_MOC_LITERAL(4, 70, 17), // "sigVideoExtenInfo"
QT_MOC_LITERAL(5, 88, 17), // "sigAudioExtenInfo"
QT_MOC_LITERAL(6, 106, 14), // "imageExtenInfo"
QT_MOC_LITERAL(7, 121, 3), // "url"
QT_MOC_LITERAL(8, 125, 50), // "QMap<dfmio::DFileInfo::Attrib..."
QT_MOC_LITERAL(9, 176, 14), // "videoExtenInfo"
QT_MOC_LITERAL(10, 191, 14), // "audioExtenInfo"
QT_MOC_LITERAL(11, 206, 18), // "slotImageExtenInfo"
QT_MOC_LITERAL(12, 225, 18), // "slotVideoExtenInfo"
QT_MOC_LITERAL(13, 244, 18) // "slotAudioExtenInfo"

    },
    "dfmplugin_detailspace::FileBaseInfoView\0"
    "sigImageExtenInfo\0\0properties\0"
    "sigVideoExtenInfo\0sigAudioExtenInfo\0"
    "imageExtenInfo\0url\0"
    "QMap<dfmio::DFileInfo::AttributeExtendID,QVariant>\0"
    "videoExtenInfo\0audioExtenInfo\0"
    "slotImageExtenInfo\0slotVideoExtenInfo\0"
    "slotAudioExtenInfo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_detailspace__FileBaseInfoView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,
       4,    1,   62,    2, 0x06 /* Public */,
       5,    1,   65,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    2,   68,    2, 0x0a /* Public */,
       9,    2,   73,    2, 0x0a /* Public */,
      10,    2,   78,    2, 0x0a /* Public */,
      11,    1,   83,    2, 0x0a /* Public */,
      12,    1,   86,    2, 0x0a /* Public */,
      13,    1,   89,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QStringList,    3,
    QMetaType::Void, QMetaType::QStringList,    3,
    QMetaType::Void, QMetaType::QStringList,    3,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 8,    7,    3,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 8,    7,    3,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 8,    7,    3,
    QMetaType::Void, QMetaType::QStringList,    3,
    QMetaType::Void, QMetaType::QStringList,    3,
    QMetaType::Void, QMetaType::QStringList,    3,

       0        // eod
};

void dfmplugin_detailspace::FileBaseInfoView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileBaseInfoView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sigImageExtenInfo((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 1: _t->sigVideoExtenInfo((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 2: _t->sigAudioExtenInfo((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 3: _t->imageExtenInfo((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QMap<dfmio::DFileInfo::AttributeExtendID,QVariant>(*)>(_a[2]))); break;
        case 4: _t->videoExtenInfo((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QMap<dfmio::DFileInfo::AttributeExtendID,QVariant>(*)>(_a[2]))); break;
        case 5: _t->audioExtenInfo((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QMap<dfmio::DFileInfo::AttributeExtendID,QVariant>(*)>(_a[2]))); break;
        case 6: _t->slotImageExtenInfo((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 7: _t->slotVideoExtenInfo((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 8: _t->slotAudioExtenInfo((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileBaseInfoView::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileBaseInfoView::sigImageExtenInfo)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileBaseInfoView::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileBaseInfoView::sigVideoExtenInfo)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileBaseInfoView::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileBaseInfoView::sigAudioExtenInfo)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_detailspace::FileBaseInfoView::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_detailspace__FileBaseInfoView.data,
    qt_meta_data_dfmplugin_detailspace__FileBaseInfoView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_detailspace::FileBaseInfoView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_detailspace::FileBaseInfoView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_detailspace__FileBaseInfoView.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int dfmplugin_detailspace::FileBaseInfoView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_detailspace::FileBaseInfoView::sigImageExtenInfo(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_detailspace::FileBaseInfoView::sigVideoExtenInfo(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_detailspace::FileBaseInfoView::sigAudioExtenInfo(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
