/****************************************************************************
** Meta object code from reading C++ file 'tagcolorlistwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-tag/widgets/tagcolorlistwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tagcolorlistwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_tag__TagColorListWidget_t {
    QByteArrayData data[12];
    char stringdata0[164];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_tag__TagColorListWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_tag__TagColorListWidget_t qt_meta_stringdata_dfmplugin_tag__TagColorListWidget = {
    {
QT_MOC_LITERAL(0, 0, 33), // "dfmplugin_tag::TagColorListWi..."
QT_MOC_LITERAL(1, 34, 17), // "hoverColorChanged"
QT_MOC_LITERAL(2, 52, 0), // ""
QT_MOC_LITERAL(3, 53, 5), // "color"
QT_MOC_LITERAL(4, 59, 19), // "checkedColorChanged"
QT_MOC_LITERAL(5, 79, 17), // "initUiForSizeMode"
QT_MOC_LITERAL(6, 97, 16), // "checkedColorList"
QT_MOC_LITERAL(7, 114, 13), // "QList<QColor>"
QT_MOC_LITERAL(8, 128, 9), // "exclusive"
QT_MOC_LITERAL(9, 138, 7), // "UseType"
QT_MOC_LITERAL(10, 146, 7), // "kAction"
QT_MOC_LITERAL(11, 154, 9) // "kProperty"

    },
    "dfmplugin_tag::TagColorListWidget\0"
    "hoverColorChanged\0\0color\0checkedColorChanged\0"
    "initUiForSizeMode\0checkedColorList\0"
    "QList<QColor>\0exclusive\0UseType\0kAction\0"
    "kProperty"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_tag__TagColorListWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       2,   36, // properties
       1,   42, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,
       4,    1,   32,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   35,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QColor,    3,
    QMetaType::Void, QMetaType::QColor,    3,

 // slots: parameters
    QMetaType::Void,

 // properties: name, type, flags
       6, 0x80000000 | 7, 0x0009510b,
       8, QMetaType::Bool, 0x00095103,

 // enums: name, alias, flags, count, data
       9,    9, 0x0,    2,   47,

 // enum data: key, value
      10, uint(dfmplugin_tag::TagColorListWidget::kAction),
      11, uint(dfmplugin_tag::TagColorListWidget::kProperty),

       0        // eod
};

void dfmplugin_tag::TagColorListWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TagColorListWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->hoverColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 1: _t->checkedColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 2: _t->initUiForSizeMode(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TagColorListWidget::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagColorListWidget::hoverColorChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TagColorListWidget::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagColorListWidget::checkedColorChanged)) {
                *result = 1;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QColor> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<TagColorListWidget *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QList<QColor>*>(_v) = _t->checkedColorList(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->exclusive(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<TagColorListWidget *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCheckedColorList(*reinterpret_cast< QList<QColor>*>(_v)); break;
        case 1: _t->setExclusive(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_tag::TagColorListWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_tag__TagColorListWidget.data,
    qt_meta_data_dfmplugin_tag__TagColorListWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_tag::TagColorListWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_tag::TagColorListWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_tag__TagColorListWidget.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int dfmplugin_tag::TagColorListWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void dfmplugin_tag::TagColorListWidget::hoverColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_tag::TagColorListWidget::checkedColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
