/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a recentmanageradaptor -i recentmanagerdbus.h -l RecentManagerDBus /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.Daemon.RecentManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "recentmanageradaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class RecentManagerAdaptor
 */

RecentManagerAdaptor::RecentManagerAdaptor(RecentManagerDBus *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

RecentManagerAdaptor::~RecentManagerAdaptor()
{
    // destructor
}

void RecentManagerAdaptor::AddItem(const QVariantMap &item)
{
    // handle method call org.deepin.Filemanager.Daemon.RecentManager.AddItem
    parent()->AddItem(item);
}

QVariantMap RecentManagerAdaptor::GetItemInfo(const QString &path)
{
    // handle method call org.deepin.Filemanager.Daemon.RecentManager.GetItemInfo
    return parent()->GetItemInfo(path);
}

QVariantList RecentManagerAdaptor::GetItemsInfo()
{
    // handle method call org.deepin.Filemanager.Daemon.RecentManager.GetItemsInfo
    return parent()->GetItemsInfo();
}

QStringList RecentManagerAdaptor::GetItemsPath()
{
    // handle method call org.deepin.Filemanager.Daemon.RecentManager.GetItemsPath
    return parent()->GetItemsPath();
}

void RecentManagerAdaptor::PurgeItems()
{
    // handle method call org.deepin.Filemanager.Daemon.RecentManager.PurgeItems
    parent()->PurgeItems();
}

qlonglong RecentManagerAdaptor::Reload()
{
    // handle method call org.deepin.Filemanager.Daemon.RecentManager.Reload
    return parent()->Reload();
}

void RecentManagerAdaptor::RemoveItems(const QStringList &hrefs)
{
    // handle method call org.deepin.Filemanager.Daemon.RecentManager.RemoveItems
    parent()->RemoveItems(hrefs);
}


#include "recentmanageradaptor.moc"
