/****************************************************************************
** Meta object code from reading C++ file 'indextask.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../dde-file-manager/src/services/textindex/task/indextask.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'indextask.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_service_textindex__IndexTask_t {
    QByteArrayData data[17];
    char stringdata0[177];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_service_textindex__IndexTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_service_textindex__IndexTask_t qt_meta_stringdata_service_textindex__IndexTask = {
    {
QT_MOC_LITERAL(0, 0, 28), // "service_textindex::IndexTask"
QT_MOC_LITERAL(1, 29, 15), // "progressChanged"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 34), // "service_textindex::IndexTask:..."
QT_MOC_LITERAL(4, 81, 4), // "type"
QT_MOC_LITERAL(5, 86, 5), // "count"
QT_MOC_LITERAL(6, 92, 8), // "finished"
QT_MOC_LITERAL(7, 101, 7), // "success"
QT_MOC_LITERAL(8, 109, 4), // "Type"
QT_MOC_LITERAL(9, 114, 6), // "Create"
QT_MOC_LITERAL(10, 121, 6), // "Update"
QT_MOC_LITERAL(11, 128, 6), // "Remove"
QT_MOC_LITERAL(12, 135, 6), // "Status"
QT_MOC_LITERAL(13, 142, 10), // "NotStarted"
QT_MOC_LITERAL(14, 153, 7), // "Running"
QT_MOC_LITERAL(15, 161, 8), // "Finished"
QT_MOC_LITERAL(16, 170, 6) // "Failed"

    },
    "service_textindex::IndexTask\0"
    "progressChanged\0\0service_textindex::IndexTask::Type\0"
    "type\0count\0finished\0success\0Type\0"
    "Create\0Update\0Remove\0Status\0NotStarted\0"
    "Running\0Finished\0Failed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_service_textindex__IndexTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       2,   34, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x06 /* Public */,
       6,    2,   29,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::LongLong,    4,    5,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    7,

 // enums: name, alias, flags, count, data
       8,    8, 0x2,    3,   44,
      12,   12, 0x2,    4,   50,

 // enum data: key, value
       9, uint(service_textindex::IndexTask::Type::Create),
      10, uint(service_textindex::IndexTask::Type::Update),
      11, uint(service_textindex::IndexTask::Type::Remove),
      13, uint(service_textindex::IndexTask::Status::NotStarted),
      14, uint(service_textindex::IndexTask::Status::Running),
      15, uint(service_textindex::IndexTask::Status::Finished),
      16, uint(service_textindex::IndexTask::Status::Failed),

       0        // eod
};

void service_textindex::IndexTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<IndexTask *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->progressChanged((*reinterpret_cast< service_textindex::IndexTask::Type(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2]))); break;
        case 1: _t->finished((*reinterpret_cast< service_textindex::IndexTask::Type(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< service_textindex::IndexTask::Type >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< service_textindex::IndexTask::Type >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (IndexTask::*)(service_textindex::IndexTask::Type , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IndexTask::progressChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (IndexTask::*)(service_textindex::IndexTask::Type , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IndexTask::finished)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject service_textindex::IndexTask::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_service_textindex__IndexTask.data,
    qt_meta_data_service_textindex__IndexTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *service_textindex::IndexTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *service_textindex::IndexTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_service_textindex__IndexTask.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int service_textindex::IndexTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void service_textindex::IndexTask::progressChanged(service_textindex::IndexTask::Type _t1, qint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void service_textindex::IndexTask::finished(service_textindex::IndexTask::Type _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
