/****************************************************************************
** Meta object code from reading C++ file 'burnjob.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-burn/utils/burnjob.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'burnjob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_burn__AbstractBurnJob_t {
    QByteArrayData data[23];
    char stringdata0[262];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_burn__AbstractBurnJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_burn__AbstractBurnJob_t qt_meta_stringdata_dfmplugin_burn__AbstractBurnJob = {
    {
QT_MOC_LITERAL(0, 0, 31), // "dfmplugin_burn::AbstractBurnJob"
QT_MOC_LITERAL(1, 32, 25), // "requestErrorMessageDialog"
QT_MOC_LITERAL(2, 58, 0), // ""
QT_MOC_LITERAL(3, 59, 5), // "title"
QT_MOC_LITERAL(4, 65, 7), // "message"
QT_MOC_LITERAL(5, 73, 20), // "requestFailureDialog"
QT_MOC_LITERAL(6, 94, 4), // "type"
QT_MOC_LITERAL(7, 99, 3), // "err"
QT_MOC_LITERAL(8, 103, 7), // "details"
QT_MOC_LITERAL(9, 111, 23), // "requestCompletionDialog"
QT_MOC_LITERAL(10, 135, 3), // "msg"
QT_MOC_LITERAL(11, 139, 4), // "icon"
QT_MOC_LITERAL(12, 144, 15), // "requestCloseTab"
QT_MOC_LITERAL(13, 160, 3), // "url"
QT_MOC_LITERAL(14, 164, 17), // "requestReloadDisc"
QT_MOC_LITERAL(15, 182, 5), // "devId"
QT_MOC_LITERAL(16, 188, 12), // "burnFinished"
QT_MOC_LITERAL(17, 201, 6), // "reuslt"
QT_MOC_LITERAL(18, 208, 12), // "onJobUpdated"
QT_MOC_LITERAL(19, 221, 18), // "dfmburn::JobStatus"
QT_MOC_LITERAL(20, 240, 6), // "status"
QT_MOC_LITERAL(21, 247, 8), // "progress"
QT_MOC_LITERAL(22, 256, 5) // "speed"

    },
    "dfmplugin_burn::AbstractBurnJob\0"
    "requestErrorMessageDialog\0\0title\0"
    "message\0requestFailureDialog\0type\0err\0"
    "details\0requestCompletionDialog\0msg\0"
    "icon\0requestCloseTab\0url\0requestReloadDisc\0"
    "devId\0burnFinished\0reuslt\0onJobUpdated\0"
    "dfmburn::JobStatus\0status\0progress\0"
    "speed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_burn__AbstractBurnJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   49,    2, 0x06 /* Public */,
       5,    3,   54,    2, 0x06 /* Public */,
       9,    2,   61,    2, 0x06 /* Public */,
      12,    1,   66,    2, 0x06 /* Public */,
      14,    1,   69,    2, 0x06 /* Public */,
      16,    2,   72,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    4,   77,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    4,
    QMetaType::Void, QMetaType::Int, QMetaType::QString, QMetaType::QStringList,    6,    7,    8,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   10,   11,
    QMetaType::Void, QMetaType::QUrl,   13,
    QMetaType::Void, QMetaType::QString,   15,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    6,   17,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 19, QMetaType::Int, QMetaType::QString, QMetaType::QStringList,   20,   21,   22,    4,

       0        // eod
};

void dfmplugin_burn::AbstractBurnJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AbstractBurnJob *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestErrorMessageDialog((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->requestFailureDialog((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3]))); break;
        case 2: _t->requestCompletionDialog((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->requestCloseTab((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 4: _t->requestReloadDisc((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->burnFinished((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 6: _t->onJobUpdated((*reinterpret_cast< dfmburn::JobStatus(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QStringList(*)>(_a[4]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AbstractBurnJob::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractBurnJob::requestErrorMessageDialog)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AbstractBurnJob::*)(int , const QString & , const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractBurnJob::requestFailureDialog)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (AbstractBurnJob::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractBurnJob::requestCompletionDialog)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (AbstractBurnJob::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractBurnJob::requestCloseTab)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (AbstractBurnJob::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractBurnJob::requestReloadDisc)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (AbstractBurnJob::*)(int , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractBurnJob::burnFinished)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_burn::AbstractBurnJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_burn__AbstractBurnJob.data,
    qt_meta_data_dfmplugin_burn__AbstractBurnJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_burn::AbstractBurnJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_burn::AbstractBurnJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_burn__AbstractBurnJob.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int dfmplugin_burn::AbstractBurnJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_burn::AbstractBurnJob::requestErrorMessageDialog(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_burn::AbstractBurnJob::requestFailureDialog(int _t1, const QString & _t2, const QStringList & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_burn::AbstractBurnJob::requestCompletionDialog(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_burn::AbstractBurnJob::requestCloseTab(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_burn::AbstractBurnJob::requestReloadDisc(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmplugin_burn::AbstractBurnJob::burnFinished(int _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
struct qt_meta_stringdata_dfmplugin_burn__EraseJob_t {
    QByteArrayData data[4];
    char stringdata0[47];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_burn__EraseJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_burn__EraseJob_t qt_meta_stringdata_dfmplugin_burn__EraseJob = {
    {
QT_MOC_LITERAL(0, 0, 24), // "dfmplugin_burn::EraseJob"
QT_MOC_LITERAL(1, 25, 13), // "eraseFinished"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 6) // "result"

    },
    "dfmplugin_burn::EraseJob\0eraseFinished\0"
    "\0result"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_burn__EraseJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,

       0        // eod
};

void dfmplugin_burn::EraseJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EraseJob *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->eraseFinished((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (EraseJob::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EraseJob::eraseFinished)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_burn::EraseJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractBurnJob::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_burn__EraseJob.data,
    qt_meta_data_dfmplugin_burn__EraseJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_burn::EraseJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_burn::EraseJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_burn__EraseJob.stringdata0))
        return static_cast<void*>(this);
    return AbstractBurnJob::qt_metacast(_clname);
}

int dfmplugin_burn::EraseJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractBurnJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_burn::EraseJob::eraseFinished(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_dfmplugin_burn__BurnISOFilesJob_t {
    QByteArrayData data[1];
    char stringdata0[32];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_burn__BurnISOFilesJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_burn__BurnISOFilesJob_t qt_meta_stringdata_dfmplugin_burn__BurnISOFilesJob = {
    {
QT_MOC_LITERAL(0, 0, 31) // "dfmplugin_burn::BurnISOFilesJob"

    },
    "dfmplugin_burn::BurnISOFilesJob"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_burn__BurnISOFilesJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void dfmplugin_burn::BurnISOFilesJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_burn::BurnISOFilesJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractBurnJob::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_burn__BurnISOFilesJob.data,
    qt_meta_data_dfmplugin_burn__BurnISOFilesJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_burn::BurnISOFilesJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_burn::BurnISOFilesJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_burn__BurnISOFilesJob.stringdata0))
        return static_cast<void*>(this);
    return AbstractBurnJob::qt_metacast(_clname);
}

int dfmplugin_burn::BurnISOFilesJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractBurnJob::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_dfmplugin_burn__BurnISOImageJob_t {
    QByteArrayData data[1];
    char stringdata0[32];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_burn__BurnISOImageJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_burn__BurnISOImageJob_t qt_meta_stringdata_dfmplugin_burn__BurnISOImageJob = {
    {
QT_MOC_LITERAL(0, 0, 31) // "dfmplugin_burn::BurnISOImageJob"

    },
    "dfmplugin_burn::BurnISOImageJob"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_burn__BurnISOImageJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void dfmplugin_burn::BurnISOImageJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_burn::BurnISOImageJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractBurnJob::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_burn__BurnISOImageJob.data,
    qt_meta_data_dfmplugin_burn__BurnISOImageJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_burn::BurnISOImageJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_burn::BurnISOImageJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_burn__BurnISOImageJob.stringdata0))
        return static_cast<void*>(this);
    return AbstractBurnJob::qt_metacast(_clname);
}

int dfmplugin_burn::BurnISOImageJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractBurnJob::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_dfmplugin_burn__BurnUDFFilesJob_t {
    QByteArrayData data[1];
    char stringdata0[32];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_burn__BurnUDFFilesJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_burn__BurnUDFFilesJob_t qt_meta_stringdata_dfmplugin_burn__BurnUDFFilesJob = {
    {
QT_MOC_LITERAL(0, 0, 31) // "dfmplugin_burn::BurnUDFFilesJob"

    },
    "dfmplugin_burn::BurnUDFFilesJob"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_burn__BurnUDFFilesJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void dfmplugin_burn::BurnUDFFilesJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_burn::BurnUDFFilesJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractBurnJob::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_burn__BurnUDFFilesJob.data,
    qt_meta_data_dfmplugin_burn__BurnUDFFilesJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_burn::BurnUDFFilesJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_burn::BurnUDFFilesJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_burn__BurnUDFFilesJob.stringdata0))
        return static_cast<void*>(this);
    return AbstractBurnJob::qt_metacast(_clname);
}

int dfmplugin_burn::BurnUDFFilesJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractBurnJob::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_dfmplugin_burn__DumpISOImageJob_t {
    QByteArrayData data[5];
    char stringdata0[111];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_burn__DumpISOImageJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_burn__DumpISOImageJob_t qt_meta_stringdata_dfmplugin_burn__DumpISOImageJob = {
    {
QT_MOC_LITERAL(0, 0, 31), // "dfmplugin_burn::DumpISOImageJob"
QT_MOC_LITERAL(1, 32, 34), // "requestOpticalDumpISOSuccessD..."
QT_MOC_LITERAL(2, 67, 0), // ""
QT_MOC_LITERAL(3, 68, 8), // "imageUrl"
QT_MOC_LITERAL(4, 77, 33) // "requestOpticalDumpISOFailedDi..."

    },
    "dfmplugin_burn::DumpISOImageJob\0"
    "requestOpticalDumpISOSuccessDialog\0\0"
    "imageUrl\0requestOpticalDumpISOFailedDialog"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_burn__DumpISOImageJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,
       4,    0,   27,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl,    3,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_burn::DumpISOImageJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DumpISOImageJob *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestOpticalDumpISOSuccessDialog((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 1: _t->requestOpticalDumpISOFailedDialog(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DumpISOImageJob::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DumpISOImageJob::requestOpticalDumpISOSuccessDialog)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DumpISOImageJob::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DumpISOImageJob::requestOpticalDumpISOFailedDialog)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_burn::DumpISOImageJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractBurnJob::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_burn__DumpISOImageJob.data,
    qt_meta_data_dfmplugin_burn__DumpISOImageJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_burn::DumpISOImageJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_burn::DumpISOImageJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_burn__DumpISOImageJob.stringdata0))
        return static_cast<void*>(this);
    return AbstractBurnJob::qt_metacast(_clname);
}

int dfmplugin_burn::DumpISOImageJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractBurnJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_burn::DumpISOImageJob::requestOpticalDumpISOSuccessDialog(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_burn::DumpISOImageJob::requestOpticalDumpISOFailedDialog()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
