/****************************************************************************
** Meta object code from reading C++ file 'wallpaperitem.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-wallpapersetting/wallpaperitem.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QQueue>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wallpaperitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_wallpapersetting__WrapperWidget_t {
    QByteArrayData data[1];
    char stringdata0[41];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_wallpapersetting__WrapperWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_wallpapersetting__WrapperWidget_t qt_meta_stringdata_ddplugin_wallpapersetting__WrapperWidget = {
    {
QT_MOC_LITERAL(0, 0, 40) // "ddplugin_wallpapersetting::Wr..."

    },
    "ddplugin_wallpapersetting::WrapperWidget"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_wallpapersetting__WrapperWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ddplugin_wallpapersetting::WrapperWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_wallpapersetting::WrapperWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_wallpapersetting__WrapperWidget.data,
    qt_meta_data_ddplugin_wallpapersetting__WrapperWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_wallpapersetting::WrapperWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_wallpapersetting::WrapperWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_wallpapersetting__WrapperWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ddplugin_wallpapersetting::WrapperWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperItem_t {
    QByteArrayData data[18];
    char stringdata0[197];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperItem_t qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperItem = {
    {
QT_MOC_LITERAL(0, 0, 40), // "ddplugin_wallpapersetting::Wa..."
QT_MOC_LITERAL(1, 41, 7), // "pressed"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 14), // "WallpaperItem*"
QT_MOC_LITERAL(4, 65, 4), // "self"
QT_MOC_LITERAL(5, 70, 7), // "hoverIn"
QT_MOC_LITERAL(6, 78, 8), // "hoverOut"
QT_MOC_LITERAL(7, 87, 3), // "tab"
QT_MOC_LITERAL(8, 91, 7), // "backtab"
QT_MOC_LITERAL(9, 99, 13), // "buttonClicked"
QT_MOC_LITERAL(10, 113, 2), // "id"
QT_MOC_LITERAL(11, 116, 15), // "onButtonClicked"
QT_MOC_LITERAL(12, 132, 18), // "onThumbnailFounded"
QT_MOC_LITERAL(13, 151, 3), // "key"
QT_MOC_LITERAL(14, 155, 6), // "pixmap"
QT_MOC_LITERAL(15, 162, 13), // "onFindAborted"
QT_MOC_LITERAL(16, 176, 15), // "QQueue<QString>"
QT_MOC_LITERAL(17, 192, 4) // "list"

    },
    "ddplugin_wallpapersetting::WallpaperItem\0"
    "pressed\0\0WallpaperItem*\0self\0hoverIn\0"
    "hoverOut\0tab\0backtab\0buttonClicked\0"
    "id\0onButtonClicked\0onThumbnailFounded\0"
    "key\0pixmap\0onFindAborted\0QQueue<QString>\0"
    "list"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_wallpapersetting__WallpaperItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,
       5,    1,   62,    2, 0x06 /* Public */,
       6,    1,   65,    2, 0x06 /* Public */,
       7,    1,   68,    2, 0x06 /* Public */,
       8,    1,   71,    2, 0x06 /* Public */,
       9,    2,   74,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    0,   79,    2, 0x08 /* Private */,
      12,    2,   80,    2, 0x08 /* Private */,
      15,    1,   85,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString,    4,   10,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QPixmap,   13,   14,
    QMetaType::Void, 0x80000000 | 16,   17,

       0        // eod
};

void ddplugin_wallpapersetting::WallpaperItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<WallpaperItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pressed((*reinterpret_cast< WallpaperItem*(*)>(_a[1]))); break;
        case 1: _t->hoverIn((*reinterpret_cast< WallpaperItem*(*)>(_a[1]))); break;
        case 2: _t->hoverOut((*reinterpret_cast< WallpaperItem*(*)>(_a[1]))); break;
        case 3: _t->tab((*reinterpret_cast< WallpaperItem*(*)>(_a[1]))); break;
        case 4: _t->backtab((*reinterpret_cast< WallpaperItem*(*)>(_a[1]))); break;
        case 5: _t->buttonClicked((*reinterpret_cast< WallpaperItem*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->onButtonClicked(); break;
        case 7: _t->onThumbnailFounded((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QPixmap(*)>(_a[2]))); break;
        case 8: _t->onFindAborted((*reinterpret_cast< const QQueue<QString>(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WallpaperItem* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WallpaperItem* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WallpaperItem* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WallpaperItem* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WallpaperItem* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WallpaperItem* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQueue<QString> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (WallpaperItem::*)(WallpaperItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WallpaperItem::pressed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (WallpaperItem::*)(WallpaperItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WallpaperItem::hoverIn)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (WallpaperItem::*)(WallpaperItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WallpaperItem::hoverOut)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (WallpaperItem::*)(WallpaperItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WallpaperItem::tab)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (WallpaperItem::*)(WallpaperItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WallpaperItem::backtab)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (WallpaperItem::*)(WallpaperItem * , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WallpaperItem::buttonClicked)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_wallpapersetting::WallpaperItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperItem.data,
    qt_meta_data_ddplugin_wallpapersetting__WallpaperItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_wallpapersetting::WallpaperItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_wallpapersetting::WallpaperItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperItem.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int ddplugin_wallpapersetting::WallpaperItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void ddplugin_wallpapersetting::WallpaperItem::pressed(WallpaperItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ddplugin_wallpapersetting::WallpaperItem::hoverIn(WallpaperItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ddplugin_wallpapersetting::WallpaperItem::hoverOut(WallpaperItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ddplugin_wallpapersetting::WallpaperItem::tab(WallpaperItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ddplugin_wallpapersetting::WallpaperItem::backtab(WallpaperItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void ddplugin_wallpapersetting::WallpaperItem::buttonClicked(WallpaperItem * _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
