/****************************************************************************
** Meta object code from reading C++ file 'emblemhelper.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-emblem/utils/emblemhelper.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'emblemhelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_emblem__GioEmblemWorker_t {
    QByteArrayData data[10];
    char stringdata0[108];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_emblem__GioEmblemWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_emblem__GioEmblemWorker_t qt_meta_stringdata_dfmplugin_emblem__GioEmblemWorker = {
    {
QT_MOC_LITERAL(0, 0, 33), // "dfmplugin_emblem::GioEmblemWo..."
QT_MOC_LITERAL(1, 34, 13), // "emblemChanged"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 3), // "url"
QT_MOC_LITERAL(4, 53, 7), // "Product"
QT_MOC_LITERAL(5, 61, 7), // "product"
QT_MOC_LITERAL(6, 69, 9), // "onProduce"
QT_MOC_LITERAL(7, 79, 15), // "FileInfoPointer"
QT_MOC_LITERAL(8, 95, 4), // "info"
QT_MOC_LITERAL(9, 100, 7) // "onClear"

    },
    "dfmplugin_emblem::GioEmblemWorker\0"
    "emblemChanged\0\0url\0Product\0product\0"
    "onProduce\0FileInfoPointer\0info\0onClear"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_emblem__GioEmblemWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   34,    2, 0x0a /* Public */,
       9,    0,   37,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 4,    3,    5,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_emblem::GioEmblemWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<GioEmblemWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->emblemChanged((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const Product(*)>(_a[2]))); break;
        case 1: _t->onProduce((*reinterpret_cast< const FileInfoPointer(*)>(_a[1]))); break;
        case 2: _t->onClear(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< FileInfoPointer >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (GioEmblemWorker::*)(const QUrl & , const Product & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GioEmblemWorker::emblemChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_emblem::GioEmblemWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_emblem__GioEmblemWorker.data,
    qt_meta_data_dfmplugin_emblem__GioEmblemWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_emblem::GioEmblemWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_emblem::GioEmblemWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_emblem__GioEmblemWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_emblem::GioEmblemWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_emblem::GioEmblemWorker::emblemChanged(const QUrl & _t1, const Product & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_dfmplugin_emblem__EmblemHelper_t {
    QByteArrayData data[12];
    char stringdata0[139];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_emblem__EmblemHelper_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_emblem__EmblemHelper_t qt_meta_stringdata_dfmplugin_emblem__EmblemHelper = {
    {
QT_MOC_LITERAL(0, 0, 30), // "dfmplugin_emblem::EmblemHelper"
QT_MOC_LITERAL(1, 31, 14), // "requestProduce"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 15), // "FileInfoPointer"
QT_MOC_LITERAL(4, 63, 4), // "info"
QT_MOC_LITERAL(5, 68, 12), // "requestClear"
QT_MOC_LITERAL(6, 81, 15), // "onEmblemChanged"
QT_MOC_LITERAL(7, 97, 3), // "url"
QT_MOC_LITERAL(8, 101, 7), // "Product"
QT_MOC_LITERAL(9, 109, 7), // "product"
QT_MOC_LITERAL(10, 117, 12), // "onUrlChanged"
QT_MOC_LITERAL(11, 130, 8) // "windowId"

    },
    "dfmplugin_emblem::EmblemHelper\0"
    "requestProduce\0\0FileInfoPointer\0info\0"
    "requestClear\0onEmblemChanged\0url\0"
    "Product\0product\0onUrlChanged\0windowId"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_emblem__EmblemHelper[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x06 /* Public */,
       5,    0,   37,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    2,   38,    2, 0x08 /* Private */,
      10,    2,   43,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 8,    7,    9,
    QMetaType::Bool, QMetaType::ULongLong, QMetaType::QUrl,   11,    7,

       0        // eod
};

void dfmplugin_emblem::EmblemHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EmblemHelper *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestProduce((*reinterpret_cast< const FileInfoPointer(*)>(_a[1]))); break;
        case 1: _t->requestClear(); break;
        case 2: _t->onEmblemChanged((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const Product(*)>(_a[2]))); break;
        case 3: { bool _r = _t->onUrlChanged((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< FileInfoPointer >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (EmblemHelper::*)(const FileInfoPointer & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EmblemHelper::requestProduce)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (EmblemHelper::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EmblemHelper::requestClear)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_emblem::EmblemHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_emblem__EmblemHelper.data,
    qt_meta_data_dfmplugin_emblem__EmblemHelper,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_emblem::EmblemHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_emblem::EmblemHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_emblem__EmblemHelper.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_emblem::EmblemHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_emblem::EmblemHelper::requestProduce(const FileInfoPointer & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_emblem::EmblemHelper::requestClear()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
