/****************************************************************************
** Meta object code from reading C++ file 'fileviewmodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-workspace/models/fileviewmodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileviewmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_workspace__FileViewModel_t {
    QByteArrayData data[65];
    char stringdata0[940];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_workspace__FileViewModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_workspace__FileViewModel_t qt_meta_stringdata_dfmplugin_workspace__FileViewModel = {
    {
QT_MOC_LITERAL(0, 0, 34), // "dfmplugin_workspace::FileView..."
QT_MOC_LITERAL(1, 35, 12), // "stateChanged"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 24), // "renameFileProcessStarted"
QT_MOC_LITERAL(4, 74, 17), // "selectAndEditFile"
QT_MOC_LITERAL(5, 92, 3), // "url"
QT_MOC_LITERAL(6, 96, 15), // "traverPrehandle"
QT_MOC_LITERAL(7, 112, 11), // "QModelIndex"
QT_MOC_LITERAL(8, 124, 5), // "index"
QT_MOC_LITERAL(9, 130, 9), // "FileView*"
QT_MOC_LITERAL(10, 140, 4), // "view"
QT_MOC_LITERAL(11, 145, 17), // "hiddenFileChanged"
QT_MOC_LITERAL(12, 163, 14), // "filtersChanged"
QT_MOC_LITERAL(13, 178, 11), // "nameFilters"
QT_MOC_LITERAL(14, 190, 13), // "QDir::Filters"
QT_MOC_LITERAL(15, 204, 7), // "filters"
QT_MOC_LITERAL(16, 212, 20), // "requestGetSourceData"
QT_MOC_LITERAL(17, 233, 25), // "requestChangeHiddenFilter"
QT_MOC_LITERAL(18, 259, 20), // "requestChangeFilters"
QT_MOC_LITERAL(19, 280, 24), // "requestChangeNameFilters"
QT_MOC_LITERAL(20, 305, 17), // "requestUpdateFile"
QT_MOC_LITERAL(21, 323, 25), // "requestRefreshAllChildren"
QT_MOC_LITERAL(22, 349, 21), // "requestClearThumbnail"
QT_MOC_LITERAL(23, 371, 19), // "requestSortChildren"
QT_MOC_LITERAL(24, 391, 13), // "Qt::SortOrder"
QT_MOC_LITERAL(25, 405, 5), // "order"
QT_MOC_LITERAL(26, 411, 26), // "dfmbase::Global::ItemRoles"
QT_MOC_LITERAL(27, 438, 4), // "role"
QT_MOC_LITERAL(28, 443, 12), // "isMixAndFile"
QT_MOC_LITERAL(29, 456, 20), // "requestSetFilterData"
QT_MOC_LITERAL(30, 477, 4), // "data"
QT_MOC_LITERAL(31, 482, 24), // "requestSetFilterCallback"
QT_MOC_LITERAL(32, 507, 22), // "FileViewFilterCallback"
QT_MOC_LITERAL(33, 530, 8), // "callback"
QT_MOC_LITERAL(34, 539, 24), // "requestShowHiddenChanged"
QT_MOC_LITERAL(35, 564, 5), // "value"
QT_MOC_LITERAL(36, 570, 19), // "requestCollapseItem"
QT_MOC_LITERAL(37, 590, 3), // "key"
QT_MOC_LITERAL(38, 594, 6), // "parent"
QT_MOC_LITERAL(39, 601, 15), // "requestTreeView"
QT_MOC_LITERAL(40, 617, 6), // "isTree"
QT_MOC_LITERAL(41, 624, 18), // "onFileThumbUpdated"
QT_MOC_LITERAL(42, 643, 5), // "thumb"
QT_MOC_LITERAL(43, 649, 13), // "onFileUpdated"
QT_MOC_LITERAL(44, 663, 4), // "show"
QT_MOC_LITERAL(45, 668, 8), // "onInsert"
QT_MOC_LITERAL(46, 677, 10), // "firstIndex"
QT_MOC_LITERAL(47, 688, 5), // "count"
QT_MOC_LITERAL(48, 694, 14), // "onInsertFinish"
QT_MOC_LITERAL(49, 709, 8), // "onRemove"
QT_MOC_LITERAL(50, 718, 14), // "onRemoveFinish"
QT_MOC_LITERAL(51, 733, 12), // "onUpdateView"
QT_MOC_LITERAL(52, 746, 25), // "onGenericAttributeChanged"
QT_MOC_LITERAL(53, 772, 38), // "dfmbase::Application::Generic..."
QT_MOC_LITERAL(54, 811, 2), // "ga"
QT_MOC_LITERAL(55, 814, 16), // "onDConfigChanged"
QT_MOC_LITERAL(56, 831, 6), // "config"
QT_MOC_LITERAL(57, 838, 15), // "onSetCursorWait"
QT_MOC_LITERAL(58, 854, 22), // "onHiddenSettingChanged"
QT_MOC_LITERAL(59, 877, 12), // "onWorkFinish"
QT_MOC_LITERAL(60, 890, 13), // "visiableCount"
QT_MOC_LITERAL(61, 904, 10), // "totalCount"
QT_MOC_LITERAL(62, 915, 13), // "onDataChanged"
QT_MOC_LITERAL(63, 929, 5), // "first"
QT_MOC_LITERAL(64, 935, 4) // "last"

    },
    "dfmplugin_workspace::FileViewModel\0"
    "stateChanged\0\0renameFileProcessStarted\0"
    "selectAndEditFile\0url\0traverPrehandle\0"
    "QModelIndex\0index\0FileView*\0view\0"
    "hiddenFileChanged\0filtersChanged\0"
    "nameFilters\0QDir::Filters\0filters\0"
    "requestGetSourceData\0requestChangeHiddenFilter\0"
    "requestChangeFilters\0requestChangeNameFilters\0"
    "requestUpdateFile\0requestRefreshAllChildren\0"
    "requestClearThumbnail\0requestSortChildren\0"
    "Qt::SortOrder\0order\0dfmbase::Global::ItemRoles\0"
    "role\0isMixAndFile\0requestSetFilterData\0"
    "data\0requestSetFilterCallback\0"
    "FileViewFilterCallback\0callback\0"
    "requestShowHiddenChanged\0value\0"
    "requestCollapseItem\0key\0parent\0"
    "requestTreeView\0isTree\0onFileThumbUpdated\0"
    "thumb\0onFileUpdated\0show\0onInsert\0"
    "firstIndex\0count\0onInsertFinish\0"
    "onRemove\0onRemoveFinish\0onUpdateView\0"
    "onGenericAttributeChanged\0"
    "dfmbase::Application::GenericAttribute\0"
    "ga\0onDConfigChanged\0config\0onSetCursorWait\0"
    "onHiddenSettingChanged\0onWorkFinish\0"
    "visiableCount\0totalCount\0onDataChanged\0"
    "first\0last"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_workspace__FileViewModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      32,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      19,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  174,    2, 0x06 /* Public */,
       3,    0,  175,    2, 0x06 /* Public */,
       4,    1,  176,    2, 0x06 /* Public */,
       6,    3,  179,    2, 0x06 /* Public */,
      11,    0,  186,    2, 0x06 /* Public */,
      12,    2,  187,    2, 0x06 /* Public */,
      16,    0,  192,    2, 0x06 /* Public */,
      17,    0,  193,    2, 0x06 /* Public */,
      18,    1,  194,    2, 0x06 /* Public */,
      19,    1,  197,    2, 0x06 /* Public */,
      20,    1,  200,    2, 0x06 /* Public */,
      21,    0,  203,    2, 0x06 /* Public */,
      22,    0,  204,    2, 0x06 /* Public */,
      23,    3,  205,    2, 0x06 /* Public */,
      29,    1,  212,    2, 0x06 /* Public */,
      31,    1,  215,    2, 0x06 /* Public */,
      34,    1,  218,    2, 0x06 /* Public */,
      36,    2,  221,    2, 0x06 /* Public */,
      39,    1,  226,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      41,    2,  229,    2, 0x0a /* Public */,
      43,    1,  234,    2, 0x0a /* Public */,
      45,    2,  237,    2, 0x0a /* Public */,
      48,    0,  242,    2, 0x0a /* Public */,
      49,    2,  243,    2, 0x0a /* Public */,
      50,    0,  248,    2, 0x0a /* Public */,
      51,    0,  249,    2, 0x0a /* Public */,
      52,    2,  250,    2, 0x0a /* Public */,
      55,    2,  255,    2, 0x0a /* Public */,
      57,    0,  260,    2, 0x0a /* Public */,
      58,    1,  261,    2, 0x0a /* Public */,
      59,    2,  264,    2, 0x0a /* Public */,
      62,    2,  269,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,    5,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 7, 0x80000000 | 9,    5,    8,   10,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QStringList, 0x80000000 | 14,   13,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, QMetaType::QStringList,   13,
    QMetaType::Void, QMetaType::QUrl,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 24, 0x80000000 | 26, QMetaType::Bool,   25,   27,   28,
    QMetaType::Void, QMetaType::QVariant,   30,
    QMetaType::Void, 0x80000000 | 32,   33,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::QString, QMetaType::QUrl,   37,   38,
    QMetaType::Void, QMetaType::Bool,   40,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl, QMetaType::QString,    5,   42,
    QMetaType::Void, QMetaType::Int,   44,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   46,   47,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   46,   47,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 53, QMetaType::QVariant,   54,   35,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   56,   37,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   60,   61,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   63,   64,

       0        // eod
};

void dfmplugin_workspace::FileViewModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileViewModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->stateChanged(); break;
        case 1: _t->renameFileProcessStarted(); break;
        case 2: _t->selectAndEditFile((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 3: _t->traverPrehandle((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2])),(*reinterpret_cast< FileView*(*)>(_a[3]))); break;
        case 4: _t->hiddenFileChanged(); break;
        case 5: _t->filtersChanged((*reinterpret_cast< QStringList(*)>(_a[1])),(*reinterpret_cast< QDir::Filters(*)>(_a[2]))); break;
        case 6: _t->requestGetSourceData(); break;
        case 7: _t->requestChangeHiddenFilter(); break;
        case 8: _t->requestChangeFilters((*reinterpret_cast< QDir::Filters(*)>(_a[1]))); break;
        case 9: _t->requestChangeNameFilters((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 10: _t->requestUpdateFile((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 11: _t->requestRefreshAllChildren(); break;
        case 12: _t->requestClearThumbnail(); break;
        case 13: _t->requestSortChildren((*reinterpret_cast< Qt::SortOrder(*)>(_a[1])),(*reinterpret_cast< dfmbase::Global::ItemRoles(*)>(_a[2])),(*reinterpret_cast< const bool(*)>(_a[3]))); break;
        case 14: _t->requestSetFilterData((*reinterpret_cast< const QVariant(*)>(_a[1]))); break;
        case 15: _t->requestSetFilterCallback((*reinterpret_cast< FileViewFilterCallback(*)>(_a[1]))); break;
        case 16: _t->requestShowHiddenChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->requestCollapseItem((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 18: _t->requestTreeView((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 19: _t->onFileThumbUpdated((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 20: _t->onFileUpdated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 21: _t->onInsert((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 22: _t->onInsertFinish(); break;
        case 23: _t->onRemove((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 24: _t->onRemoveFinish(); break;
        case 25: _t->onUpdateView(); break;
        case 26: _t->onGenericAttributeChanged((*reinterpret_cast< dfmbase::Application::GenericAttribute(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 27: _t->onDConfigChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 28: _t->onSetCursorWait(); break;
        case 29: _t->onHiddenSettingChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 30: _t->onWorkFinish((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 31: _t->onDataChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDir::Filters >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDir::Filters >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmbase::Global::ItemRoles >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileViewModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::stateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::renameFileProcessStarted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::selectAndEditFile)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)(const QUrl & , const QModelIndex & , FileView * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::traverPrehandle)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::hiddenFileChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)(QStringList , QDir::Filters );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::filtersChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestGetSourceData)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestChangeHiddenFilter)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)(QDir::Filters );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestChangeFilters)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestChangeNameFilters)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestUpdateFile)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestRefreshAllChildren)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestClearThumbnail)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)(Qt::SortOrder , dfmbase::Global::ItemRoles , const bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestSortChildren)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)(const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestSetFilterData)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)(FileViewFilterCallback );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestSetFilterCallback)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestShowHiddenChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)(const QString & , const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestCollapseItem)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (FileViewModel::*)(const bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileViewModel::requestTreeView)) {
                *result = 18;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_workspace::FileViewModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_workspace__FileViewModel.data,
    qt_meta_data_dfmplugin_workspace__FileViewModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_workspace::FileViewModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_workspace::FileViewModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_workspace__FileViewModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int dfmplugin_workspace::FileViewModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_workspace::FileViewModel::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dfmplugin_workspace::FileViewModel::renameFileProcessStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dfmplugin_workspace::FileViewModel::selectAndEditFile(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_workspace::FileViewModel::traverPrehandle(const QUrl & _t1, const QModelIndex & _t2, FileView * _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_workspace::FileViewModel::hiddenFileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void dfmplugin_workspace::FileViewModel::filtersChanged(QStringList _t1, QDir::Filters _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void dfmplugin_workspace::FileViewModel::requestGetSourceData()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void dfmplugin_workspace::FileViewModel::requestChangeHiddenFilter()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void dfmplugin_workspace::FileViewModel::requestChangeFilters(QDir::Filters _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void dfmplugin_workspace::FileViewModel::requestChangeNameFilters(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void dfmplugin_workspace::FileViewModel::requestUpdateFile(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void dfmplugin_workspace::FileViewModel::requestRefreshAllChildren()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void dfmplugin_workspace::FileViewModel::requestClearThumbnail()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void dfmplugin_workspace::FileViewModel::requestSortChildren(Qt::SortOrder _t1, dfmbase::Global::ItemRoles _t2, const bool _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void dfmplugin_workspace::FileViewModel::requestSetFilterData(const QVariant & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void dfmplugin_workspace::FileViewModel::requestSetFilterCallback(FileViewFilterCallback _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void dfmplugin_workspace::FileViewModel::requestShowHiddenChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void dfmplugin_workspace::FileViewModel::requestCollapseItem(const QString & _t1, const QUrl & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void dfmplugin_workspace::FileViewModel::requestTreeView(const bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
