/****************************************************************************
** Meta object code from reading C++ file 'global_server_defines.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../dde-file-manager/src/dfm-base/dbusservice/global_server_defines.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'global_server_defines.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19GlobalServerDefinesE_t {};
} // unnamed namespace

template <> constexpr inline auto GlobalServerDefines::qt_create_metaobjectdata<qt_meta_tag_ZN19GlobalServerDefinesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GlobalServerDefines",
        "DeviceQueryOption",
        "kNoCondition",
        "kMountable",
        "kMounted",
        "kRemovable",
        "kNotIgnored",
        "kNotMounted",
        "kOptical",
        "kSystem",
        "kLoop"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DeviceQueryOption'
        QtMocHelpers::EnumData<DeviceQueryOption>(1, 1, QMC::EnumFlags{}).add({
            {    2, DeviceQueryOption::kNoCondition },
            {    3, DeviceQueryOption::kMountable },
            {    4, DeviceQueryOption::kMounted },
            {    5, DeviceQueryOption::kRemovable },
            {    6, DeviceQueryOption::kNotIgnored },
            {    7, DeviceQueryOption::kNotMounted },
            {    8, DeviceQueryOption::kOptical },
            {    9, DeviceQueryOption::kSystem },
            {   10, DeviceQueryOption::kLoop },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN19GlobalServerDefinesE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN19GlobalServerDefinesE =
    GlobalServerDefines::qt_create_metaobjectdata<qt_meta_tag_ZN19GlobalServerDefinesE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN19GlobalServerDefinesE =
    qt_staticMetaObjectContent_ZN19GlobalServerDefinesE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN19GlobalServerDefinesE =
    qt_staticMetaObjectContent_ZN19GlobalServerDefinesE.relocatingData;

Q_CONSTINIT const QMetaObject GlobalServerDefines::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN19GlobalServerDefinesE.stringdata,
    qt_staticMetaObjectStaticContent_ZN19GlobalServerDefinesE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN19GlobalServerDefinesE.metaTypes,
    nullptr
} };

QT_WARNING_POP
