/****************************************************************************
** Meta object code from reading C++ file 'filemanager1dbus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/daemon/daemonplugin-filemanager1/filemanager1dbus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filemanager1dbus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FileManager1DBus_t {
    QByteArrayData data[12];
    char stringdata0[135];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FileManager1DBus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FileManager1DBus_t qt_meta_stringdata_FileManager1DBus = {
    {
QT_MOC_LITERAL(0, 0, 16), // "FileManager1DBus"
QT_MOC_LITERAL(1, 17, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 33, 28), // "org.freedesktop.FileManager1"
QT_MOC_LITERAL(3, 62, 11), // "ShowFolders"
QT_MOC_LITERAL(4, 74, 0), // ""
QT_MOC_LITERAL(5, 75, 4), // "URIs"
QT_MOC_LITERAL(6, 80, 9), // "StartupId"
QT_MOC_LITERAL(7, 90, 18), // "ShowItemProperties"
QT_MOC_LITERAL(8, 109, 9), // "ShowItems"
QT_MOC_LITERAL(9, 119, 5), // "Trash"
QT_MOC_LITERAL(10, 125, 4), // "Open"
QT_MOC_LITERAL(11, 130, 4) // "Args"

    },
    "FileManager1DBus\0D-Bus Interface\0"
    "org.freedesktop.FileManager1\0ShowFolders\0"
    "\0URIs\0StartupId\0ShowItemProperties\0"
    "ShowItems\0Trash\0Open\0Args"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FileManager1DBus[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       5,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // slots: name, argc, parameters, tag, flags
       3,    2,   41,    4, 0x0a /* Public */,
       7,    2,   46,    4, 0x0a /* Public */,
       8,    2,   51,    4, 0x0a /* Public */,
       9,    1,   56,    4, 0x0a /* Public */,
      10,    1,   59,    4, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::QStringList, QMetaType::QString,    5,    6,
    QMetaType::Void, QMetaType::QStringList, QMetaType::QString,    5,    6,
    QMetaType::Void, QMetaType::QStringList, QMetaType::QString,    5,    6,
    QMetaType::Void, QMetaType::QStringList,    5,
    QMetaType::Void, QMetaType::QStringList,   11,

       0        // eod
};

void FileManager1DBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileManager1DBus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->ShowFolders((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->ShowItemProperties((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 2: _t->ShowItems((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->Trash((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 4: _t->Open((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FileManager1DBus::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_FileManager1DBus.data,
    qt_meta_data_FileManager1DBus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FileManager1DBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileManager1DBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FileManager1DBus.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int FileManager1DBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
