/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a tagmanageradaptor -i tagmanagerdbus.h -l TagManagerDBus /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.Daemon.TagManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "tagmanageradaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class TagManagerAdaptor
 */

TagManagerAdaptor::TagManagerAdaptor(TagManagerDBus *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

TagManagerAdaptor::~TagManagerAdaptor()
{
    // destructor
}

bool TagManagerAdaptor::Delete(int opt, const QVariantMap &value)
{
    // handle method call org.deepin.Filemanager.Daemon.TagManager.Delete
    return parent()->Delete(opt, value);
}

bool TagManagerAdaptor::Insert(int opt, const QVariantMap &value)
{
    // handle method call org.deepin.Filemanager.Daemon.TagManager.Insert
    return parent()->Insert(opt, value);
}

QDBusVariant TagManagerAdaptor::Query(int opt)
{
    // handle method call org.deepin.Filemanager.Daemon.TagManager.Query
    return parent()->Query(opt);
}

QDBusVariant TagManagerAdaptor::Query(int opt, const QStringList &value)
{
    // handle method call org.deepin.Filemanager.Daemon.TagManager.Query
    return parent()->Query(opt, value);
}

bool TagManagerAdaptor::Update(int opt, const QVariantMap &value)
{
    // handle method call org.deepin.Filemanager.Daemon.TagManager.Update
    return parent()->Update(opt, value);
}


#include "tagmanageradaptor.moc"
