/****************************************************************************
** Meta object code from reading C++ file 'renamebar.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-workspace/views/renamebar.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'renamebar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_workspace__RenameBar_t {
    QByteArrayData data[25];
    char stringdata0[447];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_workspace__RenameBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_workspace__RenameBar_t qt_meta_stringdata_dfmplugin_workspace__RenameBar = {
    {
QT_MOC_LITERAL(0, 0, 30), // "dfmplugin_workspace::RenameBar"
QT_MOC_LITERAL(1, 31, 22), // "requestReplaceOperator"
QT_MOC_LITERAL(2, 54, 0), // ""
QT_MOC_LITERAL(3, 55, 17), // "clickCancelButton"
QT_MOC_LITERAL(4, 73, 17), // "clickRenameButton"
QT_MOC_LITERAL(5, 91, 14), // "visibleChanged"
QT_MOC_LITERAL(6, 106, 5), // "value"
QT_MOC_LITERAL(7, 112, 10), // "setVisible"
QT_MOC_LITERAL(8, 123, 7), // "visible"
QT_MOC_LITERAL(9, 131, 16), // "onVisibleChanged"
QT_MOC_LITERAL(10, 148, 22), // "onRenamePatternChanged"
QT_MOC_LITERAL(11, 171, 5), // "index"
QT_MOC_LITERAL(12, 177, 32), // "onReplaceOperatorFileNameChanged"
QT_MOC_LITERAL(13, 210, 4), // "text"
QT_MOC_LITERAL(14, 215, 32), // "onReplaceOperatorDestNameChanged"
QT_MOC_LITERAL(15, 248, 11), // "textChanged"
QT_MOC_LITERAL(16, 260, 32), // "onAddOperatorAddedContentChanged"
QT_MOC_LITERAL(17, 293, 23), // "onAddTextPatternChanged"
QT_MOC_LITERAL(18, 317, 31), // "onCustomOperatorFileNameChanged"
QT_MOC_LITERAL(19, 349, 31), // "onCustomOperatorSNNumberChanged"
QT_MOC_LITERAL(20, 381, 15), // "eventDispatcher"
QT_MOC_LITERAL(21, 397, 13), // "hideRenameBar"
QT_MOC_LITERAL(22, 411, 18), // "onSelectUrlChanged"
QT_MOC_LITERAL(23, 430, 11), // "QList<QUrl>"
QT_MOC_LITERAL(24, 442, 4) // "urls"

    },
    "dfmplugin_workspace::RenameBar\0"
    "requestReplaceOperator\0\0clickCancelButton\0"
    "clickRenameButton\0visibleChanged\0value\0"
    "setVisible\0visible\0onVisibleChanged\0"
    "onRenamePatternChanged\0index\0"
    "onReplaceOperatorFileNameChanged\0text\0"
    "onReplaceOperatorDestNameChanged\0"
    "textChanged\0onAddOperatorAddedContentChanged\0"
    "onAddTextPatternChanged\0"
    "onCustomOperatorFileNameChanged\0"
    "onCustomOperatorSNNumberChanged\0"
    "eventDispatcher\0hideRenameBar\0"
    "onSelectUrlChanged\0QList<QUrl>\0urls"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_workspace__RenameBar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   94,    2, 0x06 /* Public */,
       3,    0,   95,    2, 0x06 /* Public */,
       4,    0,   96,    2, 0x06 /* Public */,
       5,    1,   97,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,  100,    2, 0x0a /* Public */,
       9,    1,  103,    2, 0x08 /* Private */,
      10,    1,  106,    2, 0x08 /* Private */,
      12,    1,  109,    2, 0x08 /* Private */,
      14,    1,  112,    2, 0x08 /* Private */,
      16,    1,  115,    2, 0x08 /* Private */,
      17,    1,  118,    2, 0x08 /* Private */,
      18,    0,  121,    2, 0x08 /* Private */,
      19,    0,  122,    2, 0x08 /* Private */,
      20,    0,  123,    2, 0x08 /* Private */,
      21,    0,  124,    2, 0x08 /* Private */,
      22,    1,  125,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    6,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::QString,   13,
    QMetaType::Void, QMetaType::QString,   15,
    QMetaType::Void, QMetaType::QString,   13,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 23,   24,

       0        // eod
};

void dfmplugin_workspace::RenameBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RenameBar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestReplaceOperator(); break;
        case 1: _t->clickCancelButton(); break;
        case 2: _t->clickRenameButton(); break;
        case 3: _t->visibleChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->setVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->onVisibleChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->onRenamePatternChanged((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 7: _t->onReplaceOperatorFileNameChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->onReplaceOperatorDestNameChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->onAddOperatorAddedContentChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->onAddTextPatternChanged((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 11: _t->onCustomOperatorFileNameChanged(); break;
        case 12: _t->onCustomOperatorSNNumberChanged(); break;
        case 13: _t->eventDispatcher(); break;
        case 14: _t->hideRenameBar(); break;
        case 15: _t->onSelectUrlChanged((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RenameBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RenameBar::requestReplaceOperator)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RenameBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RenameBar::clickCancelButton)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (RenameBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RenameBar::clickRenameButton)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (RenameBar::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RenameBar::visibleChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_workspace::RenameBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_workspace__RenameBar.data,
    qt_meta_data_dfmplugin_workspace__RenameBar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_workspace::RenameBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_workspace::RenameBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_workspace__RenameBar.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int dfmplugin_workspace::RenameBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_workspace::RenameBar::requestReplaceOperator()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dfmplugin_workspace::RenameBar::clickCancelButton()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dfmplugin_workspace::RenameBar::clickRenameButton()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void dfmplugin_workspace::RenameBar::visibleChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
