/****************************************************************************
** Meta object code from reading C++ file 'recentmanagerdbus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/daemon/daemon-recentdaemon/recentmanagerdbus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'recentmanagerdbus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_RecentManagerDBus_t {
    QByteArrayData data[23];
    char stringdata0[265];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_RecentManagerDBus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_RecentManagerDBus_t qt_meta_stringdata_RecentManagerDBus = {
    {
QT_MOC_LITERAL(0, 0, 17), // "RecentManagerDBus"
QT_MOC_LITERAL(1, 18, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 34, 43), // "org.deepin.Filemanager.Daemon..."
QT_MOC_LITERAL(3, 78, 14), // "ReloadFinished"
QT_MOC_LITERAL(4, 93, 0), // ""
QT_MOC_LITERAL(5, 94, 9), // "timestamp"
QT_MOC_LITERAL(6, 104, 13), // "PurgeFinished"
QT_MOC_LITERAL(7, 118, 9), // "ItemAdded"
QT_MOC_LITERAL(8, 128, 4), // "path"
QT_MOC_LITERAL(9, 133, 4), // "href"
QT_MOC_LITERAL(10, 138, 8), // "modified"
QT_MOC_LITERAL(11, 147, 12), // "ItemsRemoved"
QT_MOC_LITERAL(12, 160, 5), // "paths"
QT_MOC_LITERAL(13, 166, 11), // "ItemChanged"
QT_MOC_LITERAL(14, 178, 6), // "Reload"
QT_MOC_LITERAL(15, 185, 7), // "AddItem"
QT_MOC_LITERAL(16, 193, 4), // "item"
QT_MOC_LITERAL(17, 198, 11), // "RemoveItems"
QT_MOC_LITERAL(18, 210, 5), // "hrefs"
QT_MOC_LITERAL(19, 216, 10), // "PurgeItems"
QT_MOC_LITERAL(20, 227, 12), // "GetItemsPath"
QT_MOC_LITERAL(21, 240, 12), // "GetItemsInfo"
QT_MOC_LITERAL(22, 253, 11) // "GetItemInfo"

    },
    "RecentManagerDBus\0D-Bus Interface\0"
    "org.deepin.Filemanager.Daemon.RecentManager\0"
    "ReloadFinished\0\0timestamp\0PurgeFinished\0"
    "ItemAdded\0path\0href\0modified\0ItemsRemoved\0"
    "paths\0ItemChanged\0Reload\0AddItem\0item\0"
    "RemoveItems\0hrefs\0PurgeItems\0GetItemsPath\0"
    "GetItemsInfo\0GetItemInfo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_RecentManagerDBus[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      12,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   76,    4, 0x06 /* Public */,
       6,    0,   79,    4, 0x06 /* Public */,
       7,    3,   80,    4, 0x06 /* Public */,
      11,    1,   87,    4, 0x06 /* Public */,
      13,    2,   90,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    0,   95,    4, 0x0a /* Public */,
      15,    1,   96,    4, 0x0a /* Public */,
      17,    1,   99,    4, 0x0a /* Public */,
      19,    0,  102,    4, 0x0a /* Public */,
      20,    0,  103,    4, 0x0a /* Public */,
      21,    0,  104,    4, 0x0a /* Public */,
      22,    1,  105,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::LongLong,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::LongLong,    8,    9,   10,
    QMetaType::Void, QMetaType::QStringList,   12,
    QMetaType::Void, QMetaType::QString, QMetaType::LongLong,    8,   10,

 // slots: parameters
    QMetaType::LongLong,
    QMetaType::Void, QMetaType::QVariantMap,   16,
    QMetaType::Void, QMetaType::QStringList,   18,
    QMetaType::Void,
    QMetaType::QStringList,
    QMetaType::QVariantList,
    QMetaType::QVariantMap, QMetaType::QString,    8,

       0        // eod
};

void RecentManagerDBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RecentManagerDBus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->ReloadFinished((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 1: _t->PurgeFinished(); break;
        case 2: _t->ItemAdded((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< qint64(*)>(_a[3]))); break;
        case 3: _t->ItemsRemoved((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 4: _t->ItemChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2]))); break;
        case 5: { qint64 _r = _t->Reload();
            if (_a[0]) *reinterpret_cast< qint64*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->AddItem((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 7: _t->RemoveItems((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 8: _t->PurgeItems(); break;
        case 9: { QStringList _r = _t->GetItemsPath();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 10: { QVariantList _r = _t->GetItemsInfo();
            if (_a[0]) *reinterpret_cast< QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 11: { QVariantMap _r = _t->GetItemInfo((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QVariantMap*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RecentManagerDBus::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManagerDBus::ReloadFinished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RecentManagerDBus::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManagerDBus::PurgeFinished)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (RecentManagerDBus::*)(const QString & , const QString & , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManagerDBus::ItemAdded)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (RecentManagerDBus::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManagerDBus::ItemsRemoved)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (RecentManagerDBus::*)(const QString & , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManagerDBus::ItemChanged)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject RecentManagerDBus::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_RecentManagerDBus.data,
    qt_meta_data_RecentManagerDBus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *RecentManagerDBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RecentManagerDBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_RecentManagerDBus.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int RecentManagerDBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void RecentManagerDBus::ReloadFinished(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void RecentManagerDBus::PurgeFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void RecentManagerDBus::ItemAdded(const QString & _t1, const QString & _t2, qint64 _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void RecentManagerDBus::ItemsRemoved(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void RecentManagerDBus::ItemChanged(const QString & _t1, qint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
