/****************************************************************************
** Meta object code from reading C++ file 'eventshandler.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-disk-encrypt-entry/events/eventshandler.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'eventshandler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_diskenc__EventsHandler_t {
    QByteArrayData data[26];
    char stringdata0[399];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_diskenc__EventsHandler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_diskenc__EventsHandler_t qt_meta_stringdata_dfmplugin_diskenc__EventsHandler = {
    {
QT_MOC_LITERAL(0, 0, 32), // "dfmplugin_diskenc::EventsHandler"
QT_MOC_LITERAL(1, 33, 18), // "onPreencryptResult"
QT_MOC_LITERAL(2, 52, 0), // ""
QT_MOC_LITERAL(3, 53, 15), // "onEncryptResult"
QT_MOC_LITERAL(4, 69, 17), // "onEncryptProgress"
QT_MOC_LITERAL(5, 87, 15), // "onDecryptResult"
QT_MOC_LITERAL(6, 103, 17), // "onDecryptProgress"
QT_MOC_LITERAL(7, 121, 21), // "onChgPassphraseResult"
QT_MOC_LITERAL(8, 143, 22), // "onRequestEncryptParams"
QT_MOC_LITERAL(9, 166, 9), // "encConfig"
QT_MOC_LITERAL(10, 176, 18), // "ignoreParamRequest"
QT_MOC_LITERAL(11, 195, 17), // "acquirePassphrase"
QT_MOC_LITERAL(12, 213, 3), // "dev"
QT_MOC_LITERAL(13, 217, 5), // "bool&"
QT_MOC_LITERAL(14, 223, 9), // "cancelled"
QT_MOC_LITERAL(15, 233, 22), // "acquirePassphraseByPIN"
QT_MOC_LITERAL(16, 256, 22), // "acquirePassphraseByTPM"
QT_MOC_LITERAL(17, 279, 22), // "acquirePassphraseByRec"
QT_MOC_LITERAL(18, 302, 19), // "showPreEncryptError"
QT_MOC_LITERAL(19, 322, 6), // "device"
QT_MOC_LITERAL(20, 329, 7), // "devName"
QT_MOC_LITERAL(21, 337, 4), // "code"
QT_MOC_LITERAL(22, 342, 16), // "showDecryptError"
QT_MOC_LITERAL(23, 359, 15), // "showChgPwdError"
QT_MOC_LITERAL(24, 375, 13), // "requestReboot"
QT_MOC_LITERAL(25, 389, 9) // "canUnlock"

    },
    "dfmplugin_diskenc::EventsHandler\0"
    "onPreencryptResult\0\0onEncryptResult\0"
    "onEncryptProgress\0onDecryptResult\0"
    "onDecryptProgress\0onChgPassphraseResult\0"
    "onRequestEncryptParams\0encConfig\0"
    "ignoreParamRequest\0acquirePassphrase\0"
    "dev\0bool&\0cancelled\0acquirePassphraseByPIN\0"
    "acquirePassphraseByTPM\0acquirePassphraseByRec\0"
    "showPreEncryptError\0device\0devName\0"
    "code\0showDecryptError\0showChgPwdError\0"
    "requestReboot\0canUnlock"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_diskenc__EventsHandler[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    4,   99,    2, 0x08 /* Private */,
       3,    4,  108,    2, 0x08 /* Private */,
       4,    3,  117,    2, 0x08 /* Private */,
       5,    4,  124,    2, 0x08 /* Private */,
       6,    3,  133,    2, 0x08 /* Private */,
       7,    4,  140,    2, 0x08 /* Private */,
       8,    1,  149,    2, 0x08 /* Private */,
      10,    0,  152,    2, 0x08 /* Private */,
      11,    2,  153,    2, 0x08 /* Private */,
      15,    2,  158,    2, 0x08 /* Private */,
      16,    2,  163,    2, 0x08 /* Private */,
      17,    2,  168,    2, 0x08 /* Private */,
      18,    3,  173,    2, 0x08 /* Private */,
      22,    3,  180,    2, 0x08 /* Private */,
      23,    3,  187,    2, 0x08 /* Private */,
      24,    0,  194,    2, 0x08 /* Private */,
      25,    1,  195,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::Int,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int, QMetaType::QString,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Double,    2,    2,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::Int,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Double,    2,    2,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::Int,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::QVariantMap,    9,
    QMetaType::Void,
    QMetaType::QString, QMetaType::QString, 0x80000000 | 13,   12,   14,
    QMetaType::QString, QMetaType::QString, 0x80000000 | 13,   12,   14,
    QMetaType::QString, QMetaType::QString, 0x80000000 | 13,   12,   14,
    QMetaType::QString, QMetaType::QString, 0x80000000 | 13,   12,   14,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int,   19,   20,   21,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int,   19,   20,   21,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int,   19,   20,   21,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString,   19,

       0        // eod
};

void dfmplugin_diskenc::EventsHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EventsHandler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onPreencryptResult((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 1: _t->onEncryptResult((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 2: _t->onEncryptProgress((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 3: _t->onDecryptResult((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 4: _t->onDecryptProgress((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 5: _t->onChgPassphraseResult((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 6: _t->onRequestEncryptParams((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 7: _t->ignoreParamRequest(); break;
        case 8: { QString _r = _t->acquirePassphrase((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 9: { QString _r = _t->acquirePassphraseByPIN((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->acquirePassphraseByTPM((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 11: { QString _r = _t->acquirePassphraseByRec((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->showPreEncryptError((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 13: _t->showDecryptError((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 14: _t->showChgPwdError((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 15: _t->requestReboot(); break;
        case 16: { bool _r = _t->canUnlock((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_diskenc::EventsHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_diskenc__EventsHandler.data,
    qt_meta_data_dfmplugin_diskenc__EventsHandler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_diskenc::EventsHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_diskenc::EventsHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_diskenc__EventsHandler.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_diskenc::EventsHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
