/****************************************************************************
** Meta object code from reading C++ file 'fileprovider.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-canvas/model/fileprovider.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileprovider.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_canvas__FileProvider_t {
    QByteArrayData data[28];
    char stringdata0[290];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_canvas__FileProvider_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_canvas__FileProvider_t qt_meta_stringdata_ddplugin_canvas__FileProvider = {
    {
QT_MOC_LITERAL(0, 0, 29), // "ddplugin_canvas::FileProvider"
QT_MOC_LITERAL(1, 30, 10), // "refreshEnd"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 11), // "QList<QUrl>"
QT_MOC_LITERAL(4, 54, 4), // "urls"
QT_MOC_LITERAL(5, 59, 11), // "fileRemoved"
QT_MOC_LITERAL(6, 71, 3), // "url"
QT_MOC_LITERAL(7, 75, 12), // "fileInserted"
QT_MOC_LITERAL(8, 88, 11), // "fileRenamed"
QT_MOC_LITERAL(9, 100, 6), // "oldurl"
QT_MOC_LITERAL(10, 107, 6), // "newurl"
QT_MOC_LITERAL(11, 114, 11), // "fileUpdated"
QT_MOC_LITERAL(12, 126, 15), // "fileInfoUpdated"
QT_MOC_LITERAL(13, 142, 9), // "isLinkOrg"
QT_MOC_LITERAL(14, 152, 16), // "fileThumbUpdated"
QT_MOC_LITERAL(15, 169, 5), // "thumb"
QT_MOC_LITERAL(16, 175, 17), // "traversalFinished"
QT_MOC_LITERAL(17, 193, 5), // "reset"
QT_MOC_LITERAL(18, 199, 8), // "children"
QT_MOC_LITERAL(19, 208, 6), // "insert"
QT_MOC_LITERAL(20, 215, 6), // "remove"
QT_MOC_LITERAL(21, 222, 6), // "rename"
QT_MOC_LITERAL(22, 229, 6), // "oldUrl"
QT_MOC_LITERAL(23, 236, 6), // "newUrl"
QT_MOC_LITERAL(24, 243, 6), // "update"
QT_MOC_LITERAL(25, 250, 13), // "preupdateData"
QT_MOC_LITERAL(26, 264, 17), // "onFileInfoUpdated"
QT_MOC_LITERAL(27, 282, 7) // "infoPtr"

    },
    "ddplugin_canvas::FileProvider\0refreshEnd\0"
    "\0QList<QUrl>\0urls\0fileRemoved\0url\0"
    "fileInserted\0fileRenamed\0oldurl\0newurl\0"
    "fileUpdated\0fileInfoUpdated\0isLinkOrg\0"
    "fileThumbUpdated\0thumb\0traversalFinished\0"
    "reset\0children\0insert\0remove\0rename\0"
    "oldUrl\0newUrl\0update\0preupdateData\0"
    "onFileInfoUpdated\0infoPtr"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_canvas__FileProvider[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,
       5,    1,   92,    2, 0x06 /* Public */,
       7,    1,   95,    2, 0x06 /* Public */,
       8,    2,   98,    2, 0x06 /* Public */,
      11,    1,  103,    2, 0x06 /* Public */,
      12,    2,  106,    2, 0x06 /* Public */,
      14,    2,  111,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      16,    0,  116,    2, 0x09 /* Protected */,
      17,    1,  117,    2, 0x09 /* Protected */,
      19,    1,  120,    2, 0x09 /* Protected */,
      20,    1,  123,    2, 0x09 /* Protected */,
      21,    2,  126,    2, 0x09 /* Protected */,
      24,    1,  131,    2, 0x09 /* Protected */,
      25,    1,  134,    2, 0x09 /* Protected */,
      26,    3,  137,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QUrl,    6,
    QMetaType::Void, QMetaType::QUrl,    6,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,    9,   10,
    QMetaType::Void, QMetaType::QUrl,    6,
    QMetaType::Void, QMetaType::QUrl, QMetaType::Bool,    6,   13,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QString,    6,   15,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,   18,
    QMetaType::Void, QMetaType::QUrl,    6,
    QMetaType::Void, QMetaType::QUrl,    6,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,   22,   23,
    QMetaType::Void, QMetaType::QUrl,    6,
    QMetaType::Void, QMetaType::QUrl,    6,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QString, QMetaType::Bool,    6,   27,   13,

       0        // eod
};

void ddplugin_canvas::FileProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileProvider *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->refreshEnd((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 1: _t->fileRemoved((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 2: _t->fileInserted((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 3: _t->fileRenamed((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 4: _t->fileUpdated((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 5: _t->fileInfoUpdated((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        case 6: _t->fileThumbUpdated((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 7: _t->traversalFinished(); break;
        case 8: _t->reset((*reinterpret_cast< QList<QUrl>(*)>(_a[1]))); break;
        case 9: _t->insert((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 10: _t->remove((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 11: _t->rename((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 12: _t->update((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 13: _t->preupdateData((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 14: _t->onFileInfoUpdated((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const bool(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileProvider::*)(const QList<QUrl> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileProvider::refreshEnd)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileProvider::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileProvider::fileRemoved)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileProvider::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileProvider::fileInserted)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FileProvider::*)(const QUrl & , const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileProvider::fileRenamed)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FileProvider::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileProvider::fileUpdated)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (FileProvider::*)(const QUrl & , const bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileProvider::fileInfoUpdated)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (FileProvider::*)(const QUrl & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileProvider::fileThumbUpdated)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_canvas::FileProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_canvas__FileProvider.data,
    qt_meta_data_ddplugin_canvas__FileProvider,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_canvas::FileProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_canvas::FileProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_canvas__FileProvider.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_canvas::FileProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void ddplugin_canvas::FileProvider::refreshEnd(const QList<QUrl> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ddplugin_canvas::FileProvider::fileRemoved(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ddplugin_canvas::FileProvider::fileInserted(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ddplugin_canvas::FileProvider::fileRenamed(const QUrl & _t1, const QUrl & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ddplugin_canvas::FileProvider::fileUpdated(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void ddplugin_canvas::FileProvider::fileInfoUpdated(const QUrl & _t1, const bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void ddplugin_canvas::FileProvider::fileThumbUpdated(const QUrl & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
