/****************************************************************************
** Meta object code from reading C++ file 'recentmanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/daemon/daemon-recentdaemon/recentmanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'recentmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_serverplugin_recentmanager__RecentManager_t {
    QByteArrayData data[29];
    char stringdata0[343];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_serverplugin_recentmanager__RecentManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_serverplugin_recentmanager__RecentManager_t qt_meta_stringdata_serverplugin_recentmanager__RecentManager = {
    {
QT_MOC_LITERAL(0, 0, 41), // "serverplugin_recentmanager::R..."
QT_MOC_LITERAL(1, 42, 13), // "requestReload"
QT_MOC_LITERAL(2, 56, 0), // ""
QT_MOC_LITERAL(3, 57, 8), // "xbelPath"
QT_MOC_LITERAL(4, 66, 9), // "timestamp"
QT_MOC_LITERAL(5, 76, 20), // "requestAddRecentItem"
QT_MOC_LITERAL(6, 97, 5), // "items"
QT_MOC_LITERAL(7, 103, 18), // "requestRemoveItems"
QT_MOC_LITERAL(8, 122, 5), // "paths"
QT_MOC_LITERAL(9, 128, 17), // "requestPurgeItems"
QT_MOC_LITERAL(10, 146, 14), // "reloadFinished"
QT_MOC_LITERAL(11, 161, 13), // "purgeFinished"
QT_MOC_LITERAL(12, 175, 9), // "itemAdded"
QT_MOC_LITERAL(13, 185, 4), // "path"
QT_MOC_LITERAL(14, 190, 4), // "href"
QT_MOC_LITERAL(15, 195, 8), // "modified"
QT_MOC_LITERAL(16, 204, 12), // "itemsRemoved"
QT_MOC_LITERAL(17, 217, 11), // "itemChanged"
QT_MOC_LITERAL(18, 229, 10), // "initialize"
QT_MOC_LITERAL(19, 240, 8), // "finalize"
QT_MOC_LITERAL(20, 249, 10), // "startWatch"
QT_MOC_LITERAL(21, 260, 9), // "stopWatch"
QT_MOC_LITERAL(22, 270, 6), // "reload"
QT_MOC_LITERAL(23, 277, 8), // "doReload"
QT_MOC_LITERAL(24, 286, 11), // "onItemAdded"
QT_MOC_LITERAL(25, 298, 10), // "RecentItem"
QT_MOC_LITERAL(26, 309, 4), // "item"
QT_MOC_LITERAL(27, 314, 14), // "onItemsRemoved"
QT_MOC_LITERAL(28, 329, 13) // "onItemChanged"

    },
    "serverplugin_recentmanager::RecentManager\0"
    "requestReload\0\0xbelPath\0timestamp\0"
    "requestAddRecentItem\0items\0"
    "requestRemoveItems\0paths\0requestPurgeItems\0"
    "reloadFinished\0purgeFinished\0itemAdded\0"
    "path\0href\0modified\0itemsRemoved\0"
    "itemChanged\0initialize\0finalize\0"
    "startWatch\0stopWatch\0reload\0doReload\0"
    "onItemAdded\0RecentItem\0item\0onItemsRemoved\0"
    "onItemChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_serverplugin_recentmanager__RecentManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,  109,    2, 0x06 /* Public */,
       5,    1,  114,    2, 0x06 /* Public */,
       7,    1,  117,    2, 0x06 /* Public */,
       9,    1,  120,    2, 0x06 /* Public */,
      10,    1,  123,    2, 0x06 /* Public */,
      11,    0,  126,    2, 0x06 /* Public */,
      12,    3,  127,    2, 0x06 /* Public */,
      16,    1,  134,    2, 0x06 /* Public */,
      17,    2,  137,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    0,  142,    2, 0x0a /* Public */,
      19,    0,  143,    2, 0x0a /* Public */,
      20,    0,  144,    2, 0x0a /* Public */,
      21,    0,  145,    2, 0x0a /* Public */,
      22,    0,  146,    2, 0x08 /* Private */,
      23,    1,  147,    2, 0x08 /* Private */,
      23,    0,  150,    2, 0x28 /* Private | MethodCloned */,
      24,    2,  151,    2, 0x08 /* Private */,
      27,    1,  156,    2, 0x08 /* Private */,
      28,    2,  159,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::LongLong,    3,    4,
    QMetaType::Void, QMetaType::QVariantMap,    6,
    QMetaType::Void, QMetaType::QStringList,    8,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::LongLong,    4,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::LongLong,   13,   14,   15,
    QMetaType::Void, QMetaType::QStringList,    8,
    QMetaType::Void, QMetaType::QString, QMetaType::LongLong,   13,   15,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::LongLong,    4,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 25,   13,   26,
    QMetaType::Void, QMetaType::QStringList,    8,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 25,   13,   26,

       0        // eod
};

void serverplugin_recentmanager::RecentManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RecentManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestReload((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2]))); break;
        case 1: _t->requestAddRecentItem((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 2: _t->requestRemoveItems((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 3: _t->requestPurgeItems((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->reloadFinished((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 5: _t->purgeFinished(); break;
        case 6: _t->itemAdded((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< qint64(*)>(_a[3]))); break;
        case 7: _t->itemsRemoved((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 8: _t->itemChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2]))); break;
        case 9: _t->initialize(); break;
        case 10: _t->finalize(); break;
        case 11: _t->startWatch(); break;
        case 12: _t->stopWatch(); break;
        case 13: _t->reload(); break;
        case 14: _t->doReload((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 15: _t->doReload(); break;
        case 16: _t->onItemAdded((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const RecentItem(*)>(_a[2]))); break;
        case 17: _t->onItemsRemoved((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 18: _t->onItemChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const RecentItem(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RecentManager::*)(const QString & , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManager::requestReload)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RecentManager::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManager::requestAddRecentItem)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (RecentManager::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManager::requestRemoveItems)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (RecentManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManager::requestPurgeItems)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (RecentManager::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManager::reloadFinished)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (RecentManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManager::purgeFinished)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (RecentManager::*)(const QString & , const QString & , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManager::itemAdded)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (RecentManager::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManager::itemsRemoved)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (RecentManager::*)(const QString & , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RecentManager::itemChanged)) {
                *result = 8;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject serverplugin_recentmanager::RecentManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_serverplugin_recentmanager__RecentManager.data,
    qt_meta_data_serverplugin_recentmanager__RecentManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *serverplugin_recentmanager::RecentManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *serverplugin_recentmanager::RecentManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_serverplugin_recentmanager__RecentManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int serverplugin_recentmanager::RecentManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void serverplugin_recentmanager::RecentManager::requestReload(const QString & _t1, qint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void serverplugin_recentmanager::RecentManager::requestAddRecentItem(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void serverplugin_recentmanager::RecentManager::requestRemoveItems(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void serverplugin_recentmanager::RecentManager::requestPurgeItems(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void serverplugin_recentmanager::RecentManager::reloadFinished(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void serverplugin_recentmanager::RecentManager::purgeFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void serverplugin_recentmanager::RecentManager::itemAdded(const QString & _t1, const QString & _t2, qint64 _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void serverplugin_recentmanager::RecentManager::itemsRemoved(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void serverplugin_recentmanager::RecentManager::itemChanged(const QString & _t1, qint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
