/****************************************************************************
** Meta object code from reading C++ file 'crumbinterface.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-titlebar/utils/crumbinterface.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'crumbinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_titlebar__CrumbInterface_t {
    QByteArrayData data[13];
    char stringdata0[208];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_titlebar__CrumbInterface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_titlebar__CrumbInterface_t qt_meta_stringdata_dfmplugin_titlebar__CrumbInterface = {
    {
QT_MOC_LITERAL(0, 0, 34), // "dfmplugin_titlebar::CrumbInte..."
QT_MOC_LITERAL(1, 35, 14), // "hideAddressBar"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 11), // "pauseSearch"
QT_MOC_LITERAL(4, 63, 14), // "keepAddressBar"
QT_MOC_LITERAL(5, 78, 3), // "url"
QT_MOC_LITERAL(6, 82, 23), // "hideAddrAndUpdateCrumbs"
QT_MOC_LITERAL(7, 106, 15), // "completionFound"
QT_MOC_LITERAL(8, 122, 11), // "completions"
QT_MOC_LITERAL(9, 134, 35), // "completionListTransmissionCom..."
QT_MOC_LITERAL(10, 170, 16), // "onUpdateChildren"
QT_MOC_LITERAL(11, 187, 11), // "QList<QUrl>"
QT_MOC_LITERAL(12, 199, 8) // "children"

    },
    "dfmplugin_titlebar::CrumbInterface\0"
    "hideAddressBar\0\0pauseSearch\0keepAddressBar\0"
    "url\0hideAddrAndUpdateCrumbs\0completionFound\0"
    "completions\0completionListTransmissionCompleted\0"
    "onUpdateChildren\0QList<QUrl>\0children"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_titlebar__CrumbInterface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,
       4,    1,   51,    2, 0x06 /* Public */,
       6,    1,   54,    2, 0x06 /* Public */,
       7,    1,   57,    2, 0x06 /* Public */,
       9,    0,   60,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,   61,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,    5,
    QMetaType::Void, QMetaType::QUrl,    5,
    QMetaType::Void, QMetaType::QStringList,    8,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 11,   12,

       0        // eod
};

void dfmplugin_titlebar::CrumbInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CrumbInterface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->hideAddressBar(); break;
        case 1: _t->pauseSearch(); break;
        case 2: _t->keepAddressBar((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 3: _t->hideAddrAndUpdateCrumbs((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 4: _t->completionFound((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 5: _t->completionListTransmissionCompleted(); break;
        case 6: _t->onUpdateChildren((*reinterpret_cast< QList<QUrl>(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CrumbInterface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CrumbInterface::hideAddressBar)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CrumbInterface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CrumbInterface::pauseSearch)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CrumbInterface::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CrumbInterface::keepAddressBar)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CrumbInterface::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CrumbInterface::hideAddrAndUpdateCrumbs)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (CrumbInterface::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CrumbInterface::completionFound)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (CrumbInterface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CrumbInterface::completionListTransmissionCompleted)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_titlebar::CrumbInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_titlebar__CrumbInterface.data,
    qt_meta_data_dfmplugin_titlebar__CrumbInterface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_titlebar::CrumbInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_titlebar::CrumbInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_titlebar__CrumbInterface.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_titlebar::CrumbInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_titlebar::CrumbInterface::hideAddressBar()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dfmplugin_titlebar::CrumbInterface::pauseSearch()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dfmplugin_titlebar::CrumbInterface::keepAddressBar(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_titlebar::CrumbInterface::hideAddrAndUpdateCrumbs(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_titlebar::CrumbInterface::completionFound(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmplugin_titlebar::CrumbInterface::completionListTransmissionCompleted()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
