/****************************************************************************
** Meta object code from reading C++ file 'propertydialogutil.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-propertydialog/utils/propertydialogutil.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertydialogutil.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_propertydialog__PropertyDialogUtil_t {
    QByteArrayData data[20];
    char stringdata0[359];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_propertydialog__PropertyDialogUtil_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_propertydialog__PropertyDialogUtil_t qt_meta_stringdata_dfmplugin_propertydialog__PropertyDialogUtil = {
    {
QT_MOC_LITERAL(0, 0, 44), // "dfmplugin_propertydialog::Pro..."
QT_MOC_LITERAL(1, 45, 18), // "showPropertyDialog"
QT_MOC_LITERAL(2, 64, 0), // ""
QT_MOC_LITERAL(3, 65, 11), // "QList<QUrl>"
QT_MOC_LITERAL(4, 77, 4), // "urls"
QT_MOC_LITERAL(5, 82, 6), // "option"
QT_MOC_LITERAL(6, 89, 22), // "showFilePropertyDialog"
QT_MOC_LITERAL(7, 112, 16), // "showCustomDialog"
QT_MOC_LITERAL(8, 129, 3), // "url"
QT_MOC_LITERAL(9, 133, 33), // "insertExtendedControlFileProp..."
QT_MOC_LITERAL(10, 167, 5), // "index"
QT_MOC_LITERAL(11, 173, 8), // "QWidget*"
QT_MOC_LITERAL(12, 182, 6), // "widget"
QT_MOC_LITERAL(13, 189, 30), // "addExtendedControlFileProperty"
QT_MOC_LITERAL(14, 220, 23), // "closeFilePropertyDialog"
QT_MOC_LITERAL(15, 244, 25), // "closeCustomPropertyDialog"
QT_MOC_LITERAL(16, 270, 26), // "closeAllFilePropertyDialog"
QT_MOC_LITERAL(17, 297, 22), // "closeAllPropertyDialog"
QT_MOC_LITERAL(18, 320, 17), // "createControlView"
QT_MOC_LITERAL(19, 338, 20) // "updateCloseIndicator"

    },
    "dfmplugin_propertydialog::PropertyDialogUtil\0"
    "showPropertyDialog\0\0QList<QUrl>\0urls\0"
    "option\0showFilePropertyDialog\0"
    "showCustomDialog\0url\0"
    "insertExtendedControlFileProperty\0"
    "index\0QWidget*\0widget\0"
    "addExtendedControlFileProperty\0"
    "closeFilePropertyDialog\0"
    "closeCustomPropertyDialog\0"
    "closeAllFilePropertyDialog\0"
    "closeAllPropertyDialog\0createControlView\0"
    "updateCloseIndicator"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_propertydialog__PropertyDialogUtil[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   84,    2, 0x0a /* Public */,
       1,    1,   89,    2, 0x2a /* Public | MethodCloned */,
       6,    2,   92,    2, 0x0a /* Public */,
       6,    1,   97,    2, 0x2a /* Public | MethodCloned */,
       7,    1,  100,    2, 0x0a /* Public */,
       9,    3,  103,    2, 0x0a /* Public */,
      13,    2,  110,    2, 0x0a /* Public */,
      14,    1,  115,    2, 0x0a /* Public */,
      15,    1,  118,    2, 0x0a /* Public */,
      16,    0,  121,    2, 0x0a /* Public */,
      17,    0,  122,    2, 0x0a /* Public */,
      18,    2,  123,    2, 0x0a /* Public */,
      18,    1,  128,    2, 0x2a /* Public | MethodCloned */,
      19,    0,  131,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QVariantHash,    4,    5,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QVariantHash,    4,    5,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Bool, QMetaType::QUrl,    8,
    QMetaType::Void, QMetaType::QUrl, QMetaType::Int, 0x80000000 | 11,    8,   10,   12,
    QMetaType::Void, QMetaType::QUrl, 0x80000000 | 11,    8,   12,
    QMetaType::Void, QMetaType::QUrl,    8,
    QMetaType::Void, QMetaType::QUrl,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QVariantHash,    8,    5,
    QMetaType::Void, QMetaType::QUrl,    8,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_propertydialog::PropertyDialogUtil::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyDialogUtil *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->showPropertyDialog((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QVariantHash(*)>(_a[2]))); break;
        case 1: _t->showPropertyDialog((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 2: _t->showFilePropertyDialog((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QVariantHash(*)>(_a[2]))); break;
        case 3: _t->showFilePropertyDialog((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 4: { bool _r = _t->showCustomDialog((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->insertExtendedControlFileProperty((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< QWidget*(*)>(_a[3]))); break;
        case 6: _t->addExtendedControlFileProperty((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< QWidget*(*)>(_a[2]))); break;
        case 7: _t->closeFilePropertyDialog((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 8: _t->closeCustomPropertyDialog((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 9: _t->closeAllFilePropertyDialog(); break;
        case 10: _t->closeAllPropertyDialog(); break;
        case 11: _t->createControlView((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QVariantHash(*)>(_a[2]))); break;
        case 12: _t->createControlView((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 13: _t->updateCloseIndicator(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_propertydialog::PropertyDialogUtil::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_propertydialog__PropertyDialogUtil.data,
    qt_meta_data_dfmplugin_propertydialog__PropertyDialogUtil,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_propertydialog::PropertyDialogUtil::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_propertydialog::PropertyDialogUtil::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_propertydialog__PropertyDialogUtil.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_propertydialog::PropertyDialogUtil::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
