// SPDX-FileCopyrightText: 2022 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef DFMPLUGIN_FILEOPERATIONS_GLOBAL_H
#define DFMPLUGIN_FILEOPERATIONS_GLOBAL_H

#include <dfm-base/dfm_log_defines.h>

#define DPFILEOPERATIONS_NAMESPACE dfmplugin_fileoperations

#define DPFILEOPERATIONS_BEGIN_NAMESPACE namespace DPFILEOPERATIONS_NAMESPACE {
#define DPFILEOPERATIONS_END_NAMESPACE }
#define DPFILEOPERATIONS_USE_NAMESPACE using namespace DPFILEOPERATIONS_NAMESPACE;

DPFILEOPERATIONS_BEGIN_NAMESPACE
DFM_LOG_USE_CATEGORY(DPFILEOPERATIONS_NAMESPACE)
DPFILEOPERATIONS_END_NAMESPACE

#endif   // DFMPLUGIN_FILEOPERATIONS_GLOBAL_H
