/****************************************************************************
** Meta object code from reading C++ file 'searcheventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-search/events/searcheventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'searcheventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_search__SearchEventReceiver_t {
    QByteArrayData data[20];
    char stringdata0[245];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_search__SearchEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_search__SearchEventReceiver_t qt_meta_stringdata_dfmplugin_search__SearchEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 37), // "dfmplugin_search::SearchEvent..."
QT_MOC_LITERAL(1, 38, 12), // "handleSearch"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 5), // "winId"
QT_MOC_LITERAL(4, 58, 7), // "keyword"
QT_MOC_LITERAL(5, 66, 16), // "handleStopSearch"
QT_MOC_LITERAL(6, 83, 26), // "handleShowAdvanceSearchBar"
QT_MOC_LITERAL(7, 110, 7), // "visible"
QT_MOC_LITERAL(8, 118, 16), // "handleUrlChanged"
QT_MOC_LITERAL(9, 135, 1), // "u"
QT_MOC_LITERAL(10, 137, 21), // "handleAddressInputStr"
QT_MOC_LITERAL(11, 159, 6), // "windId"
QT_MOC_LITERAL(12, 166, 8), // "QString*"
QT_MOC_LITERAL(13, 175, 3), // "str"
QT_MOC_LITERAL(14, 179, 13), // "handleFileAdd"
QT_MOC_LITERAL(15, 193, 3), // "url"
QT_MOC_LITERAL(16, 197, 16), // "handleFileDelete"
QT_MOC_LITERAL(17, 214, 16), // "handleFileRename"
QT_MOC_LITERAL(18, 231, 6), // "oldUrl"
QT_MOC_LITERAL(19, 238, 6) // "newUrl"

    },
    "dfmplugin_search::SearchEventReceiver\0"
    "handleSearch\0\0winId\0keyword\0"
    "handleStopSearch\0handleShowAdvanceSearchBar\0"
    "visible\0handleUrlChanged\0u\0"
    "handleAddressInputStr\0windId\0QString*\0"
    "str\0handleFileAdd\0url\0handleFileDelete\0"
    "handleFileRename\0oldUrl\0newUrl"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_search__SearchEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   54,    2, 0x0a /* Public */,
       5,    1,   59,    2, 0x0a /* Public */,
       6,    2,   62,    2, 0x0a /* Public */,
       8,    2,   67,    2, 0x0a /* Public */,
      10,    2,   72,    2, 0x0a /* Public */,
      14,    1,   77,    2, 0x0a /* Public */,
      16,    1,   80,    2, 0x0a /* Public */,
      17,    2,   83,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QString,    3,    4,
    QMetaType::Void, QMetaType::ULongLong,    3,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::Bool,    3,    7,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,    3,    9,
    QMetaType::Void, QMetaType::ULongLong, 0x80000000 | 12,   11,   13,
    QMetaType::Void, QMetaType::QUrl,   15,
    QMetaType::Void, QMetaType::QUrl,   15,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,   18,   19,

       0        // eod
};

void dfmplugin_search::SearchEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SearchEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleSearch((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->handleStopSearch((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 2: _t->handleShowAdvanceSearchBar((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: _t->handleUrlChanged((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 4: _t->handleAddressInputStr((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< QString*(*)>(_a[2]))); break;
        case 5: _t->handleFileAdd((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 6: _t->handleFileDelete((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 7: _t->handleFileRename((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_search::SearchEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_search__SearchEventReceiver.data,
    qt_meta_data_dfmplugin_search__SearchEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_search::SearchEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_search::SearchEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_search__SearchEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_search::SearchEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
