/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -p recentmanager_interface /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.Daemon.RecentManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef RECENTMANAGER_INTERFACE_H
#define RECENTMANAGER_INTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.deepin.Filemanager.Daemon.RecentManager
 */
class OrgDeepinFilemanagerDaemonRecentManagerInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.deepin.Filemanager.Daemon.RecentManager"; }

public:
    OrgDeepinFilemanagerDaemonRecentManagerInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgDeepinFilemanagerDaemonRecentManagerInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> AddItem(const QVariantMap &item)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(item);
        return asyncCallWithArgumentList(QStringLiteral("AddItem"), argumentList);
    }

    inline QDBusPendingReply<QVariantMap> GetItemInfo(const QString &path)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(path);
        return asyncCallWithArgumentList(QStringLiteral("GetItemInfo"), argumentList);
    }

    inline QDBusPendingReply<QVariantList> GetItemsInfo()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("GetItemsInfo"), argumentList);
    }

    inline QDBusPendingReply<QStringList> GetItemsPath()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("GetItemsPath"), argumentList);
    }

    inline QDBusPendingReply<> PurgeItems()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("PurgeItems"), argumentList);
    }

    inline QDBusPendingReply<qlonglong> Reload()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Reload"), argumentList);
    }

    inline QDBusPendingReply<> RemoveItems(const QStringList &hrefs)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(hrefs);
        return asyncCallWithArgumentList(QStringLiteral("RemoveItems"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void ItemAdded(const QString &path, const QString &href, qlonglong modified);
    void ItemChanged(const QString &path, qlonglong modified);
    void ItemsRemoved(const QStringList &paths);
    void PurgeFinished();
    void ReloadFinished(qlonglong timestamp);
};

namespace org {
  namespace deepin {
    namespace Filemanager {
      namespace Daemon {
        typedef ::OrgDeepinFilemanagerDaemonRecentManagerInterface RecentManager;
      }
    }
  }
}
#endif
