/****************************************************************************
** Meta object code from reading C++ file 'canvasmanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-canvas/canvasmanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvasmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_canvas__CanvasManager_t {
    QByteArrayData data[14];
    char stringdata0[196];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_canvas__CanvasManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_canvas__CanvasManager_t qt_meta_stringdata_ddplugin_canvas__CanvasManager = {
    {
QT_MOC_LITERAL(0, 0, 30), // "ddplugin_canvas::CanvasManager"
QT_MOC_LITERAL(1, 31, 13), // "onCanvasBuild"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 15), // "onDetachWindows"
QT_MOC_LITERAL(4, 62, 17), // "onGeometryChanged"
QT_MOC_LITERAL(5, 80, 16), // "onWallperSetting"
QT_MOC_LITERAL(6, 97, 11), // "CanvasView*"
QT_MOC_LITERAL(7, 109, 17), // "onChangeIconLevel"
QT_MOC_LITERAL(8, 127, 8), // "increase"
QT_MOC_LITERAL(9, 136, 19), // "onTrashStateChanged"
QT_MOC_LITERAL(10, 156, 13), // "onFontChanged"
QT_MOC_LITERAL(11, 170, 7), // "refresh"
QT_MOC_LITERAL(12, 178, 6), // "silent"
QT_MOC_LITERAL(13, 185, 10) // "reloadItem"

    },
    "ddplugin_canvas::CanvasManager\0"
    "onCanvasBuild\0\0onDetachWindows\0"
    "onGeometryChanged\0onWallperSetting\0"
    "CanvasView*\0onChangeIconLevel\0increase\0"
    "onTrashStateChanged\0onFontChanged\0"
    "refresh\0silent\0reloadItem"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_canvas__CanvasManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x0a /* Public */,
       3,    0,   60,    2, 0x0a /* Public */,
       4,    0,   61,    2, 0x0a /* Public */,
       5,    1,   62,    2, 0x0a /* Public */,
       7,    1,   65,    2, 0x0a /* Public */,
       9,    0,   68,    2, 0x0a /* Public */,
      10,    0,   69,    2, 0x0a /* Public */,
      11,    1,   70,    2, 0x0a /* Public */,
      13,    0,   73,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void,

       0        // eod
};

void ddplugin_canvas::CanvasManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CanvasManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onCanvasBuild(); break;
        case 1: _t->onDetachWindows(); break;
        case 2: _t->onGeometryChanged(); break;
        case 3: _t->onWallperSetting((*reinterpret_cast< CanvasView*(*)>(_a[1]))); break;
        case 4: _t->onChangeIconLevel((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->onTrashStateChanged(); break;
        case 6: _t->onFontChanged(); break;
        case 7: _t->refresh((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->reloadItem(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_canvas::CanvasManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_canvas__CanvasManager.data,
    qt_meta_data_ddplugin_canvas__CanvasManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_canvas::CanvasManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_canvas::CanvasManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_canvas__CanvasManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_canvas::CanvasManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
