/****************************************************************************
** Meta object code from reading C++ file 'tab.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-titlebar/views/tab.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tab.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_titlebar__Tab_t {
    QByteArrayData data[17];
    char stringdata0[225];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_titlebar__Tab_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_titlebar__Tab_t qt_meta_stringdata_dfmplugin_titlebar__Tab = {
    {
QT_MOC_LITERAL(0, 0, 23), // "dfmplugin_titlebar::Tab"
QT_MOC_LITERAL(1, 24, 7), // "clicked"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 8), // "moveNext"
QT_MOC_LITERAL(4, 42, 4), // "Tab*"
QT_MOC_LITERAL(5, 47, 3), // "tab"
QT_MOC_LITERAL(6, 51, 11), // "movePrevius"
QT_MOC_LITERAL(7, 63, 16), // "requestNewWindow"
QT_MOC_LITERAL(8, 80, 3), // "url"
QT_MOC_LITERAL(9, 84, 16), // "aboutToNewWindow"
QT_MOC_LITERAL(10, 101, 16), // "draggingFinished"
QT_MOC_LITERAL(11, 118, 15), // "draggingStarted"
QT_MOC_LITERAL(12, 134, 20), // "requestActiveNextTab"
QT_MOC_LITERAL(13, 155, 24), // "requestActivePreviousTab"
QT_MOC_LITERAL(14, 180, 14), // "closeRequested"
QT_MOC_LITERAL(15, 195, 20), // "onFileRootUrlChanged"
QT_MOC_LITERAL(16, 216, 8) // "geometry"

    },
    "dfmplugin_titlebar::Tab\0clicked\0\0"
    "moveNext\0Tab*\0tab\0movePrevius\0"
    "requestNewWindow\0url\0aboutToNewWindow\0"
    "draggingFinished\0draggingStarted\0"
    "requestActiveNextTab\0requestActivePreviousTab\0"
    "closeRequested\0onFileRootUrlChanged\0"
    "geometry"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_titlebar__Tab[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       1,   90, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x06 /* Public */,
       3,    1,   70,    2, 0x06 /* Public */,
       6,    1,   73,    2, 0x06 /* Public */,
       7,    1,   76,    2, 0x06 /* Public */,
       9,    1,   79,    2, 0x06 /* Public */,
      10,    0,   82,    2, 0x06 /* Public */,
      11,    0,   83,    2, 0x06 /* Public */,
      12,    0,   84,    2, 0x06 /* Public */,
      13,    0,   85,    2, 0x06 /* Public */,
      14,    0,   86,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    1,   87,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, QMetaType::QUrl,    8,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl,    8,

 // properties: name, type, flags
      16, QMetaType::QRect, 0x00095103,

       0        // eod
};

void dfmplugin_titlebar::Tab::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Tab *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->clicked(); break;
        case 1: _t->moveNext((*reinterpret_cast< Tab*(*)>(_a[1]))); break;
        case 2: _t->movePrevius((*reinterpret_cast< Tab*(*)>(_a[1]))); break;
        case 3: _t->requestNewWindow((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 4: _t->aboutToNewWindow((*reinterpret_cast< Tab*(*)>(_a[1]))); break;
        case 5: _t->draggingFinished(); break;
        case 6: _t->draggingStarted(); break;
        case 7: _t->requestActiveNextTab(); break;
        case 8: _t->requestActivePreviousTab(); break;
        case 9: _t->closeRequested(); break;
        case 10: _t->onFileRootUrlChanged((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Tab* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Tab* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Tab* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Tab::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Tab::clicked)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Tab::*)(Tab * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Tab::moveNext)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Tab::*)(Tab * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Tab::movePrevius)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Tab::*)(const QUrl );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Tab::requestNewWindow)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Tab::*)(Tab * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Tab::aboutToNewWindow)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Tab::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Tab::draggingFinished)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Tab::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Tab::draggingStarted)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Tab::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Tab::requestActiveNextTab)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Tab::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Tab::requestActivePreviousTab)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Tab::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Tab::closeRequested)) {
                *result = 9;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Tab *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QRect*>(_v) = _t->geometry(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Tab *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setGeometry(*reinterpret_cast< QRect*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_titlebar::Tab::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_titlebar__Tab.data,
    qt_meta_data_dfmplugin_titlebar__Tab,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_titlebar::Tab::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_titlebar::Tab::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_titlebar__Tab.stringdata0))
        return static_cast<void*>(this);
    return QGraphicsObject::qt_metacast(_clname);
}

int dfmplugin_titlebar::Tab::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void dfmplugin_titlebar::Tab::clicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dfmplugin_titlebar::Tab::moveNext(Tab * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_titlebar::Tab::movePrevius(Tab * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_titlebar::Tab::requestNewWindow(const QUrl _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_titlebar::Tab::aboutToNewWindow(Tab * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void dfmplugin_titlebar::Tab::draggingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void dfmplugin_titlebar::Tab::draggingStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void dfmplugin_titlebar::Tab::requestActiveNextTab()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void dfmplugin_titlebar::Tab::requestActivePreviousTab()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void dfmplugin_titlebar::Tab::closeRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
