/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -p operationsstackmanager_interface /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.Daemon.OperationsStackManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef OPERATIONSSTACKMANAGER_INTERFACE_H
#define OPERATIONSSTACKMANAGER_INTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.deepin.Filemanager.Daemon.OperationsStackManager
 */
class OrgDeepinFilemanagerDaemonOperationsStackManagerInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.deepin.Filemanager.Daemon.OperationsStackManager"; }

public:
    OrgDeepinFilemanagerDaemonOperationsStackManagerInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgDeepinFilemanagerDaemonOperationsStackManagerInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> CleanOperations()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("CleanOperations"), argumentList);
    }

    inline QDBusPendingReply<> CleanOperationsByUrl(const QStringList &urls)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(urls);
        return asyncCallWithArgumentList(QStringLiteral("CleanOperationsByUrl"), argumentList);
    }

    inline QDBusPendingReply<QVariantMap> RevocationOperations()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("RevocationOperations"), argumentList);
    }

    inline QDBusPendingReply<QVariantMap> RevocationRedoOperations()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("RevocationRedoOperations"), argumentList);
    }

    inline QDBusPendingReply<> SaveOperations(const QVariantMap &values)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(values);
        return asyncCallWithArgumentList(QStringLiteral("SaveOperations"), argumentList);
    }

    inline QDBusPendingReply<> SaveRedoOperations(const QVariantMap &values)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(values);
        return asyncCallWithArgumentList(QStringLiteral("SaveRedoOperations"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace deepin {
    namespace Filemanager {
      namespace Daemon {
        typedef ::OrgDeepinFilemanagerDaemonOperationsStackManagerInterface OperationsStackManager;
      }
    }
  }
}
#endif
