/****************************************************************************
** Meta object code from reading C++ file 'canvasviewshell.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-organizer/interface/canvasviewshell.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvasviewshell.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_organizer__CanvasViewShell_t {
    QByteArrayData data[25];
    char stringdata0[312];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_organizer__CanvasViewShell_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_organizer__CanvasViewShell_t qt_meta_stringdata_ddplugin_organizer__CanvasViewShell = {
    {
QT_MOC_LITERAL(0, 0, 35), // "ddplugin_organizer::CanvasVie..."
QT_MOC_LITERAL(1, 36, 14), // "filterDropData"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 9), // "viewIndex"
QT_MOC_LITERAL(4, 62, 16), // "const QMimeData*"
QT_MOC_LITERAL(5, 79, 8), // "mimeData"
QT_MOC_LITERAL(6, 88, 7), // "viewPos"
QT_MOC_LITERAL(7, 96, 7), // "extData"
QT_MOC_LITERAL(8, 104, 22), // "filterShortcutkeyPress"
QT_MOC_LITERAL(9, 127, 3), // "key"
QT_MOC_LITERAL(10, 131, 9), // "modifiers"
QT_MOC_LITERAL(11, 141, 14), // "filterKeyPress"
QT_MOC_LITERAL(12, 156, 11), // "filterWheel"
QT_MOC_LITERAL(13, 168, 10), // "angleDelta"
QT_MOC_LITERAL(14, 179, 4), // "ctrl"
QT_MOC_LITERAL(15, 184, 17), // "filterContextMenu"
QT_MOC_LITERAL(16, 202, 3), // "dir"
QT_MOC_LITERAL(17, 206, 11), // "QList<QUrl>"
QT_MOC_LITERAL(18, 218, 5), // "files"
QT_MOC_LITERAL(19, 224, 13), // "eventDropData"
QT_MOC_LITERAL(20, 238, 9), // "viewPoint"
QT_MOC_LITERAL(21, 248, 13), // "eventKeyPress"
QT_MOC_LITERAL(22, 262, 21), // "eventShortcutkeyPress"
QT_MOC_LITERAL(23, 284, 10), // "eventWheel"
QT_MOC_LITERAL(24, 295, 16) // "eventContextMenu"

    },
    "ddplugin_organizer::CanvasViewShell\0"
    "filterDropData\0\0viewIndex\0const QMimeData*\0"
    "mimeData\0viewPos\0extData\0"
    "filterShortcutkeyPress\0key\0modifiers\0"
    "filterKeyPress\0filterWheel\0angleDelta\0"
    "ctrl\0filterContextMenu\0dir\0QList<QUrl>\0"
    "files\0eventDropData\0viewPoint\0"
    "eventKeyPress\0eventShortcutkeyPress\0"
    "eventWheel\0eventContextMenu"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_organizer__CanvasViewShell[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    4,   74,    2, 0x06 /* Public */,
       8,    3,   83,    2, 0x06 /* Public */,
      11,    3,   90,    2, 0x06 /* Public */,
      12,    3,   97,    2, 0x06 /* Public */,
      15,    4,  104,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      19,    4,  113,    2, 0x08 /* Private */,
      21,    4,  122,    2, 0x08 /* Private */,
      21,    3,  131,    2, 0x28 /* Private | MethodCloned */,
      22,    4,  138,    2, 0x08 /* Private */,
      22,    3,  147,    2, 0x28 /* Private | MethodCloned */,
      23,    3,  154,    2, 0x08 /* Private */,
      24,    5,  161,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Bool, QMetaType::Int, 0x80000000 | 4, QMetaType::QPoint, QMetaType::VoidStar,    3,    5,    6,    7,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int,    3,    9,   10,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int,    3,    9,   10,
    QMetaType::Bool, QMetaType::Int, QMetaType::QPoint, QMetaType::Bool,    3,   13,   14,
    QMetaType::Bool, QMetaType::Int, QMetaType::QUrl, 0x80000000 | 17, QMetaType::QPoint,    3,   16,   18,    6,

 // slots: parameters
    QMetaType::Bool, QMetaType::Int, 0x80000000 | 4, QMetaType::QPoint, QMetaType::VoidStar,    3,    5,   20,    7,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::VoidStar,    3,    9,   10,    7,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int,    3,    9,   10,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::VoidStar,    3,    9,   10,    7,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int,    3,    9,   10,
    QMetaType::Bool, QMetaType::Int, QMetaType::QPoint, QMetaType::VoidStar,    3,   13,    7,
    QMetaType::Bool, QMetaType::Int, QMetaType::QUrl, 0x80000000 | 17, QMetaType::QPoint, QMetaType::VoidStar,    3,   16,   18,    6,    7,

       0        // eod
};

void ddplugin_organizer::CanvasViewShell::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CanvasViewShell *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->filterDropData((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QMimeData*(*)>(_a[2])),(*reinterpret_cast< const QPoint(*)>(_a[3])),(*reinterpret_cast< void*(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->filterShortcutkeyPress((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->filterKeyPress((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->filterWheel((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QPoint(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->filterContextMenu((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[3])),(*reinterpret_cast< const QPoint(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->eventDropData((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QMimeData*(*)>(_a[2])),(*reinterpret_cast< const QPoint(*)>(_a[3])),(*reinterpret_cast< void*(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->eventKeyPress((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< void*(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->eventKeyPress((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->eventShortcutkeyPress((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< void*(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->eventShortcutkeyPress((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->eventWheel((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QPoint(*)>(_a[2])),(*reinterpret_cast< void*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->eventContextMenu((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[3])),(*reinterpret_cast< const QPoint(*)>(_a[4])),(*reinterpret_cast< void*(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = bool (CanvasViewShell::*)(int , const QMimeData * , const QPoint & , void * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CanvasViewShell::filterDropData)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = bool (CanvasViewShell::*)(int , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CanvasViewShell::filterShortcutkeyPress)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = bool (CanvasViewShell::*)(int , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CanvasViewShell::filterKeyPress)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = bool (CanvasViewShell::*)(int , const QPoint & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CanvasViewShell::filterWheel)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = bool (CanvasViewShell::*)(int , const QUrl & , const QList<QUrl> & , const QPoint & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CanvasViewShell::filterContextMenu)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_organizer::CanvasViewShell::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_organizer__CanvasViewShell.data,
    qt_meta_data_ddplugin_organizer__CanvasViewShell,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_organizer::CanvasViewShell::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_organizer::CanvasViewShell::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_organizer__CanvasViewShell.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_organizer::CanvasViewShell::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
bool ddplugin_organizer::CanvasViewShell::filterDropData(int _t1, const QMimeData * _t2, const QPoint & _t3, void * _t4)
{
    bool _t0{};
    void *_a[] = { const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t0))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
    return _t0;
}

// SIGNAL 1
bool ddplugin_organizer::CanvasViewShell::filterShortcutkeyPress(int _t1, int _t2, int _t3)
{
    bool _t0{};
    void *_a[] = { const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t0))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
    return _t0;
}

// SIGNAL 2
bool ddplugin_organizer::CanvasViewShell::filterKeyPress(int _t1, int _t2, int _t3)
{
    bool _t0{};
    void *_a[] = { const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t0))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
    return _t0;
}

// SIGNAL 3
bool ddplugin_organizer::CanvasViewShell::filterWheel(int _t1, const QPoint & _t2, bool _t3)
{
    bool _t0{};
    void *_a[] = { const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t0))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
    return _t0;
}

// SIGNAL 4
bool ddplugin_organizer::CanvasViewShell::filterContextMenu(int _t1, const QUrl & _t2, const QList<QUrl> & _t3, const QPoint & _t4)
{
    bool _t0{};
    void *_a[] = { const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t0))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
    return _t0;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
