/****************************************************************************
** Meta object code from reading C++ file 'wlsetplugin.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-wallpapersetting/wlsetplugin.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wlsetplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_wallpapersetting__EventHandle_t {
    QByteArrayData data[8];
    char stringdata0[113];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_wallpapersetting__EventHandle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_wallpapersetting__EventHandle_t qt_meta_stringdata_ddplugin_wallpapersetting__EventHandle = {
    {
QT_MOC_LITERAL(0, 0, 38), // "ddplugin_wallpapersetting::Ev..."
QT_MOC_LITERAL(1, 39, 16), // "wallpaperSetting"
QT_MOC_LITERAL(2, 56, 0), // ""
QT_MOC_LITERAL(3, 57, 4), // "name"
QT_MOC_LITERAL(4, 62, 18), // "screenSaverSetting"
QT_MOC_LITERAL(5, 81, 17), // "hookCanvasRequest"
QT_MOC_LITERAL(6, 99, 6), // "screen"
QT_MOC_LITERAL(7, 106, 6) // "onQuit"

    },
    "ddplugin_wallpapersetting::EventHandle\0"
    "wallpaperSetting\0\0name\0screenSaverSetting\0"
    "hookCanvasRequest\0screen\0onQuit"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_wallpapersetting__EventHandle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x0a /* Public */,
       4,    1,   37,    2, 0x0a /* Public */,
       5,    1,   40,    2, 0x0a /* Public */,
       7,    0,   43,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Bool, QMetaType::QString,    3,
    QMetaType::Bool, QMetaType::QString,    3,
    QMetaType::Bool, QMetaType::QString,    6,
    QMetaType::Void,

       0        // eod
};

void ddplugin_wallpapersetting::EventHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EventHandle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->wallpaperSetting((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->screenSaverSetting((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->hookCanvasRequest((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->onQuit(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_wallpapersetting::EventHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_wallpapersetting__EventHandle.data,
    qt_meta_data_ddplugin_wallpapersetting__EventHandle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_wallpapersetting::EventHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_wallpapersetting::EventHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_wallpapersetting__EventHandle.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_wallpapersetting::EventHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}
struct qt_meta_stringdata_ddplugin_wallpapersetting__WlSetPlugin_t {
    QByteArrayData data[1];
    char stringdata0[39];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_wallpapersetting__WlSetPlugin_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_wallpapersetting__WlSetPlugin_t qt_meta_stringdata_ddplugin_wallpapersetting__WlSetPlugin = {
    {
QT_MOC_LITERAL(0, 0, 38) // "ddplugin_wallpapersetting::Wl..."

    },
    "ddplugin_wallpapersetting::WlSetPlugin"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_wallpapersetting__WlSetPlugin[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ddplugin_wallpapersetting::WlSetPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_wallpapersetting::WlSetPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<dpf::Plugin::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_wallpapersetting__WlSetPlugin.data,
    qt_meta_data_ddplugin_wallpapersetting__WlSetPlugin,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_wallpapersetting::WlSetPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_wallpapersetting::WlSetPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_wallpapersetting__WlSetPlugin.stringdata0))
        return static_cast<void*>(this);
    return dpf::Plugin::qt_metacast(_clname);
}

int ddplugin_wallpapersetting::WlSetPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = dpf::Plugin::qt_metacall(_c, _id, _a);
    return _id;
}

QT_PLUGIN_METADATA_SECTION
static constexpr unsigned char qt_pluginMetaData[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x19,  'o',  'r',  'g',  '.',  'd', 
    'e',  'e',  'p',  'i',  'n',  '.',  'p',  'l', 
    'u',  'g',  'i',  'n',  '.',  'd',  'e',  's', 
    'k',  't',  'o',  'p', 
    // "className"
    0x03,  0x6b,  'W',  'l',  'S',  'e',  't',  'P', 
    'l',  'u',  'g',  'i',  'n', 
    // "MetaData"
    0x04,  0xaa,  0x68,  'C',  'a',  't',  'e',  'g', 
    'o',  'r',  'y',  0x60,  0x6d,  'C',  'o',  'm', 
    'p',  'a',  't',  'V',  'e',  'r',  's',  'i', 
    'o',  'n',  0x65,  '1',  '.',  '0',  '.',  '0', 
    0x69,  'C',  'o',  'p',  'y',  'r',  'i',  'g', 
    'h',  't',  0x78,  0x3a,  'C',  'o',  'p',  'y', 
    'r',  'i',  'g',  'h',  't',  ' ',  '(',  'C', 
    ')',  ' ',  '2',  '0',  '2',  '2',  ' ',  'U', 
    'n',  'i',  'o',  'n',  't',  'e',  'c',  'h', 
    ' ',  'S',  'o',  'f',  't',  'w',  'a',  'r', 
    'e',  ' ',  'T',  'e',  'c',  'h',  'n',  'o', 
    'l',  'o',  'g',  'y',  ' ',  'C',  'o',  '.', 
    ',',  ' ',  'L',  't',  'd',  '.',  0x67,  'D', 
    'e',  'p',  'e',  'n',  'd',  's',  0x82,  0xa2, 
    0x64,  'N',  'a',  'm',  'e',  0x6d,  'd',  'd', 
    'p',  'l',  'u',  'g',  'i',  'n',  '-',  'c', 
    'o',  'r',  'e',  0x67,  'V',  'e',  'r',  's', 
    'i',  'o',  'n',  0x65,  '1',  '.',  '0',  '.', 
    '0',  0xa2,  0x64,  'N',  'a',  'm',  'e',  0x6f, 
    'd',  'd',  'p',  'l',  'u',  'g',  'i',  'n', 
    '-',  'c',  'a',  'n',  'v',  'a',  's',  0x67, 
    'V',  'e',  'r',  's',  'i',  'o',  'n',  0x65, 
    '1',  '.',  '0',  '.',  '0',  0x6b,  'D',  'e', 
    's',  'c',  'r',  'i',  'p',  't',  'i',  'o', 
    'n',  0x78,  0x32,  'T',  'h',  'e',  ' ',  'c', 
    'o',  'm',  'm',  'o',  'n',  ' ',  'p',  'l', 
    'u',  'g',  'i',  'n',  ' ',  'f',  'o',  'r', 
    ' ',  't',  'h',  'e',  ' ',  'f',  'i',  'l', 
    'e',  'm',  'a',  'n',  'a',  'g',  'e',  'r', 
    ' ',  'a',  'n',  'd',  ' ',  'd',  'e',  's', 
    'k',  't',  'o',  'p',  '.',  0x67,  'L',  'i', 
    'c',  'e',  'n',  's',  'e',  0x80,  0x64,  'N', 
    'a',  'm',  'e',  0x78,  0x19,  'd',  'd',  'p', 
    'l',  'u',  'g',  'i',  'n',  '-',  'w',  'a', 
    'l',  'l',  'p',  'a',  'p',  'e',  'r',  's', 
    'e',  't',  't',  'i',  'n',  'g',  0x67,  'U', 
    'r',  'l',  'L',  'i',  'n',  'k',  0x78,  0x19, 
    'h',  't',  't',  'p',  's',  ':',  '/',  '/', 
    'w',  'w',  'w',  '.',  'u',  'n',  'i',  'o', 
    'n',  't',  'e',  'c',  'h',  '.',  'c',  'o', 
    'm',  0x66,  'V',  'e',  'n',  'd',  'o',  'r', 
    0x78,  0x2b,  'T',  'h',  'e',  ' ',  'U',  'n', 
    'i',  'o',  'n',  't',  'e',  'c',  'h',  ' ', 
    'S',  'o',  'f',  't',  'w',  'a',  'r',  'e', 
    ' ',  'T',  'e',  'c',  'h',  'n',  'o',  'l', 
    'o',  'g',  'y',  ' ',  'C',  'o',  '.',  ',', 
    ' ',  'L',  't',  'd',  '.',  0x67,  'V',  'e', 
    'r',  's',  'i',  'o',  'n',  0x65,  '1',  '.', 
    '0',  '.',  '0', 
    0xff, 
};
using namespace ddplugin_wallpapersetting;
QT_MOC_EXPORT_PLUGIN(ddplugin_wallpapersetting::WlSetPlugin, WlSetPlugin)

QT_WARNING_POP
QT_END_MOC_NAMESPACE
