/****************************************************************************
** Meta object code from reading C++ file 'computermodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-computer/models/computermodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'computermodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_computer__ComputerModel_t {
    QByteArrayData data[41];
    char stringdata0[622];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_computer__ComputerModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_computer__ComputerModel_t qt_meta_stringdata_dfmplugin_computer__ComputerModel = {
    {
QT_MOC_LITERAL(0, 0, 33), // "dfmplugin_computer::ComputerM..."
QT_MOC_LITERAL(1, 34, 21), // "requestClearSelection"
QT_MOC_LITERAL(2, 56, 0), // ""
QT_MOC_LITERAL(3, 57, 3), // "url"
QT_MOC_LITERAL(4, 61, 24), // "requestHandleItemVisible"
QT_MOC_LITERAL(5, 86, 18), // "requestUpdateIndex"
QT_MOC_LITERAL(6, 105, 11), // "QModelIndex"
QT_MOC_LITERAL(7, 117, 5), // "index"
QT_MOC_LITERAL(8, 123, 11), // "onItemAdded"
QT_MOC_LITERAL(9, 135, 16), // "ComputerItemData"
QT_MOC_LITERAL(10, 152, 4), // "data"
QT_MOC_LITERAL(11, 157, 13), // "onItemRemoved"
QT_MOC_LITERAL(12, 171, 13), // "onItemUpdated"
QT_MOC_LITERAL(13, 185, 14), // "updateItemInfo"
QT_MOC_LITERAL(14, 200, 3), // "pos"
QT_MOC_LITERAL(15, 204, 17), // "onItemSizeChanged"
QT_MOC_LITERAL(16, 222, 5), // "total"
QT_MOC_LITERAL(17, 228, 4), // "free"
QT_MOC_LITERAL(18, 233, 21), // "onItemPropertyChanged"
QT_MOC_LITERAL(19, 255, 3), // "key"
QT_MOC_LITERAL(20, 259, 3), // "val"
QT_MOC_LITERAL(21, 263, 8), // "addGroup"
QT_MOC_LITERAL(22, 272, 17), // "removeOrphanGroup"
QT_MOC_LITERAL(23, 290, 9), // "DataRoles"
QT_MOC_LITERAL(24, 300, 14), // "kSizeTotalRole"
QT_MOC_LITERAL(25, 315, 14), // "kSizeUsageRole"
QT_MOC_LITERAL(26, 330, 15), // "kFileSystemRole"
QT_MOC_LITERAL(27, 346, 12), // "kRealUrlRole"
QT_MOC_LITERAL(28, 359, 14), // "kDeviceUrlRole"
QT_MOC_LITERAL(29, 374, 22), // "kDeviceIsEncryptedRole"
QT_MOC_LITERAL(30, 397, 21), // "kDeviceIsUnlockedRole"
QT_MOC_LITERAL(31, 419, 21), // "kDeviceClearDevIdRole"
QT_MOC_LITERAL(32, 441, 11), // "kSuffixRole"
QT_MOC_LITERAL(33, 453, 21), // "kProgressVisiableRole"
QT_MOC_LITERAL(34, 475, 22), // "kTotalSizeVisiableRole"
QT_MOC_LITERAL(35, 498, 21), // "kUsedSizeVisiableRole"
QT_MOC_LITERAL(36, 520, 24), // "kDeviceNameMaxLengthRole"
QT_MOC_LITERAL(37, 545, 18), // "kItemShapeTypeRole"
QT_MOC_LITERAL(38, 564, 15), // "kActionListRole"
QT_MOC_LITERAL(39, 580, 18), // "kItemIsEditingRole"
QT_MOC_LITERAL(40, 599, 22) // "kDeviceDescriptionRole"

    },
    "dfmplugin_computer::ComputerModel\0"
    "requestClearSelection\0\0url\0"
    "requestHandleItemVisible\0requestUpdateIndex\0"
    "QModelIndex\0index\0onItemAdded\0"
    "ComputerItemData\0data\0onItemRemoved\0"
    "onItemUpdated\0updateItemInfo\0pos\0"
    "onItemSizeChanged\0total\0free\0"
    "onItemPropertyChanged\0key\0val\0addGroup\0"
    "removeOrphanGroup\0DataRoles\0kSizeTotalRole\0"
    "kSizeUsageRole\0kFileSystemRole\0"
    "kRealUrlRole\0kDeviceUrlRole\0"
    "kDeviceIsEncryptedRole\0kDeviceIsUnlockedRole\0"
    "kDeviceClearDevIdRole\0kSuffixRole\0"
    "kProgressVisiableRole\0kTotalSizeVisiableRole\0"
    "kUsedSizeVisiableRole\0kDeviceNameMaxLengthRole\0"
    "kItemShapeTypeRole\0kActionListRole\0"
    "kItemIsEditingRole\0kDeviceDescriptionRole"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_computer__ComputerModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       1,  106, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,
       4,    0,   72,    2, 0x06 /* Public */,
       5,    1,   73,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    1,   76,    2, 0x09 /* Protected */,
      11,    1,   79,    2, 0x09 /* Protected */,
      12,    1,   82,    2, 0x09 /* Protected */,
      13,    1,   85,    2, 0x09 /* Protected */,
      15,    3,   88,    2, 0x09 /* Protected */,
      18,    3,   95,    2, 0x09 /* Protected */,
      21,    1,  102,    2, 0x09 /* Protected */,
      22,    0,  105,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl,    3,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    7,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::QUrl,    3,
    QMetaType::Void, QMetaType::QUrl,    3,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void, QMetaType::QUrl, QMetaType::LongLong, QMetaType::LongLong,    3,   16,   17,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QString, QMetaType::QVariant,    3,   19,   20,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,

 // enums: name, alias, flags, count, data
      23,   23, 0x0,   17,  111,

 // enum data: key, value
      24, uint(dfmplugin_computer::ComputerModel::kSizeTotalRole),
      25, uint(dfmplugin_computer::ComputerModel::kSizeUsageRole),
      26, uint(dfmplugin_computer::ComputerModel::kFileSystemRole),
      27, uint(dfmplugin_computer::ComputerModel::kRealUrlRole),
      28, uint(dfmplugin_computer::ComputerModel::kDeviceUrlRole),
      29, uint(dfmplugin_computer::ComputerModel::kDeviceIsEncryptedRole),
      30, uint(dfmplugin_computer::ComputerModel::kDeviceIsUnlockedRole),
      31, uint(dfmplugin_computer::ComputerModel::kDeviceClearDevIdRole),
      32, uint(dfmplugin_computer::ComputerModel::kSuffixRole),
      33, uint(dfmplugin_computer::ComputerModel::kProgressVisiableRole),
      34, uint(dfmplugin_computer::ComputerModel::kTotalSizeVisiableRole),
      35, uint(dfmplugin_computer::ComputerModel::kUsedSizeVisiableRole),
      36, uint(dfmplugin_computer::ComputerModel::kDeviceNameMaxLengthRole),
      37, uint(dfmplugin_computer::ComputerModel::kItemShapeTypeRole),
      38, uint(dfmplugin_computer::ComputerModel::kActionListRole),
      39, uint(dfmplugin_computer::ComputerModel::kItemIsEditingRole),
      40, uint(dfmplugin_computer::ComputerModel::kDeviceDescriptionRole),

       0        // eod
};

void dfmplugin_computer::ComputerModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ComputerModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestClearSelection((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 1: _t->requestHandleItemVisible(); break;
        case 2: _t->requestUpdateIndex((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 3: _t->onItemAdded((*reinterpret_cast< const ComputerItemData(*)>(_a[1]))); break;
        case 4: _t->onItemRemoved((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 5: _t->onItemUpdated((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 6: _t->updateItemInfo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->onItemSizeChanged((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< qlonglong(*)>(_a[2])),(*reinterpret_cast< qlonglong(*)>(_a[3]))); break;
        case 8: _t->onItemPropertyChanged((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3]))); break;
        case 9: _t->addGroup((*reinterpret_cast< const ComputerItemData(*)>(_a[1]))); break;
        case 10: _t->removeOrphanGroup(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ComputerModel::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComputerModel::requestClearSelection)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ComputerModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComputerModel::requestHandleItemVisible)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ComputerModel::*)(const QModelIndex & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComputerModel::requestUpdateIndex)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_computer::ComputerModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_computer__ComputerModel.data,
    qt_meta_data_dfmplugin_computer__ComputerModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_computer::ComputerModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_computer::ComputerModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_computer__ComputerModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int dfmplugin_computer::ComputerModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_computer::ComputerModel::requestClearSelection(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_computer::ComputerModel::requestHandleItemVisible()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dfmplugin_computer::ComputerModel::requestUpdateIndex(const QModelIndex & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
