/****************************************************************************
** Meta object code from reading C++ file 'tagmanagerdbus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/daemon/daemonplugin-tag/tagmanagerdbus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tagmanagerdbus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TagManagerDBus_t {
    QByteArrayData data[21];
    char stringdata0[251];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TagManagerDBus_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TagManagerDBus_t qt_meta_stringdata_TagManagerDBus = {
    {
QT_MOC_LITERAL(0, 0, 14), // "TagManagerDBus"
QT_MOC_LITERAL(1, 15, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 31, 40), // "org.deepin.Filemanager.Daemon..."
QT_MOC_LITERAL(3, 72, 16), // "TagsServiceReady"
QT_MOC_LITERAL(4, 89, 0), // ""
QT_MOC_LITERAL(5, 90, 12), // "NewTagsAdded"
QT_MOC_LITERAL(6, 103, 4), // "tags"
QT_MOC_LITERAL(7, 108, 11), // "TagsDeleted"
QT_MOC_LITERAL(8, 120, 16), // "TagsColorChanged"
QT_MOC_LITERAL(9, 137, 9), // "oldAndNew"
QT_MOC_LITERAL(10, 147, 15), // "TagsNameChanged"
QT_MOC_LITERAL(11, 163, 11), // "FilesTagged"
QT_MOC_LITERAL(12, 175, 11), // "fileAndTags"
QT_MOC_LITERAL(13, 187, 13), // "FilesUntagged"
QT_MOC_LITERAL(14, 201, 5), // "Query"
QT_MOC_LITERAL(15, 207, 12), // "QDBusVariant"
QT_MOC_LITERAL(16, 220, 3), // "opt"
QT_MOC_LITERAL(17, 224, 5), // "value"
QT_MOC_LITERAL(18, 230, 6), // "Insert"
QT_MOC_LITERAL(19, 237, 6), // "Delete"
QT_MOC_LITERAL(20, 244, 6) // "Update"

    },
    "TagManagerDBus\0D-Bus Interface\0"
    "org.deepin.Filemanager.Daemon.TagManager\0"
    "TagsServiceReady\0\0NewTagsAdded\0tags\0"
    "TagsDeleted\0TagsColorChanged\0oldAndNew\0"
    "TagsNameChanged\0FilesTagged\0fileAndTags\0"
    "FilesUntagged\0Query\0QDBusVariant\0opt\0"
    "value\0Insert\0Delete\0Update"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TagManagerDBus[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      12,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   76,    4, 0x06 /* Public */,
       5,    1,   77,    4, 0x06 /* Public */,
       7,    1,   80,    4, 0x06 /* Public */,
       8,    1,   83,    4, 0x06 /* Public */,
      10,    1,   86,    4, 0x06 /* Public */,
      11,    1,   89,    4, 0x06 /* Public */,
      13,    1,   92,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    2,   95,    4, 0x0a /* Public */,
      14,    1,  100,    4, 0x2a /* Public | MethodCloned */,
      18,    2,  103,    4, 0x0a /* Public */,
      19,    2,  108,    4, 0x0a /* Public */,
      20,    2,  113,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QVariantMap,    6,
    QMetaType::Void, QMetaType::QStringList,    6,
    QMetaType::Void, QMetaType::QVariantMap,    9,
    QMetaType::Void, QMetaType::QVariantMap,    9,
    QMetaType::Void, QMetaType::QVariantMap,   12,
    QMetaType::Void, QMetaType::QVariantMap,   12,

 // slots: parameters
    0x80000000 | 15, QMetaType::Int, QMetaType::QStringList,   16,   17,
    0x80000000 | 15, QMetaType::Int,   16,
    QMetaType::Bool, QMetaType::Int, QMetaType::QVariantMap,   16,   17,
    QMetaType::Bool, QMetaType::Int, QMetaType::QVariantMap,   16,   17,
    QMetaType::Bool, QMetaType::Int, QMetaType::QVariantMap,   16,   17,

       0        // eod
};

void TagManagerDBus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TagManagerDBus *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->TagsServiceReady(); break;
        case 1: _t->NewTagsAdded((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 2: _t->TagsDeleted((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 3: _t->TagsColorChanged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 4: _t->TagsNameChanged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 5: _t->FilesTagged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 6: _t->FilesUntagged((*reinterpret_cast< const QVariantMap(*)>(_a[1]))); break;
        case 7: { QDBusVariant _r = _t->Query((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusVariant*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusVariant _r = _t->Query((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusVariant*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->Insert((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->Delete((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->Update((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TagManagerDBus::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagManagerDBus::TagsServiceReady)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TagManagerDBus::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagManagerDBus::NewTagsAdded)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TagManagerDBus::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagManagerDBus::TagsDeleted)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TagManagerDBus::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagManagerDBus::TagsColorChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TagManagerDBus::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagManagerDBus::TagsNameChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (TagManagerDBus::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagManagerDBus::FilesTagged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (TagManagerDBus::*)(const QVariantMap & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TagManagerDBus::FilesUntagged)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TagManagerDBus::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TagManagerDBus.data,
    qt_meta_data_TagManagerDBus,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TagManagerDBus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagManagerDBus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TagManagerDBus.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TagManagerDBus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void TagManagerDBus::TagsServiceReady()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TagManagerDBus::NewTagsAdded(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void TagManagerDBus::TagsDeleted(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void TagManagerDBus::TagsColorChanged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void TagManagerDBus::TagsNameChanged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void TagManagerDBus::FilesTagged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void TagManagerDBus::FilesUntagged(const QVariantMap & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
