/****************************************************************************
** Meta object code from reading C++ file 'canvasviewbroker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-canvas/broker/canvasviewbroker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvasviewbroker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_canvas__CanvasViewBroker_t {
    QByteArrayData data[16];
    char stringdata0[161];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_canvas__CanvasViewBroker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_canvas__CanvasViewBroker_t qt_meta_stringdata_ddplugin_canvas__CanvasViewBroker = {
    {
QT_MOC_LITERAL(0, 0, 33), // "ddplugin_canvas::CanvasViewBr..."
QT_MOC_LITERAL(1, 34, 10), // "visualRect"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 3), // "idx"
QT_MOC_LITERAL(4, 50, 3), // "url"
QT_MOC_LITERAL(5, 54, 14), // "gridVisualRect"
QT_MOC_LITERAL(6, 69, 7), // "gridPos"
QT_MOC_LITERAL(7, 77, 9), // "viewPoint"
QT_MOC_LITERAL(8, 87, 8), // "gridSize"
QT_MOC_LITERAL(9, 96, 7), // "refresh"
QT_MOC_LITERAL(10, 104, 6), // "update"
QT_MOC_LITERAL(11, 111, 6), // "select"
QT_MOC_LITERAL(12, 118, 11), // "QList<QUrl>"
QT_MOC_LITERAL(13, 130, 4), // "urls"
QT_MOC_LITERAL(14, 135, 12), // "selectedUrls"
QT_MOC_LITERAL(15, 148, 12) // "fileOperator"

    },
    "ddplugin_canvas::CanvasViewBroker\0"
    "visualRect\0\0idx\0url\0gridVisualRect\0"
    "gridPos\0viewPoint\0gridSize\0refresh\0"
    "update\0select\0QList<QUrl>\0urls\0"
    "selectedUrls\0fileOperator"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_canvas__CanvasViewBroker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   59,    2, 0x0a /* Public */,
       5,    2,   64,    2, 0x0a /* Public */,
       6,    2,   69,    2, 0x0a /* Public */,
       8,    1,   74,    2, 0x0a /* Public */,
       9,    1,   77,    2, 0x0a /* Public */,
      10,    1,   80,    2, 0x0a /* Public */,
      11,    1,   83,    2, 0x0a /* Public */,
      14,    1,   86,    2, 0x0a /* Public */,
      15,    0,   89,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::QRect, QMetaType::Int, QMetaType::QUrl,    3,    4,
    QMetaType::QRect, QMetaType::Int, QMetaType::QPoint,    3,    6,
    QMetaType::QPoint, QMetaType::Int, QMetaType::QPoint,    3,    7,
    QMetaType::QSize, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, 0x80000000 | 12,   13,
    0x80000000 | 12, QMetaType::Int,    3,
    QMetaType::QObjectStar,

       0        // eod
};

void ddplugin_canvas::CanvasViewBroker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CanvasViewBroker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QRect _r = _t->visualRect((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QRect*>(_a[0]) = std::move(_r); }  break;
        case 1: { QRect _r = _t->gridVisualRect((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QPoint(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QRect*>(_a[0]) = std::move(_r); }  break;
        case 2: { QPoint _r = _t->gridPos((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QPoint(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QPoint*>(_a[0]) = std::move(_r); }  break;
        case 3: { QSize _r = _t->gridSize((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QSize*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->refresh((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->update((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->select((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 7: { QList<QUrl> _r = _t->selectedUrls((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QList<QUrl>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QObject* _r = _t->fileOperator();
            if (_a[0]) *reinterpret_cast< QObject**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_canvas::CanvasViewBroker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_canvas__CanvasViewBroker.data,
    qt_meta_data_ddplugin_canvas__CanvasViewBroker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_canvas::CanvasViewBroker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_canvas::CanvasViewBroker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_canvas__CanvasViewBroker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_canvas::CanvasViewBroker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
