/****************************************************************************
** Meta object code from reading C++ file 'eventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-encrypt-manager/events/eventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'eventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_encrypt_manager__EventReceiver_t {
    QByteArrayData data[27];
    char stringdata0[356];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_encrypt_manager__EventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_encrypt_manager__EventReceiver_t qt_meta_stringdata_dfmplugin_encrypt_manager__EventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 40), // "dfmplugin_encrypt_manager::Ev..."
QT_MOC_LITERAL(1, 41, 14), // "tpmIsAvailable"
QT_MOC_LITERAL(2, 56, 0), // ""
QT_MOC_LITERAL(3, 57, 14), // "getRandomByTpm"
QT_MOC_LITERAL(4, 72, 4), // "size"
QT_MOC_LITERAL(5, 77, 8), // "QString*"
QT_MOC_LITERAL(6, 86, 6), // "output"
QT_MOC_LITERAL(7, 93, 16), // "isTpmSupportAlgo"
QT_MOC_LITERAL(8, 110, 8), // "algoName"
QT_MOC_LITERAL(9, 119, 5), // "bool*"
QT_MOC_LITERAL(10, 125, 7), // "support"
QT_MOC_LITERAL(11, 133, 12), // "encrypyByTpm"
QT_MOC_LITERAL(12, 146, 8), // "hashAlgo"
QT_MOC_LITERAL(13, 155, 7), // "keyAlgo"
QT_MOC_LITERAL(14, 163, 6), // "keyPin"
QT_MOC_LITERAL(15, 170, 8), // "password"
QT_MOC_LITERAL(16, 179, 7), // "dirPath"
QT_MOC_LITERAL(17, 187, 12), // "decryptByTpm"
QT_MOC_LITERAL(18, 200, 3), // "pwd"
QT_MOC_LITERAL(19, 204, 21), // "tpmIsAvailableProcess"
QT_MOC_LITERAL(20, 226, 21), // "getRandomByTpmProcess"
QT_MOC_LITERAL(21, 248, 23), // "isTpmSupportAlgoProcess"
QT_MOC_LITERAL(22, 272, 19), // "encryptByTpmProcess"
QT_MOC_LITERAL(23, 292, 13), // "encryptParams"
QT_MOC_LITERAL(24, 306, 19), // "decryptByTpmProcess"
QT_MOC_LITERAL(25, 326, 13), // "decryptParams"
QT_MOC_LITERAL(26, 340, 15) // "ownerAuthStatus"

    },
    "dfmplugin_encrypt_manager::EventReceiver\0"
    "tpmIsAvailable\0\0getRandomByTpm\0size\0"
    "QString*\0output\0isTpmSupportAlgo\0"
    "algoName\0bool*\0support\0encrypyByTpm\0"
    "hashAlgo\0keyAlgo\0keyPin\0password\0"
    "dirPath\0decryptByTpm\0pwd\0tpmIsAvailableProcess\0"
    "getRandomByTpmProcess\0isTpmSupportAlgoProcess\0"
    "encryptByTpmProcess\0encryptParams\0"
    "decryptByTpmProcess\0decryptParams\0"
    "ownerAuthStatus"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_encrypt_manager__EventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x0a /* Public */,
       3,    2,   70,    2, 0x0a /* Public */,
       7,    2,   75,    2, 0x0a /* Public */,
      11,    5,   80,    2, 0x0a /* Public */,
      17,    3,   91,    2, 0x0a /* Public */,
      19,    0,   98,    2, 0x0a /* Public */,
      20,    2,   99,    2, 0x0a /* Public */,
      21,    2,  104,    2, 0x0a /* Public */,
      22,    1,  109,    2, 0x0a /* Public */,
      24,    2,  112,    2, 0x0a /* Public */,
      26,    0,  117,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Bool,
    QMetaType::Bool, QMetaType::Int, 0x80000000 | 5,    4,    6,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 9,    8,   10,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,   12,   13,   14,   15,   16,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, 0x80000000 | 5,   14,   16,   18,
    QMetaType::Int,
    QMetaType::Int, QMetaType::Int, 0x80000000 | 5,    4,    6,
    QMetaType::Int, QMetaType::QString, 0x80000000 | 9,    8,   10,
    QMetaType::Int, QMetaType::QVariantMap,   23,
    QMetaType::Int, QMetaType::QVariantMap, 0x80000000 | 5,   25,   18,
    QMetaType::Int,

       0        // eod
};

void dfmplugin_encrypt_manager::EventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->tpmIsAvailable();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->getRandomByTpm((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QString*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->isTpmSupportAlgo((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->encrypyByTpm((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< const QString(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->decryptByTpm((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< QString*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { int _r = _t->tpmIsAvailableProcess();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 6: { int _r = _t->getRandomByTpmProcess((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QString*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 7: { int _r = _t->isTpmSupportAlgoProcess((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 8: { int _r = _t->encryptByTpmProcess((*reinterpret_cast< const QVariantMap(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 9: { int _r = _t->decryptByTpmProcess((*reinterpret_cast< const QVariantMap(*)>(_a[1])),(*reinterpret_cast< QString*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 10: { int _r = _t->ownerAuthStatus();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_encrypt_manager::EventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_encrypt_manager__EventReceiver.data,
    qt_meta_data_dfmplugin_encrypt_manager__EventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_encrypt_manager::EventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_encrypt_manager::EventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_encrypt_manager__EventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_encrypt_manager::EventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
