// SPDX-FileCopyrightText: 2022 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "bookmarkmenuscene_p.h"

using namespace dfmplugin_bookmark;
DFMBASE_USE_NAMESPACE

BookmarkMenuScenePrivate::BookmarkMenuScenePrivate(AbstractMenuScene *qq)
    : AbstractMenuScenePrivate(qq)
{
}
