/****************************************************************************
** Meta object code from reading C++ file 'appendcompresseventreceiver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-utils/appendcompress/appendcompresseventreceiver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'appendcompresseventreceiver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_utils__AppendCompressEventReceiver_t {
    QByteArrayData data[21];
    char stringdata0[335];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_utils__AppendCompressEventReceiver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_utils__AppendCompressEventReceiver_t qt_meta_stringdata_dfmplugin_utils__AppendCompressEventReceiver = {
    {
QT_MOC_LITERAL(0, 0, 44), // "dfmplugin_utils::AppendCompre..."
QT_MOC_LITERAL(1, 45, 19), // "handleSetMouseStyle"
QT_MOC_LITERAL(2, 65, 0), // ""
QT_MOC_LITERAL(3, 66, 11), // "QList<QUrl>"
QT_MOC_LITERAL(4, 78, 8), // "fromUrls"
QT_MOC_LITERAL(5, 87, 5), // "toUrl"
QT_MOC_LITERAL(6, 93, 15), // "Qt::DropAction*"
QT_MOC_LITERAL(7, 109, 4), // "type"
QT_MOC_LITERAL(8, 114, 22), // "handleDragDropCompress"
QT_MOC_LITERAL(9, 137, 28), // "handleSetMouseStyleOnDesktop"
QT_MOC_LITERAL(10, 166, 9), // "viewIndex"
QT_MOC_LITERAL(11, 176, 16), // "const QMimeData*"
QT_MOC_LITERAL(12, 193, 4), // "mime"
QT_MOC_LITERAL(13, 198, 7), // "viewPos"
QT_MOC_LITERAL(14, 206, 7), // "extData"
QT_MOC_LITERAL(15, 214, 31), // "handleDragDropCompressOnDesktop"
QT_MOC_LITERAL(16, 246, 2), // "md"
QT_MOC_LITERAL(17, 249, 30), // "handleSetMouseStyleOnOrganizer"
QT_MOC_LITERAL(18, 280, 6), // "viewId"
QT_MOC_LITERAL(19, 287, 34), // "handleDragDropCompressOnOsrga..."
QT_MOC_LITERAL(20, 322, 12) // "handleIsDrop"

    },
    "dfmplugin_utils::AppendCompressEventReceiver\0"
    "handleSetMouseStyle\0\0QList<QUrl>\0"
    "fromUrls\0toUrl\0Qt::DropAction*\0type\0"
    "handleDragDropCompress\0"
    "handleSetMouseStyleOnDesktop\0viewIndex\0"
    "const QMimeData*\0mime\0viewPos\0extData\0"
    "handleDragDropCompressOnDesktop\0md\0"
    "handleSetMouseStyleOnOrganizer\0viewId\0"
    "handleDragDropCompressOnOsrganizer\0"
    "handleIsDrop"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_utils__AppendCompressEventReceiver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   49,    2, 0x0a /* Public */,
       8,    2,   56,    2, 0x0a /* Public */,
       9,    4,   61,    2, 0x0a /* Public */,
      15,    4,   70,    2, 0x0a /* Public */,
      17,    4,   79,    2, 0x0a /* Public */,
      19,    4,   88,    2, 0x0a /* Public */,
      20,    1,   97,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Bool, 0x80000000 | 3, QMetaType::QUrl, 0x80000000 | 6,    4,    5,    7,
    QMetaType::Bool, 0x80000000 | 3, QMetaType::QUrl,    4,    5,
    QMetaType::Bool, QMetaType::Int, 0x80000000 | 11, QMetaType::QPoint, QMetaType::VoidStar,   10,   12,   13,   14,
    QMetaType::Bool, QMetaType::Int, 0x80000000 | 11, QMetaType::QPoint, QMetaType::VoidStar,   10,   16,   13,   14,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 11, QMetaType::QPoint, QMetaType::VoidStar,   18,   12,   13,   14,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 11, QMetaType::QPoint, QMetaType::VoidStar,   18,   16,   13,   14,
    QMetaType::Bool, QMetaType::QUrl,    5,

       0        // eod
};

void dfmplugin_utils::AppendCompressEventReceiver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AppendCompressEventReceiver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->handleSetMouseStyle((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< Qt::DropAction*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->handleDragDropCompress((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->handleSetMouseStyleOnDesktop((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QMimeData*(*)>(_a[2])),(*reinterpret_cast< const QPoint(*)>(_a[3])),(*reinterpret_cast< void*(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->handleDragDropCompressOnDesktop((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QMimeData*(*)>(_a[2])),(*reinterpret_cast< const QPoint(*)>(_a[3])),(*reinterpret_cast< void*(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->handleSetMouseStyleOnOrganizer((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QMimeData*(*)>(_a[2])),(*reinterpret_cast< const QPoint(*)>(_a[3])),(*reinterpret_cast< void*(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->handleDragDropCompressOnOsrganizer((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QMimeData*(*)>(_a[2])),(*reinterpret_cast< const QPoint(*)>(_a[3])),(*reinterpret_cast< void*(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->handleIsDrop((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_utils::AppendCompressEventReceiver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_utils__AppendCompressEventReceiver.data,
    qt_meta_data_dfmplugin_utils__AppendCompressEventReceiver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_utils::AppendCompressEventReceiver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_utils::AppendCompressEventReceiver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_utils__AppendCompressEventReceiver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_utils::AppendCompressEventReceiver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
