/****************************************************************************
** Meta object code from reading C++ file 'dfmplugin_detailspace_global.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-detailspace/dfmplugin_detailspace_global.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dfmplugin_detailspace_global.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_detailspace_t {
    QByteArrayData data[24];
    char stringdata0[364];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_detailspace_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_detailspace_t qt_meta_stringdata_dfmplugin_detailspace = {
    {
QT_MOC_LITERAL(0, 0, 21), // "dfmplugin_detailspace"
QT_MOC_LITERAL(1, 22, 16), // "DetailFilterType"
QT_MOC_LITERAL(2, 39, 10), // "kNotFilter"
QT_MOC_LITERAL(3, 50, 10), // "kBasicView"
QT_MOC_LITERAL(4, 61, 9), // "kIconView"
QT_MOC_LITERAL(5, 71, 14), // "kFileNameField"
QT_MOC_LITERAL(6, 86, 14), // "kFileSizeField"
QT_MOC_LITERAL(7, 101, 18), // "kFileViewSizeField"
QT_MOC_LITERAL(8, 120, 18), // "kFileDurationField"
QT_MOC_LITERAL(9, 139, 14), // "kFileTypeField"
QT_MOC_LITERAL(10, 154, 23), // "kFileInterviewTimeField"
QT_MOC_LITERAL(11, 178, 20), // "kFileChangeTimeField"
QT_MOC_LITERAL(12, 199, 20), // "BasicFieldExpandEnum"
QT_MOC_LITERAL(13, 220, 7), // "kNotAll"
QT_MOC_LITERAL(14, 228, 9), // "kFileName"
QT_MOC_LITERAL(15, 238, 9), // "kFileSize"
QT_MOC_LITERAL(16, 248, 13), // "kFileViewSize"
QT_MOC_LITERAL(17, 262, 13), // "kFileDuration"
QT_MOC_LITERAL(18, 276, 9), // "kFileType"
QT_MOC_LITERAL(19, 286, 18), // "kFileInterviewTime"
QT_MOC_LITERAL(20, 305, 15), // "kFileChangeTIme"
QT_MOC_LITERAL(21, 321, 15), // "BasicExpandType"
QT_MOC_LITERAL(22, 337, 12), // "kFieldInsert"
QT_MOC_LITERAL(23, 350, 13) // "kFieldReplace"

    },
    "dfmplugin_detailspace\0DetailFilterType\0"
    "kNotFilter\0kBasicView\0kIconView\0"
    "kFileNameField\0kFileSizeField\0"
    "kFileViewSizeField\0kFileDurationField\0"
    "kFileTypeField\0kFileInterviewTimeField\0"
    "kFileChangeTimeField\0BasicFieldExpandEnum\0"
    "kNotAll\0kFileName\0kFileSize\0kFileViewSize\0"
    "kFileDuration\0kFileType\0kFileInterviewTime\0"
    "kFileChangeTIme\0BasicExpandType\0"
    "kFieldInsert\0kFieldReplace"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_detailspace[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       3,   14, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // enums: name, alias, flags, count, data
       1,    1, 0x0,   10,   29,
      12,   12, 0x0,    8,   49,
      21,   21, 0x0,    2,   65,

 // enum data: key, value
       2, uint(dfmplugin_detailspace::kNotFilter),
       3, uint(dfmplugin_detailspace::kBasicView),
       4, uint(dfmplugin_detailspace::kIconView),
       5, uint(dfmplugin_detailspace::kFileNameField),
       6, uint(dfmplugin_detailspace::kFileSizeField),
       7, uint(dfmplugin_detailspace::kFileViewSizeField),
       8, uint(dfmplugin_detailspace::kFileDurationField),
       9, uint(dfmplugin_detailspace::kFileTypeField),
      10, uint(dfmplugin_detailspace::kFileInterviewTimeField),
      11, uint(dfmplugin_detailspace::kFileChangeTimeField),
      13, uint(dfmplugin_detailspace::kNotAll),
      14, uint(dfmplugin_detailspace::kFileName),
      15, uint(dfmplugin_detailspace::kFileSize),
      16, uint(dfmplugin_detailspace::kFileViewSize),
      17, uint(dfmplugin_detailspace::kFileDuration),
      18, uint(dfmplugin_detailspace::kFileType),
      19, uint(dfmplugin_detailspace::kFileInterviewTime),
      20, uint(dfmplugin_detailspace::kFileChangeTIme),
      22, uint(dfmplugin_detailspace::kFieldInsert),
      23, uint(dfmplugin_detailspace::kFieldReplace),

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject dfmplugin_detailspace::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_dfmplugin_detailspace.data,
    qt_meta_data_dfmplugin_detailspace,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
