/****************************************************************************
** Meta object code from reading C++ file 'canvasmodelbroker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-canvas/broker/canvasmodelbroker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvasmodelbroker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_canvas__CanvasModelBroker_t {
    QByteArrayData data[30];
    char stringdata0[268];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_canvas__CanvasModelBroker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_canvas__CanvasModelBroker_t qt_meta_stringdata_ddplugin_canvas__CanvasModelBroker = {
    {
QT_MOC_LITERAL(0, 0, 34), // "ddplugin_canvas::CanvasModelB..."
QT_MOC_LITERAL(1, 35, 7), // "rootUrl"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 8), // "urlIndex"
QT_MOC_LITERAL(4, 53, 11), // "QModelIndex"
QT_MOC_LITERAL(5, 65, 3), // "url"
QT_MOC_LITERAL(6, 69, 5), // "index"
QT_MOC_LITERAL(7, 75, 3), // "row"
QT_MOC_LITERAL(8, 79, 7), // "fileUrl"
QT_MOC_LITERAL(9, 87, 5), // "files"
QT_MOC_LITERAL(10, 93, 11), // "QList<QUrl>"
QT_MOC_LITERAL(11, 105, 15), // "showHiddenFiles"
QT_MOC_LITERAL(12, 121, 18), // "setShowHiddenFiles"
QT_MOC_LITERAL(13, 140, 4), // "show"
QT_MOC_LITERAL(14, 145, 9), // "sortOrder"
QT_MOC_LITERAL(15, 155, 12), // "setSortOrder"
QT_MOC_LITERAL(16, 168, 5), // "order"
QT_MOC_LITERAL(17, 174, 8), // "sortRole"
QT_MOC_LITERAL(18, 183, 11), // "setSortRole"
QT_MOC_LITERAL(19, 195, 4), // "role"
QT_MOC_LITERAL(20, 200, 8), // "rowCount"
QT_MOC_LITERAL(21, 209, 4), // "data"
QT_MOC_LITERAL(22, 214, 8), // "itemRole"
QT_MOC_LITERAL(23, 223, 4), // "sort"
QT_MOC_LITERAL(24, 228, 7), // "refresh"
QT_MOC_LITERAL(25, 236, 6), // "global"
QT_MOC_LITERAL(26, 243, 2), // "ms"
QT_MOC_LITERAL(27, 246, 10), // "updateFile"
QT_MOC_LITERAL(28, 257, 5), // "fetch"
QT_MOC_LITERAL(29, 263, 4) // "take"

    },
    "ddplugin_canvas::CanvasModelBroker\0"
    "rootUrl\0\0urlIndex\0QModelIndex\0url\0"
    "index\0row\0fileUrl\0files\0QList<QUrl>\0"
    "showHiddenFiles\0setShowHiddenFiles\0"
    "show\0sortOrder\0setSortOrder\0order\0"
    "sortRole\0setSortRole\0role\0rowCount\0"
    "data\0itemRole\0sort\0refresh\0global\0ms\0"
    "updateFile\0fetch\0take"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_canvas__CanvasModelBroker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  104,    2, 0x0a /* Public */,
       3,    1,  105,    2, 0x0a /* Public */,
       6,    1,  108,    2, 0x0a /* Public */,
       8,    1,  111,    2, 0x0a /* Public */,
       9,    0,  114,    2, 0x0a /* Public */,
      11,    0,  115,    2, 0x0a /* Public */,
      12,    1,  116,    2, 0x0a /* Public */,
      14,    0,  119,    2, 0x0a /* Public */,
      15,    1,  120,    2, 0x0a /* Public */,
      17,    0,  123,    2, 0x0a /* Public */,
      18,    2,  124,    2, 0x0a /* Public */,
      20,    0,  129,    2, 0x0a /* Public */,
      21,    2,  130,    2, 0x0a /* Public */,
      23,    0,  135,    2, 0x0a /* Public */,
      24,    3,  136,    2, 0x0a /* Public */,
      24,    2,  143,    2, 0x2a /* Public | MethodCloned */,
      28,    1,  148,    2, 0x0a /* Public */,
      29,    1,  151,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::QUrl,
    0x80000000 | 4, QMetaType::QUrl,    5,
    0x80000000 | 4, QMetaType::Int,    7,
    QMetaType::QUrl, 0x80000000 | 4,    6,
    0x80000000 | 10,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   19,   16,
    QMetaType::Int,
    QMetaType::QVariant, QMetaType::QUrl, QMetaType::Int,    5,   22,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, QMetaType::Int, QMetaType::Bool,   25,   26,   27,
    QMetaType::Void, QMetaType::Bool, QMetaType::Int,   25,   26,
    QMetaType::Bool, QMetaType::QUrl,    5,
    QMetaType::Bool, QMetaType::QUrl,    5,

       0        // eod
};

void ddplugin_canvas::CanvasModelBroker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CanvasModelBroker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QUrl _r = _t->rootUrl();
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = std::move(_r); }  break;
        case 1: { QModelIndex _r = _t->urlIndex((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 2: { QModelIndex _r = _t->index((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 3: { QUrl _r = _t->fileUrl((*reinterpret_cast< const QModelIndex(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = std::move(_r); }  break;
        case 4: { QList<QUrl> _r = _t->files();
            if (_a[0]) *reinterpret_cast< QList<QUrl>*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->showHiddenFiles();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->setShowHiddenFiles((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: { int _r = _t->sortOrder();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->setSortOrder((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: { int _r = _t->sortRole();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->setSortRole((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 11: { int _r = _t->rowCount();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 12: { QVariant _r = _t->data((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->sort(); break;
        case 14: _t->refresh((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 15: _t->refresh((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 16: { bool _r = _t->fetch((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->take((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_canvas::CanvasModelBroker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_canvas__CanvasModelBroker.data,
    qt_meta_data_ddplugin_canvas__CanvasModelBroker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_canvas::CanvasModelBroker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_canvas::CanvasModelBroker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_canvas__CanvasModelBroker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_canvas::CanvasModelBroker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
