// SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include <QObject>

void errorCode_dfmio_trans()
{
    QObject::tr("File not found");
    QObject::tr("File already exists");
    QObject::tr("File is a directory");
    QObject::tr("File is not a directory");
    QObject::tr("File is a directory that isn't empty");
    QObject::tr("File is not a regular file");
    QObject::tr("File is not a symbolic link");
    QObject::tr("File cannot be mounted");
    QObject::tr("Filename has too many characters");
    QObject::tr("Filename is invalid or contains invalid characters");
    QObject::tr("File contains too many symbolic links");
    QObject::tr("No space left on drive");
    QObject::tr("Invalid argument");
    QObject::tr("Permission denied");
    QObject::tr("Operation (or one of its parameters) not supported");
    QObject::tr("File isn't mounted");
    QObject::tr("File is already mounted");
    QObject::tr("File was closed");
    QObject::tr("Operation was cancelled");
    QObject::tr("Operations are still pending");
    QObject::tr("File is read-only");
    QObject::tr("Operation timed out");
    QObject::tr("File is busy");
    QObject::tr("Failed to open the file");
    QObject::tr("Target Trash File Not exist");
}
