/****************************************************************************
** Meta object code from reading C++ file 'fileinfomodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-canvas/model/fileinfomodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileinfomodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_canvas__FileInfoModel_t {
    QByteArrayData data[22];
    char stringdata0[216];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_canvas__FileInfoModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_canvas__FileInfoModel_t qt_meta_stringdata_ddplugin_canvas__FileInfoModel = {
    {
QT_MOC_LITERAL(0, 0, 30), // "ddplugin_canvas::FileInfoModel"
QT_MOC_LITERAL(1, 31, 12), // "dataReplaced"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 6), // "oldUrl"
QT_MOC_LITERAL(4, 52, 6), // "newUrl"
QT_MOC_LITERAL(5, 59, 7), // "rootUrl"
QT_MOC_LITERAL(6, 67, 9), // "rootIndex"
QT_MOC_LITERAL(7, 77, 11), // "QModelIndex"
QT_MOC_LITERAL(8, 89, 5), // "index"
QT_MOC_LITERAL(9, 95, 3), // "url"
QT_MOC_LITERAL(10, 99, 6), // "column"
QT_MOC_LITERAL(11, 106, 8), // "fileInfo"
QT_MOC_LITERAL(12, 115, 15), // "FileInfoPointer"
QT_MOC_LITERAL(13, 131, 7), // "fileUrl"
QT_MOC_LITERAL(14, 139, 5), // "files"
QT_MOC_LITERAL(15, 145, 11), // "QList<QUrl>"
QT_MOC_LITERAL(16, 157, 7), // "refresh"
QT_MOC_LITERAL(17, 165, 6), // "parent"
QT_MOC_LITERAL(18, 172, 10), // "modelState"
QT_MOC_LITERAL(19, 183, 6), // "update"
QT_MOC_LITERAL(20, 190, 10), // "updateFile"
QT_MOC_LITERAL(21, 201, 14) // "refreshAllFile"

    },
    "ddplugin_canvas::FileInfoModel\0"
    "dataReplaced\0\0oldUrl\0newUrl\0rootUrl\0"
    "rootIndex\0QModelIndex\0index\0url\0column\0"
    "fileInfo\0FileInfoPointer\0fileUrl\0files\0"
    "QList<QUrl>\0refresh\0parent\0modelState\0"
    "update\0updateFile\0refreshAllFile"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_canvas__FileInfoModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   79,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       5,    0,   84,    2, 0x02 /* Public */,
       6,    0,   85,    2, 0x02 /* Public */,
       8,    2,   86,    2, 0x02 /* Public */,
       8,    1,   91,    2, 0x22 /* Public | MethodCloned */,
      11,    1,   94,    2, 0x02 /* Public */,
      13,    1,   97,    2, 0x02 /* Public */,
      14,    0,  100,    2, 0x02 /* Public */,
      16,    1,  101,    2, 0x02 /* Public */,
      18,    0,  104,    2, 0x02 /* Public */,
      19,    0,  105,    2, 0x02 /* Public */,
      20,    1,  106,    2, 0x02 /* Public */,
      21,    0,  109,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,    3,    4,

 // methods: parameters
    QMetaType::QUrl,
    0x80000000 | 7,
    0x80000000 | 7, QMetaType::QUrl, QMetaType::Int,    9,   10,
    0x80000000 | 7, QMetaType::QUrl,    9,
    0x80000000 | 12, 0x80000000 | 7,    8,
    QMetaType::QUrl, 0x80000000 | 7,    8,
    0x80000000 | 15,
    QMetaType::Void, 0x80000000 | 7,   17,
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,    9,
    QMetaType::Void,

       0        // eod
};

void ddplugin_canvas::FileInfoModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileInfoModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->dataReplaced((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 1: { QUrl _r = _t->rootUrl();
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = std::move(_r); }  break;
        case 2: { QModelIndex _r = _t->rootIndex();
            if (_a[0]) *reinterpret_cast< QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 3: { QModelIndex _r = _t->index((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 4: { QModelIndex _r = _t->index((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 5: { FileInfoPointer _r = _t->fileInfo((*reinterpret_cast< const QModelIndex(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< FileInfoPointer*>(_a[0]) = std::move(_r); }  break;
        case 6: { QUrl _r = _t->fileUrl((*reinterpret_cast< const QModelIndex(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = std::move(_r); }  break;
        case 7: { QList<QUrl> _r = _t->files();
            if (_a[0]) *reinterpret_cast< QList<QUrl>*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->refresh((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 9: { int _r = _t->modelState();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->update(); break;
        case 11: _t->updateFile((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 12: _t->refreshAllFile(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileInfoModel::*)(const QUrl & , const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileInfoModel::dataReplaced)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_canvas::FileInfoModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_canvas__FileInfoModel.data,
    qt_meta_data_ddplugin_canvas__FileInfoModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_canvas::FileInfoModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_canvas::FileInfoModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_canvas__FileInfoModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int ddplugin_canvas::FileInfoModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void ddplugin_canvas::FileInfoModel::dataReplaced(const QUrl & _t1, const QUrl & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
