/****************************************************************************
** Meta object code from reading C++ file 'textindexclient.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-search/utils/textindexclient.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textindexclient.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_search__TextIndexClient_t {
    QByteArrayData data[21];
    char stringdata0[238];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_search__TextIndexClient_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_search__TextIndexClient_t qt_meta_stringdata_dfmplugin_search__TextIndexClient = {
    {
QT_MOC_LITERAL(0, 0, 33), // "dfmplugin_search::TextIndexCl..."
QT_MOC_LITERAL(1, 34, 11), // "taskStarted"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 8), // "TaskType"
QT_MOC_LITERAL(4, 56, 4), // "type"
QT_MOC_LITERAL(5, 61, 4), // "path"
QT_MOC_LITERAL(6, 66, 12), // "taskFinished"
QT_MOC_LITERAL(7, 79, 7), // "success"
QT_MOC_LITERAL(8, 87, 10), // "taskFailed"
QT_MOC_LITERAL(9, 98, 5), // "error"
QT_MOC_LITERAL(10, 104, 19), // "taskProgressChanged"
QT_MOC_LITERAL(11, 124, 5), // "count"
QT_MOC_LITERAL(12, 130, 18), // "onDBusTaskFinished"
QT_MOC_LITERAL(13, 149, 25), // "onDBusTaskProgressChanged"
QT_MOC_LITERAL(14, 175, 6), // "Create"
QT_MOC_LITERAL(15, 182, 6), // "Update"
QT_MOC_LITERAL(16, 189, 6), // "Remove"
QT_MOC_LITERAL(17, 196, 13), // "ServiceStatus"
QT_MOC_LITERAL(18, 210, 9), // "Available"
QT_MOC_LITERAL(19, 220, 11), // "Unavailable"
QT_MOC_LITERAL(20, 232, 5) // "Error"

    },
    "dfmplugin_search::TextIndexClient\0"
    "taskStarted\0\0TaskType\0type\0path\0"
    "taskFinished\0success\0taskFailed\0error\0"
    "taskProgressChanged\0count\0onDBusTaskFinished\0"
    "onDBusTaskProgressChanged\0Create\0"
    "Update\0Remove\0ServiceStatus\0Available\0"
    "Unavailable\0Error"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_search__TextIndexClient[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       2,   84, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   44,    2, 0x06 /* Public */,
       6,    3,   49,    2, 0x06 /* Public */,
       8,    3,   56,    2, 0x06 /* Public */,
      10,    3,   63,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    3,   70,    2, 0x08 /* Private */,
      13,    3,   77,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString,    4,    5,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString, QMetaType::Bool,    4,    5,    7,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString, QMetaType::QString,    4,    5,    9,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString, QMetaType::LongLong,    4,    5,   11,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Bool,    4,    5,    7,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::LongLong,    4,    5,   11,

 // enums: name, alias, flags, count, data
       3,    3, 0x2,    3,   94,
      17,   17, 0x2,    3,  100,

 // enum data: key, value
      14, uint(dfmplugin_search::TextIndexClient::TaskType::Create),
      15, uint(dfmplugin_search::TextIndexClient::TaskType::Update),
      16, uint(dfmplugin_search::TextIndexClient::TaskType::Remove),
      18, uint(dfmplugin_search::TextIndexClient::ServiceStatus::Available),
      19, uint(dfmplugin_search::TextIndexClient::ServiceStatus::Unavailable),
      20, uint(dfmplugin_search::TextIndexClient::ServiceStatus::Error),

       0        // eod
};

void dfmplugin_search::TextIndexClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TextIndexClient *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->taskStarted((*reinterpret_cast< TaskType(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->taskFinished((*reinterpret_cast< TaskType(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 2: _t->taskFailed((*reinterpret_cast< TaskType(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 3: _t->taskProgressChanged((*reinterpret_cast< TaskType(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< qlonglong(*)>(_a[3]))); break;
        case 4: _t->onDBusTaskFinished((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 5: _t->onDBusTaskProgressChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< qlonglong(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TextIndexClient::*)(TaskType , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextIndexClient::taskStarted)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TextIndexClient::*)(TaskType , const QString & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextIndexClient::taskFinished)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TextIndexClient::*)(TaskType , const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextIndexClient::taskFailed)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TextIndexClient::*)(TaskType , const QString & , qlonglong );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextIndexClient::taskProgressChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_search::TextIndexClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_search__TextIndexClient.data,
    qt_meta_data_dfmplugin_search__TextIndexClient,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_search::TextIndexClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_search::TextIndexClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_search__TextIndexClient.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_search::TextIndexClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_search::TextIndexClient::taskStarted(TaskType _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_search::TextIndexClient::taskFinished(TaskType _t1, const QString & _t2, bool _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_search::TextIndexClient::taskFailed(TaskType _t1, const QString & _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_search::TextIndexClient::taskProgressChanged(TaskType _t1, const QString & _t2, qlonglong _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
