/****************************************************************************
** Meta object code from reading C++ file 'canvasview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-canvas/view/canvasview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvasview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_canvas__CanvasView_t {
    QByteArrayData data[21];
    char stringdata0[214];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_canvas__CanvasView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_canvas__CanvasView_t qt_meta_stringdata_ddplugin_canvas__CanvasView = {
    {
QT_MOC_LITERAL(0, 0, 27), // "ddplugin_canvas::CanvasView"
QT_MOC_LITERAL(1, 28, 5), // "reset"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 4), // "edit"
QT_MOC_LITERAL(4, 40, 11), // "QModelIndex"
QT_MOC_LITERAL(5, 52, 5), // "index"
QT_MOC_LITERAL(6, 58, 11), // "EditTrigger"
QT_MOC_LITERAL(7, 70, 7), // "trigger"
QT_MOC_LITERAL(8, 78, 7), // "QEvent*"
QT_MOC_LITERAL(9, 86, 5), // "event"
QT_MOC_LITERAL(10, 92, 7), // "refresh"
QT_MOC_LITERAL(11, 100, 6), // "silent"
QT_MOC_LITERAL(12, 107, 9), // "selectAll"
QT_MOC_LITERAL(13, 117, 12), // "toggleSelect"
QT_MOC_LITERAL(14, 130, 14), // "currentChanged"
QT_MOC_LITERAL(15, 145, 7), // "current"
QT_MOC_LITERAL(16, 153, 8), // "previous"
QT_MOC_LITERAL(17, 162, 16), // "selectionChanged"
QT_MOC_LITERAL(18, 179, 14), // "QItemSelection"
QT_MOC_LITERAL(19, 194, 8), // "selected"
QT_MOC_LITERAL(20, 203, 10) // "deselected"

    },
    "ddplugin_canvas::CanvasView\0reset\0\0"
    "edit\0QModelIndex\0index\0EditTrigger\0"
    "trigger\0QEvent*\0event\0refresh\0silent\0"
    "selectAll\0toggleSelect\0currentChanged\0"
    "current\0previous\0selectionChanged\0"
    "QItemSelection\0selected\0deselected"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_canvas__CanvasView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x0a /* Public */,
       3,    3,   50,    2, 0x0a /* Public */,
      10,    1,   57,    2, 0x0a /* Public */,
      12,    0,   60,    2, 0x0a /* Public */,
      13,    0,   61,    2, 0x0a /* Public */,
      14,    2,   62,    2, 0x09 /* Protected */,
      17,    2,   67,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Bool, 0x80000000 | 4, 0x80000000 | 6, 0x80000000 | 8,    5,    7,    9,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4, 0x80000000 | 4,   15,   16,
    QMetaType::Void, 0x80000000 | 18, 0x80000000 | 18,   19,   20,

       0        // eod
};

void ddplugin_canvas::CanvasView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CanvasView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->reset(); break;
        case 1: { bool _r = _t->edit((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< EditTrigger(*)>(_a[2])),(*reinterpret_cast< QEvent*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->refresh((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->selectAll(); break;
        case 4: _t->toggleSelect(); break;
        case 5: _t->currentChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 6: _t->selectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_canvas::CanvasView::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemView::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_canvas__CanvasView.data,
    qt_meta_data_ddplugin_canvas__CanvasView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_canvas::CanvasView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_canvas::CanvasView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_canvas__CanvasView.stringdata0))
        return static_cast<void*>(this);
    return QAbstractItemView::qt_metacast(_clname);
}

int ddplugin_canvas::CanvasView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
