/****************************************************************************
** Meta object code from reading C++ file 'wallpapersettings_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-wallpapersetting/private/wallpapersettings_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wallpapersettings_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperSettingsPrivate_t {
    QByteArrayData data[28];
    char stringdata0[377];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperSettingsPrivate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperSettingsPrivate_t qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperSettingsPrivate = {
    {
QT_MOC_LITERAL(0, 0, 51), // "ddplugin_wallpapersetting::Wa..."
QT_MOC_LITERAL(1, 52, 12), // "carouselTurn"
QT_MOC_LITERAL(2, 65, 0), // ""
QT_MOC_LITERAL(3, 66, 7), // "checked"
QT_MOC_LITERAL(4, 74, 14), // "switchCarousel"
QT_MOC_LITERAL(5, 89, 16), // "QAbstractButton*"
QT_MOC_LITERAL(6, 106, 10), // "toggledBtn"
QT_MOC_LITERAL(7, 117, 5), // "state"
QT_MOC_LITERAL(8, 123, 14), // "switchWaitTime"
QT_MOC_LITERAL(9, 138, 7), // "setMode"
QT_MOC_LITERAL(10, 146, 2), // "on"
QT_MOC_LITERAL(11, 149, 21), // "onListBackgroundReply"
QT_MOC_LITERAL(12, 171, 24), // "QDBusPendingCallWatcher*"
QT_MOC_LITERAL(13, 196, 5), // "watch"
QT_MOC_LITERAL(14, 202, 13), // "onItemPressed"
QT_MOC_LITERAL(15, 216, 8), // "itemData"
QT_MOC_LITERAL(16, 225, 19), // "onItemButtonClicked"
QT_MOC_LITERAL(17, 245, 14), // "WallpaperItem*"
QT_MOC_LITERAL(18, 260, 4), // "item"
QT_MOC_LITERAL(19, 265, 2), // "id"
QT_MOC_LITERAL(20, 268, 9), // "onItemTab"
QT_MOC_LITERAL(21, 278, 13), // "onItemBacktab"
QT_MOC_LITERAL(22, 292, 21), // "handleNeedCloseButton"
QT_MOC_LITERAL(23, 314, 3), // "pos"
QT_MOC_LITERAL(24, 318, 20), // "onCloseButtonClicked"
QT_MOC_LITERAL(25, 339, 14), // "onMousePressed"
QT_MOC_LITERAL(26, 354, 6), // "button"
QT_MOC_LITERAL(27, 361, 15) // "onScreenChanged"

    },
    "ddplugin_wallpapersetting::WallpaperSettingsPrivate\0"
    "carouselTurn\0\0checked\0switchCarousel\0"
    "QAbstractButton*\0toggledBtn\0state\0"
    "switchWaitTime\0setMode\0on\0"
    "onListBackgroundReply\0QDBusPendingCallWatcher*\0"
    "watch\0onItemPressed\0itemData\0"
    "onItemButtonClicked\0WallpaperItem*\0"
    "item\0id\0onItemTab\0onItemBacktab\0"
    "handleNeedCloseButton\0pos\0"
    "onCloseButtonClicked\0onMousePressed\0"
    "button\0onScreenChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_wallpapersetting__WallpaperSettingsPrivate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x0a /* Public */,
       4,    2,   82,    2, 0x0a /* Public */,
       8,    2,   87,    2, 0x0a /* Public */,
       9,    2,   92,    2, 0x0a /* Public */,
      11,    1,   97,    2, 0x0a /* Public */,
      14,    1,  100,    2, 0x0a /* Public */,
      16,    2,  103,    2, 0x0a /* Public */,
      20,    1,  108,    2, 0x0a /* Public */,
      21,    1,  111,    2, 0x0a /* Public */,
      22,    2,  114,    2, 0x0a /* Public */,
      24,    0,  119,    2, 0x0a /* Public */,
      25,    2,  120,    2, 0x0a /* Public */,
      27,    0,  125,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, 0x80000000 | 5, QMetaType::Bool,    6,    7,
    QMetaType::Void, 0x80000000 | 5, QMetaType::Bool,    6,    7,
    QMetaType::Void, 0x80000000 | 5, QMetaType::Bool,    6,   10,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, QMetaType::QString,   15,
    QMetaType::Void, 0x80000000 | 17, QMetaType::QString,   18,   19,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void, QMetaType::QString, QMetaType::QPoint,   15,   23,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint, QMetaType::Int,   23,   26,
    QMetaType::Void,

       0        // eod
};

void ddplugin_wallpapersetting::WallpaperSettingsPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<WallpaperSettingsPrivate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->carouselTurn((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->switchCarousel((*reinterpret_cast< QAbstractButton*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->switchWaitTime((*reinterpret_cast< QAbstractButton*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: _t->setMode((*reinterpret_cast< QAbstractButton*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 4: _t->onListBackgroundReply((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        case 5: _t->onItemPressed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->onItemButtonClicked((*reinterpret_cast< WallpaperItem*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 7: _t->onItemTab((*reinterpret_cast< WallpaperItem*(*)>(_a[1]))); break;
        case 8: _t->onItemBacktab((*reinterpret_cast< WallpaperItem*(*)>(_a[1]))); break;
        case 9: _t->handleNeedCloseButton((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QPoint(*)>(_a[2]))); break;
        case 10: _t->onCloseButtonClicked(); break;
        case 11: _t->onMousePressed((*reinterpret_cast< const QPoint(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 12: _t->onScreenChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractButton* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractButton* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractButton* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusPendingCallWatcher* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WallpaperItem* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WallpaperItem* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WallpaperItem* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_wallpapersetting::WallpaperSettingsPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperSettingsPrivate.data,
    qt_meta_data_ddplugin_wallpapersetting__WallpaperSettingsPrivate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_wallpapersetting::WallpaperSettingsPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_wallpapersetting::WallpaperSettingsPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_wallpapersetting__WallpaperSettingsPrivate.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_wallpapersetting::WallpaperSettingsPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
