/****************************************************************************
** Meta object code from reading C++ file 'computerview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-computer/views/computerview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'computerview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_computer__ComputerView_t {
    QByteArrayData data[20];
    char stringdata0[284];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_computer__ComputerView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_computer__ComputerView_t qt_meta_stringdata_dfmplugin_computer__ComputerView = {
    {
QT_MOC_LITERAL(0, 0, 32), // "dfmplugin_computer::ComputerView"
QT_MOC_LITERAL(1, 33, 12), // "enterPressed"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 11), // "QModelIndex"
QT_MOC_LITERAL(4, 59, 5), // "index"
QT_MOC_LITERAL(5, 65, 25), // "handleComputerItemVisible"
QT_MOC_LITERAL(6, 91, 4), // "cdTo"
QT_MOC_LITERAL(7, 96, 13), // "onMenuRequest"
QT_MOC_LITERAL(8, 110, 3), // "pos"
QT_MOC_LITERAL(9, 114, 15), // "onRenameRequest"
QT_MOC_LITERAL(10, 130, 5), // "winId"
QT_MOC_LITERAL(11, 136, 3), // "url"
QT_MOC_LITERAL(12, 140, 18), // "handleDisksVisible"
QT_MOC_LITERAL(13, 159, 20), // "handleUserDirVisible"
QT_MOC_LITERAL(14, 180, 23), // "handle3rdEntriesVisible"
QT_MOC_LITERAL(15, 204, 25), // "handleDiskSplitterVisible"
QT_MOC_LITERAL(16, 230, 18), // "onSelectionChanged"
QT_MOC_LITERAL(17, 249, 14), // "QItemSelection"
QT_MOC_LITERAL(18, 264, 8), // "selected"
QT_MOC_LITERAL(19, 273, 10) // "deselected"

    },
    "dfmplugin_computer::ComputerView\0"
    "enterPressed\0\0QModelIndex\0index\0"
    "handleComputerItemVisible\0cdTo\0"
    "onMenuRequest\0pos\0onRenameRequest\0"
    "winId\0url\0handleDisksVisible\0"
    "handleUserDirVisible\0handle3rdEntriesVisible\0"
    "handleDiskSplitterVisible\0onSelectionChanged\0"
    "QItemSelection\0selected\0deselected"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_computer__ComputerView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   67,    2, 0x0a /* Public */,
       6,    1,   68,    2, 0x08 /* Private */,
       7,    1,   71,    2, 0x08 /* Private */,
       9,    2,   74,    2, 0x08 /* Private */,
      12,    0,   79,    2, 0x08 /* Private */,
      13,    0,   80,    2, 0x08 /* Private */,
      14,    0,   81,    2, 0x08 /* Private */,
      15,    0,   82,    2, 0x08 /* Private */,
      16,    2,   83,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QPoint,    8,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::QUrl,   10,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17, 0x80000000 | 17,   18,   19,

       0        // eod
};

void dfmplugin_computer::ComputerView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ComputerView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->enterPressed((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 1: _t->handleComputerItemVisible(); break;
        case 2: _t->cdTo((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 3: _t->onMenuRequest((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 4: _t->onRenameRequest((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        case 5: _t->handleDisksVisible(); break;
        case 6: _t->handleUserDirVisible(); break;
        case 7: _t->handle3rdEntriesVisible(); break;
        case 8: _t->handleDiskSplitterVisible(); break;
        case 9: _t->onSelectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ComputerView::*)(const QModelIndex & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComputerView::enterPressed)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_computer::ComputerView::staticMetaObject = { {
    QMetaObject::SuperData::link<Dtk::Widget::DListView::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_computer__ComputerView.data,
    qt_meta_data_dfmplugin_computer__ComputerView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_computer::ComputerView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_computer::ComputerView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_computer__ComputerView.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "dfmbase::AbstractBaseView"))
        return static_cast< dfmbase::AbstractBaseView*>(this);
    return Dtk::Widget::DListView::qt_metacast(_clname);
}

int dfmplugin_computer::ComputerView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Dtk::Widget::DListView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_computer::ComputerView::enterPressed(const QModelIndex & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
