/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a diskencryptadaptor -i diskencryptdbus.h -l DiskEncryptDBus /build/deepin-file-manager/src/dde-file-manager/assets/dbus/org.deepin.Filemanager.DiskEncrypt.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "diskencryptadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class DiskEncryptAdaptor
 */

DiskEncryptAdaptor::DiskEncryptAdaptor(DiskEncryptDBus *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

DiskEncryptAdaptor::~DiskEncryptAdaptor()
{
    // destructor
}

QString DiskEncryptAdaptor::ChangeEncryptPassphress(const QVariantMap &params)
{
    // handle method call org.deepin.Filemanager.DiskEncrypt.ChangeEncryptPassphress
    return parent()->ChangeEncryptPassphress(params);
}

QString DiskEncryptAdaptor::DecryptDisk(const QVariantMap &params)
{
    // handle method call org.deepin.Filemanager.DiskEncrypt.DecryptDisk
    return parent()->DecryptDisk(params);
}

int DiskEncryptAdaptor::EncryptStatus(const QString &device)
{
    // handle method call org.deepin.Filemanager.DiskEncrypt.EncryptStatus
    return parent()->EncryptStatus(device);
}

bool DiskEncryptAdaptor::HasPendingTask()
{
    // handle method call org.deepin.Filemanager.DiskEncrypt.HasPendingTask
    return parent()->HasPendingTask();
}

void DiskEncryptAdaptor::IgnoreParamRequest()
{
    // handle method call org.deepin.Filemanager.DiskEncrypt.IgnoreParamRequest
    parent()->IgnoreParamRequest();
}

bool DiskEncryptAdaptor::IsWorkerRunning()
{
    // handle method call org.deepin.Filemanager.DiskEncrypt.IsWorkerRunning
    return parent()->IsWorkerRunning();
}

QString DiskEncryptAdaptor::PrepareEncryptDisk(const QVariantMap &params)
{
    // handle method call org.deepin.Filemanager.DiskEncrypt.PrepareEncryptDisk
    return parent()->PrepareEncryptDisk(params);
}

QString DiskEncryptAdaptor::QueryTPMToken(const QString &device)
{
    // handle method call org.deepin.Filemanager.DiskEncrypt.QueryTPMToken
    return parent()->QueryTPMToken(device);
}

void DiskEncryptAdaptor::ResumeEncryption(const QString &device)
{
    // handle method call org.deepin.Filemanager.DiskEncrypt.ResumeEncryption
    parent()->ResumeEncryption(device);
}

void DiskEncryptAdaptor::SetEncryptParams(const QVariantMap &params)
{
    // handle method call org.deepin.Filemanager.DiskEncrypt.SetEncryptParams
    parent()->SetEncryptParams(params);
}

QString DiskEncryptAdaptor::UnfinishedDecryptJob()
{
    // handle method call org.deepin.Filemanager.DiskEncrypt.UnfinishedDecryptJob
    return parent()->UnfinishedDecryptJob();
}


#include "diskencryptadaptor.moc"
