/****************************************************************************
** Meta object code from reading C++ file 'traversaldirthreadmanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-workspace/utils/traversaldirthreadmanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'traversaldirthreadmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_workspace__TraversalDirThreadManager_t {
    QByteArrayData data[17];
    char stringdata0[329];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_workspace__TraversalDirThreadManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_workspace__TraversalDirThreadManager_t qt_meta_stringdata_dfmplugin_workspace__TraversalDirThreadManager = {
    {
QT_MOC_LITERAL(0, 0, 46), // "dfmplugin_workspace::Traversa..."
QT_MOC_LITERAL(1, 47, 21), // "updateChildrenManager"
QT_MOC_LITERAL(2, 69, 0), // ""
QT_MOC_LITERAL(3, 70, 22), // "QList<FileInfoPointer>"
QT_MOC_LITERAL(4, 93, 8), // "children"
QT_MOC_LITERAL(5, 102, 14), // "traversalToken"
QT_MOC_LITERAL(6, 117, 19), // "updateLocalChildren"
QT_MOC_LITERAL(7, 137, 22), // "QList<SortInfoPointer>"
QT_MOC_LITERAL(8, 160, 39), // "dfmio::DEnumerator::SortRoleC..."
QT_MOC_LITERAL(9, 200, 8), // "sortRole"
QT_MOC_LITERAL(10, 209, 13), // "Qt::SortOrder"
QT_MOC_LITERAL(11, 223, 9), // "sortOrder"
QT_MOC_LITERAL(12, 233, 15), // "isMixDirAndFile"
QT_MOC_LITERAL(13, 249, 17), // "traversalFinished"
QT_MOC_LITERAL(14, 267, 20), // "traversalRequestSort"
QT_MOC_LITERAL(15, 288, 20), // "iteratorInitFinished"
QT_MOC_LITERAL(16, 309, 19) // "onAsyncIteratorOver"

    },
    "dfmplugin_workspace::TraversalDirThreadManager\0"
    "updateChildrenManager\0\0QList<FileInfoPointer>\0"
    "children\0traversalToken\0updateLocalChildren\0"
    "QList<SortInfoPointer>\0"
    "dfmio::DEnumerator::SortRoleCompareFlag\0"
    "sortRole\0Qt::SortOrder\0sortOrder\0"
    "isMixDirAndFile\0traversalFinished\0"
    "traversalRequestSort\0iteratorInitFinished\0"
    "onAsyncIteratorOver"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_workspace__TraversalDirThreadManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   44,    2, 0x06 /* Public */,
       6,    5,   49,    2, 0x06 /* Public */,
      13,    1,   60,    2, 0x06 /* Public */,
      14,    1,   63,    2, 0x06 /* Public */,
      15,    0,   66,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      16,    0,   67,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString,    4,    5,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 8, 0x80000000 | 10, QMetaType::Bool, QMetaType::QString,    4,    9,   11,   12,    5,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void dfmplugin_workspace::TraversalDirThreadManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TraversalDirThreadManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateChildrenManager((*reinterpret_cast< const QList<FileInfoPointer>(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 1: _t->updateLocalChildren((*reinterpret_cast< const QList<SortInfoPointer>(*)>(_a[1])),(*reinterpret_cast< dfmio::DEnumerator::SortRoleCompareFlag(*)>(_a[2])),(*reinterpret_cast< Qt::SortOrder(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4])),(*reinterpret_cast< QString(*)>(_a[5]))); break;
        case 2: _t->traversalFinished((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: _t->traversalRequestSort((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 4: _t->iteratorInitFinished(); break;
        case 5: _t->onAsyncIteratorOver(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<FileInfoPointer> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<SortInfoPointer> >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< dfmio::DEnumerator::SortRoleCompareFlag >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TraversalDirThreadManager::*)(const QList<FileInfoPointer> , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TraversalDirThreadManager::updateChildrenManager)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TraversalDirThreadManager::*)(const QList<SortInfoPointer> , dfmio::DEnumerator::SortRoleCompareFlag , Qt::SortOrder , bool , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TraversalDirThreadManager::updateLocalChildren)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TraversalDirThreadManager::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TraversalDirThreadManager::traversalFinished)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TraversalDirThreadManager::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TraversalDirThreadManager::traversalRequestSort)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TraversalDirThreadManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TraversalDirThreadManager::iteratorInitFinished)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_workspace::TraversalDirThreadManager::staticMetaObject = { {
    QMetaObject::SuperData::link<TraversalDirThread::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_workspace__TraversalDirThreadManager.data,
    qt_meta_data_dfmplugin_workspace__TraversalDirThreadManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_workspace::TraversalDirThreadManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_workspace::TraversalDirThreadManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_workspace__TraversalDirThreadManager.stringdata0))
        return static_cast<void*>(this);
    return TraversalDirThread::qt_metacast(_clname);
}

int dfmplugin_workspace::TraversalDirThreadManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TraversalDirThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_workspace::TraversalDirThreadManager::updateChildrenManager(const QList<FileInfoPointer> _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_workspace::TraversalDirThreadManager::updateLocalChildren(const QList<SortInfoPointer> _t1, dfmio::DEnumerator::SortRoleCompareFlag _t2, Qt::SortOrder _t3, bool _t4, QString _t5)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_workspace::TraversalDirThreadManager::traversalFinished(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_workspace::TraversalDirThreadManager::traversalRequestSort(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_workspace::TraversalDirThreadManager::iteratorInitFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
