// This file is autogenerated. Changes will be overwritten.
#include "PHM5QH5PCQ/moc_trashcoreeventreceiver.cpp"
#include "PHM5QH5PCQ/moc_trashcoreeventsender.cpp"
#include "EWIEGA46WW/moc_trashcore.cpp"
#include "DWW7M6GQBY/moc_trashpropertydialog.cpp"
