// SPDX-FileCopyrightText: 2021 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef DFMPLUGIN_SMBBROWSER_GLOBAL_H
#define DFMPLUGIN_SMBBROWSER_GLOBAL_H

#include <dfm-base/dfm_log_defines.h>

#define DPSMBBROWSER_NAMESPACE dfmplugin_smbbrowser
#define DPSMBBROWSER_BEGIN_NAMESPACE namespace DPSMBBROWSER_NAMESPACE {
#define DPSMBBROWSER_END_NAMESPACE }
#define DPSMBBROWSER_USE_NAMESPACE using namespace DPSMBBROWSER_NAMESPACE;

DPSMBBROWSER_BEGIN_NAMESPACE
DFM_LOG_USE_CATEGORY(DPSMBBROWSER_NAMESPACE)
DPSMBBROWSER_END_NAMESPACE

#endif   // DFMPLUGIN_SMBBROWSER_GLOBAL_H
