/****************************************************************************
** Meta object code from reading C++ file 'bluetoothdevice.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/common/dfmplugin-utils/bluetooth/private/bluetoothdevice.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bluetoothdevice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_utils__BluetoothDevice_t {
    QByteArrayData data[16];
    char stringdata0[189];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_utils__BluetoothDevice_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_utils__BluetoothDevice_t qt_meta_stringdata_dfmplugin_utils__BluetoothDevice = {
    {
QT_MOC_LITERAL(0, 0, 32), // "dfmplugin_utils::BluetoothDevice"
QT_MOC_LITERAL(1, 33, 11), // "nameChanged"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 4), // "name"
QT_MOC_LITERAL(4, 51, 12), // "aliasChanged"
QT_MOC_LITERAL(5, 64, 5), // "alias"
QT_MOC_LITERAL(6, 70, 13), // "pairedChanged"
QT_MOC_LITERAL(7, 84, 6), // "paired"
QT_MOC_LITERAL(8, 91, 12), // "stateChanged"
QT_MOC_LITERAL(9, 104, 5), // "State"
QT_MOC_LITERAL(10, 110, 5), // "state"
QT_MOC_LITERAL(11, 116, 14), // "trustedChanged"
QT_MOC_LITERAL(12, 131, 7), // "trusted"
QT_MOC_LITERAL(13, 139, 17), // "kStateUnavailable"
QT_MOC_LITERAL(14, 157, 15), // "kStateAvailable"
QT_MOC_LITERAL(15, 173, 15) // "kStateConnected"

    },
    "dfmplugin_utils::BluetoothDevice\0"
    "nameChanged\0\0name\0aliasChanged\0alias\0"
    "pairedChanged\0paired\0stateChanged\0"
    "State\0state\0trustedChanged\0trusted\0"
    "kStateUnavailable\0kStateAvailable\0"
    "kStateConnected"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_utils__BluetoothDevice[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       1,   54, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x06 /* Public */,
       4,    1,   42,    2, 0x06 /* Public */,
       6,    1,   45,    2, 0x06 /* Public */,
       8,    1,   48,    2, 0x06 /* Public */,
      11,    1,   51,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::Bool,   12,

 // enums: name, alias, flags, count, data
       9,    9, 0x0,    3,   59,

 // enum data: key, value
      13, uint(dfmplugin_utils::BluetoothDevice::kStateUnavailable),
      14, uint(dfmplugin_utils::BluetoothDevice::kStateAvailable),
      15, uint(dfmplugin_utils::BluetoothDevice::kStateConnected),

       0        // eod
};

void dfmplugin_utils::BluetoothDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BluetoothDevice *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->nameChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->aliasChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->pairedChanged((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 3: _t->stateChanged((*reinterpret_cast< const State(*)>(_a[1]))); break;
        case 4: _t->trustedChanged((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (BluetoothDevice::*)(const QString & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothDevice::nameChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (BluetoothDevice::*)(const QString & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothDevice::aliasChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (BluetoothDevice::*)(const bool & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothDevice::pairedChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (BluetoothDevice::*)(const State & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothDevice::stateChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (BluetoothDevice::*)(const bool ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BluetoothDevice::trustedChanged)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_utils::BluetoothDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_utils__BluetoothDevice.data,
    qt_meta_data_dfmplugin_utils__BluetoothDevice,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_utils::BluetoothDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_utils::BluetoothDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_utils__BluetoothDevice.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_utils::BluetoothDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void dfmplugin_utils::BluetoothDevice::nameChanged(const QString & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< dfmplugin_utils::BluetoothDevice *>(this), &staticMetaObject, 0, _a);
}

// SIGNAL 1
void dfmplugin_utils::BluetoothDevice::aliasChanged(const QString & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< dfmplugin_utils::BluetoothDevice *>(this), &staticMetaObject, 1, _a);
}

// SIGNAL 2
void dfmplugin_utils::BluetoothDevice::pairedChanged(const bool & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< dfmplugin_utils::BluetoothDevice *>(this), &staticMetaObject, 2, _a);
}

// SIGNAL 3
void dfmplugin_utils::BluetoothDevice::stateChanged(const State & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< dfmplugin_utils::BluetoothDevice *>(this), &staticMetaObject, 3, _a);
}

// SIGNAL 4
void dfmplugin_utils::BluetoothDevice::trustedChanged(const bool _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< dfmplugin_utils::BluetoothDevice *>(this), &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
