/****************************************************************************
** Meta object code from reading C++ file 'collectionmodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/desktop/ddplugin-organizer/models/collectionmodel_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'collectionmodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ddplugin_organizer__CollectionModelPrivate_t {
    QByteArrayData data[18];
    char stringdata0[259];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ddplugin_organizer__CollectionModelPrivate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ddplugin_organizer__CollectionModelPrivate_t qt_meta_stringdata_ddplugin_organizer__CollectionModelPrivate = {
    {
QT_MOC_LITERAL(0, 0, 42), // "ddplugin_organizer::Collectio..."
QT_MOC_LITERAL(1, 43, 17), // "sourceDataChanged"
QT_MOC_LITERAL(2, 61, 0), // ""
QT_MOC_LITERAL(3, 62, 11), // "QModelIndex"
QT_MOC_LITERAL(4, 74, 13), // "sourceTopleft"
QT_MOC_LITERAL(5, 88, 17), // "sourceBottomright"
QT_MOC_LITERAL(6, 106, 12), // "QVector<int>"
QT_MOC_LITERAL(7, 119, 5), // "roles"
QT_MOC_LITERAL(8, 125, 20), // "sourceAboutToBeReset"
QT_MOC_LITERAL(9, 146, 11), // "sourceReset"
QT_MOC_LITERAL(10, 158, 18), // "sourceRowsInserted"
QT_MOC_LITERAL(11, 177, 12), // "sourceParent"
QT_MOC_LITERAL(12, 190, 5), // "start"
QT_MOC_LITERAL(13, 196, 3), // "end"
QT_MOC_LITERAL(14, 200, 26), // "sourceRowsAboutToBeRemoved"
QT_MOC_LITERAL(15, 227, 17), // "sourceDataRenamed"
QT_MOC_LITERAL(16, 245, 6), // "oldUrl"
QT_MOC_LITERAL(17, 252, 6) // "newUrl"

    },
    "ddplugin_organizer::CollectionModelPrivate\0"
    "sourceDataChanged\0\0QModelIndex\0"
    "sourceTopleft\0sourceBottomright\0"
    "QVector<int>\0roles\0sourceAboutToBeReset\0"
    "sourceReset\0sourceRowsInserted\0"
    "sourceParent\0start\0end\0"
    "sourceRowsAboutToBeRemoved\0sourceDataRenamed\0"
    "oldUrl\0newUrl"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ddplugin_organizer__CollectionModelPrivate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   44,    2, 0x0a /* Public */,
       8,    0,   51,    2, 0x0a /* Public */,
       9,    0,   52,    2, 0x0a /* Public */,
      10,    3,   53,    2, 0x0a /* Public */,
      14,    3,   60,    2, 0x0a /* Public */,
      15,    2,   67,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3, 0x80000000 | 6,    4,    5,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,   11,   12,   13,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,   11,   12,   13,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QUrl,   16,   17,

       0        // eod
};

void ddplugin_organizer::CollectionModelPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CollectionModelPrivate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sourceDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2])),(*reinterpret_cast< const QVector<int>(*)>(_a[3]))); break;
        case 1: _t->sourceAboutToBeReset(); break;
        case 2: _t->sourceReset(); break;
        case 3: _t->sourceRowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 4: _t->sourceRowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 5: _t->sourceDataRenamed((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<int> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ddplugin_organizer::CollectionModelPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ddplugin_organizer__CollectionModelPrivate.data,
    qt_meta_data_ddplugin_organizer__CollectionModelPrivate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ddplugin_organizer::CollectionModelPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ddplugin_organizer::CollectionModelPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ddplugin_organizer__CollectionModelPrivate.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ddplugin_organizer::CollectionModelPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
