/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -i screensaver_interface.h -p :screensaver_interface.cpp /home/lee/wk/V20/github/dde-file-manager/src/dde-wallpaper-chooser/dbus/com.deepin.ScreenSaver.xml
 *
 * qdbusxml2cpp is Copyright (C) 2017 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "screensaver_interface.h"
/*
 * Implementation of interface class ComDeepinScreenSaverInterface
 */

ComDeepinScreenSaverInterface::ComDeepinScreenSaverInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

ComDeepinScreenSaverInterface::~ComDeepinScreenSaverInterface()
{
}

