/****************************************************************************
** Meta object code from reading C++ file 'shortcuthelper.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../dde-file-manager/src/plugins/filemanager/dfmplugin-workspace/utils/shortcuthelper.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'shortcuthelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_dfmplugin_workspace__ShortcutHelper_t {
    QByteArrayData data[21];
    char stringdata0[257];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_dfmplugin_workspace__ShortcutHelper_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_dfmplugin_workspace__ShortcutHelper_t qt_meta_stringdata_dfmplugin_workspace__ShortcutHelper = {
    {
QT_MOC_LITERAL(0, 0, 35), // "dfmplugin_workspace::Shortcut..."
QT_MOC_LITERAL(1, 36, 15), // "acitonTriggered"
QT_MOC_LITERAL(2, 52, 0), // ""
QT_MOC_LITERAL(3, 53, 9), // "copyFiles"
QT_MOC_LITERAL(4, 63, 8), // "cutFiles"
QT_MOC_LITERAL(5, 72, 10), // "pasteFiles"
QT_MOC_LITERAL(6, 83, 9), // "undoFiles"
QT_MOC_LITERAL(7, 93, 11), // "deleteFiles"
QT_MOC_LITERAL(8, 105, 11), // "moveToTrash"
QT_MOC_LITERAL(9, 117, 11), // "touchFolder"
QT_MOC_LITERAL(10, 129, 17), // "toggleHiddenFiles"
QT_MOC_LITERAL(11, 147, 17), // "showFilesProperty"
QT_MOC_LITERAL(12, 165, 12), // "previewFiles"
QT_MOC_LITERAL(13, 178, 10), // "openAction"
QT_MOC_LITERAL(14, 189, 11), // "QList<QUrl>"
QT_MOC_LITERAL(15, 201, 4), // "urls"
QT_MOC_LITERAL(16, 206, 11), // "DirOpenMode"
QT_MOC_LITERAL(17, 218, 8), // "openMode"
QT_MOC_LITERAL(18, 227, 14), // "openInTerminal"
QT_MOC_LITERAL(19, 242, 4), // "cdUp"
QT_MOC_LITERAL(20, 247, 9) // "redoFiles"

    },
    "dfmplugin_workspace::ShortcutHelper\0"
    "acitonTriggered\0\0copyFiles\0cutFiles\0"
    "pasteFiles\0undoFiles\0deleteFiles\0"
    "moveToTrash\0touchFolder\0toggleHiddenFiles\0"
    "showFilesProperty\0previewFiles\0"
    "openAction\0QList<QUrl>\0urls\0DirOpenMode\0"
    "openMode\0openInTerminal\0cdUp\0redoFiles"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_dfmplugin_workspace__ShortcutHelper[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   94,    2, 0x09 /* Protected */,
       3,    0,   95,    2, 0x09 /* Protected */,
       4,    0,   96,    2, 0x09 /* Protected */,
       5,    0,   97,    2, 0x09 /* Protected */,
       6,    0,   98,    2, 0x09 /* Protected */,
       7,    0,   99,    2, 0x09 /* Protected */,
       8,    0,  100,    2, 0x09 /* Protected */,
       9,    0,  101,    2, 0x09 /* Protected */,
      10,    0,  102,    2, 0x09 /* Protected */,
      11,    0,  103,    2, 0x09 /* Protected */,
      12,    0,  104,    2, 0x09 /* Protected */,
      13,    2,  105,    2, 0x09 /* Protected */,
      13,    1,  110,    2, 0x29 /* Protected | MethodCloned */,
      18,    0,  113,    2, 0x09 /* Protected */,
      19,    0,  114,    2, 0x09 /* Protected */,
      20,    0,  115,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14, 0x80000000 | 16,   15,   17,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void dfmplugin_workspace::ShortcutHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ShortcutHelper *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->acitonTriggered(); break;
        case 1: _t->copyFiles(); break;
        case 2: _t->cutFiles(); break;
        case 3: _t->pasteFiles(); break;
        case 4: _t->undoFiles(); break;
        case 5: _t->deleteFiles(); break;
        case 6: _t->moveToTrash(); break;
        case 7: _t->touchFolder(); break;
        case 8: _t->toggleHiddenFiles(); break;
        case 9: _t->showFilesProperty(); break;
        case 10: _t->previewFiles(); break;
        case 11: _t->openAction((*reinterpret_cast< const QList<QUrl>(*)>(_a[1])),(*reinterpret_cast< const DirOpenMode(*)>(_a[2]))); break;
        case 12: _t->openAction((*reinterpret_cast< const QList<QUrl>(*)>(_a[1]))); break;
        case 13: _t->openInTerminal(); break;
        case 14: _t->cdUp(); break;
        case 15: _t->redoFiles(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject dfmplugin_workspace::ShortcutHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_dfmplugin_workspace__ShortcutHelper.data,
    qt_meta_data_dfmplugin_workspace__ShortcutHelper,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *dfmplugin_workspace::ShortcutHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dfmplugin_workspace::ShortcutHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_dfmplugin_workspace__ShortcutHelper.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dfmplugin_workspace::ShortcutHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
